from numba import jit, int64, float64
from kolya import parameters
import math

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def X(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^0 level functions of the LO tree level contribution for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return 1-r**8-2*y**3+y**4+6*r**4*(2*logy+((-2+y)*y)/(-1+y))+2*r**2*(
        -4+y**3)-(2*r**6*(-4+8*y-4*y**2+y**3))/(-1+y)**2
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-3*r**10*(-1+y)**3+30*r**4*(-1+y)**2*(8+6*logy*(-1+y)-14*y+3*y**2
        +y**3)+5*r**2*(-1+y)**3*(-9+y**4)-10*r**6*(-1+y)*(-24-18*logy*(-1
        +y)**2+30*y+3*y**2-6*y**3+y**4)-5*r**8*(9-27*y+27*y**2-9*y**3
        +2*y**4)+(-1+y)**4*(-3-3*y-3*y**2-3*y**3+2*y**4)+10*MBhat**2*(-1
        +y)*(-(r**8*(-1+y)**2)+6*r**4*(-1+y)*(2*logy*(-1+y)+(-2+y)*y)
        +2*r**2*(-1+y)**2*(-4+y**3)-2*r**6*(-4+8*y-4*y**2+y**3)+(-1+y)**3*(
        -1-y-y**2+y**3))+MBhat*(-7*r**10*(-1+y)**3+5*r**2*(-1+y)**3*(27
        -8*y**3+y**4)+10*r**6*(-1+y)*(-32-6*logy*(-1+y)**2+58*y-23*y**2
        +2*y**3+y**4)-5*r**8*(9-27*y+27*y**2-13*y**3+2*y**4)+10*r**4*(-1
        +y)**2*(-16-30*logy*(-1+y)+46*y-15*y**2-5*y**3+2*y**4)-(-1+y)**4*(
        -13-13*y-13*y**2+7*y**3+2*y**4)))/(10*(-1+y)**3)
    if (dQ2 == 0 and dr == 0 and i==2):
        return MBhat**4*(1-r**8-2*y**3+y**4+6*r**4*(-1+2*logy+(1-y)**(-1)+y)
        +2*r**2*(-4+y**3)-(2*r**6*(-4+8*y-4*y**2+y**3))/(-1+y)**2)+MBhat**3*(
        -13/5-(7*r**10)/5+r**8*(7+2/(-1+y)**3+4/(-1+y)**2-2*y)+4*y**3
        -y**4-(2*y**5)/5+2*r**6*(-16-6*logy+6/(-1+y)**2+22/(-1+y)+4*y
        +y**2)+r**4*(56-60*logy+24/(-1+y)-36*y-6*y**2+4*y**3)+r**2*(27
        -8*y**3+y**4))+MBhat*(-13/15-r**12/3+r**10*(2+6/(5*(-1+y)**4)+4/(
        -1+y)**3+4/(-1+y)**2-(4*y)/5)+y**4-(2*y**6)/15+r**8*(-27-12*logy
        +8/(-1+y)**3+34/(-1+y)**2+60/(-1+y)+4*y)+(4*r**6*(6-96*logy
        +15/(-1+y)**2+93/(-1+y)-33*y+y**3))/3-2*r**2*(-8+y**4)+r**4*(163
        -84*logy+24/(-1+y)-60*y-18*y**2-4*y**3+y**4))+MBhat**2*((-8*r**12)/15
        +(r**10*(13-3/(-1+y)**4-5/(-1+y)**3-3*y))/5+2*r**4*(-78+54*logy
        -18/(-1+y)+36*y+9*y**2-2*y**3)+r**2*(-159/5+6*y**3+y**4-y**5/5)
        +(r**8*(-3+9*y-9*y**2-3*y**3-2*y**4+y**5))/(-1+y)**3+(35-30*y**3
        -15*y**4+9*y**5+y**6)/15+r**6*(76*logy+8*y-6*y**2+(4*y**3)/3+(4*(36
        -55*y+13*y**2))/(-1+y)**2))+(2-2*r**12-3*y**5+y**6+30*r**4*(-28
        +12*logy-3/(-1+y)+9*y+3*y**2+y**3)+3*r**2*(-16+y**5)-(3*r**10*(
        -16+64*y-96*y**2+64*y**3-16*y**4+3*y**5))/(-1+y)**4+15*r**6*(64*logy
        -(2*y*(-96+144*y-32*y**2-8*y**3+y**4))/(3*(-1+y)**2))+15*r**8*(24*logy
        +(-50+126*y-90*y**2+6*y**3+6*y**4-y**5)/(-1+y)**3))/15
    if (dQ2 == 0 and dr == 0 and i==3):
        return (5-5*r**14-7*y**6+2*y**7+70*r**6*(-365/2+150*logy-8/(-1+y)**2-78/(
        -1+y)+88*y+21*y**2+4*y**3-y**4/2)+70*r**4*(-169/2+30*logy-6/(
        -1+y)+24*y+9*y**2+4*y**3+(3*y**4)/2)+(7*r**10*(785+300*logy*(-1
        +y)**4-2840*y+3660*y**2-1840*y**3+160*y**4+60*y**5-9*y**6))/(-1+y)**4
        +(35*r**8*(-225+300*logy*(-1+y)**3+375*y+75*y**2-325*y**3+75*y**4
        +15*y**5-2*y**6))/(-1+y)**3+7*r**2*(-25+y**6)-(7*r**12*(25-125*y
        +250*y**2-250*y**3+125*y**4-25*y**5+4*y**6))/(-1+y)**5+70*MBhat**6*(1
        -r**8-2*y**3+y**4+6*r**4*(-1+2*logy+(1-y)**(-1)+y)+2*r**2*(-4
        +y**3)-(2*r**6*(-4+8*y-4*y**2+y**3))/(-1+y)**2)-(21*MBhat**5*(7*r**10*(
        -1+y)**3+10*r**4*(-1+y)**2*(16+30*logy*(-1+y)-46*y+15*y**2+5*y**3
        -2*y**4)-5*r**2*(-1+y)**3*(27-8*y**3+y**4)-10*r**6*(-1+y)*(-32-6*logy*(
        -1+y)**2+58*y-23*y**2+2*y**3+y**4)+5*r**8*(9-27*y+27*y**2-13*y**3
        +2*y**4)+(-1+y)**4*(-13-13*y-13*y**2+7*y**3+2*y**4)))/(-1+y)**3
        -(7*MBhat**4*(16*r**12*(-1+y)**4+30*r**4*(-1+y)**3*(-96-90*logy*(-1
        +y)+186*y-45*y**2-19*y**3+4*y**4)+3*r**2*(-1+y)**4*(273-60*y**3
        -5*y**4+2*y**5)-(-1+y)**5*(-61-61*y-61*y**2-y**3+14*y**4+2*y**5)
        -10*r**6*(-1+y)**2*(360+174*logy*(-1+y)**2-546*y+99*y**2+46*y**3
        -23*y**4+4*y**5)+3*r**10*(-33+132*y-198*y**2+152*y**3-53*y**4+6*y**5)
        -15*r**8*(-3+12*y-18*y**2+24*y**3-13*y**4-4*y**5+2*y**6)))/(-1+y)**4
        +MBhat**3*(-348-30*r**14+14*r**12*(10+2/(-1+y)**5+3/(-1+y)**4-2*y)
        +140*y**3+245*y**4-35*y**6-2*y**7+70*r**4*(773-420*logy+120/(-1
        +y)-300*y-90*y**2-10*y**3+3*y**4)+7*r**2*(868-80*y**3-75*y**4+y**6)
        +(7*r**10*(4-16*y+24*y**2+64*y**3+29*y**4-36*y**5+9*y**6))/(-1+y)**4
        +70*r**6*(-564*logy-176*y+6*y**2+(-511+634*y-23*y**2)/(-1+y)**2)
        +70*r**8*(-48*logy+6*y-(9*y**2)/2+y**3+(452-1248*y+1071*y**2-191*y**3)/(2*(
        -1+y)**3)))+MBhat*(-43-13*r**14+7*r**12*(13+12/(-1+y)**5+54/(-1
        +y)**4+90/(-1+y)**3+60/(-1+y)**2-6*y)+42*y**5+7*y**6-6*y**7+70*r**6*(999/2
        -570*logy+42/(-1+y)**2+354/(-1+y)-300*y-57*y**2-4*y**3+(3*y**4)/2)
        +70*r**4*(4119/10-162*logy+36/(-1+y)-126*y-45*y**2-18*y**3-(9*y**4)/2
        +(3*y**5)/5)-7*r**2*(-175+12*y**5+y**6)+70*r**8*(-330*logy-81*y+(3*y**2)/2
        +y**3-(3*(-325+809*y-588*y**2+86*y**3))/(2*(-1+y)**3))+21*r**10*(
        -60*logy-(317-1208*y+1692*y**2-1008*y**3+192*y**4-16*y**5+y**6)/(
        -1+y)**4))+MBhat**2*(-22*r**14+42*r**12*(3-2/(-1+y)**5-6/(-1+y)**4
        -5/(-1+y)**3-y)+70*r**4*(-4002/5+360*logy-90/(-1+y)+270*y+90*y**2
        +30*y**3-(3*y**5)/5)-7*r**2*(524-45*y**4-18*y**5+y**6)+(21*r**10*(28
        -112*y+168*y**2-112*y**3-17*y**4+2*y**5+y**6))/(-1+y)**4+3*(54
        -35*y**4-28*y**5+7*y**6+2*y**7)+70*r**6*(828*logy+(3*(55+166*y-359*y**2
        +92*y**3+18*y**4-2*y**5))/(-1+y)**2)+70*r**8*(228*logy+(-1170+3054*y
        -2370*y**2+334*y**3+69*y**4-15*y**5+2*y**6)/(2*(-1+y)**3))))/70
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-3*r**16*(-1+y)**6+42*r**4*(-1+y)**5*(182+60*logy*(-1+y)-242*y
        +30*y**2+10*y**3+5*y**4+3*y**5+2*y**6)+14*r**12*(-1+y)*(-546+180*logy*(
        -1+y)**5+2550*y-4650*y**2+4050*y**3-1575*y**4+135*y**5+30*y**6-4*y**7)
        +28*r**6*(-1+y)**4*(-924+720*logy*(-1+y)**2+2568*y-2004*y**2+240*y**3
        +60*y**4+24*y**5+12*y**6-y**7)+4*r**2*(-1+y)**6*(-36+y**7)-84*r**10*(
        -1+y)**2*(-308-240*logy*(-1+y)**4+992*y-1008*y**2+192*y**3+192*y**4
        -48*y**5-8*y**6+y**7)+(-1+y)**7*(-3-3*y-3*y**2-3*y**3-3*y**4-3*y**5
        -3*y**6+y**7)-4*r**14*(-36+216*y-540*y**2+720*y**3-540*y**4+216*y**5
        -36*y**6+5*y**7)+70*MBhat**8*(-1+y)**4*(-(r**8*(-1+y)**2)+6*r**4*(-1
        +y)*(2*logy*(-1+y)+(-2+y)*y)+2*r**2*(-1+y)**2*(-4+y**3)-2*r**6*(
        -4+8*y-4*y**2+y**3)+(-1+y)**3*(-1-y-y**2+y**3))-28*MBhat**7*(
        -1+y)**3*(7*r**10*(-1+y)**3+10*r**4*(-1+y)**2*(16+30*logy*(-1+y)
        -46*y+15*y**2+5*y**3-2*y**4)-5*r**2*(-1+y)**3*(27-8*y**3+y**4)-10*r**6*(
        -1+y)*(-32-6*logy*(-1+y)**2+58*y-23*y**2+2*y**3+y**4)+5*r**8*(9
        -27*y+27*y**2-13*y**3+2*y**4)+(-1+y)**4*(-13-13*y-13*y**2+7*y**3
        +2*y**4))-70*r**8*(-1+y)**3*(-540*logy*(-1+y)**3+y*(540-1350*y+990*y**2
        -135*y**3-27*y**4-9*y**5+y**6))+28*MBhat**6*(-1+y)**2*(-8*r**12*(-1
        +y)**4+30*r**4*(-1+y)**3*(44+42*logy*(-1+y)-86*y+21*y**2+9*y**3
        -2*y**4)+(-1+y)**5*(-29-29*y-29*y**2+y**3+6*y**4+y**5)+10*r**6*(
        -1+y)**2*(168+78*logy*(-1+y)**2-258*y+51*y**2+20*y**3-11*y**4+2*y**5)
        -3*r**10*(-17+68*y-102*y**2+78*y**3-27*y**4+3*y**5)-r**2*(-1+y)**4*(387
        -90*y**3-5*y**4+3*y**5)+5*r**8*(-9+36*y-54*y**2+54*y**3-25*y**4
        -5*y**5+3*y**6))-4*MBhat*(2*r**16*(-1+y)**6+14*r**10*(-1+y)**2*(1678
        +840*logy*(-1+y)**4-5872*y+7128*y**2-3072*y**3-72*y**4+168*y**5
        +y**6-y**7)+35*r**8*(-1+y)**3*(-450+1080*logy*(-1+y)**3+270*y+1350*y**2
        -1530*y**3+270*y**4+54*y**5+5*y**6-y**7)+28*r**6*(-1+y)**4*(-1009
        +1020*logy*(-1+y)**2+3038*y-2539*y**2+340*y**3+85*y**4+34*y**5+7*y**6
        -y**7)+7*r**4*(-1+y)**5*(1852+660*logy*(-1+y)-2512*y+330*y**2+110*y**3
        +55*y**4+33*y**5+13*y**6-y**7)+7*r**12*(-1+y)*(-332+60*logy*(-1
        +y)**5+1600*y-3050*y**2+2850*y**3-1275*y**4+195*y**5-13*y**6+y**7)
        +(-1+y)**7*(-8-8*y-8*y**2-8*y**3-8*y**4-8*y**5-y**6+y**7)+2*r**2*(
        -1+y)**6*(-162+7*y**6+y**7)+4*r**14*(-9+54*y-135*y**2+180*y**3-135*y**4
        +54*y**5-16*y**6+2*y**7))+MBhat**4*(-25*r**16*(-1+y)**6+700*r**8*(-1
        +y)**3*(-240+90*logy*(-1+y)**3+630*y-495*y**2+74*y**3+8*y**4)+700*r**6*(
        -1+y)**4*(102+360*logy*(-1+y)**2+156*y-438*y**2+121*y**3+18*y**4
        -y**5)+14*r**12*y**3*(50-35*y-56*y**2+63*y**3-26*y**4+4*y**5)+14*r**4*(
        -1+y)**5*(16080+8400*logy*(-1+y)-24480*y+4200*y**2+1490*y**3+595*y**4
        +27*y**5-12*y**6)-4*r**2*(-1+y)**6*(4500-175*y**3-385*y**4-126*y**5
        +7*y**6+y**7)+(-1+y)**7*(-825-825*y-825*y**2-685*y**3-195*y**4
        +85*y**5+29*y**6+y**7)+28*r**10*(-1+y)**2*(100-400*y+600*y**2-445*y**3
        -65*y**4+60*y**5-17*y**6+2*y**7)-4*r**14*(-30+180*y-450*y**2+635*y**3
        -520*y**4+243*y**5-58*y**6+5*y**7))-2*MBhat**2*(7*r**16*(-1+y)**6+70*r**8*(
        -1+y)**3*(1575-1680*logy*(-1+y)**3-3045*y+525*y**2+1505*y**3-420*y**4
        -57*y**5+4*y**6)-14*r**2*(-1+y)**6*(-185+9*y**5+5*y**6)-14*r**6*(-1
        +y)**4*(-6237+9660*logy*(-1+y)**2+22134*y-20727*y**2+3220*y**3+805*y**4
        +247*y**5+y**6-3*y**7)-14*r**4*(-1+y)**5*(5355+2100*logy*(-1+y)
        -7455*y+1050*y**2+350*y**3+175*y**4+96*y**5+11*y**6-2*y**7)-14*r**10*(
        -1+y)**2*(3857+1260*logy*(-1+y)**4-14168*y+18732*y**2-9968*y**3
        +1232*y**4+135*y**5-17*y**6+2*y**7)-(-1+y)**7*(-77-77*y-77*y**2
        -77*y**3-77*y**4-35*y**5+7*y**6+3*y**7)-14*r**12*(35-210*y+525*y**2
        -700*y**3+525*y**4-159*y**5-21*y**6+5*y**7)+2*r**14*(-35+210*y-525*y**2
        +700*y**3-525*y**4+273*y**5-77*y**6+9*y**7))-4*MBhat**5*(-1+y)*(30*r**14*(
        -1+y)**5+35*r**4*(-1+y)**4*(889+588*logy*(-1+y)-1477*y+294*y**2
        +118*y**3+11*y**4-3*y**5)+70*r**6*(-1+y)**3*(367+372*logy*(-1+y)**2
        -362*y-191*y**2+124*y**3-10*y**4+2*y**5)-7*r**2*(-1+y)**5*(628-80*y**3
        -45*y**4+y**6)-35*r**8*(-1+y)**2*(269-60*logy*(-1+y)**3-747*y+657*y**2
        -139*y**3+33*y**4-15*y**5+2*y**6)+(-1+y)**6*(-257-257*y-257*y**2
        -117*y**3+23*y**4+23*y**5+2*y**6)+7*r**12*(27-135*y+270*y**2-290*y**3
        +165*y**4-45*y**5+4*y**6)-7*r**10*(44-220*y+440*y**2-520*y**3+305*y**4
        -25*y**5-33*y**6+9*y**7))-4*MBhat**3*(5*r**16*(-1+y)**6+175*r**8*(-1
        +y)**3*(-411+252*logy*(-1+y)**3+981*y-603*y**2-51*y**3+58*y**4)
        +35*r**6*(-1+y)**4*(-573+2460*logy*(-1+y)**2+3606*y-4263*y**2+820*y**3
        +195*y**4+22*y**5-3*y**6)+7*r**10*(-1+y)**2*(1545+300*logy*(-1+y)**4
        -5880*y+8220*y**2-4880*y**3+800*y**4-60*y**5+17*y**6-2*y**7)+7*r**4*(
        -1+y)**5*(8531+3780*logy*(-1+y)-12311*y+1890*y**2+630*y**3+330*y**4
        +96*y**5-5*y**6-y**7)+(-1+y)**7*(-110-110*y-110*y**2-110*y**3-75*y**4
        -5*y**5+9*y**6+y**7)-r**2*(-1+y)**6*(3015-140*y**4-168*y**5-21*y**6
        +2*y**7)+r**14*(-33+198*y-495*y**2+660*y**3-565*y**4+282*y**5-75*y**6
        +8*y**7)-7*r**12*(5-30*y+75*y**2-100*y**3+20*y**4+34*y**5-y**6-4*y**7
        +y**8)))/(70*(-1+y)**6)

    return 0.

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xpi(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^2 level functions of mupi for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-3+24*r**2+3*r**8+5*y**4+(4*r**6*(-6+24*y-36*y**2+24*y**3-11*y**4
        +2*y**5))/(-1+y)**4+6*r**4*(-6*logy+(y*(6-15*y+11*y**2-4*y**3))/(
        -1+y)**3))/6
    if (dQ2 == 0 and dr == 0 and i==1):
        return (9*r**10*(-1+y)**5+5*r**2*(-1+y)**5*(27+5*y**4)-30*r**4*(-1+y)**2*(24
        +18*logy*(-1+y)**3-90*y+117*y**2-57*y**3+7*y**4+3*y**5)+(-1+y)**5*(
        -9-25*y**4+24*y**5)+10*r**6*(-1+y)*(-72-54*logy*(-1+y)**4+234*y
        -243*y**2+54*y**3+28*y**4-28*y**5+3*y**6)+5*r**8*(27-135*y+270*y**2
        -270*y**3+125*y**4-49*y**5+8*y**6)-10*MBhat*y**3*(-4*r**2*(-1+y)**5
        +(-1+y)**5*(-2+4*y+y**2)+6*r**4*(-1+y)**2*(-4+4*y-5*y**2+y**3)
        -r**8*(10+10*y-11*y**2+3*y**3)+4*r**6*(7-7*y+3*y**2-4*y**3+y**4))
        +10*MBhat**2*(-1+y)*(24*r**2*(-1+y)**4+3*r**8*(-1+y)**4+(-1+y)**4*(
        -3+5*y**4)+4*r**6*(-6+24*y-36*y**2+24*y**3-11*y**4+2*y**5)-6*r**4*(
        -1+y)*(6*logy*(-1+y)**3+y*(-6+15*y-11*y**2+4*y**3))))/(60*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (6*r**12*(-1+y)**6+36*r**2*(-1+y)**6*(4+y**5)-90*r**4*(-1+y)**3*(25
        +12*logy*(-1+y)**3-87*y+105*y**2-47*y**3+3*y**4+3*y**5)+3*(-1
        +y)**6*(-2-12*y**5+7*y**6)+45*r**8*(-1+y)*(50-24*logy*(-1+y)**5
        -226*y+392*y**2-312*y**3+96*y**4-4*y**5-9*y**6+y**7)+36*r**10*(-4
        +24*y-60*y**2+80*y**3-60*y**4+21*y**5-7*y**6+y**7)+12*MBhat*y**4*(
        -1+r**2+y)**2*(-2*(-1+y)**4*(-5+2*y+y**2)+2*r**2*(-1+y)**3*(-5-y
        +2*y**2)+3*r**6*(10+3*y-4*y**2+y**3)+r**4*(-50+63*y-42*y**2+35*y**3
        -6*y**4))-30*MBhat**3*(-1+y)*y**3*(-4*r**2*(-1+y)**5+(-1+y)**5*(-2
        +4*y+y**2)+6*r**4*(-1+y)**2*(-4+4*y-5*y**2+y**3)-r**8*(10+10*y
        -11*y**2+3*y**3)+4*r**6*(7-7*y+3*y**2-4*y**3+y**4))-60*r**6*(-1
        +y)**2*(48*logy*(-1+y)**4+y*(48-168*y+208*y**2-100*y**3+18*y**4
        +3*y**5))+MBhat**2*(-40*r**12*(-1+y)**6+15*r**2*(-1+y)**6*(-51+8*y**3
        +y**4)+90*r**4*(-1+y)**3*(4+18*logy*(-1+y)**3-30*y+57*y**2-45*y**3
        +7*y**4-5*y**5)+(-1+y)**6*(67-120*y**3-15*y**4+96*y**5+5*y**6)+9*r**10*(37
        -222*y+555*y**2-780*y**3+585*y**4-234*y**5+39*y**6)-10*r**6*(-1+y)**2*(
        -216+78*logy*(-1+y)**4+942*y-1569*y**2+1370*y**3-476*y**4+204*y**5
        -47*y**6+8*y**7)-15*r**8*(93-558*y+1395*y**2-1948*y**3+1543*y**4
        -702*y**5+208*y**6-34*y**7+3*y**8))+15*MBhat**4*(-1+y)**2*(24*r**2*(
        -1+y)**4+3*r**8*(-1+y)**4+(-1+y)**4*(-3+5*y**4)+4*r**6*(-6+24*y
        -36*y**2+24*y**3-11*y**4+2*y**5)-6*r**4*(-1+y)*(6*logy*(-1+y)**3
        +y*(-6+15*y-11*y**2+4*y**3))))/(90*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-15+15*r**14-147*y**6+64*y**7+35*r**6*(495-900*logy-96/(-1+y)**4
        -536/(-1+y)**3-1160/(-1+y)**2-900/(-1+y)-704*y-126*y**2-5*y**4)
        +21*r**2*(25+7*y**6)-(105*r**8*(-225+300*logy*(-1+y)**5+825*y-900*y**2
        -100*y**3+800*y**4-460*y**5+99*y**6+9*y**7))/(-1+y)**5-(21*r**10*(785
        +300*logy*(-1+y)**6-4410*y+10125*y**2-12000*y**3+7500*y**4-2100*y**5
        +183*y**6+86*y**7-9*y**8))/(-1+y)**6+(7*r**12*(75-525*y+1575*y**2
        -2625*y**3+2625*y**4-1575*y**5+441*y**6-127*y**7+16*y**8))/(-1+y)**7
        +105*r**4*(-60*logy-64*y-18*y**2+5*y**4+(-157+467*y-447*y**2+129*y**3)/(
        -1+y)**3)+(42*MBhat*y**5*(-1+r**2+y)**3*(-((-1+y)**4*(-17+2*y+3*y**2))
        +r**2*(-1+y)**3*(-29-14*y+7*y**2)+r**6*(63+7*y-13*y**2+3*y**3)
        +r**4*(-109+136*y-72*y**2+52*y**3-7*y**4)))/(-1+y)**7+(210*MBhat**5*y**3*(4*r**2*(
        -1+y)**5-(-1+y)**5*(-2+4*y+y**2)-6*r**4*(-1+y)**2*(-4+4*y-5*y**2
        +y**3)+r**8*(10+10*y-11*y**2+3*y**3)-4*r**6*(7-7*y+3*y**2-4*y**3
        +y**4)))/(-1+y)**5+MBhat**2*(362+74760*logy*r**6-110*r**14+420*r**12*(101/30
        -6/(-1+y)**7-21/(-1+y)**6-126/(5*(-1+y)**5)-21/(2*(-1+y)**4))
        -1365*y**4-504*y**5+553*y**6+72*y**7-7*r**2*(1084-345*y**4-72*y**5
        +5*y**6)+(70*r**6*(825-2232*y+1212*y**2+1328*y**3-1835*y**4+240*y**5
        -62*y**6-16*y**7))/(-1+y)**4-(21*r**10*(372-2232*y+5580*y**2-7440*y**3
        +6565*y**4-2694*y**5+792*y**6-106*y**7+3*y**8))/(-1+y)**6-(35*r**8*(1050
        +24*logy*(-1+y)**5-5274*y+10608*y**2-10688*y**3+6469*y**4-1433*y**5
        +479*y**6-67*y**7+8*y**8))/(-1+y)**5+420*r**4*(108*logy+12*y+3*y**2
        -4*y**4+(192-672*y+813*y**2-363*y**3)/(-1+y)**3))+70*MBhat**6*(-3
        +24*r**2+3*r**8+5*y**4+(4*r**6*(-6+24*y-36*y**2+24*y**3-11*y**4
        +2*y**5))/(-1+y)**4+6*r**4*(-6*logy+(y*(6-15*y+11*y**2-4*y**3))/(
        -1+y)**3))+420*MBhat**3*((-6*r**14)/7+(r**12*(106+30/(-1+y)**7+65/(
        -1+y)**6+35/(-1+y)**5-5*y))/15+r**10*(-21+26/(-1+y)**6+416/(5*(
        -1+y)**5)+477/(5*(-1+y)**4)+42/(-1+y)**3+(22*y)/5)+r**2*(228/5
        -8*y**3-3*y**4+(2*y**5)/5)+(-632+630*y**3+770*y**4-420*y**5-196*y**6
        -5*y**7)/210-(2*r**6*(381+156*logy*(-1+y)**4-1368*y+1740*y**2-908*y**3
        -11*y**4+52*y**5-40*y**6+4*y**7))/(3*(-1+y)**4)-(r**8*(72-360*y+720*y**2
        -830*y**3+376*y**4-154*y**5+8*y**6-y**7+y**8))/(2*(-1+y)**5)+r**4*(
        -168*logy-34*y+13*y**3+(2*(-97+358*y-450*y**2+209*y**3))/(-1+y)**3))
        +420*MBhat**4*(161/60-(4*r**12)/3+r**10*(45/4-6/(-1+y)**6-72/(5*(
        -1+y)**5)-9/(-1+y)**4)-4*y**3+(3*y**4)/4+2*y**5+y**6/6-(r**2*(129
        -16*y**3+3*y**4))/4-(r**8*(-159+795*y-1590*y**2+1766*y**3-925*y**4
        +305*y**5-54*y**6+6*y**7))/(4*(-1+y)**5)+r**4*(81*logy-(3*(-32+150*y
        -231*y**2+147*y**3-21*y**4+7*y**5))/(2*(-1+y)**3))+r**6*(logy+(648
        -2586*y+3867*y**2-2902*y**3+868*y**4-324*y**5+85*y**6-16*y**7)/(6*(
        -1+y)**4))))/420
    if (dQ2 == 0 and dr == 0 and i==4):
        return (9*r**16*(-1+y)**8+16*r**2*(-1+y)**8*(27+8*y**7)-42*r**4*(-1+y)**5*(546
        +180*logy*(-1+y)**3-1818*y+2088*y**2-876*y**3+45*y**4+9*y**5+3*y**6
        +47*y**7-24*y**8)-336*r**10*(-1+y)**2*(231+180*logy*(-1+y)**6-1206*y
        +2475*y**2-2400*y**3+900*y**4+180*y**5-210*y**6+52*y**7+3*y**8)+(
        -1+y)**8*(-9-128*y**7+45*y**8)-56*r**6*(-1+y)**4*(-1386+1080*logy*(
        -1+y)**4+6624*y-12096*y**2+10224*y**3-3636*y**4+216*y**5+36*y**6
        +136*y**7-49*y**8+6*y**9)+8*r**14*(-54+432*y-1512*y**2+3024*y**3
        -3780*y**4+3024*y**5-1512*y**6+352*y**7-89*y**8+10*y**9)+14*r**12*(
        -1+y)*(1638-540*logy*(-1+y)**7-10926*y+30888*y**2-47700*y**3+42975*y**4
        -22005*y**5+5445*y**6-503*y**7-124*y**8+12*y**9)+8*MBhat*y**6*(-1+r**2
        +y)**4*(-((-1+y)**4*(-98-5*y+16*y**2))+r**2*(-1+y)**3*(-252-139*y
        +43*y**2)+4*r**6*(112+4*y-14*y**2+3*y**3)+r**4*(-798+988*y-456*y**2
        +300*y**3-34*y**4))-280*MBhat**7*(-1+y)**3*y**3*(-4*r**2*(-1+y)**5+(
        -1+y)**5*(-2+4*y+y**2)+6*r**4*(-1+y)**2*(-4+4*y-5*y**2+y**3)-r**8*(10
        +10*y-11*y**2+3*y**3)+4*r**6*(7-7*y+3*y**2-4*y**3+y**4))-70*r**8*(
        -1+y)**3*(1620*logy*(-1+y)**5+y*(-1620+7290*y-12690*y**2+10395*y**3
        -3699*y**4+270*y**5+194*y**6-25*y**7+5*y**8))+28*MBhat**6*(-1+y)**2*(
        -40*r**12*(-1+y)**6-5*r**2*(-1+y)**6*(207-24*y**3+7*y**4)+30*r**4*(
        -1+y)**3*(60+90*logy*(-1+y)**3-270*y+405*y**2-249*y**3+35*y**4
        -9*y**5)+(-1+y)**6*(85-120*y**3+35*y**4+48*y**5+5*y**6)+9*r**10*(35
        -210*y+525*y**2-740*y**3+555*y**4-222*y**5+37*y**6)+10*r**6*(-1+y)**2*(360
        +30*logy*(-1+y)**4-1410*y+2055*y**2-1478*y**3+420*y**4-148*y**5
        +41*y**6-8*y**7)-5*r**8*(225-1350*y+3375*y**2-4764*y**3+3839*y**4
        -1758*y**5+510*y**6-86*y**7+9*y**8))-8*MBhat**5*(-1+y)*(180*r**14*(
        -1+y)**7-42*r**2*(-1+y)**7*(228-40*y**3-5*y**4+2*y**5)+210*r**4*(
        -1+y)**4*(194+168*logy*(-1+y)**3-750*y+1002*y**2-507*y**3+7*y**4
        +36*y**5-10*y**6)+70*r**6*(-1+y)**3*(762+312*logy*(-1+y)**4-2736*y
        +3480*y**2-1816*y**3+62*y**4+89*y**5-56*y**6+5*y**7)+(-1+y)**7*(632
        -630*y**3-350*y**4+252*y**5+112*y**6+5*y**7)+105*r**8*(-1+y)**2*(72
        -360*y+720*y**2-830*y**3+420*y**4-152*y**5+10*y**6-y**7+y**8)+14*r**12*(106
        -742*y+2226*y**2-3815*y**3+3885*y**4-2401*y**5+847*y**6-141*y**7
        +5*y**8)-42*r**10*(124-868*y+2604*y**2-4500*y**3+4685*y**4-2996*y**5
        +1176*y**6-244*y**7+19*y**8))-8*MBhat**3*(30*r**16*(-1+y)**8+140*r**6*(
        -1+y)**4*(78+840*logy*(-1+y)**4+528*y-2472*y**2+3328*y**3-1742*y**4
        +88*y**5+65*y**6-20*y**7)-r**2*(-1+y)**8*(5840-980*y**4-672*y**5-14*y**6
        +5*y**7)+21*r**4*(-1+y)**5*(4532+2160*logy*(-1+y)**3-15756*y+18996*y**2
        -8492*y**3+470*y**4-227*y**5+201*y**6+9*y**7-13*y**8)+(-1+y)**8*(230
        -350*y**4-483*y**5+70*y**6+92*y**7+6*y**8)+35*r**8*(-1+y)**3*(-2580
        +960*logy*(-1+y)**5+11940*y-21480*y**2+18280*y**3-7230*y**4+249*y**5
        +115*y**6-82*y**7+8*y**8)+21*r**10*(-1+y)**2*(-80+480*y-1200*y**2
        +1600*y**3-1900*y**4+688*y**5-438*y**6+53*y**7-4*y**8+y**9)+4*r**14*(
        -82+656*y-2296*y**2+4592*y**3-6020*y**4+4942*y**5-2576*y**6+796*y**7
        -122*y**8+5*y**9)-7*r**12*(-220+1760*y-6160*y**2+12320*y**3-16310*y**4
        +13951*y**5-7616*y**6+2812*y**7-590*y**8+53*y**9))-2*MBhat**2*(35*r**16*(
        -1+y)**8-14*r**2*(-1+y)**8*(-425+144*y**5+53*y**6)-70*r**6*(-1+y)**4*(
        -1737+3660*logy*(-1+y)**4+10608*y-23232*y**2+22808*y**3-9362*y**4
        +156*y**5+28*y**6+100*y**7-41*y**8)+70*r**8*(-1+y)**3*(3375-2400*logy*(
        -1+y)**5-14475*y+22950*y**2-14950*y**3+1475*y**4+3041*y**5-479*y**6
        +91*y**7+16*y**8)-(-1+y)**8*(185-1008*y**5-602*y**6+272*y**7+63*y**8)
        -2*r**14*(275-2200*y+7700*y**2-15400*y**3+19250*y**4-17416*y**5+8708*y**6
        -2488*y**7+311*y**8)+14*r**10*(-1+y)**2*(-3585-300*logy*(-1+y)**6
        +21210*y-52125*y**2+68000*y**3-49500*y**4+22788*y**5-3811*y**6+1026*y**7
        -111*y**8+8*y**9)+14*r**4*(-1+y)**5*(-11075-4500*logy*(-1+y)**3+37725*y
        -44475*y**2+19325*y**3-1125*y**4+423*y**5+521*y**6-531*y**7+42*y**8
        +10*y**9)-14*r**12*(-325+2600*y-9100*y**2+18200*y**3-22750*y**4+19856*y**5
        -11386*y**6+3672*y**7-855*y**8+88*y**9))+MBhat**4*(-525*r**16*(-1+y)**8
        +1400*r**6*(-1+y)**4*(411+468*logy*(-1+y)**4-1176*y+828*y**2+368*y**3
        -681*y**4+156*y**5-32*y**6)-28*r**2*(-1+y)**8*(3020-320*y**3-375*y**4
        -24*y**5+9*y**6)+210*r**4*(-1+y)**5*(3568+2160*logy*(-1+y)**3-12864*y
        +16104*y**2-7496*y**3+159*y**4+211*y**5+73*y**6-35*y**7)+(-1+y)**8*(4365
        -2240*y**3-5810*y**4-896*y**5+1736*y**6+320*y**7+5*y**8)-140*r**8*(
        -1+y)**3*(1800-330*logy*(-1+y)**5-8670*y+16515*y**2-15015*y**3+7800*y**4
        -1602*y**5+700*y**6-164*y**7+16*y**8)-84*r**10*(-1+y)**2*(-140+840*y
        -2100*y**2+3440*y**3-1905*y**4+926*y**5+198*y**6-194*y**7+35*y**8)
        -40*r**14*(-107+856*y-2996*y**2+6104*y**3-7756*y**4+6356*y**5-3304*y**6
        +1016*y**7-154*y**8+6*y**9)+14*r**12*(-960+7680*y-26880*y**2+55520*y**3
        -70955*y**4+58632*y**5-31668*y**6+10152*y**7-1533*y**8-8*y**9+20*y**10))
        +70*MBhat**8*(-1+y)**4*(24*r**2*(-1+y)**4+3*r**8*(-1+y)**4+(-1+y)**4*(
        -3+5*y**4)+4*r**6*(-6+24*y-36*y**2+24*y**3-11*y**4+2*y**5)-6*r**4*(
        -1+y)*(6*logy*(-1+y)**3+y*(-6+15*y-11*y**2+4*y**3))))/(420*(-1
        +y)**8)

    return 0.

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def XG(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^2 level functions of muG for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-9-15*r**8-8*y**3-5*y**4-(24*r**2*(1-y+y**3))/(-1+y)+(18*r**4*(
        -4+2*logy*(-1+y)**2+10*y-7*y**2+2*y**3))/(-1+y)**2-(4*r**6*(18
        -54*y+54*y**2-28*y**3+5*y**4))/(-1+y)**3)/6
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-9*r**10*(-1+y)**4+r**8*(79-316*y+474*y**2-316*y**3+129*y**4-20*y**5)
        +r**2*(-1+y)**3*(7+48*logy*(-1+y)-55*y+24*y**2+8*y**3-27*y**4
        -5*y**5)+2*r**6*(-1+y)*(24+54*logy*(-1+y)**3-126*y+207*y**2-123*y**3
        +55*y**4-5*y**5)-(-1+y)**4*(15+7*y**4+4*y**5)+6*r**4*(-1+y)**2*(
        -6*logy*(-1+y)**2+y*(-6+9*y-2*y**2+y**3))-2*MBhat**2*(-1+y)*(15*r**8*(
        -1+y)**3+24*r**2*(-1+y)**2*(1-y+y**3)-18*r**4*(-1+y)*(-4+2*logy*(
        -1+y)**2+10*y-7*y**2+2*y**3)+4*r**6*(18-54*y+54*y**2-28*y**3+5*y**4)
        +(-1+y)**3*(9+8*y**3+5*y**4))-2*MBhat*(14*r**10*(-1+y)**4+2*r**2*(
        -1+y)**3*(-17+12*logy*(-1+y)+5*y+6*y**2-24*y**3+6*y**4)-(-1
        +y)**4*(18+10*y**3+6*y**4+y**5)+5*r**8*(-14+56*y-84*y**2+66*y**3
        -24*y**4+3*y**5)-2*r**4*(-1+y)**2*(52-12*logy*(-1+y)**2-116*y+70*y**2
        +8*y**3-31*y**4+5*y**5)+r**6*(144-576*y+864*y**2-644*y**3+218*y**4
        +4*y**5-10*y**6)))/(12*(-1+y)**4)
    if (dQ2 == 0 and dr == 0 and i==2):
        return -1/6*(MBhat**4*(15*r**8*(-1+y)**3+24*r**2*(-1+y)**2*(1-y+y**3)-18*r**4*(
        -1+y)*(-4+2*logy*(-1+y)**2+10*y-7*y**2+2*y**3)+4*r**6*(18-54*y
        +54*y**2-28*y**3+5*y**4)+(-1+y)**3*(9+8*y**3+5*y**4)))/(-1+y)**3-(MBhat**3*(14*r**10*(-1+y)**4+2*r**2*(-1+y)**3*(-17+12*logy*(-1+y)        +5*y+6*y**2-24*y**3+6*y**4)-(-1+y)**4*(18+10*y**3+6*y**4+y**5)
        +5*r**8*(-14+56*y-84*y**2+66*y**3-24*y**4+3*y**5)-2*r**4*(-1+y)**2*(52
        -12*logy*(-1+y)**2-116*y+70*y**2+8*y**3-31*y**4+5*y**5)+r**6*(144
        -576*y+864*y**2-644*y**3+218*y**4+4*y**5-10*y**6)))/(3*(-1+y)**4)        -(2*r**12*(-1+y)**5+2*r**4*(-1+y)**3*(-121+12*logy*(-1+y)**2+254*y
        -139*y**2+4*y**3+y**4+4*y**5)-4*r**6*(-1+y)**2*(52-24*logy*(-1+y)**3
        -132*y+96*y**2-8*y**3-6*y**4+7*y**5)-r**8*(-1+y)*(22+72*logy*(
        -1+y)**4-16*y-120*y**2+224*y**3-128*y**4+62*y**5-5*y**6)+(-1+y)**5*(6
        +2*y**5+y**6)+2*r**2*(-1+y)**4*(-38-24*logy*(-1+y)+62*y-12*y**2
        -4*y**3-2*y**4+5*y**5+y**6)+2*r**10*(14-70*y+140*y**2-140*y**3+70*y**4
        -23*y**5+3*y**6))/(6*(-1+y)**5)+MBhat*((-10*r**12)/9+r**10*(32/5-4/(
        -1+y)**5-12/(-1+y)**4-10/(-1+y)**3-2*y)+r**4*(-92+24*logy-14/(
        -1+y)+10*y-2*y**2-6*y**3)+r**2*(38-24*logy+16/(-1+y)-8*y
        +4*y**2+8*y**3+(4*y**4)/3)+(2*r**8*(-42+168*y-252*y**2+168*y**3-80*y**4
        +11*y**5))/(3*(-1+y)**4)+(2*r**6*(-324+24*logy*(-1+y)**3+948*y-912*y**2
        +280*y**3-60*y**4-39*y**5+5*y**6))/(9*(-1+y)**3)+(2*(106+30*y**4
        +24*y**5+5*y**6))/45)+MBhat**2*((-20*r**12)/9+r**10*(101/10+2/(-1+y)**5
        +3/(-1+y)**4-2*y)+r**4*(52-18*logy+6/(-1+y)**2+46/(-1+y)+16*y
        +19*y**2-4*y**3)+r**2*(-51/2+24*logy-24/(-1+y)-12*y**2-2*y**3
        -y**4/6)+(-739-180*y**3-225*y**4-96*y**5-5*y**6)/90+(r**8*(-99+396*y
        -594*y**2+528*y**3-107*y**4-40*y**5+15*y**6))/(6*(-1+y)**4)+(r**6*(78*logy
        +(-48+66*y+51*y**2-59*y**3+411*y**4-165*y**5+20*y**6)/(-1+y)**3))/9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-5*r**14)/28+r**4*(203/4+15*logy-3/(-1+y)**2-21/(-1+y)-(9*y**2)/2
        -4*y**3-(9*y**4)/4)+r**2*(-653/20+12*logy-4/(-1+y)+8*y+2*y**2
        -y**4-(8*y**5)/5-y**6/4)+(-345-91*y**6-40*y**7)/420+(r**10*(-245
        +300*logy*(-1+y)**5+925*y-1100*y**2+100*y**3+700*y**4-440*y**5+219*y**6
        -15*y**7))/(20*(-1+y)**5)+(r**6*(-1260*logy-352*y+18*y**2+32*y**3
        +5*y**4+(-945+3743*y-4911*y**2+2177*y**3)/(-1+y)**3))/12+MBhat**4*(
        -1253/60-(20*r**12)/3+r**10*(651/20+6/(-1+y)**5+9/(-1+y)**4-6*y)
        -6*y**3-(23*y**4)/4-(11*y**5)/5-y**6/6+(r**8*(-283+168/(-1+y)**4
        +408/(-1+y)**3+246/(-1+y)**2+60*y+30*y**2))/4+(3*r**4*(96-30*logy
        +10/(-1+y)**2+82/(-1+y)+32*y+37*y**2-8*y**3))/2+r**6*(122-logy
        +76/(-1+y)**3+253/(-1+y)**2+271/(-1+y)-8*y-(65*y**2)/2+(20*y**3)/3)
        +r**2*(-251/4+60*logy-60/(-1+y)-30*y**2+2*y**3+(3*y**4)/4))-(MBhat**6*(15*r**8*(
        -1+y)**3+24*r**2*(-1+y)**2*(1-y+y**3)-18*r**4*(-1+y)*(-4+2*logy*(
        -1+y)**2+10*y-7*y**2+2*y**3)+4*r**6*(18-54*y+54*y**2-28*y**3+5*y**4)
        +(-1+y)**3*(9+8*y**3+5*y**4)))/(6*(-1+y)**3)-(MBhat**5*(14*r**10*(-1
        +y)**4+2*r**2*(-1+y)**3*(-17+12*logy*(-1+y)+5*y+6*y**2-24*y**3
        +6*y**4)-(-1+y)**4*(18+10*y**3+6*y**4+y**5)+5*r**8*(-14+56*y-84*y**2
        +66*y**3-24*y**4+3*y**5)-2*r**4*(-1+y)**2*(52-12*logy*(-1+y)**2-116*y
        +70*y**2+8*y**3-31*y**4+5*y**5)+r**6*(144-576*y+864*y**2-644*y**3
        +218*y**4+4*y**5-10*y**6)))/(2*(-1+y)**4)+(r**8*(-180*logy+(-535+1960*y
        -2580*y**2+1360*y**3-160*y**4-36*y**5+35*y**6)/(-1+y)**4))/4+r**12*((
        -2*y)/3+(219-1274*y+3045*y**2-3780*y**3+2485*y**4-714*y**5+119*y**6)/(60*(
        -1+y)**6))+MBhat**3*(866/35-(15*r**14)/7+(r**12*(149-25/(-1+y)**6
        -35/(-1+y)**5-25*y))/15+(7*y**3)/3+(35*y**4)/6+4*y**5+(13*y**6)/15
        +y**7/42+r**10*(-35/2-126/(5*(-1+y)**5)-633/(10*(-1+y)**4)-42/(
        -1+y)**3+(6*y)/5+3*y**2)+r**4*(-515+132*logy-102/(-1+y)+30*y
        -36*y**2-13*y**3)+r**2*(912/5-120*logy+80/(-1+y)-40*y+20*y**2
        +20*y**3+(5*y**4)/2-(2*y**5)/5)+(r**6*(-903+324*logy*(-1+y)**3+2385*y
        -1899*y**2+389*y**3-358*y**4+66*y**5-12*y**6))/(3*(-1+y)**3)+(r**8*(
        -20+80*y-120*y**2+30*y**3-271*y**4+160*y**5-48*y**6+5*y**7))/(2*(
        -1+y)**4))+MBhat*((-13*r**14)/21+r**12*(14/3-5/(-1+y)**6-21/(-1
        +y)**5-63/(2*(-1+y)**4)-35/(2*(-1+y)**3)-(3*y)/2)+r**2*(427/3-60*logy
        +24/(-1+y)-36*y-6*y**2+4*y**3+9*y**4+3*y**5+y**6/6)-(r**4*(744
        -1488*y+744*y**2-29*y**5+4*y**6+y**7))/(2*(-1+y)**2)-(r**10*(-330
        +1650*y-3300*y**2+3300*y**3-1650*y**4+704*y**5-103*y**6+5*y**7))/(10*(
        -1+y)**5)+(r**8*(1500+240*logy*(-1+y)**4-5760*y+8160*y**2-4960*y**3
        +1000*y**4-213*y**5-47*y**6+5*y**7))/(6*(-1+y)**4)+(1180+231*y**5
        +203*y**6+45*y**7)/210+r**6*(340*logy+(-75-795*y+2325*y**2-1795*y**3
        +255*y**4+21*y**5-38*y**6)/(3*(-1+y)**3)))+MBhat**2*((-55*r**14)/42+r**12*(15/2
        +5/(-1+y)**6+14/(-1+y)**5+21/(2*(-1+y)**4)-2*y)+r**4*(644-96*logy
        -15/(-1+y)**2-12/(-1+y)-78*y-15*y**2+16*y**3+4*y**4)+r**2*(-709/3
        +120*logy-60/(-1+y)+60*y-20*y**3-(45*y**4)/4-y**5+y**6/12)+(
        -2262-315*y**4-476*y**5-203*y**6-20*y**7)/140+(r**10*(364-1820*y
        +3640*y**2-3640*y**3+3025*y**4-833*y**5+41*y**6+15*y**7))/(20*(-1+y)**5)
        +r**6*(-342*logy+70*y+47*y**2-4*y**3+(753-1435*y+223*y**2+631*y**3)/(2*(
        -1+y)**3))+r**8*(2*logy+(-738+2976*y-4512*y**2+3056*y**3-23*y**4
        +600*y**5-191*y**6+20*y**7)/(12*(-1+y)**4)))
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-45*r**16*(-1+y)**7+42*r**4*(-1+y)**5*(358+660*logy*(-1+y)**2-56*y
        -632*y**2+220*y**3+55*y**4+22*y**5+11*y**6-28*y**7)+336*r**10*(-1
        +y)**2*(419-120*logy*(-1+y)**5-1975*y+3650*y**2-3250*y**3+1325*y**4
        -145*y**5-20*y**6+16*y**7)+14*r**12*(-1+y)*(654+540*logy*(-1+y)**6
        -3384*y+6840*y**2-6420*y**3+2115*y**4+774*y**5-669*y**6+340*y**7
        -20*y**8)+16*r**2*(-1+y)**6*(1241+420*logy*(-1+y)-1661*y+210*y**2
        +70*y**3+35*y**4+21*y**5+14*y**6-30*y**7-5*y**8)+280*r**6*(-1+y)**4*(
        -774-360*logy*(-1+y)**3+2682*y-3222*y**2+1434*y**3-90*y**4-18*y**5
        -6*y**6-2*y**7+y**8)+70*r**8*(-1+y)**3*(-1160-2940*logy*(-1+y)**4
        +1700*y+3330*y**2-8100*y**3+4965*y**4-588*y**5-98*y**6+36*y**7+5*y**8)
        -8*r**14*(158-1106*y+3318*y**2-5530*y**3+5530*y**4-3318*y**5+1106*y**6
        -258*y**7+25*y**8)-(-1+y)**7*(291+64*y**7+25*y**8)-70*MBhat**8*(-1
        +y)**4*(15*r**8*(-1+y)**3+24*r**2*(-1+y)**2*(1-y+y**3)-18*r**4*(-1
        +y)*(-4+2*logy*(-1+y)**2+10*y-7*y**2+2*y**3)+4*r**6*(18-54*y
        +54*y**2-28*y**3+5*y**4)+(-1+y)**3*(9+8*y**3+5*y**4))-280*MBhat**7*(
        -1+y)**3*(14*r**10*(-1+y)**4+2*r**2*(-1+y)**3*(-17+12*logy*(-1+y)
        +5*y+6*y**2-24*y**3+6*y**4)-(-1+y)**4*(18+10*y**3+6*y**4+y**5)
        +5*r**8*(-14+56*y-84*y**2+66*y**3-24*y**4+3*y**5)-2*r**4*(-1+y)**2*(52
        -12*logy*(-1+y)**2-116*y+70*y**2+8*y**3-31*y**4+5*y**5)+r**6*(144
        -576*y+864*y**2-644*y**3+218*y**4+4*y**5-10*y**6))-28*MBhat**6*(-1
        +y)**2*(200*r**12*(-1+y)**5-5*r**2*(-1+y)**4*(13+336*logy*(-1+y)-349*y
        +168*y**2-196*y**3+21*y**4+7*y**5)+30*r**4*(-1+y)**3*(-36+42*logy*(
        -1+y)**2+114*y-99*y**2+74*y**3-79*y**4+12*y**5)+(-1+y)**5*(589
        +180*y**3+155*y**4+56*y**5+5*y**6)+9*r**10*(121-605*y+1210*y**2-1310*y**3
        +755*y**4-211*y**5+20*y**6)-10*r**6*(-1+y)**2*(-96-30*logy*(-1+y)**3
        +318*y-363*y**2+187*y**3+301*y**4-155*y**5+20*y**6)-5*r**8*(455-2275*y
        +4550*y**2-4946*y**3+2795*y**4-499*y**5-125*y**6+45*y**7))-8*MBhat**5*(
        -1+y)*(450*r**14*(-1+y)**6+210*r**4*(-1+y)**5*(-296-96*logy*(-1+y)
        +392*y-48*y**2+29*y**3+4*y**4)+21*r**2*(-1+y)**5*(694+840*logy*(
        -1+y)-1534*y+420*y**2-60*y**3-75*y**4-9*y**5+4*y**6)+70*r**6*(
        -1+y)**3*(579-300*logy*(-1+y)**3-1437*y+987*y**2-109*y**3+298*y**4
        -105*y**5+17*y**6)-105*r**8*(-1+y)**2*(64-256*y+384*y**2-306*y**3
        -111*y**4+138*y**5-48*y**6+5*y**7)-(-1+y)**6*(3712+490*y**3+805*y**4
        +504*y**5+112*y**6+5*y**7)+14*r**12*(-184+1104*y-2760*y**2+3855*y**3
        -3110*y**4+1419*y**5-324*y**6+25*y**7)-21*r**10*(-262+1572*y-3930*y**2
        +5600*y**3-4455*y**4+1656*y**5-73*y**6-138*y**7+30*y**8))-2*MBhat**2*(175*r**16*(
        -1+y)**7+70*r**6*(-1+y)**4*(6765+9660*logy*(-1+y)**3-29955*y+44445*y**2
        -24475*y**3+2415*y**4+555*y**5-133*y**6-41*y**7)+210*r**8*(-1+y)**3*(3365
        +1800*logy*(-1+y)**4-11660*y+13890*y**2-5660*y**3-385*y**4+252*y**5
        -111*y**6+4*y**7)+42*r**2*(-1+y)**6*(-3881-1680*logy*(-1+y)+5561*y
        -840*y**2-280*y**3-140*y**4+36*y**5+87*y**6+17*y**7)-14*r**4*(-1+y)**5*(32507
        +4140*logy*(-1+y)**2-60874*y+26297*y**2+1380*y**3+345*y**4-762*y**5
        -251*y**6+88*y**7+10*y**8)-14*r**10*(-1+y)**2*(3645-300*logy*(-1+y)**5
        -17925*y+35100*y**2-34100*y**3+16300*y**4-296*y**5+821*y**6-217*y**7
        +20*y**8)+18*r**14*(89-623*y+1869*y**2-3115*y**3+3115*y**4-2149*y**5
        +903*y**6-209*y**7+20*y**8)+(-1+y)**7*(5095+504*y**5+826*y**6+368*y**7
        +45*y**8)-14*r**12*(307-2149*y+6447*y**2-10745*y**3+10745*y**4-8067*y**5
        +4403*y**6-1045*y**7+104*y**8))-8*MBhat*(20*r**16*(-1+y)**7+35*r**8*(
        -1+y)**3*(-2480-2400*logy*(-1+y)**4+7520*y-6480*y**2-480*y**3+2520*y**4
        -480*y**5+3*y**6+33*y**7)+r**2*(-1+y)**6*(15616+5880*logy*(-1+y)
        -21496*y+2940*y**2+980*y**3+490*y**4+294*y**5-294*y**6-195*y**7-15*y**8)
        +70*r**6*(-1+y)**4*(-1494-1080*logy*(-1+y)**3+5562*y-7182*y**2+3474*y**3
        -270*y**4-54*y**5-20*y**6+9*y**7+y**8)-7*r**10*(-1+y)**2*(-4768+840*logy*(
        -1+y)**5+23000*y-43900*y**2+41100*y**3-18450*y**4+2850*y**5-478*y**6
        -55*y**7+5*y**8)+7*r**4*(-1+y)**5*(4256+1920*logy*(-1+y)**2-6592*y
        +1376*y**2+640*y**3+160*y**4+64*y**5-133*y**6-16*y**7+5*y**8)-(-1
        +y)**7*(324+49*y**6+45*y**7+10*y**8)+4*r**14*(73-511*y+1533*y**2
        -2555*y**3+2555*y**4-1533*y**5+651*y**6-153*y**7+15*y**8)+7*r**12*(
        -288+2016*y-6048*y**2+10080*y**3-10080*y**4+6048*y**5-2381*y**6+743*y**7
        -95*y**8+5*y**9))-8*MBhat**3*(75*r**16*(-1+y)**7+350*r**6*(-1+y)**4*(9
        -516*logy*(-1+y)**3+489*y-1263*y**2+937*y**3-139*y**4+5*y**5+8*y**6)
        +35*r**8*(-1+y)**3*(-3730-1200*logy*(-1+y)**4+13720*y-18180*y**2
        +9720*y**3-1105*y**4+343*y**5-39*y**6+6*y**7)+21*r**4*(-1+y)**5*(9142
        -360*logy*(-1+y)**2-18644*y+9682*y**2-120*y**3-255*y**4-147*y**5
        +49*y**6+13*y**7)-21*r**10*(-1+y)**2*(10-50*y+100*y**2-100*y**3-400*y**4
        -436*y**5+213*y**6-54*y**7+5*y**8)+r**2*(-1+y)**6*(55970+29400*logy*(
        -1+y)-85370*y+14700*y**2+4900*y**3+700*y**4-1904*y**5-749*y**6
        -12*y**7+5*y**8)-(-1+y)**7*(2885+175*y**4+469*y**5+343*y**6+86*y**7
        +5*y**8)+2*r**14*(241-1687*y+5061*y**2-8435*y**3+9135*y**4-6181*y**5
        +2527*y**6-561*y**7+50*y**8)-7*r**12*(110-770*y+2310*y**2-3850*y**3
        +4725*y**4-3612*y**5+1260*y**6-144*y**7-39*y**8+10*y**9))-MBhat**4*(875*r**16*(
        -1+y)**7-1400*r**6*(-1+y)**4*(477-540*logy*(-1+y)**3-891*y+81*y**2
        +485*y**3-66*y**4+32*y**5)+70*r**4*(-1+y)**5*(-18456+3120*logy*(-1
        +y)**2+40032*y-23136*y**2+1520*y**3+755*y**4-190*y**5-105*y**6)-28*r**2*(
        -1+y)**6*(11920+8400*logy*(-1+y)-20320*y+4200*y**2+1000*y**3-535*y**4
        -453*y**5-21*y**6+9*y**7)+140*r**8*(-1+y)**3*(1820+330*logy*(-1+y)**4
        -6950*y+9765*y**2-5810*y**3-370*y**4+216*y**5-78*y**6+12*y**7)+(
        -1+y)**7*(32805+1120*y**3+4830*y**4+5488*y**5+2240*y**6+288*y**7+5*y**8)
        +40*r**14*(107-749*y+2247*y**2-3885*y**3+4095*y**4-2667*y**5+1029*y**6
        -207*y**7+15*y**8)-28*r**10*(-1+y)**2*(40-200*y+400*y**2+560*y**3
        +4565*y**4-3661*y**5+1735*y**6-431*y**7+40*y**8)-14*r**12*(360-2520*y
        +7560*y**2-14200*y**3+14245*y**4-6139*y**5-399*y**6+1705*y**7-712*y**8
        +100*y**9)))/(420*(-1+y)**7)

    return 0.

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def XD(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^3 level functions of rhoD for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (77-5*r**8-12*logy*(4+3*r**4)-48*y-24*y**2+16*y**3+y**4+(8*r**2*(
        -11+22*y-11*y**2-4*y**3+2*y**4))/(-1+y)**2-(4*r**6*(-2+10*y-20*y**2
        +12*y**3-5*y**4+y**5))/(-1+y)**5-(2*r**4*(-12+66*y-135*y**2+110*y**3
        -43*y**4+8*y**5))/(-1+y)**4)/6
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-60*logy*(24+8*r**2-69*r**4+27*r**6+16*MBhat*(-3+r**2)+6*MBhat**2*(4
        +3*r**4))*(-1+y)**6+30*MBhat**2*(-1+y)*(-5*r**8*(-1+y)**5+(-1+y)**5*(77
        -48*y-24*y**2+16*y**3+y**4)+8*r**2*(-1+y)**3*(-11+22*y-11*y**2
        -4*y**3+2*y**4)-4*r**6*(-2+10*y-20*y**2+12*y**3-5*y**4+y**5)-2*r**4*(12
        -78*y+201*y**2-245*y**3+153*y**4-51*y**5+8*y**6))-2*MBhat*(168*r**10*(
        -1+y)**6+3*(-1+y)**6*(904-480*y-240*y**2+70*y**3+y**5)-40*r**2*(
        -1+y)**4*(133-278*y+151*y**2+23*y**3-22*y**4+5*y**5)+30*r**4*(-1
        +y)**2*(112-448*y+672*y**2-410*y**3+76*y**4+21*y**5-12*y**6+y**7)
        -120*r**6*(4-24*y+60*y**2-65*y**3+31*y**4-4*y**5-3*y**6+y**7)+5*r**8*(
        -88+528*y-1320*y**2+1970*y**3-1620*y**4+753*y**5-178*y**6+15*y**7))
        -5*(9*r**10*(-1+y)**6-3*(-1+y)**6*(243-96*y-48*y**2-32*y**3+11*y**4)
        +6*r**4*(-1+y)**2*(-216+726*y-813*y**2+266*y**3+34*y**4-12*y**5+3*y**6)
        -r**2*(-1+y)**4*(-1831+3566*y-1687*y**2-32*y**3-173*y**4+58*y**5
        +3*y**6)+r**8*(41-246*y+615*y**2-820*y**3+405*y**4-138*y**5+23*y**6)
        -6*r**6*(-24+90*y-63*y**2-186*y**3+347*y**4-242*y**5+99*y**6-22*y**7
        +y**8)))/(180*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-2*logy*(16*MBhat**3*(-3+r**2)+3*MBhat**4*(4+3*r**4)+16*MBhat*(-3+25*r**4)
        +MBhat**2*(72-24*r**2-153*r**4+35*r**6)+2*(6+4*r**2-131*r**4-56*r**6
        +9*r**8)))/3+(-2238+8840*r**2+10550*r**4-18400*r**6+1150*r**8+88*r**10
        +10*r**12+16386*y-61400*r**2*y-89570*r**4*y+122080*r**6*y-6970*r**8*y
        -616*r**10*y-70*r**12*y-51678*y**2+182520*r**2*y**2+323730*r**4*y**2-342720*r**6*y**2
        +17130*r**8*y**2+1848*r**10*y**2+210*r**12*y**2+91170*y**3-300840*r**2*y**3
        -649590*r**4*y**3+524160*r**6*y**3-20990*r**8*y**3-3080*r**10*y**3-350*r**12*y**3
        -97470*y**4+296640*r**2*y**4+787140*r**4*y**4-465360*r**6*y**4+11540*r**8*y**4
        +3080*r**10*y**4+350*r**12*y**4+63330*y**5-174240*r**2*y**5-582900*r**4*y**5
        +233520*r**6*y**5-900*r**8*y**5-1272*r**10*y**5-210*r**12*y**5-22465*y**6
        +54480*r**2*y**6+251070*r**4*y**6-56780*r**6*y**6-1955*r**8*y**6+364*r**10*y**6
        +70*r**12*y**6+775*y**7-3120*r**2*y**7-53610*r**4*y**7+2420*r**6*y**7+1325*r**8*y**7
        -52*r**10*y**7-10*r**12*y**7+4845*y**8-5160*r**2*y**8+3030*r**4*y**8+1260*r**6*y**8
        -345*r**8*y**8-4315*y**9+3160*r**2*y**9+70*r**4*y**9-180*r**6*y**9+15*r**8*y**9
        +2203*y**10-1000*r**2*y**10+80*r**4*y**10-621*y**11+120*r**2*y**11+83*y**12
        -5*y**13+15*MBhat**4*(-1+y)**2*(-5*r**8*(-1+y)**5+(-1+y)**5*(77-48*y
        -24*y**2+16*y**3+y**4)+8*r**2*(-1+y)**3*(-11+22*y-11*y**2-4*y**3
        +2*y**4)-4*r**6*(-2+10*y-20*y**2+12*y**3-5*y**4+y**5)-2*r**4*(12
        -78*y+201*y**2-245*y**3+153*y**4-51*y**5+8*y**6))-2*MBhat**3*(-1
        +y)*(168*r**10*(-1+y)**6+3*(-1+y)**6*(904-480*y-240*y**2+70*y**3
        +y**5)-40*r**2*(-1+y)**4*(133-278*y+151*y**2+23*y**3-22*y**4+5*y**5)
        +30*r**4*(-1+y)**2*(112-448*y+672*y**2-410*y**3+76*y**4+21*y**5-12*y**6
        +y**7)-120*r**6*(4-24*y+60*y**2-65*y**3+31*y**4-4*y**5-3*y**6+y**7)
        +5*r**8*(-88+528*y-1320*y**2+1970*y**3-1620*y**4+753*y**5-178*y**6
        +15*y**7))-4*MBhat*(20*r**12*(-1+y)**7+120*r**2*(-1+y)**5*(-56+112*y
        -56*y**2-3*y**4+y**5)+2*(-1+y)**7*(982-360*y-180*y**2-120*y**3
        +3*y**5)-5*r**4*(-1+y)**3*(-380+320*y+1920*y**2-3680*y**3+2024*y**4
        -195*y**5-56*y**6+11*y**7)+5*r**6*(-1+y)**2*(-640+3200*y-6400*y**2
        +6400*y**3-2984*y**4+565*y**5-5*y**6-17*y**7+y**8)-15*r**8*(-20+140*y
        -420*y**2+700*y**3-616*y**4+273*y**5-55*y**6-5*y**7+3*y**8)+r**10*(64
        -448*y+1344*y**2-2240*y**3+2720*y**4-1869*y**5+763*y**6-169*y**7
        +15*y**8))-MBhat**2*(280*r**12*(-1+y)**7-(-1+y)**7*(9825-4320*y-2160*y**2
        -480*y**3+135*y**4+y**6)+15*r**2*(-1+y)**5*(1773-3642*y+1917*y**2
        +96*y**3-47*y**4-2*y**5+y**6)+30*r**4*(-1+y)**3*(-644+2270*y-2793*y**2
        +1186*y**3+14*y**4-92*y**5+23*y**6)+10*r**6*(-1+y)**2*(-176+670*y
        -815*y**2-269*y**3+520*y**4-278*y**5+51*y**6+9*y**7)+3*r**10*(385
        -2695*y+8085*y**2-14115*y**3+14805*y**4-9555*y**5+3633*y**6-711*y**7
        +48*y**8)-15*r**8*(111-777*y+2331*y**2-4205*y**3+4585*y**4-3007*y**5
        +1176*y**6-216*y**7-3*y**8+5*y**9)))/(90*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return -(logy*(8+8*r**2-585*r**4-925*r**6-165*r**8+15*r**10+16*MBhat**5*(
        -3+r**2)+MBhat**6*(8+6*r**4)+32*MBhat**3*(-5+2*r**2+36*r**4)+MBhat**4*(120
        -56*r**2-237*r**4+43*r**6)+16*MBhat*(-3-r**2+117*r**4+95*r**6)+2*MBhat**2*(60
        -8*r**2-1104*r**4-335*r**6+29*r**8)))+(11897-66843*r**2-377475*r**4
        +207025*r**6+235375*r**8-9597*r**10-357*r**12-25*r**14-98536*y+531384*r**2*y
        +3265500*r**4*y-1267700*r**6*y-1813700*r**8*y+70476*r**10*y+2856*r**12*y
        +200*r**14*y+358316*y**2-1846404*r**2*y**2-12412050*r**4*y**2+2882950*r**6*y**2
        +6070750*r**8*y**2-221466*r**10*y**2-9996*r**12*y**2-700*r**14*y**2-747992*y**3
        +3661448*r**2*y**3+27117300*r**4*y**3-2139900*r**6*y**3-11494700*r**8*y**3
        +384132*r**10*y**3+19992*r**12*y**3+1400*r**14*y**3+982030*y**4-4529770*r**2*y**4
        -37336425*r**4*y**4-2764125*r**6*y**4+13398175*r**8*y**4-391965*r**10*y**4
        -24990*r**12*y**4-1750*r**14*y**4-832664*y**5+3576776*r**2*y**5+33308940*r**4*y**5
        +7650300*r**6*y**5-9748340*r**8*y**5+225372*r**10*y**5+19992*r**12*y**5+1400*r**14*y**5
        +448427*y**6-1758743*r**2*y**6-18945220*r**4*y**6-7458500*r**6*y**6+4240845*r**8*y**6
        -64421*r**10*y**6-7056*r**12*y**6-700*r**14*y**6-146784*y**7+499408*r**2*y**7
        +6386100*r**4*y**7+3704260*r**6*y**7-959700*r**8*y**7+3948*r**10*y**7+1736*r**12*y**7
        +200*r**14*y**7+38445*y**8-86751*r**2*y**8-1028685*r**4*y**8-866705*r**6*y**8
        +62230*r**8*y**8+5082*r**10*y**8-217*r**12*y**8-25*r**14*y**8-31976*y**9
        +42280*r**2*y**9+14840*r**4*y**9+42840*r**6*y**9+10360*r**8*y**9-1624*r**10*y**9
        +35434*y**10-37058*r**2*y**10+7350*r**4*y**10+9310*r**6*y**10-1295*r**8*y**10
        +63*r**10*y**10-25256*y**11+18984*r**2*y**11-840*r**4*y**11+280*r**6*y**11
        +11228*y**12-5404*r**2*y**12+665*r**4*y**12-35*r**6*y**12-2968*y**13+728*r**2*y**13
        +431*y**14-35*r**2*y**14-32*y**15+70*MBhat**6*(-1+y)**3*(-5*r**8*(-1+y)**5
        +(-1+y)**5*(77-48*y-24*y**2+16*y**3+y**4)+8*r**2*(-1+y)**3*(-11
        +22*y-11*y**2-4*y**3+2*y**4)-4*r**6*(-2+10*y-20*y**2+12*y**3-5*y**4
        +y**5)-2*r**4*(12-78*y+201*y**2-245*y**3+153*y**4-51*y**5+8*y**6))
        -14*MBhat**5*(-1+y)**2*(168*r**10*(-1+y)**6+3*(-1+y)**6*(904-480*y
        -240*y**2+70*y**3+y**5)-40*r**2*(-1+y)**4*(133-278*y+151*y**2+23*y**3
        -22*y**4+5*y**5)+30*r**4*(-1+y)**2*(112-448*y+672*y**2-410*y**3+76*y**4
        +21*y**5-12*y**6+y**7)-120*r**6*(4-24*y+60*y**2-65*y**3+31*y**4
        -4*y**5-3*y**6+y**7)+5*r**8*(-88+528*y-1320*y**2+1970*y**3-1620*y**4
        +753*y**5-178*y**6+15*y**7))-2*MBhat*(104*r**14*(-1+y)**8+56*r**2*(
        -1+y)**6*(-2943+5826*y-2853*y**2-20*y**3-5*y**4-77*y**5+9*y**6+3*y**7)
        -(-1+y)**8*(-32936+10080*y+5040*y**2+3360*y**3+2520*y**4+231*y**5
        -56*y**6+15*y**7)-280*r**6*(-1+y)**3*(1665-7185*y+11520*y**2-7720*y**3
        +1010*y**4+881*y**5-201*y**6-15*y**7+3*y**8)+35*r**8*(-1+y)**2*(2120
        -12720*y+31800*y**2-42400*y**3+31800*y**4-11871*y**5+1882*y**6-42*y**7
        -30*y**8+y**9)-21*r**4*(-1+y)**4*(19224-95616*y+180864*y**2-158016*y**3
        +58224*y**4-4009*y**5-604*y**6-184*y**7-4*y**8+y**9)-168*r**10*(27
        -216*y+756*y**2-1512*y**3+1890*y**4-1372*y**5+528*y**6-102*y**7-2*y**8
        +3*y**9)+7*r**12*(-56+448*y-1568*y**2+3136*y**3-3920*y**4+4081*y**5
        -2408*y**6+868*y**7-176*y**8+15*y**9))+7*MBhat**4*(-1+y)*(-560*r**12*(
        -1+y)**7+(-1+y)**7*(16005-7200*y-3600*y**2-480*y**3+105*y**4+2*y**6)
        -5*r**2*(-1+y)**5*(8807-18286*y+9815*y**2+544*y**3-455*y**4+46*y**5
        +9*y**6)-30*r**4*(-1+y)**3*(-1072+3814*y-4773*y**2+2106*y**3-6*y**4
        -172*y**5+43*y**6)-10*r**6*(-1+y)**2*(-280+1142*y-1639*y**2+11*y**3
        +548*y**4-322*y**5+39*y**6+21*y**7)-3*r**10*(785-5495*y+16485*y**2
        -28755*y**3+30135*y**4-19425*y**5+7371*y**6-1437*y**7+96*y**8)+5*r**8*(625
        -4375*y+13125*y**2-23795*y**3+26285*y**4-17499*y**5+6895*y**6-1273*y**7
        -18*y**8+30*y**9))-7*MBhat**2*(110*r**14*(-1+y)**8+(-1+y)**8*(-21278
        +7200*y+3600*y**2+2400*y**3+825*y**4-48*y**5+5*y**6)+r**2*(-1+y)**6*(91700
        -184360*y+93140*y**2-320*y**3+2245*y**4+310*y**5-314*y**6-2*y**7
        +y**8)+60*r**4*(-1+y)**4*(841-5572*y+12774*y**2-12932*y**3+5396*y**4
        -452*y**5-79*y**6-8*y**7+2*y**8)-5*r**8*(-1+y)**2*(-1294+7068*y-15582*y**2
        +17296*y**3-6777*y**4+846*y**5+618*y**6-258*y**7+3*y**8)+10*r**6*(
        -1+y)**3*(12783-59895*y+109740*y**2-96340*y**3+37415*y**4-4015*y**5
        -470*y**6+46*y**7+16*y**8)+2*r**12*(-290+2320*y-8120*y**2+16240*y**3
        -21875*y**4+18760*y**5-10388*y**6+3544*y**7-659*y**8+48*y**9)-3*r**10*(
        -316+2528*y-8848*y**2+17696*y**3-25795*y**4+25144*y**5-15197*y**6
        +6032*y**7-1369*y**8+120*y**9+5*y**10))-2*MBhat**3*(960*r**14*(-1+y)**8
        +(-1+y)**8*(87184-33600*y-16800*y**2-8050*y**3-560*y**4+126*y**5
        +y**7)-14*r**2*(-1+y)**6*(22064-45088*y+23504*y**2+230*y**3+260*y**4
        -167*y**5-6*y**6+3*y**7)-70*r**4*(-1+y)**4*(-1776+3648*y+1440*y**2
        -7967*y**3+5308*y**4-692*y**5-108*y**6+27*y**7)+280*r**6*(-1+y)**3*(
        -392+1960*y-3920*y**2+3965*y**3-1825*y**4+296*y**5+40*y**6-27*y**7
        +3*y**8)-35*r**8*(-1+y)**2*(496-2976*y+7440*y**2-10710*y**3+7092*y**4
        -2288*y**5-164*y**6+351*y**7-86*y**8+5*y**9)+14*r**12*(-336+2688*y
        -9408*y**2+19341*y**3-24920*y**4+20818*y**5-11144*y**6+3604*y**7-608*y**8
        +35*y**9)-14*r**10*(-624+4992*y-17472*y**2+36694*y**3-47760*y**4+39549*y**5
        -20832*y**6+6156*y**7-576*y**8-163*y**9+36*y**10)))/(420*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-39213+295376*r**2+3814356*r**4+1916880*r**6-4547200*r**8-1484112*r**10
        +42924*r**12+944*r**14+45*r**16-840*logy*(12+16*r**2-2157*r**4-6840*r**6
        -4495*r**8-456*r**10+27*r**12+32*MBhat**7*(-3+r**2)+3*MBhat**8*(4+3*r**4)
        +96*MBhat**5*(-7+4*r**2+47*r**4)+2*MBhat**6*(168-88*r**2-321*r**4+51*r**6)
        +32*MBhat**3*(-21+5*r**2+738*r**4+510*r**6)+5*MBhat**4*(168-80*r**2
        -2820*r**4-756*r**6+51*r**8)+32*MBhat*(-3-2*r**2+336*r**4+714*r**6
        +245*r**8)+6*MBhat**2*(56+8*r**2-3665*r**4-4775*r**6-620*r**8+29*r**10))*(
        -1+y)**9+362997*y-2644944*r**2*y-36141084*r**4*y-22997520*r**6*y+37149000*r**8*y
        +12973968*r**10*y-363636*r**12*y-8496*r**14*y-405*r**16*y-1497348*y**2
        +10519296*r**2*y**2+152717796*r**4*y**2+117845280*r**6*y**2-131604900*r**8*y**2
        -50172192*r**10*y**2+1352484*r**12*y**2+33984*r**14*y**2+1620*r**16*y**2
        +3614772*y**3-24383744*r**2*y**3-378084084*r**4*y**3-343919520*r**6*y**3
        +261768500*r**8*y**3+112471968*r**10*y**3-2883636*r**12*y**3-79296*r**14*y**3
        -3780*r**16*y**3-5633838*y**4+36293376*r**2*y**4+605175606*r**4*y**4+636536880*r**6*y**4
        -313360950*r**8*y**4-160664112*r**10*y**4+3849174*r**12*y**4+118944*r**14*y**4
        +5670*r**16*y**4+5887854*y**5-35954688*r**2*y**5-650834982*r**4*y**5-781326000*r**6*y**5
        +218210790*r**8*y**5+151011504*r**10*y**5-3277638*r**12*y**5-118944*r**14*y**5
        -5670*r**16*y**5-4136916*y**6+23687552*r**2*y**6+471939468*r**4*y**6+641541600*r**6*y**6
        -66182060*r**8*y**6-92630496*r**10*y**6+1708812*r**12*y**6+79296*r**14*y**6
        +3780*r**16*y**6+1892004*y**7-9985536*r**2*y**7-224008092*r**4*y**7-343872480*r**6*y**7
        -17005380*r**8*y**7+35177184*r**10*y**7-503580*r**12*y**7-24384*r**14*y**7
        -1620*r**16*y**7-502716*y**8+2400624*r**2*y**8+64181628*r**4*y**8+111757800*r**6*y**8
        +21401100*r**8*y**8-7214256*r**10*y**8+68670*r**12*y**8+5256*r**14*y**8+405*r**16*y**8
        +7884*y**9-141296*r**2*y**9-8990772*r**4*y**9-18107880*r**6*y**9-6283060*r**8*y**9
        +490224*r**10*y**9+11130*r**12*y**9-584*r**14*y**9-45*r**16*y**9+127764*y**10
        -210336*r**2*y**10+223776*r**4*y**10+603120*r**6*y**10+382200*r**8*y**10
        +45360*r**10*y**10-4872*r**12*y**10-177156*y**11+233184*r**2*y**11+2016*r**4*y**11
        -18480*r**6*y**11+71400*r**8*y**11-5040*r**10*y**11+168*r**12*y**11+161574*y**12
        -165760*r**2*y**12+4746*r**4*y**12+37800*r**6*y**12+770*r**8*y**12-98406*y**13
        +73920*r**2*y**13-2394*r**4*y**13+2520*r**6*y**13-210*r**8*y**13+39204*y**14
        -19680*r**2*y**14+2016*r**4*y**14-9780*y**15+2848*r**2*y**15+1425*y**16
        -192*r**2*y**16-105*y**17+210*MBhat**8*(-1+y)**4*(-5*r**8*(-1+y)**5+(
        -1+y)**5*(77-48*y-24*y**2+16*y**3+y**4)+8*r**2*(-1+y)**3*(-11+22*y
        -11*y**2-4*y**3+2*y**4)-4*r**6*(-2+10*y-20*y**2+12*y**3-5*y**4+y**5)
        -2*r**4*(12-78*y+201*y**2-245*y**3+153*y**4-51*y**5+8*y**6))-56*MBhat**7*(
        -1+y)**3*(168*r**10*(-1+y)**6+3*(-1+y)**6*(904-480*y-240*y**2+70*y**3
        +y**5)-40*r**2*(-1+y)**4*(133-278*y+151*y**2+23*y**3-22*y**4+5*y**5)
        +30*r**4*(-1+y)**2*(112-448*y+672*y**2-410*y**3+76*y**4+21*y**5-12*y**6
        +y**7)-120*r**6*(4-24*y+60*y**2-65*y**3+31*y**4-4*y**5-3*y**6+y**7)
        +5*r**8*(-88+528*y-1320*y**2+1970*y**3-1620*y**4+753*y**5-178*y**6
        +15*y**7))-28*MBhat**6*(-1+y)**2*(840*r**12*(-1+y)**7-3*(-1+y)**7*(7395
        -3360*y-1680*y**2-160*y**3+25*y**4+y**6)+5*r**2*(-1+y)**5*(12295
        -25646*y+13879*y**2+800*y**3-769*y**4+98*y**5+15*y**6)+30*r**4*(-1
        +y)**3*(-1500+5358*y-6753*y**2+3026*y**3-26*y**4-252*y**5+63*y**6)
        +30*r**6*(-1+y)**2*(-128+538*y-821*y**2+97*y**3+192*y**4-122*y**5
        +9*y**6+11*y**7)+9*r**10*(395-2765*y+8295*y**2-14465*y**3+15155*y**4
        -9765*y**5+3703*y**6-721*y**7+48*y**8)-5*r**8*(917-6419*y+19257*y**2
        -34975*y**3+38815*y**4-25977*y**5+10262*y**6-1898*y**7-27*y**8+45*y**9))
        -8*MBhat*(48*r**16*(-1+y)**9-3*(-1+y)**9*(-12352+3360*y+1680*y**2
        +1120*y**3+840*y**4+672*y**5+84*y**6-17*y**7+8*y**8)+42*r**4*(-1+y)**5*(
        -47376+216384*y-378336*y**2+305984*y**3-103376*y**4+5376*y**5+996*y**6
        +301*y**7+96*y**8+11*y**9)-r**2*(-1+y)**7*(252496-498272*y+242416*y**2
        +2240*y**3+560*y**4+224*y**5+3892*y**6-41*y**7-170*y**8+15*y**9)-35*r**8*(
        -1+y)**3*(-49840+275520*y-618240*y**2+706720*y**3-412440*y**4+94416*y**5
        +6620*y**6-3573*y**7-174*y**8+31*y**9)+21*r**10*(-1+y)**2*(-5264+36848*y
        -110544*y**2+184240*y**3-184240*y**4+110544*y**5-34380*y**6+4619*y**7
        -93*y**8-51*y**9+y**10)-42*r**6*(-1+y)**4*(8456+14840*y-172480*y**2
        +362880*y**3-324240*y**4+121968*y**5-9860*y**6-1695*y**7-225*y**8-5*y**9
        +y**10)-84*r**12*(-56+504*y-2016*y**2+4704*y**3-7056*y**4+7056*y**5
        -4344*y**6+1461*y**7-259*y**8+y**9+5*y**10)+4*r**14*(52-468*y+1872*y**2
        -4368*y**3+6552*y**4-6552*y**5+6048*y**6-3132*y**7+1008*y**8-187*y**9
        +15*y**10))-24*MBhat**5*(-1+y)*(960*r**14*(-1+y)**8+(-1+y)**8*(59688
        -23520*y-11760*y**2-4690*y**3-560*y**4+42*y**5+y**7)-14*r**2*(-1
        +y)**6*(15344-31648*y+16784*y**2+230*y**3-100*y**4-47*y**5-6*y**6
        +3*y**7)-70*r**4*(-1+y)**4*(-1396+3328*y-480*y**2-4287*y**3+3284*y**4
        -497*y**5-52*y**6+16*y**7)+70*r**6*(-1+y)**3*(-928+4640*y-9280*y**2
        +9460*y**3-4316*y**4+619*y**5+165*y**6-91*y**7+11*y**8)-35*r**8*(-1
        +y)**2*(376-2256*y+5640*y**2-8310*y**3+5796*y**4-1946*y**5-152*y**6
        +333*y**7-86*y**8+5*y**9)+14*r**12*(-356+2848*y-9968*y**2+20461*y**3
        -26320*y**4+21938*y**5-11704*y**6+3764*y**7-628*y**8+35*y**9)-28*r**10*(
        -344+2752*y-9632*y**2+20139*y**3-26360*y**4+22069*y**5-11732*y**6
        +3544*y**7-380*y**8-74*y**9+18*y**10))-6*MBhat**2*(245*r**16*(-1+y)**9
        -14*r**2*(-1+y)**7*(-69959+139438*y-69239*y**2-160*y**3-40*y**4-928*y**5
        -335*y**6+94*y**7+9*y**8)+(-1+y)**9*(-161735+47040*y+23520*y**2
        +15680*y**3+11760*y**4+4704*y**5+42*y**6+64*y**7+15*y**8)+70*r**8*(
        -1+y)**3*(-28565+163950*y-387555*y**2+479540*y**3-322455*y**4+104982*y**5
        -11042*y**6-666*y**7+55*y**8+16*y**9)+70*r**6*(-1+y)**4*(43841-161905*y
        +180560*y**2+10440*y**3-148470*y**4+86418*y**5-10484*y**6-1220*y**7
        -95*y**8+19*y**9)-14*r**10*(-1+y)**2*(4465-29515*y+82455*y**2-125245*y**3
        +110020*y**4-42936*y**5+8249*y**6+989*y**7-721*y**8+39*y**9)+14*r**4*(
        -1+y)**5*(299065-1416160*y+2564040*y**2-2149160*y**3+757190*y**4-44940*y**5
        -8078*y**6-2348*y**7-443*y**8-8*y**9+2*y**10)+2*r**14*(805-7245*y
        +28980*y**2-67620*y**3+101430*y**4-111510*y**5+80724*y**6-38916*y**7
        +11889*y**8-2041*y**9+144*y**10)-14*r**12*(191-1719*y+6876*y**2-16044*y**3
        +24066*y**4-30498*y**5+28026*y**6-15674*y**7+6021*y**8-1389*y**9+144*y**10))
        -8*MBhat**3*(480*r**16*(-1+y)**9-3*(-1+y)**9*(-74584+23520*y+11760*y**2
        +7840*y**3+4340*y**4+749*y**5+14*y**7)-420*r**6*(-1+y)**4*(6696-29400*y
        +48600*y**2-35000*y**3+7230*y**4+2395*y**5-665*y**6-65*y**7+13*y**8)
        -r**2*(-1+y)**7*(1177320-2371440*y+1202520*y**2-5600*y**3+11620*y**4
        +11158*y**5-1176*y**6-879*y**7-6*y**8+3*y**9)-21*r**4*(-1+y)**5*(103136
        -530624*y+1032096*y**2-924224*y**3+348536*y**4-24375*y**5-4420*y**6
        -920*y**7-60*y**8+15*y**9)+35*r**8*(-1+y)**3*(9320-55920*y+139800*y**2
        -186400*y**3+142380*y**4-53043*y**5+7750*y**6+303*y**7-324*y**8+34*y**9)
        -21*r**10*(-1+y)**2*(-1112+7784*y-23352*y**2+38920*y**3-44740*y**4
        +24822*y**5-8234*y**6+929*y**7+261*y**8-83*y**9+5*y**10)+4*r**14*(736
        -6624*y+26496*y**2-61824*y**3+96936*y**4-101430*y**5+72030*y**6-34020*y**7
        +10080*y**8-1645*y**9+105*y**10)-21*r**12*(320-2880*y+11520*y**2-26880*y**3
        +43840*y**4-48207*y**5+35223*y**6-17604*y**7+5556*y**8-929*y**9+33*y**10
        +8*y**11))+MBhat**4*(-7875*r**16*(-1+y)**9-4200*r**6*(-1+y)**4*(2507
        -11779*y+21668*y**2-19164*y**3+7475*y**4-763*y**5-140*y**6+28*y**7)
        +3*(-1+y)**9*(674625-235200*y-117600*y**2-69440*y**3-22330*y**4-896*y**5
        -56*y**6+y**8)-28*r**2*(-1+y)**7*(320700-653400*y+338700*y**2-1120*y**3
        +3935*y**4+290*y**5-696*y**6-18*y**7+9*y**8)-210*r**4*(-1+y)**5*(14528
        -114512*y+284568*y**2-302832*y**3+131365*y**4-11988*y**5-1798*y**6
        -228*y**7+57*y**8)-140*r**8*(-1+y)**3*(4040-22710*y+52185*y**2-63850*y**3
        +29355*y**4-2898*y**5-5954*y**6+3720*y**7-876*y**8+88*y**9)+84*r**10*(
        -1+y)**2*(-1100+7700*y-23100*y**2+43620*y**3-33775*y**4+13833*y**5
        +3682*y**6-6718*y**7+2985*y**8-559*y**9+32*y**10)-40*r**14*(1057-9513*y
        +38052*y**2-90468*y**3+138600*y**4-142632*y**5+99120*y**6-45360*y**7
        +12780*y**8-1900*y**9+96*y**10)+42*r**12*(2000-18000*y+72000*y**2
        -174720*y**3+268275*y**4-271915*y**5+184044*y**6-77916*y**7+16689*y**8
        +359*y**9-956*y**10+140*y**11)))/(1260*(-1+y)**9)

    return 0.

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def XLS(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^3 level functions of rhoLS for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (9-24*r**2+15*r**8+y**4-(6*r**4*(12+6*logy*(-1+y)**3-42*y+51*y**2
        -31*y**3+8*y**4))/(-1+y)**3+(4*r**6*(-18+72*y-108*y**2+84*y**3-31*y**4
        +4*y**5))/(-1+y)**4)/6
    if (dQ2 == 0 and dr == 0 and i==1):
        return (10*MBhat**2*(-1+y)*(-24*r**2*(-1+y)**4+15*r**8*(-1+y)**4+(-1+y)**4*(9
        +y**4)-6*r**4*(-1+y)*(12+6*logy*(-1+y)**3-42*y+51*y**2-31*y**3
        +8*y**4)+4*r**6*(-18+72*y-108*y**2+84*y**3-31*y**4+4*y**5))+2*MBhat*(84*r**10*(
        -1+y)**5+20*r**2*(-1+y)**5*(21+2*y**3)-(-1+y)**5*(84-30*y**3+y**5)
        -30*r**4*(-1+y)**2*(-28+84*y-84*y**2+34*y**3+4*y**4-7*y**5+y**6)
        +5*r**8*(84-420*y+840*y**2-910*y**3+530*y**4-151*y**5+15*y**6)-20*r**6*(42
        -210*y+420*y**2-450*y**3+267*y**4-73*y**5+2*y**6+2*y**7))-5*(-9*r**10*(
        -1+y)**5+r**2*(-1+y)**5*(-7+48*logy+48*y+24*y**2+16*y**3-y**4)
        +3*(-1+y)**5*(-5+3*y**4)+r**8*(-79+395*y-790*y**2+790*y**3-465*y**4
        +141*y**5-16*y**6)-6*r**6*(-1+y)*(8-18*logy*(-1+y)**4-50*y+111*y**2
        -110*y**3+68*y**4-20*y**5+y**6)+6*r**4*(-1+y)**2*(-6*logy*(-1+y)**3
        +y*(6-15*y+11*y**2-13*y**3+7*y**4))))/(60*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (90+30*r**12-72*y**5-3*y**6+90*r**4*(-124/3+4*logy+2/(-1+y)**3
        +7/(-1+y)**2+4/(-1+y)-3*y**2-(8*y**3)/3)-60*r**2*(-19+12*logy
        +12*y+6*y**2+4*y**3+3*y**4)+(60*r**6*(52+24*logy*(-1+y)**4-184*y
        +228*y**2-104*y**3+2*y**4+24*y**5-9*y**6))/(-1+y)**4-(15*r**8*(-22
        +72*logy*(-1+y)**5+38*y+104*y**2-344*y**3+352*y**4-224*y**5+63*y**6
        -3*y**7))/(-1+y)**5+(12*r**10*(-35+210*y-525*y**2+700*y**3-525*y**4
        +246*y**5-62*y**6+6*y**7))/(-1+y)**6+15*MBhat**4*(9-24*r**2+15*r**8
        +6*r**4*(7-6*logy+2/(-1+y)**3+(-1+y)**(-2)-6/(-1+y)-8*y)+y**4
        +4*r**6*(-15+3/(-1+y)**4+4/(-1+y)**3-2/(-1+y)**2+4*y))+(6*MBhat**3*(84*r**10*(
        -1+y)**5+20*r**2*(-1+y)**5*(21+2*y**3)-(-1+y)**5*(84-30*y**3+y**5)
        -30*r**4*(-1+y)**2*(-28+84*y-84*y**2+34*y**3+4*y**4-7*y**5+y**6)
        +5*r**8*(84-420*y+840*y**2-910*y**3+530*y**4-151*y**5+15*y**6)-20*r**6*(42
        -210*y+420*y**2-450*y**3+267*y**4-73*y**5+2*y**6+2*y**7)))/(-1+y)**5
        +(12*MBhat*(10*r**12*(-1+y)**6+10*r**2*(-1+y)**6*(-1+12*logy+12*y
        +6*y**2+4*y**3)+4*(-1+y)**6*(-8+5*y**4+y**5)+15*r**4*(-1+y)**3*(
        -34-24*logy*(-1+y)**3+126*y-162*y**2+78*y**3-12*y**4-y**5+y**6)
        -5*r**6*(-1+y)**2*(136-544*y+816*y**2-544*y**3+194*y**4-23*y**5-6*y**6
        +y**7)+3*r**10*(-26+156*y-390*y**2+520*y**3-430*y**4+207*y**5-52*y**6
        +5*y**7)-5*r**8*(-56+336*y-840*y**2+1120*y**3-906*y**4+459*y**5-128*y**6
        +15*y**7)))/(-1+y)**6+MBhat**2*(665+280*r**12-240*y**3-165*y**4+y**6
        +90*r**10*(-89/6+2/(-1+y)**6+12/(5*(-1+y)**5)-3/(5*(-1+y)**4)+(12*y)/5)
        -15*r**2*(209+48*logy+48*y+24*y**2+16*y**3+y**4)-(15*r**8*(137-685*y
        +1370*y**2-1610*y**3+905*y**4-181*y**5-35*y**6+15*y**7))/(-1+y)**5
        -(10*r**6*(216+210*logy*(-1+y)**4-654*y+561*y**2-194*y**3-484*y**4
        +468*y**5-145*y**6+16*y**7))/(-1+y)**4+90*r**4*(38*logy+(-124+258*y
        -87*y**2-85*y**3+135*y**4-69*y**5+8*y**6)/(3*(-1+y)**3))))/90
    if (dQ2 == 0 and dr == 0 and i==3):
        return (MBhat**6*(9-24*r**2+15*r**8+6*r**4*(7-6*logy+2/(-1+y)**3+(-1+y)**(
        -2)-6/(-1+y)-8*y)+y**4+4*r**6*(-15+3/(-1+y)**4+4/(-1+y)**3
        -2/(-1+y)**2+4*y)))/6+(MBhat**5*(84*r**10*(-1+y)**5+20*r**2*(-1+y)**5*(21
        +2*y**3)-(-1+y)**5*(84-30*y**3+y**5)-30*r**4*(-1+y)**2*(-28+84*y
        -84*y**2+34*y**3+4*y**4-7*y**5+y**6)+5*r**8*(84-420*y+840*y**2-910*y**3
        +530*y**4-151*y**5+15*y**6)-20*r**6*(42-210*y+420*y**2-450*y**3+267*y**4
        -73*y**5+2*y**6+2*y**7)))/(10*(-1+y)**5)+MBhat**2*((11*r**14)/6+r**12*(
        -71/6+6/(-1+y)**7+15/(-1+y)**6+42/(5*(-1+y)**5)-21/(10*(-1+y)**4)
        +(12*y)/5)+r**6*(527/2+274*logy+90/(-1+y)**4+316/(-1+y)**3+286/(
        -1+y)**2-306/(-1+y)-16*y-31*y**2+(4*y**3)/3)+r**4*(-665+276*logy
        +30/(-1+y)**3+105/(-1+y)**2+24/(-1+y)+180*y+27*y**2-4*y**3
        -2*y**4)+(830-375*y**4-312*y**5-29*y**6)/60+r**2*(131/3-72*logy
        -72*y-36*y**2-24*y**3-(21*y**4)/4-y**6/60)-(r**10*(-580+3480*y-8700*y**2
        +11600*y**3-10975*y**4+5706*y**5-1504*y**6+118*y**7+15*y**8))/(20*(
        -1+y)**6)-(r**8*(-1086+696*logy*(-1+y)**5+4734*y-7728*y**2+5408*y**3
        -2719*y**4-181*y**5+547*y**6-143*y**7+16*y**8))/(12*(-1+y)**5))+MBhat**4*(221/12
        +(28*r**12)/3-8*y**3-(13*y**4)/4+y**6/30+r**10*(-187/4+6/(-1+y)**6
        +36/(5*(-1+y)**5)-9/(5*(-1+y)**4)+(36*y)/5)+r**8*(323/4+36/(-1
        +y)**5+66/(-1+y)**4+15/(-1+y)**3-15/(2*(-1+y)**2)-24*y-(15*y**2)/2)
        -(r**2*(425+48*logy+48*y+24*y**2+16*y**3+3*y**4))/4+r**4*(105*logy
        -12*y-(69*y**2)/2+8*y**3+(-124+255*y-108*y**2+7*y**3)/(-1+y)**3)
        +r**6*(-43*logy+8*y+(51*y**2)/2-(16*y**3)/3+(-84+285*y-347*y**2
        +289*y**3-83*y**4)/(-1+y)**4))+MBhat**3*(-736/35+(24*r**14)/7+5*y**3
        +(28*y**4)/3+(9*y**5)/5-y**7/210+r**12*(-84/5-2/(-1+y)**7-7/(3*(
        -1+y)**6)+7/(15*(-1+y)**5)+(7*y)/3)+(r**10*(114-130/(-1+y)**6
        -272/(-1+y)**5-111/(-1+y)**4+37/(-1+y)**3-25*y-18*y**2))/5+(r**2*(388
        +240*logy+240*y+120*y**2+50*y**3+y**5))/5+(4*r**6*(-258+1032*y
        -1548*y**2+1017*y**3-410*y**4+110*y**5-21*y**6+3*y**7))/(3*(-1+y)**4)
        -(r**8*(128-640*y+1280*y**2-1130*y**3+874*y**4-510*y**5+214*y**6
        -53*y**7+5*y**8))/(2*(-1+y)**5)+r**4*(-288*logy+(-216+936*y-1368*y**2
        +749*y**3-143*y**4-3*y**5+5*y**6)/(-1+y)**3))+MBhat*((26*r**14)/35
        +r**12*(-34/5-6/(-1+y)**7-23/(-1+y)**6-147/(5*(-1+y)**5)-21/(2*(
        -1+y)**4)+7/(2*(-1+y)**3)+(3*y)/2)+r**2*(-408/5+48*logy+48*y
        +24*y**2+16*y**3+12*y**4+(6*y**5)/5)+(-360+245*y**5+84*y**6+3*y**7)/70
        +(2*r**10*(120-720*y+1800*y**2-2400*y**3+1800*y**4-895*y**5+252*y**6
        -33*y**7+y**8))/(5*(-1+y)**6)-(r**8*(-440+2200*y-4400*y**2+4400*y**3
        -2200*y**4+629*y**5-77*y**6-5*y**7+y**8))/(2*(-1+y)**5)+r**4*(-72*logy
        -63*y-12*y**2+3*y**3+y**5/10+(3*(-694+2097*y-2087*y**2+664*y**3))/(5*(
        -1+y)**3))+2*r**6*(-180*logy+(-75+120*y+180*y**2-480*y**3+300*y**4
        -64*y**5-4*y**6+2*y**7)/(-1+y)**4))+(345+75*r**14-315*y**6-16*y**7
        -21*r**2*(-573+240*logy+240*y+120*y**2+80*y**3+60*y**4+48*y**5+y**6)
        -(105*r**4*(-275+60*logy*(-1+y)**3+765*y-675*y**2+165*y**3+15*y**4
        +3*y**5-17*y**6+11*y**7))/(-1+y)**3+(21*r**10*(-245-300*logy*(-1+y)**6
        +1170*y-2025*y**2+1200*y**3+600*y**4-1140*y**5+805*y**6-214*y**7
        +9*y**8))/(-1+y)**6+(7*r**12*(219-1533*y+4599*y**2-7665*y**3+7665*y**4
        -4599*y**5+1785*y**6-383*y**7+32*y**8))/(-1+y)**7+105*r**8*(180*logy
        +(-535+2495*y-4540*y**2+3940*y**3-1520*y**4+124*y**5+117*y**6-33*y**7)/(
        -1+y)**5)+35*r**6*(1260*logy-(945-5040*y+10080*y**2-9240*y**3+3570*y**4
        -252*y**5-204*y**6+44*y**7+y**8)/(-1+y)**4))/420

    if (dQ2 == 0 and dr == 0 and i==4):
        return (45*r**16*(-1+y)**8-16*r**2*(-1+y)**8*(-1241+420*logy+420*y+210*y**2
        +140*y**3+105*y**4+84*y**5+70*y**6+2*y**7)+336*r**10*(-1+y)**2*(419
        +120*logy*(-1+y)**6-2394*y+5625*y**2-6900*y**3+4575*y**4-1470*y**5
        +125*y**6+58*y**7-13*y**8)+840*r**6*(-1+y)**4*(-258+120*logy*(-1+y)**4
        +1152*y-1968*y**2+1552*y**3-508*y**4+24*y**5+4*y**6+8*y**7-y**8)
        -3*(-1+y)**8*(-97+96*y**7+5*y**8)-42*r**4*(-1+y)**5*(-358+660*logy*(
        -1+y)**3+414*y+576*y**2-852*y**3+165*y**4+33*y**5+11*y**6-41*y**7
        +32*y**8)-70*r**8*(-1+y)**3*(1160-2940*logy*(-1+y)**5-2860*y-1630*y**2
        +11430*y**3-13065*y**4+5553*y**5-490*y**6-246*y**7+27*y**8+y**9)+42*r**12*(
        -1+y)*(218-180*logy*(-1+y)**7-1346*y+3408*y**2-4420*y**3+2845*y**4
        -447*y**5-481*y**6+427*y**7-108*y**8+4*y**9)+8*r**14*(-158+1264*y
        -4424*y**2+8848*y**3-11060*y**4+8848*y**5-4424*y**6+1464*y**7-273*y**8
        +20*y**9)+70*MBhat**8*(-1+y)**4*(-24*r**2*(-1+y)**4+15*r**8*(-1+y)**4
        +(-1+y)**4*(9+y**4)-6*r**4*(-1+y)*(12+6*logy*(-1+y)**3-42*y+51*y**2
        -31*y**3+8*y**4)+4*r**6*(-18+72*y-108*y**2+84*y**3-31*y**4+4*y**5))
        -56*MBhat**7*(-1+y)**3*(-84*r**10*(-1+y)**5-20*r**2*(-1+y)**5*(21+2*y**3)
        +(-1+y)**5*(84-30*y**3+y**5)+30*r**4*(-1+y)**2*(-28+84*y-84*y**2
        +34*y**3+4*y**4-7*y**5+y**6)-5*r**8*(84-420*y+840*y**2-910*y**3
        +530*y**4-151*y**5+15*y**6)+20*r**6*(42-210*y+420*y**2-450*y**3+267*y**4
        -73*y**5+2*y**6+2*y**7))+28*MBhat**6*(-1+y)**2*(280*r**12*(-1+y)**6
        -5*r**2*(-1+y)**6*(641+48*logy+48*y+24*y**2+16*y**3+5*y**4)+(-1
        +y)**6*(515-240*y**3-75*y**4+y**6)+30*r**4*(-1+y)**3*(-124+102*logy*(
        -1+y)**3+270*y-117*y**2-63*y**3+109*y**4-55*y**5+8*y**6)-10*r**6*(
        -1+y)**2*(264+102*logy*(-1+y)**4-954*y+1227*y**2-854*y**3-76*y**4
        +348*y**5-139*y**6+16*y**7)+3*r**10*(-505+3030*y-7575*y**2+10580*y**3
        -8565*y**4+3930*y**5-907*y**6+72*y**7)-5*r**8*(-569+3414*y-8535*y**2
        +12100*y**3-10055*y**4+4470*y**5-752*y**6-118*y**7+45*y**8))-8*MBhat**5*(
        -1+y)*(-720*r**14*(-1+y)**7-42*r**2*(-1+y)**7*(398+120*logy+120*y
        +60*y**2+10*y**3+y**5)+210*r**4*(-1+y)**4*(114+216*logy*(-1+y)**3
        -558*y+882*y**2-515*y**3+107*y**4-2*y**6)+(-1+y)**7*(3072-1050*y**3
        -1120*y**4-210*y**5+y**7)-70*r**6*(-1+y)**3*(-624+2496*y-3744*y**2
        +2436*y**3-1058*y**4+371*y**5-102*y**6+15*y**7)+105*r**8*(-1+y)**2*(16
        -80*y+160*y**2-10*y**3+182*y**4-284*y**5+184*y**6-53*y**7+5*y**8)
        -14*r**12*(294-2058*y+6174*y**2-10605*y**3+11095*y**4-7147*y**5+2709*y**6
        -527*y**7+35*y**8)+84*r**10*(93-651*y+1953*y**2-3430*y**3+3650*y**4
        -2275*y**5+735*y**6-41*y**7-43*y**8+9*y**9))+2*MBhat**2*(245*r**16*(
        -1+y)**8-42*r**2*(-1+y)**8*(-2335+1200*logy+1200*y+600*y**2+400*y**3
        +300*y**4+96*y**5+3*y**6)+210*r**8*(-1+y)**3*(-3295+1080*logy*(-1
        +y)**5+15395*y-28090*y**2+24490*y**3-9545*y**4+1165*y**5+291*y**6
        -63*y**7)+70*r**6*(-1+y)**4*(-2451+10500*logy*(-1+y)**4+20304*y-51456*y**2
        +55304*y**3-24326*y**4+2532*y**5+724*y**6+60*y**7-19*y**8)-(-1+y)**8*(
        -4397+2016*y**5+2030*y**6+320*y**7+9*y**8)+14*r**4*(-1+y)**5*(41305
        +6300*logy*(-1+y)**3-130215*y+139665*y**2-52855*y**3+1575*y**4+1035*y**5
        +137*y**6+189*y**7+6*y**8-2*y**9)-14*r**10*(-1+y)**2*(3573+1740*logy*(
        -1+y)**6-19698*y+44025*y**2-50000*y**3+28800*y**4-12204*y**5+1099*y**6
        +630*y**7-141*y**8+16*y**9)+18*r**14*(-135+1080*y-3780*y**2+7560*y**3
        -9450*y**4+8120*y**5-4564*y**6+1592*y**7-307*y**8+24*y**9)-14*r**12*(
        -545+4360*y-15260*y**2+30520*y**3-38150*y**4+33832*y**5-21470*y**6
        +8408*y**7-1879*y**8+184*y**9))+8*MBhat*(24*r**16*(-1+y)**8+r**2*(-1
        +y)**8*(-12708+5040*logy+5040*y+2520*y**2+1680*y**3+1260*y**4+1008*y**5
        +182*y**6+3*y**7)-42*r**4*(-1+y)**5*(966-120*logy*(-1+y)**3-2778*y
        +2598*y**2-746*y**3-30*y**4-6*y**5+28*y**6-13*y**7+y**8)+(-1+y)**8*(
        -300+210*y**6+93*y**7+4*y**8)+35*r**8*(-1+y)**3*(2820-2160*logy*(
        -1+y)**5-11940*y+18480*y**2-11280*y**3+240*y**4+2112*y**5-598*y**6
        -37*y**7+11*y**8)-21*r**10*(-1+y)**2*(1100-6600*y+16500*y**2-22000*y**3
        +16500*y**4-6600*y**5+1550*y**6-167*y**7-4*y**8+y**9)+14*r**6*(-1
        +y)**4*(6534-6120*logy*(-1+y)**4-32256*y+60624*y**2-52656*y**3+19284*y**4
        -1224*y**5-474*y**6-99*y**7-4*y**8+y**9)+4*r**14*(-93+744*y-2604*y**2
        +5208*y**3-6510*y**4+5208*y**5-2884*y**6+1004*y**7-193*y**8+15*y**9)
        +28*r**12*(111-888*y+3108*y**2-6216*y**3+7770*y**4-6216*y**5+3294*y**6
        -1197*y**7+262*y**8-29*y**9+y**10))+8*MBhat**3*(120*r**16*(-1+y)**8
        -140*r**6*(-1+y)**4*(888+1200*logy*(-1+y)**4-2352*y+1128*y**2+1648*y**3
        -1567*y**4+375*y**5+38*y**6-11*y**7)+r**2*(-1+y)**8*(-18600+16800*logy
        +16800*y+8400*y**2+5600*y**3+2450*y**4+126*y**5+y**7)+(-1+y)**8*(
        -2360+630*y**4+1197*y**5+392*y**6+24*y**7)-21*r**4*(-1+y)**5*(9992
        +3360*logy*(-1+y)**3-33336*y+38376*y**2-16152*y**3+940*y**4+295*y**5
        +155*y**6+15*y**7-5*y**8)+35*r**8*(-1+y)**3*(2080-10400*y+20800*y**2
        -20800*y**3+9910*y**4-2759*y**5+417*y**6-32*y**7+4*y**8)-21*r**10*(
        -1+y)**2*(-360+2160*y-5400*y**2+7200*y**3-4370*y**4+2182*y**5-810*y**6
        +247*y**7-54*y**8+5*y**9)+4*r**14*(-208+1664*y-5824*y**2+11648*y**3
        -15260*y**4+13090*y**5-7280*y**6+2500*y**7-470*y**8+35*y**9)-7*r**12*(
        -200+1600*y-5600*y**2+11200*y**3-15800*y**4+14953*y**5-8624*y**6
        +2932*y**7-452*y**8-21*y**9+12*y**10))+MBhat**4*(1575*r**16*(-1+y)**8
        -28*r**2*(-1+y)**8*(1120+3600*logy+3600*y+1800*y**2+920*y**3+75*y**4
        +3*y**6)+210*r**4*(-1+y)**5*(5168+3120*logy*(-1+y)**3-18624*y+23304*y**2
        -10888*y**3+959*y**4+323*y**5+57*y**6-19*y**7)+280*r**6*(-1+y)**4*(4305
        +1740*logy*(-1+y)**4-15480*y+19740*y**2-9680*y**3+1445*y**4+392*y**5
        -84*y**6-8*y**7)+(-1+y)**8*(26425-3360*y**3-11970*y**4-7168*y**5
        -784*y**6+y**8)+140*r**8*(-1+y)**3*(960-510*logy*(-1+y)**5-4290*y
        +7305*y**2-5325*y**3+3870*y**4-1452*y**5+430*y**6-134*y**7+16*y**8)
        -84*r**10*(-1+y)**2*(-200+1200*y-3000*y**2+3160*y**3-4715*y**4+4138*y**5
        -2414*y**6+906*y**7-191*y**8+16*y**9)+40*r**14*(-203+1624*y-5684*y**2
        +11648*y**3-15064*y**4+12656*y**5-6832*y**6+2240*y**7-388*y**8+24*y**9)
        -14*r**12*(-640+5120*y-17920*y**2+39200*y**3-51205*y**4+38920*y**5
        -15596*y**6+568*y**7+2437*y**8-1024*y**9+140*y**10)))/(420*(-1+y)**8)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm1(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m1 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-40*r**2*(-1+y)**6-5*r**8*(-1+y)**6+5*(-1+y)**6*(1+y**4)-4*r**6*(
        -10+60*y-150*y**2+200*y**3-145*y**4+90*y**5-29*y**6+4*y**7)+2*r**4*(
        -1+y)*(30*logy*(-1+y)**5+y*(-30+135*y-235*y**2+185*y**3-103*y**4
        +24*y**5)))/(40*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-2*MBhat*y**3*(60*r**2*(-1+y)**7+(-1+y)**7*(-30+40*y+3*y**2)-6*r**4*(
        -1+y)**2*y*(80-97*y+97*y**2-37*y**3+5*y**4)+3*r**8*(-10-130*y-7*y**2
        +49*y**3-27*y**4+5*y**5)-4*r**6*(-15-175*y+292*y**2-210*y**3+145*y**4
        -43*y**5+6*y**6))+3*(-3*r**10*(-1+y)**7+5*r**2*(-1+y)**7*(-9+y**4)
        +(-1+y)**7*(3-5*y**4+16*y**5)+6*r**4*(-1+y)**2*(40+30*logy*(-1+y)**5
        -230*y+535*y**2-635*y**3+390*y**4-128*y**5+7*y**6+5*y**7)+r**8*(
        -45+315*y-945*y**2+1575*y**3-1585*y**4+775*y**5-441*y**6+127*y**7
        -16*y**8)+2*r**6*(-1+y)*(120+90*logy*(-1+y)**6-630*y+1305*y**2
        -1290*y**3+505*y**4-86*y**5-141*y**6+62*y**7-5*y**8))-6*MBhat**2*(
        -1+y)*(40*r**2*(-1+y)**6+5*r**8*(-1+y)**6-5*(-1+y)**6*(1+y**4)+4*r**6*(
        -10+60*y-150*y**2+200*y**3-145*y**4+90*y**5-29*y**6+4*y**7)-2*r**4*(
        -1+y)*(30*logy*(-1+y)**5+y*(-30+135*y-235*y**2+185*y**3-103*y**4
        +24*y**5))))/(240*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-10*r**12*(-1+y)**8+120*r**2*(-1+y)**8*(-2+y**5)+(-1+y)**8*(10-120*y**5
        +133*y**6)+30*r**4*(-1+y)**3*(125+60*logy*(-1+y)**5-685*y+1520*y**2
        -1720*y**3+1010*y**4-286*y**5-3*y**6+15*y**7)-15*r**8*(-1+y)*(250
        -120*logy*(-1+y)**7-1630*y+4470*y**2-6610*y**3+5560*y**4-2408*y**5
        +727*y**6+55*y**7-59*y**8+5*y**9)-12*r**10*(-20+160*y-560*y**2+1120*y**3
        -1400*y**4+1150*y**5-401*y**6+208*y**7-53*y**8+6*y**9)-4*MBhat*y**4*(100*r**2*(
        -1+y)**8+4*(-1+y)**8*(-25+24*y+5*y**2)+15*r**4*(-1+y)**3*(-40-y
        +y**2-13*y**3+5*y**4)-5*r**6*(-1+y)**2*(280+321*y-134*y**2+182*y**3
        -54*y**4+5*y**5)+3*r**10*(-100-397*y+56*y**2+44*y**3-28*y**4+5*y**5)
        +5*r**8*(220+337*y-728*y**2+302*y**3-148*y**4+17*y**5))-10*MBhat**3*(
        -1+y)*y**3*(60*r**2*(-1+y)**7+(-1+y)**7*(-30+40*y+3*y**2)-6*r**4*(
        -1+y)**2*y*(80-97*y+97*y**2-37*y**3+5*y**4)+3*r**8*(-10-130*y-7*y**2
        +49*y**3-27*y**4+5*y**5)-4*r**6*(-15-175*y+292*y**2-210*y**3+145*y**4
        -43*y**5+6*y**6))+20*r**6*(-1+y)**2*(240*logy*(-1+y)**6+y*(240-1320*y
        +2960*y**2-3420*y**3+2004*y**4-749*y**5+90*y**6+15*y**7))-5*MBhat**2*(8*r**12*(
        -1+y)**8-r**2*(-1+y)**8*(-477+80*y**3+15*y**4)-(-1+y)**8*(35-80*y**3
        +49*y**4+64*y**5+y**6)-6*r**4*(-1+y)**3*(300+270*logy*(-1+y)**5-1770*y
        +4215*y**2-5195*y**3+3290*y**4-1044*y**5+47*y**6+13*y**7)-3*r**10*(15
        -120*y+420*y**2-920*y**3+892*y**4-728*y**5+364*y**6-104*y**7+13*y**8)
        -2*r**6*(-1+y)**2*(1080+570*logy*(-1+y)**6-5910*y+13065*y**2-15130*y**3
        +8065*y**4-2102*y**5-861*y**6+446*y**7-109*y**8+16*y**9)+r**8*(-45
        +360*y-1260*y**2+1640*y**3-3130*y**4+3416*y**5-2759*y**6+2656*y**7
        -1079*y**8+216*y**9-15*y**10))-15*MBhat**4*(-1+y)**2*(40*r**2*(-1+y)**6
        +5*r**8*(-1+y)**6-5*(-1+y)**6*(1+y**4)+4*r**6*(-10+60*y-150*y**2
        +200*y**3-145*y**4+90*y**5-29*y**6+4*y**7)-2*r**4*(-1+y)*(30*logy*(
        -1+y)**5+y*(-30+135*y-235*y**2+185*y**3-103*y**4+24*y**5))))/(600*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-42*MBhat*y**5*(2*r**2*(-1+y)**9*(159+10*y)+(-1+y)**9*(-303+148*y
        +57*y**2)+20*r**6*(-1+y)**3*(-447+148*y-138*y**2+12*y**3+5*y**4)
        -15*r**4*(-1+y)**4*(197-193*y+116*y**2-20*y**3-5*y**4+y**5)-5*r**8*(
        -1+y)**2*(2091+1031*y-200*y**2+408*y**3-95*y**4+5*y**5)+3*r**12*(
        -399-889*y+156*y**2+36*y**3-29*y**4+5*y**5)+6*r**10*(947+335*y
        -1608*y**2+542*y**3-233*y**4+15*y**5+2*y**6))-210*MBhat**5*(-1+y)**2*y**3*(60*r**2*(
        -1+y)**7+(-1+y)**7*(-30+40*y+3*y**2)-6*r**4*(-1+y)**2*y*(80-97*y
        +97*y**2-37*y**3+5*y**4)+3*r**8*(-10-130*y-7*y**2+49*y**3-27*y**4
        +5*y**5)-4*r**6*(-15-175*y+292*y**2-210*y**3+145*y**4-43*y**5+6*y**6))
        -2*MBhat**3*(1800*r**14*(-1+y)**9+420*r**2*(-1+y)**9*(-228+26*y**3+15*y**4)
        +(-1+y)**9*(6320-7770*y**3-5460*y**4+9408*y**5+1960*y**6+15*y**7)
        +210*r**4*(-1+y)**4*(1940+1680*logy*(-1+y)**5-11380*y+26960*y**2
        -32855*y**3+20695*y**4-6164*y**5+328*y**6-29*y**7+25*y**8)+280*r**6*(
        -1+y)**3*(1905+780*logy*(-1+y)**6-10650*y+24285*y**2-29200*y**3
        +18165*y**4-6564*y**5+917*y**6-486*y**7+144*y**8-16*y**9)+84*r**10*(
        -620+5580*y-22320*y**2+53650*y**3-80865*y**4+82827*y**5-57208*y**6
        +24372*y**7-6093*y**8+677*y**9)+70*r**12*(212-1908*y+7632*y**2-18207*y**3
        +26943*y**4-27090*y**5+18186*y**6-7884*y**7+2016*y**8-239*y**9+3*y**10)
        -105*r**8*(-1+y)**2*(-720+5040*y-15120*y**2+27490*y**3-27250*y**4
        +21630*y**5-8042*y**6+2243*y**7-225*y**8-11*y**9+5*y**10))+105*MBhat**4*(
        -1+y)*(-16*r**12*(-1+y)**8+r**2*(-1+y)**8*(-819+160*y**3+15*y**4)
        +(-1+y)**8*(61-160*y**3+113*y**4+80*y**5+2*y**6)+6*r**4*(-1+y)**3*(480
        +450*logy*(-1+y)**5-2850*y+6825*y**2-8485*y**3+5410*y**4-1704*y**5
        +73*y**6+11*y**7)+3*r**10*(33-264*y+924*y**2-2008*y**3+1994*y**4
        -1624*y**5+812*y**6-232*y**7+29*y**8)+2*r**6*(-1+y)**2*(1800+870*logy*(
        -1+y)**6-9930*y+22215*y**2-26390*y**3+14615*y**4-3946*y**5-1299*y**6
        +706*y**7-203*y**8+32*y**9)+r**8*(-45+360*y-1260*y**2+4280*y**3-3220*y**4
        +248*y**5+1870*y**6-3608*y**7+1729*y**8-384*y**9+30*y**10))+3*(-25*r**14*(
        -1+y)**9+7*r**2*(-1+y)**9*(-125+133*y**6)+(-1+y)**9*(25-931*y**6
        +640*y**7)+35*r**4*(-1+y)**4*(785+300*logy*(-1+y)**5-4225*y+9200*y**2
        -10200*y**3+5850*y**4-1470*y**5-216*y**6+240*y**7-75*y**8+15*y**9)
        +35*r**8*(-1+y)**2*(-1125+1500*logy*(-1+y)**7+6375*y-13875*y**2+12625*y**3
        +500*y**4-10800*y**5+7919*y**6-3653*y**7+549*y**8+45*y**9)+7*r**10*(
        -1+y)*(3925+1500*logy*(-1+y)**8-29900*y+98650*y**2-183300*y**3
        +208125*y**4-145500*y**5+56223*y**6-15804*y**7+354*y**8+552*y**9-45*y**10)
        +35*r**6*(-1+y)**3*(-1125+1500*logy*(-1+y)**6+8250*y-25125*y**2+41000*y**3
        -38250*y**4+19800*y**5-5598*y**6+248*y**7+135*y**8+30*y**9-5*y**10)
        -7*r**12*(125-1125*y+4500*y**2-10500*y**3+15750*y**4-15750*y**5+11032*y**6
        -2888*y**7+1377*y**8-313*y**9+32*y**10))-3*MBhat**2*(110*r**14*(-1+y)**9
        +35*r**2*(-1+y)**9*(524-237*y**4-48*y**5+y**6)-420*r**4*(-1+y)**4*(592
        +300*logy*(-1+y)**5-3260*y+7270*y**2-8270*y**3+4725*y**4-1109*y**5
        -111*y**6+43*y**7)-(-1+y)**9*(810-7245*y**4+1008*y**5+3689*y**6+240*y**7)
        -210*r**12*(-4+36*y-144*y**2+336*y**3-637*y**4+357*y**5-252*y**6
        +108*y**7-27*y**8+3*y**9)-70*r**6*(-1+y)**3*(825+4140*logy*(-1+y)**6
        -810*y-10395*y**2+34560*y**3-49575*y**4+33102*y**5-12641*y**6+1476*y**7
        -174*y**8+32*y**9)-35*r**8*(-1+y)**2*(-5850+2280*logy*(-1+y)**7+38670*y
        -108030*y**2+164090*y**3-151085*y**4+71379*y**5-20614*y**6-734*y**7
        +795*y**8-157*y**9+16*y**10)-21*r**10*(-140+1260*y-5040*y**2+11760*y**3
        -11335*y**4+14383*y**5-13983*y**6+6807*y**7-5073*y**8+1577*y**9-221*y**10
        +5*y**11))-210*MBhat**6*(-1+y)**3*(40*r**2*(-1+y)**6+5*r**8*(-1+y)**6
        -5*(-1+y)**6*(1+y**4)+4*r**6*(-10+60*y-150*y**2+200*y**3-145*y**4
        +90*y**5-29*y**6+4*y**7)-2*r**4*(-1+y)*(30*logy*(-1+y)**5+y*(-30
        +135*y-235*y**2+185*y**3-103*y**4+24*y**5))))/(8400*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-280*MBhat**7*(-1+y)**3*y**3*(60*r**2*(-1+y)**7+(-1+y)**7*(-30+40*y
        +3*y**2)-6*r**4*(-1+y)**2*y*(80-97*y+97*y**2-37*y**3+5*y**4)+3*r**8*(
        -10-130*y-7*y**2+49*y**3-27*y**4+5*y**5)-4*r**6*(-15-175*y+292*y**2
        -210*y**3+145*y**4-43*y**5+6*y**6))-8*MBhat*y**6*(-1+r**2+y)**2*((
        -1+y)**8*(-2772+625*y+480*y**2)-r**2*(-1+y)**7*(-2352-1543*y+561*y**2)
        +60*r**10*(-224-340*y+60*y**2+5*y**3-6*y**4+y**5)+3*r**4*(-1+y)**3*(
        -12376+16595*y-10743*y**2+4059*y**3-969*y**4+74*y**5)-r**6*(-1+y)**2*(69552
        -26867*y+16416*y**2-2706*y**3-1048*y**4+93*y**5)-4*r**8*(-12747+5531*y
        +9175*y**2-4440*y**3+2815*y**4-343*y**5+9*y**6))-8*MBhat**5*(-1+y)*(1800*r**14*(
        -1+y)**9+420*r**2*(-1+y)**9*(-228+26*y**3+5*y**4)+(-1+y)**9*(6320
        -7770*y**3-1260*y**4+5376*y**5+1120*y**6+15*y**7)+210*r**4*(-1+y)**4*(1940
        +1680*logy*(-1+y)**5-11380*y+26960*y**2-32855*y**3+20815*y**4-6161*y**5
        +325*y**6+10*y**7+10*y**8)+70*r**6*(-1+y)**3*(7620+3120*logy*(-1+y)**6
        -42600*y+97140*y**2-116800*y**3+73500*y**4-25293*y**5+3266*y**6-1398*y**7
        +414*y**8-49*y**9)+70*r**12*(212-1908*y+7632*y**2-18207*y**3+26943*y**4
        -27090*y**5+18186*y**6-7884*y**7+2016*y**8-239*y**9+3*y**10)-105*r**8*(
        -1+y)**2*(-720+5040*y-15120*y**2+27490*y**3-27690*y**4+20516*y**5
        -7700*y**6+1981*y**7-191*y**8-11*y**9+5*y**10)-42*r**10*(1240-11160*y
        +44640*y**2-107300*y**3+162030*y**4-164763*y**5+113057*y**6-48708*y**7
        +12402*y**8-1453*y**9+15*y**10))+420*MBhat**6*(-1+y)**2*(-8*r**12*(-1
        +y)**8+r**2*(-1+y)**8*(-387+80*y**3+5*y**4)+(-1+y)**8*(29-80*y**3
        +59*y**4+32*y**5+y**6)+6*r**4*(-1+y)**3*(220+210*logy*(-1+y)**5-1310*y
        +3145*y**2-3925*y**3+2510*y**4-788*y**5+33*y**6+3*y**7)+3*r**10*(17
        -136*y+476*y**2-1032*y**3+1032*y**4-840*y**5+420*y**6-120*y**7+15*y**8)
        +2*r**6*(-1+y)**2*(840+390*logy*(-1+y)**6-4650*y+10455*y**2-12550*y**3
        +7055*y**4-1930*y**5-579*y**6+322*y**7-99*y**8+16*y**9)+r**8*(-45
        +360*y-1260*y**2+3400*y**3-3190*y**4+1304*y**5+327*y**6-1520*y**7
        +793*y**8-184*y**9+15*y**10))-8*MBhat**3*(300*r**16*(-1+y)**10-r**2*(
        -1+y)**10*(58400-14280*y**4-7308*y**5-140*y**6+15*y**7)+(-1+y)**10*(2300
        -7980*y**4-6573*y**5+4732*y**6+1788*y**7+60*y**8)+280*r**6*(-1+y)**4*(390
        +4200*logy*(-1+y)**6+1860*y-17250*y**2+44000*y**3-55230*y**4+35097*y**5
        -11071*y**6+684*y**7+51*y**8-y**9)-105*r**4*(-1+y)**5*(-9064-4320*logy*(
        -1+y)**5+49640*y-110080*y**2+124480*y**3-72260*y**4+18715*y**5-1347*y**6
        +1020*y**7-480*y**8+45*y**9+3*y**10)-35*r**8*(-1+y)**3*(25800-9600*logy*(
        -1+y)**7-171000*y+479400*y**2-731800*y**3+663580*y**4-332697*y**5
        +96995*y**6-9191*y**7+2949*y**8-700*y**9+64*y**10)+20*r**14*(-164+1640*y
        -7380*y**2+19680*y**3-36120*y**4+41874*y**5-35196*y**6+20310*y**7-7740*y**8
        +1775*y**9-194*y**10+3*y**11)-21*r**10*(-1+y)**2*(800-6400*y+22400*y**2
        -44800*y**3+77720*y**4-57988*y**5+55476*y**6-18987*y**7+6000*y**8-598*y**9
        -28*y**10+5*y**11)-7*r**12*(-2200+22000*y-99000*y**2+264000*y**3-489660*y**4
        +595149*y**5-507390*y**6+309225*y**7-115380*y**8+25915*y**9-2674*y**10
        +15*y**11))-6*MBhat**2*(35*r**16*(-1+y)**10-14*r**2*(-1+y)**10*(-925
        +864*y**5+249*y**6)-(-1+y)**10*(385-8736*y**5-1554*y**6+3296*y**7
        +399*y**8)+70*r**6*(-1+y)**4*(6237-9660*logy*(-1+y)**6-47082*y+146685*y**2
        -243880*y**3+231210*y**4-115320*y**5+24390*y**6+3544*y**7-1407*y**8
        +98*y**9+5*y**10)-10*r**14*(35-350*y+1575*y**2-4200*y**3+7350*y**4
        -12852*y**5+4326*y**6-2760*y**7+1035*y**8-230*y**9+23*y**10)-70*r**8*(
        -1+y)**3*(-7875+8400*logy*(-1+y)**7+46725*y-110775*y**2+125825*y**3
        -52325*y**4-37053*y**5+43650*y**6-21054*y**7+2697*y**8-287*y**9+32*y**10)
        +70*r**4*(-1+y)**5*(-5355-2100*logy*(-1+y)**5+28875*y-63000*y**2
        +70000*y**3-40250*y**4+11734*y**5-1994*y**6+442*y**7+505*y**8-325*y**9
        +38*y**10+2*y**11)-14*r**10*(-1+y)**2*(19285+6300*logy*(-1+y)**8-147980*y
        +492730*y**2-926660*y**3+1070125*y**4-807356*y**5+329779*y**6-87260*y**7
        +1510*y**8+1480*y**9-269*y**10+16*y**11)+14*r**12*(-175+1750*y-7875*y**2
        +21000*y**3-36750*y**4+27444*y**5-25296*y**6+24140*y**7-8675*y**8+5810*y**9
        -1549*y**10+176*y**11))+MBhat**4*(-9975*r**16*(-1+y)**10-420*r**2*(-1
        +y)**10*(2380-256*y**3-215*y**4-16*y**5+y**6)+(-1+y)**10*(53975-40320*y**3
        -83510*y**4+26880*y**5+34888*y**6+3200*y**7+15*y**8)+210*r**4*(-1+y)**5*(37072
        +23760*logy*(-1+y)**5-209120*y+477640*y**2-557800*y**3+333645*y**4
        -83217*y**5-5050*y**6+3850*y**7-395*y**8+15*y**9)+280*r**6*(-1+y)**4*(22545
        +23100*logy*(-1+y)**6-112170*y+211125*y**2-170320*y**3+6865*y**4
        +66774*y**5-43435*y**6+7016*y**7-1128*y**8+128*y**9)+420*r**8*(-1+y)**3*(
        -4400+2250*logy*(-1+y)**7+28550*y-77775*y**2+108915*y**3-92065*y**4
        +29969*y**5-3204*y**6-4510*y**7+848*y**8-160*y**9+32*y**10)-84*r**10*(
        -1+y)**2*(-10100+80800*y-282800*y**2+586720*y**3-728475*y**4+637320*y**5
        -331793*y**6+113584*y**7-20309*y**8+1208*y**9+45*y**10)-120*r**14*(
        -751+7510*y-33795*y**2+91240*y**3-159880*y**4+193200*y**5-162540*y**6
        +94200*y**7-36150*y**8+8400*y**9-950*y**10+20*y**11)+70*r**12*(-5120
        +51200*y-230400*y**2+625344*y**3-1102565*y**4+1348370*y**5-1155969*y**6
        +678456*y**7-265575*y**8+63710*y**9-7647*y**10+184*y**11+12*y**12))
        -210*MBhat**8*(-1+y)**4*(40*r**2*(-1+y)**6+5*r**8*(-1+y)**6-5*(-1+y)**6*(1
        +y**4)+4*r**6*(-10+60*y-150*y**2+200*y**3-145*y**4+90*y**5-29*y**6
        +4*y**7)-2*r**4*(-1+y)*(30*logy*(-1+y)**5+y*(-30+135*y-235*y**2
        +185*y**3-103*y**4+24*y**5)))+15*(-3*r**16*(-1+y)**10+16*r**2*(-1+y)**10*(
        -9+16*y**7)+(-1+y)**10*(3-256*y**7+129*y**8)+336*r**10*(-1+y)**2*(77
        +60*logy*(-1+y)**8-556*y+1706*y**2-2852*y**3+2725*y**4-1340*y**5
        +110*y**6+156*y**7-117*y**8+20*y**9+y**10)+42*r**4*(-1+y)**5*(182
        +60*logy*(-1+y)**5-970*y+2090*y**2-2290*y**3+1295*y**4-319*y**5
        +10*y**6-90*y**7+135*y**8-75*y**9+16*y**10)+14*r**12*(-1+y)*(-546
        +180*logy*(-1+y)**9+4734*y-18126*y**2+40134*y**3-56421*y**4+51885*y**5
        -30810*y**6+10514*y**7-2661*y**8+129*y**9+52*y**10-4*y**11)-8*r**14*(
        -18+180*y-810*y**2+2160*y**3-3780*y**4+4536*y**5-3780*y**6+2320*y**7
        -475*y**8+210*y**9-43*y**10+4*y**11)-56*r**6*(-1+y)**4*(462-360*logy*(
        -1+y)**6-3132*y+8910*y**2-13680*y**3+12060*y**4-5904*y**5+1344*y**6
        +200*y**7-225*y**8+90*y**9-29*y**10+4*y**11)-70*r**8*(-1+y)**3*(-540*logy*(
        -1+y)**7+y*(540-3510*y+9630*y**2-14355*y**3+12393*y**4-6021*y**5
        +1711*y**6-115*y**7-27*y**8-7*y**9+y**10))))/(8400*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm2(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m2 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (165*r**8*(-1+y)**6+12*logy*(8+27*r**4)*(-1+y)**6+(-1+y)**6*(-533
        +96*y+48*y**2-64*y**3+3*y**4)-24*r**2*(-1+y)**3*(43-129*y+129*y**2
        -31*y**3-16*y**4+8*y**5)+4*r**6*(14-84*y+210*y**2-64*y**3-141*y**4
        +222*y**5-121*y**6+24*y**7)+6*r**4*(-120+774*y-2097*y**2+2986*y**3
        -2432*y**4+1152*y**5-295*y**6+32*y**7))/(72*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (60*logy*(-32+128*r**2-111*r**4+81*r**6+MBhat*(16-48*r**2)+2*MBhat**2*(8
        +27*r**4))*(-1+y)**7+10*MBhat**2*(-1+y)*(165*r**8*(-1+y)**6+(-1+y)**6*(
        -533+96*y+48*y**2-64*y**3+3*y**4)-24*r**2*(-1+y)**3*(43-129*y+129*y**2
        -31*y**3-16*y**4+8*y**5)+4*r**6*(14-84*y+210*y**2-64*y**3-141*y**4
        +222*y**5-121*y**6+24*y**7)+6*r**4*(-120+774*y-2097*y**2+2986*y**3
        -2432*y**4+1152*y**5-295*y**6+32*y**7))+5*(99*r**10*(-1+y)**7+(-1
        +y)**7*(125-384*y-192*y**2-128*y**3+69*y**4)+r**2*(-1+y)**4*(947
        -4377*y+6681*y**2-3763*y**3+285*y**4+105*y**5-73*y**6+3*y**7)+6*r**4*(
        -1+y)**2*(-168+1062*y-2679*y**2+3419*y**3-2138*y**4+644*y**5-99*y**6
        +7*y**7)+r**8*(-83+581*y-1743*y**2+2905*y**3-1015*y**4-543*y**5+1057*y**6
        -535*y**7+96*y**8)+2*r**6*(184-1774*y+7023*y**2-15107*y**3+18167*y**4
        -12891*y**5+5617*y**6-1357*y**7+129*y**8+9*y**9))+2*MBhat*(1512*r**10*(
        -1+y)**7-(-1+y)**7*(-3128-480*y-240*y**2+50*y**3+3*y**5)-60*r**2*(
        -1+y)**4*(-174+498*y-462*y**2+91*y**3+85*y**4-69*y**5+15*y**6)-30*r**4*(
        -1+y)**2*(432-2160*y+4320*y**2-4180*y**3+2000*y**4-283*y**5-117*y**6
        +33*y**7+3*y**8)+15*r**8*(168-1176*y+3528*y**2-6510*y**3+7250*y**4
        -5117*y**5+2219*y**6-537*y**7+55*y**8)-20*r**6*(-232+1624*y-4872*y**2
        +7565*y**3-6845*y**4+3738*y**5-1174*y**6+241*y**7-57*y**8+12*y**9)))/(720*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (5258-34560*r**2+38850*r**4-11200*r**6+1350*r**8+192*r**10+110*r**12
        +60*logy*(MBhat**3*(16-48*r**2)+16*MBhat*(7-24*r**2+9*r**4)+MBhat**4*(8
        +27*r**4)+MBhat**2*(-96+288*r**2-273*r**4+127*r**6)+2*(-20+72*r**2
        +147*r**4-88*r**6+27*r**8))*(-1+y)**8-44464*y+285120*r**2*y-293160*r**4*y
        +79040*r**6*y-7560*r**8*y-1536*r**10*y-880*r**12*y+165224*y**2-1032480*r**2*y**2
        +955500*r**4*y**2-234400*r**6*y**2+13500*r**8*y**2+5376*r**10*y**2+3080*r**12*y**2
        -352848*y**3+2145600*r**2*y**3-1746360*r**4*y**3+370240*r**6*y**3+3240*r**8*y**3
        -10752*r**10*y**3-6160*r**12*y**3+474660*y**4-2802960*r**2*y**4+1935990*r**4*y**4
        -314960*r**6*y**4-49410*r**8*y**4+13440*r**10*y**4+7700*r**12*y**4-412608*y**5
        +2362560*r**2*y**5-1302600*r**4*y**5+110560*r**6*y**5+73800*r**8*y**5-5280*r**10*y**5
        -6160*r**12*y**5+223693*y**6-1257120*r**2*y**6+486810*r**4*y**6+32700*r**6*y**6
        -54015*r**8*y**6+156*r**10*y**6+3080*r**12*y**6-57656*y**7+380640*r**2*y**7
        -63720*r**4*y**7-49920*r**6*y**7+24480*r**8*y**7+1632*r**10*y**7-880*r**12*y**7
        -20090*y**8-36000*r**2*y**8-18480*r**4*y**8+22680*r**6*y**8-6030*r**8*y**8
        -852*r**10*y**8+110*r**12*y**8+35816*y**9-18240*r**2*y**9+8400*r**4*y**9
        -5280*r**6*y**9+600*r**8*y**9+144*r**10*y**9-25810*y**10+9600*r**2*y**10
        -1230*r**4*y**10+540*r**6*y**10+45*r**8*y**10+11336*y**11-2400*r**2*y**11
        -2828*y**12+240*r**2*y**12+328*y**13-11*y**14+5*MBhat**4*(-1+y)**2*(165*r**8*(
        -1+y)**6+(-1+y)**6*(-533+96*y+48*y**2-64*y**3+3*y**4)-24*r**2*(
        -1+y)**3*(43-129*y+129*y**2-31*y**3-16*y**4+8*y**5)+4*r**6*(14-84*y
        +210*y**2-64*y**3-141*y**4+222*y**5-121*y**6+24*y**7)+6*r**4*(-120
        +774*y-2097*y**2+2986*y**3-2432*y**4+1152*y**5-295*y**6+32*y**7))
        +2*MBhat**3*(-1+y)*(1512*r**10*(-1+y)**7-(-1+y)**7*(-3128-480*y-240*y**2
        +50*y**3+3*y**5)-60*r**2*(-1+y)**4*(-174+498*y-462*y**2+91*y**3+85*y**4
        -69*y**5+15*y**6)-30*r**4*(-1+y)**2*(432-2160*y+4320*y**2-4180*y**3
        +2000*y**4-283*y**5-117*y**6+33*y**7+3*y**8)+15*r**8*(168-1176*y
        +3528*y**2-6510*y**3+7250*y**4-5117*y**5+2219*y**6-537*y**7+55*y**8)
        -20*r**6*(-232+1624*y-4872*y**2+7565*y**3-6845*y**4+3738*y**5-1174*y**6
        +241*y**7-57*y**8+12*y**9))+4*MBhat*(180*r**12*(-1+y)**8+120*r**2*(
        -1+y)**5*(-104+360*y-432*y**2+192*y**3-15*y**4+3*y**5)-4*(-1+y)**8*(599
        -420*y-210*y**2-140*y**3+30*y**4+3*y**5)-15*r**4*(-1+y)**3*(-1196
        +6124*y-12608*y**2+13088*y**3-6848*y**4+1571*y**5-23*y**6-73*y**7
        +13*y**8)-5*r**6*(-1+y)**2*(-1984+11904*y-29760*y**2+39680*y**3-28728*y**4
        +11571*y**5-2170*y**6+54*y**7+30*y**8+3*y**9)-15*r**8*(124-992*y
        +3472*y**2-6944*y**3+8076*y**4-5661*y**5+2376*y**6-462*y**7+11*y**9)
        +3*r**10*(-128+1024*y-3584*y**2+7168*y**3-10480*y**4+9689*y**5-5992*y**6
        +2372*y**7-544*y**8+55*y**9))+MBhat**2*(2520*r**12*(-1+y)**8+(-1+y)**8*(295
        -5760*y-2880*y**2-480*y**3+405*y**4+y**6)+15*r**2*(-1+y)**5*(-113
        -813*y+2541*y**2-2127*y**3+603*y**4-337*y**5+49*y**6+5*y**7)-30*r**4*(
        -1+y)**3*(-316+1034*y-703*y**2-1309*y**3+2122*y**4-1568*y**5+789*y**6
        -209*y**7+16*y**8)-10*r**6*(-1+y)**2*(-776+3894*y-7449*y**2+4298*y**3
        -885*y**4-786*y**5+109*y**6+330*y**7-207*y**8+32*y**9)+3*r**10*(-2405
        +19240*y-67340*y**2+140760*y**3-184860*y**4+158760*y**5-88452*y**6
        +30456*y**7-5751*y**8+432*y**9)-15*r**8*(-295+2360*y-8260*y**2+18792*y**3
        -26366*y**4+23048*y**5-12457*y**6+3328*y**7+91*y**8-296*y**9+55*y**10)))/(360*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-158887+1385685*r**2+1378125*r**4-3158575*r**6+610575*r**8-53613*r**10
        -2485*r**12-825*r**14+420*logy*(-128+480*r**2+5895*r**4+1955*r**6
        -765*r**8+135*r**10-48*MBhat**5*(-1+3*r**2)+2*MBhat**6*(8+27*r**4)
        +32*MBhat**3*(35-114*r**2+57*r**4)+48*MBhat*(13-49*r**2-247*r**4+55*r**6)
        +3*MBhat**4*(-160+448*r**2-435*r**4+173*r**6)+6*MBhat**2*(-200+720*r**2
        +898*r**4-647*r**6+115*r**8))*(-1+y)**9+1483743*y-12672765*r**2*y
        -14879025*r**4*y+27606075*r**6*y-5173875*r**8*y+425817*r**10*y+22365*r**12*y
        +7425*r**14*y-6176892*y**2+51598260*r**2*y**2+70657650*r**4*y**2-106729350*r**6*y**2
        +19249650*r**8*y**2-1448118*r**10*y**2-89460*r**12*y**2-29700*r**14*y**2
        +15057868*y**3-122815140*r**2*y**3-194578650*r**4*y**3+239181950*r**6*y**3
        -41060250*r**8*y**3+2698542*r**10*y**3+208740*r**12*y**3+69300*r**14*y**3
        -23715762*y**4+188456310*r**2*y**4+343861875*r**4*y**4-341529825*r**6*y**4
        +54843075*r**8*y**4-2857113*r**10*y**4-313110*r**12*y**4-103950*r**14*y**4
        +25070514*y**5-193536630*r**2*y**5-406254555*r**4*y**5+320838105*r**6*y**5
        -46746315*r**8*y**5+1428273*r**10*y**5+313110*r**12*y**5+103950*r**14*y**5
        -17855131*y**6+133281771*r**2*y**6+322819140*r**4*y**6-196723100*r**6*y**6
        +24655995*r**8*y**6-16527*r**10*y**6-118776*r**12*y**6-69300*r**14*y**6+8397027*y**7
        -59708019*r**2*y**7-167955900*r**4*y**7+74582340*r**6*y**7-7138635*r**8*y**7
        -377265*r**10*y**7+18984*r**12*y**7+29700*r**14*y**7-2704923*y**8+16367001*r**2*y**8
        +52786755*r**4*y**8-14868945*r**6*y**8+495390*r**8*y**8+266490*r**10*y**8
        +14679*r**12*y**8-7425*r**14*y**8+1148947*y**9-3007809*r**2*y**9-7782495*r**4*y**9
        +505925*r**6*y**9+369390*r**8*y**9-75390*r**10*y**9-8351*r**12*y**9+825*r**14*y**9
        -1152354*y**10+1195026*r**2*y**10-264390*r**4*y**10+356370*r**6*y**10-116445*r**8*y**10
        +8337*r**10*y**10+1344*r**12*y**10+1035426*y**11-818706*r**2*y**11+247590*r**4*y**11
        -64050*r**6*y**11+11445*r**8*y**11+567*r**10*y**11-617260*y**12+348684*r**2*y**12
        -39795*r**4*y**12+3185*r**6*y**12+234780*y**13-82236*r**2*y**13+3675*r**4*y**13
        -105*r**6*y**13-52935*y**14+8799*r**2*y**14+6127*y**15-231*r**2*y**15-288*y**16
        +70*MBhat**6*(-1+y)**3*(165*r**8*(-1+y)**6+(-1+y)**6*(-533+96*y+48*y**2
        -64*y**3+3*y**4)-24*r**2*(-1+y)**3*(43-129*y+129*y**2-31*y**3-16*y**4
        +8*y**5)+4*r**6*(14-84*y+210*y**2-64*y**3-141*y**4+222*y**5-121*y**6
        +24*y**7)+6*r**4*(-120+774*y-2097*y**2+2986*y**3-2432*y**4+1152*y**5
        -295*y**6+32*y**7))+42*MBhat**5*(-1+y)**2*(1512*r**10*(-1+y)**7-(-1
        +y)**7*(-3128-480*y-240*y**2+50*y**3+3*y**5)-60*r**2*(-1+y)**4*(-174
        +498*y-462*y**2+91*y**3+85*y**4-69*y**5+15*y**6)-30*r**4*(-1+y)**2*(432
        -2160*y+4320*y**2-4180*y**3+2000*y**4-283*y**5-117*y**6+33*y**7+3*y**8)
        +15*r**8*(168-1176*y+3528*y**2-6510*y**3+7250*y**4-5117*y**5+2219*y**6
        -537*y**7+55*y**8)-20*r**6*(-232+1624*y-4872*y**2+7565*y**3-6845*y**4
        +3738*y**5-1174*y**6+241*y**7-57*y**8+12*y**9))+21*MBhat**4*(-1+y)*(5040*r**12*(
        -1+y)**8+(-1+y)**8*(-35-9600*y-4800*y**2-320*y**3+465*y**4+2*y**6)
        +5*r**2*(-1+y)**5*(-1625-501*y+8565*y**2-8999*y**3+3333*y**4-2127*y**5
        +367*y**6+27*y**7)-30*r**4*(-1+y)**3*(-800+3130*y-4085*y**2+801*y**3
        +2106*y**4-2492*y**5+1479*y**6-411*y**7+32*y**8)-10*r**6*(-1+y)**2*(
        -1736+9378*y-20331*y**2+18270*y**3-10263*y**4+2826*y**5-1001*y**6
        +798*y**7-405*y**8+64*y**9)+3*r**10*(-4975+39800*y-139300*y**2+290760*y**3
        -381270*y**4+326760*y**5-181524*y**6+62232*y**7-11667*y**8+864*y**9)
        -5*r**8*(-1687+13496*y-47236*y**2+108104*y**3-154276*y**4+137816*y**5
        -76342*y**6+21560*y**7-85*y**8-1680*y**9+330*y**10))+6*MBhat*(936*r**14*(
        -1+y)**9+(-1+y)**9*(-111256+43680*y+21840*y**2+14560*y**3+10920*y**4
        -399*y**5-448*y**6+33*y**7)-14*r**2*(-1+y)**6*(59972-191676*y+209316*y**2
        -81532*y**3+2940*y**4+1731*y**5-1393*y**6+153*y**7+9*y**8)-140*r**6*(
        -1+y)**3*(3630-23100*y+61710*y**2-88880*y**3+73260*y**4-32877*y**5
        +7250*y**6-498*y**7-90*y**8+15*y**9)+7*r**4*(-1+y)**4*(59048-176680*y
        +56960*y**2+338240*y**3-465520*y**4+212519*y**5-26035*y**6-460*y**7
        +500*y**8-15*y**9+3*y**10)-35*r**8*(-1+y)**2*(6280-43960*y+131880*y**2
        -219800*y**3+219800*y**4-126867*y**5+42277*y**6-6676*y**7+252*y**8
        +51*y**9+3*y**10)+7*r**12*(344-3096*y+12384*y**2-28896*y**3+43344*y**4
        -52983*y**5+41727*y**6-22788*y**7+8172*y**8-1733*y**9+165*y**10)+14*r**10*(1796
        -16164*y+64656*y**2-150864*y**3+226296*y**4-212751*y**5+124647*y**6
        -44946*y**7+7314*y**8+229*y**9-225*y**10+12*y**11))+2*MBhat**3*(27360*r**14*(
        -1+y)**9-(-1+y)**9*(300528-235200*y-117600*y**2-54250*y**3+10080*y**4
        +1260*y**5+3*y**7)-84*r**2*(-1+y)**6*(17344-61152*y+74832*y**2-34004*y**3
        +2900*y**4-823*y**5+109*y**6-9*y**7+3*y**8)-210*r**4*(-1+y)**4*(-9704
        +50344*y-105248*y**2+111533*y**3-60441*y**4+15768*y**5-1744*y**6-65*y**7
        +37*y**8)+560*r**6*(-1+y)**3*(1894-11364*y+28410*y**2-38135*y**3+27393*y**4
        -10758*y**5+1796*y**6+102*y**7-99*y**8+11*y**9)-105*r**8*(-1+y)**2*(
        -2192+15344*y-46032*y**2+81310*y**3-73490*y**4+37338*y**5-4614*y**6
        -5007*y**7+3009*y**8-681*y**9+55*y**10)+14*r**12*(7144-64296*y+257184*y**2
        -616161*y**3+952329*y**4-991998*y**5+701022*y**6-328788*y**7+96372*y**8
        -15433*y**9+945*y**10)-84*r**10*(1472-13248*y+52992*y**2-130648*y**3
        +203852*y**4-209205*y**5+142245*y**6-59088*y**7+11142*y**8+1457*y**9
        -1133*y**10+162*y**11))+21*MBhat**2*(990*r**14*(-1+y)**9-(-1+y)**9*(
        -48526+24000*y+12000*y**2+8000*y**3+1875*y**4-672*y**5+13*y**6)-r**2*(
        -1+y)**6*(-304540+1000020*y-1129620*y**2+462940*y**3-27495*y**4+1845*y**5
        +1994*y**6-342*y**7-3*y**8+y**9)+20*r**4*(-1+y)**4*(-18874+88982*y
        -164494*y**2+146534*y**3-59827*y**4+7307*y**5+1223*y**6-475*y**7-28*y**8
        +12*y**9)+10*r**6*(-1+y)**3*(-14407+78678*y-173403*y**2+192384*y**3
        -103023*y**4+22446*y**5+2735*y**6-2076*y**7+234*y**8+32*y**9)-5*r**8*(
        -1+y)**2*(4658-29846*y+79878*y**2-113810*y**3+73685*y**4-29115*y**5
        +2378*y**6+754*y**7+105*y**8-239*y**9+32*y**10)+6*r**12*(600-5400*y
        +21600*y**2-50400*y**3+80955*y**4-86835*y**5+63924*y**6-31716*y**7
        +10089*y**8-1841*y**9+144*y**10)-r**10*(1924-17316*y+69264*y**2-161616*y**3
        +331149*y**4-451317*y**5+388281*y**6-227409*y**7+81891*y**8-15739*y**9
        +723*y**10+165*y**11)))/(5040*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (255697-2858224*r**2-14906388*r**4+8110480*r**6+10740800*r**8-1437072*r**10
        +91924*r**12+2288*r**14+495*r**16+840*logy*(-88+352*r**2+11841*r**4
        +19240*r**6+2615*r**8-696*r**10+81*r**12+MBhat**7*(32-96*r**2)+MBhat**8*(8
        +27*r**4)+32*MBhat**5*(49-156*r**2+87*r**4)+2*MBhat**6*(-224+608*r**2
        -597*r**4+219*r**6)+32*MBhat**3*(91-355*r**2-1431*r**4+435*r**6)+32*MBhat*(19
        -78*r**2-1458*r**4-1174*r**6+165*r**8)+5*MBhat**4*(-560+2016*r**2+1908*r**4
        -1644*r**6+237*r**8)+2*MBhat**2*(-896+3648*r**2+34725*r**4+6095*r**6
        -3780*r**8+363*r**10))*(-1+y)**10-2630890*y+28877920*r**2*y+159010320*r**4*y
        -64943200*r**6*y-105211400*r**8*y+13786080*r**10*y-851200*r**12*y-22880*r**14*y
        -4950*r**16*y+12208605*y**2-131429040*r**2*y**2-765278640*r**4*y**2+211436400*r**6*y**2
        +462468300*r**8*y**2-59114160*r**10*y**2+3490200*r**12*y**2+102960*r**14*y**2
        +22275*r**16*y**2-33665080*y**3+354912640*r**2*y**3+2189939640*r**4*y**3
        -321406400*r**6*y**3-1200299800*r**8*y**3+148868160*r**10*y**3-8286600*r**12*y**3
        -274560*r**14*y**3-59400*r**16*y**3+61131490*y**4-629967520*r**2*y**4-4130787570*r**4*y**4
        +77613200*r**6*y**4+2034626650*r**8*y**4-242980080*r**10*y**4+12460350*r**12*y**4
        +480480*r**14*y**4+103950*r**16*y**4-76462428*y**5+768379584*r**2*y**5+5374695564*r**4*y**5
        +585651360*r**6*y**5-2349294780*r**8*y**5+267021216*r**10*y**5-12094740*r**12*y**5
        -576576*r**14*y**5-124740*r**16*y**5+66823330*y**6-652734880*r**2*y**6-4896663450*r**4*y**6
        -1166830000*r**6*y**6+1865488450*r**8*y**6-197962800*r**10*y**6+7221270*r**12*y**6
        +480480*r**14*y**6+103950*r**16*y**6-40388152*y**7+381825664*r**2*y**7+3096523080*r**4*y**7
        +1151644480*r**6*y**7-1000381480*r**8*y**7+96100032*r**10*y**7-2473464*r**12*y**7
        -168960*r**14*y**7-59400*r**16*y**7+16126440*y**8-147329520*r**2*y**8-1310673840*r**4*y**8
        -674260440*r**6*y**8+342835920*r**8*y**8-28232400*r**10*y**8+348390*r**12*y**8
        +33000*r**14*y**8+22275*r**16*y**8-3423640*y**9+33159520*r**2*y**9+341913600*r**4*y**9
        +230129200*r**6*y**9-65427040*r**8*y**9+3793440*r**10*y**9+150500*r**12*y**9
        +9520*r**14*y**9-4950*r**16*y**9-909920*y**10-1314736*r**2*y**10-45826284*r**4*y**10
        -38064040*r**6*y**10+3845380*r**8*y**10+301056*r**10*y**10-65534*r**12*y**10
        -6232*r**14*y**10+495*r**16*y**10+2219160*y**11-3245760*r**2*y**11+2815680*r**4*y**11
        +386400*r**6*y**11+730800*r**8*y**11-157920*r**10*y**11+8400*r**12*y**11
        +960*r**14*y**11-2439626*y**12+2922752*r**2*y**12-1148490*r**4*y**12+568120*r**6*y**12
        -128170*r**8*y**12+14448*r**10*y**12+504*r**12*y**12+1841420*y**13-1709120*r**2*y**13
        +464100*r**4*y**13-39760*r**6*y**13+6580*r**8*y**13-947850*y**14+633600*r**2*y**14
        -86730*r**4*y**14+4200*r**6*y**14-210*r**8*y**14+321176*y**15-137024*r**2*y**15
        +9408*r**4*y**15-67025*y**16+14720*r**2*y**16+7698*y**17-576*r**2*y**17
        -405*y**18+70*MBhat**8*(-1+y)**4*(165*r**8*(-1+y)**6+(-1+y)**6*(-533
        +96*y+48*y**2-64*y**3+3*y**4)-24*r**2*(-1+y)**3*(43-129*y+129*y**2
        -31*y**3-16*y**4+8*y**5)+4*r**6*(14-84*y+210*y**2-64*y**3-141*y**4
        +222*y**5-121*y**6+24*y**7)+6*r**4*(-120+774*y-2097*y**2+2986*y**3
        -2432*y**4+1152*y**5-295*y**6+32*y**7))+56*MBhat**7*(-1+y)**3*(1512*r**10*(
        -1+y)**7-(-1+y)**7*(-3128-480*y-240*y**2+50*y**3+3*y**5)-60*r**2*(
        -1+y)**4*(-174+498*y-462*y**2+91*y**3+85*y**4-69*y**5+15*y**6)-30*r**4*(
        -1+y)**2*(432-2160*y+4320*y**2-4180*y**3+2000*y**4-283*y**5-117*y**6
        +33*y**7+3*y**8)+15*r**8*(168-1176*y+3528*y**2-6510*y**3+7250*y**4
        -5117*y**5+2219*y**6-537*y**7+55*y**8)-20*r**6*(-232+1624*y-4872*y**2
        +7565*y**3-6845*y**4+3738*y**5-1174*y**6+241*y**7-57*y**8+12*y**9))
        +28*MBhat**6*(-1+y)**2*(7560*r**12*(-1+y)**8+(-1+y)**8*(-365-13440*y
        -6720*y**2-160*y**3+525*y**4+3*y**6)+5*r**2*(-1+y)**5*(-2911+1437*y
        +9507*y**2-11617*y**3+4857*y**4-3243*y**5+587*y**6+39*y**7)-30*r**4*(
        -1+y)**3*(-1284+5226*y-7467*y**2+2911*y**3+2090*y**4-3416*y**5+2169*y**6
        -613*y**7+48*y**8)-10*r**6*(-1+y)**2*(-2696+14862*y-33213*y**2+32242*y**3
        -19641*y**4+6438*y**5-2111*y**6+1266*y**7-603*y**8+96*y**9)+9*r**10*(
        -2515+20120*y-70420*y**2+146920*y**3-192560*y**4+164920*y**5-91532*y**6
        +31336*y**7-5861*y**8+432*y**9)-5*r**8*(-2489+19912*y-69692*y**2+159832*y**3
        -229454*y**4+206488*y**5-115313*y**6+33136*y**7-443*y**8-2472*y**9
        +495*y**10))+8*MBhat**5*(-1+y)*(27360*r**14*(-1+y)**9-(-1+y)**9*(199896
        -164640*y-82320*y**2-30730*y**3+5040*y**4+756*y**5+3*y**7)+210*r**4*(
        -1+y)**4*(6116-31972*y+67424*y**2-72269*y**3+39897*y**4-11055*y**5
        +1675*y**6-154*y**7+2*y**8)-84*r**2*(-1+y)**6*(11104-39552*y+48912*y**2
        -22484*y**3+2000*y**4-643*y**5+109*y**6-9*y**7+3*y**8)+70*r**6*(-1
        +y)**3*(9200-55200*y+138000*y**2-186040*y**3+132960*y**4-51351*y**5
        +7858*y**6+978*y**7-702*y**8+97*y**9)-105*r**8*(-1+y)**2*(-1448+10136*y
        -30408*y**2+55270*y**3-51074*y**4+25788*y**5-1908*y**6-5073*y**7+2943*y**8
        -681*y**9+55*y**10)+14*r**12*(7684-69156*y+276624*y**2-661521*y**3
        +1020369*y**4-1060038*y**5+746382*y**6-348228*y**7+101232*y**8-15973*y**9
        +945*y**10)-42*r**10*(3328-29952*y+119808*y**2-293552*y**3+460648*y**4
        -478917*y**5+331533*y**6-143268*y**7+31032*y**8+1117*y**9-2101*y**10
        +324*y**11))+2*MBhat**2*(6615*r**16*(-1+y)**10-(-1+y)**10*(-2140499
        +752640*y+376320*y**2+250880*y**3+188160*y**4+61824*y**5-9170*y**6
        -288*y**7+99*y**8)-42*r**2*(-1+y)**7*(-446157+1411431*y-1520871*y**2
        +579917*y**3-18240*y**4-8544*y**5+2741*y**6+2401*y**7-449*y**8+11*y**9)
        +210*r**8*(-1+y)**3*(39685-262675*y+735105*y**2-1119335*y**3+987035*y**4
        -478317*y**5+115774*y**6-4802*y**7-2187*y**8+109*y**9+48*y**10)+70*r**6*(
        -1+y)**4*(485033-2837058*y+6873225*y**2-8798600*y**3+6233250*y**4
        -2270712*y**5+324374*y**6+9384*y**7-4995*y**8-566*y**9+105*y**10)-42*r**4*(
        -1+y)**5*(-230815+1848575*y-5433400*y**2+7748400*y**3-5610450*y**4
        +1820910*y**5-128162*y**6-18750*y**7+45*y**8+295*y**9-10*y**10+2*y**11)
        -42*r**10*(-1+y)**2*(-16453+124364*y-406234*y**2+744708*y**3-829245*y**4
        +486588*y**5-183359*y**6+23868*y**7+2418*y**8-1112*y**9-175*y**10+32*y**11)
        +6*r**14*(-4945+49450*y-222525*y**2+593400*y**3-1038450*y**4+1357020*y**5
        -1226946*y**6+786360*y**7-348345*y**8+101170*y**9-17245*y**10+1296*y**11)
        -14*r**12*(-421+4210*y-18945*y**2+50520*y**3-88410*y**4+278604*y**5
        -456888*y**6+400740*y**7-242265*y**8+90790*y**9-19903*y**10+1968*y**11))
        +8*MBhat*(432*r**16*(-1+y)**10+(-1+y)**10*(-202768+63840*y+31920*y**2
        +21280*y**3+15960*y**4+12768*y**5+728*y**6-585*y**7+72*y**8)-42*r**4*(
        -1+y)**5*(117288-703080*y+1697760*y**2-2086560*y**3+1334880*y**4-383616*y**5
        +18880*y**6+4465*y**7+175*y**8+45*y**9+3*y**10)+3*r**2*(-1+y)**7*(
        -674992+2112336*y-2243376*y**2+835152*y**3-21840*y**4-4368*y**5-9576*y**6
        +9005*y**7+201*y**8-313*y**9+11*y**10)-105*r**8*(-1+y)**3*(-6560+51200*y
        -172080*y**2+323760*y**3-369960*y**4+258936*y**5-102928*y**6+21203*y**7
        -1525*y**8-147*y**9+21*y**10)+14*r**6*(-1+y)**4*(-454088+2442768*y
        -5261640*y**2+5606720*y**3-2796240*y**4+273216*y**5+234424*y**6-49185*y**7
        -2130*y**8+770*y**9-18*y**10+3*y**11)-21*r**10*(-1+y)**2*(-15408+123264*y
        -431424*y**2+862848*y**3-1078560*y**4+862848*y**5-414376*y**6+117575*y**7
        -15592*y**8+486*y**9+96*y**10+3*y**11)+12*r**14*(-108+1080*y-4860*y**2
        +12960*y**3-22680*y**4+27216*y**5-28840*y**6+19780*y**7-9660*y**8+3155*y**9
        -618*y**10+55*y**11)+28*r**12*(-892+8920*y-40140*y**2+107040*y**3
        -187320*y**4+224784*y**5-177660*y**6+89739*y**7-28680*y**8+4210*y**9
        +98*y**10-105*y**11+6*y**12))+8*MBhat**3*(4560*r**16*(-1+y)**10+(-1
        +y)**10*(-746232+305760*y+152880*y**2+101920*y**3+52080*y**4+1197*y**5
        -1680*y**6+96*y**7)-280*r**6*(-1+y)**4*(7447-49902*y+140415*y**2-213320*y**3
        +186300*y**4-88464*y**5+20708*y**6-1506*y**7-252*y**8+44*y**9)+r**2*(
        -1+y)**7*(-5582280+17939640*y-19728840*y**2+7769080*y**3-347760*y**4
        -58800*y**5+61040*y**6-5043*y**7+9*y**8-9*y**9+3*y**10)-21*r**4*(-1
        +y)**5*(-179608+669080*y-765760*y**2+2560*y**3+569520*y**4-339383*y**5
        +46835*y**6+1140*y**7-940*y**8-105*y**9+21*y**10)+35*r**8*(-1+y)**3*(
        -32120+224840*y-674520*y**2+1124200*y**3-1136560*y**4+653619*y**5
        -211093*y**6+27613*y**7+1701*y**8-1204*y**9+124*y**10)-21*r**10*(-1
        +y)**2*(5608-44864*y+157024*y**2-314048*y**3+432600*y**4-324968*y**5
        +155824*y**6-34313*y**7-1760*y**8+3178*y**9-736*y**10+55*y**11)+4*r**14*(
        -5158+51580*y-232110*y**2+618960*y**3-1129380*y**4+1416702*y**5-1253280*y**6
        +778530*y**7-330930*y**8+90865*y**9-14284*y**10+945*y**11)-7*r**12*(
        -4360+43600*y-196200*y**2+523200*y**3-1009920*y**4+1337895*y**5-1219350*y**6
        +786819*y**7-341640*y**8+91705*y**9-11530*y**10-435*y**11+216*y**12))
        +MBhat**4*(80325*r**16*(-1+y)**10+(-1+y)**10*(4526235-2352000*y-1176000*y**2
        -689920*y**3-129150*y**4+36288*y**5+168*y**6+3*y**8)+84*r**2*(-1+y)**7*(331740
        -1096020*y+1247220*y**2-515260*y**3+30505*y**4-3195*y**5-648*y**6
        +64*y**7-9*y**8+3*y**9)+280*r**6*(-1+y)**4*(-51035+281550*y-629895*y**2
        +717280*y**3-403395*y**4+99702*y**5+3497*y**6-6048*y**7+912*y**8+32*y**9)
        +210*r**4*(-1+y)**5*(-169872+811200*y-1527000*y**2+1400120*y**3-606703*y**4
        +94907*y**5+3150*y**6-1950*y**7-615*y**8+123*y**9)+420*r**8*(-1+y)**3*(
        -5360+35150*y-97155*y**2+148375*y**3-104925*y**4+39101*y**5+6164*y**6
        -12318*y**7+5968*y**8-1328*y**9+128*y**10)-84*r**10*(-1+y)**2*(6540
        -52320*y+183120*y**2-401120*y**3+416065*y**4-259688*y**5+25111*y**6
        +82960*y**7-67529*y**8+24792*y**9-4419*y**10+288*y**11)+120*r**14*(
        -2821+28210*y-126945*y**2+344680*y**3-615664*y**4+758856*y**5-655788*y**6
        +394800*y**7-160590*y**8+41260*y**9-5798*y**10+304*y**11)-14*r**12*(
        -36640+366400*y-1648800*y**2+4576320*y**3-8176995*y**4+9897630*y**5
        -8262567*y**6+4584360*y**7-1508265*y**8+158290*y**9+78815*y**10-32328*y**11
        +3780*y**12)))/(5040*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm3(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m3 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-24*logy*(-1+y)**5+54*r**8*(-1+y)**5-2*(-1+y)**4*(13-25*y+6*y**2
        -y**3+y**4)+6*r**2*(-1+y)**2*(12-36*y+36*y**2-15*y**3-y**4+2*y**5)
        -3*r**4*(48-240*y+480*y**2-530*y**3+340*y**4-113*y**5+15*y**6)+r**6*(152
        -760*y+1520*y**2-1730*y**3+1090*y**4-353*y**5+45*y**6))/(18*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (1598+110*r**2-2880*r**4+1840*r**6-830*r**8+162*r**10-60*logy*(10
        +4*MBhat**2+34*r**2-MBhat*(14+9*r**2+3*r**4))*(-1+y)**6-10188*y
        -2700*r**2*y+17280*r**4*y-11040*r**6*y+4980*r**8*y-972*r**10*y+27270*y**2
        +12870*r**2*y**2-43200*r**4*y**2+27600*r**6*y**2-12450*r**8*y**2+2430*r**10*y**2
        -39360*y**3-27360*r**2*y**3+57600*r**4*y**3-36800*r**6*y**3+16600*r**8*y**3
        -3240*r**10*y**3+32550*y**4+30990*r**2*y**4-44730*r**4*y**4+30630*r**6*y**4
        -14250*r**8*y**4+2430*r**10*y**4-14991*y**5-19260*r**2*y**5+21300*r**4*y**5
        -17040*r**6*y**5+7275*r**8*y**5-972*r**10*y**5+3516*y**6+5970*r**2*y**6
        -6390*r**4*y**6+5770*r**6*y**6-2000*r**8*y**6+162*r**10*y**6-645*y**7-480*r**2*y**7
        +1080*r**4*y**7-1020*r**6*y**7+225*r**8*y**7+360*y**8-180*r**2*y**8-60*r**4*y**8
        +60*r**6*y**8-125*y**9+40*r**2*y**9+18*y**10-3*y**11-10*MBhat**2*(-1
        +y)*(-54*r**8*(-1+y)**5+2*(-1+y)**4*(13-25*y+6*y**2-y**3+y**4)
        -6*r**2*(-1+y)**2*(12-36*y+36*y**2-15*y**3-y**4+2*y**5)+r**6*(-152
        +760*y-1520*y**2+1730*y**3-1090*y**4+353*y**5-45*y**6)+3*r**4*(48
        -240*y+480*y**2-530*y**3+340*y**4-113*y**5+15*y**6))+5*MBhat*(147*r**10*(
        -1+y)**6-(-1+y)**5*(-305+473*y-84*y**2-28*y**3-11*y**4+3*y**5)
        -3*r**2*(-1+y)**3*(143-393*y+339*y**2-109*y**3-2*y**4+6*y**5)-3*r**4*(
        -1+y)**2*(172-700*y+1074*y**2-900*y**3+257*y**4+76*y**5-59*y**6
        +8*y**7)+3*r**8*(-185+1110*y-2775*y**2+3940*y**3-3270*y**4+1560*y**5
        -386*y**6+36*y**7)+r**6*(800-4800*y+12000*y**2-17104*y**3+14529*y**4
        -6870*y**5+1460*y**6+30*y**7-45*y**8)))/(180*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-3344-9128*r**2+6760*r**4+7520*r**6-2320*r**8+584*r**10-72*r**12
        -120*logy*(8+2*MBhat**4+68*r**2+92*r**4+MBhat**2*(30+62*r**2)-MBhat**3*(14
        +9*r**2+3*r**4)+MBhat*(-26-121*r**2-45*r**4+2*r**6))*(-1+y)**7+24368*y
        +72056*r**2*y-36280*r**4*y-52640*r**6*y+16240*r**8*y-4088*r**10*y+504*r**12*y
        -76464*y**2-244728*r**2*y**2+70200*r**4*y**2+157920*r**6*y**2-48720*r**8*y**2
        +12264*r**10*y**2-1512*r**12*y**2+134160*y**3+465000*r**2*y**3-39720*r**4*y**3
        -263200*r**6*y**3+81200*r**8*y**3-20440*r**10*y**3+2520*r**12*y**3-142560*y**4
        -536400*r**2*y**4-56880*r**4*y**4+263200*r**6*y**4-81200*r**8*y**4+20440*r**10*y**4
        -2520*r**12*y**4+92199*y**5+378711*r**2*y**5+113190*r**4*y**5-162210*r**6*y**5
        +53235*r**8*y**5-13965*r**10*y**5+1512*r**12*y**5-33753*y**6-153897*r**2*y**6
        -81930*r**4*y**6+64070*r**6*y**6-24685*r**8*y**6+5915*r**10*y**6-504*r**12*y**6
        +4899*y**7+28911*r**2*y**7+29550*r**4*y**7-17870*r**6*y**7+7375*r**8*y**7
        -1385*r**10*y**7+72*r**12*y**7+1155*y**8-345*r**2*y**8-5370*r**4*y**8+3570*r**6*y**8
        -1185*r**8*y**8+135*r**10*y**8-955*y**9+5*r**2*y**9+500*r**4*y**9-360*r**6*y**9
        +60*r**8*y**9+325*y**10-203*r**2*y**10-20*r**4*y**10-15*y**11+21*r**2*y**11
        -15*y**12-3*r**2*y**12-10*MBhat**4*(-1+y)**2*(-54*r**8*(-1+y)**5+2*(
        -1+y)**4*(13-25*y+6*y**2-y**3+y**4)-6*r**2*(-1+y)**2*(12-36*y
        +36*y**2-15*y**3-y**4+2*y**5)+r**6*(-152+760*y-1520*y**2+1730*y**3
        -1090*y**4+353*y**5-45*y**6)+3*r**4*(48-240*y+480*y**2-530*y**3+340*y**4
        -113*y**5+15*y**6))+10*MBhat**3*(-1+y)*(147*r**10*(-1+y)**6-(-1+y)**5*(
        -305+473*y-84*y**2-28*y**3-11*y**4+3*y**5)-3*r**2*(-1+y)**3*(143
        -393*y+339*y**2-109*y**3-2*y**4+6*y**5)-3*r**4*(-1+y)**2*(172-700*y
        +1074*y**2-900*y**3+257*y**4+76*y**5-59*y**6+8*y**7)+3*r**8*(-185
        +1110*y-2775*y**2+3940*y**3-3270*y**4+1560*y**5-386*y**6+36*y**7)
        +r**6*(800-4800*y+12000*y**2-17104*y**3+14529*y**4-6870*y**5+1460*y**6
        +30*y**7-45*y**8))+2*MBhat*(175*r**12*(-1+y)**7-5*r**2*(-1+y)**4*(-499
        +2949*y-5127*y**2+3161*y**3-438*y**4-102*y**5+8*y**6)+(-1+y)**6*(4700
        -6260*y+780*y**2+260*y**3+220*y**4+60*y**5-y**6+y**7)+15*r**4*(-1
        +y)**3*(571-2104*y+2796*y**2-1504*y**3+376*y**4-20*y**5-27*y**6+8*y**7)
        -20*r**6*(-1+y)**2*(-145+719*y-1423*y**2+1403*y**3-1024*y**4+380*y**5
        -37*y**6-13*y**7+2*y**8)+3*r**10*(335-2345*y+7035*y**2-11725*y**3
        +12670*y**4-8610*y**5+3542*y**6-794*y**7+72*y**8)-10*r**8*(236-1652*y
        +4956*y**2-8260*y**3+8995*y**4-6573*y**5+2975*y**6-761*y**7+84*y**8))
        +2*MBhat**2*(480*r**12*(-1+y)**7-3*(-1+y)**6*(1466-2066*y+300*y**2
        +125*y**3+55*y**4)-5*r**2*(-1+y)**4*(-634+1158*y-42*y**2-679*y**3
        +315*y**4-51*y**5+5*y**6)+30*r**4*(-1+y)**3*(-20+80*y-120*y**2+17*y**3
        -95*y**4+118*y**5-50*y**6+7*y**7)-30*r**6*(-1+y)**2*(-84+420*y-840*y**2
        +1025*y**3-448*y**4-80*y**5+153*y**6-51*y**7+5*y**8)+3*r**10*(706
        -4942*y+14826*y**2-25655*y**3+27125*y**4-17745*y**5+6895*y**6-1405*y**7
        +105*y**8)-5*r**8*(706-4942*y+14826*y**2-26075*y**3+28025*y**4-17955*y**5
        +6265*y**6-691*y**7-213*y**8+54*y**9)))/(180*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-840*logy*(22+4*MBhat**6+342*r**2+990*r**4+710*r**6+30*MBhat**4*(5
        +9*r**2)-3*MBhat**5*(14+9*r**2+3*r**4)+12*MBhat**2*(20+135*r**2+111*r**4)
        +MBhat**3*(-260-990*r**2-243*r**4+3*r**6)+3*MBhat*(-38-405*r**2-690*r**4
        -165*r**6+10*r**8))*(-1+y)**8-140*MBhat**6*(-1+y)**3*(-54*r**8*(-1+y)**5
        +2*(-1+y)**4*(13-25*y+6*y**2-y**3+y**4)-6*r**2*(-1+y)**2*(12-36*y
        +36*y**2-15*y**3-y**4+2*y**5)+r**6*(-152+760*y-1520*y**2+1730*y**3
        -1090*y**4+353*y**5-45*y**6)+3*r**4*(48-240*y+480*y**2-530*y**3+340*y**4
        -113*y**5+15*y**6))+210*MBhat**5*(-1+y)**2*(147*r**10*(-1+y)**6-(-1
        +y)**5*(-305+473*y-84*y**2-28*y**3-11*y**4+3*y**5)-3*r**2*(-1+y)**3*(143
        -393*y+339*y**2-109*y**3-2*y**4+6*y**5)-3*r**4*(-1+y)**2*(172-700*y
        +1074*y**2-900*y**3+257*y**4+76*y**5-59*y**6+8*y**7)+3*r**8*(-185
        +1110*y-2775*y**2+3940*y**3-3270*y**4+1560*y**5-386*y**6+36*y**7)
        +r**6*(800-4800*y+12000*y**2-17104*y**3+14529*y**4-6870*y**5+1460*y**6
        +30*y**7-45*y**8))+42*MBhat**4*(-1+y)*(960*r**12*(-1+y)**7-30*r**2*(
        -1+y)**4*(-215+465*y-195*y**2-98*y**3+79*y**4-19*y**5+3*y**6)+(
        -1+y)**6*(-7198+10198*y-1500*y**2-650*y**3-250*y**4-3*y**5+3*y**6)
        +30*r**4*(-1+y)**3*(56-224*y+336*y**2-350*y**3-43*y**4+204*y**5-98*y**6
        +14*y**7)-10*r**6*(-1+y)**2*(-688+3440*y-6880*y**2+7990*y**3-3911*y**4
        +y**5+822*y**6-300*y**7+30*y**8)+6*r**10*(733-5131*y+15393*y**2-26600*y**3
        +28070*y**4-18312*y**5+7084*y**6-1432*y**7+105*y**8)-15*r**8*(526-3682*y
        +11046*y**2-19320*y**3+20740*y**4-13405*y**5+4795*y**6-609*y**7-127*y**8
        +36*y**9))+4*(135*r**14*(-1+y)**8+105*r**4*(-1+y)**4*(813-5232*y+11808*y**2
        -11832*y**3+4938*y**4-396*y**5-134*y**6+2*y**7)-21*r**2*(-1+y)**5*(6009
        -21447*y+26577*y**2-12279*y**3+855*y**4+171*y**5+87*y**6+43*y**7
        +4*y**8)+(-1+y)**7*(-18709+23329*y-2310*y**2-770*y**3-385*y**4-231*y**5
        -406*y**6-107*y**7+9*y**8)-35*r**6*(-1+y)**3*(-5245+21965*y-33280*y**2
        +19080*y**3+1110*y**4-4482*y**5+1310*y**6-190*y**7+20*y**8)-105*r**8*(
        -1+y)**2*(525-3150*y+7875*y**2-10500*y**3+7875*y**4-3150*y**5+813*y**6
        -181*y**7+23*y**8)+7*r**12*(-227+1816*y-6356*y**2+12712*y**3-15890*y**4
        +12712*y**5-7196*y**6+2596*y**7-527*y**8+45*y**9)+21*r**10*(471-3768*y
        +13188*y**2-26376*y**3+32970*y**4-26376*y**5+14206*y**6-5576*y**7+1462*y**8
        -210*y**9+9*y**10))+21*MBhat*(130*r**14*(-1+y)**8+(-1+y)**7*(16992
        -21552*y+2280*y**2+760*y**3+380*y**4+444*y**5+207*y**6+9*y**7)+30*r**4*(
        -1+y)**4*(694-16*y-5496*y**2+9184*y**3-5056*y**4+744*y**5+83*y**6
        -20*y**7+3*y**8)+10*r**6*(-1+y)**3*(-6595+30995*y-57040*y**2+50440*y**3
        -20270*y**4+3802*y**5-355*y**6-73*y**7+32*y**8)+r**2*(-1+y)**5*(64090
        -240870*y+313770*y**2-153190*y**3+12150*y**4+2910*y**5+1469*y**6+153*y**7
        -3*y**8+y**9)-5*r**8*(-1+y)**2*(1680-9840*y+23880*y**2-30640*y**3
        +21780*y**4-13056*y**5+4715*y**6-738*y**7-9*y**8+8*y**9)+4*r**12*(
        -265+2120*y-7420*y**2+14840*y**3-18550*y**4+16100*y**5-9184*y**6
        +3272*y**7-652*y**8+54*y**9)+3*r**10*(1214-9712*y+33992*y**2-67984*y**3
        +84980*y**4-74032*y**5+46403*y**6-18952*y**7+4628*y**8-552*y**9+15*y**10))
        +6*MBhat**2*(1320*r**14*(-1+y)**8-210*r**4*(-1+y)**4*(1114-3568*y+3576*y**2
        -608*y**3-596*y**4+271*y**5-66*y**6+12*y**7)-21*r**2*(-1+y)**5*(8252
        -35556*y+51756*y**2-28052*y**3+2800*y**4+899*y**5+103*y**6-3*y**7
        +y**8)-(-1+y)**7*(112584-146184*y+16800*y**2+5600*y**3+4060*y**4
        +2583*y**5+357*y**6-3*y**7+3*y**8)+70*r**6*(-1+y)**3*(692-3460*y
        +6920*y**2-6920*y**3+2140*y**4-641*y**5+527*y**6-208*y**7+26*y**8)
        -105*r**8*(-1+y)**2*(152-912*y+2280*y**2-3040*y**3+3620*y**4-1847*y**5
        +298*y**6+116*y**7-52*y**8+5*y**9)+14*r**12*(-578+4624*y-16184*y**2
        +32368*y**3-42560*y**4+36694*y**5-20552*y**6+7132*y**7-1364*y**8+105*y**9)
        -21*r**10*(-916+7328*y-25648*y**2+51296*y**3-69060*y**4+63033*y**5
        -37044*y**6+13476*y**7-2634*y**8+151*y**9+18*y**10))+7*MBhat**3*(2430*r**14*(
        -1+y)**8+15*r**2*(-1+y)**5*(96-8208*y+20088*y**2-14616*y**3+2677*y**4
        +289*y**5-9*y**6+3*y**7)+(-1+y)**7*(90780-121980*y+15600*y**2+5920*y**3
        +3935*y**4+945*y**5-6*y**6+6*y**7)-30*r**4*(-1+y)**4*(-3297+12216*y
        -16380*y**2+8656*y**3-3019*y**4+1152*y**5-312*y**6+24*y**7)+10*r**6*(
        -1+y)**3*(431-2119*y+4148*y**2-188*y**3+10819*y**4-12827*y**5+7094*y**6
        -1966*y**7+224*y**8)-15*r**8*(-1+y)**2*(948-5688*y+14220*y**2-23520*y**3
        +13265*y**4+1274*y**5-6447*y**6+3728*y**7-892*y**8+72*y**9)+10*r**12*(
        -1183+9464*y-33124*y**2+67928*y**3-87934*y**4+73976*y**5-40012*y**6
        +13160*y**7-2293*y**8+144*y**9)-3*r**10*(-7080+56640*y-198240*y**2
        +415040*y**3-542335*y**4+444920*y**5-222208*y**6+55784*y**7+641*y**8
        -3792*y**9+630*y**10)))/(2520*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-26371-322696*r**2-680904*r**4+159656*r**6+780500*r**8+100296*r**10
        -11704*r**12+1304*r**14-81*r**16-840*logy*(7+MBhat**8+172*r**2+828*r**4
        +1268*r**6+530*r**8+2*MBhat**6*(35+59*r**2)-MBhat**7*(14+9*r**2+3*r**4)
        +70*MBhat**4*(4+24*r**2+15*r**4)-MBhat**5*(182+627*r**2+108*r**4+3*r**6)
        +2*MBhat**2*(77+1047*r**2+2349*r**4+1165*r**6)+MBhat**3*(-266-2495*r**2
        -3285*r**4-585*r**6+30*r**8)+MBhat*(-50-933*r**2-3180*r**4-3010*r**6
        -450*r**8+21*r**10))*(-1+y)**9+243219*y+3048744*r**2*y+6823656*r**4*y
        -371784*r**6*y-6579300*r**8*y-902664*r**10*y+105336*r**12*y-11736*r**14*y
        +729*r**16*y-999336*y**2-12845136*r**2*y**2-30424464*r**4*y**2-3305904*r**6*y**2
        +24313800*r**8*y**2+3610656*r**10*y**2-421344*r**12*y**2+46944*r**14*y**2
        -2916*r**16*y**2+2402344*y**3+31705744*r**2*y**3+79336656*r**4*y**3+20495216*r**6*y**3
        -51389800*r**8*y**3-8424864*r**10*y**3+983136*r**12*y**3-109536*r**14*y**3
        +6804*r**16*y**3-3726996*y**4-50592696*r**2*y**4-133610904*r**4*y**4-53110344*r**6*y**4
        +67735500*r**8*y**4+12637296*r**10*y**4-1474704*r**12*y**4+164304*r**14*y**4
        -10206*r**16*y**4+3875172*y**5+54233592*r**2*y**5+151138008*r**4*y**5+79951368*r**6*y**5
        -56516460*r**8*y**5-12637296*r**10*y**5+1474704*r**12*y**5-164304*r**14*y**5
        +10206*r**16*y**5-2706928*y**6-39189808*r**2*y**6-115364592*r**4*y**6-75668432*r**6*y**6
        +28328440*r**8*y**6+8424864*r**10*y**6-983136*r**12*y**6+109536*r**14*y**6
        -6804*r**16*y**6+1230384*y**7+18529104*r**2*y**7+57795948*r**4*y**7+45182508*r**6*y**7
        -6746880*r**8*y**7-3662568*r**10*y**7+448812*r**12*y**7-52884*r**14*y**7
        +2916*r**16*y**7-331866*y**8-5281596*r**2*y**8-17616522*r**4*y**8-15972852*r**6*y**8
        -482580*r**8*y**8+1036812*r**10*y**8-151998*r**12*y**8+16596*r**14*y**8-729*r**16*y**8
        +35374*y**9+732094*r**2*y**9+2729223*r**4*y**9+2646623*r**6*y**9+706370*r**8*y**9
        -216468*r**10*y**9+35287*r**12*y**9-2969*r**14*y**9+81*r**16*y**9+13656*y**10
        -21294*r**2*y**10-150129*r**4*y**10+58401*r**6*y**10-169470*r**8*y**10+37548*r**10*y**10
        -4557*r**12*y**10+225*r**14*y**10-15624*y**11+11046*r**2*y**11+30786*r**4*y**11
        -68334*r**6*y**11+21630*r**8*y**11-3612*r**10*y**11+168*r**12*y**11+9212*y**12
        -10318*r**2*y**12-4284*r**4*y**12+4046*r**6*y**12-1750*r**8*y**12-1428*y**13
        +3402*r**2*y**13-2709*r**4*y**13-189*r**6*y**13-1728*y**14+102*r**2*y**14
        +231*r**4*y**14+21*r**6*y**14+1168*y**15-302*r**2*y**15-267*y**16+18*r**2*y**16
        +15*y**17-35*MBhat**8*(-1+y)**4*(-54*r**8*(-1+y)**5+2*(-1+y)**4*(13
        -25*y+6*y**2-y**3+y**4)-6*r**2*(-1+y)**2*(12-36*y+36*y**2-15*y**3
        -y**4+2*y**5)+r**6*(-152+760*y-1520*y**2+1730*y**3-1090*y**4+353*y**5
        -45*y**6)+3*r**4*(48-240*y+480*y**2-530*y**3+340*y**4-113*y**5+15*y**6))
        +70*MBhat**7*(-1+y)**3*(147*r**10*(-1+y)**6-(-1+y)**5*(-305+473*y
        -84*y**2-28*y**3-11*y**4+3*y**5)-3*r**2*(-1+y)**3*(143-393*y+339*y**2
        -109*y**3-2*y**4+6*y**5)-3*r**4*(-1+y)**2*(172-700*y+1074*y**2-900*y**3
        +257*y**4+76*y**5-59*y**6+8*y**7)+3*r**8*(-185+1110*y-2775*y**2+3940*y**3
        -3270*y**4+1560*y**5-386*y**6+36*y**7)+r**6*(800-4800*y+12000*y**2
        -17104*y**3+14529*y**4-6870*y**5+1460*y**6+30*y**7-45*y**8))+14*MBhat**6*(
        -1+y)**2*(1440*r**12*(-1+y)**7+(-1+y)**6*(-9998+14198*y-2100*y**2
        -925*y**3-335*y**4-6*y**5+6*y**6)-5*r**2*(-1+y)**4*(-1946+4422*y
        -2298*y**2-497*y**3+633*y**4-177*y**5+31*y**6)+30*r**4*(-1+y)**3*(132
        -528*y+792*y**2-717*y**3+9*y**4+290*y**5-146*y**6+21*y**7)-10*r**6*(
        -1+y)**2*(-1124+5620*y-11240*y**2+12905*y**3-6478*y**4+242*y**5+1185*y**6
        -447*y**7+45*y**8)+9*r**10*(742-5194*y+15582*y**2-26915*y**3+28385*y**4
        -18501*y**5+7147*y**6-1441*y**7+105*y**8)-5*r**8*(2450-17150*y+51450*y**2
        -89845*y**3+96415*y**4-62475*y**5+22505*y**6-2963*y**7-549*y**8+162*y**9))
        +35*MBhat**5*(486*r**14*(-1+y)**9+(-1+y)**8*(12516-16884*y+2184*y**2
        +872*y**3+523*y**4+117*y**5)+3*r**2*(-1+y)**6*(-902-2310*y+9834*y**2
        -8294*y**3+1801*y**4+85*y**5+7*y**6+3*y**7)-6*r**4*(-1+y)**5*(-1584
        +5904*y-7992*y**2+4144*y**3-1891*y**4+1092*y**5-393*y**6+48*y**7)
        +2*r**6*(-1+y)**4*(-1309+6509*y-12928*y**2+16648*y**3-1469*y**4-8267*y**5
        +6650*y**6-2122*y**7+248*y**8)-3*r**8*(-1+y)**3*(1892-11352*y+28380*y**2
        -42400*y**3+30365*y**4-7918*y**5-3739*y**6+3392*y**7-892*y**8+72*y**9)
        -3*r**10*(-1+y)**2*(1818-12726*y+38178*y**2-67342*y**3+70399*y**4
        -44121*y**5+14903*y**6-1457*y**7-546*y**8+126*y**9)+4*r**12*(644-5796*y
        +23184*y**2-54936*y**3+84546*y**4-87570*y**5+61404*y**6-28476*y**7
        +8199*y**8-1271*y**9+72*y**10))+2*MBhat**2*(630*r**16*(-1+y)**9-(-1
        +y)**8*(256316-320996*y+32340*y**2+10780*y**3+5390*y**4+5061*y**5
        +4235*y**6+994*y**7)-42*r**2*(-1+y)**6*(32793-119319*y+150729*y**2
        -71183*y**3+5235*y**4+1068*y**5+596*y**6+214*y**7+7*y**8)-70*r**6*(
        -1+y)**4*(-23359+102815*y-170680*y**2+124080*y**3-27090*y**4-6231*y**5
        +2975*y**6-550*y**7+92*y**8)-21*r**4*(-1+y)**5*(-9102+130368*y-383472*y**2
        +443568*y**3-204852*y**4+20829*y**5+5084*y**6-86*y**7-36*y**8+9*y**9)
        +105*r**8*(-1+y)**3*(-1810+10860*y-27150*y**2+36200*y**3-27150*y**4
        +7941*y**5-1166*y**6+467*y**7-166*y**8+19*y**9)-21*r**10*(-1+y)**2*(
        -546+3822*y-11466*y**2+19110*y**3-19110*y**4+26544*y**5-14524*y**6
        +3556*y**7-36*y**8-135*y**9+15*y**10)+3*r**14*(1714-15426*y+61704*y**2
        -143976*y**3+215964*y**4-228438*y**5+165522*y**6-80388*y**7+24822*y**8
        -4333*y**9+315*y**10)-7*r**12*(2342-21078*y+84312*y**2-196728*y**3
        +295092*y**4-319473*y**5+250677*y**6-132048*y**7+45162*y**8-9068*y**9
        +810*y**10))+MBhat*(420*r**16*(-1+y)**9-42*r**2*(-1+y)**6*(-35952+126516*y
        -154506*y**2+70162*y**3-4665*y**4-933*y**5-342*y**6-286*y**7-75*y**8
        +y**9)+140*r**6*(-1+y)**4*(-13817+51025*y-56900*y**2-3300*y**3+46800*y**4
        -27420*y**5+4105*y**6+283*y**7-80*y**8+12*y**9)-(-1+y)**8*(-178052
        +220052*y-21000*y**2-7000*y**3-3500*y**4-2100*y**5-3017*y**6-1879*y**7
        -162*y**8+18*y**9)+105*r**8*(-1+y)**3*(13280-76080*y+179400*y**2-221200*y**3
        +147900*y**4-48360*y**5+7149*y**6-566*y**7-27*y**8+24*y**9)-21*r**4*(
        -1+y)**5*(74680-425920*y+893280*y**2-849920*y**3+339680*y**4-25440*y**5
        -6209*y**6-1724*y**7+136*y**8-4*y**9+y**10)-42*r**10*(-1+y)**2*(-2144
        +14588*y-42294*y**2+67550*y**3-63875*y**4+35385*y**5-17164*y**6+5576*y**7
        -901*y**8+35*y**9+4*y**10)+12*r**14*(385-3465*y+13860*y**2-32340*y**3
        +48510*y**4-48510*y**5+35112*y**6-17208*y**7+5382*y**8-958*y**9+72*y**10)
        +7*r**12*(-3112+28008*y-112032*y**2+261408*y**3-392112*y**4+392112*y**5
        -283269*y**6+154221*y**7-56574*y**8+12766*y**9-1461*y**10+45*y**11))
        +MBhat**3*(2835*r**16*(-1+y)**9+105*r**4*(-1+y)**5*(14826-33024*y-3024*y**2
        +54576*y**3-39177*y**4+7292*y**5+292*y**6-108*y**7+27*y**8)-70*r**6*(
        -1+y)**4*(24427-115115*y+212680*y**2-189280*y**3+72500*y**4-14440*y**5
        +3265*y**6-725*y**7+40*y**8)+7*r**2*(-1+y)**6*(338070-1313610*y+1762710*y**2
        -886970*y**3+74760*y**4+20040*y**5+7817*y**6+549*y**7-9*y**8+3*y**9)
        +(-1+y)**8*(815073-1038513*y+111720*y**2+37240*y**3+22715*y**4+20013*y**5
        +7707*y**6+525*y**7-3*y**8+3*y**9)+35*r**8*(-1+y)**3*(-2230+12660*y
        -29490*y**2+35720*y**3-5145*y**4+13062*y**5-12860*y**6+6162*y**7-1539*y**8
        +160*y**9)-21*r**10*(-1+y)**2*(-1662+11634*y-34902*y**2+58170*y**3
        -90300*y**4+54308*y**5-12971*y**6-4227*y**7+3767*y**8-929*y**9+72*y**10)
        +10*r**14*(1813-16317*y+65268*y**2-152292*y**3+236754*y**4-248850*y**5
        +177996*y**6-84924*y**7+25551*y**8-4279*y**9+288*y**10)-7*r**12*(6170
        -55530*y+222120*y**2-518280*y**3+830655*y**4-912135*y**5+668472*y**6
        -323928*y**7+96237*y**8-13853*y**9-138*y**10+210*y**11))+MBhat**4*(5250*r**16*(
        -1+y)**9-210*r**4*(-1+y)**5*(6714-22656*y+25584*y**2-8588*y**3-698*y**4
        +703*y**5-152*y**6+38*y**7)-42*r**2*(-1+y)**6*(19360-91680*y+142080*y**2
        -81060*y**3+9200*y**4+2547*y**5+259*y**6-9*y**7+3*y**8)-2*(-1+y)**8*(386325
        -503925*y+58800*y**2+20650*y**3+14630*y**4+7749*y**5+1071*y**6-6*y**7
        +6*y**8)-350*r**6*(-1+y)**4*(-496+2480*y-4960*y**2+5242*y**3-398*y**4
        -887*y**5+395*y**6-76*y**7+8*y**8)+210*r**8*(-1+y)**3*(10-60*y+150*y**2
        -1350*y**3-3670*y**4+5073*y**5-3632*y**6+1523*y**7-354*y**8+35*y**9)
        +45*r**14*(608-5472*y+21888*y**2-51996*y**3+79884*y**4-82530*y**5
        +57666*y**6-26604*y**7+7596*y**8-1159*y**9+63*y**10)-42*r**10*(-1+y)**2*(
        -800+5600*y-16800*y**2+35000*y**3-23680*y**4+161*y**5+14553*y**6
        -12623*y**7+5093*y**8-994*y**9+70*y**10)-7*r**12*(7340-66060*y+264240*y**2
        -641760*y**3+989940*y**4-996282*y**5+653478*y**6-256572*y**7+40518*y**8
        +9973*y**9-5535*y**10+720*y**11)))/(630*(-1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm4(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m4 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (19*r**8*(-1+y)**5+12*logy*(4+r**4)*(-1+y)**5+(-1+y)**4*(67-115*y
        +24*y**2+7*y**4+y**5)+8*r**2*(-1+y)**2*(-7+21*y-21*y**2+6*y**3-3*y**4
        +2*y**5)+4*r**6*(14-70*y+140*y**2-162*y**3+95*y**4-28*y**5+3*y**6)
        -2*r**4*(24-114*y+213*y**2-245*y**3+169*y**4-53*y**5+6*y**6))/(24*(
        -1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-3411+9125*r**2-7200*r**4+2160*r**6-845*r**8+171*r**10+60*logy*(24
        -4*r**2-51*r**4+9*r**6+6*MBhat**2*(4+r**4)+4*MBhat*(-12-r**2+3*r**4))*(
        -1+y)**6+21906*y-54990*r**2*y+40140*r**4*y-12420*r**6*y+5070*r**8*y
        -1026*r**10*y-59085*y**2+138195*r**2*y**2-91170*r**4*y**2+29430*r**6*y**2
        -12675*r**8*y**2+2565*r**10*y**2+85980*y**3-185460*r**2*y**3+106260*r**4*y**3
        -36540*r**6*y**3+16900*r**8*y**3-3420*r**10*y**3-71400*y**4+140370*r**2*y**4
        -67950*r**4*y**4+30450*r**6*y**4-15225*r**8*y**4+2565*r**10*y**4+31512*y**5
        -56280*r**2*y**5+24900*r**4*y**5-17820*r**6*y**5+7230*r**8*y**5-1026*r**10*y**5
        -3792*y**6+7230*r**2*y**6-5550*r**4*y**6+5610*r**6*y**6-1835*r**8*y**6+171*r**10*y**6
        -3240*y**7+3300*r**2*y**7+540*r**4*y**7-900*r**6*y**7+180*r**8*y**7+1995*y**8
        -1815*r**2*y**8+30*r**4*y**8+30*r**6*y**8-450*y**9+310*r**2*y**9-51*y**10
        +15*r**2*y**10+36*y**11+30*MBhat**2*(-1+y)*(19*r**8*(-1+y)**5+(-1+y)**4*(67
        -115*y+24*y**2+7*y**4+y**5)+8*r**2*(-1+y)**2*(-7+21*y-21*y**2+6*y**3
        -3*y**4+2*y**5)+4*r**6*(14-70*y+140*y**2-162*y**3+95*y**4-28*y**5
        +3*y**6)-2*r**4*(24-114*y+213*y**2-245*y**3+169*y**4-53*y**5+6*y**6))
        +2*MBhat*(462*r**10*(-1+y)**6+10*r**2*(-1+y)**3*(359-1065*y+1047*y**2
        -331*y**3+72*y**4-36*y**5+2*y**6)-(-1+y)**5*(2598-4038*y+720*y**2
        +90*y**3+80*y**4+67*y**5+3*y**6)-30*r**4*(-1+y)**2*(16-76*y+138*y**2
        -220*y**3+89*y**4+3*y**5-11*y**6+y**7)+5*r**8*(-398+2388*y-5970*y**2
        +8470*y**3-6840*y**4+3135*y**5-722*y**6+57*y**7)-10*r**6*(-300+1800*y
        -4500*y**2+6546*y**3-5689*y**4+2790*y**5-696*y**6+40*y**7+9*y**8)))/(720*(
        -1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (2146-9932*r**2-6170*r**4+15280*r**6-1570*r**8+284*r**10-38*r**12
        +60*logy*(12-4*r**2-250*r**4-72*r**6+6*r**8+3*MBhat**4*(4+r**4)+4*MBhat**3*(
        -12-r**2+3*r**4)-4*MBhat*(12+5*r**2-53*r**4+2*r**6)+MBhat**2*(72
        +28*r**2-69*r**4+19*r**6))*(-1+y)**7-15742*y+69764*r**2*y+58190*r**4*y
        -102640*r**6*y+10630*r**8*y-1988*r**10*y+266*r**12*y+49746*y**2-210132*r**2*y**2
        -227070*r**4*y**2+292800*r**6*y**2-30630*r**8*y**2+5964*r**10*y**2-798*r**12*y**2
        -87950*y**3+351900*r**2*y**3+483450*r**4*y**3-457760*r**6*y**3+48530*r**8*y**3
        -9940*r**10*y**3+1330*r**12*y**3+94250*y**4-354000*r**2*y**4-614700*r**4*y**4
        +419960*r**6*y**4-45380*r**8*y**4+9940*r**10*y**4-1330*r**12*y**4-61728*y**5
        +214134*r**2*y**5+475800*r**4*y**5-226620*r**6*y**5+29010*r**8*y**5-7278*r**10*y**5
        +798*r**12*y**5+23877*y**6-72858*r**2*y**6-216630*r**4*y**6+70720*r**6*y**6
        -13765*r**8*y**6+2870*r**10*y**6-266*r**12*y**6-6075*y**7+12774*r**2*y**7
        +51570*r**4*y**7-13480*r**6*y**7+3715*r**8*y**7-626*r**10*y**7+38*r**12*y**7
        +2811*y**8-3390*r**2*y**8-4110*r**4*y**8+2040*r**6*y**8-555*r**8*y**8+54*r**10*y**8
        -1945*y**9+2510*r**2*y**9-370*r**4*y**9-300*r**6*y**9+15*r**8*y**9+637*y**10
        -842*r**2*y**10+40*r**4*y**10+45*y**11+54*r**2*y**11-91*y**12+18*r**2*y**12
        +19*y**13+15*MBhat**4*(-1+y)**2*(19*r**8*(-1+y)**5+(-1+y)**4*(67-115*y
        +24*y**2+7*y**4+y**5)+8*r**2*(-1+y)**2*(-7+21*y-21*y**2+6*y**3-3*y**4
        +2*y**5)+4*r**6*(14-70*y+140*y**2-162*y**3+95*y**4-28*y**5+3*y**6)
        -2*r**4*(24-114*y+213*y**2-245*y**3+169*y**4-53*y**5+6*y**6))-2*MBhat**3*(
        -1+y)*(-462*r**10*(-1+y)**6-10*r**2*(-1+y)**3*(359-1065*y+1047*y**2
        -331*y**3+72*y**4-36*y**5+2*y**6)+(-1+y)**5*(2598-4038*y+720*y**2
        +90*y**3+80*y**4+67*y**5+3*y**6)+30*r**4*(-1+y)**2*(16-76*y+138*y**2
        -220*y**3+89*y**4+3*y**5-11*y**6+y**7)-5*r**8*(-398+2388*y-5970*y**2
        +8470*y**3-6840*y**4+3135*y**5-722*y**6+57*y**7)+10*r**6*(-300+1800*y
        -4500*y**2+6546*y**3-5689*y**4+2790*y**5-696*y**6+40*y**7+9*y**8))
        +4*MBhat*(55*r**12*(-1+y)**7-(-1+y)**6*(1922-2642*y+360*y**2+120*y**3
        -40*y**4+10*y**5+27*y**6+3*y**7)-5*r**2*(-1+y)**4*(-1045+3075*y-2985*y**2
        +935*y**3+26*y**4-78*y**5+20*y**6+4*y**7)+5*r**4*(-1+y)**3*(577-1672*y
        +1236*y**2+448*y**3-442*y**4+117*y**5-23*y**6+11*y**7)-5*r**6*(-1
        +y)**2*(-12+36*y-12*y**2-68*y**3-690*y**4+447*y**5-135*y**6+y**7
        +y**8)+r**10*(347-2429*y+7287*y**2-12145*y**3+13240*y**4-8715*y**5
        +3437*y**6-719*y**7+57*y**8)-5*r**8*(154-1078*y+3234*y**2-5390*y**3
        +6096*y**4-4639*y**5+2137*y**6-581*y**7+67*y**8))+MBhat**2*(680*r**12*(
        -1+y)**7+(-1+y)**6*(9765-14085*y+2160*y**2+500*y**3-105*y**4+261*y**5
        +63*y**6+y**7)+5*r**2*(-1+y)**4*(-3611+10497*y-9993*y**2+3063*y**3
        -327*y**4+45*y**5+35*y**6+3*y**7)+30*r**4*(-1+y)**3*(-260+902*y-1077*y**2
        +286*y**3-150*y**4+156*y**5-85*y**6+12*y**7)-10*r**6*(-1+y)**2*(-464
        +2434*y-5153*y**2+6921*y**3-3688*y**4+622*y**5+385*y**6-157*y**7
        +12*y**8)+3*r**10*(1165-8155*y+24465*y**2-42235*y**3+44065*y**4-28287*y**5
        +10661*y**6-2051*y**7+132*y**8)-5*r**8*(1393-9751*y+29253*y**2-51447*y**3
        +55299*y**4-36165*y**5+13790*y**6-2354*y**7-75*y**8+57*y**9)))/(360*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (420*logy*(6*MBhat**6*(4+r**4)+12*MBhat**5*(-12-r**2+3*r**4)+3*MBhat**4*(120
        +60*r**2-87*r**4+29*r**6)-4*MBhat**3*(120+114*r**2-303*r**4+55*r**6)
        -12*MBhat*(12+9*r**2-342*r**4-185*r**6+10*r**8)+6*MBhat**2*(60+68*r**2
        -552*r**4-49*r**6+19*r**8)+3*(8-4*r**2-595*r**4-795*r**6-95*r**8
        +5*r**10))*(-1+y)**8+210*MBhat**6*(-1+y)**3*(19*r**8*(-1+y)**5+(-1
        +y)**4*(67-115*y+24*y**2+7*y**4+y**5)+8*r**2*(-1+y)**2*(-7+21*y
        -21*y**2+6*y**3-3*y**4+2*y**5)+4*r**6*(14-70*y+140*y**2-162*y**3
        +95*y**4-28*y**5+3*y**6)-2*r**4*(24-114*y+213*y**2-245*y**3+169*y**4
        -53*y**5+6*y**6))-42*MBhat**5*(-1+y)**2*(-462*r**10*(-1+y)**6-10*r**2*(
        -1+y)**3*(359-1065*y+1047*y**2-331*y**3+72*y**4-36*y**5+2*y**6)
        +(-1+y)**5*(2598-4038*y+720*y**2+90*y**3+80*y**4+67*y**5+3*y**6)
        +30*r**4*(-1+y)**2*(16-76*y+138*y**2-220*y**3+89*y**4+3*y**5-11*y**6
        +y**7)-5*r**8*(-398+2388*y-5970*y**2+8470*y**3-6840*y**4+3135*y**5
        -722*y**6+57*y**7)+10*r**6*(-300+1800*y-4500*y**2+6546*y**3-5689*y**4
        +2790*y**5-696*y**6+40*y**7+9*y**8))+21*MBhat**4*(-1+y)*(1360*r**12*(
        -1+y)**7+(-1+y)**6*(16119-23319*y+3600*y**2+760*y**3-45*y**4+393*y**5
        +90*y**6+2*y**7)+5*r**2*(-1+y)**4*(-5397+15471*y-14391*y**2+4213*y**3
        -627*y**4+249*y**5-y**6+3*y**7)+30*r**4*(-1+y)**3*(-280+946*y-1071*y**2
        +54*y**3-154*y**4+292*y**5-171*y**6+24*y**7)-10*r**6*(-1+y)**2*(-1144
        +5894*y-12223*y**2+15579*y**3-8684*y**4+1718*y**5+683*y**6-311*y**7
        +24*y**8)+3*r**10*(2387-16709*y+50127*y**2-86465*y**3+90125*y**4-57771*y**5
        +21721*y**6-4159*y**7+264*y**8)-5*r**8*(2955-20685*y+62055*y**2-108809*y**3
        +117023*y**4-76821*y**5+29393*y**6-5111*y**7-114*y**8+114*y**9))+6*MBhat*(286*r**14*(
        -1+y)**8+70*r**6*(-1+y)**3*(-4995+22755*y-39960*y**2+32560*y**3-10730*y**4
        +988*y**5+95*y**6+41*y**7+14*y**8)-(-1+y)**7*(32516-42596*y+5040*y**2
        +1680*y**3+840*y**4-819*y**5-350*y**6+272*y**7+57*y**8)+21*r**4*(-1
        +y)**4*(-11316+58944*y-115776*y**2+104544*y**3-39816*y**4+3731*y**5
        +826*y**6-84*y**7+26*y**8+y**9)-35*r**8*(-1+y)**2*(300-1560*y+3180*y**2
        -3040*y**3+1080*y**4-2577*y**5+1365*y**6-462*y**7+33*y**8+y**9)-7*r**2*(
        -1+y)**5*(-19894+58602*y-56982*y**2+17914*y**3+270*y**4+300*y**5
        -783*y**6+9*y**7+81*y**8+3*y**9)+7*r**12*(-356+2848*y-9968*y**2+19936*y**3
        -24920*y**4+22015*y**5-12152*y**6+4156*y**7-776*y**8+57*y**9)+21*r**10*(354
        -2832*y+9912*y**2-19824*y**3+24780*y**4-22766*y**5+15287*y**6-6388*y**7
        +1672*y**8-198*y**9+3*y**10))+3*(95*r**14*(-1+y)**8+35*r**4*(-1+y)**4*(9803
        -46352*y+83808*y**2-70152*y**3+24678*y**4-1428*y**5-520*y**6-92*y**7
        +15*y**8)-35*r**8*(-1+y)**2*(4895-28230*y+67155*y**2-83840*y**3+57180*y**4
        -19452*y**5+3397*y**6-390*y**7+85*y**8)+(-1+y)**7*(11547-14907*y
        +1680*y**2+560*y**3+280*y**4+168*y**5-511*y**6-25*y**7+88*y**8)-35*r**6*(
        -1+y)**3*(-7085+25885*y-27920*y**2-3880*y**3+25790*y**4-14698*y**5
        +2564*y**6-60*y**7+43*y**8+y**9)+7*r**2*(-1+y)**5*(-11243+33969*y
        -34329*y**2+11683*y**3-60*y**4-12*y**5+97*y**6-339*y**7+55*y**8+19*y**9)
        +7*r**12*(-143+1144*y-4004*y**2+8008*y**3-10010*y**4+8008*y**5-4928*y**6
        +1656*y**7-315*y**8+24*y**9)+7*r**10*(1421-11068*y+37538*y**2-72276*y**3
        +86145*y**4-64716*y**5+33509*y**6-13180*y**7+3026*y**8-408*y**9+9*y**10))
        +3*MBhat**2*(1870*r**14*(-1+y)**8-420*r**4*(-1+y)**4*(-83+1436*y-4362*y**2
        +5116*y**3-2238*y**4+390*y**5-39*y**6+20*y**7)+(-1+y)**7*(149374
        -199774*y+25200*y**2+8400*y**3+315*y**4-2415*y**5+1351*y**6+713*y**7
        +36*y**8)+70*r**6*(-1+y)**3*(4761-23217*y+44964*y**2-43004*y**3+16717*y**4
        -3353*y**5+462*y**6-354*y**7+48*y**8)-7*r**2*(-1+y)**5*(59908-171564*y
        +159324*y**2-44948*y**3-2985*y**4+2511*y**5+696*y**6-504*y**7-39*y**8
        +y**9)-35*r**8*(-1+y)**2*(-98+132*y+1038*y**2-3664*y**3+14253*y**4
        -9666*y**5+3834*y**6-198*y**7-123*y**8+12*y**9)+14*r**12*(-934+7472*y
        -26152*y**2+52304*y**3-68845*y**4+58352*y**5-31948*y**6+10712*y**7
        -1933*y**8+132*y**9)-21*r**10*(-1724+13792*y-48272*y**2+96544*y**3
        -130345*y**4+118140*y**5-69051*y**6+25824*y**7-5331*y**8+404*y**9+19*y**10))
        +2*MBhat**3*(6570*r**14*(-1+y)**8-210*r**4*(-1+y)**4*(-1261+3832*y-3324*y**2
        -373*y**3+181*y**4+324*y**5-236*y**6+17*y**7)-21*r**2*(-1+y)**5*(-25936
        +73248*y-66408*y**2+18016*y**3-45*y**4-905*y**5+385*y**6+45*y**7)
        -(-1+y)**7*(265604-366404*y+50400*y**2+14490*y**3-3255*y**4+2373*y**5
        +2842*y**6+347*y**7+3*y**8)+70*r**6*(-1+y)**3*(-813+4725*y-11100*y**2
        +15940*y**3-3953*y**4-2855*y**5+3350*y**6-982*y**7+104*y**8)-105*r**8*(
        -1+y)**2*(892-5352*y+13380*y**2-20590*y**3+13977*y**4-4216*y**5-1203*y**6
        +1179*y**7-286*y**8+19*y**9)+14*r**12*(-2671+21368*y-74788*y**2+153041*y**3
        -196630*y**4+163730*y**5-87220*y**6+27980*y**7-4645*y**8+255*y**9)
        -21*r**10*(-3896+31168*y-109088*y**2+227416*y**3-296675*y**4+247528*y**5
        -130704*y**6+38856*y**7-3939*y**8-864*y**9+198*y**10)))/(5040*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (38331-354512*r**2-3746316*r**4-1189104*r**6+4303600*r**8+984144*r**10
        -38388*r**12+2416*r**14-171*r**16+840*logy*(12-8*r**2-2247*r**4-6408*r**6
        -3625*r**8-240*r**10+9*r**12+3*MBhat**8*(4+r**4)+8*MBhat**7*(-12-r**2
        +3*r**4)+2*MBhat**6*(168+92*r**2-105*r**4+39*r**6)-8*MBhat**5*(84+99*r**2
        -144*r**4+49*r**6)+5*MBhat**4*(168+296*r**2-924*r**4+92*r**6+45*r**8)
        -8*MBhat**3*(84+175*r**2-1359*r**4-455*r**6+110*r**8)-8*MBhat*(12
        +13*r**2-1110*r**4-2004*r**6-430*r**8+21*r**10)+6*MBhat**2*(56+108*r**2
        -2309*r**4-2235*r**6+30*r**8+21*r**10))*(-1+y)**9-355059*y+3197328*r**2*y
        +35604324*r**4*y+16084656*r**6*y-35687400*r**8*y-8655696*r**10*y+337932*r**12*y
        -21744*r**14*y+1539*r**16*y+1465596*y**2-12819552*r**2*y**2-150910956*r**4*y**2
        -88560864*r**6*y**2+129047100*r**8*y**2+33715584*r**10*y**2-1317708*r**12*y**2
        +86976*r**14*y**2-6156*r**16*y**2-3540684*y**3+29992928*r**2*y**3+374775324*r**4*y**3
        +271234656*r**6*y**3-264569900*r**8*y**3-76250496*r**10*y**3+2983932*r**12*y**3
        -202944*r**14*y**3+14364*r**16*y**3+5522706*y**4-45130512*r**2*y**4-601800066*r**4*y**4
        -519889104*r**6*y**4+332909850*r**8*y**4+110142144*r**10*y**4-4317138*r**12*y**4
        +304416*r**14*y**4-21546*r**16*y**4-5776722*y**5+45299856*r**2*y**5+649364562*r**4*y**5
        +655533648*r**6*y**5-256175850*r**8*y**5-105061824*r**10*y**5+4126626*r**12*y**5
        -304416*r**14*y**5+21546*r**16*y**5+4062828*y**6-30341024*r**2*y**6-472546788*r**4*y**6
        -550059552*r**6*y**6+106838900*r**8*y**6+65807616*r**10*y**6-2592324*r**12*y**6
        +202944*r**14*y**6-14364*r**16*y**6-1863996*y**7+13087488*r**2*y**7+225200052*r**4*y**7
        +300191808*r**6*y**7-8995140*r**8*y**7-26016480*r**10*y**7+1129044*r**12*y**7
        -107616*r**14*y**7+6156*r**16*y**7+524964*y**8-3330672*r**2*y**8-64926708*r**4*y**8
        -98719992*r**6*y**8-12229140*r**8*y**8+6095040*r**10*y**8-377706*r**12*y**8
        +31464*r**14*y**8-1539*r**16*y**8-113556*y**9+476288*r**2*y**9+9328452*r**4*y**9
        +15490608*r**6*y**9+5309500*r**8*y**9-840000*r**10*y**9+74634*r**12*y**9
        -5296*r**14*y**9+171*r**16*y**9+87252*y**10-197232*r**2*y**10-352296*r**4*y**10
        +128856*r**6*y**10-802200*r**8*y**10+93744*r**10*y**10-9072*r**12*y**10+360*r**14*y**10
        -93828*y**11+222768*r**2*y**11-33936*r**4*y**11-239904*r**6*y**11+63000*r**8*y**11
        -13776*r**10*y**11+168*r**12*y**11+56070*y**12-148736*r**2*y**12+77154*r**4*y**12
        -5544*r**6*y**12-12110*r**8*y**12-9030*y**13+51744*r**2*y**13-36666*r**4*y**13
        +336*r**6*y**13-210*r**8*y**13-11100*y**14-4176*r**2*y**14+3864*r**4*y**14
        -504*r**6*y**14+8364*y**15-2512*r**2*y**15-2391*y**16+528*r**2*y**16+255*y**17
        +210*MBhat**8*(-1+y)**4*(19*r**8*(-1+y)**5+(-1+y)**4*(67-115*y+24*y**2
        +7*y**4+y**5)+8*r**2*(-1+y)**2*(-7+21*y-21*y**2+6*y**3-3*y**4+2*y**5)
        +4*r**6*(14-70*y+140*y**2-162*y**3+95*y**4-28*y**5+3*y**6)-2*r**4*(24
        -114*y+213*y**2-245*y**3+169*y**4-53*y**5+6*y**6))-56*MBhat**7*(-1
        +y)**3*(-462*r**10*(-1+y)**6-10*r**2*(-1+y)**3*(359-1065*y+1047*y**2
        -331*y**3+72*y**4-36*y**5+2*y**6)+(-1+y)**5*(2598-4038*y+720*y**2
        +90*y**3+80*y**4+67*y**5+3*y**6)+30*r**4*(-1+y)**2*(16-76*y+138*y**2
        -220*y**3+89*y**4+3*y**5-11*y**6+y**7)-5*r**8*(-398+2388*y-5970*y**2
        +8470*y**3-6840*y**4+3135*y**5-722*y**6+57*y**7)+10*r**6*(-300+1800*y
        -4500*y**2+6546*y**3-5689*y**4+2790*y**5-696*y**6+40*y**7+9*y**8))
        +28*MBhat**6*(-1+y)**2*(2040*r**12*(-1+y)**7+3*(-1+y)**6*(7491-10851*y
        +1680*y**2+340*y**3+5*y**4+175*y**5+39*y**6+y**7)+5*r**2*(-1+y)**4*(
        -7183+20445*y-18789*y**2+5363*y**3-927*y**4+453*y**5-37*y**6+3*y**7)
        +30*r**4*(-1+y)**3*(-300+990*y-1065*y**2-178*y**3-158*y**4+428*y**5
        -257*y**6+36*y**7)-30*r**6*(-1+y)**2*(-608+3118*y-6431*y**2+8079*y**3
        -4560*y**4+938*y**5+327*y**6-155*y**7+12*y**8)+9*r**10*(1203-8421*y
        +25263*y**2-43565*y**3+45395*y**4-29085*y**5+10927*y**6-2089*y**7+132*y**8)
        -5*r**8*(4517-31619*y+94857*y**2-166171*y**3+178747*y**4-117477*y**5
        +44996*y**6-7868*y**7-153*y**8+171*y**9))-8*MBhat**5*(-1+y)*(-6570*r**14*(
        -1+y)**8+21*r**2*(-1+y)**5*(-15486+42498*y-36558*y**2+8666*y**3-305*y**4
        -125*y**5+185*y**6+5*y**7)+210*r**4*(-1+y)**4*(-684+2160*y-2088*y**2
        +75*y**3-261*y**4+441*y**5-259*y**6+28*y**7)+(-1+y)**7*(184880-255440*y
        +35280*y**2+9450*y**3-1575*y**4+1953*y**5+1708*y**6+221*y**7+3*y**8)
        -70*r**6*(-1+y)**3*(-849+4833*y-11136*y**2+15736*y**3-6023*y**4-1514*y**5
        +2945*y**6-979*y**7+107*y**8)+105*r**8*(-1+y)**2*(1200-7200*y+18000*y**2
        -26750*y**3+20009*y**4-7462*y**5-175*y**6+1045*y**7-286*y**8+19*y**9)
        -14*r**12*(-2836+22688*y-79408*y**2+162281*y**3-208180*y**4+172970*y**5
        -91840*y**6+29300*y**7-4810*y**8+255*y**9)+21*r**10*(-4590+36720*y
        -128520*y**2+266280*y**3-347445*y**4+291438*y**5-155008*y**6+47168*y**7
        -5491*y**8-750*y**9+198*y**10))+6*MBhat**2*(595*r**16*(-1+y)**9-70*r**6*(
        -1+y)**4*(-35501+150685*y-234320*y**2+144920*y**3-5410*y**4-21826*y**5
        +6028*y**6-268*y**7+267*y**8+y**9)+70*r**8*(-1+y)**3*(-7185+43470*y
        -109755*y**2+148140*y**3-112905*y**4+38982*y**5-6784*y**6+186*y**7
        -265*y**8+36*y**9)+14*r**2*(-1+y)**6*(-46539+133137*y-123417*y**2
        +34659*y**3+1620*y**4+924*y**5-983*y**6-795*y**7+223*y**8+51*y**9)
        +(-1+y)**8*(162621-209661*y+23520*y**2+7840*y**3+3920*y**4-1176*y**5
        -3514*y**6+114*y**7+599*y**8+57*y**9)-14*r**4*(-1+y)**5*(-174453+836352*y
        -1531608*y**2+1298152*y**3-463078*y**4+31584*y**5+10946*y**6+436*y**7
        -9*y**8+76*y**9+2*y**10)-14*r**10*(-1+y)**2*(2989-19663*y+54579*y**2
        -82145*y**3+71120*y**4+156*y**5-14527*y**6+10117*y**7-1485*y**8-33*y**9
        +12*y**10)+6*r**14*(907-8163*y+32652*y**2-76188*y**3+114282*y**4-121506*y**5
        +86436*y**6-41004*y**7+12231*y**8-2019*y**9+132*y**10)-14*r**12*(1411
        -12699*y+50796*y**2-118524*y**3+177786*y**4-193926*y**5+151378*y**6
        -78502*y**7+27333*y**8-5557*y**9+504*y**10))+8*MBhat*(132*r**16*(-1
        +y)**9+35*r**8*(-1+y)**3*(31580-179160*y+416940*y**2-504320*y**3+326640*y**4
        -99696*y**5+11273*y**6+45*y**7+267*y**8+31*y**9)-(-1+y)**8*(36744
        -46824*y+5040*y**2+1680*y**3+840*y**4+504*y**5-903*y**6-668*y**7
        +161*y**8+66*y**9)-21*r**10*(-1+y)**2*(-1564+10108*y-27384*y**2+39760*y**3
        -32410*y**4+13566*y**5-9258*y**6+3325*y**7-1029*y**8+85*y**9+y**10)
        +21*r**4*(-1+y)**5*(-73780+339520*y-598080*y**2+487520*y**3-166280*y**4
        +8880*y**5+2433*y**6+1178*y**7-72*y**8-2*y**9+3*y**10)+14*r**6*(-1
        +y)**4*(-47112+115320*y+69960*y**2-470760*y**3+535980*y**4-227436*y**5
        +26880*y**6+3055*y**7-35*y**8+145*y**9+3*y**10)+4*r**14*(380-3420*y
        +13680*y**2-31920*y**3+47880*y**4-47880*y**5+35532*y**6-16848*y**7
        +5067*y**8-848*y**9+57*y**10)-r**2*(-1+y)**6*(-223796+660468*y-644088*y**2
        +203776*y**3+2730*y**4+546*y**5+2786*y**6-5391*y**7-1173*y**8+725*y**9
        +57*y**10)+7*r**12*(-828+7452*y-29808*y**2+69552*y**3-104328*y**4
        +104328*y**5-80385*y**6+48269*y**7-18246*y**8+4494*y**9-509*y**10+9*y**11))
        +8*MBhat**3*(1095*r**16*(-1+y)**9-70*r**6*(-1+y)**4*(19749-93285*y+172920*y**2
        -154720*y**3+60770*y**4-11206*y**5+997*y**6-737*y**7+40*y**8)+21*r**4*(
        -1+y)**5*(-39082+210688*y-424752*y**2+391888*y**3-150637*y**4+17533*y**5
        +1728*y**6+8*y**7+183*y**8+3*y**9)-(-1+y)**8*(227927-298487*y+35280*y**2
        +11760*y**3+3255*y**4-3990*y**5-896*y**6+1313*y**7+309*y**8+9*y**9)
        +35*r**8*(-1+y)**3*(-4090+21900*y-46830*y**2+49240*y**3-12315*y**4
        +4083*y**5-1356*y**6+2673*y**7-783*y**8+78*y**9)+r**2*(-1+y)**6*(625670
        -1730010*y+1509510*y**2-356170*y**3-44730*y**4+3486*y**5+20349*y**6
        -2586*y**7-1986*y**8-16*y**9+3*y**10)-21*r**10*(-1+y)**2*(-1762+12334*y
        -37002*y**2+61670*y**3-79970*y**4+48950*y**5-18949*y**6+1916*y**7+824*y**8
        -270*y**9+19*y**10)+2*r**14*(4039-36351*y+145404*y**2-339276*y**3
        +526974*y**4-548226*y**5+386904*y**6-181116*y**7+52929*y**8-8431*y**9
        +510*y**10)-7*r**12*(3310-29790*y+119160*y**2-278040*y**3+443595*y**4
        -481722*y**5+352119*y**6-174060*y**7+53865*y**8-8830*y**9+327*y**10
        +66*y**11))+MBhat**4*(17325*r**16*(-1+y)**9+210*r**4*(-1+y)**5*(-1568
        -12208*y+55272*y**2-73936*y**3+30647*y**4-4500*y**5-50*y**6-380*y**7
        +3*y**8)-280*r**6*(-1+y)**4*(-9285+47805*y-99060*y**2+105640*y**3
        -42705*y**4+5343*y**5+3430*y**6-664*y**7+56*y**8)-28*r**2*(-1+y)**6*(154200
        -418200*y+351600*y**2-74360*y**3-10365*y**4+6267*y**5+386*y**6-1110*y**7
        -21*y**8+3*y**9)+(-1+y)**8*(2084525-2790125*y+352800*y**2+110880*y**3
        -1610*y**4-16870*y**5+16912*y**6+7744*y**7+541*y**8+3*y**9)+140*r**8*(
        -1+y)**3*(5560-32010*y+75975*y**2-103910*y**3+41475*y**4+6042*y**5
        -21562*y**6+10104*y**7-2286*y**8+212*y**9)-84*r**10*(-1+y)**2*(-2800
        +19600*y-58800*y**2+114520*y**3-96035*y**4+42105*y**5+7798*y**6-17002*y**7
        +7713*y**8-1467*y**9+88*y**10)+40*r**14*(2633-23697*y+94788*y**2-224784*y**3
        +343728*y**4-352926*y**5+244482*y**6-111348*y**7+31122*y**8-4553*y**9
        +219*y**10)-14*r**12*(17200-154800*y+619200*y**2-1495680*y**3+2304835*y**4
        -2353299*y**5+1603812*y**6-689388*y**7+156537*y**8-2773*y**9-6664*y**10
        +1020*y**11)))/(5040*(-1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm5(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m5 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-195*r**8*(-1+y)**5+12*logy*(16+27*r**4)*(-1+y)**5-24*r**2*(-1+y)**3*(
        -31+62*y-31*y**2-6*y**3+2*y**4)+(-1+y)**4*(397-589*y+96*y**2
        +176*y**3+13*y**4+3*y**5)-4*r**6*(214-1070*y+2140*y**2-2344*y**3
        +1355*y**4-379*y**5+36*y**6)+6*r**4*(168-894*y+1923*y**2-2223*y**3
        +1469*y**4-523*y**5+80*y**6))/(72*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (60*logy*(8*MBhat*(-7+r**2)+MBhat**2*(32+54*r**4)+3*(8+8*r**2-37*r**4
        +27*r**6))*(-1+y)**6+10*MBhat**2*(-1+y)*(-195*r**8*(-1+y)**5-24*r**2*(
        -1+y)**3*(-31+62*y-31*y**2-6*y**3+2*y**4)+(-1+y)**4*(397-589*y
        +96*y**2+176*y**3+13*y**4+3*y**5)-4*r**6*(214-1070*y+2140*y**2-2344*y**3
        +1355*y**4-379*y**5+36*y**6)+6*r**4*(168-894*y+1923*y**2-2223*y**3
        +1469*y**4-523*y**5+80*y**6))-15*(39*r**10*(-1+y)**6-(-1+y)**5*(249
        -345*y+48*y**2+16*y**3+73*y**4+23*y**5)-r**2*(-1+y)**4*(567-1038*y
        +423*y**2+32*y**3+113*y**4+30*y**5+y**6)-2*r**4*(-1+y)**2*(-120+258*y
        +57*y**2-482*y**3+460*y**4-172*y**5+47*y**6)+r**8*(-297+1782*y-4455*y**2
        +5940*y**3-4945*y**4+2322*y**5-555*y**6+48*y**7)+2*r**6*(168-1170*y
        +3411*y**2-5358*y**3+5211*y**4-3202*y**5+1125*y**6-190*y**7+5*y**8))
        -2*MBhat*(1428*r**10*(-1+y)**6+20*r**2*(-1+y)**4*(463-938*y+481*y**2
        +161*y**3-124*y**4+29*y**5)+(-1+y)**5*(3668-5348*y+840*y**2+1510*y**3
        +290*y**4-3*y**5+3*y**6)-30*r**4*(-1+y)**2*(416-1664*y+2496*y**2
        -1700*y**3+310*y**4+169*y**5-80*y**6+5*y**7)+5*r**8*(-1276+7656*y
        -19140*y**2+26990*y**3-21960*y**4+10149*y**5-2374*y**6+195*y**7)-20*r**6*(
        -592+3552*y-8880*y**2+12365*y**3-10125*y**4+4680*y**5-1051*y**6+33*y**7
        +18*y**8)))/(720*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (1222-10480*r**2-11570*r**4+21760*r**6+490*r**8-1552*r**10+130*r**12
        +60*logy*(8-32*r**2-298*r**4-176*r**6+54*r**8+8*MBhat**3*(-7+r**2)
        +MBhat**4*(16+27*r**4)+8*MBhat*(-5+2*r**2+67*r**4)+MBhat**2*(72+8*r**2
        -273*r**4+127*r**6))*(-1+y)**7-9034*y+75280*r**2*y+98870*r**4*y-141760*r**6*y
        -6670*r**8*y+10864*r**10*y-910*r**12*y+28782*y**2-232560*r**2*y**2-359190*r**4*y**2
        +388320*r**6*y**2+31350*r**8*y**2-32592*r**10*y**2+2730*r**12*y**2-51330*y**3
        +401040*r**2*y**3+723810*r**4*y**3-573280*r**6*y**3-74930*r**8*y**3+54320*r**10*y**3
        -4550*r**12*y**3+55530*y**4-417840*r**2*y**4-880260*r**4*y**4+480880*r**6*y**4
        +103280*r**8*y**4-54320*r**10*y**4+4550*r**12*y**4-36174*y**5+263040*r**2*y**5
        +652980*r**4*y**5-209040*r**6*y**5-92880*r**8*y**5+36192*r**10*y**5-2730*r**12*y**5
        +10949*y**6-89520*r**2*y**6-279030*r**4*y**6+19140*r**6*y**6+54655*r**8*y**6
        -14140*r**10*y**6+910*r**12*y**6+4669*y**7+5760*r**2*y**7+55410*r**4*y**7
        +21300*r**6*y**7-18145*r**8*y**7+2884*r**10*y**7-130*r**12*y**7-8889*y**8
        +8160*r**2*y**8+690*r**4*y**8-8580*r**6*y**8+2925*r**8*y**8-216*r**10*y**8
        +5935*y**9-2960*r**2*y**9-2110*r**4*y**9+1260*r**6*y**9-75*r**8*y**9-1663*y**10
        -160*r**2*y**10+400*r**4*y**10-135*y**11+240*r**2*y**11+149*y**12-11*y**13
        +5*MBhat**4*(-1+y)**2*(-195*r**8*(-1+y)**5-24*r**2*(-1+y)**3*(-31+62*y
        -31*y**2-6*y**3+2*y**4)+(-1+y)**4*(397-589*y+96*y**2+176*y**3+13*y**4
        +3*y**5)-4*r**6*(214-1070*y+2140*y**2-2344*y**3+1355*y**4-379*y**5
        +36*y**6)+6*r**4*(168-894*y+1923*y**2-2223*y**3+1469*y**4-523*y**5
        +80*y**6))-2*MBhat**3*(-1+y)*(1428*r**10*(-1+y)**6+20*r**2*(-1+y)**4*(463
        -938*y+481*y**2+161*y**3-124*y**4+29*y**5)+(-1+y)**5*(3668-5348*y
        +840*y**2+1510*y**3+290*y**4-3*y**5+3*y**6)-30*r**4*(-1+y)**2*(416
        -1664*y+2496*y**2-1700*y**3+310*y**4+169*y**5-80*y**6+5*y**7)+5*r**8*(
        -1276+7656*y-19140*y**2+26990*y**3-21960*y**4+10149*y**5-2374*y**6
        +195*y**7)-20*r**6*(-592+3552*y-8880*y**2+12365*y**3-10125*y**4+4680*y**5
        -1051*y**6+33*y**7+18*y**8))-4*MBhat*(170*r**12*(-1+y)**7+20*r**2*(
        -1+y)**5*(313-638*y+331*y**2-4*y**3+56*y**4+2*y**5)+2*(-1+y)**6*(777
        -1077*y+150*y**2+50*y**3+250*y**4+84*y**5+6*y**6)+5*r**4*(-1+y)**3*(350
        -3008*y+7728*y**2-8368*y**3+3652*y**4-57*y**5-184*y**6+31*y**7)-5*r**6*(
        -1+y)**2*(-1840+9200*y-18400*y**2+18400*y**3-10028*y**4+2359*y**5
        -115*y**6-61*y**7+5*y**8)-5*r**8*(746-5222*y+15666*y**2-26110*y**3
        +27448*y**4-18543*y**5+7625*y**6-1793*y**7+183*y**8)+r**10*(1156-8092*y
        +24276*y**2-40460*y**3+43460*y**4-28833*y**5+11431*y**6-2413*y**7+195*y**8))
        +MBhat**2*(-1960*r**12*(-1+y)**7+5*r**2*(-1+y)**5*(5899-11702*y+5755*y**2
        +1472*y**3-481*y**4+194*y**5+15*y**6)+(-1+y)**6*(10305-14625*y+2160*y**2
        +2880*y**3+1755*y**4+405*y**5-y**6+y**7)-30*r**4*(-1+y)**3*(844-2830*y
        +3153*y**2-770*y**3-1248*y**4+1036*y**5-369*y**6+40*y**7)+10*r**6*(
        -1+y)**2*(-1240+5438*y-8971*y**2+7487*y**3+302*y**4-3844*y**5+2565*y**6
        -633*y**7+48*y**8)-3*r**10*(2995-20965*y+62895*y**2-108825*y**3+114555*y**4
        -74445*y**5+28623*y**6-5721*y**7+408*y**8)+5*r**8*(2659-18613*y+55839*y**2
        -98009*y**3+104573*y**4-66747*y**5+23674*y**6-2854*y**7-717*y**8+195*y**9)))/(360*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-4177+442323*r**2+1028475*r**4-606025*r**6-900375*r**8+23877*r**10
        +16877*r**12-975*r**14+420*logy*(8-312*r**2-2385*r**4-3125*r**6-765*r**8
        +135*r**10+24*MBhat**5*(-7+r**2)+MBhat**6*(32+54*r**4)+16*MBhat**3*(
        -25+15*r**2+306*r**4)+3*MBhat**4*(120-8*r**2-435*r**4+173*r**6)+24*MBhat*(
        -3+37*r**2+281*r**4+265*r**6)+6*MBhat**2*(40-136*r**2-1334*r**4-647*r**6
        +115*r**8))*(-1+y)**8+36776*y-3669624*r**2*y-9229500*r**4*y+3535700*r**6*y
        +6881700*r**8*y-134316*r**10*y-135016*r**12*y+7800*r**14*y-142156*y**2
        +13367844*r**2*y**2+36310050*r**4*y**2-7124950*r**6*y**2-22800750*r**8*y**2
        +243306*r**10*y**2+472556*r**12*y**2-27300*r**14*y**2+315672*y**3-27958728*r**2*y**3
        -81969300*r**4*y**3+1999900*r**6*y**3+42602700*r**8*y**3+42588*r**10*y**3
        -945112*r**12*y**3+54600*r**14*y**3-441630*y**4+36782970*r**2*y**4+116485425*r**4*y**4
        +15875125*r**6*y**4-48755175*r**8*y**4-847035*r**10*y**4+1181390*r**12*y**4
        -68250*r**14*y**4+400344*y**5-31260936*r**2*y**5-107212140*r**4*y**5-31075100*r**6*y**5
        +34505940*r**8*y**5+1471428*r**10*y**5-945112*r**12*y**5+54600*r**14*y**5
        -239197*y**6+16875117*r**2*y**6+62939520*r**4*y**6+27730640*r**6*y**6-14090055*r**8*y**6
        -1421469*r**10*y**6+523712*r**12*y**6-27300*r**14*y**6+133472*y**7-5466720*r**2*y**7
        -21916020*r**4*y**7-12994660*r**6*y**7+2364180*r**8*y**7+851844*r**10*y**7
        -174664*r**12*y**7+7800*r**14*y**7-175549*y**8+1108863*r**2*y**8+3603705*r**4*y**8
        +2702525*r**6*y**8+356790*r**8*y**8-270438*r**10*y**8+30905*r**12*y**8-975*r**14*y**8
        +271320*y**9-413784*r**2*y**9+14280*r**4*y**9+28840*r**6*y**9-189000*r**8*y**9
        +41160*r**10*y**9-2016*r**12*y**9-270102*y**10+280518*r**2*y**10-48930*r**4*y**10
        -80570*r**6*y**10+24045*r**8*y**10-945*r**10*y**10+159096*y**11-99288*r**2*y**11
        -9240*r**4*y**11+8680*r**6*y**11-48916*y**12+9156*r**2*y**12+3675*r**4*y**12
        -105*r**6*y**12+3752*y**13+2520*r**2*y**13+1535*y**14-231*r**2*y**14-240*y**15
        +70*MBhat**6*(-1+y)**3*(-195*r**8*(-1+y)**5-24*r**2*(-1+y)**3*(-31+62*y
        -31*y**2-6*y**3+2*y**4)+(-1+y)**4*(397-589*y+96*y**2+176*y**3+13*y**4
        +3*y**5)-4*r**6*(214-1070*y+2140*y**2-2344*y**3+1355*y**4-379*y**5
        +36*y**6)+6*r**4*(168-894*y+1923*y**2-2223*y**3+1469*y**4-523*y**5
        +80*y**6))-42*MBhat**5*(-1+y)**2*(1428*r**10*(-1+y)**6+20*r**2*(-1+y)**4*(463
        -938*y+481*y**2+161*y**3-124*y**4+29*y**5)+(-1+y)**5*(3668-5348*y
        +840*y**2+1510*y**3+290*y**4-3*y**5+3*y**6)-30*r**4*(-1+y)**2*(416
        -1664*y+2496*y**2-1700*y**3+310*y**4+169*y**5-80*y**6+5*y**7)+5*r**8*(
        -1276+7656*y-19140*y**2+26990*y**3-21960*y**4+10149*y**5-2374*y**6
        +195*y**7)-20*r**6*(-592+3552*y-8880*y**2+12365*y**3-10125*y**4+4680*y**5
        -1051*y**6+33*y**7+18*y**8))-21*MBhat**4*(-1+y)*(3920*r**12*(-1+y)**7
        -5*r**2*(-1+y)**5*(10097-20290*y+10241*y**2+2848*y**3-1301*y**4+298*y**5
        +27*y**6)-(-1+y)**6*(16875-24075*y+3600*y**2+5520*y**3+2415*y**4
        +465*y**5-2*y**6+2*y**7)+30*r**4*(-1+y)**3*(1568-5402*y+6363*y**2
        -2022*y**3-2036*y**4+1900*y**5-691*y**6+80*y**7)-10*r**6*(-1+y)**2*(
        -2984+13882*y-25169*y**2+23821*y**3-6182*y**4-4868*y**5+4575*y**6
        -1251*y**7+96*y**8)+3*r**10*(6185-43295*y+129885*y**2-224475*y**3
        +235935*y**4-152985*y**5+58611*y**6-11637*y**7+816*y**8)-5*r**8*(6209
        -43463*y+130389*y**2-227203*y**3+241801*y**4-155295*y**5+55979*y**6
        -7517*y**7-1290*y**8+390*y**9))-21*MBhat**2*(770*r**14*(-1+y)**8+(-1
        +y)**7*(-12082+16882*y-2400*y**2-800*y**3-3475*y**4-2493*y**5-445*y**6
        +13*y**7)+r**2*(-1+y)**6*(-94900+206120*y-119380*y**2+5440*y**3-8345*y**4
        -2990*y**5-344*y**6-2*y**7+y**8)-20*r**4*(-1+y)**4*(4894-27580*y
        +57378*y**2-54260*y**3+20999*y**4-540*y**5-551*y**6+8*y**7+12*y**8)
        +10*r**6*(-1+y)**3*(-19959+92031*y-164652*y**2+138772*y**3-51131*y**4
        -845*y**5+3786*y**6-930*y**7+48*y**8)-5*r**8*(-1+y)**2*(5042-27492*y
        +60450*y**2-66800*y**3+46725*y**4-9606*y**5-2748*y**6+2646*y**7-585*y**8
        +48*y**9)+2*r**12*(-2480+19840*y-69440*y**2+138880*y**3-182735*y**4
        +156520*y**5-86828*y**6+29704*y**7-5549*y**8+408*y**9)+r**10*(10508
        -84064*y+294224*y**2-588448*y**3+789635*y**4-709528*y**5+409151*y**6
        -148960*y**7+29429*y**8-1752*y**9-195*y**10))-6*MBhat*(884*r**14*(-1
        +y)**8+14*r**2*(-1+y)**6*(17018-38476*y+23678*y**2-1480*y**3-370*y**4
        +1109*y**5+192*y**6+9*y**7)+140*r**6*(-1+y)**3*(3295-13295*y+18640*y**2
        -8040*y**3-3930*y**4+4227*y**5-483*y**6-93*y**7+15*y**8)-(-1+y)**7*(
        -11188+16228*y-2520*y**2-840*y**3-420*y**4-6237*y**5-1743*y**6-33*y**7
        +33*y**8)-7*r**4*(-1+y)**4*(-56788+294592*y-576768*y**2+519392*y**3
        -197288*y**4+15753*y**5-892*y**6+568*y**7-12*y**8+3*y**9)-35*r**8*(
        -1+y)**2*(5700-34200*y+85500*y**2-114000*y**3+85500*y**4-37587*y**5
        +7630*y**6-666*y**7-42*y**8+5*y**9)+21*r**12*(-412+3296*y-11536*y**2
        +23072*y**3-28840*y**4+24899*y**5-13832*y**6+4732*y**7-884*y**8+65*y**9)
        +14*r**10*(3142-25136*y+87976*y**2-175952*y**3+219940*y**4-184247*y**5
        +104116*y**6-36914*y**7+7876*y**8-819*y**9+18*y**10))-2*MBhat**3*(18000*r**14*(
        -1+y)**8-210*r**4*(-1+y)**4*(-336+6240*y-19152*y**2+23325*y**3-12380*y**4
        +1964*y**5-144*y**6+3*y**7)+84*r**2*(-1+y)**6*(10286-21172*y+11186*y**2
        +940*y**3+660*y**4+103*y**5-6*y**6+3*y**7)+(-1+y)**7*(209352-293352*y
        +42000*y**2+40250*y**3+50470*y**4+17220*y**5+1260*y**6-3*y**7+3*y**8)
        +280*r**6*(-1+y)**3*(3226-16130*y+32260*y**2-32410*y**3+19346*y**4
        -6754*y**5+2089*y**6-485*y**7+58*y**8)-105*r**8*(-1+y)**2*(-1256+7536*y
        -18840*y**2+22510*y**3-23540*y**4+17270*y**5-10360*y**6+4129*y**7-874*y**8
        +65*y**9)+14*r**12*(-6016+48128*y-168448*y**2+346031*y**3-448000*y**4
        +376922*y**5-203896*y**6+67076*y**7-11692*y**8+735*y**9)-84*r**10*(
        -1338+10704*y-37464*y**2+79128*y**3-103210*y**4+83855*y**5-41230*y**6
        +9572*y**7+779*y**8-949*y**9+153*y**10)))/(5040*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (70*MBhat**8*(-1+y)**4*(-195*r**8*(-1+y)**5+12*logy*(16+27*r**4)*(-1
        +y)**5-24*r**2*(-1+y)**3*(-31+62*y-31*y**2-6*y**3+2*y**4)+(-1+y)**4*(397
        -589*y+96*y**2+176*y**3+13*y**4+3*y**5)-4*r**6*(214-1070*y+2140*y**2
        -2344*y**3+1355*y**4-379*y**5+36*y**6)+6*r**4*(168-894*y+1923*y**2
        -2223*y**3+1469*y**4-523*y**5+80*y**6))+56*MBhat**7*(-1+y)**3*(-1428*r**10*(
        -1+y)**6+240*logy*(-7+r**2)*(-1+y)**6-20*r**2*(-1+y)**4*(463-938*y
        +481*y**2+161*y**3-124*y**4+29*y**5)-(-1+y)**5*(3668-5348*y+840*y**2
        +1510*y**3+290*y**4-3*y**5+3*y**6)+30*r**4*(-1+y)**2*(416-1664*y
        +2496*y**2-1700*y**3+310*y**4+169*y**5-80*y**6+5*y**7)-5*r**8*(-1276
        +7656*y-19140*y**2+26990*y**3-21960*y**4+10149*y**5-2374*y**6+195*y**7)
        +20*r**6*(-592+3552*y-8880*y**2+12365*y**3-10125*y**4+4680*y**5-1051*y**6
        +33*y**7+18*y**8))+84*MBhat**6*(-1+y)**2*(-1960*r**12*(-1+y)**7+60*logy*(56
        -8*r**2-199*r**4+73*r**6)*(-1+y)**7+5*r**2*(-1+y)**5*(4765-9626*y
        +4909*y**2+1408*y**3-707*y**4+134*y**5+13*y**6)+(-1+y)**6*(7815-11175*y
        +1680*y**2+2720*y**3+1025*y**4+175*y**5-y**6+y**7)-10*r**4*(-1+y)**3*(2292
        -7974*y+9573*y**2-3274*y**3-2824*y**4+2764*y**5-1013*y**6+120*y**7)
        +10*r**6*(-1+y)**2*(-1576+7442*y-13789*y**2+13385*y**3-4222*y**4-1964*y**5
        +2195*y**6-623*y**7+48*y**8)-3*r**10*(3125-21875*y+65625*y**2-113375*y**3
        +119105*y**4-77175*y**5+29533*y**6-5851*y**7+408*y**8)+5*r**8*(3253
        -22771*y+68313*y**2-118799*y**3+126343*y**4-81281*y**5+29428*y**6
        -4060*y**7-621*y**8+195*y**9))+3*(-195*r**16*(-1+y)**9-280*r**6*(-1
        +y)**4*(954+7800*logy*(-1+y)**5-12570*y+44640*y**2-70640*y**3+54820*y**4
        -18764*y**5+1300*y**6+164*y**7+21*y**8-5*y**9)-16*r**2*(-1+y)**7*(
        -22263+6720*logy*(-1+y)**2+51246*y-32343*y**2+2240*y**3+560*y**4
        +224*y**5+112*y**6-536*y**7+50*y**8+10*y**9)+14*r**4*(-1+y)**5*(104734
        -74220*logy*(-1+y)**4-493156*y+888174*y**2-740556*y**3+259359*y**4
        -14844*y**5-2474*y**6-1644*y**7+631*y**8+16*y**9)+112*r**10*(-1+y)**2*(5753
        -1740*logy*(-1+y)**7-38531*y+109503*y**2-170325*y**3+155100*y**4
        -80880*y**5+20870*y**6-366*y**7-613*y**8+89*y**9)-(-1+y)**8*(3811
        -3811*y-2752*y**7+417*y**8+95*y**9)+14*r**12*(-1+y)*(1378+1620*logy*(
        -1+y)**8-9404*y+26434*y**2-37748*y**3+24505*y**4+3076*y**5-16658*y**6
        +16108*y**7-5831*y**8+960*y**9-20*y**10)-70*r**8*(-1+y)**3*(21040+19620*logy*(
        -1+y)**6-106620*y+207690*y**2-178820*y**3+36015*y**4+44454*y**5-27029*y**6
        +1948*y**7+575*y**8-54*y**9+y**10)-8*r**14*(578-5202*y+20808*y**2
        -48552*y**3+72828*y**4-72828*y**5+48552*y**6-23008*y**7+6687*y**8-1043*y**9
        +60*y**10))-8*MBhat**5*(-1+y)*(18000*r**14*(-1+y)**8-1680*logy*(-35
        +24*r**2+411*r**4)*(-1+y)**8+84*r**2*(-1+y)**6*(7156-14792*y+7876*y**2
        +980*y**3+100*y**4+83*y**5-6*y**6+3*y**7)-210*r**4*(-1+y)**4*(14+3232*y
        -11424*y**2+14957*y**3-8728*y**4+1907*y**5-328*y**6+34*y**7)+(-1
        +y)**7*(144084-202884*y+29400*y**2+36050*y**3+29470*y**4+10164*y**5
        +756*y**6-3*y**7+3*y**8)+70*r**6*(-1+y)**3*(7384-36920*y+73840*y**2
        -74440*y**3+47300*y**4-19939*y**5+8011*y**6-2123*y**7+247*y**8)-105*r**8*(
        -1+y)**2*(236-1416*y+3540*y**2-7330*y**3+1516*y**4+5240*y**5-7140*y**6
        +3763*y**7-874*y**8+65*y**9)+14*r**12*(-6526+52208*y-182728*y**2+374591*y**3
        -483700*y**4+405482*y**5-218176*y**6+71156*y**7-12202*y**8+735*y**9)
        -42*r**10*(-3832+30656*y-107296*y**2+222992*y**3-290340*y**4+240003*y**5
        -122724*y**6+32988*y**7-1050*y**8-1703*y**9+306*y**10))+2*MBhat**2*(
        -5145*r**16*(-1+y)**9+840*logy*(56-1848*r**2-14115*r**4-18425*r**6
        -3780*r**8+363*r**10)*(-1+y)**9-42*r**2*(-1+y)**7*(-125307+287574*y
        -180747*y**2+12320*y**3+3080*y**4-3616*y**5-2077*y**6-198*y**7+11*y**8)
        -210*r**8*(-1+y)**3*(48845-277950*y+649515*y**2-790420*y**3+517215*y**4
        -166230*y**5+9024*y**6+3694*y**7-661*y**8+8*y**9)-(-1+y)**8*(-65837
        +112877*y-23520*y**2-7840*y**3-3920*y**4-58800*y**5-44366*y**6-3586*y**7
        +813*y**8+99*y**9)+70*r**6*(-1+y)**4*(110639-332095*y+111440*y**2
        +625560*y**3-865530*y**4+394062*y**5-26728*y**6-6400*y**7-301*y**8
        +105*y**9)-42*r**4*(-1+y)**5*(-291065+1446560*y-2734440*y**2+2387560*y**3
        -879190*y**4+63900*y**5+2622*y**6+412*y**7+287*y**8-8*y**9+2*y**10)
        +42*r**10*(-1+y)**2*(-16061+105167*y-290091*y**2+432665*y**3-369140*y**4
        +212760*y**5-48663*y**6+2405*y**7+2647*y**8-537*y**9+48*y**10)-6*r**14*(7495
        -67455*y+269820*y**2-629580*y**3+944370*y**4-999810*y**5+718284*y**6
        -344556*y**7+104499*y**8-17731*y**9+1224*y**10)+14*r**12*(9571-86139*y
        +344556*y**2-803964*y**3+1205946*y**4-1296522*y**5+990030*y**6-501630*y**7
        +168705*y**8-33505*y**9+2952*y**10))+8*MBhat*(-408*r**16*(-1+y)**9+1680*logy*(
        -1+118*r**2+984*r**4+1642*r**6+695*r**8)*(-1+y)**9-42*r**4*(-1+y)**5*(47592
        -229728*y+423312*y**2-360928*y**3+129592*y**4-7872*y**5-2032*y**6
        +303*y**7-2*y**8+3*y**9)+r**2*(-1+y)**7*(-648968+1496176*y-946328*y**2
        +66080*y**3+16520*y**4+6608*y**5-17780*y**6-2711*y**7+130*y**8+33*y**9)
        +(-1+y)**8*(2944-1264*y-840*y**2-280*y**3-140*y**4-84*y**5-6020*y**6
        -1363*y**7+267*y**8+60*y**9)-35*r**8*(-1+y)**3*(-56840+307680*y-669120*y**2
        +725360*y**3-377220*y**4+50808*y**5+28312*y**6-4737*y**7-474*y**8+71*y**9)
        +14*r**6*(-1+y)**4*(-28972-52180*y+596960*y**2-1253760*y**3+1119480*y**4
        -420936*y**5+32300*y**6+1925*y**7+875*y**8-15*y**9+3*y**10)+21*r**10*(
        -1+y)**2*(-13864+97048*y-291144*y**2+485240*y**3-485240*y**4+291144*y**5
        -106684*y**6+18363*y**7-1565*y**8-23*y**9+5*y**10)-4*r**14*(1366-12294*y
        +49176*y**2-114744*y**3+172116*y**4-172116*y**5+123984*y**6-59076*y**7
        +17694*y**8-2941*y**9+195*y**10)-28*r**12*(-1444+12996*y-51984*y**2
        +121296*y**3-181944*y**4+181944*y**5-126378*y**6+61101*y**7-18834*y**8
        +3586*y**9-348*y**10+9*y**11))+24*MBhat**3*(-1000*r**16*(-1+y)**9+1680*logy*(
        -7+75*r**2+591*r**4+545*r**6)*(-1+y)**9-140*r**6*(-1+y)**4*(7355-30235*y
        +44120*y**2-22320*y**3-5300*y**4+8528*y**5-1306*y**6-78*y**7+20*y**8)
        +(-1+y)**8*(-26412+38172*y-5880*y**2-1960*y**3-6720*y**4-9807*y**5
        -2905*y**6-200*y**7+32*y**8)+r**2*(-1+y)**7*(-496420+1118840*y-685420*y**2
        +42000*y**3-10920*y**4-19040*y**5-3920*y**6+y**7-2*y**8+y**9)-7*r**4*(
        -1+y)**5*(121948-629632*y+1228128*y**2-1102432*y**3+412248*y**4-23929*y**5
        -1884*y**6-1024*y**7-84*y**8+21*y**9)-35*r**8*(-1+y)**3*(-9460+56760*y
        -141900*y**2+189200*y**3-139760*y**4+61745*y**5-13286*y**6+2153*y**7
        -280*y**8+28*y**9)+7*r**10*(-1+y)**2*(4516-31612*y+94836*y**2-158060*y**3
        +136080*y**4-93464*y**5+39344*y**6-14505*y**7+4459*y**8-859*y**9+65*y**10)
        -4*r**14*(1559-14031*y+56124*y**2-130956*y**3+204134*y**4-214452*y**5
        +153258*y**6-73032*y**7+21933*y**8-3662*y**9+245*y**10)+7*r**12*(1540
        -13860*y+55440*y**2-129360*y**3+210480*y**4-232275*y**5+169635*y**6
        -82812*y**7+24708*y**8-3457*y**9-107*y**10+68*y**11))+MBhat**4*(-45675*r**16*(
        -1+y)**9+4200*logy*(112-352*r**2-3516*r**4-1644*r**6+237*r**8)*(-1
        +y)**9+28*r**2*(-1+y)**7*(313740-680280*y+392940*y**2-5120*y**3+21205*y**4
        +7750*y**5+174*y**6-18*y**7+9*y**8)+210*r**4*(-1+y)**5*(42400-239920*y
        +500520*y**2-477680*y**3+191343*y**4-10492*y**5-2442*y**6-492*y**7
        +123*y**8)+280*r**6*(-1+y)**4*(64475-297715*y+533780*y**2-454100*y**3
        +178615*y**4-12863*y**5-2248*y**6-40*y**7+176*y**8)+(-1+y)**8*(1163925
        -1634325*y+235200*y**2+165760*y**3+319550*y**4+184674*y**5+35448*y**6
        +168*y**7-3*y**8+3*y**9)-140*r**8*(-1+y)**3*(-12760+69450*y-152295*y**2
        +165350*y**3-140265*y**4+67086*y**5-34598*y**6+13632*y**7-3576*y**8
        +376*y**9)+84*r**10*(-1+y)**2*(1900-13300*y+39900*y**2-48740*y**3
        +106365*y**4-122971*y**5+106204*y**6-60180*y**7+20819*y**8-3869*y**9
        +272*y**10)-40*r**14*(5425-48825*y+195300*y**2-464940*y**3+715428*y**4
        -740628*y**5+518952*y**6-240408*y**7+69102*y**8-10678*y**9+600*y**10)
        +14*r**12*(19280-173520*y+694080*y**2-1712160*y**3+2635245*y**4-2607285*y**5
        +1654548*y**6-585252*y**7+35883*y**8+59773*y**9-23532*y**10+2940*y**11)))/(5040*(
        -1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm6(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m6 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-1125*r**8*(-1+y)**5+60*logy*(-8+9*r**4)*(-1+y)**5+5*(-1+y)**5*(257
        -96*y-48*y**2+16*y**3+9*y**4)-24*r**2*(-1+y)**2*(-75+225*y-225*y**2
        +145*y**3-120*y**4+42*y**5)+6*r**4*(240-1290*y+2805*y**2-3145*y**3
        +1895*y**4-577*y**5+72*y**6)-4*r**6*(770-3850*y+7700*y**2-8600*y**3
        +5785*y**4-2021*y**5+288*y**6))/(360*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (60*logy*(8*MBhat*(5+13*r**2)+2*MBhat**2*(-8+9*r**4)+3*(-8-40*r**2
        +9*r**4+9*r**6))*(-1+y)**6+2*MBhat**2*(-1+y)*(-1125*r**8*(-1+y)**5
        +5*(-1+y)**5*(257-96*y-48*y**2+16*y**3+9*y**4)-24*r**2*(-1+y)**2*(
        -75+225*y-225*y**2+145*y**3-120*y**4+42*y**5)+6*r**4*(240-1290*y
        +2805*y**2-3145*y**3+1895*y**4-577*y**5+72*y**6)-4*r**6*(770-3850*y
        +7700*y**2-8600*y**3+5785*y**4-2021*y**5+288*y**6))-3*(225*r**10*(-1
        +y)**6-(-1+y)**6*(1825-480*y-240*y**2-160*y**3+65*y**4+48*y**5)
        +2*r**4*(-1+y)**2*(360-1710*y+3105*y**2-2610*y**3+1400*y**4-908*y**5
        +243*y**6)-r**2*(-1+y)**3*(1135-1005*y-2595*y**2+3265*y**3-1055*y**4
        +669*y**5-301*y**6+15*y**7)+r**8*(-1135+6810*y-17025*y**2+22700*y**3
        -18735*y**4+10302*y**5-3061*y**6+384*y**7)+2*r**6*(440-2910*y+8085*y**2
        -12130*y**3+11145*y**4-6966*y**5+2971*y**6-698*y**7+63*y**8))-2*MBhat*(1260*r**10*(
        -1+y)**6+(-1+y)**6*(4180-1200*y-600*y**2+230*y**3+120*y**4+9*y**5)
        +4*r**2*(-1+y)**3*(1135-2625*y+1455*y**2-110*y**3+525*y**4-393*y**5
        +109*y**6)-6*r**4*(-1+y)**2*(120-480*y+720*y**2+240*y**3-1790*y**4
        +1667*y**5-612*y**6+63*y**7)+r**8*(-4100+24600*y-61500*y**2+87130*y**3
        -75360*y**4+38019*y**5-10274*y**6+1125*y**7)-4*r**6*(-980+5880*y-14700*y**2
        +20365*y**3-16455*y**4+6672*y**5-335*y**6-591*y**7+144*y**8)))/(720*(
        -1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-21070-64000*r**2+111650*r**4-27200*r**6+5950*r**8-6080*r**10+750*r**12
        +300*logy*(8*MBhat**3*(5+13*r**2)+MBhat**4*(-8+9*r**4)+8*MBhat*(7+64*r**2
        +13*r**4)+3*MBhat**2*(-24-136*r**2-5*r**4+19*r**6)+2*(-8-104*r**2
        -47*r**4+24*r**6+9*r**8))*(-1+y)**7+152290*y+510400*r**2*y-753350*r**4*y
        +176000*r**6*y-47050*r**8*y+42560*r**10*y-5250*r**12*y-473670*y**2-1749600*r**2*y**2
        +2161350*r**4*y**2-477600*r**6*y**2+160050*r**8*y**2-127680*r**10*y**2+15750*r**12*y**2
        +823050*y**3+3352800*r**2*y**3-3404850*r**4*y**3+695200*r**6*y**3-304550*r**8*y**3
        +212800*r**10*y**3-26250*r**12*y**3-865050*y**4-3898800*r**2*y**4+3158100*r**4*y**4
        -569200*r**6*y**4+351800*r**8*y**4-212800*r**10*y**4+26250*r**12*y**4+550998*y**5
        +2776440*r**2*y**5-1688820*r**4*y**5+239040*r**6*y**5-264720*r**8*y**5+137832*r**10*y**5
        -15750*r**12*y**5-190713*y**6-1144920*r**2*y**6+425550*r**4*y**6-21620*r**6*y**6
        +146485*r**8*y**6-62972*r**10*y**6+5250*r**12*y**6+6447*y**7+226200*r**2*y**7
        +38310*r**4*y**7-23140*r**6*y**7-59755*r**8*y**7+15908*r**10*y**7-750*r**12*y**7
        +37053*y**8-4440*r**2*y**8-71130*r**4*y**8+9780*r**6*y**8+12735*r**8*y**8
        -1728*r**10*y**8-28475*y**9-7640*r**2*y**9+27430*r**4*y**9-1260*r**6*y**9
        -945*r**8*y**9+9467*y**10+5720*r**2*y**10-4240*r**4*y**10+963*y**11-2520*r**2*y**11
        -1617*y**12+360*r**2*y**12+327*y**13+5*MBhat**4*(-1+y)**2*(-1125*r**8*(
        -1+y)**5+5*(-1+y)**5*(257-96*y-48*y**2+16*y**3+9*y**4)-24*r**2*(
        -1+y)**2*(-75+225*y-225*y**2+145*y**3-120*y**4+42*y**5)+6*r**4*(240
        -1290*y+2805*y**2-3145*y**3+1895*y**4-577*y**5+72*y**6)-4*r**6*(770
        -3850*y+7700*y**2-8600*y**3+5785*y**4-2021*y**5+288*y**6))-10*MBhat**3*(
        -1+y)*(1260*r**10*(-1+y)**6+(-1+y)**6*(4180-1200*y-600*y**2+230*y**3
        +120*y**4+9*y**5)+4*r**2*(-1+y)**3*(1135-2625*y+1455*y**2-110*y**3
        +525*y**4-393*y**5+109*y**6)-6*r**4*(-1+y)**2*(120-480*y+720*y**2
        +240*y**3-1790*y**4+1667*y**5-612*y**6+63*y**7)+r**8*(-4100+24600*y
        -61500*y**2+87130*y**3-75360*y**4+38019*y**5-10274*y**6+1125*y**7)
        -4*r**6*(-980+5880*y-14700*y**2+20365*y**3-16455*y**4+6672*y**5-335*y**6
        -591*y**7+144*y**8))-4*MBhat*(750*r**12*(-1+y)**7+2*(-1+y)**7*(8815
        -2100*y-1050*y**2-700*y**3+180*y**4+192*y**5+30*y**6)+20*r**2*(-1
        +y)**4*(-1000+4920*y-7800*y**2+4520*y**3-465*y**4-123*y**5-19*y**6
        +15*y**7)+5*r**4*(-1+y)**3*(-8390+32000*y-44880*y**2+26800*y**3-6760*y**4
        +2865*y**5-1448*y**6+317*y**7)-5*r**6*(-1+y)**2*(320-1600*y+3200*y**2
        -3200*y**3+2200*y**4-2915*y**5+2727*y**6-843*y**7+63*y**8)-5*r**8*(1130
        -7910*y+23730*y**2-39550*y**3+42040*y**4-30837*y**5+14579*y**6-3587*y**7
        +405*y**8)+r**10*(3680-25760*y+77280*y**2-128800*y**3+137260*y**4
        -98511*y**5+43337*y**6-10691*y**7+1125*y**8))-15*MBhat**2*(456*r**12*(
        -1+y)**7-(-1+y)**7*(5659-1440*y-720*y**2-64*y**3+177*y**4+64*y**5
        +y**6)-r**2*(-1+y)**4*(1429+3873*y-16113*y**2+13611*y**3-2735*y**4
        +397*y**5-93*y**6+15*y**7)+10*r**4*(-1+y)**3*(452-1778*y+2607*y**2
        -1390*y**3-236*y**4+436*y**5-203*y**6+40*y**7)-2*r**6*(-1+y)**2*(-40
        -370*y+2165*y**2-5105*y**3+8890*y**4-9376*y**5+4913*y**6-1333*y**7
        +128*y**8)+r**10*(1541-10787*y+32361*y**2-56191*y**3+61285*y**4-41811*y**5
        +17297*y**6-3911*y**7+360*y**8)+r**8*(-875+6125*y-18375*y**2+33345*y**3
        -36485*y**4+18899*y**5+490*y**6-5142*y**7+2393*y**8-375*y**9)))/(1800*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (403225+3573045*r**2-2653875*r**4-2867375*r**6+1551375*r**8-67725*r**10
        +66955*r**12-5625*r**14+2100*logy*(-40-936*r**2-1755*r**4+505*r**6
        +225*r**8+45*r**10+24*MBhat**5*(5+13*r**2)+2*MBhat**6*(-8+9*r**4)-72*MBhat*(
        -3-55*r**2-65*r**4+15*r**6)+9*MBhat**4*(-40-232*r**2-19*r**4+29*r**6)
        -16*MBhat**3*(-35-333*r**2-87*r**4+39*r**6)+6*MBhat**2*(-80-1096*r**2
        -694*r**4+167*r**6+57*r**8))*(-1+y)**8-3309800*y-30549960*r**2*y+17545500*r**4*y
        +23999500*r**6*y-11938500*r**8*y+636300*r**10*y-535640*r**12*y+45000*r**14*y
        +11920300*y**2+114787260*r**2*y**2-46667250*r**4*y**2-88240250*r**6*y**2
        +39894750*r**8*y**2-2605050*r**10*y**2+1874740*r**12*y**2-157500*r**14*y**2
        -24624600*y**3-247920120*r**2*y**3+58936500*r**4*y**3+186378500*r**6*y**3
        -75379500*r**8*y**3+6092100*r**10*y**3-3749480*r**12*y**3+315000*r**14*y**3
        +31956750*y**4+337418550*r**2*y**4-22073625*r**4*y**4-247820125*r**6*y**4
        +87609375*r**8*y**4-8938125*r**10*y**4+4686850*r**12*y**4-393750*r**14*y**4
        -26741400*y**5-297453240*r**2*y**5-33938100*r**4*y**5+213103100*r**6*y**5
        -63472500*r**8*y**5+8473500*r**10*y**5-3749480*r**12*y**5+315000*r**14*y**5
        +14179753*y**6+167020287*r**2*y**6+51291240*r**4*y**6-116279240*r**6*y**6
        +27413715*r**8*y**6-5392023*r**10*y**6+1992928*r**12*y**6-157500*r**14*y**6
        -4564736*y**7-55274016*r**2*y**7-28954380*r**4*y**7+36655780*r**6*y**7-6204660*r**8*y**7
        +2622396*r**10*y**7-775544*r**12*y**7+45000*r**14*y**7+1216405*y**8+8071161*r**2*y**8
        +6232695*r**4*y**8-4108685*r**6*y**8+589890*r**8*y**8-1001490*r**10*y**8
        +169519*r**12*y**8-5625*r**14*y**8-1042104*y**9+959448*r**2*y**9+993720*r**4*y**9
        -1359400*r**6*y**9-83160*r**8*y**9+192024*r**10*y**9-16128*r**12*y**9+1090782*y**10
        -932190*r**2*y**10-975030*r**4*y**10+619010*r**6*y**10+19215*r**8*y**10-11907*r**10*y**10
        -677208*y**11+334488*r**2*y**11+308280*r**4*y**11-79240*r**6*y**11+205156*y**12
        -6804*r**2*y**12-45675*r**4*y**12-1575*r**6*y**12+3640*y**13-34776*r**2*y**13
        -20051*y**14+6867*r**2*y**14+3888*y**15+70*MBhat**6*(-1+y)**3*(-1125*r**8*(
        -1+y)**5+5*(-1+y)**5*(257-96*y-48*y**2+16*y**3+9*y**4)-24*r**2*(
        -1+y)**2*(-75+225*y-225*y**2+145*y**3-120*y**4+42*y**5)+6*r**4*(240
        -1290*y+2805*y**2-3145*y**3+1895*y**4-577*y**5+72*y**6)-4*r**6*(770
        -3850*y+7700*y**2-8600*y**3+5785*y**4-2021*y**5+288*y**6))-210*MBhat**5*(
        -1+y)**2*(1260*r**10*(-1+y)**6+(-1+y)**6*(4180-1200*y-600*y**2+230*y**3
        +120*y**4+9*y**5)+4*r**2*(-1+y)**3*(1135-2625*y+1455*y**2-110*y**3
        +525*y**4-393*y**5+109*y**6)-6*r**4*(-1+y)**2*(120-480*y+720*y**2
        +240*y**3-1790*y**4+1667*y**5-612*y**6+63*y**7)+r**8*(-4100+24600*y
        -61500*y**2+87130*y**3-75360*y**4+38019*y**5-10274*y**6+1125*y**7)
        -4*r**6*(-980+5880*y-14700*y**2+20365*y**3-16455*y**4+6672*y**5-335*y**6
        -591*y**7+144*y**8))+315*MBhat**4*(-1+y)*(-912*r**12*(-1+y)**7+(-1
        +y)**7*(9493-2400*y-1200*y**2+32*y**3+289*y**4+80*y**5+2*y**6)+r**2*(
        -1+y)**4*(1723+8751*y-29631*y**2+23957*y**3-4415*y**4+125*y**5+115*y**6
        +15*y**7)-2*r**4*(-1+y)**3*(4160-16070*y+22965*y**2-11290*y**3-3760*y**4
        +5268*y**5-2273*y**6+400*y**7)+r**10*(-3307+23149*y-69447*y**2+120257*y**3
        -130445*y**4+88347*y**5-36169*y**6+8047*y**7-720*y**8)+2*r**6*(-1+y)**2*(
        -520+1730*y-1285*y**2-3695*y**3+13150*y**4-16416*y**5+9191*y**6-2603*y**7
        +256*y**8)+r**8*(2885-20195*y+60585*y**2-106415*y**3+114405*y**4-66835*y**5
        +12383*y**6+6839*y**7-4402*y**8+750*y**9))-6*MBhat*(3900*r**14*(-1+y)**8
        +(-1+y)**8*(351420-75600*y-37800*y**2-25200*y**3-18900*y**4+4221*y**5
        +5516*y**6+981*y**7)+140*r**6*(-1+y)**3*(1725-11325*y+29400*y**2-38400*y**3
        +25950*y**4-8889*y**5+2755*y**6-1031*y**7+175*y**8)-105*r**4*(-1+y)**4*(21980
        -72320*y+77280*y**2-20320*y**3-10520*y**4+3681*y**5-452*y**6+152*y**7
        -12*y**8+3*y**9)+42*r**2*(-1+y)**5*(-49150+180450*y-229950*y**2+109650*y**3
        -8250*y**4-677*y**5-1819*y**6-61*y**7+117*y**8+10*y**9)-105*r**8*(
        -1+y)**2*(-900+5400*y-13500*y**2+18000*y**3-13500*y**4+4901*y**5
        -1030*y**6+1222*y**7-334*y**8+21*y**9)+21*r**12*(-1300+10400*y-36400*y**2
        +72800*y**3-91000*y**4+77021*y**5-46648*y**6+17828*y**7-3916*y**8+375*y**9)
        +42*r**10*(1490-11920*y+41720*y**2-83440*y**3+104300*y**4-88213*y**5
        +55766*y**6-25198*y**7+6368*y**8-921*y**9+48*y**10))-2*MBhat**3*(50760*r**14*(
        -1+y)**8+420*r**2*(-1+y)**5*(-7710+36450*y-56430*y**2+32576*y**3
        -4083*y**4-507*y**5-21*y**6+45*y**7)+(-1+y)**8*(2504520-588000*y
        -294000*y**2-116830*y**3+49140*y**4+39312*y**5+5880*y**6+45*y**7)+210*r**4*(
        -1+y)**4*(-31676+119744*y-165696*y**2+95349*y**3-17976*y**4+3420*y**5
        -1808*y**6+323*y**7)+280*r**6*(-1+y)**3*(-1405+4685*y-3520*y**2-7100*y**3
        +16765*y**4-11699*y**5+3383*y**6-1117*y**7+224*y**8)-105*r**8*(-1+y)**2*(
        -5880+35280*y-88200*y**2+119290*y**3-115200*y**4+90726*y**5-47828*y**6
        +18651*y**7-4254*y**8+375*y**9)+70*r**12*(-2380+19040*y-66640*y**2
        +137501*y**3-182728*y**4+159110*y**5-90328*y**6+31964*y**7-6304*y**8
        +513*y**9)-84*r**10*(-170+1360*y-4760*y**2+16050*y**3-25155*y**4+3708*y**5
        +28266*y**6-33096*y**7+18582*y**8-5460*y**9+675*y**10))-3*MBhat**2*(18810*r**14*(
        -1+y)**8+140*r**4*(-1+y)**4*(52136-187724*y+239946*y**2-118324*y**3
        +10261*y**4+2628*y**5-799*y**6+256*y**7)-(-1+y)**8*(1504350-336000*y
        -168000*y**2-112000*y**3-15855*y**4+30576*y**5+11291*y**6+720*y**7)
        +70*r**6*(-1+y)**3*(8115-30555*y+36060*y**2-2660*y**3-15905*y**4-6023*y**5
        +12742*y**6-4238*y**7+640*y**8)+35*r**2*(-1+y)**5*(142828-560004*y
        +757284*y**2-383948*y**3+28623*y**4+12675*y**5+1114*y**6-342*y**7-153*y**8
        +3*y**9)+210*r**12*(-412+3296*y-11536*y**2+23072*y**3-30247*y**4+27272*y**5
        -16156*y**6+6056*y**7-1297*y**8+120*y**9)-35*r**8*(-1+y)**2*(5870-28380*y
        +50430*y**2-33040*y**3-6945*y**4+44910*y**5-55816*y**6+22698*y**7-4911*y**8
        +384*y**9)-7*r**10*(-6500+52000*y-182000*y**2+364000*y**3-546185*y**4
        +630232*y**5-431009*y**6+138880*y**7-16631*y**8-3912*y**9+1125*y**10)))/(25200*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (4200*logy*(16*MBhat**7*(5+13*r**2)+MBhat**8*(-8+9*r**4)+6*MBhat**6*(
        -56-328*r**2-33*r**4+39*r**6)-16*MBhat**5*(-49-474*r**2-135*r**4+78*r**6)
        -48*MBhat**3*(-21-395*r**2-489*r**4+119*r**6+40*r**8)-16*MBhat*(-11
        -328*r**2-954*r**4-40*r**6+235*r**8)+MBhat**4*(-1120-15680*r**2-10404*r**4
        +3660*r**6+675*r**8)+2*MBhat**2*(-280-6744*r**2-13437*r**4+1633*r**6
        +1620*r**8+189*r**10)+3*(-8-288*r**2-1143*r**4-280*r**6+615*r**8
        +72*r**10+9*r**12))*(-1+y)**9+70*MBhat**8*(-1+y)**4*(-1125*r**8*(-1+y)**5
        +5*(-1+y)**5*(257-96*y-48*y**2+16*y**3+9*y**4)-24*r**2*(-1+y)**2*(
        -75+225*y-225*y**2+145*y**3-120*y**4+42*y**5)+6*r**4*(240-1290*y
        +2805*y**2-3145*y**3+1895*y**4-577*y**5+72*y**6)-4*r**6*(770-3850*y
        +7700*y**2-8600*y**3+5785*y**4-2021*y**5+288*y**6))-280*MBhat**7*(-1
        +y)**3*(1260*r**10*(-1+y)**6+(-1+y)**6*(4180-1200*y-600*y**2+230*y**3
        +120*y**4+9*y**5)+4*r**2*(-1+y)**3*(1135-2625*y+1455*y**2-110*y**3
        +525*y**4-393*y**5+109*y**6)-6*r**4*(-1+y)**2*(120-480*y+720*y**2
        +240*y**3-1790*y**4+1667*y**5-612*y**6+63*y**7)+r**8*(-4100+24600*y
        -61500*y**2+87130*y**3-75360*y**4+38019*y**5-10274*y**6+1125*y**7)
        -4*r**6*(-980+5880*y-14700*y**2+20365*y**3-16455*y**4+6672*y**5-335*y**6
        -591*y**7+144*y**8))+420*MBhat**6*(-1+y)**2*(-1368*r**12*(-1+y)**7+(
        -1+y)**7*(13327-3360*y-1680*y**2+128*y**3+401*y**4+96*y**5+3*y**6)
        +r**2*(-1+y)**4*(2017+13629*y-43149*y**2+34303*y**3-6095*y**4-147*y**5
        +323*y**6+15*y**7)-2*r**4*(-1+y)**3*(6060-23250*y+32895*y**2-15630*y**3
        -6340*y**4+8356*y**5-3531*y**6+600*y**7)-3*r**10*(1691-11837*y+35511*y**2
        -61441*y**3+66535*y**4-44961*y**5+18347*y**6-4061*y**7+360*y**8)+2*r**6*(
        -1+y)**2*(-1000+3830*y-4735*y**2-2285*y**3+17410*y**4-23456*y**5
        +13469*y**6-3873*y**7+384*y**8)+r**8*(4895-34265*y+102795*y**2-179485*y**3
        +192325*y**4-114771*y**5+25256*y**6+8536*y**7-6411*y**8+1125*y**9))
        -8*MBhat**5*(-1+y)*(50760*r**14*(-1+y)**8+210*r**4*(-1+y)**4*(-23286
        +87744*y-120816*y**2+68549*y**3-11216*y**4+555*y**5-360*y**6+6*y**7)
        +420*r**2*(-1+y)**5*(-5710+26610*y-40830*y**2+23536*y**3-3153*y**4
        -261*y**5+17*y**6+15*y**7)+(-1+y)**8*(1764060-411600*y-205800*y**2
        -58030*y**3+34020*y**4+23184*y**5+3360*y**6+45*y**7)+70*r**6*(-1+y)**3*(
        -4660+13940*y-4480*y**2-38000*y**3+73660*y**4-55541*y**5+21713*y**6
        -6997*y**7+1085*y**8)-105*r**8*(-1+y)**2*(-3620+21720*y-54300*y**2
        +74090*y**3-76320*y**4+67932*y**5-41464*y**6+17841*y**7-4254*y**8+375*y**9)
        +70*r**12*(-2830+22640*y-79240*y**2+162701*y**3-214228*y**4+184310*y**5
        -102928*y**6+35564*y**7-6754*y**8+513*y**9)-42*r**10*(-4020+32160*y
        -112560*y**2+238180*y**3-316370*y**4+243187*y**5-85316*y**6-12164*y**7
        +25348*y**8-9795*y**9+1350*y**10))-2*MBhat**2*(17955*r**16*(-1+y)**9
        -(-1+y)**9*(5709665-1176000*y-588000*y**2-392000*y**3-294000*y**4
        -55776*y**5+81662*y**6+32112*y**7+2943*y**8)+70*r**6*(-1+y)**4*(-625981
        +3227885*y-6700720*y**2+7027320*y**3-3758610*y**4+872022*y**5-55456*y**6
        -7144*y**7+3359*y**8+45*y**9)+210*r**8*(-1+y)**3*(-76625+427350*y
        -971175*y**2+1132900*y**3-687675*y**4+184542*y**5-20124*y**6+13986*y**7
        -3579*y**8+440*y**9)-42*r**2*(-1+y)**6*(-1240305+4395315*y-5406915*y**2
        +2476705*y**3-168600*y**4-17832*y**5-26355*y**6-10023*y**7+1839*y**8
        +651*y**9)+210*r**4*(-1+y)**5*(148857-326688*y-47448*y**2+569112*y**3
        -411018*y**4+52692*y**5+11762*y**6-2108*y**7+345*y**8+120*y**9+6*y**10)
        -42*r**10*(-1+y)**2*(-27255+171885*y-449505*y**2+616875*y**3-451500*y**4
        +165912*y**5+17807*y**6-96029*y**7+32433*y**8-5727*y**9+384*y**10)
        +30*r**14*(3649-32841*y+131364*y**2-306516*y**3+459774*y**4-480942*y**5
        +366996*y**6-189684*y**7+63621*y**8-12469*y**9+1080*y**10)-14*r**12*(3665
        -32985*y+131940*y**2-307860*y**3+461790*y**4-596574*y**5+723714*y**6
        -545346*y**7+204219*y**8-47531*y**9+4968*y**10))-8*MBhat*(1800*r**16*(
        -1+y)**9+(-1+y)**9*(461480-92400*y-46200*y**2-30800*y**3-23100*y**4
        -18480*y**5+3668*y**6+5643*y**7+972*y**8)+35*r**8*(-1+y)**3*(90800
        -488400*y+1051800*y**2-1120400*y**3+558300*y**4-54120*y**5-51304*y**6
        +19149*y**7-5598*y**8+733*y**9)-210*r**4*(-1+y)**5*(828+34848*y-128592*y**2
        +162048*y**3-78672*y**4+7632*y**5+1182*y**6+415*y**7-70*y**8+15*y**9
        +6*y**10)-70*r**6*(-1+y)**4*(-136240+676400*y-1340800*y**2+1324800*y**3
        -650400*y**4+125280*y**5+3200*y**6-4705*y**7+605*y**8+75*y**9+9*y**10)
        -21*r**10*(-1+y)**2*(9120-63840*y+191520*y**2-319200*y**3+319200*y**4
        -191520*y**5+57072*y**6-1427*y**7+4421*y**8-1089*y**9+63*y**10)+20*r**14*(856
        -7704*y+30816*y**2-71904*y**3+107856*y**4-107856*y**5+75432*y**6-39348*y**7
        +13212*y**8-2593*y**9+225*y**10)+r**2*(-1+y)**6*(-5839840+20274720*y
        -24407520*y**2+10891040*y**3-688800*y**4-137760*y**5+23632*y**6-100437*y**7
        -10161*y**8+7585*y**9+981*y**10)+28*r**12*(-1970+17730*y-70920*y**2
        +165480*y**3-248220*y**4+248220*y**5-173109*y**6+94038*y**7-39312*y**8
        +9308*y**9-1317*y**10+72*y**11))-3*(1125*r**16*(-1+y)**9-(-1+y)**9*(172165
        -33600*y-16800*y**2-11200*y**3-8400*y**4-6720*y**5-5600*y**6+4032*y**7
        +1095*y**8)+70*r**4*(-1+y)**5*(-26586+174924*y-399546*y**2+403524*y**3
        -169461*y**4+13716*y**5+2286*y**6+548*y**7+91*y**8+96*y**9)-112*r**10*(
        -1+y)**2*(-11205+75735*y-217755*y**2+344025*y**3-320400*y**4+173340*y**5
        -48330*y**6+4846*y**7-727*y**8+171*y**9)+280*r**6*(-1+y)**4*(-26450
        +128050*y-245600*y**2+231600*y**3-105300*y**4+16860*y**5+700*y**6
        +380*y**7-577*y**8+125*y**9+12*y**10)+70*r**8*(-1+y)**3*(-18000+71100*y
        -67050*y**2-95100*y**3+255825*y**4-213030*y**5+72405*y**6-4412*y**7
        -2655*y**8+502*y**9+15*y**10)+120*r**14*(154-1386*y+5544*y**2-12936*y**3
        +19404*y**4-19404*y**5+12936*y**6-5824*y**7+1971*y**8-379*y**9+32*y**10)
        -16*r**2*(-1+y)**6*(-178485+611055*y-724455*y**2+317085*y**3-18900*y**4
        -3780*y**5-1260*y**6+1448*y**7-2874*y**8+284*y**9+162*y**10)+14*r**12*(
        -1090+12510*y-62190*y**2+177510*y**3-322965*y**4+391005*y**5-317370*y**6
        +175762*y**7-75933*y**8+27257*y**9-4748*y**10+252*y**11))-24*MBhat**3*(2820*r**16*(
        -1+y)**9+(-1+y)**9*(829280-176400*y-88200*y**2-58800*y**3-26040*y**4
        +8715*y**5+8708*y**6+1684*y**7+60*y**8)+140*r**6*(-1+y)**4*(9663-55455*y
        +128760*y**2-152560*y**3+93030*y**4-24624*y**5+2782*y**6-790*y**7+98*y**8)
        -35*r**4*(-1+y)**5*(151548-488832*y+498528*y**2-97632*y**3-93812*y**4
        +27563*y**5-1596*y**6+384*y**7+144*y**8+9*y**9)+35*r**8*(-1+y)**3*(20500
        -113400*y+254700*y**2-291600*y**3+166120*y**4-20541*y**5-21382*y**6
        +5167*y**7-1164*y**8+200*y**9)-r**2*(-1+y)**6*(5000500-18319500*y
        +23296500*y**2-11083500*y**3+760060*y**4+209664*y**5+113708*y**6-2283*y**7
        -6339*y**8-185*y**9+15*y**10)+60*r**14*(197-1773*y+7092*y**2-16548*y**3
        +25802*y**4-28224*y**5+21210*y**6-10800*y**7+3555*y**8-680*y**9+57*y**10)
        -7*r**10*(-1+y)**2*(17460-122220*y+366660*y**2-611100*y**3+599420*y**4
        -402560*y**5+236796*y**6-87859*y**7+26245*y**8-4977*y**9+375*y**10)
        -7*r**12*(-500+4500*y-18000*y**2+42000*y**3-38980*y**4-9057*y**5+23733*y**6
        +1548*y**7-10512*y**8+7293*y**9-2325*y**10+300*y**11))+MBhat**4*(-105525*r**16*(
        -1+y)**9+210*r**4*(-1+y)**5*(-504384+1809456*y-2298024*y**2+1119216*y**3
        -93799*y**4-14660*y**5-1590*y**6-1140*y**7+45*y**8)+(-1+y)**9*(21254325
        -4704000*y-2352000*y**2-1299200*y**3-62370*y**4+340704*y**5+117432*y**6
        +9600*y**7+45*y**8)-280*r**6*(-1+y)**4*(13955-14875*y-107500*y**2
        +298300*y**3-280505*y**4+67753*y**5+22664*y**6-7552*y**7+800*y**8)
        -140*r**2*(-1+y)**6*(520500-2031900*y+2737500*y**2-1390820*y**3+118095*y**4
        +37779*y**5+3764*y**6-1476*y**7-171*y**8+9*y**9)-140*r**8*(-1+y)**3*(
        -62600+355350*y-827625*y**2+1023850*y**3-755895*y**4+304962*y**5-26554*y**6
        +7248*y**7-4752*y**8+1016*y**9)+84*r**10*(-1+y)**2*(15300-107100*y
        +321300*y**2-534300*y**3+686195*y**4-698789*y**5+443016*y**6-214120*y**7
        +72313*y**8-14535*y**9+1200*y**10)-40*r**14*(7991-71919*y+287676*y**2
        -688884*y**3+1084356*y**4-1155420*y**5+841344*y**6-411336*y**7+128214*y**8
        -22706*y**9+1692*y**10)+70*r**12*(-2800+25200*y-100800*y**2+209760*y**3
        -304545*y**4+412137*y**5-468804*y**6+383220*y**7-220815*y**8+85015*y**9
        -19620*y**10+2052*y**11)))/(25200*(-1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm7(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m7 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return -1/30*(y**3*(10*(-1+y)**5+6*r**2*(-1+y)**2*(5-5*y+2*y**2)-3*r**4*(10
        -10*y-y**2+y**3)+r**6*(10+10*y-11*y**2+3*y**3)))/(-1+y)**5
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-2*MBhat**2*(-1+y)*y**3*(10*(-1+y)**5+6*r**2*(-1+y)**2*(5-5*y+2*y**2)
        -3*r**4*(10-10*y-y**2+y**3)+r**6*(10+10*y-11*y**2+3*y**3))+y**4*((
        -10+y)*(-1+y)**6-12*r**2*(-1+y)**3*(5-4*y+y**2)+6*r**4*(-1+y)**2*(
        -20+2*y+3*y**2)-3*r**8*(10+3*y-4*y**2+y**3)-2*r**6*(-50+38*y
        +27*y**2-16*y**3+y**4))+MBhat*(7*r**10*(-1+y)**6+(-1+y)**6*(3+20*y**3
        +5*y**4)+r**8*(-35+210*y-525*y**2+760*y**3-570*y**4+228*y**5-38*y**6)
        -r**2*(-1+y)**3*(125+60*logy*(-1+y)**3-435*y+525*y**2-355*y**3+150*y**4
        -66*y**5+8*y**6)+r**4*(-1+y)**2*(-160-60*logy*(-1+y)**4+580*y-750*y**2
        +620*y**3-215*y**4+96*y**5-31*y**6+4*y**7)+r**6*(60-360*y+900*y**2
        -1400*y**3+1225*y**4-546*y**5+140*y**6-22*y**7+3*y**8)))/(60*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-30*MBhat**4*(-1+y)**2*y**3*(10*(-1+y)**5+6*r**2*(-1+y)**2*(5-5*y+2*y**2)
        -3*r**4*(10-10*y-y**2+y**3)+r**6*(10+10*y-11*y**2+3*y**3))-3*y**5*(
        -1+r**2+y)**2*(-((-1+y)**5*(-19+6*y))+3*r**6*(63+7*y-13*y**2+3*y**3)
        +r**2*(-1+y)**2*(267-107*y-47*y**2+7*y**3)+r**4*(-437+359*y+195*y**2
        -109*y**3-8*y**4))+2*MBhat*(25*r**12*(-1+y)**7-(-1+y)**7*(-50-120*y**4
        -48*y**5+5*y**6)+15*r**2*(-1+y)**4*(-155-60*logy*(-1+y)**3+525*y
        -615*y**2+265*y**3+90*y**4-78*y**5+16*y**6)-45*r**4*(-1+y)**3*(25
        +60*logy*(-1+y)**4-40*y-60*y**2+160*y**3-210*y**4+60*y**5-11*y**6
        +4*y**7)-9*r**10*(-25+175*y-525*y**2+875*y**3-980*y**4+588*y**5-196*y**6
        +28*y**7)+30*r**8*(-35+245*y-735*y**2+1225*y**3-1360*y**4+924*y**5
        -308*y**6+44*y**7)+10*r**6*(-1+y)**2*(210+60*logy*(-1+y)**5-1110*y
        +2370*y**2-2570*y**3+2095*y**4-575*y**5+53*y**6-7*y**7+2*y**8))+30*MBhat**2*(8*r**12*(
        -1+y)**7-(-1+y)**7*(4+13*y**3+16*y**4)-r**2*(-1+y)**4*(-298-120*logy*(
        -1+y)**3+1014*y-1194*y**2+633*y**3-105*y**4+21*y**5+y**6)-10*r**4*(
        -1+y)**3*(-28-24*logy*(-1+y)**4+88*y-84*y**2+41*y**3+y**4+18*y**5
        -10*y**6+y**7)+r**8*(-100+700*y-2100*y**2+3765*y**3-4065*y**4+2667*y**5
        -1085*y**6+239*y**7-21*y**8)+2*r**6*(-1+y)**2*(20-100*y+200*y**2
        -415*y**3+270*y**4-192*y**5+71*y**6-11*y**7+y**8)+r**10*(46-322*y
        +966*y**2-1673*y**3+1715*y**4-1071*y**5+385*y**6-67*y**7+3*y**8))
        +30*MBhat**3*(-1+y)*(7*r**10*(-1+y)**6+(-1+y)**6*(3+20*y**3+5*y**4)
        +r**8*(-35+210*y-525*y**2+760*y**3-570*y**4+228*y**5-38*y**6)-r**2*(
        -1+y)**3*(125+60*logy*(-1+y)**3-435*y+525*y**2-355*y**3+150*y**4
        -66*y**5+8*y**6)+r**4*(-1+y)**2*(-160-60*logy*(-1+y)**4+580*y-750*y**2
        +620*y**3-215*y**4+96*y**5-31*y**6+4*y**7)+r**6*(60-360*y+900*y**2
        -1400*y**3+1225*y**4-546*y**5+140*y**6-22*y**7+3*y**8)))/(900*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-140*MBhat**6*(-1+y)**3*y**3*(10*(-1+y)**5+6*r**2*(-1+y)**2*(5-5*y
        +2*y**2)-3*r**4*(10-10*y-y**2+y**3)+r**6*(10+10*y-11*y**2+3*y**3))
        -14*y**6*(-1+r**2+y)**3*(-((-1+y)**5*(1+6*y))+3*r**2*(-1+y)**2*(104
        -45*y-22*y**2+3*y**3)+r**6*(224+8*y-28*y**2+6*y**3)-3*r**4*(179
        -160*y-64*y**2+42*y**3+3*y**4))+210*MBhat**4*(-1+y)*(16*r**12*(-1
        +y)**7-(-1+y)**7*(8+26*y**3+22*y**4+y**5)+2*r**2*(-1+y)**4*(298
        +120*logy*(-1+y)**3-1014*y+1194*y**2-633*y**3+135*y**4-45*y**5+5*y**6)
        +2*r**4*(-1+y)**3*(280+240*logy*(-1+y)**4-880*y+840*y**2-410*y**3
        +50*y**4-186*y**5+91*y**6-10*y**7)+r**8*(-200+1400*y-4200*y**2+7530*y**3
        -8160*y**4+5355*y**5-2149*y**6+463*y**7-39*y**8)+2*r**6*(-1+y)**2*(40
        -200*y+400*y**2-830*y**3+590*y**4-372*y**5+127*y**6-21*y**7+2*y**8)
        +2*r**10*(46-322*y+966*y**2-1673*y**3+1715*y**4-1071*y**5+385*y**6
        -67*y**7+3*y**8))+210*MBhat**5*(-1+y)**2*(7*r**10*(-1+y)**6+(-1+y)**6*(3
        +20*y**3+5*y**4)+r**8*(-35+210*y-525*y**2+760*y**3-570*y**4+228*y**5
        -38*y**6)-r**2*(-1+y)**3*(125+60*logy*(-1+y)**3-435*y+525*y**2-355*y**3
        +150*y**4-66*y**5+8*y**6)+r**4*(-1+y)**2*(-160-60*logy*(-1+y)**4
        +580*y-750*y**2+620*y**3-215*y**4+96*y**5-31*y**6+4*y**7)+r**6*(60
        -360*y+900*y**2-1400*y**3+1225*y**4-546*y**5+140*y**6-22*y**7+3*y**8))
        +2*MBhat**2*(330*r**14*(-1+y)**8+630*r**4*(-1+y)**4*(-41+180*logy*(-1
        +y)**4+344*y-876*y**2+944*y**3-541*y**4+67*y**5-16*y**6+14*y**7)
        +(-1+y)**8*(-1100-735*y**4-1113*y**5-28*y**6+15*y**7)+105*r**2*(-1
        +y)**5*(688+240*logy*(-1+y)**3-2304*y+2664*y**2-1128*y**3-117*y**4
        +50*y**5+32*y**6-6*y**7+y**8)-70*r**6*(-1+y)**3*(1725-300*logy*(-1
        +y)**5-8325*y+15900*y**2-14900*y**3+8755*y**4-1796*y**5+569*y**6
        -256*y**7+20*y**8)+105*r**8*(-1+y)**2*(-300+1800*y-4500*y**2+6000*y**3
        -5775*y**4+2525*y**5-799*y**6+162*y**7-14*y**8+y**9)+70*r**12*(-43
        +344*y-1204*y**2+2408*y**3-3178*y**4+2618*y**5-1372*y**6+428*y**7
        -67*y**8+3*y**9)-63*r**10*(-200+1600*y-5600*y**2+11200*y**3-14995*y**4
        +13049*y**5-7028*y**6+2380*y**7-437*y**8+31*y**9))+MBhat*(130*r**14*(
        -1+y)**8-(-1+y)**8*(-500-252*y**5-217*y**6+108*y**7)-630*r**4*(-1
        +y)**4*(-74+120*logy*(-1+y)**4+416*y-864*y**2+816*y**3-324*y**4
        -124*y**5+45*y**6+8*y**7+5*y**8)-70*r**6*(-1+y)**3*(-1875+300*logy*(
        -1+y)**5+9075*y-17400*y**2+16400*y**3-7450*y**4-1294*y**5+625*y**6
        +79*y**7+40*y**8)-28*r**12*(65-520*y+1820*y**2-3640*y**3+4550*y**4
        -4144*y**5+2072*y**6-592*y**7+74*y**8)+35*r**8*(-1+y)**2*(900+1200*logy*(
        -1+y)**6-4200*y+6900*y**2-3200*y**3-3600*y**4+9684*y**5-3185*y**6
        +78*y**7+39*y**8+4*y**9)-7*r**2*(-1+y)**5*(5510+1800*logy*(-1+y)**3
        -18330*y+21030*y**2-8810*y**3+450*y**4-2826*y**5+1153*y**6+357*y**7
        +21*y**8+5*y**9)-63*r**10*(-250+2000*y-7000*y**2+14000*y**3-17500*y**4
        +15204*y**5-8603*y**6+2424*y**7-264*y**8-12*y**9+y**10))+7*MBhat**3*(270*r**14*(
        -1+y)**8-(-1+y)**8*(-380-440*y**3-705*y**4-264*y**5+10*y**6)-15*r**2*(
        -1+y)**5*(1968+720*logy*(-1+y)**3-6624*y+7704*y**2-3584*y**3+123*y**4
        +127*y**5-39*y**6+5*y**7)-30*r**4*(-1+y)**4*(439+1140*logy*(-1+y)**4
        -616*y-1356*y**2+2664*y**3-2011*y**4+40*y**5+112*y**6+8*y**7)-15*r**8*(
        -1+y)**2*(60-360*y+900*y**2-2840*y**3+2145*y**4-2006*y**5+913*y**6
        -176*y**7+4*y**8)-10*r**6*(-1+y)**3*(-1815-60*logy*(-1+y)**5+9135*y
        -18420*y**2+15940*y**3-8915*y**4+139*y**5+922*y**6-218*y**7+16*y**8)
        +10*r**12*(-167+1336*y-4676*y**2+9576*y**3-12222*y**4+10080*y**5-5292*y**6
        +1656*y**7-261*y**8+12*y**9)-3*r**10*(-1240+9920*y-34720*y**2+73320*y**3
        -96455*y**4+81928*y**5-45584*y**6+15304*y**7-2543*y**8+40*y**9+30*y**10)))/(4200*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-35*MBhat**8*(-1+y)**4*y**3*(10*(-1+y)**5+6*r**2*(-1+y)**2*(5-5*y+2*y**2)
        -3*r**4*(10-10*y-y**2+y**3)+r**6*(10+10*y-11*y**2+3*y**3))-y**7*(
        -1+r**2+y)**4*(-4*(-1+y)**5*(13+5*y)+15*r**6*(60-5*y**2+y**3)+2*r**2*(
        -1+y)**2*(650-336*y-123*y**2+19*y**3)+r**4*(-2252+2168*y+603*y**2
        -487*y**3-32*y**4))+70*MBhat**7*(-1+y)**3*(7*r**10*(-1+y)**6+(-1+y)**6*(3
        +20*y**3+5*y**4)+r**8*(-35+210*y-525*y**2+760*y**3-570*y**4+228*y**5
        -38*y**6)-r**2*(-1+y)**3*(125+60*logy*(-1+y)**3-435*y+525*y**2-355*y**3
        +150*y**4-66*y**5+8*y**6)+r**4*(-1+y)**2*(-160-60*logy*(-1+y)**4
        +580*y-750*y**2+620*y**3-215*y**4+96*y**5-31*y**6+4*y**7)+r**6*(60
        -360*y+900*y**2-1400*y**3+1225*y**4-546*y**5+140*y**6-22*y**7+3*y**8))
        +70*MBhat**6*(-1+y)**2*(24*r**12*(-1+y)**7-(-1+y)**7*(12+39*y**3+28*y**4
        +2*y**5)+3*r**2*(-1+y)**4*(298+120*logy*(-1+y)**3-1014*y+1194*y**2
        -633*y**3+145*y**4-53*y**5+7*y**6)+6*r**4*(-1+y)**3*(140+120*logy*(
        -1+y)**4-440*y+420*y**2-205*y**3+35*y**4-94*y**5+44*y**6-5*y**7)
        +3*r**10*(46-322*y+966*y**2-1673*y**3+1715*y**4-1071*y**5+385*y**6
        -67*y**7+3*y**8)+2*r**6*(-1+y)**2*(60-300*y+600*y**2-1245*y**3+910*y**4
        -552*y**5+183*y**6-31*y**7+3*y**8)-3*r**8*(100-700*y+2100*y**2-3765*y**3
        +4085*y**4-2681*y**5+1071*y**6-229*y**7+19*y**8))+7*MBhat**5*(-1+y)*(270*r**14*(
        -1+y)**8+(-1+y)**8*(280+440*y**3+465*y**4+168*y**5)+30*r**4*(-1
        +y)**4*(-364-960*logy*(-1+y)**4+496*y+1176*y**2-2184*y**3+1381*y**4
        +140*y**5-145*y**6+4*y**7)-15*r**2*(-1+y)**5*(1658+600*logy*(-1+y)**3
        -5574*y+6474*y**2-3054*y**3+303*y**4-29*y**5-7*y**6+5*y**7)-15*r**8*(
        -1+y)**2*(200-1200*y+3000*y**2-5640*y**3+4785*y**4-3062*y**5+1089*y**6
        -176*y**7+4*y**8)-10*r**6*(-1+y)**3*(-1395+60*logy*(-1+y)**5+6915*y
        -13680*y**2+10800*y**3-4725*y**4-1011*y**5+1028*y**6-232*y**7+20*y**8)
        +20*r**12*(-86+688*y-2408*y**2+4928*y**3-6286*y**4+5180*y**5-2716*y**6
        +848*y**7-133*y**8+6*y**9)-3*r**10*(-1390+11120*y-38920*y**2+81720*y**3
        -107585*y**4+91336*y**5-50288*y**6+16648*y**7-2711*y**8+40*y**9+30*y**10))
        +MBhat**2*(105*r**16*(-1+y)**9+(-1+y)**9*(-825+126*y**5-112*y**6+48*y**7
        +27*y**8)+140*r**6*(-1+y)**4*(-2043+1260*logy*(-1+y)**5+8955*y-14760*y**2
        +10560*y**3-2130*y**4-2409*y**5+377*y**6-52*y**7+122*y**8)+84*r**2*(
        -1+y)**6*(985+300*logy*(-1+y)**3-3255*y+3705*y**2-1535*y**3+75*y**4
        -174*y**5-30*y**6+74*y**7+13*y**8+2*y**9)+210*r**4*(-1+y)**5*(-972
        +960*logy*(-1+y)**4+4848*y-9192*y**2+8048*y**3-2972*y**4-249*y**5
        +20*y**6+50*y**7+38*y**8+3*y**9)-210*r**8*(-1+y)**3*(-200+300*logy*(
        -1+y)**6+1500*y-4650*y**2+7700*y**3-7275*y**4+5133*y**5-1281*y**6
        +229*y**7-76*y**8+5*y**9)+42*r**10*(-1+y)**2*(1150-8050*y+24150*y**2
        -40250*y**3+40250*y**4-28812*y**5+10168*y**6-2216*y**7+292*y**8-13*y**9
        +y**10)+30*r**14*(46-414*y+1656*y**2-3864*y**3+5796*y**4-6174*y**5
        +4242*y**6-1908*y**7+522*y**8-73*y**9+3*y**10)-14*r**12*(660-5940*y
        +23760*y**2-55440*y**3+83160*y**4-88389*y**5+64309*y**6-29376*y**7
        +8514*y**8-1336*y**9+78*y**10))+MBhat*(20*r**16*(-1+y)**9-(-1+y)**9*(
        -120+203*y**6+68*y**7+42*y**8)-280*r**6*(-1+y)**4*(-234+180*logy*(
        -1+y)**5+990*y-1530*y**2+930*y**3-15*y**4-177*y**5-320*y**6+65*y**7
        +35*y**8+10*y**9)-20*r**14*(-20+180*y-720*y**2+1680*y**3-2520*y**4
        +2520*y**5-1932*y**6+828*y**7-207*y**8+23*y**9)-35*r**8*(-1+y)**3*(1700
        -1200*logy*(-1+y)**6-11400*y+32100*y**2-48800*y**3+42600*y**4-20640*y**5
        +1091*y**6+474*y**7+207*y**8+28*y**9)+35*r**4*(-1+y)**5*(1540-1200*logy*(
        -1+y)**4-7360*y+13440*y**2-11360*y**3+4040*y**4-240*y**5+1115*y**6
        -212*y**7-192*y**8-52*y**9+y**10)+14*r**10*(-1+y)**2*(-3670+2100*logy*(
        -1+y)**7+23590*y-63420*y**2+91000*y**3-72625*y**4+28875*y**5+3983*y**6
        -3143*y**7+24*y**8+64*y**9+2*y**10)-2*r**2*(-1+y)**6*(7270+2100*logy*(
        -1+y)**3-23910*y+27060*y**2-11120*y**3+525*y**4+105*y**5-3199*y**6
        +339*y**7+1062*y**8+170*y**9+18*y**10)-7*r**12*(780-7020*y+28080*y**2
        -65520*y**3+98280*y**4-98280*y**5+70189*y**6-34133*y**7+8442*y**8-778*y**9
        -43*y**10+3*y**11))+MBhat**3*(315*r**16*(-1+y)**9-(-1+y)**9*(-2275
        -595*y**4-980*y**5-581*y**6+52*y**7+5*y**8)-70*r**6*(-1+y)**4*(-7167
        +2940*logy*(-1+y)**5+32895*y-58440*y**2+48640*y**3-20470*y**4+46*y**5
        -283*y**6+475*y**7+8*y**8)-105*r**4*(-1+y)**5*(-2494+3720*logy*(-1
        +y)**4+13696*y-27984*y**2+26096*y**3-11259*y**4+316*y**5+140*y**6
        +100*y**7+45*y**8)+35*r**8*(-1+y)**3*(1350+1200*logy*(-1+y)**6-6900*y
        +13650*y**2-12200*y**3+11865*y**4+78*y**5+2026*y**6-1446*y**7+297*y**8
        -20*y**9)-7*r**2*(-1+y)**6*(27950+9000*logy*(-1+y)**3-92850*y+106350*y**2
        -44450*y**3-690*y**4-1326*y**5+1333*y**6+297*y**7+21*y**8+5*y**9)
        -21*r**10*(-1+y)**2*(-650+4550*y-13650*y**2+22750*y**3-32970*y**4
        +20810*y**5-11501*y**6+3819*y**7-625*y**8+27*y**9)+10*r**14*(287-2583*y
        +10332*y**2-24108*y**3+37422*y**4-38430*y**5+26628*y**6-12132*y**7
        +3393*y**8-497*y**9+24*y**10)-7*r**12*(1550-13950*y+55800*y**2-130200*y**3
        +206815*y**4-220759*y**5+157416*y**6-76344*y**7+23361*y**8-3929*y**9
        +230*y**10+10*y**11))+MBhat**4*(700*r**16*(-1+y)**9+2*(-1+y)**9*(-1450
        -700*y**3-1120*y**4-1071*y**5-133*y**6+10*y**7)+70*r**4*(-1+y)**5*(
        -1164+5520*logy*(-1+y)**4+10176*y-26304*y**2+27736*y**3-13924*y**4
        +1095*y**5-10*y**6+190*y**7)+70*r**2*(-1+y)**6*(3540+1200*logy*(-1
        +y)**3-11820*y+13620*y**2-5950*y**3+30*y**4+117*y**5+49*y**6-9*y**7
        +3*y**8)-70*r**6*(-1+y)**4*(5340-1200*logy*(-1+y)**5-25500*y+48000*y**2
        -42250*y**3+20910*y**4-2391*y**5+87*y**6-196*y**7+20*y**8)+70*r**8*(
        -1+y)**3*(-700+4200*y-10500*y**2+11900*y**3-11220*y**4+3537*y**5
        -629*y**6+33*y**7+3*y**8+y**9)-14*r**10*(-1+y)**2*(100-700*y+2100*y**2
        +3850*y**3-1090*y**4+4893*y**5-2947*y**6+445*y**7+309*y**8-130*y**9
        +10*y**10)+5*r**14*(904-8136*y+32544*y**2-77196*y**3+117684*y**4-120330*y**5
        +82866*y**6-37404*y**7+10296*y**8-1459*y**9+63*y**10)-35*r**12*(280
        -2520*y+10080*y**2-24640*y**3+38360*y**4-39942*y**5+28350*y**6-13116*y**7
        +3564*y**8-381*y**9-47*y**10+12*y**11)))/(1050*(-1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm8(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m8 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-15*r**8*(-1+y)**5+5*(-1+y)**5*(3+8*y**3+3*y**4)-24*r**2*(-1+y)**2*(
        -5+15*y-15*y**2-5*y**4+4*y**5)-4*r**6*(30-150*y+300*y**2-250*y**3
        +245*y**4-112*y**5+21*y**6)+6*r**4*(-1+y)*(30*logy*(-1+y)**4+y*(30
        -105*y+110*y**2-55*y**3+14*y**4)))/(480*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-9*r**10*(-1+y)**6+(-1+y)**6*(9+65*y**4+44*y**5)+6*r**4*(-1+y)**2*(
        -120+90*logy*(-1+y)**4+570*y-1035*y**2+870*y**3-300*y**4+116*y**5
        -41*y**6)+3*r**2*(-1+y)**3*(45-135*y+135*y**2-45*y**3+155*y**4-17*y**5
        -47*y**6+5*y**7)-2*r**6*(-1+y)*(360-270*logy*(-1+y)**5-1530*y+2385*y**2
        -1485*y**3+590*y**4+196*y**5-155*y**6+23*y**7)-3*r**8*(-45+270*y
        -675*y**2+900*y**3-445*y**4+414*y**5-167*y**6+28*y**7)+2*MBhat*(70*r**10*(
        -1+y)**6-(-1+y)**6*(90+50*y**3+50*y**4+3*y**5)-2*r**2*(-1+y)**3*(85
        -60*logy*(-1+y)**3-195*y+105*y**2+235*y**3-60*y**4-72*y**5+46*y**6)
        -r**8*(350-2100*y+5250*y**2-7690*y**3+5580*y**4-2127*y**5+302*y**6
        +15*y**7)+2*r**4*(-1+y)**2*(-260+60*logy*(-1+y)**4+1100*y-1770*y**2
        +1300*y**3-715*y**4+447*y**5-197*y**6+23*y**7)+2*r**6*(360-2160*y
        +5400*y**2-7730*y**3+6445*y**4-3162*y**5+1010*y**6-184*y**7+21*y**8))
        -2*MBhat**2*(-1+y)*(15*r**8*(-1+y)**5-5*(-1+y)**5*(3+8*y**3+3*y**4)
        +24*r**2*(-1+y)**2*(-5+15*y-15*y**2-5*y**4+4*y**5)+4*r**6*(30-150*y
        +300*y**2-250*y**3+245*y**4-112*y**5+21*y**6)-6*r**4*(-1+y)*(30*logy*(
        -1+y)**4+y*(30-105*y+110*y**2-55*y**3+14*y**4))))/(960*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (30*MBhat**3*(-1+y)*(70*r**10*(-1+y)**6-(-1+y)**6*(90+50*y**3+50*y**4
        +3*y**5)-2*r**2*(-1+y)**3*(85-60*logy*(-1+y)**3-195*y+105*y**2+235*y**3
        -60*y**4-72*y**5+46*y**6)-r**8*(350-2100*y+5250*y**2-7690*y**3+5580*y**4
        -2127*y**5+302*y**6+15*y**7)+2*r**4*(-1+y)**2*(-260+60*logy*(-1+y)**4
        +1100*y-1770*y**2+1300*y**3-715*y**4+447*y**5-197*y**6+23*y**7)+2*r**6*(360
        -2160*y+5400*y**2-7730*y**3+6445*y**4-3162*y**5+1010*y**6-184*y**7
        +21*y**8))+4*MBhat*(125*r**12*(-1+y)**7-(-1+y)**7*(530+360*y**4+384*y**5
        +55*y**6)+15*r**2*(-1+y)**4*(165+180*logy*(-1+y)**3-675*y+945*y**2
        -495*y**3-250*y**4+174*y**5+12*y**6-20*y**7)-45*r**4*(-1+y)**3*(-195
        +60*logy*(-1+y)**4+840*y-1380*y**2+1040*y**3-200*y**4+5*y**5-27*y**6
        +13*y**7)-9*r**10*(-105+735*y-2205*y**2+3675*y**3-4310*y**4+2471*y**5
        -777*y**6+91*y**7+5*y**8)+5*r**6*(-1+y)**2*(1620-120*logy*(-1+y)**5
        -7980*y+15660*y**2-15260*y**3+8680*y**4-2915*y**5+1307*y**6-343*y**7
        +23*y**8)-15*r**8*(210-1470*y+4410*y**2-7350*y**3+8270*y**4-5891*y**5
        +2397*y**6-641*y**7+65*y**8))+15*MBhat**2*(184*r**12*(-1+y)**7+(-1
        +y)**7*(323+116*y**3+177*y**4+64*y**5+y**6)-10*r**4*(-1+y)**3*(20
        +30*logy*(-1+y)**4-50*y+15*y**2-130*y**3+160*y**4-180*y**5+53*y**6
        +4*y**7)+r**2*(-1+y)**4*(-371-960*logy*(-1+y)**3+2073*y-3513*y**2
        +3471*y**3-1155*y**4+417*y**5-73*y**6+15*y**7)+2*r**6*(-1+y)**2*(1320
        +570*logy*(-1+y)**5-7170*y+15765*y**2-18405*y**3+10650*y**4-3616*y**5
        +573*y**6-153*y**7+28*y**8)+r**10*(1123-7861*y+23583*y**2-40829*y**3
        +41615*y**4-25809*y**5+9163*y**6-1549*y**7+60*y**8)+r**8*(-3085+21595*y
        -64785*y**2+111555*y**3-115815*y**4+74481*y**5-29750*y**6+6362*y**7
        -573*y**8+15*y**9))-15*MBhat**4*(-1+y)**2*(15*r**8*(-1+y)**5-5*(-1
        +y)**5*(3+8*y**3+3*y**4)+24*r**2*(-1+y)**2*(-5+15*y-15*y**2-5*y**4
        +4*y**5)+4*r**6*(30-150*y+300*y**2-250*y**3+245*y**4-112*y**5+21*y**6)
        -6*r**4*(-1+y)*(30*logy*(-1+y)**4+y*(30-105*y+110*y**2-55*y**3
        +14*y**4)))+3*(-10*r**12*(-1+y)**7+(-1+y)**7*(10+134*y**5+109*y**6)
        +30*r**4*(-1+y)**3*(-125+60*logy*(-1+y)**4+560*y-960*y**2+760*y**3
        -250*y**4+78*y**5+39*y**6-36*y**7)+5*r**8*(-1+y)*(750+360*logy*(
        -1+y)**6-4140*y+9270*y**2-10560*y**3+6120*y**4-2482*y**5+9*y**6
        +156*y**7-23*y**8)+10*r**2*(-1+y)**4*(24-72*y+72*y**2-24*y**3+169*y**5
        -75*y**6-33*y**7+11*y**8)-6*r**10*(40-280*y+840*y**2-1400*y**3+1400*y**4
        -459*y**5+399*y**6-141*y**7+21*y**8)-20*r**6*(-1+y)**2*(-240*logy*(
        -1+y)**5+y*(240-1080*y+1880*y**2-1540*y**3+691*y**4-196*y**5+41*y**6))))/(7200*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-75*r**14*(-1+y)**8+(-1+y)**8*(75+1631*y**6+1416*y**7)+105*r**4*(
        -1+y)**4*(-785+300*logy*(-1+y)**4+3440*y-5760*y**2+4440*y**3-1410*y**4
        +60*y**5+676*y**6-108*y**7-145*y**8)+105*r**8*(-1+y)**2*(1125+1500*logy*(
        -1+y)**6-5250*y+8625*y**2-4000*y**3-4500*y**4+6300*y**5-3819*y**6
        +938*y**7-139*y**8)+21*r**10*(-1+y)*(-3925+1500*logy*(-1+y)**7+25975*y
        -72675*y**2+110625*y**3-97500*y**4+48000*y**5-16999*y**6+789*y**7+499*y**8
        -69*y**9)+35*r**6*(-1+y)**3*(3375+4500*logy*(-1+y)**5-21375*y+54000*y**2
        -69000*y**3+45750*y**4-13650*y**5+728*y**6+1808*y**7-853*y**8-15*y**9)
        +21*r**2*(-1+y)**5*(125-375*y+375*y**2-125*y**3+1491*y**6-953*y**7
        -167*y**8+109*y**9)-7*r**12*(-375+3000*y-10500*y**2+21000*y**3-26250*y**4
        +21000*y**5-4928*y**6+4024*y**7-1259*y**8+168*y**9)+210*MBhat**5*(-1
        +y)**2*(70*r**10*(-1+y)**6-(-1+y)**6*(90+50*y**3+50*y**4+3*y**5)-2*r**2*(
        -1+y)**3*(85-60*logy*(-1+y)**3-195*y+105*y**2+235*y**3-60*y**4
        -72*y**5+46*y**6)-r**8*(350-2100*y+5250*y**2-7690*y**3+5580*y**4
        -2127*y**5+302*y**6+15*y**7)+2*r**4*(-1+y)**2*(-260+60*logy*(-1+y)**4
        +1100*y-1770*y**2+1300*y**3-715*y**4+447*y**5-197*y**6+23*y**7)+2*r**6*(360
        -2160*y+5400*y**2-7730*y**3+6445*y**4-3162*y**5+1010*y**6-184*y**7
        +21*y**8))+105*MBhat**4*(-1+y)*(368*r**12*(-1+y)**7+(-1+y)**7*(637
        +232*y**3+289*y**4+84*y**5+2*y**6)+r**2*(-1+y)**4*(-877-1920*logy*(
        -1+y)**3+4551*y-7431*y**2+7077*y**3-2775*y**4+885*y**5-5*y**6+15*y**7)
        -2*r**4*(-1+y)**3*(-160+570*logy*(-1+y)**4+1210*y-2955*y**2+1310*y**3
        +700*y**4-1452*y**5+407*y**6+40*y**7)+2*r**6*(-1+y)**2*(3000+870*logy*(
        -1+y)**5-15870*y+33915*y**2-38295*y**3+21890*y**4-7036*y**5+991*y**6
        -283*y**7+56*y**8)+r**10*(2237-15659*y+46977*y**2-81343*y**3+82915*y**4
        -51429*y**5+18263*y**6-3089*y**7+120*y**8)+r**8*(-6035+42245*y-126735*y**2
        +218385*y**3-227595*y**4+146385*y**5-57757*y**6+12139*y**7-1062*y**8
        +30*y**9))+MBhat**2*(7590*r**14*(-1+y)**8-1260*r**4*(-1+y)**4*(1284-60*logy*(
        -1+y)**4-5196*y+7914*y**2-5396*y**3+909*y**4+342*y**5-251*y**6+64*y**7)
        +(-1+y)**8*(54430+17745*y**4+26796*y**5+11291*y**6+660*y**7)+70*r**6*(
        -1+y)**3*(-10275+10020*logy*(-1+y)**5+41355*y-57660*y**2+24260*y**3
        +14705*y**4-17197*y**5+5158*y**6-1682*y**7+40*y**8)-105*r**2*(-1+y)**5*(5844
        +3840*logy*(-1+y)**3-21372*y+27132*y**2-12884*y**3-1659*y**4+1421*y**5
        +250*y**6-126*y**7-47*y**8+y**9)+105*r**8*(-1+y)**2*(250+2280*logy*(
        -1+y)**6+780*y-8790*y**2+23120*y**3-37515*y**4+23590*y**5-11632*y**6
        +2046*y**7-277*y**8+28*y**9)+70*r**12*(-916+7328*y-25648*y**2+51296*y**3
        -68299*y**4+56000*y**5-29260*y**6+9080*y**7-1405*y**8+60*y**9)+63*r**10*(4140
        -33120*y+115920*y**2-231840*y**3+305135*y**4-261092*y**5+143319*y**6
        -52080*y**7+10521*y**8-908*y**9+5*y**10))+2*MBhat*(650*r**14*(-1+y)**8
        -(-1+y)**8*(5900+4221*y**5+4921*y**6+981*y**7)-70*r**6*(-1+y)**3*(
        -375+5100*logy*(-1+y)**5-3225*y+19200*y**2-36200*y**3+30850*y**4
        -11108*y**5+1685*y**6-577*y**7+110*y**8)+7*r**2*(-1+y)**5*(17750+9000*logy*(
        -1+y)**3-62250*y+75750*y**2-34250*y**3+2250*y**4-10188*y**5+6259*y**6
        +1131*y**7-717*y**8-55*y**9)+315*r**4*(-1+y)**4*(1240-4960*y+7440*y**2
        -4960*y**3+1240*y**4-533*y**5+206*y**6-16*y**7+6*y**8+y**9)+35*r**8*(
        -1+y)**2*(-7500-1200*logy*(-1+y)**6+43800*y-105900*y**2+135200*y**3
        -95400*y**4+42903*y**5-9985*y**6+3396*y**7-597*y**8+23*y**9)-7*r**12*(1000
        -8000*y+28000*y**2-56000*y**3+70000*y**4-68537*y**5+32536*y**6-8756*y**7
        +892*y**8+45*y**9)-63*r**10*(-550+4400*y-15400*y**2+30800*y**3-38500*y**4
        +35458*y**5-22241*y**6+7748*y**7-1868*y**8+146*y**9+7*y**10))+2*MBhat**3*(11430*r**14*(
        -1+y)**8+105*r**2*(-1+y)**5*(2280+2160*logy*(-1+y)**3-9000*y+12240*y**2
        -7368*y**3+839*y**4+91*y**5+13*y**6-55*y**7)-(-1+y)**8*(45940+9590*y**3
        +17955*y**4+13104*y**5+1890*y**6+15*y**7)-210*r**4*(-1+y)**4*(-2387
        +60*logy*(-1+y)**4+9608*y-14532*y**2+11143*y**3-4157*y**4+2020*y**5
        -856*y**6+121*y**7)+70*r**6*(-1+y)**3*(-75-3180*logy*(-1+y)**5+3555*y
        -15060*y**2+23820*y**3-11655*y**4-3993*y**5+7766*y**6-2134*y**7+128*y**8)
        +105*r**8*(-1+y)**2*(-3540+21240*y-53100*y**2+73230*y**3-51775*y**4
        +20332*y**5-1131*y**6-1143*y**7+202*y**8+5*y**9)+70*r**12*(-1085+8680*y
        -30380*y**2+62153*y**3-79114*y**4+65030*y**5-33964*y**6+10532*y**7
        -1627*y**8+69*y**9)-21*r**10*(-10240+81920*y-286720*y**2+588200*y**3
        -750885*y**4+620856*y**5-333368*y**6+106168*y**7-16241*y**8+160*y**9
        +150*y**10))-70*MBhat**6*(-1+y)**3*(15*r**8*(-1+y)**5-5*(-1+y)**5*(3
        +8*y**3+3*y**4)+24*r**2*(-1+y)**2*(-5+15*y-15*y**2-5*y**4+4*y**5)
        +4*r**6*(30-150*y+300*y**2-250*y**3+245*y**4-112*y**5+21*y**6)-6*r**4*(
        -1+y)*(30*logy*(-1+y)**4+y*(30-105*y+110*y**2-55*y**3+14*y**4))))/(33600*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-45*r**16*(-1+y)**9+(-1+y)**9*(45+1472*y**7+1295*y**8)+336*r**10*(
        -1+y)**2*(-1155+900*logy*(-1+y)**7+7185*y-18405*y**2+24375*y**3
        -16500*y**4+3600*y**5+1950*y**6-2058*y**7+461*y**8-53*y**9)+210*r**4*(
        -1+y)**5*(-546+180*logy*(-1+y)**4+2364*y-3906*y**2+2964*y**3-921*y**4
        +36*y**5+6*y**6+612*y**7-289*y**8-44*y**9)+14*r**12*(-1+y)*(8190
        +2700*logy*(-1+y)**8-62820*y+209070*y**2-392940*y**3+453375*y**4
        -324900*y**5+137250*y**6-42428*y**7+2399*y**8+716*y**9-92*y**10)+280*r**6*(
        -1+y)**4*(1386+1080*logy*(-1+y)**5-8010*y+18720*y**2-22320*y**3
        +13860*y**4-3852*y**5+180*y**6+400*y**7+77*y**8-130*y**9-11*y**10)
        -120*r**14*(18-162*y+648*y**2-1512*y**3+2268*y**4-2268*y**5+1512*y**6
        -268*y**7+207*y**8-58*y**9+7*y**10)+48*r**2*(-1+y)**6*(45-135*y+135*y**2
        -45*y**3+816*y**7-628*y**8-37*y**9+59*y**10)+280*MBhat**7*(-1+y)**3*(70*r**10*(
        -1+y)**6-(-1+y)**6*(90+50*y**3+50*y**4+3*y**5)-2*r**2*(-1+y)**3*(85
        -60*logy*(-1+y)**3-195*y+105*y**2+235*y**3-60*y**4-72*y**5+46*y**6)
        -r**8*(350-2100*y+5250*y**2-7690*y**3+5580*y**4-2127*y**5+302*y**6
        +15*y**7)+2*r**4*(-1+y)**2*(-260+60*logy*(-1+y)**4+1100*y-1770*y**2
        +1300*y**3-715*y**4+447*y**5-197*y**6+23*y**7)+2*r**6*(360-2160*y
        +5400*y**2-7730*y**3+6445*y**4-3162*y**5+1010*y**6-184*y**7+21*y**8))
        -210*r**8*(-1+y)**3*(-2700*logy*(-1+y)**6+y*(-2700+14850*y-33300*y**2
        +38475*y**3-23490*y**4+6615*y**5+236*y**6-725*y**7+234*y**8+5*y**9))
        +140*MBhat**6*(-1+y)**2*(552*r**12*(-1+y)**7+(-1+y)**7*(951+348*y**3
        +401*y**4+104*y**5+3*y**6)+3*r**2*(-1+y)**4*(-461-960*logy*(-1+y)**3
        +2343*y-3783*y**2+3561*y**3-1465*y**4+451*y**5+21*y**6+5*y**7)-6*r**4*(
        -1+y)**3*(-140+330*logy*(-1+y)**4+890*y-1995*y**2+1090*y**3+200*y**4
        -668*y**5+183*y**6+20*y**7)+3*r**10*(1117-7819*y+23457*y**2-40619*y**3
        +41405*y**4-25683*y**5+9121*y**6-1543*y**7+60*y**8)+2*r**6*(-1+y)**2*(4680
        +1170*logy*(-1+y)**5-24570*y+52065*y**2-58185*y**3+33130*y**4-10456*y**5
        +1409*y**6-413*y**7+84*y**8)+3*r**8*(-2995+20965*y-62895*y**2+108405*y**3
        -113125*y**4+72763*y**5-28588*y**6+5972*y**7-517*y**8+15*y**9))+8*MBhat**5*(
        -1+y)*(11430*r**14*(-1+y)**8+210*r**4*(-1+y)**4*(1802+120*logy*(-1
        +y)**4-7088*y+10392*y**2-8023*y**3+3557*y**4-2005*y**5+775*y**6-82*y**7)
        -105*r**2*(-1+y)**5*(-1950-1800*logy*(-1+y)**3+7650*y-10350*y**2
        +6378*y**3-1339*y**4+257*y**5+11*y**6+15*y**7)-(-1+y)**8*(38520+9590*y**3
        +12915*y**4+7728*y**5+1120*y**6+15*y**7)+70*r**6*(-1+y)**3*(-1695-3060*logy*(
        -1+y)**5+11535*y-30720*y**2+39080*y**3-20335*y**4-1078*y**5+6459*y**6
        -1791*y**7+105*y**8)+105*r**8*(-1+y)**2*(-3960+23760*y-59400*y**2
        +81630*y**3-59915*y**4+23974*y**5-2283*y**6-1013*y**7+202*y**8+5*y**9)
        +70*r**12*(-1110+8880*y-31080*y**2+63553*y**3-80864*y**4+66430*y**5
        -34664*y**6+10732*y**7-1652*y**8+69*y**9)-21*r**10*(-10870+86960*y
        -304360*y**2+623480*y**3-798795*y**4+661542*y**5-352856*y**6+111376*y**7
        -16757*y**8+130*y**9+150*y**10))+2*MBhat**2*(2415*r**16*(-1+y)**9+(
        -1+y)**9*(43755+13608*y**5+21994*y**6+10464*y**7+981*y**8)+70*r**6*(
        -1+y)**4*(43047+60660*logy*(-1+y)**5-275895*y+703440*y**2-905640*y**3
        +604470*y**4-169674*y**5+152*y**6+8648*y**7-2293*y**8-15*y**9)+210*r**8*(
        -1+y)**3*(19875+10800*logy*(-1+y)**6-108450*y+238725*y**2-264300*y**3
        +144225*y**4-28494*y**5-6972*y**6+1478*y**7-487*y**8+20*y**9)+42*r**2*(
        -1+y)**6*(-33555-14400*logy*(-1+y)**3+115065*y-136665*y**2+59955*y**3
        -3600*y**4+5976*y**5-2145*y**6-2381*y**7+493*y**8+217*y**9)-210*r**4*(
        -1+y)**5*(14763+3660*logy*(-1+y)**4-55392*y+75768*y**2-43192*y**3
        +7138*y**4-3624*y**5+2086*y**6-524*y**7+67*y**8+52*y**9+2*y**10)+90*r**14*(303
        -2727*y+10908*y**2-25452*y**3+38178*y**4-41370*y**5+28308*y**6-12732*y**7
        +3483*y**8-487*y**9+20*y**10)+42*r**10*(-1+y)**2*(-8485+6300*logy*(
        -1+y)**7+53095*y-137235*y**2+184625*y**3-129500*y**4-9096*y**5+32389*y**6
        -28703*y**7+5371*y**8-529*y**9+28*y**10)-14*r**12*(11085-99765*y+399060*y**2
        -931140*y**3+1396710*y**4-1484082*y**5+1081622*y**6-513258*y**7+168687*y**8
        -31463*y**9+2544*y**10))+8*MBhat*(100*r**16*(-1+y)**9-(-1+y)**9*(1620
        +1211*y**6+1481*y**7+354*y**8)-35*r**8*(-1+y)**3*(12400+12000*logy*(
        -1+y)**6-62400*y+120000*y**2-100000*y**3+15000*y**4+30000*y**5-19433*y**6
        +3453*y**7-771*y**8+71*y**9)+r**2*(-1+y)**6*(78080+29400*logy*(-1
        +y)**3-263640*y+307740*y**2-131980*y**3+7350*y**4+1470*y**5-28364*y**6
        +16539*y**7+5277*y**8-2225*y**9-327*y**10)+70*r**6*(-1+y)**4*(-7470
        -5400*logy*(-1+y)**5+42750*y-99000*y**2+117000*y**3-72000*y**4+19800*y**5
        -2050*y**6+175*y**7-65*y**8+65*y**9+3*y**10)-20*r**14*(-73+657*y
        -2628*y**2+6132*y**3-9198*y**4+9198*y**5-7728*y**6+3132*y**7-738*y**8
        +67*y**9+3*y**10)+35*r**4*(-1+y)**5*(4256+1920*logy*(-1+y)**4-15104*y
        +18816*y**2-8704*y**3+256*y**4+384*y**5-2627*y**6+1274*y**7-36*y**8
        +34*y**9+11*y**10)+7*r**10*(-1+y)**2*(23840-4200*logy*(-1+y)**7-162680*y
        +473340*y**2-759500*y**3+722750*y**4-404250*y**5+149192*y**6-26627*y**7
        +7191*y**8-959*y**9+23*y**10)-7*r**12*(1440-12960*y+51840*y**2-120960*y**3
        +181440*y**4-181440*y**5+140503*y**6-78851*y**7+23874*y**8-5266*y**9
        +359*y**10+21*y**11))+8*MBhat**3*(1905*r**16*(-1+y)**9-(-1+y)**9*(31175
        +5285*y**4+10955*y**5+8113*y**6+1684*y**7+55*y**8)-70*r**6*(-1+y)**4*(171
        +16980*logy*(-1+y)**5-17835*y+78120*y**2-134720*y**3+113960*y**4
        -45528*y**5+9139*y**6-2415*y**7+236*y**8)+105*r**4*(-1+y)**5*(14186
        +1320*logy*(-1+y)**4-55424*y+80496*y**2-51024*y**3+8661*y**4+1421*y**5
        -712*y**6-52*y**7+93*y**8+3*y**9)+105*r**8*(-1+y)**3*(-7050-3600*logy*(
        -1+y)**6+38700*y-85950*y**2+96600*y**3-53165*y**4+14837*y**5-1586*y**6
        +2541*y**7-667*y**8+40*y**9)+3*r**2*(-1+y)**6*(203950+105000*logy*(
        -1+y)**3-716850*y+874350*y**2-396450*y**3-3780*y**4+1288*y**5+17591*y**6
        -1426*y**7-2148*y**8-50*y**9+5*y**10)+21*r**10*(-1+y)**2*(7950-55650*y
        +166950*y**2-278250*y**3+300840*y**4-174860*y**5+68257*y**6-9308*y**7
        -410*y**8+156*y**9+5*y**10)+30*r**14*(551-4959*y+19836*y**2-46284*y**3
        +72086*y**4-73962*y**5+51240*y**6-23340*y**7+6525*y**8-955*y**9+46*y**10)
        -7*r**12*(9050-81450*y+325800*y**2-760200*y**3+1188205*y**4-1242368*y**5
        +877587*y**6-426228*y**7+130107*y**8-21848*y**9+1295*y**10+50*y**11))
        +MBhat**4*(34825*r**16*(-1+y)**9+(-1+y)**9*(373975+39200*y**3+102410*y**4
        +108528*y**5+39144*y**6+3040*y**7+15*y**8)+70*r**4*(-1+y)**5*(-139824
        -1680*logy*(-1+y)**4+557616*y-833064*y**2+575056*y**3-143959*y**4
        +10020*y**5+4490*y**6-2660*y**7+45*y**8)-280*r**6*(-1+y)**4*(8385-18300*logy*(
        -1+y)**5-23625*y+1500*y**2+52800*y**3-58135*y**4+13001*y**5+9658*y**6
        -2824*y**7+320*y**8)-140*r**2*(-1+y)**6*(29340+19200*logy*(-1+y)**3
        -107220*y+136020*y**2-68140*y**3+1845*y**4+3501*y**5+184*y**6-528*y**7
        -45*y**8+3*y**9)+140*r**8*(-1+y)**3*(24400+6750*logy*(-1+y)**6-139650*y
        +328875*y**2-404150*y**3+222165*y**4-26874*y**5-55282*y**6+27444*y**7
        -5856*y**8+428*y**9)-28*r**10*(-1+y)**2*(-33700+235900*y-707700*y**2
        +1237100*y**3-1112195*y**4+612969*y**5-71876*y**6-78580*y**7+40347*y**8
        -6545*y**9+200*y**10)+40*r**14*(6083-54747*y+218988*y**2-518952*y**3
        +789768*y**4-805770*y**5+553182*y**6-248508*y**7+67842*y**8-9443*y**9
        +381*y**10)-70*r**12*(9920-89280*y+357120*y**2-851680*y**3+1293755*y**4
        -1314555*y**5+903252*y**6-399756*y**7+101709*y**8-9361*y**9-1400*y**10
        +276*y**11))-70*MBhat**8*(-1+y)**4*(15*r**8*(-1+y)**5-5*(-1+y)**5*(3
        +8*y**3+3*y**4)+24*r**2*(-1+y)**2*(-5+15*y-15*y**2-5*y**4+4*y**5)
        +4*r**6*(30-150*y+300*y**2-250*y**3+245*y**4-112*y**5+21*y**6)-6*r**4*(
        -1+y)*(30*logy*(-1+y)**4+y*(30-105*y+110*y**2-55*y**3+14*y**4))))/(33600*(
        -1+y)**9)
    
    return 0.

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xm9(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^4 level functions of m9 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (60*logy*(-1+y)**5-15*r**8*(-1+y)**5+5*(-1+y)**4*(25-37*y+6*y**2
        +8*y**3+10*y**4)+6*r**2*(-1+y)**2*(-40+120*y-120*y**2+45*y**3-25*y**4
        +18*y**5)+r**6*(-80+400*y-800*y**2+890*y**3-460*y**4+101*y**5-3*y**6)
        +3*r**4*(60-300*y+600*y**2-650*y**3+415*y**4-146*y**5+21*y**6))/(90*(
        -1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-9*r**10*(-1+y)**6+3*r**4*(-1+y)**3*(180-300*logy*(-1+y)**3-240*y
        -210*y**2+370*y**3-145*y**4+31*y**5)-3*(-1+y)**5*(13-13*y-25*y**4
        -16*y**5+y**6)+r**2*(-1+y)**3*(-905-420*logy*(-1+y)**3+3135*y-3765*y**2
        +1675*y**3-210*y**4-102*y**5+148*y**6)+r**8*(85-510*y+1275*y**2-1700*y**3
        +1470*y**4-609*y**5+112*y**6-3*y**7)+3*r**6*(-160+960*y-2400*y**2
        +3200*y**3-2545*y**4+1218*y**5-288*y**6+14*y**7+y**8)+2*MBhat**2*(
        -1+y)*(60*logy*(-1+y)**5-15*r**8*(-1+y)**5+5*(-1+y)**4*(25-37*y
        +6*y**2+8*y**3+10*y**4)+6*r**2*(-1+y)**2*(-40+120*y-120*y**2+45*y**3
        -25*y**4+18*y**5)+r**6*(-80+400*y-800*y**2+890*y**3-460*y**4+101*y**5
        -3*y**6)+3*r**4*(60-300*y+600*y**2-650*y**3+415*y**4-146*y**5+21*y**6))
        +MBhat*(-63*r**10*(-1+y)**6+60*logy*(-2+9*r**2+3*r**4)*(-1+y)**6-(
        -1+y)**5*(193-313*y+60*y**2+170*y**3+115*y**4+15*y**5)+3*r**2*(-1
        +y)**3*(455-1545*y+1815*y**2-705*y**3+50*y**4-78*y**5+24*y**6)-3*r**4*(
        -1+y)**2*(-600+2340*y-3390*y**2+2260*y**3-685*y**4+128*y**5-43*y**6
        +2*y**7)-3*r**8*(-125+750*y-1875*y**2+2630*y**3-2060*y**4+894*y**5
        -184*y**6+10*y**7)+r**6*(-940+5640*y-14100*y**2+19640*y**3-16065*y**4
        +7434*y**5-1750*y**6+138*y**7+3*y**8)))/(180*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-2510-18070*r**2-4300*r**4+22200*r**6+2950*r**8-290*r**10+20*r**12
        +600*logy*(-1+MBhat**4-17*r**2-44*r**4-22*r**6-3*MBhat**2*r**2*(9+8*r**2)
        +MBhat**3*(-2+9*r**2+3*r**4)+MBhat*(2+35*r**2+63*r**4-2*r**6))*(-1
        +y)**7+18170*y+136690*r**2*y+56500*r**4*y-142200*r**6*y-20650*r**8*y
        +2030*r**10*y-140*r**12*y-56610*y**2-445770*r**2*y**2-261900*r**4*y**2
        +380400*r**6*y**2+61950*r**8*y**2-6090*r**10*y**2+420*r**12*y**2+98550*y**3
        +814350*r**2*y**3+621300*r**4*y**3-541600*r**6*y**3-103250*r**8*y**3+10150*r**10*y**3
        -700*r**12*y**3-103800*y**4-903600*r**2*y**4-852300*r**4*y**4+426100*r**6*y**4
        +103250*r**8*y**4-10150*r**10*y**4+700*r**12*y**4+66879*y**5+612435*r**2*y**5
        +696090*r**4*y**5-160230*r**6*y**5-65235*r**8*y**5+7161*r**10*y**5-420*r**12*y**5
        -26534*y**6-235365*r**2*y**6-323100*r**4*y**6-410*r**6*y**6+25780*r**8*y**6
        -2471*r**10*y**6+140*r**12*y**6+9326*y**7+36915*r**2*y**7+68580*r**4*y**7
        +20330*r**6*y**7-4840*r**8*y**7+389*r**10*y**7-20*r**12*y**7-6216*y**8+3105*r**2*y**8
        +1860*r**4*y**8-4410*r**6*y**8+30*r**8*y**8-9*r**10*y**8+3450*y**9+655*r**2*y**9
        -3110*r**4*y**9-180*r**6*y**9+15*r**8*y**9-384*y**10-1885*r**2*y**10+380*r**4*y**10
        -516*y**11+555*r**2*y**11+214*y**12-15*r**2*y**12-19*y**13+10*MBhat**4*(
        -1+y)**2*(-15*r**8*(-1+y)**5+5*(-1+y)**4*(25-37*y+6*y**2+8*y**3
        +10*y**4)+6*r**2*(-1+y)**2*(-40+120*y-120*y**2+45*y**3-25*y**4+18*y**5)
        +r**6*(-80+400*y-800*y**2+890*y**3-460*y**4+101*y**5-3*y**6)+3*r**4*(60
        -300*y+600*y**2-650*y**3+415*y**4-146*y**5+21*y**6))-10*MBhat**3*(
        -1+y)*(63*r**10*(-1+y)**6+(-1+y)**5*(193-313*y+60*y**2+170*y**3
        +115*y**4+15*y**5)-3*r**2*(-1+y)**3*(455-1545*y+1815*y**2-705*y**3
        +50*y**4-78*y**5+24*y**6)+3*r**4*(-1+y)**2*(-600+2340*y-3390*y**2
        +2260*y**3-685*y**4+128*y**5-43*y**6+2*y**7)+3*r**8*(-125+750*y-1875*y**2
        +2630*y**3-2060*y**4+894*y**5-184*y**6+10*y**7)+r**6*(940-5640*y
        +14100*y**2-19640*y**3+16065*y**4-7434*y**5+1750*y**6-138*y**7-3*y**8))
        -2*MBhat*(75*r**12*(-1+y)**7+5*r**2*(-1+y)**4*(-3505+12615*y-15765*y**2
        +7355*y**3-930*y**4+78*y**5+104*y**6)-(-1+y)**6*(2580-3180*y+300*y**2
        +100*y**3-640*y**4-306*y**5-59*y**6+5*y**7)-15*r**4*(-1+y)**3*(745
        -1720*y+60*y**2+2480*y**3-1870*y**4+300*y**5-9*y**6+26*y**7)+10*r**6*(
        -1+y)**2*(1170-5910*y+11970*y**2-12170*y**3+6715*y**4-1775*y**5+169*y**6
        -26*y**7+y**8)+3*r**10*(235-1645*y+4935*y**2-8225*y**3+8820*y**4
        -5572*y**5+2044*y**6-372*y**7+20*y**8)-10*r**8*(340-2380*y+7140*y**2
        -11900*y**3+12440*y**4-8211*y**5+3157*y**6-631*y**7+45*y**8))-10*MBhat**2*(52*r**12*(
        -1+y)**7-(-1+y)**6*(-200+200*y+153*y**3+138*y**4+69*y**5)-15*r**4*(
        -1+y)**4*(236-612*y+468*y**2-46*y**3-53*y**4+7*y**5+2*y**6)-3*r**2*(
        -1+y)**4*(-929+3327*y-4137*y**2+1754*y**3-15*y**4-57*y**5+33*y**6)
        -r**6*(-1+y)**2*(-1220+6100*y-12200*y**2+13010*y**3-6565*y**4+1241*y**5
        +77*y**6-37*y**7+2*y**8)+3*r**10*(115-805*y+2415*y**2-4144*y**3+4270*y**4
        -2688*y**5+980*y**6-176*y**7+9*y**8)-3*r**8*(320-2240*y+6720*y**2
        -11545*y**3+12045*y**4-7651*y**5+2835*y**6-497*y**7+8*y**8+5*y**9)))/(900*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (75260+824670*r**2+995400*r**4-978250*r**6-871500*r**8-48510*r**10
        +3080*r**12-150*r**14+4200*logy*(-4+2*MBhat**6-93*r**2-360*r**4-365*r**6
        -90*r**8-3*MBhat**4*r**2*(47+33*r**2)+3*MBhat**5*(-2+9*r**2+3*r**4)
        +MBhat**3*(20+354*r**2+483*r**4-3*r**6)-3*MBhat**2*(10+162*r**2+335*r**4
        +59*r**6)+MBhat*(18+339*r**2+972*r**4+525*r**6-30*r**8))*(-1+y)**8
        -618880*y-6987960*r**2*y-9475200*r**4*y+6293000*r**6*y+6594000*r**8*y
        +388080*r**10*y-24640*r**12*y+1200*r**14*y+2233280*y**2+26020260*r**2*y**2
        +39211200*r**4*y**2-15893500*r**6*y**2-21567000*r**8*y**2-1358280*r**10*y**2
        +86240*r**12*y**2-4200*r**14*y**2-4623360*y**3-55686120*r**2*y**3-92534400*r**4*y**3
        +17479000*r**6*y**3+39606000*r**8*y**3+2716560*r**10*y**3-172480*r**12*y**3
        +8400*r**14*y**3+6014400*y**4+75076050*r**2*y**4+136836000*r**4*y**4-386750*r**6*y**4
        -44215500*r**8*y**4-3395700*r**10*y**4+215600*r**12*y**4-10500*r**14*y**4
        -5046720*y**5-65529240*r**2*y**5-130636800*r**4*y**5-21152600*r**6*y**5
        +30080400*r**8*y**5+2716560*r**10*y**5-172480*r**12*y**5+8400*r**14*y**5
        +2674469*y**6+36432921*r**2*y**6+79412970*r**4*y**6+24840830*r**6*y**6-11418855*r**8*y**6
        -1424199*r**10*y**6+102704*r**12*y**6-4200*r**14*y**6-793708*y**7-12092808*r**2*y**7
        -28674240*r**4*y**7-13068160*r**6*y**7+1533420*r**8*y**7+484008*r**10*y**7
        -30352*r**12*y**7+1200*r**14*y**7+5480*y**8+2178078*r**2*y**8+5015010*r**4*y**8
        +2927120*r**6*y**8+358470*r**8*y**8-77700*r**10*y**8+4172*r**12*y**8-150*r**14*y**8
        +184968*y**9-385056*r**2*y**9-139440*r**4*y**9+2800*r**6*y**9-87780*r**8*y**9
        -1008*r**10*y**9-84*r**12*y**9-177954*y**10+199290*r**2*y**10+14910*r**4*y**10
        -55370*r**6*y**10-11655*r**8*y**10+189*r**10*y**10+96096*y**11-44016*r**2*y**11
        -28560*r**4*y**11-8120*r**6*y**11-23212*y**12-11382*r**2*y**12+3150*r**4*y**12
        -1960*y**13+5712*r**2*y**13+2117*y**14-399*r**2*y**14-276*y**15+140*MBhat**6*(
        -1+y)**3*(-15*r**8*(-1+y)**5+5*(-1+y)**4*(25-37*y+6*y**2+8*y**3
        +10*y**4)+6*r**2*(-1+y)**2*(-40+120*y-120*y**2+45*y**3-25*y**4+18*y**5)
        +r**6*(-80+400*y-800*y**2+890*y**3-460*y**4+101*y**5-3*y**6)+3*r**4*(60
        -300*y+600*y**2-650*y**3+415*y**4-146*y**5+21*y**6))-210*MBhat**5*(
        -1+y)**2*(63*r**10*(-1+y)**6+(-1+y)**5*(193-313*y+60*y**2+170*y**3
        +115*y**4+15*y**5)-3*r**2*(-1+y)**3*(455-1545*y+1815*y**2-705*y**3
        +50*y**4-78*y**5+24*y**6)+3*r**4*(-1+y)**2*(-600+2340*y-3390*y**2
        +2260*y**3-685*y**4+128*y**5-43*y**6+2*y**7)+3*r**8*(-125+750*y-1875*y**2
        +2630*y**3-2060*y**4+894*y**5-184*y**6+10*y**7)+r**6*(940-5640*y
        +14100*y**2-19640*y**3+16065*y**4-7434*y**5+1750*y**6-138*y**7-3*y**8))
        +210*MBhat**4*(-1+y)*(-104*r**12*(-1+y)**7+(-1+y)**6*(-361+361*y
        +306*y**3+201*y**4+90*y**5+3*y**6)+3*r**4*(-1+y)**4*(2180-5880*y
        +4890*y**2-830*y**3-385*y**4+39*y**5+20*y**6)+r**2*(-1+y)**4*(-4669
        +16827*y-21057*y**2+8849*y**3+120*y**4-240*y**5+50*y**6)+r**6*(-1
        +y)**2*(-2920+14600*y-29200*y**2+30820*y**3-15965*y**4+3301*y**5+109*y**6
        -77*y**7+4*y**8)-3*r**10*(233-1631*y+4893*y**2-8393*y**3+8645*y**4
        -5439*y**5+1981*y**6-355*y**7+18*y**8)+r**8*(2005-14035*y+42105*y**2
        -72245*y**3+75440*y**4-47985*y**5+17731*y**6-3097*y**7+51*y**8+30*y**9))
        -3*MBhat*(390*r**14*(-1+y)**8-210*r**4*(-1+y)**4*(-1716+13344*y-32976*y**2
        +34944*y**3-15216*y**4+1444*y**5+155*y**6+42*y**7+15*y**8)-70*r**6*(
        -1+y)**3*(-17625+77625*y-129000*y**2+94000*y**3-20750*y**4-7202*y**5
        +2405*y**6+227*y**7+80*y**8)-(-1+y)**7*(108040-133240*y+12600*y**2
        +4200*y**3+2100*y**4-7644*y**5-2695*y**6-275*y**7+114*y**8)+35*r**8*(
        -1+y)**2*(-4600+28800*y-75600*y**2+106800*y**3-86100*y**4+41952*y**5
        -10405*y**6+744*y**7-33*y**8+2*y**9)-7*r**2*(-1+y)**5*(127010-448830*y
        +550530*y**2-251310*y**3+16950*y**4+8562*y**5-1651*y**6-639*y**7-147*y**8
        +5*y**9)+28*r**12*(-190+1520*y-5320*y**2+10640*y**3-13300*y**4+11522*y**5
        -6076*y**6+1916*y**7-307*y**8+15*y**9)+21*r**10*(2010-16080*y+56280*y**2
        -112560*y**3+140700*y**4-117236*y**5+64617*y**6-21096*y**7+3576*y**8
        -212*y**9+y**10))-3*MBhat**2*(1430*r**14*(-1+y)**8+70*r**4*(-1+y)**4*(4991
        +136*y-40404*y**2+67136*y**3-35909*y**4+2958*y**5+1196*y**6+76*y**7)
        +35*r**2*(-1+y)**5*(32480-116880*y+146040*y**2-68120*y**3+6789*y**4
        +147*y**5-15*y**6-207*y**7+6*y**8)+(-1+y)**7*(174960-216960*y+21000*y**2
        +7000*y**3-12565*y**4-11697*y**5-3710*y**6-58*y**7+30*y**8)-70*r**6*(
        -1+y)**3*(15315-73035*y+137220*y**2-125420*y**3+55015*y**4-7121*y**5
        -1316*y**6-26*y**7+40*y**8)-35*r**8*(-1+y)**2*(4520-27120*y+67800*y**2
        -90400*y**3+74025*y**4-31920*y**5+6567*y**6-456*y**7-18*y**8+2*y**9)
        +210*r**12*(-65+520*y-1820*y**2+3640*y**3-4746*y**4+3948*y**5-2100*y**6
        +672*y**7-111*y**8+6*y**9)-7*r**10*(-8680+69440*y-243040*y**2+486080*y**3
        -630655*y**4+536666*y**5-290752*y**6+97040*y**7-17143*y**8+1014*y**9
        +30*y**10))-7*MBhat**3*(1530*r**14*(-1+y)**8-15*r**2*(-1+y)**5*(21832
        -79656*y+100896*y**2-46096*y**3+3097*y**4+693*y**5-461*y**6+15*y**7)
        -(-1+y)**7*(53580-65580*y+6000*y**2-3640*y**3-8765*y**4-4659*y**5
        -966*y**6+30*y**7)-30*r**4*(-1+y)**4*(10181-31064*y+27276*y**2+2256*y**3
        -12249*y**4+4080*y**5-532*y**6+172*y**7)+10*r**6*(-1+y)**3*(5885-29605*y
        +59660*y**2-59540*y**3+40105*y**4-19001*y**5+6782*y**6-1558*y**7+56*y**8)
        -15*r**8*(-1+y)**2*(2620-15720*y+39300*y**2-55960*y**3+39355*y**4
        -13194*y**5-573*y**6+1696*y**7-384*y**8+20*y**9)+10*r**12*(-1093+8744*y
        -30604*y**2+62384*y**3-79618*y**4+65660*y**5-34468*y**6+10784*y**7
        -1699*y**8+78*y**9)-3*r**10*(-10640+85120*y-297920*y**2+610560*y**3
        -782445*y**4+643272*y**5-335496*y**6+100536*y**7-12297*y**8-960*y**9
        +270*y**10)))/(12600*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-60465-1018720*r**2-2594760*r**4+157080*r**6+2586500*r**8+903840*r**10
        +27720*r**12-1240*r**14+45*r**16+4200*logy*(-3+MBhat**8-98*r**2-564*r**4
        -948*r**6-520*r**8-78*r**10-2*MBhat**6*r**2*(67+42*r**2)+2*MBhat**7*(
        -2+9*r**2+3*r**4)+MBhat**5*(28+498*r**2+588*r**4+6*r**6)-2*MBhat**4*(35
        +555*r**2+1008*r**4+135*r**6)+MBhat**3*(84+1510*r**2+3834*r**4+1482*r**6
        -60*r**8)-MBhat**2*(56+1218*r**2+4152*r**4+3046*r**6+165*r**8)+MBhat*(20
        +534*r**2+2388*r**4+2776*r**6+720*r**8-42*r**10))*(-1+y)**9+556785*y
        +9580080*r**2*y+25721640*r**4*y+2567880*r**6*y-21094500*r**8*y-7806960*r**10*y
        -249480*r**12*y+11160*r**14*y-405*r**16*y-2283840*y**2-40172520*r**2*y**2
        -113546160*r**4*y**2-28188720*r**6*y**2+74550000*r**8*y**2+29753640*r**10*y**2
        +997920*r**12*y**2-44640*r**14*y**2+1620*r**16*y**2+5480160*y**3+98675080*r**2*y**3
        +293366640*r**4*y**3+113552880*r**6*y**3-147742000*r**8*y**3-65493960*r**10*y**3
        -2328480*r**12*y**3+104160*r**14*y**3-3780*r**16*y**3-8484840*y**4-156656220*r**2*y**4
        -489794760*r**4*y**4-253942920*r**6*y**4+175749000*r**8*y**4+91361340*r**10*y**4
        +3492720*r**12*y**4-156240*r**14*y**4+5670*r**16*y**4+8802360*y**5+167028540*r**2*y**5
        +549488520*r**4*y**5+354279240*r**6*y**5-120712200*r**8*y**5-83105820*r**10*y**5
        -3492720*r**12*y**5+156240*r**14*y**5-5670*r**16*y**5-6132840*y**6-119995960*r**2*y**6
        -416070480*r**4*y**6-319799760*r**6*y**6+34610800*r**8*y**6+48524280*r**10*y**6
        +2328480*r**12*y**6-104160*r**14*y**6+3780*r**16*y**6+2782416*y**7+56351616*r**2*y**7
        +206761800*r**4*y**7+184851240*r**6*y**7+11300520*r**8*y**7-16778496*r**10*y**7
        -1042776*r**12*y**7+53640*r**14*y**7-1620*r**16*y**7-774534*y**8-15846024*r**2*y**8
        -62454630*r**4*y**8-64208340*r**6*y**8-12397560*r**8*y**8+2525124*r**10*y**8
        +311934*r**12*y**8-13860*r**14*y**8+405*r**16*y**8+176886*y**9+1900996*r**2*y**9
        +9536520*r**4*y**9+11285610*r**6*y**9+3221540*r**8*y**9+191604*r**10*y**9
        -44436*r**12*y**9+1690*r**14*y**9-45*r**16*y**9-164064*y**10+408156*r**2*y**10
        -506310*r**4*y**10-589470*r**6*y**10-11760*r**8*y**10-64176*r**10*y**10-966*r**12*y**10
        -30*r**14*y**10+205296*y**11-427644*r**2*y**11+135240*r**4*y**11+58380*r**6*y**11
        -45360*r**8*y**11-10416*r**10*y**11+84*r**12*y**11-165816*y**12+232904*r**2*y**12
        -43890*r**4*y**12-15120*r**6*y**12-14980*r**8*y**12+82824*y**13-65856*r**2*y**13
        +840*r**4*y**13-8190*r**6*y**13-22296*y**14+3924*r**2*y**14-210*r**4*y**14
        +210*r**6*y**14+1344*y**15+1924*r**2*y**15+759*y**16-276*r**2*y**16-135*y**17
        +70*MBhat**8*(-1+y)**4*(-15*r**8*(-1+y)**5+5*(-1+y)**4*(25-37*y+6*y**2
        +8*y**3+10*y**4)+6*r**2*(-1+y)**2*(-40+120*y-120*y**2+45*y**3-25*y**4
        +18*y**5)+r**6*(-80+400*y-800*y**2+890*y**3-460*y**4+101*y**5-3*y**6)
        +3*r**4*(60-300*y+600*y**2-650*y**3+415*y**4-146*y**5+21*y**6))-140*MBhat**7*(
        -1+y)**3*(63*r**10*(-1+y)**6+(-1+y)**5*(193-313*y+60*y**2+170*y**3
        +115*y**4+15*y**5)-3*r**2*(-1+y)**3*(455-1545*y+1815*y**2-705*y**3
        +50*y**4-78*y**5+24*y**6)+3*r**4*(-1+y)**2*(-600+2340*y-3390*y**2
        +2260*y**3-685*y**4+128*y**5-43*y**6+2*y**7)+3*r**8*(-125+750*y-1875*y**2
        +2630*y**3-2060*y**4+894*y**5-184*y**6+10*y**7)+r**6*(940-5640*y
        +14100*y**2-19640*y**3+16065*y**4-7434*y**5+1750*y**6-138*y**7-3*y**8))
        +140*MBhat**6*(-1+y)**2*(-156*r**12*(-1+y)**7+r**2*(-1+y)**4*(-6551+23673*y
        -29703*y**2+12436*y**3+285*y**4-309*y**5+y**6)+3*(-1+y)**6*(-174
        +174*y+153*y**3+88*y**4+37*y**5+2*y**6)+3*r**4*(-1+y)**4*(3180-8700*y
        +7440*y**2-1430*y**3-505*y**4+43*y**5+30*y**6)+3*r**6*(-1+y)**2*(-1540
        +7700*y-15400*y**2+16210*y**3-8455*y**4+1787*y**5+47*y**6-39*y**7
        +2*y**8)-9*r**10*(117-819*y+2457*y**2-4214*y**3+4340*y**4-2730*y**5
        +994*y**6-178*y**7+9*y**8)+r**8*(3050-21350*y+64050*y**2-109855*y**3
        +114745*y**4-73017*y**5+26957*y**6-4703*y**7+78*y**8+45*y**9))-14*MBhat**5*(
        -1+y)*(1530*r**14*(-1+y)**8+(-1+y)**7*(-38100+46500*y-4200*y**2
        +4240*y**3+4925*y**4+2823*y**5+612*y**6)-15*r**2*(-1+y)**5*(14822-54426*y
        +69366*y**2-31386*y**3+1237*y**4+849*y**5-253*y**6+15*y**7)-30*r**4*(
        -1+y)**4*(7946-25904*y+27096*y**2-5184*y**3-6639*y**4+3180*y**5-505*y**6
        +94*y**7)+10*r**6*(-1+y)**3*(-1135+5855*y-12160*y**2+13480*y**3-185*y**4
        -8351*y**5+5768*y**6-1402*y**7+50*y**8)-15*r**8*(-1+y)**2*(3980-23880*y
        +59700*y**2-83160*y**3+61915*y**4-23478*y**5+1771*y**6+1516*y**7-384*y**8
        +20*y**9)+20*r**12*(-569+4552*y-15932*y**2+32452*y**3-41384*y**4+34090*y**5
        -17864*y**6+5572*y**7-872*y**8+39*y**9)-3*r**10*(-12050+96400*y-337400*y**2
        +689520*y**3-884715*y**4+729624*y**5-381192*y**6+115032*y**7-14649*y**8
        -840*y**9+270*y**10))-MBhat**2*(1365*r**16*(-1+y)**9+140*r**6*(-1+y)**4*(
        -107509+446165*y-663880*y**2+359280*y**3+48810*y**4-101772*y**5+13901*y**6
        +3929*y**7+476*y**8)-42*r**2*(-1+y)**6*(-246710+861930*y-1044630*y**2
        +470010*y**3-30450*y**4-11046*y**5-665*y**6+731*y**7+267*y**8+3*y**9)
        +420*r**4*(-1+y)**5*(-19554+119736*y-262644*y**2+258136*y**3-106054*y**4
        +9081*y**5+907*y**6+482*y**7+69*y**8+9*y**9)-210*r**8*(-1+y)**3*(-23800
        +139500*y-338850*y**2+435300*y**3-309975*y**4+118164*y**5-17098*y**6
        -1028*y**7-63*y**8+30*y**9)+(-1+y)**8*(1040965-1276165*y+117600*y**2
        +39200*y**3+19600*y**4-28308*y**5-27146*y**6-3970*y**7+453*y**8+171*y**9)
        -42*r**10*(-1+y)**2*(-12730+89110*y-267330*y**2+445550*y**3-445550*y**4
        +292446*y**5-106669*y**6+19483*y**7-1461*y**8-11*y**9+2*y**10)+180*r**14*(99
        -891*y+3564*y**2-8316*y**3+12474*y**4-13104*y**5+9114*y**6-4176*y**7
        +1179*y**8-176*y**9+9*y**10)-28*r**12*(4190-37710*y+150840*y**2-351960*y**3
        +527940*y**4-547911*y**5+391671*y**6-182844*y**7+53991*y**8-8759*y**9
        +552*y**10))-2*MBhat*(180*r**16*(-1+y)**9-105*r**8*(-1+y)**3*(34900
        -195000*y+444300*y**2-520400*y**3+318300*y**4-84120*y**5-2297*y**6
        +3192*y**7+351*y**8+54*y**9)-140*r**6*(-1+y)**4*(-21512+65920*y-27740*y**2
        -111060*y**3+159630*y**4-73566*y**5+6700*y**6+1165*y**7+355*y**8+60*y**9)
        -(-1+y)**8*(193760-235760*y+21000*y**2+7000*y**3+3500*y**4+2100*y**5
        -6937*y**6-1865*y**7+264*y**8+138*y**9)+105*r**4*(-1+y)**5*(38332-201088*y
        +397152*y**2-360288*y**3+137832*y**4-9552*y**5-2189*y**6-160*y**7-140*y**8
        -20*y**9+y**10)+42*r**10*(-1+y)**2*(2530-19810*y+66780*y**2-126000*y**3
        +144375*y**4-101325*y**5+46819*y**6-11464*y**7+727*y**8+7*y**9+y**10)
        +60*r**14*(56-504*y+2016*y**2-4704*y**3+7056*y**4-7056*y**5+5124*y**6
        -2316*y**7+639*y**8-91*y**9+4*y**10)-6*r**2*(-1+y)**6*(421230-1450590*y
        +1730940*y**2-763880*y**3+46725*y**4+9345*y**5+10213*y**6-2658*y**7
        -699*y**8-85*y**9+19*y**10)+7*r**12*(-5660+50940*y-203760*y**2+475440*y**3
        -713160*y**4+713160*y**5-493311*y**6+233367*y**7-66258*y**8+9722*y**9
        -483*y**10+3*y**11))-2*MBhat**3*(1785*r**16*(-1+y)**9-210*r**6*(-1+y)**4*(
        -33321+151785*y-266520*y**2+217120*y**3-71410*y**4-2662*y**5+4441*y**6
        -45*y**7+84*y**8)-105*r**4*(-1+y)**5*(-5726+99584*y-302736*y**2+355184*y**3
        -163281*y**4+13604*y**5+3300*y**6+440*y**7+135*y**8)-3*(-1+y)**8*(250335
        -309135*y+29400*y**2+9800*y**3-3465*y**4-11095*y**5-4347*y**6-759*y**7
        +61*y**8+5*y**9)+35*r**8*(-1+y)**3*(350+1500*y-14550*y**2+37400*y**3
        -34065*y**4+34782*y**5-18806*y**6+5076*y**7-897*y**8+10*y**9)-7*r**2*(
        -1+y)**6*(816150-2901450*y+3580950*y**2-1646650*y**3+132390*y**4+25386*y**5
        -1133*y**6-1857*y**7-441*y**8+15*y**9)-21*r**10*(-1+y)**2*(-6530+45710*y
        -137130*y**2+228550*y**3-255330*y**4+154690*y**5-54859*y**6+7701*y**7
        +985*y**8-367*y**9+20*y**10)+10*r**14*(1723-15507*y+62028*y**2-144732*y**3
        +223398*y**4-229950*y**5+159852*y**6-73188*y**7+20637*y**8-3073*y**9
        +156*y**10)-21*r**12*(3450-31050*y+124200*y**2-289800*y**3+449295*y**4
        -469567*y**5+329448*y**6-154032*y**7+44433*y**8-6637*y**9+230*y**10
        +30*y**11))-2*MBhat**4*(3675*r**16*(-1+y)**9+210*r**4*(-1+y)**5*(9028
        -15952*y-16392*y**2+52148*y**3-33932*y**4+4655*y**5+465*y**6+190*y**7)
        +105*r**2*(-1+y)**6*(35570-128910*y+162210*y**2-75370*y**3+6285*y**4
        +639*y**5-47*y**6-103*y**7+6*y**8)+(-1+y)**8*(615075-762075*y+73500*y**2
        +7700*y**3-39515*y**4-30387*y**5-10479*y**6-879*y**7+60*y**8)-70*r**6*(
        -1+y)**4*(37085-177325*y+334400*y**2-309050*y**3+144565*y**4-29744*y**5
        +4013*y**6-1924*y**7+260*y**8)+105*r**8*(-1+y)**3*(-1000+6000*y-15000*y**2
        +18800*y**3-22395*y**4+16452*y**5-9324*y**6+3668*y**7-747*y**8+46*y**9)
        -21*r**10*(-1+y)**2*(-4550+31850*y-95550*y**2+170950*y**3-154465*y**4
        +78253*y**5-7147*y**6-12915*y**7+6664*y**8-1210*y**9+60*y**10)+15*r**14*(1842
        -16578*y+66312*y**2-156828*y**3+238812*y**4-243810*y**5+167538*y**6
        -75372*y**7+20628*y**8-2887*y**9+119*y**10)-35*r**12*(2360-21240*y
        +84960*y**2-202440*y**3+308490*y**4-312732*y**5+212226*y**6-91908*y**7
        +22167*y**8-1478*y**9-483*y**10+78*y**11)))/(6300*(-1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr1(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r1 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-35085*r**8*(-1+y)**7+300*logy*(4+9*r**4)*(-1+y)**7+5*(-1+y)**5*(
        -1127+2494*y-1487*y**2+1184*y**3-1435*y**4+662*y**5+45*y**6)+24*r**2*(
        -1+y)**3*(-175+700*y-1050*y**2-170*y**3+1705*y**4-1514*y**5+432*y**6)
        -20*r**6*(2930-20510*y+61530*y**2-110206*y**3+126235*y**4-94137*y**5
        +43841*y**6-11591*y**7+1332*y**8)+6*r**4*(2280-16410*y+50805*y**2
        -99905*y**3+135020*y**4-122850*y**5+71087*y**6-23339*y**7+3312*y**8))/(1800*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (32531-14645*r**2-4320*r**4-61520*r**6+69005*r**8-21051*r**10+60*logy*(
        -448-140*r**2-165*r**4+135*r**6+24*MBhat*(17+5*r**2)+10*MBhat**2*(4
        +9*r**4))*(-1+y)**8-287128*y+108760*r**2*y+24660*r**4*y+500260*r**6*y
        -552040*r**8*y+168408*r**10*y+1112468*y**2-347060*r**2*y**2-46710*r**4*y**2
        -1783310*r**6*y**2+1932140*r**8*y**2-589428*r**10*y**2-2475816*y**3+615720*r**2*y**3
        +1020*r**4*y**3+3642220*r**6*y**3-3864280*r**8*y**3+1178856*r**10*y**3+3472725*y**4
        -677565*r**2*y**4+252120*r**4*y**4-5008160*r**6*y**4+5129420*r**8*y**4-1473570*r**10*y**4
        -3162576*y**5+546288*r**2*y**5-725844*r**4*y**5+4971340*r**6*y**5-4633240*r**8*y**5
        +1178856*r**10*y**5+1849128*y**6-461664*r**2*y**6+1049436*r**4*y**6-3580604*r**6*y**6
        +2862560*r**8*y**6-589428*r**10*y**6-653088*y**7+447504*r**2*y**7-887868*r**4*y**7
        +1799092*r**6*y**7-1138840*r**8*y**7+168408*r**10*y**7+111141*y**8-346191*r**2*y**8
        +448248*r**4*y**8-572192*r**6*y**8+262235*r**8*y**8-21051*r**10*y**8+13720*y**9
        +169784*r**2*y**9-126336*r**4*y**9+99504*r**6*y**9-26640*r**8*y**9-23660*y**10
        -45964*r**2*y**10+15594*r**4*y**10-6630*r**6*y**10+14920*y**11+4808*r**2*y**11
        -5085*y**12+225*r**2*y**12+720*y**13+2*MBhat**2*(-1+y)*(-35085*r**8*(
        -1+y)**7+5*(-1+y)**5*(-1127+2494*y-1487*y**2+1184*y**3-1435*y**4
        +662*y**5+45*y**6)+24*r**2*(-1+y)**3*(-175+700*y-1050*y**2-170*y**3
        +1705*y**4-1514*y**5+432*y**6)-20*r**6*(2930-20510*y+61530*y**2-110206*y**3
        +126235*y**4-94137*y**5+43841*y**6-11591*y**7+1332*y**8)+6*r**4*(2280
        -16410*y+50805*y**2-99905*y**3+135020*y**4-122850*y**5+71087*y**6
        -23339*y**7+3312*y**8))-6*MBhat*(17864*r**10*(-1+y)**8+(-1+y)**6*(1016
        -6112*y+7136*y**2+230*y**3-1980*y**4+645*y**5+170*y**6+15*y**7)-4*r**2*(
        -1+y)**4*(700-2500*y+3150*y**2+1825*y**3-7720*y**4+7626*y**5-3306*y**6
        +513*y**7)-2*r**4*(-1+y)**2*(-860+5160*y-12900*y**2+440*y**3+23060*y**4
        -22549*y**5+278*y**6+10300*y**7-5954*y**8+1105*y**9)+5*r**8*(-8368
        +66944*y-234304*y**2+488546*y**3-655064*y**4+582535*y**5-340424*y**6
        +125332*y**7-26192*y**8+2339*y**9)-20*r**6*(-1202+9616*y-33656*y**2
        +72427*y**3-102474*y**4+95247*y**5-55610*y**6+17525*y**7-1120*y**8
        -975*y**9+222*y**10)))/(3600*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-128778+58772*r**2+45850*r**4+4400*r**6+39850*r**8-24772*r**10+4678*r**12
        +60*logy*(-804-2084*r**2-790*r**4-200*r**6+90*r**8+24*MBhat**3*(17
        +5*r**2)+5*MBhat**4*(4+9*r**4)+8*MBhat*(257+355*r**2+90*r**4)+3*MBhat**2*(
        -560-292*r**2-25*r**4+95*r**6))*(-1+y)**9+1207242*y-403908*r**2*y
        -365250*r**4*y-27600*r**6*y-364050*r**8*y+222948*r**10*y-42102*r**12*y
        -5046048*y**2+1052952*r**2*y**2+1247700*r**4*y**2+56400*r**6*y**2+1480500*r**8*y**2
        -891792*r**10*y**2+168408*r**12*y**2+12352992*y**3-956408*r**2*y**3-2342500*r**4*y**3
        +12400*r**6*y**3-3519300*r**8*y**3+2080848*r**10*y**3-392952*r**12*y**3-19542528*y**4
        -1191228*r**2*y**4+2518350*r**4*y**4-270600*r**6*y**4+5392350*r**8*y**4-3121272*r**10*y**4
        +589428*r**12*y**4+20756400*y**5+4350324*r**2*y**5-1372158*r**4*y**5+755256*r**6*y**5
        -5831070*r**8*y**5+3283632*r**10*y**5-589428*r**12*y**5-14837809*y**6-5569272*r**2*y**6
        +117582*r**4*y**6-1318028*r**6*y**6+4693827*r**8*y**6-2433036*r**10*y**6
        +392952*r**12*y**6+6895401*y**7+3990720*r**2*y**7+155634*r**4*y**7+1480668*r**6*y**7
        -2831643*r**8*y**7+1274004*r**10*y**7-168408*r**12*y**7-1856934*y**8-1704372*r**2*y**8
        +137592*r**4*y**8-1047792*r**6*y**8+1238022*r**8*y**8-439056*r**10*y**8+42102*r**12*y**8
        +147126*y**9+441268*r**2*y**9-240928*r**4*y**9+450368*r**6*y**9-349918*r**8*y**9
        +88744*r**10*y**9-4678*r**12*y**9+85434*y**10-83016*r**2*y**10+124830*r**4*y**10
        -106428*r**6*y**10+54747*r**8*y**10-7992*r**10*y**10-39258*y**11+13560*r**2*y**11
        -29550*r**4*y**11+10956*r**6*y**11-3315*r**8*y**11+4872*y**12+2588*r**2*y**12
        +2848*r**4*y**12+3192*y**13-2340*r**2*y**13-1473*y**14+360*r**2*y**14+169*y**15
        +MBhat**4*(-1+y)**2*(-35085*r**8*(-1+y)**7+5*(-1+y)**5*(-1127+2494*y
        -1487*y**2+1184*y**3-1435*y**4+662*y**5+45*y**6)+24*r**2*(-1+y)**3*(
        -175+700*y-1050*y**2-170*y**3+1705*y**4-1514*y**5+432*y**6)-20*r**6*(2930
        -20510*y+61530*y**2-110206*y**3+126235*y**4-94137*y**5+43841*y**6
        -11591*y**7+1332*y**8)+6*r**4*(2280-16410*y+50805*y**2-99905*y**3
        +135020*y**4-122850*y**5+71087*y**6-23339*y**7+3312*y**8))-6*MBhat**3*(
        -1+y)*(17864*r**10*(-1+y)**8+(-1+y)**6*(1016-6112*y+7136*y**2+230*y**3
        -1980*y**4+645*y**5+170*y**6+15*y**7)-4*r**2*(-1+y)**4*(700-2500*y
        +3150*y**2+1825*y**3-7720*y**4+7626*y**5-3306*y**6+513*y**7)-2*r**4*(
        -1+y)**2*(-860+5160*y-12900*y**2+440*y**3+23060*y**4-22549*y**5+278*y**6
        +10300*y**7-5954*y**8+1105*y**9)+5*r**8*(-8368+66944*y-234304*y**2
        +488546*y**3-655064*y**4+582535*y**5-340424*y**6+125332*y**7-26192*y**8
        +2339*y**9)-20*r**6*(-1202+9616*y-33656*y**2+72427*y**3-102474*y**4
        +95247*y**5-55610*y**6+17525*y**7-1120*y**8-975*y**9+222*y**10))-4*MBhat*(6380*r**12*(
        -1+y)**9+2*(-1+y)**7*(32789-80998*y+55919*y**2-5140*y**3-430*y**4
        -1321*y**5-48*y**6+39*y**7+30*y**8)+2*r**2*(-1+y)**5*(-22465+68560*y
        -60240*y**2-2440*y**3+17485*y**4+1692*y**5-3994*y**6+388*y**7+150*y**8)
        -3*r**4*(-1+y)**3*(7900-43800*y+98700*y**2-113600*y**3+53670*y**4
        +8893*y**5-20410*y**6+8076*y**7-1420*y**8+71*y**9)-r**6*(-1+y)**2*(
        -6600+46200*y-138600*y**2+231000*y**3-374970*y**4+400623*y**5-247751*y**6
        +60566*y**7+6384*y**8-6757*y**9+1105*y**10)+3*r**10*(7826-70434*y
        +281736*y**2-657384*y**3+1031176*y**4-1114713*y**5+839985*y**6-430260*y**7
        +142650*y**8-27545*y**9+2339*y**10)-5*r**8*(5350-48150*y+192600*y**2
        -449400*y**3+723264*y**4-837747*y**5+691731*y**6-395214*y**7+146376*y**8
        -31969*y**9+3159*y**10))-3*MBhat**2*(26584*r**12*(-1+y)**9-5*(-1+y)**7*(9299
        -25318*y+19379*y**2-1600*y**3-1203*y**4+70*y**5-18*y**6+62*y**7+y**8)
        -r**2*(-1+y)**5*(-36461+128324*y-157446*y**2+49684*y**3+37934*y**4
        -36732*y**5+12778*y**6-1612*y**7+75*y**8)+2*r**4*(-1+y)**3*(2860-16410*y
        +38775*y**2-75710*y**3+78725*y**4-45374*y**5+14621*y**6-3338*y**7-109*y**8
        +200*y**9)-2*r**6*(-1+y)**2*(-2640+15630*y-36915*y**2+121735*y**3
        -104185*y**4-78007*y**5+249239*y**6-224467*y**7+107205*y**8-27515*y**9
        +2960*y**10)+r**10*(76685-690165*y+2760660*y**2-6621940*y**3+10361700*y**4
        -11001924*y**5+7977900*y**6-3878460*y**7+1199295*y**8-209815*y**9+15312*y**10)
        -5*r**8*(11939-107451*y+429804*y**2-1063324*y**3+1718598*y**4-1838918*y**5
        +1277177*y**6-522705*y**7+79005*y**8+29835*y**9-16299*y**10+2339*y**11)))/(1800*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (4953145+5802489*r**2-6249915*r**4-4058075*r**6-27825*r**8-656145*r**10
        +271411*r**12-35085*r**14+420*logy*(-3440-22596*r**2-17055*r**4-6175*r**6
        -675*r**8+225*r**10+72*MBhat**5*(17+5*r**2)+10*MBhat**6*(4+9*r**4)
        +16*MBhat**3*(1369+1788*r**2+315*r**4-195*r**6)+24*MBhat*(631+2717*r**2
        +1364*r**4+200*r**6)+3*MBhat**4*(-2912-1612*r**2+15*r**4+435*r**6)
        +6*MBhat**2*(-4356-11124*r**2-3476*r**4+445*r**6+285*r**8))*(-1+y)**10
        -50976250*y-67515210*r**2*y+55336050*r**4*y+37987250*r**6*y-5250*r**8*y
        +6655950*r**10*y-2714110*r**12*y+350850*r**14*y+236617125*y**2+351270045*r**2*y**2
        -213196725*r**4*y**2-157975125*r**6*y**2+1441125*r**8*y**2-30424275*r**10*y**2
        +12213495*r**12*y**2-1578825*r**14*y**2-652651000*y**3-1079074920*r**2*y**3
        +461078100*r**4*y**3+382364500*r**6*y**3-8095500*r**8*y**3+82548900*r**10*y**3
        -32569320*r**12*y**3+4210200*r**14*y**3+1185483250*y**4+2173090710*r**2*y**4
        -591993675*r**4*y**4-591332875*r**6*y**4+22672125*r**8*y**4-147295575*r**10*y**4
        +56996310*r**12*y**4-7367850*r**14*y**4-1483261500*y**5-3006302292*r**2*y**5
        +409542210*r**4*y**5+600672450*r**6*y**5-39113550*r**8*y**5+180723690*r**10*y**5
        -68395572*r**12*y**5+8841420*r**14*y**5+1296794401*y**6+2903712987*r**2*y**6
        -39841725*r**4*y**6-394310245*r**6*y**6+50885100*r**8*y**6-161388906*r**10*y**6
        +59584098*r**12*y**6-7367850*r**14*y**6-784901518*y**7-1943069226*r**2*y**7
        -195030024*r**4*y**7+157635688*r**6*y**7-56239890*r**8*y**7+109437090*r**10*y**7
        -37359840*r**12*y**7+4210200*r**14*y**7+318131400*y**8+868163142*r**2*y**8
        +186869151*r**4*y**8-36711633*r**6*y**8+48998355*r**8*y**8-56719215*r**10*y**8
        +16943535*r**12*y**8-1578825*r**14*y**8-83031010*y**9-235001970*r**2*y**9
        -85254330*r**4*y**9+10474870*r**6*y**9-29782200*r**8*y**9+21938280*r**10*y**9
        -5142550*r**12*y**9+350850*r**14*y**9+17967175*y**10+26263923*r**2*y**10
        +22134945*r**4*y**10-7410655*r**6*y**10+11449935*r**8*y**10-5558595*r**10*y**10
        +924511*r**12*y**10-35085*r**14*y**10-9335172*y**11+4883844*r**2*y**11-4244940*r**4*y**11
        +3236100*r**6*y**11-2400090*r**8*y**11+780570*r**10*y**11-74592*r**12*y**11
        +6489406*y**12-2894514*r**2*y**12+1044519*r**4*y**12-619913*r**6*y**12+217665*r**8*y**12
        -41769*r**10*y**12-3005800*y**13+730128*r**2*y**13-204666*r**4*y**13+49238*r**6*y**13
        +840435*y**14-32235*r**2*y**14+11025*r**4*y**14-1575*r**6*y**14-118870*y**15
        -30450*r**2*y**15+5071*y**16+3549*r**2*y**16-288*y**17+14*MBhat**6*(-1
        +y)**3*(-35085*r**8*(-1+y)**7+5*(-1+y)**5*(-1127+2494*y-1487*y**2
        +1184*y**3-1435*y**4+662*y**5+45*y**6)+24*r**2*(-1+y)**3*(-175+700*y
        -1050*y**2-170*y**3+1705*y**4-1514*y**5+432*y**6)-20*r**6*(2930-20510*y
        +61530*y**2-110206*y**3+126235*y**4-94137*y**5+43841*y**6-11591*y**7
        +1332*y**8)+6*r**4*(2280-16410*y+50805*y**2-99905*y**3+135020*y**4
        -122850*y**5+71087*y**6-23339*y**7+3312*y**8))-126*MBhat**5*(-1+y)**2*(17864*r**10*(
        -1+y)**8+(-1+y)**6*(1016-6112*y+7136*y**2+230*y**3-1980*y**4+645*y**5
        +170*y**6+15*y**7)-4*r**2*(-1+y)**4*(700-2500*y+3150*y**2+1825*y**3
        -7720*y**4+7626*y**5-3306*y**6+513*y**7)-2*r**4*(-1+y)**2*(-860+5160*y
        -12900*y**2+440*y**3+23060*y**4-22549*y**5+278*y**6+10300*y**7-5954*y**8
        +1105*y**9)+5*r**8*(-8368+66944*y-234304*y**2+488546*y**3-655064*y**4
        +582535*y**5-340424*y**6+125332*y**7-26192*y**8+2339*y**9)-20*r**6*(
        -1202+9616*y-33656*y**2+72427*y**3-102474*y**4+95247*y**5-55610*y**6
        +17525*y**7-1120*y**8-975*y**9+222*y**10))+21*MBhat**4*(-1+y)*(-159504*r**12*(
        -1+y)**9+(-1+y)**7*(246439-667598*y+508519*y**2-39040*y**3-35485*y**4
        +2570*y**5+225*y**6+1140*y**7+30*y**8)+r**2*(-1+y)**5*(-204121+719764*y
        -886206*y**2+275924*y**3+250264*y**4-258900*y**5+101150*y**6-15380*y**7
        +225*y**8)-6*r**4*(-1+y)**3*(5000-30150*y+75825*y**2-157370*y**3+189295*y**4
        -142082*y**5+69635*y**6-22534*y**7+2381*y**8+400*y**9)+2*r**6*(-1+y)**2*(
        -46600+313150*y-893775*y**2+1879235*y**3-1980365*y**4+662373*y**5
        +835547*y**6-1107143*y**7+596793*y**8-161775*y**9+17760*y**10)-3*r**10*(160387
        -1443483*y+5773932*y**2-13833308*y**3+21607542*y**4-22887990*y**5+16545228*y**6
        -8009532*y**7+2461743*y**8-426647*y**9+30624*y**10)+5*r**8*(85435-768915*y
        +3075660*y**2-7539228*y**3+12110328*y**4-12986040*y**5+9162222*y**6
        -3936510*y**7+754245*y**8+121235*y**9-92466*y**10+14034*y**11))-6*MBhat*(33176*r**14*(
        -1+y)**10+(-1+y)**8*(3253996-7568072*y+4844116*y**2-353360*y**3
        -88340*y**4+11557*y**5-69230*y**6-6584*y**7-1070*y**8+507*y**9)+14*r**2*(
        -1+y)**6*(42558-496272*y+1396488*y**2-1583072*y**3+721808*y**4-73209*y**5
        +170*y**6-9758*y**7-678*y**8+207*y**9+30*y**10)+28*r**6*(-1+y)**3*(48350
        -326450*y+937350*y**2-1478250*y**3+1373250*y**4-670179*y**5+116757*y**6
        +24216*y**7-3366*y**8-1889*y**9+371*y**10)-7*r**4*(-1+y)**4*(366968
        -1874448*y+3704040*y**2-3301920*y**3+839640*y**4+580263*y**5-342042*y**6
        -4485*y**7+22080*y**8-1391*y**9-270*y**10+45*y**11)-7*r**8*(-1+y)**2*(13700
        -109600*y+383600*y**2-767200*y**3+959000*y**4-1435261*y**5+1441984*y**6
        -888737*y**7+243292*y**8-17579*y**9-5744*y**10+1105*y**11)+7*r**12*(
        -25016+250160*y-1125720*y**2+3001920*y**3-5253360*y**4+6581295*y**5
        -5917254*y**6+3834555*y**7-1728000*y**8+512645*y**9-89858*y**10+7017*y**11)
        +14*r**10*(22186-221860*y+998370*y**2-2662320*y**3+4659060*y**4-5952321*y**5
        +5743716*y**6-4122519*y**7+2133000*y**8-739615*y**9+160042*y**10-18405*y**11
        +666*y**12))-2*MBhat**3*(811800*r**14*(-1+y)**10+84*r**2*(-1+y)**6*(-86596
        +274864*y-269256*y**2+27974*y**3+73149*y**4-21920*y**5-1210*y**6-110*y**7
        +225*y**8)+(-1+y)**8*(9658968-23917776*y+16558728*y**2-1357510*y**3
        -432040*y**4-280966*y**5-8988*y**6+8949*y**7+5790*y**8+45*y**9)+42*r**4*(
        -1+y)**4*(-75900+430200*y-999900*y**2+1254505*y**3-825450*y**4+277557*y**5
        -69796*y**6+39567*y**7-13194*y**8+1611*y**9)+56*r**6*(-1+y)**3*(-5375
        +25925*y-36825*y**2+70015*y**3+226745*y**4-564255*y**5+578021*y**6
        -312245*y**7+113892*y**8-26848*y**9+2950*y**10)-21*r**8*(-1+y)**2*(15800
        -126400*y+442400*y**2-1485910*y**3+1322140*y**4+677584*y**5-2949192*y**6
        +3179877*y**7-1931124*y**8+700890*y**9-140000*y**10+11695*y**11)+14*r**12*(
        -194924+1949240*y-8771580*y**2+23852985*y**3-42919170*y**4+53471691*y**5
        -46862844*y**6+28752630*y**7-12015900*y**8+3218495*y**9-486338*y**10
        +29907*y**11)-84*r**10*(-32748+327480*y-1473660*y**2+4113950*y**3-7523945*y**4
        +9289236*y**5-7762965*y**6+4193556*y**7-1228275*y**8-6230*y**9+147539*y**10
        -49680*y**11+5742*y**12))-3*MBhat**2*(219318*r**14*(-1+y)**10-(-1+y)**8*(9651678
        -22962396*y+15140238*y**2-1219680*y**3-143745*y**4-263046*y**5-83566*y**6
        -6910*y**7+4307*y**8+720*y**9)+28*r**4*(-1+y)**4*(165019-885834*y
        +1901745*y**2-2014260*y**3+930330*y**4-6612*y**5-137223*y**6+39330*y**7
        -8067*y**8+1172*y**9)+21*r**2*(-1+y)**6*(207868-386512*y-310152*y**2
        +1096688*y**3-697307*y**4+59820*y**5+33675*y**6+2280*y**7-345*y**8
        -260*y**9+5*y**10)+14*r**6*(-1+y)**3*(-64775+480125*y-1533825*y**2
        +2743275*y**3-3416295*y**4+2369109*y**5-724087*y**6-118091*y**7+120462*y**8
        -32418*y**9+3720*y**10)-21*r**8*(-1+y)**2*(30450-232200*y+767100*y**2
        -1427800*y**3+2332145*y**4-2226280*y**5+1158653*y**6-65784*y**7-215751*y**8
        +121872*y**9-30325*y**10+2960*y**11)+14*r**12*(-63602+636020*y-2862090*y**2
        +7632240*y**3-13818525*y**4+17485650*y**5-15740529*y**6+10005720*y**7
        -4383765*y**8+1254890*y**9-209705*y**10+15312*y**11)-7*r**10*(-152804
        +1528040*y-6876180*y**2+18336480*y**3-34448655*y**4+46465734*y**5-44590080*y**6
        +30025626*y**7-13633110*y**8+3911330*y**9-579596*y**10+6198*y**11+7017*y**12)))/(25200*(
        -1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-9874571-39163504*r**2+19975116*r**4+13370224*r**6+15352400*r**8-197904*r**10
        +740404*r**12-223216*r**14+21051*r**16+840*logy*(-2972-35848*r**2-51753*r**4
        -24952*r**6-8375*r**8-480*r**10+135*r**12+48*MBhat**7*(17+5*r**2)+5*MBhat**8*(4
        +9*r**4)+16*MBhat**5*(1967+2511*r**2+360*r**4-390*r**6)+2*MBhat**6*(
        -4144-2348*r**2+105*r**4+585*r**6)+48*MBhat*(391+3483*r**2+3565*r**4
        +1461*r**6+100*r**8)-16*MBhat**3*(-4585-18833*r**2-8793*r**4-735*r**6
        +600*r**8)+3*MBhat**4*(-21000-51464*r**2-15244*r**4+4300*r**6+1125*r**8)
        +2*MBhat**2*(-25088-156996*r**2-110169*r**4-32383*r**6+4650*r**8+945*r**10))*(
        -1+y)**11+111116761*y+460910864*r**2*y-176253756*r**4*y-126112784*r**6*y
        -161841400*r**8*y+2580144*r**10*y-8257844*r**12*y+2455376*r**14*y-231561*r**16*y
        -569314445*y**2-2470172080*r**2*y**2+642169920*r**4*y**2+515285680*r**6*y**2
        +770514500*r**8*y**2-15118320*r**10*y**2+41912920*r**12*y**2-12276880*r**14*y**2
        +1157805*r**16*y**2+1753712135*y**3+7962575440*r**2*y**3-1129513560*r**4*y**3
        -1161596240*r**6*y**3-2182568500*r**8*y**3+52746960*r**10*y**3-127817760*r**12*y**3
        +36830640*r**14*y**3-3473415*r**16*y**3-3610404070*y**4-17167284080*r**2*y**4
        +465785670*r**4*y**4+1458605680*r**6*y**4+4074943250*r**8*y**4-122125920*r**10*y**4
        +260313270*r**12*y**4-73661280*r**14*y**4+6946830*r**16*y**4+5219333378*y**5
        +26021610832*r**2*y**5+2217086382*r**4*y**5-658709072*r**6*y**5-5240610550*r**8*y**5
        +197587488*r**10*y**5-371922978*r**12*y**5+103125792*r**14*y**5-9725562*r**16*y**5
        -5411562338*y**6-28340259472*r**2*y**6-5564470422*r**4*y**6-955186288*r**6*y**6
        +4698915550*r**8*y**6-228633888*r**10*y**6+380654778*r**12*y**6-103125792*r**14*y**6
        +9725562*r**16*y**6+4030059430*y**7+22230701840*r**2*y**7+6843194610*r**4*y**7
        +2068413200*r**6*y**7-2900598890*r**8*y**7+203501760*r**10*y**7-289754430*r**12*y**7
        +76673760*r**14*y**7-6946830*r**16*y**7-2116936328*y**8-12359578576*r**2*y**8
        -5211116736*r**4*y**8-1911531496*r**6*y**8+1191111544*r**8*y**8-156336768*r**10*y**8
        +169841910*r**12*y**8-41638440*r**14*y**8+3473415*r**16*y**8+746562968*y**9
        +4680151696*r**2*y**9+2523749256*r**4*y**9+1046383576*r**6*y**9-315494704*r**8*y**9
        +104481888*r**10*y**9-77041930*r**12*y**9+16641880*r**14*y**9-1157805*r**16*y**9
        -149676928*y**10-1121287568*r**2*y**10-726671988*r**4*y**10-352064552*r**6*y**10
        +64630580*r**8*y**10-53932368*r**10*y**10+26726294*r**12*y**10-4510616*r**14*y**10
        +231561*r**16*y**10-8801632*y**11+163469296*r**2*y**11+90114948*r**4*y**11
        +71985592*r**6*y**11-19865860*r**8*y**11+18699408*r**10*y**11-6138202*r**12*y**11
        +729736*r**14*y**11-21051*r**16*y**11+32738590*y**12-37693264*r**2*y**12
        +12078990*r**4*y**12-10645880*r**6*y**12+6437270*r**8*y**12-3542448*r**10*y**12
        +780696*r**12*y**12-53280*r**14*y**12-29103914*y**13+25009712*r**2*y**13
        -8288154*r**4*y**13+2278696*r**6*y**13-983794*r**8*y**13+289968*r**10*y**13
        -37128*r**12*y**13+17956234*y**14-11931376*r**2*y**14+2638314*r**4*y**14
        -564536*r**6*y**14+61754*r**8*y**14-7561054*y**15+3417520*r**2*y**15-505470*r**4*y**15
        +98280*r**6*y**15-3150*r**8*y**15+2066459*y**16-493568*r**2*y**16+26880*r**4*y**16
        -10080*r**6*y**16-342025*y**17+16864*r**2*y**17+33957*y**18-576*r**2*y**18
        -2607*y**19+14*MBhat**8*(-1+y)**4*(-35085*r**8*(-1+y)**7+5*(-1+y)**5*(
        -1127+2494*y-1487*y**2+1184*y**3-1435*y**4+662*y**5+45*y**6)+24*r**2*(
        -1+y)**3*(-175+700*y-1050*y**2-170*y**3+1705*y**4-1514*y**5+432*y**6)
        -20*r**6*(2930-20510*y+61530*y**2-110206*y**3+126235*y**4-94137*y**5
        +43841*y**6-11591*y**7+1332*y**8)+6*r**4*(2280-16410*y+50805*y**2
        -99905*y**3+135020*y**4-122850*y**5+71087*y**6-23339*y**7+3312*y**8))
        -168*MBhat**7*(-1+y)**3*(17864*r**10*(-1+y)**8+(-1+y)**6*(1016-6112*y
        +7136*y**2+230*y**3-1980*y**4+645*y**5+170*y**6+15*y**7)-4*r**2*(-1
        +y)**4*(700-2500*y+3150*y**2+1825*y**3-7720*y**4+7626*y**5-3306*y**6
        +513*y**7)-2*r**4*(-1+y)**2*(-860+5160*y-12900*y**2+440*y**3+23060*y**4
        -22549*y**5+278*y**6+10300*y**7-5954*y**8+1105*y**9)+5*r**8*(-8368
        +66944*y-234304*y**2+488546*y**3-655064*y**4+582535*y**5-340424*y**6
        +125332*y**7-26192*y**8+2339*y**9)-20*r**6*(-1202+9616*y-33656*y**2
        +72427*y**3-102474*y**4+95247*y**5-55610*y**6+17525*y**7-1120*y**8
        -975*y**9+222*y**10))+28*MBhat**6*(-1+y)**2*(-239256*r**12*(-1+y)**9
        +(-1+y)**7*(353393-955426*y+726353*y**2-54080*y**3-52925*y**4+4090*y**5
        +720*y**6+1350*y**7+45*y**8)+r**2*(-1+y)**5*(-298859+1054556*y-1300074*y**2
        +402796*y**3+386726*y**4-407604*y**5+163966*y**6-25924*y**7+225*y**8)
        -6*r**4*(-1+y)**3*(7140-43890*y+112875*y**2-239030*y**3+299865*y**4
        -238790*y**5+124649*y**6-41730*y**7+4871*y**8+600*y**9)-9*r**10*(81363
        -732267*y+2929068*y**2-7014892*y**3+10951128*y**4-11591352*y**5+8370852*y**6
        -4046868*y**7+1241397*y**8-214493*y**9+15312*y**10)+2*r**6*(-1+y)**2*(
        -85280+579410*y-1676805*y**2+3393265*y**3-3648175*y**4+1558767*y**5
        +923377*y**6-1540885*y**7+871971*y**8-241005*y**9+26640*y**10)+5*r**8*(135053
        -1215477*y+4861908*y**2-11888484*y**3+19064862*y**4-20455326*y**5+14492913*y**6
        -6304905*y**7+1271475*y**8+152965*y**9-136035*y**10+21051*y**11))-8*MBhat**5*(
        -1+y)*(811800*r**14*(-1+y)**10+84*r**2*(-1+y)**6*(-64131+206304*y
        -209016*y**2+30414*y**3+55664*y**4-23612*y**5+2784*y**6-498*y**7+75*y**8)
        +(-1+y)**8*(6904692-17113944*y+11861532*y**2-925750*y**3-395920*y**4
        -170002*y**5-4956*y**6+5673*y**7+3270*y**8+45*y**9)+42*r**4*(-1+y)**4*(
        -52200+298800*y-703800*y**2+913705*y**3-664440*y**4+304236*y**5-131026*y**6
        +63795*y**7-17454*y**8+1824*y**9)+14*r**6*(-1+y)**3*(-41300+242300*y
        -563100*y**2+973060*y**3-217930*y**4-1055151*y**5+1568831*y**6-1067282*y**7
        +474720*y**8-127663*y**9+15115*y**10)-21*r**8*(-1+y)**2*(69300-554400*y
        +1940400*y**2-4481910*y**3+5558780*y**4-3463246*y**5-172712*y**6+2004217*y**7
        -1643024*y**8+669300*y**9-140000*y**10+11695*y**11)+14*r**12*(-214064
        +2140640*y-9632880*y**2+26149785*y**3-46938570*y**4+58294971*y**5-50882244*y**6
        +31049430*y**7-12877200*y**8+3409895*y**9-505478*y**10+29907*y**11)
        -42*r**10*(-88974+889740*y-4003830*y**2+11045260*y**3-20113570*y**4
        +25016139*y**5-21390024*y**6+12197847*y**7-4175280*y**8+498125*y**9
        +205426*y**10-92343*y**11+11484*y**12))-2*MBhat**2*(209349*r**16*(-1+y)**11
        -42*r**2*(-1+y)**7*(1745509-10121956*y+21462774*y**2-20588356*y**3
        +8287009*y**4-668640*y**5-39327*y**6-63844*y**7-21230*y**8-420*y**9
        +417*y**10)-(-1+y)**9*(71347859-163769638*y+102958739*y**2-7024640*y**3
        -1756160*y**4-4928*y**5-813694*y**6-569860*y**7-35725*y**8-4194*y**9
        +1521*y**10)+14*r**6*(-1+y)**4*(-3937709+25620983*y-70062519*y**2+103170005*y**3
        -86168930*y**4+36882258*y**5-5325506*y**6-842182*y**7+34431*y**8-26137*y**9
        +9961*y**10+225*y**11)+42*r**8*(-1+y)**3*(11475-184800*y+1018800*y**2
        -2905600*y**3+4934000*y**4-6466480*y**5+4487115*y**6-1464376*y**7-131197*y**8
        +147912*y**9-35009*y**10+3520*y**11)+42*r**4*(-1+y)**5*(2154669-10724634*y
        +20201445*y**2-15918360*y**3+921870*y**4+6139680*y**5-3123404*y**6+227820*y**7
        +93519*y**8+3290*y**9-3345*y**10+540*y**11+30*y**12)-42*r**10*(-1+y)**2*(
        -57123+495207*y-1895778*y**2+4196682*y**3-5898123*y**4+7393251*y**5
        -6042019*y**6+3209283*y**7-723798*y**8-29078*y**9+67161*y**10-18561*y**11
        +1776*y**12)+6*r**14*(189233-2081563*y+10407815*y**2-31223445*y**3
        +62446890*y**4-90588750*y**5+95778942*y**6-74399322*y**7+41751105*y**8
        -16443515*y**9+4299295*y**10-666461*y**11+45936*y**12)-14*r**12*(134177
        -1475947*y+7379735*y**2-22139205*y**3+44278410*y**4-66593022*y**5+76061436*y**6
        -64446564*y**7+39771285*y**8-17113895*y**9+4938109*y**10-863639*y**11
        +69120*y**12))-24*MBhat*(5104*r**16*(-1+y)**11-(-1+y)**9*(-2423796
        +5504472*y-3409116*y**2+218960*y**3+54740*y**4+21896*y**5-14770*y**6
        +32641*y**7+6688*y**8+421*y**9+24*y**10)+r**2*(-1+y)**7*(5607748-28282432*y
        +54126528*y**2-47787232*y**3+17798248*y**4-1170288*y**5-254674*y**6
        +54545*y**7-90892*y**8-9682*y**9-102*y**10+169*y**11)+7*r**8*(-1+y)**3*(
        -164700+1293600*y-4431600*y**2+8639200*y**3-10463000*y**4+8034400*y**5
        -3513830*y**6+697161*y**7+17664*y**8+4082*y**9-6270*y**10+813*y**11)
        -14*r**6*(-1+y)**4*(-180086+1085282*y-2642226*y**2+3176470*y**3-1642420*y**4
        -241788*y**5+648146*y**6-235575*y**7+13897*y**8+1292*y**9-382*y**10
        +95*y**11+15*y**12)-14*r**4*(-1+y)**5*(314310-1458060*y+2361750*y**2
        -1010000*y**3-1381500*y**4+1836000*y**5-744428*y**6+76085*y**7-2902*y**8
        +7000*y**9-20*y**10-185*y**11+30*y**12)-7*r**10*(-1+y)**2*(-2428+21852*y
        -87408*y**2+203952*y**3-305928*y**4+305928*y**5-651166*y**6+705457*y**7
        -450443*y**8+123522*y**9-13504*y**10-583*y**11+221*y**12)+4*r**14*(9115
        -100265*y+501325*y**2-1503975*y**3+3007950*y**4-4211130*y**5+4386858*y**6
        -3367098*y**7+1908495*y**8-765160*y**9+204490*y**10-32624*y**11+2339*y**12)
        +28*r**12*(-3259+35849*y-179245*y**2+537735*y**3-1075470*y**4+1505658*y**5
        -1591655*y**6+1309896*y**7-824289*y**8+385230*y**9-122624*y**10+24841*y**11
        -2778*y**12+111*y**13))-8*MBhat**3*(135300*r**16*(-1+y)**11+(-1+y)**9*(23342400
        -54387600*y+34896600*y**2-2567600*y**3-446390*y**4-260267*y**5-364140*y**6
        -46545*y**7-4212*y**8+2214*y**9+180*y**10)+28*r**6*(-1+y)**4*(369025
        -2539075*y+7462875*y**2-12129425*y**3+11905625*y**4-6678087*y**5+1914413*y**6
        -178703*y**7+19866*y**8-5950*y**9+556*y**10)-r**2*(-1+y)**7*(-2274828
        +40738752*y-124387008*y**2+146203552*y**3-67661758*y**4+5701584*y**5
        +1189832*y**6+650821*y**7+22626*y**8-13674*y**9-600*y**10+45*y**11)
        -21*r**4*(-1+y)**5*(839732-4334952*y+8727060*y**2-8118880*y**3+2508720*y**4
        +1133903*y**5-930602*y**6+146139*y**7+4300*y**8-2975*y**9+630*y**10
        +45*y**11)+7*r**8*(-1+y)**3*(304900-2295200*y+7457200*y**2-13570400*y**3
        +16842850*y**4-10238027*y**5+1510340*y**6+2393168*y**7-1503634*y**8
        +504809*y**9-105976*y**10+10370*y**11)-21*r**10*(-1+y)**2*(-7100+63900*y
        -255600*y**2+596400*y**3-1944210*y**4+2249618*y**5-1280700*y**6-182475*y**7
        +636153*y**8-437232*y**9+159150*y**10-30323*y**11+2339*y**12)+4*r**14*(149857
        -1648427*y+8242135*y**2-24726405*y**3+50770770*y**4-73816176*y**5+77744100*y**6
        -59480124*y**7+32700855*y**8-12545170*y**9+3166988*y**10-467350*y**11
        +29907*y**12)-7*r**12*(118396-1302356*y+6511780*y**2-19535340*y**3
        +41582940*y**4-63312411*y**5+69097089*y**6-54131541*y**7+29809443*y**8
        -11067925*y**9+2448479*y**10-187123*y**11-39087*y**12+7656*y**13))-3*MBhat**4*(770175*r**16*(
        -1+y)**11+28*r**2*(-1+y)**7*(808976-1691984*y-549864*y**2+3487336*y**3
        -2416859*y**4+277420*y**5+102245*y**6+5000*y**7-1825*y**8-300*y**9
        +15*y**10)-(-1+y)**9*(45972575-109585150*y+72432575*y**2-5612320*y**3
        -1039710*y**4-1293348*y**5-323246*y**6-19424*y**7+13663*y**8+3170*y**9
        +15*y**10)-56*r**6*(-1+y)**4*(93025-715675*y+2372775*y**2-4312325*y**3
        +5303925*y**4-3868879*y**5+1543699*y**6-216553*y**7+240*y**8+1328*y**9
        +40*y**10)-14*r**4*(-1+y)**5*(-1484752+7993872*y-17240760*y**2+18581360*y**3
        -9267995*y**4+1087522*y**5+518303*y**6-61900*y**7+14675*y**8-6150*y**9
        +225*y**10)+28*r**8*(-1+y)**3*(-92400+705450*y-2334075*y**2+3854150*y**3
        -6601050*y**4+7743486*y**5-6604129*y**6+3897226*y**7-1863884*y**8+641936*y**9
        -131470*y**10+11960*y**11)-28*r**10*(-1+y)**2*(-54200+487800*y-1951200*y**2
        +5465560*y**3-6625015*y**4+3125399*y**5+3191061*y**6-6530061*y**7+5573535*y**8
        -2815855*y**9+859291*y**10-145163*y**11+10208*y**12)+8*r**14*(364891
        -4013801*y+20069005*y**2-61085655*y**3+124570560*y**4-178856664*y**5
        +184810164*y**6-137960460*y**7+73445130*y**8-26962210*y**9+6384642*y**10
        -851022*y**11+45100*y**12)-14*r**12*(255648-2812128*y+14060640*y**2
        -43766560*y**3+89892465*y**4-127171995*y**5+126038791*y**6-85983189*y**7
        +37481103*y**8-7532965*y**9-1618155*y**10+1517697*y**11-401228*y**12
        +39876*y**13)))/(25200*(-1+y)**11)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr2(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r2 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-1020*(-1+y)**7-680*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(160+420*logy*(
        -1+y)**2+100*y-470*y**2+312*y**3-145*y**4+26*y**5-5*y**6)+2*r**4*(
        -1+y)**3*(-200+800*y-1200*y**2+330*y**3+505*y**4-294*y**5+17*y**6)
        -r**6*(-1+y)*(-600+900*logy*(-1+y)**6+4500*y-13950*y**2+20060*y**3
        -15255*y**4+7320*y**5-2309*y**6+354*y**7)-10*r**8*(70-490*y+1470*y**2
        -2806*y**3+3445*y**4-2568*y**5+1176*y**6-308*y**7+35*y**8))/(150*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-6120*(-1+y)**8-1224*r**12*(-1+y)**8-2*r**2*(-1+y)**6*(27013-19740*logy*(
        -1+y)**2-73766*y+56623*y**2-6580*y**3-3115*y**4+5*y**5+75*y**6+75*y**7)
        +3*r**6*(-1+y)**2*(-2100+600*logy*(-1+y)**6+13200*y-34800*y**2+49400*y**3
        -34765*y**4+8658*y**5-866*y**6+1546*y**7-513*y**8)+r**4*(-1+y)**4*(70600
        +17700*logy*(-1+y)**4-264700*y+361650*y**2-205700*y**3+29465*y**4
        +8832*y**5+112*y**6-688*y**7-75*y**8)-r**8*(-1+y)*(-4600+8100*logy*(
        -1+y)**7+24100*y-43950*y**2+16550*y**3+17335*y**4-8325*y**5+4574*y**6
        -3398*y**7+489*y**8+105*y**9)-5*r**10*(-334+2672*y-9352*y**2+18704*y**3
        -27871*y**4+30026*y**5-19648*y**6+8048*y**7-1951*y**8+210*y**9)-6*MBhat**2*(
        -1+y)*(1020*(-1+y)**7+680*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(160+420*logy*(
        -1+y)**2+100*y-470*y**2+312*y**3-145*y**4+26*y**5-5*y**6)-2*r**4*(
        -1+y)**3*(-200+800*y-1200*y**2+330*y**3+505*y**4-294*y**5+17*y**6)
        +r**6*(-1+y)*(-600+900*logy*(-1+y)**6+4500*y-13950*y**2+20060*y**3
        -15255*y**4+7320*y**5-2309*y**6+354*y**7)+10*r**8*(70-490*y+1470*y**2
        -2806*y**3+3445*y**4-2568*y**5+1176*y**6-308*y**7+35*y**8))-2*MBhat*(
        -6120*(-1+y)**8+3738*r**12*(-1+y)**8+6*r**4*(-1+y)**4*(2000+450*logy*(
        -1+y)**4-7550*y+10425*y**2-7175*y**3+3510*y**4-2028*y**5+873*y**6
        -139*y**7)+r**2*(-1+y)**6*(-12388+22440*logy*(-1+y)**2+47216*y-46048*y**2
        +11935*y**3-2135*y**4+970*y**5-195*y**6-15*y**7)+3*r**6*(-1+y)**2*(3050
        +300*logy*(-1+y)**6-18000*y+44100*y**2-52370*y**3+31420*y**4-9128*y**5
        +101*y**6+605*y**7+17*y**8-35*y**9)+15*r**10*(-662+5296*y-18536*y**2
        +38569*y**3-51481*y**4+44657*y**5-25172*y**6+8824*y**7-1715*y**8+136*y**9)
        -5*r**8*(-710+5680*y-19880*y**2+46750*y**3-75782*y**4+78964*y**5-51167*y**6
        +19654*y**7-3470*y**8-144*y**9+105*y**10)))/(900*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-6120*(-1+y)**9-544*r**14*(-1+y)**9+r**2*(-1+y)**7*(-161086+74280*logy*(
        -1+y)**2+396452*y-272506*y**2+24760*y**3+6190*y**4+4870*y**5+943*y**6
        -176*y**7-107*y**8)+4*r**8*(-1+y)**2*(3400+1500*logy*(-1+y)**7-25300*y
        +81150*y**2-145750*y**3+158875*y**4-94338*y**5+19758*y**6+450*y**7
        +993*y**8-318*y**9)+2*r**4*(-1+y)**5*(106697+55740*logy*(-1+y)**4
        -371048*y+445092*y**2-185248*y**3-9428*y**4+9321*y**5+3770*y**6+1042*y**7
        -375*y**8-75*y**9)-2*r**6*(-1+y)**3*(10100+46200*logy*(-1+y)**6-14400*y
        -102600*y**2+367800*y**3-506850*y**4+335658*y**5-96243*y**6+8628*y**7
        -2361*y**8+628*y**9)-r**10*(-1+y)*(12730+10800*logy*(-1+y)**8-91040*y
        +275440*y**2-450080*y**3+411400*y**4-238370*y**5+118591*y**6-33968*y**7
        +2818*y**8-66*y**9+105*y**10)-2*r**12*(443-3987*y+15948*y**2-37212*y**3
        +55818*y**4-68121*y**5+66543*y**6-38772*y**7+14193*y**8-3152*y**9+315*y**10)
        -6*MBhat**4*(-1+y)**2*(1020*(-1+y)**7+680*r**10*(-1+y)**7-5*r**2*(-1
        +y)**5*(160+420*logy*(-1+y)**2+100*y-470*y**2+312*y**3-145*y**4
        +26*y**5-5*y**6)-2*r**4*(-1+y)**3*(-200+800*y-1200*y**2+330*y**3
        +505*y**4-294*y**5+17*y**6)+r**6*(-1+y)*(-600+900*logy*(-1+y)**6
        +4500*y-13950*y**2+20060*y**3-15255*y**4+7320*y**5-2309*y**6+354*y**7)
        +10*r**8*(70-490*y+1470*y**2-2806*y**3+3445*y**4-2568*y**5+1176*y**6
        -308*y**7+35*y**8))-4*MBhat**3*(-1+y)*(-6120*(-1+y)**8+3738*r**12*(
        -1+y)**8+6*r**4*(-1+y)**4*(2000+450*logy*(-1+y)**4-7550*y+10425*y**2
        -7175*y**3+3510*y**4-2028*y**5+873*y**6-139*y**7)+r**2*(-1+y)**6*(
        -12388+22440*logy*(-1+y)**2+47216*y-46048*y**2+11935*y**3-2135*y**4
        +970*y**5-195*y**6-15*y**7)+3*r**6*(-1+y)**2*(3050+300*logy*(-1+y)**6
        -18000*y+44100*y**2-52370*y**3+31420*y**4-9128*y**5+101*y**6+605*y**7
        +17*y**8-35*y**9)+15*r**10*(-662+5296*y-18536*y**2+38569*y**3-51481*y**4
        +44657*y**5-25172*y**6+8824*y**7-1715*y**8+136*y**9)-5*r**8*(-710+5680*y
        -19880*y**2+46750*y**3-75782*y**4+78964*y**5-51167*y**6+19654*y**7
        -3470*y**8-144*y**9+105*y**10))-4*MBhat*(-6120*(-1+y)**9+890*r**14*(
        -1+y)**9+2*r**4*(-1+y)**5*(66380+20100*logy*(-1+y)**4-245420*y+327930*y**2
        -178420*y**3+22555*y**4+6696*y**5+38*y**6+64*y**7-75*y**8)+r**2*(-1
        +y)**7*(-92786+53280*logy*(-1+y)**2+238852*y-172706*y**2+17760*y**3
        +6750*y**4+1461*y**5+143*y**6-109*y**7-25*y**8)+3*r**6*(-1+y)**3*(
        -14950-4800*logy*(-1+y)**6+84900*y-197850*y**2+239800*y**3-152735*y**4
        +44236*y**5-3835*y**6+77*y**7+110*y**8+7*y**9)-r**8*(-1+y)**2*(13200
        +600*logy*(-1+y)**7-93000*y+281100*y**2-472700*y**3+441215*y**4-218991*y**5
        +59332*y**6-4567*y**7-2238*y**8+214*y**9+35*y**10)+3*r**12*(1088-9792*y
        +39168*y**2-91392*y**3+143923*y**4-157905*y**5+116994*y**6-58236*y**7
        +18639*y**8-3431*y**9+272*y**10)-5*r**10*(34-306*y+1224*y**2-2856*y**3
        +14181*y**4-39708*y**5+52731*y**6-38724*y**7+17361*y**8-4414*y**9+477*y**10))
        -2*MBhat**2*(18360*(-1+y)**9+5988*r**14*(-1+y)**9+5*r**2*(-1+y)**7*(26320
        -21600*logy*(-1+y)**2-74240*y+58720*y**2-8502*y**3-1599*y**4-408*y**5
        +58*y**6+46*y**7+y**8)+3*r**4*(-1+y)**5*(-58071-11220*logy*(-1+y)**4
        +221064*y-309156*y**2+187974*y**3-43001*y**4+4328*y**5-3302*y**6+618*y**7
        +50*y**8)+3*r**6*(-1+y)**3*(4500-1200*logy*(-1+y)**6-28200*y+74100*y**2
        -110780*y**3+88375*y**4-31162*y**5+4868*y**6-334*y**7-947*y**8+300*y**9)
        +r**8*(-1+y)**2*(500+13800*logy*(-1+y)**7-17300*y+100200*y**2-327740*y**3
        +485765*y**4-330087*y**5+93494*y**6+26998*y**7-31155*y**8+8665*y**9
        -700*y**10)+3*r**12*(7125-64125*y+256500*y**2-612170*y**3+952950*y**4
        -998136*y**5+710220*y**6-336420*y**7+100125*y**8-16465*y**9+1068*y**10)
        -15*r**10*(1776-15984*y+63936*y**2-155426*y**3+248652*y**4-265918*y**5
        +190395*y**6-88543*y**7+23397*y**8-1753*y**9-668*y**10+136*y**11)))/(900*r**2*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-28560*(-1+y)**10-1360*r**16*(-1+y)**10+r**2*(-1+y)**8*(-1477670+546000*logy*(
        -1+y)**2+3501340*y-2296670*y**2+182000*y**3+45500*y**4+18200*y**5
        +19439*y**6+5962*y**7-941*y**8-240*y**9)+7*r**4*(-1+y)**6*(211392+236040*logy*(
        -1+y)**4-609528*y+442212*y**2+177272*y**3-280358*y**4+47208*y**5
        +5365*y**6+4732*y**7+1968*y**8-492*y**9-107*y**10)+35*r**10*(-1+y)**2*(
        -2494+1800*logy*(-1+y)**8+21752*y-83332*y**2+183464*y**3-254530*y**4
        +228824*y**5-117695*y**6+19368*y**7+2654*y**8+40*y**9-67*y**10)-14*r**6*(
        -1+y)**4*(-360765+240300*logy*(-1+y)**6+2404890*y-6733125*y**2+10179000*y**3
        -8835750*y**4+4255200*y**5-951365*y**6+33966*y**7+9105*y**8-1516*y**9
        +600*y**10)+14*r**8*(-1+y)**3*(345400-184800*logy*(-1+y)**7-2233000*y
        +6052200*y**2-8793400*y**3+7176400*y**4-3012240*y**5+371795*y**6+114583*y**7
        -25104*y**8+1704*y**9-283*y**10+25*y**11)-7*r**12*(-1+y)*(-14640+9000*logy*(
        -1+y)**9+122760*y-450540*y**2+943260*y**3-1225890*y**4+999090*y**5
        -536407*y**6+223839*y**7-56196*y**8+5644*y**9-231*y**10+63*y**11)-14*r**14*(
        -179+1790*y-8055*y**2+21480*y**3-37590*y**4+45108*y**5-46634*y**6
        +42480*y**7-22405*y**8+7390*y**9-1509*y**10+140*y**11)-28*MBhat**6*(
        -1+y)**3*(1020*(-1+y)**7+680*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(160
        +420*logy*(-1+y)**2+100*y-470*y**2+312*y**3-145*y**4+26*y**5-5*y**6)
        -2*r**4*(-1+y)**3*(-200+800*y-1200*y**2+330*y**3+505*y**4-294*y**5
        +17*y**6)+r**6*(-1+y)*(-600+900*logy*(-1+y)**6+4500*y-13950*y**2
        +20060*y**3-15255*y**4+7320*y**5-2309*y**6+354*y**7)+10*r**8*(70-490*y
        +1470*y**2-2806*y**3+3445*y**4-2568*y**5+1176*y**6-308*y**7+35*y**8))
        -28*MBhat**5*(-1+y)**2*(-6120*(-1+y)**8+3738*r**12*(-1+y)**8+6*r**4*(
        -1+y)**4*(2000+450*logy*(-1+y)**4-7550*y+10425*y**2-7175*y**3+3510*y**4
        -2028*y**5+873*y**6-139*y**7)+r**2*(-1+y)**6*(-12388+22440*logy*(-1
        +y)**2+47216*y-46048*y**2+11935*y**3-2135*y**4+970*y**5-195*y**6
        -15*y**7)+3*r**6*(-1+y)**2*(3050+300*logy*(-1+y)**6-18000*y+44100*y**2
        -52370*y**3+31420*y**4-9128*y**5+101*y**6+605*y**7+17*y**8-35*y**9)
        +15*r**10*(-662+5296*y-18536*y**2+38569*y**3-51481*y**4+44657*y**5
        -25172*y**6+8824*y**7-1715*y**8+136*y**9)-5*r**8*(-710+5680*y-19880*y**2
        +46750*y**3-75782*y**4+78964*y**5-51167*y**6+19654*y**7-3470*y**8-144*y**9
        +105*y**10))-14*MBhat**4*(-1+y)*(30600*(-1+y)**9+11976*r**14*(-1+y)**9
        +2*r**2*(-1+y)**7*(104587-88260*logy*(-1+y)**2-297434*y+236977*y**2
        -35930*y**3-4880*y**4-2045*y**5+215*y**6+155*y**7+5*y**8)+r**4*(-1
        +y)**5*(-277826-49620*logy*(-1+y)**4+1061684*y-1493286*y**2+922144*y**3
        -228541*y**4+34800*y**5-19700*y**6+3020*y**7+225*y**8)+3*r**6*(-1+y)**3*(6900
        -1800*logy*(-1+y)**6-43200*y+113400*y**2-172160*y**3+141985*y**4
        -53666*y**5+8870*y**6+878*y**7-2407*y**8+600*y**9)+r**8*(-1+y)**2*(5600
        +19500*logy*(-1+y)**7-58700*y+244350*y**2-672030*y**3+954195*y**4
        -651849*y**5+182414*y**6+57394*y**7-62799*y**8+17225*y**9-1400*y**10)
        +6*r**12*(7329-65961*y+263844*y**2-629306*y**3+978654*y**4-1023840*y**5
        +727356*y**6-343764*y**7+101961*y**8-16669*y**9+1068*y**10)-5*r**10*(10990
        -98910*y+395640*y**2-960612*y**3+1538487*y**4-1653405*y**5+1192044*y**6
        -558954*y**7+150381*y**8-12679*y**9-3798*y**10+816*y**11))-2*MBhat*(
        -85680*(-1+y)**10+4628*r**16*(-1+y)**10+r**2*(-1+y)**8*(-3114252+1288560*logy*(
        -1+y)**2+7517064*y-5047092*y**2+429520*y**3+107380*y**4+67564*y**5
        +28847*y**6+3053*y**7-1003*y**8-321*y**9)+7*r**4*(-1+y)**6*(582678
        +372360*logy*(-1+y)**4-1958352*y+2192808*y**2-717152*y**3-193072*y**4
        +67992*y**5+18871*y**6+5588*y**7+108*y**8-452*y**9-25*y**10)+14*r**8*(
        -1+y)**3*(274450-66000*logy*(-1+y)**7-1855150*y+5334450*y**2-8428750*y**3
        +7851250*y**4-4217274*y**5+1149197*y**6-116129*y**7+2619*y**8+91*y**9
        +206*y**10)+14*r**6*(-1+y)**4*(196273-284340*logy*(-1+y)**6-1461978*y
        +4507965*y**2-7432320*y**3+6995940*y**4-3647067*y**5+895203*y**6-52890*y**7
        -540*y**8-1456*y**9+135*y**10+15*y**11)-7*r**10*(-1+y)**2*(-35740+6000*logy*(
        -1+y)**8+291920*y-1045720*y**2+2147440*y**3-2768300*y**4+2149672*y**5
        -910133*y**6+211459*y**7-9344*y**8-9922*y**9+1313*y**10+35*y**11)+14*r**14*(
        -1589+15890*y-71505*y**2+190680*y**3-333690*y**4+420777*y**5-391356*y**6
        +251685*y**7-111015*y**8+32150*y**9-5459*y**10+408*y**11)+7*r**12*(
        -2414+24140*y-108630*y**2+289680*y**3-506940*y**4+475920*y**5-47673*y**6
        -312858*y**7+301065*y**8-146980*y**9+39271*y**10-4686*y**11+105*y**12))
        -MBhat**2*(428400*(-1+y)**10+32934*r**16*(-1+y)**10+r**2*(-1+y)**8*(10202676
        -4907280*logy*(-1+y)**2-25312632*y+17563596*y**2-1635760*y**3-499975*y**4
        -225610*y**5-51289*y**6+1448*y**7+3446*y**8+300*y**9)-7*r**4*(-1+y)**6*(2141728
        +858960*logy*(-1+y)**4-7707952*y+9844008*y**2-4844752*y**3+324973*y**4
        +199680*y**5+31240*y**6+9020*y**7-2655*y**8-340*y**9+10*y**10)+42*r**6*(
        -1+y)**4*(63323+145860*logy*(-1+y)**6-234078*y+147615*y**2+532480*y**3
        -1133765*y**4+883506*y**5-277721*y**6+17980*y**7+2796*y**8-1036*y**9
        +100*y**10)+14*r**8*(-1+y)**3*(-81375+3300*logy*(-1+y)**7+566325*y
        -1687425*y**2+2789275*y**3-2857435*y**4+1690197*y**5-384111*y**6-33663*y**7
        +31686*y**8-14134*y**9+2260*y**10)+7*r**10*(-1+y)**2*(60700+51600*logy*(
        -1+y)**8-434000*y+1312600*y**2-2143600*y**3+1481195*y**4+101120*y**5
        -598142*y**6+406996*y**7-101861*y**8-11728*y**9+8060*y**10-700*y**11)
        +42*r**14*(-3687+36870*y-165915*y**2+442440*y**3-796880*y**4+1009400*y**5
        -896784*y**6+559320*y**7-239115*y**8+66190*y**9-10535*y**10+712*y**11)
        -21*r**12*(-12344+123440*y-555480*y**2+1481280*y**3-2736865*y**4+3668886*y**5
        -3485303*y**6+2314376*y**7-1068335*y**8+321110*y**9-53913*y**10+2876*y**11
        +272*y**12))-2*MBhat**3*(-285600*(-1+y)**10+41010*r**16*(-1+y)**10+7*r**4*(
        -1+y)**6*(844508+219360*logy*(-1+y)**4-3158672*y+4299288*y**2-2442212*y**3
        +403813*y**4+39270*y**5+12990*y**6-2060*y**7-255*y**8-30*y**9)+r**2*(
        -1+y)**8*(-4003892+2388960*logy*(-1+y)**2+10396744*y-7587332*y**2
        +835660*y**3+256935*y**4+68376*y**5+7133*y**6-3524*y**7-890*y**8-10*y**9)
        -14*r**6*(-1+y)**4*(135565+51900*logy*(-1+y)**6-761490*y+1748025*y**2
        -2077190*y**3+1280475*y**4-340476*y**5+15293*y**6+5424*y**7-3678*y**8
        +452*y**9)+14*r**8*(-1+y)**3*(-32525-8100*logy*(-1+y)**7+235775*y
        -735675*y**2+1307285*y**3-1291145*y**4+643455*y**5-102611*y**6-86035*y**7
        +64668*y**8-16872*y**9+1680*y**10)-7*r**10*(-1+y)**2*(43100-344800*y
        +1206800*y**2-2551580*y**3+3165795*y**4-2178358*y**5+704374*y**6+165526*y**7
        -266507*y**8+110180*y**9-20610*y**10+1360*y**11)+14*r**14*(-12079+120790*y
        -543555*y**2+1472090*y**3-2641170*y**4+3268636*y**5-2839074*y**6+1721130*y**7
        -707175*y**8+184595*y**9-26693*y**10+1497*y**11)-7*r**12*(-35756+357560*y
        -1609020*y**2+4427860*y**3-8074045*y**4+10027608*y**5-8613525*y**6+5049282*y**7
        -1869795*y**8+334000*y**9+26297*y**10-23670*y**11+3204*y**12)))/(4200*r**2*(
        -1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-42*MBhat**8*(-1+y)**4*(1020*(-1+y)**7+680*r**10*(-1+y)**7-5*r**2*(
        -1+y)**5*(160+420*logy*(-1+y)**2+100*y-470*y**2+312*y**3-145*y**4
        +26*y**5-5*y**6)-2*r**4*(-1+y)**3*(-200+800*y-1200*y**2+330*y**3
        +505*y**4-294*y**5+17*y**6)+r**6*(-1+y)*(-600+900*logy*(-1+y)**6
        +4500*y-13950*y**2+20060*y**3-15255*y**4+7320*y**5-2309*y**6+354*y**7)
        +10*r**8*(70-490*y+1470*y**2-2806*y**3+3445*y**4-2568*y**5+1176*y**6
        -308*y**7+35*y**8))-56*MBhat**7*(-1+y)**3*(-6120*(-1+y)**8+3738*r**12*(
        -1+y)**8+6*r**4*(-1+y)**4*(2000+450*logy*(-1+y)**4-7550*y+10425*y**2
        -7175*y**3+3510*y**4-2028*y**5+873*y**6-139*y**7)+r**2*(-1+y)**6*(
        -12388+22440*logy*(-1+y)**2+47216*y-46048*y**2+11935*y**3-2135*y**4
        +970*y**5-195*y**6-15*y**7)+3*r**6*(-1+y)**2*(3050+300*logy*(-1+y)**6
        -18000*y+44100*y**2-52370*y**3+31420*y**4-9128*y**5+101*y**6+605*y**7
        +17*y**8-35*y**9)+15*r**10*(-662+5296*y-18536*y**2+38569*y**3-51481*y**4
        +44657*y**5-25172*y**6+8824*y**7-1715*y**8+136*y**9)-5*r**8*(-710+5680*y
        -19880*y**2+46750*y**3-75782*y**4+78964*y**5-51167*y**6+19654*y**7
        -3470*y**8-144*y**9+105*y**10))-28*MBhat**6*(-1+y)**2*(42840*(-1+y)**9
        +17964*r**14*(-1+y)**9+r**2*(-1+y)**7*(286748-245040*logy*(-1+y)**2
        -818536*y+654308*y**2-101210*y**3-11525*y**4-6140*y**5+570*y**6+390*y**7
        +15*y**8)+r**4*(-1+y)**5*(-381439-65580*logy*(-1+y)**4+1460176*y
        -2059104*y**2+1280366*y**3-328079*y**4+56616*y**5-29494*y**6+4186*y**7
        +300*y**8)+3*r**6*(-1+y)**3*(9300-2400*logy*(-1+y)**6-58200*y+152700*y**2
        -233540*y**3+195595*y**4-76170*y**5+12872*y**6+2090*y**7-3867*y**8
        +900*y**9)+r**8*(-1+y)**2*(10700+25200*logy*(-1+y)**7-100100*y+388500*y**2
        -1016320*y**3+1422625*y**4-973611*y**5+271334*y**6+87790*y**7-94443*y**8
        +25785*y**9-2100*y**10)+9*r**12*(7397-66573*y+266292*y**2-635018*y**3
        +987222*y**4-1032408*y**5+733068*y**6-346212*y**7+102573*y**8-16737*y**9
        +1068*y**10)-5*r**10*(16652-149868*y+599472*y**2-1454946*y**3+2331018*y**4
        -2509056*y**5+1812903*y**6-852279*y**7+230571*y**8-20099*y**9-5592*y**10
        +1224*y**11))-2*(21420*(-1+y)**11+612*r**18*(-1+y)**11-7*r**2*(-1
        +y)**9*(-261274+83820*logy*(-1+y)**2+606368*y-387004*y**2+27940*y**3
        +6985*y**4+2794*y**5+1397*y**6+1880*y**7+731*y**8-150*y**9+3*y**10)
        -2*r**4*(-1+y)**7*(315124+1469580*logy*(-1+y)**4+209084*y-3252786*y**2
        +5107684*y**3-2746501*y**4+293916*y**5+48986*y**6+6022*y**7+13908*y**8
        +5318*y**9-1457*y**10-180*y**11)-42*r**12*(-1+y)**2*(1396+2100*logy*(
        -1+y)**9-14664*y+68106*y**2-184114*y**3+320271*y**4-373191*y**5+292894*y**6
        -135040*y**7+16782*y**8+4592*y**9-431*y**10+39*y**11)+21*r**6*(-1+y)**5*(
        -1127558+469740*logy*(-1+y)**6+7235088*y-19496940*y**2+28344620*y**3
        -23607165*y**4+10852086*y**5-2278421*y**6+67100*y**7+6288*y**8+5042*y**9
        -470*y**10+450*y**11)+14*r**8*(-1+y)**4*(-440942+1672440*logy*(-1+y)**7
        +1414154*y+1611078*y**2-14392210*y**3+29026060*y**4-29122716*y**5+15561112*y**6
        -3902356*y**7+209673*y**8+37334*y**9-62*y**10+750*y**11-75*y**12)+7*r**14*(
        -1+y)*(16114+8100*logy*(-1+y)**10-153040*y+648180*y**2-1606980*y**3
        +2569215*y**4-2742858*y**5+1945515*y**6-929964*y**7+336660*y**8-74590*y**9
        +7084*y**10-258*y**11+42*y**12)+14*r**16*(88-968*y+4840*y**2-14520*y**3
        +29040*y**4-40656*y**5+40656*y**6-36480*y**7+31695*y**8-15340*y**9
        +4598*y**10-868*y**11+75*y**12)-7*r**10*(-1+y)**3*(-2302090-887700*logy*(
        -1+y)**8+17529020*y-57800770*y**2+107316340*y**3-121717625*y**4+84946300*y**5
        -34187950*y**6+6279920*y**7+16136*y**8-92948*y**9+688*y**10+304*y**11
        +75*y**12))-3*MBhat**2*(399840*(-1+y)**11+10479*r**18*(-1+y)**11-r**2*(
        -1+y)**9*(-19480241+7351680*logy*(-1+y)**2+46312162*y-30507761*y**2
        +2450560*y**3+612640*y**4+311416*y**5+200970*y**6+56812*y**7+2521*y**8
        -1878*y**9-321*y**10)-14*r**4*(-1+y)**7*(1644202+1425240*logy*(-1+y)**4
        -5151568*y+4876872*y**2-400768*y**3-1325048*y**4+273696*y**5+54653*y**6
        +21132*y**7+5844*y**8-1076*y**9-291*y**10)+28*r**8*(-1+y)**4*(-2037157
        +1081140*logy*(-1+y)**7+13178959*y-35752887*y**2+52020165*y**3-42560190*y**4
        +17942334*y**5-2249803*y**6-645491*y**7+105753*y**8+15969*y**9-4412*y**10
        +200*y**11)+14*r**10*(-1+y)**3*(182640+28500*logy*(-1+y)**8-1432620*y
        +4900170*y**2-9534340*y**3+11518925*y**4-9064120*y**5+4398455*y**6-883088*y**7
        -29271*y**8+27236*y**9-12867*y**10+1960*y**11)+14*r**12*(-1+y)**2*(
        -56506+27600*logy*(-1+y)**9+480954*y-1799616*y**2+3867904*y**3-5222256*y**4
        +4143576*y**5-1651187*y**6+258819*y**7+132540*y**8-76460*y**9+8829*y**10
        +903*y**11-140*y**12)+28*r**6*(-1+y)**5*(-2047394+1567920*logy*(-1
        +y)**6+13852284*y-39334470*y**2+60285560*y**3-53053770*y**4+25924290*y**5
        -5904751*y**6+239400*y**7+35406*y**8+4100*y**9+1245*y**10-210*y**11
        -10*y**12)-28*r**14*(4626-50886*y+254430*y**2-763290*y**3+1526580*y**4
        -2264178*y**5+2610664*y**6-2226916*y**7+1352405*y**8-588555*y**9+173349*y**10
        -30723*y**11+2494*y**12)+4*r**16*(15667-172337*y+861685*y**2-2585055*y**3
        +5170110*y**4-7456890*y**5+7954338*y**6-6114318*y**7+3373095*y**8-1300585*y**9
        +330605*y**10-49279*y**11+3204*y**12))+12*MBhat**5*(-1+y)*(199920*(
        -1+y)**10-41010*r**16*(-1+y)**10-7*r**4*(-1+y)**6*(578988+138960*logy*(
        -1+y)**4-2176992*y+2987568*y**2-1728532*y**3+313593*y**4+12486*y**5
        +12838*y**6-2316*y**7+45*y**8-30*y**9)+r**2*(-1+y)**8*(2704888-1643040*logy*(
        -1+y)**2-7052816*y+5169448*y**2-587020*y**3-162435*y**4-47922*y**5
        -5131*y**6+1998*y**7+540*y**8+10*y**9)+14*r**6*(-1+y)**4*(90715+37500*logy*(
        -1+y)**6-506790*y+1154475*y**2-1357790*y**3+822270*y**4-207768*y**5
        +3788*y**6+5655*y**7-3348*y**8+473*y**9)+14*r**8*(-1+y)**3*(19325+7500*logy*(
        -1+y)**7-142775*y+454575*y**2-834585*y**3+849930*y**4-424464*y**5
        +43279*y**6+90602*y**7-62430*y**8+16658*y**9-1715*y**10)+7*r**10*(-1
        +y)**2*(43440-347520*y+1216320*y**2-2570620*y**3+3288565*y**4-2452668*y**5
        +957374*y**6+31286*y**7-227137*y**8+105410*y**9-20610*y**10+1360*y**11)
        -14*r**14*(-12969+129690*y-583605*y**2+1578890*y**3-2828070*y**4+3492916*y**5
        -3025974*y**6+1827930*y**7-747225*y**8+193495*y**9-27583*y**10+1497*y**11)
        +7*r**12*(-42284+422840*y-1902780*y**2+5211220*y**3-9485935*y**4+11838576*y**5
        -10262919*y**6+6100662*y**7-2331045*y**8+466420*y**9+4079*y**10-22038*y**11
        +3204*y**12))-4*MBhat*(-85680*(-1+y)**11+2136*r**18*(-1+y)**11+r**2*(
        -1+y)**9*(-5626826+1942080*logy*(-1+y)**2+13195732*y-8539946*y**2
        +647360*y**3+161840*y**4+64736*y**5+53914*y**6+31327*y**7+3400*y**8
        -597*y**9-180*y**10)+3*r**4*(-1+y)**7*(1542354+2446080*logy*(-1+y)**4
        -3723336*y+692844*y**2+4430264*y**3-3553646*y**4+489216*y**5+65611*y**6
        +37609*y**7+16598*y**8+1458*y**9-1217*y**10-107*y**11)+21*r**10*(-1
        +y)**3*(-551270-117000*logy*(-1+y)**8+4293160*y-14558060*y**2+28024120*y**3
        -33392150*y**4+25075720*y**5-11388600*y**6+2724643*y**7-241338*y**8
        -1024*y**9+1210*y**10+149*y**11)-21*r**6*(-1+y)**5*(-1834990+964200*logy*(
        -1+y)**6+11974140*y-32827950*y**2+48591600*y**3-41264700*y**4+19398480*y**5
        -4197453*y**6+131320*y**7+25633*y**8+3050*y**9+1205*y**10+170*y**11
        -25*y**12)-21*r**12*(-1+y)**2*(13906+4200*logy*(-1+y)**9-129354*y
        +536316*y**2-1301804*y**3+2040906*y**4-2146746*y**5+1432533*y**6-526862*y**7
        +107407*y**8-3138*y**9-5011*y**10+656*y**11+7*y**12)+14*r**8*(-1+y)**4*(1866199
        -2011980*logy*(-1+y)**7-11051413*y+26112309*y**2-29436655*y**3+11831830*y**4
        +6984762*y**5-9351659*y**6+3360740*y**7-306288*y**8-13288*y**9-3737*y**10
        +690*y**11+30*y**12)+6*r**16*(2165-23815*y+119075*y**2-357225*y**3
        +714450*y**4-1000230*y**5+1048838*y**6-846778*y**7+480395*y**8-189585*y**9
        +49885*y**10-7799*y**11+544*y**12)+7*r**14*(3866-42526*y+212630*y**2
        -637890*y**3+1275780*y**4-1786092*y**5+1615437*y**6-692847*y**7-84963*y**8
        +233845*y**9-127919*y**10+34669*y**11-4095*y**12+105*y**13))-4*MBhat**3*(
        -599760*(-1+y)**11+20505*r**18*(-1+y)**11+r**2*(-1+y)**9*(-20655435
        +8727600*logy*(-1+y)**2+50038470*y-33746835*y**2+2909200*y**3+785470*y**4
        +419944*y**5+162897*y**6+26277*y**7-3006*y**8-1587*y**9-75*y**10)-14*r**8*(
        -1+y)**4*(-1634795+463500*logy*(-1+y)**7+10980065*y-31317945*y**2
        +48952075*y**3-44932825*y**4+23524347*y**5-5947738*y**6+348598*y**7
        +88359*y**8-27385*y**9+2524*y**10)-21*r**6*(-1+y)**5*(-697894+1258920*logy*(
        -1+y)**6+5446284*y-17392470*y**2+29484560*y**3-28410415*y**4+15120324*y**5
        -3777791*y**6+212812*y**7+13875*y**8+5350*y**9-1305*y**10+30*y**11)
        +r**4*(-1+y)**7*(29276154+15907080*logy*(-1+y)**4-101197536*y+119982144*y**2
        -48173936*y**3-4022041*y**4+3214764*y**5+679385*y**6+202918*y**7-57*y**8
        -10992*y**9-225*y**10+30*y**11)+7*r**10*(-1+y)**3*(78830-90000*logy*(
        -1+y)**8-720640*y+2882240*y**2-6604480*y**3+9896600*y**4-8934616*y**5
        +4240945*y**6-1018871*y**7-132452*y**8+193552*y**9-49343*y**10+4435*y**11)
        -21*r**12*(-1+y)**2*(-21890+197010*y-788040*y**2+1838760*y**3-2962585*y**4
        +3150249*y**5-2016177*y**6+811347*y**7-141729*y**8-30339*y**9+21037*y**10
        -4155*y**11+272*y**12)+2*r**16*(53587-589457*y+2947285*y**2-8841855*y**3
        +18048270*y**4-26232486*y**5+27428310*y**6-20776074*y**7+11277255*y**8
        -4253095*y**9+1048223*y**10-149185*y**11+8982*y**12)-7*r**14*(29278
        -322058*y+1610290*y**2-4830870*y**3+10078125*y**4-15228621*y**5+16510245*y**6
        -12885483*y**7+7249179*y**8-2818915*y**9+693275*y**10-84457*y**11-1056*y**12
        +1068*y**13))+MBhat**4*(-2998800*(-1+y)**11-169575*r**18*(-1+y)**11
        +42*r**4*(-1+y)**7*(2474082+886440*logy*(-1+y)**4-9009888*y+11741952*y**2
        -6066348*y**3+621437*y**4+193760*y**5+35730*y**6+5440*y**7-1985*y**8
        -60*y**9)+r**2*(-1+y)**9*(-68186025+33474000*logy*(-1+y)**2+169846050*y
        -118397025*y**2+11291560*y**3+3409000*y**4+1389640*y**5+337134*y**6
        +5652*y**7-17241*y**8-2130*y**9-15*y**10)-28*r**8*(-1+y)**4*(-299275
        +67500*logy*(-1+y)**7+2027425*y-5846025*y**2+9254375*y**3-9126425*y**4
        +5431101*y**5-1546841*y**6+274187*y**7-104790*y**8+11128*y**9+740*y**10)
        -42*r**6*(-1+y)**5*(485448+993360*logy*(-1+y)**6-1919328*y+1818240*y**2
        +2536160*y**3-6894095*y**4+5677822*y**5-1820847*y**6+102500*y**7+31625*y**8
        -9950*y**9+825*y**10)-42*r**10*(-1+y)**3*(97400+46500*logy*(-1+y)**8
        -732700*y+2378450*y**2-4417600*y**3+4343125*y**4-1782292*y**5-345957*y**6
        +1062668*y**7-706957*y**8+243508*y**9-47205*y**10+4160*y**11)+42*r**12*(
        -1+y)**2*(-16750+150750*y-603000*y**2+1586180*y**3-2205685*y**4+1410605*y**5
        -43960*y**6-948996*y**7+977847*y**8-510923*y**9+152512*y**10-23924*y**11
        +1424*y**12)-12*r**16*(63043-693473*y+3467365*y**2-10523615*y**3+21436780*y**4
        -30690002*y**5+31592022*y**6-23467950*y**7+12410640*y**8-4512805*y**9
        +1052931*y**10-136731*y**11+6835*y**12)+14*r**14*(82696-909656*y+4548280*y**2
        -14040720*y**3+28952055*y**4-41265705*y**5+41653557*y**6-29705103*y**7
        +14277846*y**8-4010380*y**9+245630*y**10+246194*y**11-83676*y**12+8982*y**13)))/(6300*r**2*(
        -1+y)**11)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr3(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r3 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (3240*(-1+y)**7-3165*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(835+600*logy*(
        -1+y)**2-1070*y-65*y**2+284*y**3+65*y**4-118*y**5-15*y**6)+12*r**4*(
        -1+y)**3*(200-800*y+1200*y**2-1035*y**3+915*y**4-597*y**5+171*y**6)
        +18*r**6*(-1+y)*(-450+150*logy*(-1+y)**6+2850*y-7575*y**2+11240*y**3
        -10595*y**4+6405*y**5-2236*y**6+341*y**7)-10*r**8*(980-6860*y+20580*y**2
        -35398*y**3+38770*y**4-27219*y**5+11753*y**6-2819*y**7+285*y**8))/(450*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (6480*(-1+y)**8-1899*r**12*(-1+y)**8+r**2*(-1+y)**6*(26899-27120*logy*(
        -1+y)**2-80918*y+67579*y**2-9040*y**3-4045*y**4-160*y**5+375*y**6
        +150*y**7)+r**4*(-1+y)**4*(-37925-8400*logy*(-1+y)**4+143300*y-198150*y**2
        +115300*y**3-25210*y**4+10932*y**5-9638*y**6+2612*y**7+75*y**8)+6*r**6*(
        -1+y)**2*(1800-1050*logy*(-1+y)**6-11850*y+32775*y**2-48950*y**3
        +43630*y**4-27996*y**5+15017*y**6-5452*y**7+906*y**8)+2*r**8*(-1+y)*(8000
        +4050*logy*(-1+y)**7-60050*y+194325*y**2-352225*y**3+397420*y**4
        -306750*y**5+162713*y**6-53201*y**7+8718*y**8-390*y**9)-5*r**10*(-2329
        +18632*y-65212*y**2+130424*y**3-166246*y**4+143966*y**5-83728*y**6
        +30848*y**7-6421*y**8+570*y**9)-2*MBhat**2*(-1+y)*(-3240*(-1+y)**7
        +3165*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(835+600*logy*(-1+y)**2-1070*y
        -65*y**2+284*y**3+65*y**4-118*y**5-15*y**6)-12*r**4*(-1+y)**3*(200
        -800*y+1200*y**2-1035*y**3+915*y**4-597*y**5+171*y**6)-18*r**6*(-1
        +y)*(-450+150*logy*(-1+y)**6+2850*y-7575*y**2+11240*y**3-10595*y**4
        +6405*y**5-2236*y**6+341*y**7)+10*r**8*(980-6860*y+20580*y**2-35398*y**3
        +38770*y**4-27219*y**5+11753*y**6-2819*y**7+285*y**8))-6*MBhat*(2160*(
        -1+y)**8+1281*r**12*(-1+y)**8+r**2*(-1+y)**6*(169-4320*logy*(-1+y)**2
        -4658*y+6649*y**2-2330*y**3+455*y**4-75*y**5+65*y**6+5*y**7)-r**4*(
        -1+y)**4*(1625+300*logy*(-1+y)**4-6200*y+8700*y**2-3850*y**3-2595*y**4
        +2756*y**5-946*y**6+78*y**7)+5*r**10*(-857+6856*y-23996*y**2+49064*y**3
        -65181*y**4+57417*y**5-33152*y**6+12004*y**7-2450*y**8+211*y**9)-r**6*(
        -1+y)**2*(2700+300*logy*(-1+y)**6-15900*y+38850*y**2-52630*y**3
        +46755*y**4-25072*y**5+3974*y**6+2970*y**7-1667*y**8+260*y**9)-5*r**8*(
        -1000+8000*y-28000*y**2+57090*y**3-76159*y**4+68358*y**5-39859*y**6
        +13598*y**7-1845*y**8-278*y**9+95*y**10)))/(900*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (3240*(-1+y)**9-422*r**14*(-1+y)**9+r**2*(-1+y)**7*(56352-29160*logy*(
        -1+y)**2-141864*y+100092*y**2-9720*y**3-2430*y**4-2145*y**5+104*y**6
        -13*y**7+44*y**8)+r**4*(-1+y)**5*(-67928-39360*logy*(-1+y)**4+232352*y
        -269808*y**2+101152*y**3+14072*y**4-9729*y**5+2920*y**6-3058*y**7+600*y**8
        +75*y**9)+6*r**6*(-1+y)**3*(525+3900*logy*(-1+y)**6+750*y-13575*y**2
        +37600*y**3-47700*y**4+31657*y**5-13022*y**6+5187*y**7-1744*y**8+262*y**9)
        +2*r**8*(-1+y)**2*(-4300-4800*logy*(-1+y)**7+34900*y-121500*y**2
        +236100*y**3-278100*y**4+203637*y**5-105767*y**6+45275*y**7-12957*y**8
        +1632*y**9)+r**10*(-1+y)*(-7240+5400*logy*(-1+y)**8+63320*y-243220*y**2
        +536840*y**3-746650*y**4+683135*y**5-445108*y**6+207284*y**7-60784*y**8
        +9033*y**9-390*y**10)-9*r**12*(472-4248*y+16992*y**2-39648*y**3+59472*y**4
        -60149*y**5+42857*y**6-21128*y**7+6732*y**8-1223*y**9+95*y**10)+MBhat**4*(
        -1+y)**2*(3240*(-1+y)**7-3165*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(-835
        -600*logy*(-1+y)**2+1070*y+65*y**2-284*y**3-65*y**4+118*y**5+15*y**6)
        +12*r**4*(-1+y)**3*(200-800*y+1200*y**2-1035*y**3+915*y**4-597*y**5
        +171*y**6)+18*r**6*(-1+y)*(-450+150*logy*(-1+y)**6+2850*y-7575*y**2
        +11240*y**3-10595*y**4+6405*y**5-2236*y**6+341*y**7)-10*r**8*(980-6860*y
        +20580*y**2-35398*y**3+38770*y**4-27219*y**5+11753*y**6-2819*y**7+285*y**8))
        -6*MBhat**3*(-1+y)*(2160*(-1+y)**8+1281*r**12*(-1+y)**8+r**2*(-1+y)**6*(169
        -4320*logy*(-1+y)**2-4658*y+6649*y**2-2330*y**3+455*y**4-75*y**5
        +65*y**6+5*y**7)-r**4*(-1+y)**4*(1625+300*logy*(-1+y)**4-6200*y
        +8700*y**2-3850*y**3-2595*y**4+2756*y**5-946*y**6+78*y**7)+5*r**10*(
        -857+6856*y-23996*y**2+49064*y**3-65181*y**4+57417*y**5-33152*y**6
        +12004*y**7-2450*y**8+211*y**9)-r**6*(-1+y)**2*(2700+300*logy*(-1
        +y)**6-15900*y+38850*y**2-52630*y**3+46755*y**4-25072*y**5+3974*y**6
        +2970*y**7-1667*y**8+260*y**9)-5*r**8*(-1000+8000*y-28000*y**2+57090*y**3
        -76159*y**4+68358*y**5-39859*y**6+13598*y**7-1845*y**8-278*y**9+95*y**10))
        -2*MBhat*(6480*(-1+y)**9+915*r**14*(-1+y)**9+r**2*(-1+y)**7*(57534
        -39120*logy*(-1+y)**2-154188*y+116214*y**2-13040*y**3-5450*y**4-194*y**5
        -197*y**6+136*y**7+25*y**8)+r**4*(-1+y)**5*(-72265-30300*logy*(-1
        +y)**4+258760*y-327540*y**2+157760*y**3-12215*y**4+792*y**5-5074*y**6
        +928*y**7+150*y**8)+3*r**6*(-1+y)**3*(5875+900*logy*(-1+y)**6-34350*y
        +83175*y**2-106400*y**3+78140*y**4-40014*y**5+17865*y**6-5348*y**7
        +885*y**8-68*y**9)+2*r**8*(-1+y)**2*(9150+300*logy*(-1+y)**7-64350*y
        +194100*y**2-325600*y**3+332830*y**4-229617*y**5+105609*y**6-25779*y**7
        +1269*y**8+718*y**9-130*y**10)+3*r**12*(1753-15777*y+63108*y**2-147252*y**3
        +224263*y**4-238365*y**5+176334*y**6-88296*y**7+28404*y**8-5266*y**9
        +422*y**10)-10*r**10*(1327-11943*y+47772*y**2-111468*y**3+168813*y**4
        -177939*y**5+133998*y**6-70242*y**7+24138*y**8-4912*y**9+456*y**10))
        +MBhat**2*(19440*(-1+y)**9-4288*r**14*(-1+y)**9-6*r**4*(-1+y)**5*(15677
        +3840*logy*(-1+y)**4-58868*y+80622*y**2-45063*y**3+5912*y**4+914*y**5
        +374*y**6-191*y**7-25*y**8)+5*r**2*(-1+y)**7*(12824-14400*logy*(-1
        +y)**2-40048*y+34424*y**2-5958*y**3-606*y**4-306*y**5+127*y**6+46*y**7
        +y**8)-6*r**6*(-1+y)**3*(-3500+1800*logy*(-1+y)**6+22800*y-62400*y**2
        +89610*y**3-68125*y**4+29844*y**5-10266*y**6+2908*y**7-561*y**8+50*y**9)
        -6*r**12*(2305-20745*y+82980*y**2-197005*y**3+310080*y**4-332973*y**5
        +245070*y**6-121500*y**7+38610*y**8-7035*y**9+549*y**10)+2*r**8*(-1
        +y)**2*(8100+6900*logy*(-1+y)**7-63600*y+214950*y**2-401480*y**3
        +443855*y**4-350019*y**5+221278*y**6-107674*y**7+38115*y**8-8795*y**9
        +950*y**10)+15*r**10*(256-2304*y+9216*y**2-23286*y**3+43042*y**4-52948*y**5
        +38715*y**6-13523*y**7-1668*y**8+3782*y**9-1493*y**10+211*y**11)))/(450*r**2*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (45360*(-1+y)**10-3165*r**16*(-1+y)**10-r**2*(-1+y)**8*(-1779485+697200*logy*(
        -1+y)**2+4256170*y-2825285*y**2+232400*y**3+58100*y**4+23240*y**5
        +30394*y**6-3128*y**7+2014*y**8-300*y**9)+21*r**4*(-1+y)**6*(-45579
        -102480*logy*(-1+y)**4+79836*y+85206*y**2-261764*y**3+167921*y**4
        -20496*y**5-4420*y**6+976*y**7-1356*y**8+64*y**9+44*y**10)+63*r**6*(
        -1+y)**4*(-75745+33900*logy*(-1+y)**6+488370*y-1322625*y**2+1933000*y**3
        -1619250*y**4+749400*y**5-157120*y**6-1572*y**7+6915*y**8-1678*y**9
        +225*y**10)+105*r**10*(-1+y)**2*(1363-2700*logy*(-1+y)**8-13604*y
        +58414*y**2-142028*y**3+215335*y**4-210068*y**5+130700*y**6-57616*y**7
        +21092*y**8-5100*y**9+504*y**10)+63*r**12*(-1+y)*(1295+1500*logy*(
        -1+y)**9-13155*y+59370*y**2-156530*y**3+266295*y**4-304095*y**5+236002*y**6
        -133994*y**7+56016*y**8-14924*y**9+2006*y**10-78*y**11)-7*r**8*(-1
        +y)**3*(542675-311100*logy*(-1+y)**7-3487625*y+9374025*y**2-13445675*y**3
        +10723550*y**4-4256430*y**5+320840*y**6+361876*y**7-155013*y**8+30163*y**9
        -2701*y**10+75*y**11)-7*r**14*(-6737+67370*y-303165*y**2+808440*y**3
        -1414770*y**4+1697724*y**5-1423422*y**6+860640*y**7-367665*y**8+103070*y**9
        -16577*y**10+1140*y**11)-14*MBhat**6*(-1+y)**3*(-3240*(-1+y)**7+3165*r**10*(
        -1+y)**7+5*r**2*(-1+y)**5*(835+600*logy*(-1+y)**2-1070*y-65*y**2
        +284*y**3+65*y**4-118*y**5-15*y**6)-12*r**4*(-1+y)**3*(200-800*y
        +1200*y**2-1035*y**3+915*y**4-597*y**5+171*y**6)-18*r**6*(-1+y)*(
        -450+150*logy*(-1+y)**6+2850*y-7575*y**2+11240*y**3-10595*y**4+6405*y**5
        -2236*y**6+341*y**7)+10*r**8*(980-6860*y+20580*y**2-35398*y**3+38770*y**4
        -27219*y**5+11753*y**6-2819*y**7+285*y**8))-126*MBhat**5*(-1+y)**2*(2160*(
        -1+y)**8+1281*r**12*(-1+y)**8+r**2*(-1+y)**6*(169-4320*logy*(-1+y)**2
        -4658*y+6649*y**2-2330*y**3+455*y**4-75*y**5+65*y**6+5*y**7)-r**4*(
        -1+y)**4*(1625+300*logy*(-1+y)**4-6200*y+8700*y**2-3850*y**3-2595*y**4
        +2756*y**5-946*y**6+78*y**7)+5*r**10*(-857+6856*y-23996*y**2+49064*y**3
        -65181*y**4+57417*y**5-33152*y**6+12004*y**7-2450*y**8+211*y**9)-r**6*(
        -1+y)**2*(2700+300*logy*(-1+y)**6-15900*y+38850*y**2-52630*y**3
        +46755*y**4-25072*y**5+3974*y**6+2970*y**7-1667*y**8+260*y**9)-5*r**8*(
        -1000+8000*y-28000*y**2+57090*y**3-76159*y**4+68358*y**5-39859*y**6
        +13598*y**7-1845*y**8-278*y**9+95*y**10))+21*MBhat**4*(-1+y)*(32400*(
        -1+y)**9-8576*r**14*(-1+y)**9+r**2*(-1+y)**7*(101341-116880*logy*(
        -1+y)**2-319562*y+276661*y**2-50540*y**3-2015*y**4-2900*y**5+895*y**6
        +310*y**7+10*y**8)+r**4*(-1+y)**5*(-150199-37680*logy*(-1+y)**4+563116*y
        -769314*y**2+425456*y**3-45734*y**4-21900*y**5+5150*y**6-320*y**7+225*y**8)
        -6*r**6*(-1+y)**3*(-5200+2550*logy*(-1+y)**6+33750*y-92025*y**2+130270*y**3
        -92620*y**4+31692*y**5-5515*y**6+364*y**7-216*y**8+100*y**9)+2*r**8*(
        -1+y)**2*(8200+9750*logy*(-1+y)**7-67150*y+235575*y**2-450735*y**3
        +490290*y**4-393288*y**5+279843*y**6-162147*y**7+67512*y**8-17200*y**9
        +1900*y**10)-3*r**12*(9853-88677*y+354708*y**2-841192*y**3+1320078*y**4
        -1411650*y**5+1033452*y**6-508788*y**7+160137*y**8-28773*y**9+2196*y**10)
        +5*r**10*(3865-34785*y+139140*y**2-335352*y**3+554922*y**4-627900*y**5
        +459984*y**6-195714*y**7+27261*y**8+15701*y**9-8388*y**10+1266*y**11))
        -3*MBhat*(90720*(-1+y)**10+4758*r**16*(-1+y)**10-r**2*(-1+y)**8*(-2366268
        +1055040*logy*(-1+y)**2+5787576*y-3948828*y**2+351680*y**3+87920*y**4
        +64736*y**5+8393*y**6+3782*y**7-487*y**8-264*y**9)-7*r**4*(-1+y)**6*(269322
        +329640*logy*(-1+y)**4-747648*y+462192*y**2+351152*y**3-417428*y**4
        +70638*y**5+2559*y**6+6192*y**7+972*y**8-518*y**9-25*y**10)-14*r**8*(
        -1+y)**3*(126675-25500*logy*(-1+y)**7-861225*y+2494425*y**2-3978875*y**3
        +3755750*y**4-2081676*y**5+669603*y**6-133371*y**7+13581*y**8+59*y**9
        +94*y**10)+7*r**10*(-1+y)**2*(-53460+6000*logy*(-1+y)**8+433680*y
        -1541880*y**2+3139760*y**3-4008700*y**4+3299828*y**5-1886667*y**6+746316*y**7
        -164706*y**8+12172*y**9+1237*y**10-260*y**11)-14*r**6*(-1+y)**4*(150772
        -89760*logy*(-1+y)**6-994392*y+2755260*y**2-4122480*y**3+3540660*y**4
        -1697373*y**5+415137*y**6-54735*y**7+9840*y**8-2119*y**9+135*y**10
        +15*y**11)+42*r**14*(-988+9880*y-44460*y**2+118560*y**3-207480*y**4
        +251139*y**5-221102*y**6+139895*y**7-61230*y**8+17475*y**9-2908*y**10
        +211*y**11)+7*r**12*(25226-252260*y+1135170*y**2-3027120*y**3+5297460*y**4
        -6380070*y**5+5519097*y**6-3510588*y**7+1600365*y**8-491930*y**9+93821*y**10
        -9456*y**11+285*y**12))-3*MBhat**2*(-226800*(-1+y)**10+11792*r**16*(
        -1+y)**10+r**2*(-1+y)**8*(-3588722+1910160*logy*(-1+y)**2+9087604*y
        -6453962*y**2+636720*y**3+213150*y**4+65730*y**5+11613*y**6+354*y**7
        -1737*y**8-150*y**9)+28*r**8*(-1+y)**3*(8600+3600*logy*(-1+y)**7-63800*y
        +204000*y**2-365200*y**3+391285*y**4-215532*y**5+63891*y**6-24297*y**7
        +6534*y**8+29*y**9-110*y**10)+7*r**4*(-1+y)**6*(529936+385920*logy*(
        -1+y)**4-1733824*y+1828896*y**2-447424*y**3-265574*y**4+64410*y**5
        +13265*y**6+5800*y**7-1800*y**8-170*y**9+5*y**10)-14*r**6*(-1+y)**4*(
        -33697+63660*logy*(-1+y)**6+265842*y-855585*y**2+1459080*y**3-1391265*y**4
        +677916*y**5-120831*y**6-10770*y**7+9756*y**8-2396*y**9+150*y**10)
        -7*r**10*(-1+y)**2*(20850+25800*logy*(-1+y)**8-141000*y+390300*y**2
        -539800*y**3+322940*y**4-107410*y**5-104779*y**6+218702*y**7-136432*y**8
        +44414*y**9-9055*y**10+950*y**11)+14*r**14*(-4393+43930*y-197685*y**2
        +527160*y**3-933345*y**4+1179150*y**5-1065981*y**6+681780*y**7-300960*y**8
        +87010*y**9-14740*y**10+1098*y**11)-7*r**12*(-10096+100960*y-454320*y**2
        +1211520*y**3-2156010*y**4+2871114*y**5-2863797*y**6+2006964*y**7-945015*y**8
        +280690*y**9-42787*y**10+144*y**11+633*y**12))-MBhat**3*(907200*(-1
        +y)**10+60930*r**16*(-1+y)**10+21*r**4*(-1+y)**6*(-444832-194640*logy*(
        -1+y)**4+1584688*y-1987752*y**2+928948*y**3-33577*y**4-29670*y**5
        -12550*y**6+3100*y**7+255*y**8+30*y**9)+r**2*(-1+y)**8*(7497684-5214720*logy*(
        -1+y)**2-20210088*y+15319764*y**2-1874320*y**3-537145*y**4-77392*y**5
        -17451*y**6+13848*y**7+2670*y**8+30*y**9)+126*r**6*(-1+y)**4*(15995
        +3300*logy*(-1+y)**6-92670*y+221775*y**2-273370*y**3+172725*y**4
        -51708*y**5+369*y**6+9342*y**7-3624*y**8+366*y**9)+14*r**8*(-1+y)**3*(170275
        +24300*logy*(-1+y)**7-1216225*y+3733725*y**2-6373595*y**3+6485615*y**4
        -4253085*y**5+1759967*y**6-358205*y**7+14454*y**8+134*y**9+940*y**10)
        -21*r**10*(-1+y)**2*(-79300+634400*y-2220400*y**2+4445320*y**3-5558855*y**4
        +4806742*y**5-2953026*y**6+1296726*y**7-432207*y**8+116230*y**9-22460*y**10
        +2110*y**11)+14*r**14*(-11327+113270*y-509715*y**2+1380870*y**3-2534910*y**4
        +3252858*y**5-2958522*y**6+1902840*y**7-846225*y**8+247010*y**9-42389*y**10
        +3216*y**11)-63*r**12*(4168-41680*y+187560*y**2-493980*y**3+830235*y**4
        -983424*y**5+880095*y**6-609486*y**7+332235*y**8-143900*y**9+47489*y**10
        -10410*y**11+1098*y**12)))/(6300*r**2*(-1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (45360*(-1+y)**11-1899*r**18*(-1+y)**11-r**2*(-1+y)**9*(-3191659+1056720*logy*(
        -1+y)**2+7440038*y-4776739*y**2+352240*y**3+88060*y**4+35224*y**5
        +17612*y**6+30320*y**7-4309*y**8+3450*y**9-117*y**10)-8*r**4*(-1+y)**7*(
        -264392+729960*logy*(-1+y)**4+1787528*y-4141212*y**2+4220728*y**3
        -1785142*y**4+145992*y**5+24332*y**6+9424*y**7-1569*y**8+2651*y**9
        +601*y**10-75*y**11)+42*r**6*(-1+y)**5*(-602602+144060*logy*(-1+y)**6
        +3759672*y-9831360*y**2+13828780*y**3-11091885*y**4+4868934*y**5-955549*y**6
        +24700*y**7-10863*y**8+12208*y**9-2365*y**10+210*y**11)+168*r**12*(
        -1+y)**2*(-712-3000*logy*(-1+y)**9+9408*y-51132*y**2+155308*y**3
        -295962*y**4+371562*y**5-310708*y**6+168400*y**7-64689*y**8+20941*y**9
        -4453*y**10+357*y**11)+14*r**14*(-1+y)*(-3874+8100*logy*(-1+y)**10
        +46840*y-251280*y**2+791580*y**3-1628265*y**4+2294118*y**5-2251965*y**6
        +1531884*y**7-777135*y**8+297340*y**9-73009*y**10+8958*y**11-312*y**12)
        +14*r**10*(-1+y)**3*(1307360+543300*logy*(-1+y)**8-9915580*y+32531330*y**2
        -59991860*y**3+67383625*y**4-46300700*y**5+18079550*y**6-3001780*y**7
        -256114*y**8+171652*y**9-20612*y**10+604*y**11-75*y**12)-28*r**8*(-1
        +y)**4*(55408-838560*logy*(-1+y)**7+450704*y-4287072*y**2+13015040*y**3
        -20352440*y**4+18081384*y**5-8962088*y**6+2114444*y**7-69552*y**8-47791*y**9
        +4213*y**10-525*y**11+75*y**12)-4*r**16*(9796-107756*y+538780*y**2
        -1616340*y**3+3232680*y**4-4525752*y**5+4525752*y**6-3241260*y**7+1701540*y**8
        -641005*y**9+160391*y**10-23131*y**11+1425*y**12)-14*MBhat**8*(-1+y)**4*(
        -3240*(-1+y)**7+3165*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(835+600*logy*(
        -1+y)**2-1070*y-65*y**2+284*y**3+65*y**4-118*y**5-15*y**6)-12*r**4*(
        -1+y)**3*(200-800*y+1200*y**2-1035*y**3+915*y**4-597*y**5+171*y**6)
        -18*r**6*(-1+y)*(-450+150*logy*(-1+y)**6+2850*y-7575*y**2+11240*y**3
        -10595*y**4+6405*y**5-2236*y**6+341*y**7)+10*r**8*(980-6860*y+20580*y**2
        -35398*y**3+38770*y**4-27219*y**5+11753*y**6-2819*y**7+285*y**8))-168*MBhat**7*(
        -1+y)**3*(2160*(-1+y)**8+1281*r**12*(-1+y)**8+r**2*(-1+y)**6*(169
        -4320*logy*(-1+y)**2-4658*y+6649*y**2-2330*y**3+455*y**4-75*y**5
        +65*y**6+5*y**7)-r**4*(-1+y)**4*(1625+300*logy*(-1+y)**4-6200*y
        +8700*y**2-3850*y**3-2595*y**4+2756*y**5-946*y**6+78*y**7)+5*r**10*(
        -857+6856*y-23996*y**2+49064*y**3-65181*y**4+57417*y**5-33152*y**6
        +12004*y**7-2450*y**8+211*y**9)-r**6*(-1+y)**2*(2700+300*logy*(-1
        +y)**6-15900*y+38850*y**2-52630*y**3+46755*y**4-25072*y**5+3974*y**6
        +2970*y**7-1667*y**8+260*y**9)-5*r**8*(-1000+8000*y-28000*y**2+57090*y**3
        -76159*y**4+68358*y**5-39859*y**6+13598*y**7-1845*y**8-278*y**9+95*y**10))
        +28*MBhat**6*(-1+y)**2*(45360*(-1+y)**9-12864*r**14*(-1+y)**9-2*r**4*(
        -1+y)**5*(103168+26160*logy*(-1+y)**4-386512*y+527448*y**2-290267*y**3
        +27998*y**4+19158*y**5-6272*y**6+893*y**7-150*y**8)+r**2*(-1+y)**7*(138562
        -161760*logy*(-1+y)**2-438884*y+381202*y**2-71290*y**3-1000*y**4
        -4270*y**5+1155*y**6+390*y**7+15*y**8)-6*r**6*(-1+y)**3*(-6900+3300*logy*(
        -1+y)**6+44700*y-121650*y**2+170930*y**3-117115*y**4+33540*y**5-764*y**6
        -2180*y**7+129*y**8+150*y**9)-18*r**12*(2516-22644*y+90576*y**2-214729*y**3
        +336666*y**4-359559*y**5+262794*y**6-129096*y**7+40509*y**8-7246*y**9
        +549*y**10)+2*r**8*(-1+y)**2*(8300+12600*logy*(-1+y)**7-70700*y+256200*y**2
        -499990*y**3+536725*y**4-436557*y**5+338408*y**6-216620*y**7+96909*y**8
        -25605*y**9+2850*y**10)+5*r**10*(6962-62658*y+250632*y**2-600846*y**3
        +980718*y**4-1096956*y**5+803823*y**6-350859*y**7+59526*y**8+20056*y**9
        -12297*y**10+1899*y**11))-4*MBhat**5*(-1+y)*(635040*(-1+y)**10+60930*r**16*(
        -1+y)**10-21*r**4*(-1+y)**6*(300302+134040*logy*(-1+y)**4-1067168*y
        +1332672*y**2-613428*y**3+9147*y**4+31254*y**5+2402*y**6-1244*y**7
        +45*y**8-30*y**9)+r**2*(-1+y)**8*(5081256-3571680*logy*(-1+y)**2-13734192*y
        +10438776*y**2-1326640*y**3-308245*y**4-69244*y**5-9177*y**6+8136*y**7
        +1620*y**8+30*y**9)+126*r**6*(-1+y)**4*(10120+2400*logy*(-1+y)**6
        -58320*y+138600*y**2-166970*y**3+94585*y**4-11694*y**5-17496*y**6
        +14690*y**7-4509*y**8+434*y**9)+14*r**8*(-1+y)**3*(115375+22500*logy*(
        -1+y)**7-830125*y+2569125*y**2-4419995*y**3+4488635*y**4-2875383*y**5
        +1126313*y**6-203531*y**7+6840*y**8-4174*y**9+1720*y**10)+28*r**14*(
        -7036+70360*y-316620*y**2+855135*y**3-1555680*y**4+1972299*y**5-1767486*y**6
        +1116120*y**7-484875*y**8+137230*y**9-22567*y**10+1608*y**11)-21*r**10*(
        -1+y)**2*(-52760+422080*y-1477280*y**2+2959080*y**3-3668835*y**4+3137982*y**5
        -1941826*y**6+903086*y**7-343087*y**8+107110*y**9-22460*y**10+2110*y**11)
        -63*r**12*(662-6620*y+29790*y**2-73260*y**3+87205*y**4-58168*y**5
        +50697*y**6-80226*y**7+98835*y**8-76560*y**9+36113*y**10-9566*y**11
        +1098*y**12))-4*MBhat**2*(-317520*(-1+y)**11+5628*r**18*(-1+y)**11+r**2*(
        -1+y)**9*(-11810888+4683840*logy*(-1+y)**2+28305616*y-18836648*y**2
        +1561280*y**3+390320*y**4+224798*y**5+98875*y**6+20236*y**7+6433*y**8
        -984*y**9-198*y**10)+21*r**4*(-1+y)**7*(165094+713280*logy*(-1+y)**4
        +52904*y-1505916*y**2+2430504*y**3-1320906*y**4+146052*y**5+19981*y**6
        +5204*y**7+4398*y**8-192*y**9-147*y**10)-21*r**10*(-1+y)**3*(49780
        +6000*logy*(-1+y)**8-392240*y+1348840*y**2-2641680*y**3+3218100*y**4
        -2509890*y**5+1053685*y**6-212156*y**7+70948*y**8-22668*y**9+471*y**10
        +270*y**11)-14*r**8*(-1+y)**4*(-1039777+669540*logy*(-1+y)**7+6608899*y
        -17483307*y**2+24452065*y**3-18593590*y**4+6503124*y**5-44683*y**6-511451*y**7
        +69033*y**8+27559*y**9-8332*y**10+300*y**11)-42*r**6*(-1+y)**5*(-589408
        +163440*logy*(-1+y)**6+3699888*y-9740040*y**2+13803920*y**3-11170140*y**4
        +4972965*y**5-1034382*y**6+53235*y**7+1347*y**8+1525*y**9+360*y**10
        -105*y**11-5*y**12)-21*r**12*(-1+y)**2*(-26022+13800*logy*(-1+y)**9
        +220398*y-819492*y**2+1746548*y**3-2330022*y**4+1961202*y**5-1178439*y**6
        +439203*y**7-36150*y**8-30000*y**9+10803*y**10-1899*y**11+190*y**12)
        +18*r**16*(2377-26147*y+130735*y**2-392205*y**3+784410*y**4-1104180*y**5
        +1158528*y**6-898368*y**7+503745*y**8-198110*y**9+51700*y**10-7994*y**11
        +549*y**12)-14*r**14*(6584-72424*y+362120*y**2-1086360*y**3+2172720*y**4
        -3047037*y**5+3258486*y**6-2717844*y**7+1664670*y**8-712745*y**9+204886*y**10
        -35942*y**11+2886*y**12))-12*MBhat*(30240*(-1+y)**11+732*r**18*(-1
        +y)**11-r**2*(-1+y)**9*(-1582188+567840*logy*(-1+y)**2+3732216*y
        -2433948*y**2+189280*y**3+47320*y**4+18928*y**5+19187*y**6+3566*y**7
        +1505*y**8+264*y**9-50*y**10)-2*r**4*(-1+y)**7*(-120412+1219260*logy*(
        -1+y)**4+1700908*y-4989882*y**2+5765108*y**3-2660537*y**4+243852*y**5
        +45507*y**6+4548*y**7+5041*y**8+3236*y**9-349*y**10-44*y**11)+7*r**10*(
        -1+y)**3*(291080+60000*logy*(-1+y)**8-2268640*y+7700240*y**2-14840480*y**3
        +17710600*y**4-13328480*y**5+6095525*y**6-1576132*y**7+209362*y**8-3724*y**9
        -2815*y**10+24*y**11)+14*r**12*(-1+y)**2*(11092+2100*logy*(-1+y)**9
        -101928*y+417162*y**2-998578*y**3+1541967*y**4-1594887*y**5+1102941*y**6
        -542529*y**7+190639*y**8-38101*y**9+2733*y**10+107*y**11-26*y**12)
        +7*r**6*(-1+y)**5*(-947560+232800*logy*(-1+y)**6+5918160*y-15493800*y**2
        +21822400*y**3-17530800*y**4+7710720*y**5-1505847*y**6-5070*y**7+34217*y**8
        -3500*y**9+495*y**10+130*y**11-25*y**12)+28*r**8*(-1+y)**4*(-101731
        +145620*logy*(-1+y)**7+566497*y-1189821*y**2+963695*y**3+310480*y**4
        -1205628*y**5+913771*y**6-292465*y**7+33527*y**8+252*y**9+463*y**10
        -115*y**11-5*y**12)+4*r**16*(2245-24695*y+123475*y**2-370425*y**3
        +740850*y**4-1037190*y**5+1041194*y**6-780814*y**7+430610*y**8-166980*y**9
        +42680*y**10-6412*y**11+422*y**12)+7*r**14*(-8168+89848*y-449240*y**2
        +1347720*y**3-2695440*y**4+3773616*y**5-3775391*y**6+2768101*y**7-1519231*y**8
        +608965*y**9-167013*y**10+28903*y**11-2765*y**12+95*y**13))-4*MBhat**3*(635040*(
        -1+y)**11+10155*r**18*(-1+y)**11+r**2*(-1+y)**9*(15793275-7123200*logy*(
        -1+y)**2-38709750*y+26478075*y**2-2374400*y**3-676655*y**4-329966*y**5
        -79758*y**6-22008*y**7+1944*y**8+1488*y**9+75*y**10)+14*r**8*(-1+y)**4*(
        -625855+193500*logy*(-1+y)**7+4187485*y-11885205*y**2+18454175*y**3
        -16715300*y**4+8419578*y**5-2030387*y**6+66227*y**7+136041*y**8-46265*y**9
        +4226*y**10)+7*r**10*(-1+y)**3*(-136570+90000*logy*(-1+y)**8+1182560*y
        -4498960*y**2+9837920*y**3-13620775*y**4+11773784*y**5-6993305*y**6
        +2838154*y**7-559427*y**8+3952*y**9+9157*y**10-290*y**11)+21*r**6*(
        -1+y)**5*(-677786+267480*logy*(-1+y)**6+4334196*y-11637930*y**2+16854640*y**3
        -13941485*y**4+6287916*y**5-1280149*y**6+42038*y**7+13575*y**8+2900*y**9
        -1305*y**10+30*y**11)-r**4*(-1+y)**7*(10648146+15970920*logy*(-1+y)**4
        -26621664*y+7990656*y**2+26614736*y**3-22532834*y**4+3138366*y**5+413035*y**6
        +223862*y**7+11262*y**8-12378*y**9-225*y**10+30*y**11)-21*r**12*(-1
        +y)**2*(33850-304650*y+1218600*y**2-2843400*y**3+4291130*y**4-4183416*y**5
        +2891283*y**6-1455843*y**7+515256*y**8-133104*y**9+28387*y**10-4755*y**11
        +422*y**12)+2*r**16*(21697-238667*y+1193335*y**2-3580005*y**3+7220070*y**4
        -10633266*y**5+11472090*y**6-9044574*y**7+5159055*y**8-2073445*y**9
        +556193*y**10-89155*y**11+6432*y**12)-7*r**14*(-4322+47542*y-237710*y**2
        +713130*y**3-1418025*y**4+1622469*y**5-1062225*y**6+440127*y**7-125601*y**8
        +47135*y**9-41965*y**10+26963*y**11-8616*y**12+1098*y**13))-3*MBhat**4*(
        -1058400*(-1+y)**11+18025*r**18*(-1+y)**11+28*r**4*(-1+y)**7*(568184
        +443280*logy*(-1+y)**4-1829456*y+1857624*y**2-348776*y**3-354481*y**4
        +81010*y**5+16950*y**6+2620*y**7-1205*y**8-30*y**9)+r**2*(-1+y)**9*(
        -16086025+8652000*logy*(-1+y)**2+40824050*y-29064025*y**2+2952880*y**3
        +911050*y**4+270340*y**5+60522*y**6+1016*y**7-6293*y**8-710*y**9-5*y**10)
        +56*r**8*(-1+y)**4*(4975-4500*logy*(-1+y)**7-30325*y+75225*y**2-105625*y**3
        +119575*y**4-51104*y**5+59964*y**6-101748*y**7+49685*y**8-8462*y**9
        +440*y**10)-14*r**6*(-1+y)**5*(-204848+242640*logy*(-1+y)**6+1471728*y
        -4407240*y**2+7139840*y**3-6658755*y**4+3365678*y**5-797053*y**6+72000*y**7
        +19375*y**8-9950*y**9+825*y**10)+28*r**10*(-1+y)**3*(-68800-23250*logy*(
        -1+y)**8+527150*y-1752025*y**2+3296950*y**3-3754250*y**4+2859204*y**5
        -1317741*y**6+230134*y**7+52684*y**8-28596*y**9+3560*y**10+30*y**11)
        -28*r**12*(-1+y)**2*(30400-273600*y+1094400*y**2-2570560*y**3+3876495*y**4
        -4192925*y**5+3223850*y**6-1742388*y**7+677541*y**8-199619*y**9+46926*y**10
        -8212*y**11+732*y**12)+4*r**16*(4318-47498*y+237490*y**2-732490*y**3
        +1630880*y**4-2616502*y**5+3063522*y**6-2635050*y**7+1652640*y**8-737055*y**9
        +221881*y**10-40481*y**11+3385*y**12)-14*r**14*(-19968+219648*y-1098240*y**2
        +3295360*y**3-6548915*y**4+9281885*y**5-9716201*y**6+7561099*y**7-4381443*y**8
        +1896015*y**9-614735*y**10+147993*y**11-24642*y**12+2144*y**13)))/(6300*r**2*(
        -1+y)**11)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr4(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r4 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-3060*(-1+y)**7+4410*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(-1310+60*logy*(
        -1+y)**2+2680*y-1400*y**2-316*y**3+335*y**4-58*y**5+15*y**6)-6*r**4*(
        -1+y)**3*(600-2400*y+3600*y**2-2380*y**3+795*y**4-256*y**5+83*y**6)
        -3*r**6*(-1+y)*(-2400+900*logy*(-1+y)**6+15300*y-40950*y**2+62560*y**3
        -60005*y**4+36270*y**5-12559*y**6+1904*y**7)+10*r**8*(1150-8050*y
        +24150*y**2-42032*y**3+46265*y**4-32721*y**5+14287*y**6-3481*y**7+360*y**8))/(450*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-6120*(-1+y)**8+2646*r**12*(-1+y)**8+12*r**2*(-1+y)**6*(517+990*logy*(
        -1+y)**2-44*y-968*y**2+330*y**3+365*y**4-195*y**5+55*y**6-15*y**7)
        +3*r**6*(-1+y)**2*(900+3600*logy*(-1+y)**6-1800*y-6300*y**2+26400*y**3
        -41515*y**4+37758*y**5-22266*y**6+7846*y**7-1263*y**8)-r**4*(-1+y)**4*(6950
        +5700*logy*(-1+y)**4-22100*y+21750*y**2-3100*y**3-2915*y**4+168*y**5
        -412*y**6+88*y**7+75*y**8)+3*r**8*(-1+y)*(-5000-2700*logy*(-1+y)**7
        +37700*y-122550*y**2+223150*y**3-260795*y**4+207025*y**5-111858*y**6
        +37466*y**7-6413*y**8+315*y**9)+5*r**10*(-2696+21568*y-75488*y**2
        +150976*y**3-195179*y**4+169924*y**5-99632*y**6+37192*y**7-7889*y**8
        +720*y**9)-2*MBhat**2*(-1+y)*(3060*(-1+y)**7-4410*r**10*(-1+y)**7
        +5*r**2*(-1+y)**5*(-1310+60*logy*(-1+y)**2+2680*y-1400*y**2-316*y**3
        +335*y**4-58*y**5+15*y**6)+6*r**4*(-1+y)**3*(600-2400*y+3600*y**2
        -2380*y**3+795*y**4-256*y**5+83*y**6)+3*r**6*(-1+y)*(-2400+900*logy*(
        -1+y)**6+15300*y-40950*y**2+62560*y**3-60005*y**4+36270*y**5-12559*y**6
        +1904*y**7)-10*r**8*(1150-8050*y+24150*y**2-42032*y**3+46265*y**4
        -32721*y**5+14287*y**6-3481*y**7+360*y**8))+2*MBhat*(6120*(-1+y)**8
        +6237*r**12*(-1+y)**8+r**2*(-1+y)**6*(-15337-2040*logy*(-1+y)**2+28634*y
        -12277*y**2-4685*y**3+3760*y**4-770*y**5+120*y**6+15*y**7)+3*r**4*(
        -1+y)**4*(3975+600*logy*(-1+y)**4-15300*y+21750*y**2-12650*y**3
        +2055*y**4+456*y**5-196*y**6+78*y**7)+15*r**10*(-1263+10104*y-35364*y**2
        +72881*y**3-96544*y**4+84563*y**5-48468*y**6+17376*y**7-3495*y**8+294*y**9)
        -3*r**6*(-1+y)**2*(2850-17100*y+42750*y**2-59470*y**3+48395*y**4-19278*y**5
        -2249*y**6+5905*y**7-2358*y**8+315*y**9)-5*r**8*(-3710+29680*y-103880*y**2
        +215290*y**3-290603*y**4+260896*y**5-151238*y**6+50986*y**7-6665*y**8
        -1116*y**9+360*y**10)))/(900*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-6120*(-1+y)**9+1176*r**14*(-1+y)**9+r**2*(-1+y)**7*(-34826+32280*logy*(
        -1+y)**2+101932*y-83246*y**2+10760*y**3+2690*y**4+4280*y**5-1587*y**6
        +654*y**7-117*y**8)+4*r**8*(-1+y)**2*(-10100+8100*logy*(-1+y)**7+62600*y
        -159450*y**2+209050*y**3-138175*y**4+16287*y**5+44833*y**6-36925*y**7
        +12468*y**8-1668*y**9)-2*r**4*(-1+y)**5*(-42787+6060*logy*(-1+y)**4
        +177208*y-277932*y**2+197408*y**3-55412*y**4+2994*y**5-1630*y**6+748*y**7
        -435*y**8+90*y**9)-6*r**6*(-1+y)**3*(15000-600*logy*(-1+y)**6-90600*y
        +228300*y**2-307400*y**3+233550*y**4-94464*y**5+13569*y**6+3726*y**7
        -1737*y**8+176*y**9)+18*r**12*(543-4887*y+19548*y**2-45612*y**3+68418*y**4
        -70236*y**5+50298*y**6-24992*y**7+8073*y**8-1497*y**9+120*y**10)+r**10*(
        -1+y)*(13570-10800*logy*(-1+y)**8-119360*y+460960*y**2-1022720*y**3
        +1429600*y**4-1362380*y**5+917359*y**6-437432*y**7+132682*y**8-20784*y**9
        +945*y**10)-2*MBhat**4*(-1+y)**2*(3060*(-1+y)**7-4410*r**10*(-1+y)**7
        +5*r**2*(-1+y)**5*(-1310+60*logy*(-1+y)**2+2680*y-1400*y**2-316*y**3
        +335*y**4-58*y**5+15*y**6)+6*r**4*(-1+y)**3*(600-2400*y+3600*y**2
        -2380*y**3+795*y**4-256*y**5+83*y**6)+3*r**6*(-1+y)*(-2400+900*logy*(
        -1+y)**6+15300*y-40950*y**2+62560*y**3-60005*y**4+36270*y**5-12559*y**6
        +1904*y**7)-10*r**8*(1150-8050*y+24150*y**2-42032*y**3+46265*y**4
        -32721*y**5+14287*y**6-3481*y**7+360*y**8))+4*MBhat**3*(-1+y)*(6120*(
        -1+y)**8+6237*r**12*(-1+y)**8+r**2*(-1+y)**6*(-15337-2040*logy*(-1
        +y)**2+28634*y-12277*y**2-4685*y**3+3760*y**4-770*y**5+120*y**6+15*y**7)
        +3*r**4*(-1+y)**4*(3975+600*logy*(-1+y)**4-15300*y+21750*y**2-12650*y**3
        +2055*y**4+456*y**5-196*y**6+78*y**7)+15*r**10*(-1263+10104*y-35364*y**2
        +72881*y**3-96544*y**4+84563*y**5-48468*y**6+17376*y**7-3495*y**8+294*y**9)
        -3*r**6*(-1+y)**2*(2850-17100*y+42750*y**2-59470*y**3+48395*y**4-19278*y**5
        -2249*y**6+5905*y**7-2358*y**8+315*y**9)-5*r**8*(-3710+29680*y-103880*y**2
        +215290*y**3-290603*y**4+260896*y**5-151238*y**6+50986*y**7-6665*y**8
        -1116*y**9+360*y**10))+4*MBhat*(6120*(-1+y)**9+1485*r**14*(-1+y)**9
        +r**2*(-1+y)**7*(4396-18480*logy*(-1+y)**2-27272*y+32116*y**2-6160*y**3
        -4450*y**4+1049*y**5-258*y**6+9*y**7+30*y**8)+3*r**4*(-1+y)**5*(-4795
        +3300*logy*(-1+y)**4+22480*y-40320*y**2+33480*y**3-10745*y**4-264*y**5
        +458*y**6-176*y**7+50*y**8)+3*r**6*(-1+y)**3*(5325-3900*logy*(-1+y)**6
        -35850*y+101325*y**2-154600*y**3+136585*y**4-69436*y**5+18560*y**6
        -1177*y**7-635*y**8+143*y**9)+9*r**12*(819-7371*y+29484*y**2-68796*y**3
        +106224*y**4-112950*y**5+83362*y**6-41628*y**7+13347*y**8-2463*y**9
        +196*y**10)-r**8*(-1+y)**2*(-21100+147700*y-443100*y**2+738500*y**3
        -769265*y**4+519141*y**5-221632*y**6+47917*y**7+138*y**8-2214*y**9
        +315*y**10)-15*r**10*(992-8928*y+35712*y**2-83328*y**3+128643*y**4
        -138984*y**5+106553*y**6-56812*y**7+19893*y**8-4132*y**9+391*y**10))
        +2*MBhat**2*(-18360*(-1+y)**9+8612*r**14*(-1+y)**9-5*r**2*(-1+y)**7*(
        -6276-5040*logy*(-1+y)**2+7512*y+1284*y**2-3132*y**3+111*y**4+222*y**5
        -92*y**6+46*y**7+y**8)-r**4*(-1+y)**5*(28837+13740*logy*(-1+y)**4
        -101608*y+124932*y**2-51428*y**3-1153*y**4+684*y**5+1594*y**6-496*y**7
        +150*y**8)+3*r**6*(-1+y)**3*(3700+6000*logy*(-1+y)**6-16200*y+22500*y**2
        -1520*y**3-30625*y**4+41762*y**5-30768*y**6+13534*y**7-3503*y**8+400*y**9)
        +3*r**12*(9685-87165*y+348660*y**2-831720*y**3+1296360*y**4-1370664*y**5
        +988680*y**6-477180*y**7+146025*y**8-25135*y**9+1782*y**10)-r**8*(-1
        +y)**2*(3100+13800*logy*(-1+y)**7-35500*y+154800*y**2-306840*y**3
        +415815*y**4-463737*y**5+420494*y**6-262202*y**7+104295*y**8-23985*y**9
        +2400*y**10)-5*r**10*(4412-39708*y+158832*y**2-390732*y**3+636234*y**4
        -694026*y**5+495795*y**6-213351*y**7+38979*y**8+8479*y**9-5796*y**10
        +882*y**11)))/(900*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-28*MBhat**6*(-1+y)**3*(3060*(-1+y)**7-4410*r**10*(-1+y)**7+5*r**2*(
        -1+y)**5*(-1310+60*logy*(-1+y)**2+2680*y-1400*y**2-316*y**3+335*y**4
        -58*y**5+15*y**6)+6*r**4*(-1+y)**3*(600-2400*y+3600*y**2-2380*y**3
        +795*y**4-256*y**5+83*y**6)+3*r**6*(-1+y)*(-2400+900*logy*(-1+y)**6
        +15300*y-40950*y**2+62560*y**3-60005*y**4+36270*y**5-12559*y**6+1904*y**7)
        -10*r**8*(1150-8050*y+24150*y**2-42032*y**3+46265*y**4-32721*y**5
        +14287*y**6-3481*y**7+360*y**8))+84*MBhat**5*(-1+y)**2*(6120*(-1+y)**8
        +6237*r**12*(-1+y)**8+r**2*(-1+y)**6*(-15337-2040*logy*(-1+y)**2+28634*y
        -12277*y**2-4685*y**3+3760*y**4-770*y**5+120*y**6+15*y**7)+3*r**4*(
        -1+y)**4*(3975+600*logy*(-1+y)**4-15300*y+21750*y**2-12650*y**3
        +2055*y**4+456*y**5-196*y**6+78*y**7)+15*r**10*(-1263+10104*y-35364*y**2
        +72881*y**3-96544*y**4+84563*y**5-48468*y**6+17376*y**7-3495*y**8+294*y**9)
        -3*r**6*(-1+y)**2*(2850-17100*y+42750*y**2-59470*y**3+48395*y**4-19278*y**5
        -2249*y**6+5905*y**7-2358*y**8+315*y**9)-5*r**8*(-3710+29680*y-103880*y**2
        +215290*y**3-290603*y**4+260896*y**5-151238*y**6+50986*y**7-6665*y**8
        -1116*y**9+360*y**10))+42*MBhat**4*(-1+y)*(-30600*(-1+y)**9+17224*r**14*(
        -1+y)**9+2*r**2*(-1+y)**7*(28278+19260*logy*(-1+y)**2-37296*y-612*y**2
        +13680*y**3-2745*y**4+60*y**5+130*y**6-140*y**7-5*y**8)-3*r**4*(-1
        +y)**5*(16908+7260*logy*(-1+y)**4-60372*y+76038*y**2-33252*y**3+203*y**4
        +400*y**5+1200*y**6-360*y**7+75*y**8)+3*r**6*(-1+y)**3*(6500+8400*logy*(
        -1+y)**6-30600*y+51300*y**2-29440*y**3-19735*y**4+45766*y**5-39270*y**6
        +19222*y**7-5743*y**8+800*y**9)+12*r**12*(5063-45567*y+182268*y**2
        -434382*y**3+675963*y**4-713115*y**5+512862*y**6-246528*y**7+74997*y**8
        -12788*y**9+891*y**10)-r**8*(-1+y)**2*(-8800+19500*logy*(-1+y)**7
        +42100*y-58050*y**2+55770*y**3+49245*y**4-306399*y**5+505414*y**6
        -412006*y**7+189351*y**8-47025*y**9+4800*y**10)-15*r**10*(3840-34560*y
        +138240*y**2-335976*y**3+539541*y**4-584385*y**5+420382*y**6-187842*y**7
        +41013*y**8+2783*y**9-3624*y**10+588*y**11))-7*(12240*(-1+y)**10-1260*r**16*(
        -1+y)**10-r**2*(-1+y)**8*(-230270+121200*logy*(-1+y)**2+581740*y
        -412070*y**2+40400*y**3+10100*y**4+4040*y**5+7741*y**6-1802*y**7+1321*y**8
        -120*y**9)-15*r**10*(-1+y)**2*(15906+9000*logy*(-1+y)**8-118248*y
        +377868*y**2-671736*y**3+713670*y**4-444936*y**5+125615*y**6+24208*y**7
        -34126*y**8+11040*y**9-1277*y**10)-3*r**4*(-1+y)**6*(210732+6840*logy*(
        -1+y)**4-836088*y+1240452*y**2-813288*y**3+196482*y**4+1368*y**5
        -2485*y**6+2532*y**7-832*y**8+908*y**9-117*y**10)+6*r**6*(-1+y)**4*(28935
        +78300*logy*(-1+y)**6-95310*y+3375*y**2+387000*y**3-681750*y**4+507600*y**5
        -162715*y**6+8066*y**7+6705*y**8-1616*y**9-50*y**10)+3*r**12*(-1+y)*(7140
        +9000*logy*(-1+y)**9-73260*y+333540*y**2-886260*y**3+1518390*y**4
        -1745190*y**5+1419583*y**6-835751*y**7+358164*y**8-99196*y**9+14159*y**10
        -567*y**11)-2*r**8*(-1+y)**3*(230950+54600*logy*(-1+y)**7-1671250*y
        +5204850*y**2-9056950*y**3+9534700*y**4-6103020*y**5+2192935*y**6-296701*y**7
        -66762*y**8+27662*y**9-2249*y**10+75*y**11)-2*r**14*(-7723+77230*y
        -347535*y**2+926760*y**3-1621830*y**4+1946196*y**5-1656858*y**6+1006560*y**7
        -433185*y**8+123130*y**9-20233*y**10+1440*y**11))+18*MBhat*(28560*(
        -1+y)**10+2574*r**16*(-1+y)**10-r**2*(-1+y)**8*(-291664+199920*logy*(
        -1+y)**2+783248*y-591544*y**2+66640*y**3+16660*y**4+17388*y**5+504*y**6
        +1031*y**7+374*y**8-117*y**9)+14*r**4*(-1+y)**6*(-53278+2640*logy*(
        -1+y)**4+215752*y-328908*y**2+224552*y**3-58778*y**4+1518*y**5-611*y**6
        +132*y**7-238*y**8+22*y**9+5*y**10)+14*r**8*(-1+y)**3*(725-13500*logy*(
        -1+y)**7+8425*y-72525*y**2+215375*y**3-333500*y**4+302408*y**5-161474*y**6
        +46868*y**7-3548*y**8-1272*y**9+198*y**10)-14*r**6*(-1+y)**4*(-39359
        -15780*logy*(-1+y)**6+220374*y-503595*y**2+592560*y**3-365520*y**4
        +98611*y**5+1626*y**6-5730*y**7+570*y**8+248*y**9-30*y**10+5*y**11)
        -7*r**10*(-1+y)**2*(23080-184640*y+646240*y**2-1292480*y**3+1615600*y**4
        -1339424*y**5+749136*y**6-280303*y**7+60398*y**8-4726*y**9-546*y**10
        +105*y**11)+14*r**14*(-1347+13470*y-60615*y**2+161640*y**3-282870*y**4
        +348201*y**5-307328*y**6+194405*y**7-85095*y**8+24300*y**9-4047*y**10
        +294*y**11)+14*r**12*(4474-44740*y+201330*y**2-536880*y**3+939540*y**4
        -1152350*y**5+1026603*y**6-670222*y**7+314510*y**8-100120*y**9+19859*y**10
        -2064*y**11+60*y**12))+3*MBhat**2*(-428400*(-1+y)**10+47366*r**16*(-1
        +y)**10+r**2*(-1+y)**8*(-1748436+1967280*logy*(-1+y)**2+5464152*y
        -4699356*y**2+655760*y**3+285425*y**4+80570*y**5-8141*y**6+10672*y**7
        -2486*y**8-360*y**9)+14*r**6*(-1+y)**4*(-259219+15420*logy*(-1+y)**6
        +1570734*y-3973095*y**2+5374560*y**3-4112955*y**4+1715682*y**5-331887*y**6
        +12360*y**7-2388*y**8+2908*y**9-300*y**10)+7*r**4*(-1+y)**6*(604848
        -113040*logy*(-1+y)**4-2532432*y+4024728*y**2-2909232*y**3+821643*y**4
        -18180*y**5+3580*y**6-340*y**7+735*y**8-400*y**9+10*y**10)+14*r**8*(
        -1+y)**3*(-133375+56100*logy*(-1+y)**7+877525*y-2436225*y**2+3667675*y**3
        -3248515*y**4+1608753*y**5-293939*y**6-126787*y**7+76314*y**8-13966*y**9
        +940*y**10)-7*r**10*(-1+y)**2*(61100+51600*logy*(-1+y)**8-437200*y
        +1323800*y**2-2166000*y**3+2429445*y**4-1775580*y**5+648658*y**6+170896*y**7
        -265211*y**8+113172*y**9-24840*y**10+2400*y**11)+14*r**14*(-16849+168490*y
        -758205*y**2+2021880*y**3-3625860*y**4+4553640*y**5-4065348*y**6+2558640*y**7
        -1106505*y**8+311230*y**9-50725*y**10+3564*y**11)-7*r**12*(-45848+458480*y
        -2063160*y**2+5501760*y**3-10115355*y**4+13418502*y**5-12845121*y**6
        +8697492*y**7-4007145*y**8+1179470*y**9-186191*y**10+5352*y**11+1764*y**12))
        +4*MBhat**3*(428400*(-1+y)**10+81810*r**16*(-1+y)**10+21*r**4*(-1+y)**6*(
        -30654+27720*logy*(-1+y)**4+150336*y-280944*y**2+245306*y**3-84069*y**4
        -985*y**5+3230*y**6-720*y**7+165*y**8+15*y**9)+r**2*(-1+y)**8*(85728
        -1147440*logy*(-1+y)**2-1318896*y+1806888*y**2-464590*y**3-161665*y**4
        +19481*y**5-7497*y**6+1296*y**7+1440*y**8+15*y**9)+42*r**6*(-1+y)**4*(17095
        -14700*logy*(-1+y)**6-117270*y+337275*y**2-521195*y**3+458125*y**4
        -217713*y**5+40184*y**6+9662*y**7-5739*y**8+776*y**9)+14*r**8*(-1+y)**3*(78175
        +11700*logy*(-1+y)**7-558925*y+1717725*y**2-2918465*y**3+3107705*y**4
        -2175045*y**5+1021304*y**6-311510*y**7+69123*y**8-13867*y**9+1780*y**10)
        -21*r**10*(-1+y)**2*(-23400+187200*y-655200*y**2+1248890*y**3-1600835*y**4
        +1517879*y**5-1133212*y**6+656237*y**7-291184*y**8+92160*y**9-17645*y**10
        +1470*y**11)+7*r**14*(-43138+431380*y-1941210*y**2+5264130*y**3-9461340*y**4
        +11777892*y**5-10312428*y**6+6319110*y**7-2636100*y**8+704215*y**9-105946*y**10
        +6459*y**11)-21*r**12*(-11182+111820*y-503190*y**2+1417670*y**3-2644815*y**4
        +3329851*y**5-2815395*y**6+1518159*y**7-422340*y**8-30025*y**9+69569*y**10
        -22795*y**11+2673*y**12)))/(12600*r**2*(-1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-42840*(-1+y)**11+2646*r**18*(-1+y)**11+6*r**2*(-1+y)**9*(-278651
        +112980*logy*(-1+y)**2+670282*y-448121*y**2+37660*y**3+9415*y**4
        +3766*y**5+1883*y**6+3880*y**7-246*y**8+780*y**9-18*y**10)-42*r**6*(
        -1+y)**5*(-214398+212940*logy*(-1+y)**6+1499328*y-4387140*y**2+6914220*y**3
        -6250365*y**4+3138966*y**5-736101*y**6+30500*y**7+1338*y**8+4992*y**9
        -1160*y**10-60*y**11)+28*r**4*(-1+y)**7*(166492+28140*logy*(-1+y)**4
        -637828*y+900462*y**2-544028*y**3+107867*y**4+5628*y**5+938*y**6-554*y**7
        +294*y**8+224*y**9+409*y**10-30*y**11)-84*r**12*(-1+y)**2*(21084-9900*logy*(
        -1+y)**9-179856*y+674874*y**2-1455906*y**3+1975959*y**4-1726479*y**5
        +943086*y**6-269000*y**7-3972*y**8+32718*y**9-10189*y**10+1041*y**11)
        -42*r**14*(-1+y)*(-1122+2700*logy*(-1+y)**10+13920*y-76140*y**2
        +243540*y**3-507195*y**4+722034*y**5-715095*y**6+513652*y**7-271030*y**8
        +106220*y**9-27182*y**10+3564*y**11-126*y**12)-84*r**8*(-1+y)**4*(-126494
        +85080*logy*(-1+y)**7+800378*y-2103354*y**2+2910030*y**3-2165580*y**4
        +703788*y**5+63184*y**6-91992*y**7+5641*y**8+6353*y**9-1459*y**10+135*y**11
        -30*y**12)+14*r**10*(-1+y)**3*(-221590+84300*logy*(-1+y)**8+1857020*y
        -6836770*y**2+14460340*y**3-19255625*y**4+16584700*y**5-9079150*y**6
        +2873720*y**7-344464*y**8-76448*y**9+27388*y**10-1796*y**11+75*y**12)
        +4*r**16*(11204-123244*y+616220*y**2-1848660*y**3+3697320*y**4-5176248*y**5
        +5176248*y**6-3764940*y**7+1985385*y**8-753095*y**9+191059*y**10-28169*y**11
        +1800*y**12)-14*MBhat**8*(-1+y)**4*(3060*(-1+y)**7-4410*r**10*(-1+y)**7
        +5*r**2*(-1+y)**5*(-1310+60*logy*(-1+y)**2+2680*y-1400*y**2-316*y**3
        +335*y**4-58*y**5+15*y**6)+6*r**4*(-1+y)**3*(600-2400*y+3600*y**2
        -2380*y**3+795*y**4-256*y**5+83*y**6)+3*r**6*(-1+y)*(-2400+900*logy*(
        -1+y)**6+15300*y-40950*y**2+62560*y**3-60005*y**4+36270*y**5-12559*y**6
        +1904*y**7)-10*r**8*(1150-8050*y+24150*y**2-42032*y**3+46265*y**4
        -32721*y**5+14287*y**6-3481*y**7+360*y**8))+56*MBhat**7*(-1+y)**3*(6120*(
        -1+y)**8+6237*r**12*(-1+y)**8+r**2*(-1+y)**6*(-15337-2040*logy*(-1
        +y)**2+28634*y-12277*y**2-4685*y**3+3760*y**4-770*y**5+120*y**6+15*y**7)
        +3*r**4*(-1+y)**4*(3975+600*logy*(-1+y)**4-15300*y+21750*y**2-12650*y**3
        +2055*y**4+456*y**5-196*y**6+78*y**7)+15*r**10*(-1263+10104*y-35364*y**2
        +72881*y**3-96544*y**4+84563*y**5-48468*y**6+17376*y**7-3495*y**8+294*y**9)
        -3*r**6*(-1+y)**2*(2850-17100*y+42750*y**2-59470*y**3+48395*y**4-19278*y**5
        -2249*y**6+5905*y**7-2358*y**8+315*y**9)-5*r**8*(-3710+29680*y-103880*y**2
        +215290*y**3-290603*y**4+260896*y**5-151238*y**6+50986*y**7-6665*y**8
        -1116*y**9+360*y**10))-28*MBhat**6*(-1+y)**2*(42840*(-1+y)**9-25836*r**14*(
        -1+y)**9-3*r**2*(-1+y)**7*(27244+17280*logy*(-1+y)**2-37208*y+1324*y**2
        +13020*y**3-3475*y**4+450*y**5+20*y**6-110*y**7-5*y**8)+r**4*(-1+y)**5*(72611
        +29820*logy*(-1+y)**4-260624*y+331296*y**2-148084*y**3+2371*y**4
        +1716*y**5+5606*y**6-1664*y**7+300*y**8)-3*r**6*(-1+y)**3*(9300+10800*logy*(
        -1+y)**6-45000*y+80100*y**2-57360*y**3-8845*y**4+49770*y**5-47772*y**6
        +24910*y**7-7983*y**8+1200*y**9)-9*r**12*(10273-92457*y+369828*y**2
        -881112*y**3+1370448*y**4-1444752*y**5+1038072*y**6-498348*y**7+151317*y**8
        -25723*y**9+1782*y**10)+3*r**8*(-1+y)**2*(-6900+8400*logy*(-1+y)**7
        +39900*y-90300*y**2+139460*y**3-105775*y**4-49687*y**5+196778*y**6
        -187270*y**7+91469*y**8-23355*y**9+2400*y**10)+5*r**10*(18628-167652*y
        +670608*y**2-1625124*y**3+2601012*y**4-2812284*y**5+2026497*y**6-913701*y**7
        +207099*y**8+8219*y**9-15948*y**10+2646*y**11))+8*MBhat**5*(-1+y)*(299880*(
        -1+y)**10+81810*r**16*(-1+y)**10-r**2*(-1+y)**8*(6588+759360*logy*(
        -1+y)**2+746184*y-1132452*y**2+335230*y**3+68215*y**4+2548*y**5+2079*y**6
        -1107*y**7-810*y**8-15*y**9)+21*r**4*(-1+y)**6*(-16269+17820*logy*(
        -1+y)**4+82896*y-159984*y**2+144866*y**3-51834*y**4-193*y**5+1856*y**6
        -192*y**7+15*y**8+15*y**9)+21*r**6*(-1+y)**4*(18215-17700*logy*(-1
        +y)**6-126990*y+370575*y**2-578590*y**3+506495*y**4-227118*y**5+24688*y**6
        +22855*y**7-9573*y**8+1123*y**9)+7*r**8*(-1+y)**3*(93050+23400*logy*(
        -1+y)**7-674750*y+2106150*y**2-3621430*y**3+3907615*y**4-2792667*y**5
        +1377712*y**6-479269*y**7+138660*y**8-34376*y**9+4505*y**10)-21*r**10*(
        -1+y)**2*(-8520+68160*y-238560*y**2+415610*y**3-504470*y**4+529484*y**5
        -523312*y**6+413957*y**7-235069*y**8+86295*y**9-17645*y**10+1470*y**11)
        +7*r**14*(-47593+475930*y-2141685*y**2+5798730*y**3-10396890*y**4+12900552*y**5
        -11247978*y**6+6853710*y**7-2836575*y**8+748765*y**9-110401*y**10+6459*y**11)
        -21*r**12*(-18553+185530*y-834885*y**2+2302190*y**3-4219995*y**4+5302417*y**5
        -4582203*y**6+2643069*y**7-917115*y**8+112265*y**9+45638*y**10-21031*y**11
        +2673*y**12))+MBhat**2*(-1199520*(-1+y)**11+45213*r**18*(-1+y)**11+r**2*(
        -1+y)**9*(-19906093+11000640*logy*(-1+y)**2+50812826*y-36407053*y**2
        +3666880*y**3+916720*y**4+618688*y**5+315770*y**6+50516*y**7+38573*y**8
        +66*y**9-1053*y**10)+42*r**4*(-1+y)**7*(1232502+39240*logy*(-1+y)**4
        -4890768*y+7257672*y**2-4759968*y**3+1150752*y**4+696*y**5-2857*y**6
        +5172*y**7+3884*y**8+764*y**9-201*y**10)+28*r**8*(-1+y)**4*(809521
        +119580*logy*(-1+y)**7-5786227*y+17777211*y**2-30465745*y**3+31512070*y**4
        -19824402*y**5+7114759*y**6-1175977*y**7-15309*y**8+6143*y**9+8236*y**10
        -600*y**11)+42*r**10*(-1+y)**3*(252360+76500*logy*(-1+y)**8-1942380*y
        +6492330*y**2-12270660*y**3+14267325*y**4-10571580*y**5+4766995*y**6
        -1062112*y**7-79079*y**8+85764*y**9-12583*y**10+540*y**11)-84*r**6*(
        -1+y)**5*(199486+395520*logy*(-1+y)**6-801396*y+816930*y**2+888360*y**3
        -2643870*y**4+2246700*y**5-791861*y**6+87650*y**7-594*y**8+350*y**9
        +55*y**10-120*y**11-10*y**12)-42*r**12*(-1+y)**2*(-52646+27600*logy*(
        -1+y)**9+446214*y-1660656*y**2+3543664*y**3-4735896*y**4+4439256*y**5
        -2888977*y**6+1237369*y**7-233940*y**8-24040*y**9+21419*y**10-4887*y**11
        +480*y**12)+36*r**16*(8641-95051*y+475255*y**2-1425765*y**3+2851530*y**4
        -4086810*y**5+4277994*y**6-3285894*y**7+1819785*y**8-704605*y**9+180125*y**10
        -27067*y**11+1782*y**12)-28*r**14*(22922-252142*y+1260710*y**2-3782130*y**3
        +7564260*y**4-11037516*y**5+12159798*y**6-10071042*y**7+6120885*y**8
        -2616185*y**9+751003*y**10-131021*y**11+10458*y**12))+4*MBhat*(85680*(
        -1+y)**11+3564*r**18*(-1+y)**11-r**2*(-1+y)**9*(-2276986+1051680*logy*(
        -1+y)**2+5605652*y-3854506*y**2+350560*y**3+87640*y**4+35056*y**5
        +42749*y**6+10907*y**7+4625*y**8+2073*y**9-210*y**10)-3*r**4*(-1+y)**7*(2067574
        +207480*logy*(-1+y)**4-8062816*y+11679264*y**2-7371216*y**3+1635324*y**4
        +41496*y**5-3969*y**6+919*y**7+5508*y**8+5318*y**9+363*y**10-117*y**11)
        +21*r**10*(-1+y)**3*(-27230-57000*logy*(-1+y)**8+160840*y-334940*y**2
        +137880*y**3+625650*y**4-1298520*y**5+1233875*y**6-654093*y**7+187888*y**8
        -14176*y**9-4235*y**10+501*y**11)+42*r**6*(-1+y)**5*(-67005+171900*logy*(
        -1+y)**6+573930*y-1950525*y**2+3460200*y**3-3454650*y**4+1897560*y**5
        -487874*y**6+20575*y**7+9129*y**8-925*y**9-220*y**10+60*y**11-15*y**12)
        -14*r**8*(-1+y)**4*(555899-151980*logy*(-1+y)**7-3739313*y+10686009*y**2
        -16746155*y**3+15416330*y**4-8185938*y**5+2178941*y**6-126410*y**7-49338*y**8
        +1762*y**9+1613*y**10+90*y**11+30*y**12)-21*r**12*(-1+y)**2*(-33366
        +300294*y-1201176*y**2+2802744*y**3-4204116*y**4+4204116*y**5-2893313*y**6
        +1376392*y**7-452007*y**8+89588*y**9-7909*y**10-126*y**11+63*y**12)
        +6*r**16*(6015-66165*y+330825*y**2-992475*y**3+1984950*y**4-2778930*y**5
        +2842042*y**6-2139542*y**7+1181155*y**8-458865*y**9+117645*y**10-17751*y**11
        +1176*y**12)+14*r**14*(-12623+138853*y-694265*y**2+2082795*y**3-4165590*y**4
        +5831826*y**5-5932191*y**6+4487061*y**7-2537001*y**8+1053665*y**9-302368*y**10
        +55148*y**11-5490*y**12+180*y**13))+4*MBhat**3*(599760*(-1+y)**11+27270*r**18*(
        -1+y)**11-3*r**2*(-1+y)**9*(-1790030+1302000*logy*(-1+y)**2+4882060*y
        -3743030*y**2+434000*y**3+136465*y**4+77658*y**5+14959*y**6+4959*y**7
        +1098*y**8-469*y**9-30*y**10)+42*r**8*(-1+y)**4*(19560-48000*logy*(
        -1+y)**7-88920*y+98760*y**2+171400*y**3-576525*y**4+684199*y**5-408971*y**6
        +113141*y**7+8028*y**8-10120*y**9+1208*y**10)+r**4*(-1+y)**7*(-13151304
        +661920*logy*(-1+y)**4+53267136*y-81224544*y**2+55473536*y**3-14443259*y**4
        +252336*y**5-49280*y**6-68938*y**7-10863*y**8+4272*y**9+330*y**10-30*y**11)
        +42*r**6*(-1+y)**5*(208128+84960*logy*(-1+y)**6-1163808*y+2654640*y**2
        -3114720*y**3+1914130*y**4-527463*y**5+24192*y**6+3356*y**7+4125*y**8
        -600*y**9-315*y**10+15*y**11)+7*r**10*(-1+y)**3*(-180080+72000*logy*(
        -1+y)**8+1512640*y-5582240*y**2+11836480*y**3-15449225*y**4+13471216*y**5
        -8057170*y**6+3355421*y**7-861673*y**8+134648*y**9-18232*y**10+2015*y**11)
        -21*r**12*(-1+y)**2*(21960-197640*y+790560*y**2-1844640*y**3+2555005*y**4
        -2398569*y**5+1631922*y**6-898152*y**7+394809*y**8-143181*y**9+40988*y**10
        -7410*y**11+588*y**12)+4*r**16*(34744-382184*y+1910920*y**2-5732760*y**3
        +11702190*y**4-16955232*y**5+17800020*y**6-13567488*y**7+7423185*y**8
        -2829640*y**9+708026*y**10-103120*y**11+6459*y**12)-42*r**14*(4112-45232*y
        +226160*y**2-678480*y**3+1438325*y**4-2212534*y**5+2463590*y**6-1969992*y**7
        +1109091*y**8-423060*y**9+97625*y**10-8563*y**11-1339*y**12+297*y**13))
        +MBhat**4*(-2998800*(-1+y)**11+218925*r**18*(-1+y)**11-42*r**4*(-1+y)**7*(
        -598782+115560*logy*(-1+y)**4+2510688*y-3997152*y**2+2900548*y**3
        -825287*y**4+21900*y**5-7870*y**6+1520*y**7+195*y**8+120*y**9)+42*r**6*(
        -1+y)**5*(-464448+26640*logy*(-1+y)**6+2813328*y-7113240*y**2+9603040*y**3
        -7274105*y**4+2910678*y**5-458653*y**6-34000*y**7+3675*y**8+6150*y**9
        -825*y**10)+r**2*(-1+y)**9*(-10478925+12894000*logy*(-1+y)**2+33851850*y
        -29819925*y**2+4549160*y**3+1654100*y**4+466760*y**5+24654*y**6+30612*y**7
        -11301*y**8-2370*y**9-15*y**10)+28*r**8*(-1+y)**4*(-409975+85500*logy*(
        -1+y)**7+2784325*y-8053725*y**2+12779375*y**3-12167825*y**4+6773199*y**5
        -1781759*y**6-166387*y**7+201240*y**8-35528*y**9+1460*y**10)+42*r**10*(
        -1+y)**3*(-108400-46500*logy*(-1+y)**8+820700*y-2686450*y**2+4847600*y**3
        -5844875*y**4+4853592*y**5-2808993*y**6+1094532*y**7-351393*y**8+109092*y**9
        -24545*y**10+2540*y**11)-42*r**12*(-1+y)**2*(18450-166050*y+664200*y**2
        -1366780*y**3+2307935*y**4-3194715*y**5+3480420*y**6-2783724*y**7+1644183*y**8
        -696647*y**9+197128*y**10-32856*y**11+2376*y**12)+12*r**16*(73207-805277*y
        +4026385*y**2-12236935*y**3+24949520*y**4-35826448*y**5+37026528*y**6
        -27647400*y**7+14723610*y**8-5407820*y**9+1281544*y**10-171044*y**11
        +9090*y**12)-14*r**14*(56704-623744*y+3118720*y**2-9817080*y**3+20491845*y**4
        -29292615*y**5+29009763*y**6-19445097*y**7+7955904*y**8-1056220*y**9
        -796510*y**10+515186*y**11-129774*y**12+12918*y**13)))/(6300*r**2*(-1
        +y)**11)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr5(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r5 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (2880*(-1+y)**7-1905*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(-485+1680*logy*(
        -1+y)**2+2650*y-3005*y**2+1328*y**3-745*y**4+194*y**5+15*y**6)-24*r**4*(
        -1+y)**3*(125-500*y+750*y**2-540*y**3+485*y**4-288*y**5+64*y**6)
        +6*r**6*(-1+y)*(-1500+450*logy*(-1+y)**6+9450*y-24975*y**2+37790*y**3
        -33770*y**4+17340*y**5-5111*y**6+656*y**7)-20*r**8*(430-3010*y+9030*y**2
        -15578*y**3+16415*y**4-10425*y**5+3913*y**6-751*y**7+48*y**8))/(1800*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (5760*(-1+y)**8-1143*r**12*(-1+y)**8+r**2*(-1+y)**6*(45943-35040*logy*(
        -1+y)**2-126926*y+98503*y**2-11680*y**3-6265*y**4+530*y**5+375*y**6)
        -3*r**4*(-1+y)**4*(31475-125900*y+188850*y**2-125900*y**3+32140*y**4
        +2332*y**5-1258*y**6-228*y**7+25*y**8)+6*r**6*(-1+y)**2*(7800+2550*logy*(
        -1+y)**6-44250*y+102975*y**2-124550*y**3+86885*y**4-35902*y**5+9089*y**6
        -2834*y**7+547*y**8)+2*r**8*(-1+y)*(6200+4050*logy*(-1+y)**7-47450*y
        +156525*y**2-289225*y**3+352315*y**4-282315*y**5+136931*y**6-41687*y**7
        +5901*y**8-75*y**9)-15*r**10*(-631+5048*y-17668*y**2+35336*y**3-45724*y**4
        +38936*y**5-20512*y**6+6600*y**7-1113*y**8+64*y**9)-2*MBhat**2*(-1
        +y)*(-2880*(-1+y)**7+1905*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(-485+1680*logy*(
        -1+y)**2+2650*y-3005*y**2+1328*y**3-745*y**4+194*y**5+15*y**6)+24*r**4*(
        -1+y)**3*(125-500*y+750*y**2-540*y**3+485*y**4-288*y**5+64*y**6)
        -6*r**6*(-1+y)*(-1500+450*logy*(-1+y)**6+9450*y-24975*y**2+37790*y**3
        -33770*y**4+17340*y**5-5111*y**6+656*y**7)+20*r**8*(430-3010*y+9030*y**2
        -15578*y**3+16415*y**4-10425*y**5+3913*y**6-751*y**7+48*y**8))-2*MBhat*(5760*(
        -1+y)**8+3696*r**12*(-1+y)**8-3*r**2*(-1+y)**6*(-7568+8640*logy*(
        -1+y)**2+23776*y-20528*y**2+4110*y**3-60*y**4+115*y**5-10*y**6+5*y**7)
        +8*r**4*(-1+y)**4*(-2950-600*logy*(-1+y)**4+11200*y-15600*y**2+9275*y**3
        -3410*y**4+1578*y**5-808*y**6+139*y**7)-6*r**6*(-1+y)**2*(4000-24000*y
        +60000*y**2-85530*y**3+71380*y**4-32637*y**5+6394*y**6+1190*y**7-582*y**8
        +25*y**9)+5*r**10*(-3632+29056*y-101696*y**2+208054*y**3-269456*y**4
        +224761*y**5-120344*y**6+38956*y**7-6584*y**8+381*y**9)-120*r**8*(-280
        +2240*y-7840*y**2+16125*y**3-21286*y**4+18419*y**5-10400*y**6+3673*y**7
        -694*y**8+39*y**9+4*y**10)))/(3600*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (2880*(-1+y)**9-254*r**14*(-1+y)**9-16*r**4*(-1+y)**5*(9931+420*logy*(
        -1+y)**4-39304*y+58116*y**2-37904*y**3+9056*y**4+288*y**5-80*y**6
        +116*y**7-75*y**8)+r**2*(-1+y)**7*(57474-29520*logy*(-1+y)**2-144468*y
        +101754*y**2-9840*y**3-2460*y**4-1608*y**5-979*y**6+302*y**7+65*y**8)
        +2*r**6*(-1+y)**3*(7375+66300*logy*(-1+y)**6+22050*y-254025*y**2
        +670200*y**3-834150*y**4+545952*y**5-178197*y**6+22602*y**7-2919*y**8
        +752*y**9)+4*r**8*(-1+y)**2*(-21200+4800*logy*(-1+y)**7+143600*y
        -414000*y**2+656400*y**3-614400*y**4+348672*y**5-120997*y**6+26515*y**7
        -6687*y**8+1017*y**9)+r**10*(-1+y)*(-4130+5400*logy*(-1+y)**8+38440*y
        -156140*y**2+362680*y**3-528950*y**4+543640*y**5-392771*y**6+173008*y**7
        -49958*y**8+6696*y**9-75*y**10)-4*r**12*(844-7596*y+30384*y**2-70896*y**3
        +106344*y**4-109656*y**5+78213*y**6-35037*y**7+9783*y**8-1447*y**9
        +72*y**10)-MBhat**4*(-1+y)**2*(-2880*(-1+y)**7+1905*r**10*(-1+y)**7
        +5*r**2*(-1+y)**5*(-485+1680*logy*(-1+y)**2+2650*y-3005*y**2+1328*y**3
        -745*y**4+194*y**5+15*y**6)+24*r**4*(-1+y)**3*(125-500*y+750*y**2
        -540*y**3+485*y**4-288*y**5+64*y**6)-6*r**6*(-1+y)*(-1500+450*logy*(
        -1+y)**6+9450*y-24975*y**2+37790*y**3-33770*y**4+17340*y**5-5111*y**6
        +656*y**7)+20*r**8*(430-3010*y+9030*y**2-15578*y**3+16415*y**4-10425*y**5
        +3913*y**6-751*y**7+48*y**8))+2*MBhat**3*(-1+y)*(-5760*(-1+y)**8-3696*r**12*(
        -1+y)**8+8*r**4*(-1+y)**4*(2950+600*logy*(-1+y)**4-11200*y+15600*y**2
        -9275*y**3+3410*y**4-1578*y**5+808*y**6-139*y**7)+3*r**2*(-1+y)**6*(
        -7568+8640*logy*(-1+y)**2+23776*y-20528*y**2+4110*y**3-60*y**4+115*y**5
        -10*y**6+5*y**7)+6*r**6*(-1+y)**2*(4000-24000*y+60000*y**2-85530*y**3
        +71380*y**4-32637*y**5+6394*y**6+1190*y**7-582*y**8+25*y**9)-5*r**10*(
        -3632+29056*y-101696*y**2+208054*y**3-269456*y**4+224761*y**5-120344*y**6
        +38956*y**7-6584*y**8+381*y**9)+120*r**8*(-280+2240*y-7840*y**2+16125*y**3
        -21286*y**4+18419*y**5-10400*y**6+3673*y**7-694*y**8+39*y**9+4*y**10))
        -4*MBhat*(2880*(-1+y)**9+440*r**14*(-1+y)**9-16*r**4*(-1+y)**5*(5315
        +300*logy*(-1+y)**4-20960*y+30840*y**2-19960*y**3+4840*y**4+3*y**5
        +119*y**6-53*y**7)+2*r**2*(-1+y)**7*(19572-11760*logy*(-1+y)**2-50904*y
        +37212*y**2-3920*y**3-1220*y**4-755*y**5+90*y**6+45*y**7)+r**6*(-1
        +y)**3*(48200+26400*logy*(-1+y)**6-262800*y+577800*y**2-638400*y**3
        +367920*y**4-97467*y**5+6930*y**6-2304*y**7-930*y**8+331*y**9)-r**8*(
        -1+y)**2*(-12800+89600*y-268800*y**2+448000*y**3-492160*y**4+365709*y**5
        -174283*y**6+55498*y**7-7158*y**8-31*y**9+25*y**10)+r**12*(3344-30096*y
        +120384*y**2-280896*y**3+432384*y**4-453231*y**5+317031*y**6-147204*y**7
        +42516*y**8-6629*y**9+381*y**10)-5*r**10*(2104-18936*y+75744*y**2
        -176736*y**3+272496*y**4-291375*y**5+215319*y**6-108486*y**7+36414*y**8
        -7171*y**9+627*y**10))+MBhat**2*(17280*(-1+y)**9-6216*r**14*(-1+y)**9
        -5*r**2*(-1+y)**7*(-27767+21600*logy*(-1+y)**2+77134*y-60167*y**2
        +7488*y**3+3207*y**4-78*y**5-104*y**6-2*y**7+y**8)+r**4*(-1+y)**5*(
        -242009-7680*logy*(-1+y)**4+960356*y-1425174*y**2+928996*y**3-232654*y**4
        -3948*y**5-638*y**6+1172*y**7+75*y**8)+6*r**6*(-1+y)**3*(11500+3150*logy*(
        -1+y)**6-65850*y+155175*y**2-179470*y**3+113125*y**4-45038*y**5+20257*y**6
        -14266*y**7+4247*y**8-400*y**9)+2*r**8*(-1+y)**2*(-26800+5250*logy*(
        -1+y)**7+182350*y-528675*y**2+925015*y**3-981265*y**4+608697*y**5
        -197869*y**6-9623*y**7+25185*y**8-5975*y**9+320*y**10)-3*r**12*(11155
        -100395*y+401580*y**2-951740*y**3+1463040*y**4-1506672*y**5+1048740*y**6
        -481140*y**7+136125*y**8-20405*y**9+1056*y**10)+5*r**10*(12791-115119*y
        +460476*y**2-1102956*y**3+1729422*y**4-1828110*y**5+1314879*y**6-629319*y**7
        +183501*y**8-25589*y**9-357*y**10+381*y**11)))/(1800*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (40320*(-1+y)**10-1905*r**16*(-1+y)**10-r**2*(-1+y)**8*(-1500185+621600*logy*(
        -1+y)**2+3621970*y-2432585*y**2+207200*y**3+51800*y**4+20720*y**5
        +8785*y**6+22702*y**7-2039*y**8-1728*y**9)-21*r**4*(-1+y)**6*(254103
        +15360*logy*(-1+y)**4-1001052*y+1470858*y**2-949852*y**3+222103*y**4
        +3072*y**5+4543*y**6-4604*y**7+3450*y**8-1020*y**9-65*y**10)+105*r**10*(
        -1+y)**2*(24631+4500*logy*(-1+y)**8-192548*y+655918*y**2-1269836*y**3
        +1524295*y**4-1156436*y**5+551213*y**6-163764*y**7+30142*y**8-6472*y**9
        +841*y**10)+21*r**6*(-1+y)**4*(-461895+486900*logy*(-1+y)**6+3258270*y
        -9606375*y**2+15243000*y**3-13866750*y**4+7007400*y**5-1629430*y**6
        +25168*y**7+33045*y**8-4238*y**9+1325*y**10)+21*r**12*(-1+y)*(1185
        +4500*logy*(-1+y)**9-15165*y+80910*y**2-242790*y**3+458685*y**4-572085*y**5
        +519605*y**6-347333*y**7+139482*y**8-37958*y**9+4757*y**10-45*y**11)
        +7*r**8*(-1+y)**3*(-1556975+1214700*logy*(-1+y)**7+9684125*y-24800925*y**2
        +32831975*y**3-22203350*y**4+4819110*y**5+2833090*y**6-1928122*y**7
        +342861*y**8-38491*y**9+5107*y**10+75*y**11)-7*r**14*(-5309+53090*y
        -238905*y**2+637080*y**3-1114890*y**4+1337868*y**5-1144878*y**6+702720*y**7
        -273585*y**8+67370*y**9-8849*y**10+384*y**11)-14*MBhat**6*(-1+y)**3*(
        -2880*(-1+y)**7+1905*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(-485+1680*logy*(
        -1+y)**2+2650*y-3005*y**2+1328*y**3-745*y**4+194*y**5+15*y**6)+24*r**4*(
        -1+y)**3*(125-500*y+750*y**2-540*y**3+485*y**4-288*y**5+64*y**6)
        -6*r**6*(-1+y)*(-1500+450*logy*(-1+y)**6+9450*y-24975*y**2+37790*y**3
        -33770*y**4+17340*y**5-5111*y**6+656*y**7)+20*r**8*(430-3010*y+9030*y**2
        -15578*y**3+16415*y**4-10425*y**5+3913*y**6-751*y**7+48*y**8))+42*MBhat**5*(
        -1+y)**2*(-5760*(-1+y)**8-3696*r**12*(-1+y)**8+8*r**4*(-1+y)**4*(2950
        +600*logy*(-1+y)**4-11200*y+15600*y**2-9275*y**3+3410*y**4-1578*y**5
        +808*y**6-139*y**7)+3*r**2*(-1+y)**6*(-7568+8640*logy*(-1+y)**2+23776*y
        -20528*y**2+4110*y**3-60*y**4+115*y**5-10*y**6+5*y**7)+6*r**6*(-1
        +y)**2*(4000-24000*y+60000*y**2-85530*y**3+71380*y**4-32637*y**5+6394*y**6
        +1190*y**7-582*y**8+25*y**9)-5*r**10*(-3632+29056*y-101696*y**2+208054*y**3
        -269456*y**4+224761*y**5-120344*y**6+38956*y**7-6584*y**8+381*y**9)
        +120*r**8*(-280+2240*y-7840*y**2+16125*y**3-21286*y**4+18419*y**5
        -10400*y**6+3673*y**7-694*y**8+39*y**9+4*y**10))-21*MBhat**4*(-1+y)*(
        -28800*(-1+y)**9+12432*r**14*(-1+y)**9+r**4*(-1+y)**5*(389593+15360*logy*(
        -1+y)**4-1543012*y+2283798*y**2-1480292*y**3+368888*y**4+900*y**5
        +5050*y**6-1660*y**7-225*y**8)+r**2*(-1+y)**7*(-231727+180960*logy*(
        -1+y)**2+644414*y-503167*y**2+63200*y**3+25805*y**4-250*y**5-665*y**6
        -20*y**7+10*y**8)+6*r**6*(-1+y)**3*(-15200-3750*logy*(-1+y)**6+87450*y
        -207375*y**2+234390*y**3-139365*y**4+54174*y**5-31425*y**6+25698*y**7
        -7947*y**8+800*y**9)-2*r**8*(-1+y)**2*(-59800+6450*logy*(-1+y)**7
        +412150*y-1213875*y**2+2139255*y**3-2314845*y**4+1499709*y**5-532669*y**6
        +22441*y**7+44469*y**8-11875*y**9+640*y**10)+3*r**12*(22691-204219*y
        +816876*y**2-1935484*y**3+2974086*y**4-3061350*y**5+2129484*y**6-975996*y**7
        +275679*y**8-41191*y**9+2112*y**10)-5*r**10*(27475-247275*y+989100*y**2
        -2364924*y**3+3702024*y**4-3910200*y**5+2808102*y**6-1339974*y**7+390141*y**8
        -54709*y**9-522*y**10+762*y**11))-6*MBhat*(40320*(-1+y)**10+2288*r**16*(
        -1+y)**10-r**2*(-1+y)**8*(-1137648+517440*logy*(-1+y)**2+2792736*y
        -1913808*y**2+172480*y**3+43120*y**4+15085*y**5+24934*y**6+2122*y**7
        -2186*y**8-195*y**9)+56*r**4*(-1+y)**6*(-63054-3480*logy*(-1+y)**4
        +248736*y-366144*y**2+237136*y**3-55804*y**4-1827*y**5+1196*y**6-1130*y**7
        +270*y**8+45*y**9)+56*r**8*(-1+y)**3*(-75950+27000*logy*(-1+y)**7
        +504650*y-1419450*y**2+2176750*y**3-1940500*y**4+992559*y**5-262437*y**6
        +25974*y**7-2934*y**8-11*y**9+89*y**10)+21*r**6*(-1+y)**4*(-84528+210240*logy*(
        -1+y)**6+717408*y-2424240*y**2+4283520*y**3-4263840*y**4+2354787*y**5
        -630898*y**6+46755*y**7-320*y**8+901*y**9-30*y**10+5*y**11)-7*r**10*(
        -1+y)**2*(31760-254080*y+889280*y**2-1778560*y**3+2223200*y**4-1945063*y**5
        +1241512*y**6-531821*y**7+169936*y**8-24557*y**9+1048*y**10+25*y**11)
        +7*r**14*(-3568+35680*y-160560*y**2+428160*y**3-749280*y**4+921627*y**5
        -810222*y**6+485655*y**7-197640*y**8+50905*y**9-7186*y**10+381*y**11)
        +168*r**12*(714-7140*y+32130*y**2-85680*y**3+149940*y**4-184002*y**5
        +163836*y**6-104304*y**7+46280*y**8-14165*y**9+2616*y**10-227*y**11
        +2*y**12))-21*MBhat**2*(-28800*(-1+y)**10+2442*r**16*(-1+y)**10+3*r**2*(
        -1+y)**8*(-194974+100320*logy*(-1+y)**2+490268*y-345454*y**2+33440*y**3
        +7705*y**4+7054*y**5+1782*y**6-506*y**7-115*y**8)+r**4*(-1+y)**6*(1580492
        +61440*logy*(-1+y)**4-6260528*y+9267912*y**2-6055728*y**3+1482507*y**4
        -20220*y**5+36725*y**6-5480*y**7-2655*y**8+20*y**9-5*y**10)-12*r**6*(
        -1+y)**4*(35541+94320*logy*(-1+y)**6-118926*y+14355*y**2+452460*y**3
        -796330*y**4+586352*y**5-188287*y**6+14090*y**7+187*y**8-92*y**9+50*y**10)
        +6*r**8*(-1+y)**3*(84075-15300*logy*(-1+y)**7-573225*y+1666125*y**2
        -2669775*y**3+2462455*y**4-1282981*y**5+339043*y**6-27521*y**7+12542*y**8
        -3858*y**9+320*y**10)-r**10*(-1+y)**2*(71150+17400*logy*(-1+y)**8
        -551800*y+1861700*y**2-3561000*y**3+4748055*y**4-4377120*y**5+2606387*y**6
        -1072696*y**7+202871*y**8-4552*y**9-3955*y**10+320*y**11)+6*r**14*(
        -3082+30820*y-138690*y**2+369840*y**3-659715*y**4+819630*y**5-709359*y**6
        +428520*y**7-175215*y**8+45390*y**9-6475*y**10+352*y**11)-3*r**12*(
        -17124+171240*y-770580*y**2+2054880*y**3-3703385*y**4+4731930*y**5
        -4288984*y**6+2755606*y**7-1240210*y**8+363870*y**9-61412*y**10+4042*y**11
        +127*y**12))-2*MBhat**3*(403200*(-1+y)**10+66240*r**16*(-1+y)**10+42*r**6*(
        -1+y)**4*(158880+86400*logy*(-1+y)**6-866880*y+1908000*y**2-2087735*y**3
        +1152650*y**4-262419*y**5+10712*y**6-19529*y**7+4158*y**8-237*y**9)
        -42*r**4*(-1+y)**6*(289456+6720*logy*(-1+y)**4-1151104*y+1713216*y**2
        -1123774*y**3+271976*y**4+8445*y**5+660*y**6-1240*y**7+60*y**8-15*y**9)
        -r**2*(-1+y)**8*(-5445504+3333120*logy*(-1+y)**2+14224128*y-10445184*y**2
        +1077650*y**3+421820*y**4+155540*y**5-13860*y**6-8175*y**7-30*y**8
        +15*y**9)+56*r**8*(-1+y)**3*(8600-60200*y+180600*y**2-253255*y**3
        +260185*y**4-246285*y**5+193903*y**6-130405*y**7+46731*y**8-9779*y**9
        +905*y**10)-21*r**10*(-1+y)**2*(16000-128000*y+448000*y**2-1061530*y**3
        +1471920*y**4-1166978*y**5+499424*y**6+49841*y**7-147352*y**8+65200*y**9
        -11880*y**10+635*y**11)+14*r**14*(-27232+272320*y-1225440*y**2+3305325*y**3
        -5888610*y**4+7219023*y**5-6195252*y**6+3693990*y**7-1481400*y**8+371935*y**9
        -50014*y**10+2331*y**11)-42*r**12*(-18288+182880*y-822960*y**2+2246290*y**3
        -4066340*y**4+5059825*y**5-4409910*y**6+2669811*y**7-1067280*y**8+249035*y**9
        -19750*y**10-4105*y**11+792*y**12)))/(25200*r**2*(-1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (40320*(-1+y)**11-1143*r**18*(-1+y)**11-r**2*(-1+y)**9*(-2423383+870240*logy*(
        -1+y)**2+5717006*y-3728743*y**2+290080*y**3+72520*y**4+29008*y**5
        +14504*y**6-1696*y**7+28139*y**8+1674*y**9-2469*y**10)-48*r**4*(-1
        +y)**7*(225941+15120*logy*(-1+y)**4-888644*y+1302726*y**2-838244*y**3
        +194441*y**4+3024*y**5+504*y**6+2896*y**7-3940*y**8+2480*y**9-440*y**10
        -72*y**11)+336*r**12*(-1+y)**2*(-13547+2100*logy*(-1+y)**9+119823*y
        -469842*y**2+1071098*y**3-1562547*y**4+1509627*y**5-962318*y**6+395446*y**7
        -103013*y**8+16337*y**9-3097*y**10+353*y**11)+56*r**8*(-1+y)**4*(-252278
        +1251960*logy*(-1+y)**7+513986*y+2839902*y**2-13496890*y**3+24451540*y**4
        -23434644*y**5+12193408*y**6-2956504*y**7+92457*y**8+48881*y**9-2033*y**10
        +375*y**11)+42*r**6*(-1+y)**5*(-1405654+880620*logy*(-1+y)**6+9314544*y
        -25928220*y**2+38974060*y**3-33633645*y**4+16095318*y**5-3563173*y**6
        +143540*y**7-16681*y**8+22306*y**9-3755*y**10+1120*y**11)+14*r**14*(
        -1+y)*(962+8100*logy*(-1+y)**10-1520*y-33660*y**2+211260*y**3-612705*y**4
        +1075446*y**5-1236405*y**6+1039044*y**7-661905*y**8+245050*y**9-63163*y**10
        +7416*y**11-60*y**12)+42*r**10*(-1+y)**3*(1159440+593700*logy*(-1+y)**8
        -8681820*y+28011570*y**2-50481940*y**3+54790625*y**4-35520700*y**5
        +12219150*y**6-1039860*y**7-590046*y**8+137768*y**9-13788*y**10+1176*y**11
        +25*y**12)-24*r**16*(1282-14102*y+70510*y**2-211530*y**3+423060*y**4
        -592284*y**5+592284*y**6-432820*y**7+233255*y**8-80285*y**9+17677*y**10
        -2087*y**11+80*y**12)-14*MBhat**8*(-1+y)**4*(-2880*(-1+y)**7+1905*r**10*(
        -1+y)**7+5*r**2*(-1+y)**5*(-485+1680*logy*(-1+y)**2+2650*y-3005*y**2
        +1328*y**3-745*y**4+194*y**5+15*y**6)+24*r**4*(-1+y)**3*(125-500*y
        +750*y**2-540*y**3+485*y**4-288*y**5+64*y**6)-6*r**6*(-1+y)*(-1500
        +450*logy*(-1+y)**6+9450*y-24975*y**2+37790*y**3-33770*y**4+17340*y**5
        -5111*y**6+656*y**7)+20*r**8*(430-3010*y+9030*y**2-15578*y**3+16415*y**4
        -10425*y**5+3913*y**6-751*y**7+48*y**8))+56*MBhat**7*(-1+y)**3*(-5760*(
        -1+y)**8-3696*r**12*(-1+y)**8+8*r**4*(-1+y)**4*(2950+600*logy*(-1
        +y)**4-11200*y+15600*y**2-9275*y**3+3410*y**4-1578*y**5+808*y**6
        -139*y**7)+3*r**2*(-1+y)**6*(-7568+8640*logy*(-1+y)**2+23776*y-20528*y**2
        +4110*y**3-60*y**4+115*y**5-10*y**6+5*y**7)+6*r**6*(-1+y)**2*(4000
        -24000*y+60000*y**2-85530*y**3+71380*y**4-32637*y**5+6394*y**6+1190*y**7
        -582*y**8+25*y**9)-5*r**10*(-3632+29056*y-101696*y**2+208054*y**3
        -269456*y**4+224761*y**5-120344*y**6+38956*y**7-6584*y**8+381*y**9)
        +120*r**8*(-280+2240*y-7840*y**2+16125*y**3-21286*y**4+18419*y**5
        -10400*y**6+3673*y**7-694*y**8+39*y**9+4*y**10))-28*MBhat**6*(-1+y)**2*(
        -40320*(-1+y)**9+18648*r**14*(-1+y)**9+3*r**4*(-1+y)**5*(179059+7680*logy*(
        -1+y)**4-708556*y+1047474*y**2-677196*y**3+168374*y**4-716*y**5+3154*y**6
        -716*y**7-125*y**8)+r**2*(-1+y)**7*(-324619+253920*logy*(-1+y)**2
        +903158*y-705499*y**2+88960*y**3+35575*y**4-110*y**5-810*y**6-30*y**7
        +15*y**8)+6*r**6*(-1+y)**3*(-18900-4350*logy*(-1+y)**6+109050*y-259575*y**2
        +289310*y**3-165605*y**4+63310*y**5-42593*y**6+37130*y**7-11647*y**8
        +1200*y**9)-2*r**8*(-1+y)**2*(-92800+7650*logy*(-1+y)**7+641950*y
        -1899075*y**2+3353495*y**3-3648425*y**4+2390721*y**5-867469*y**6+54505*y**7
        +63753*y**8-17775*y**9+960*y**10)+9*r**12*(11409-102681*y+410724*y**2
        -973076*y**3+1495044*y**4-1538676*y**5+1070076*y**6-490284*y**7+138411*y**8
        -20659*y**9+1056*y**10)-15*r**10*(14053-126477*y+505908*y**2-1208964*y**3
        +1891542*y**4-1997430*y**5+1433775*y**6-683543*y**7+198927*y**8-27943*y**9
        -229*y**10+381*y**11))-8*MBhat**5*(-1+y)*(282240*(-1+y)**10+66240*r**16*(
        -1+y)**10+42*r**6*(-1+y)**4*(110680+60000*logy*(-1+y)**6-604080*y
        +1330200*y**2-1449335*y**3+784730*y**4-164952*y**5+3782*y**6-17225*y**7
        +5088*y**8-568*y**9)+42*r**4*(-1+y)**6*(-204416-1920*logy*(-1+y)**4
        +815744*y-1219776*y**2+804414*y**3-194536*y**4-8397*y**5+1244*y**6
        +392*y**7-60*y**8+15*y**9)-r**2*(-1+y)**8*(-3801456+2345280*logy*(
        -1+y)**2+9948192*y-7319376*y**2+748370*y**3+319340*y**4+92120*y**5
        -6300*y**6-4395*y**7-30*y**8+15*y**9)+14*r**8*(-1+y)**3*(-4000+28000*y
        -84000*y**2+330980*y**3-435740*y**4+111987*y**5+252763*y**6-355126*y**7
        +165450*y**8-39209*y**9+3695*y**10)-21*r**10*(-1+y)**2*(37040-296320*y
        +1037120*y**2-2239770*y**3+3018640*y**4-2534008*y**5+1285584*y**6-248859*y**7
        -81912*y**8+58930*y**9-11880*y**10+635*y**11)+14*r**14*(-28552+285520*y
        -1284840*y**2+3463725*y**3-6165810*y**4+7551663*y**5-6472452*y**6+3852390*y**7
        -1540800*y**8+385135*y**9-51334*y**10+2331*y**11)-84*r**12*(-10816+108160*y
        -486720*y**2+1323785*y**3-2389810*y**4+2972720*y**5-2590086*y**6+1567023*y**7
        -628500*y**8+149090*y**9-13380*y**10-1862*y**11+396*y**12))-2*MBhat**2*(
        -564480*(-1+y)**11+16317*r**18*(-1+y)**11+r**2*(-1+y)**9*(-21287887
        +8823360*logy*(-1+y)**2+51399134*y-34522927*y**2+2941120*y**3+735280*y**4
        +205408*y**5+319466*y**6+188972*y**7-12079*y**8-12942*y**9-585*y**10)
        +42*r**4*(-1+y)**7*(1867413+82560*logy*(-1+y)**4-7387092*y+10915518*y**2
        -7111892*y**3+1695413*y**4+37632*y**5-21211*y**6+13740*y**7+4158*y**8
        -2772*y**9-155*y**10)+42*r**10*(-1+y)**3*(-567235-90000*logy*(-1+y)**8
        +4447880*y-15207580*y**2+29575160*y**3-35708950*y**4+26839640*y**5
        -11975435*y**6+2810192*y**7-160521*y**8+27256*y**9-7887*y**10+560*y**11)
        -14*r**8*(-1+y)**4*(-11131853+7461060*logy*(-1+y)**7+70461911*y-185272023*y**2
        +256559285*y**3-191275010*y**4+63444786*y**5+3952918*y**6-7805854*y**7
        +991527*y**8-529*y**9-7223*y**10+1425*y**11)-42*r**6*(-1+y)**5*(-2951107
        +3418260*logy*(-1+y)**6+21124902*y-63067035*y**2+101180680*y**3-92976810*y**4
        +47554560*y**5-11478368*y**6+539380*y**7+52323*y**8+18490*y**9-325*y**10
        -60*y**11+10*y**12)-42*r**12*(-1+y)**2*(-31789+8700*logy*(-1+y)**9
        +277401*y-1070454*y**2+2393326*y**3-3407289*y**4+3594321*y**5-2851145*y**6
        +1506265*y**7-614262*y**8+128818*y**9-12613*y**10-3*y**11+64*y**12)
        +6*r**16*(27599-303589*y+1517945*y**2-4553835*y**3+9107670*y**4-12996690*y**5
        +13537986*y**6-10070886*y**7+5347815*y**8-1956845*y**9+461065*y**10
        -60923*y**11+3168*y**12)-14*r**14*(44911-494021*y+2470105*y**2-7410315*y**3
        +14820630*y**4-21307602*y**5+22823616*y**6-17953104*y**7+10244355*y**8
        -4226545*y**9+1167023*y**10-193645*y**11+14592*y**12))-8*MBhat*(40320*(
        -1+y)**11+1056*r**18*(-1+y)**11-3*r**2*(-1+y)**9*(-648928+248640*logy*(
        -1+y)**2+1546496*y-1021888*y**2+82880*y**3+20720*y**4+8288*y**5+1456*y**6
        +8157*y**7+2582*y**8-739*y**9-144*y**10)+42*r**6*(-1+y)**5*(-616560
        +484800*logy*(-1+y)**6+4184160*y-11914800*y**2+18310400*y**3-16156800*y**4
        +7917120*y**5-1792016*y**6+48695*y**7+19086*y**8-600*y**9+1090*y**10
        -15*y**11)+r**4*(-1+y)**7*(-7926848-456960*logy*(-1+y)**4+31250432*y
        -45961728*y**2+29727232*y**3-6974848*y**4-91392*y**5-109984*y**6+120931*y**7
        -75372*y**8+2738*y**9+6388*y**10+195*y**11)+7*r**10*(-1+y)**3*(1877440
        +552000*logy*(-1+y)**8-14467520*y+48428320*y**2-91704640*y**3+106902800*y**4
        -77794240*y**5+34062880*y**6-8008061*y**7+705736*y**8-43882*y**9-40*y**10
        +887*y**11)+42*r**8*(-1+y)**4*(-449632+608640*logy*(-1+y)**7+2538784*y
        -5486112*y**2+4883040*y**3+442560*y**4-4526016*y**5+3673312*y**6-1184615*y**7
        +101329*y**8+5154*y**9+66*y**10-35*y**11+5*y**12)-21*r**12*(-1+y)**2*(
        -13504+121536*y-486144*y**2+1134336*y**3-1701504*y**4+1701504*y**5
        -1225760*y**6+674747*y**7-253011*y**8+76714*y**9-10634*y**10+531*y**11
        +5*y**12)+4*r**16*(3920-43120*y+215600*y**2-646800*y**3+1293600*y**4
        -1811040*y**5+1852032*y**6-1401972*y**7+733980*y**8-265265*y**9+61435*y**10
        -7871*y**11+381*y**12)+84*r**14*(-1272+13992*y-69960*y**2+209880*y**3
        -419760*y**4+587664*y**5-598072*y**6+454601*y**7-253263*y**8+99550*y**9
        -27702*y**10+4725*y**11-387*y**12+4*y**13))-8*MBhat**3*(282240*(-1+y)**11
        +11040*r**18*(-1+y)**11-r**2*(-1+y)**9*(-8009040+3662400*logy*(-1+y)**2
        +19680480*y-13502640*y**2+1220800*y**3+271600*y**4+176701*y**5+136374*y**6
        +12579*y**7-9324*y**8-1050*y**9)+28*r**8*(-1+y)**4*(-1008820+363600*logy*(
        -1+y)**7+6698140*y-18821820*y**2+28824500*y**3-25523000*y**4+12725979*y**5
        -3061781*y**6+151106*y**7-1542*y**8-785*y**9+383*y**10)+3*r**4*(-1
        +y)**7*(-8742032-248640*logy*(-1+y)**4+34719488*y-51581952*y**2+33890688*y**3
        -8312512*y**4+23842*y**5-39816*y**6-55855*y**7+20980*y**8+1920*y**9
        -20*y**10+5*y**11)+21*r**6*(-1+y)**5*(-479536+1500480*logy*(-1+y)**6
        +4377696*y-15445680*y**2+28096640*y**3-28516160*y**4+15883741*y**5
        -4177454*y**6+227693*y**7+29600*y**8+475*y**9-450*y**10+75*y**11)+21*r**10*(
        -1+y)**3*(-14640+117120*y-409920*y**2+819840*y**3-775200*y**4+425073*y**5
        -258760*y**6+147368*y**7-140584*y**8+42509*y**9-8176*y**10+770*y**11)
        -21*r**12*(-1+y)**2*(-4240+38160*y-152640*y**2+356160*y**3-754240*y**4
        +1052654*y**5-867966*y**6+516201*y**7-143025*y**8+6480*y**9+8176*y**10
        -2087*y**11+127*y**12)+12*r**16*(7012-77132*y+385660*y**2-1156980*y**3
        +2348120*y**4-3375526*y**5+3476690*y**6-2585814*y**7+1369830*y**8-499345*y**9
        +116963*y**10-15295*y**11+777*y**12)-7*r**14*(32528-357808*y+1789040*y**2
        -5367120*y**3+11040960*y**4-16306287*y**5+17371941*y**6-13461393*y**7
        +7563219*y**8-2953445*y**9+748231*y**10-104123*y**11+3729*y**12+528*y**13))
        -MBhat**4*(-2822400*(-1+y)**11+187425*r**18*(-1+y)**11+28*r**4*(-1+y)**7*(5727804
        +79680*logy*(-1+y)**4-22831536*y+34087944*y**2-22519856*y**3+5551289*y**4
        +11340*y**5+76385*y**6-19520*y**7-3345*y**8+180*y**9-45*y**10)+3*r**2*(
        -1+y)**9*(-19068875+9912000*logy*(-1+y)**2+48049750*y-33936875*y**2
        +3223360*y**3+915390*y**4+635012*y**5+145446*y**6-33488*y**7-8115*y**8
        -10*y**9+5*y**10)-168*r**8*(-1+y)**4*(-281625+49500*logy*(-1+y)**7
        +1921875*y-5592375*y**2+9042125*y**3-8452725*y**4+4422703*y**5-1074783*y**6
        -38259*y**7+35100*y**8-8356*y**9+1120*y**10)-42*r**6*(-1+y)**5*(1219184
        +2624880*logy*(-1+y)**6-4690224*y+3850920*y**2+8096080*y**3-18983035*y**4
        +15157026*y**5-5050401*y**6+370900*y**7+19875*y**8-8550*y**9+1425*y**10)
        +28*r**10*(-1+y)**3*(-71800-38250*logy*(-1+y)**8+536150*y-1723525*y**2
        +2432450*y**3-3247600*y**4+4379234*y**5-4308461*y**6+3544294*y**7-1704506*y**8
        +549664*y**9-104660*y**10+8560*y**11)-84*r**12*(-1+y)**2*(-14300+128700*y
        -514800*y**2+1404080*y**3-2176865*y**4+1928361*y**5-942981*y**6-132867*y**7
        +456741*y**8-286829*y**9+89885*y**10-13749*y**11+704*y**12)+8*r**16*(140959
        -1550549*y+7752745*y**2-23463195*y**3+47572140*y**4-67689636*y**5+69146616*y**6
        -50847720*y**7+26516820*y**8-9446140*y**9+2132108*y**10-260068*y**11
        +11040*y**12)-42*r**14*(55264-607904*y+3039520*y**2-9315680*y**3+19132945*y**4
        -27424635*y**5+28146503*y**6-20708437*y**7+10609599*y**8-3529845*y**9
        +614405*y**10+9521*y**11-24364*y**12+3108*y**13)))/(25200*r**2*(-1+y)**11)
    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr6(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r6 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (6480*(-1+y)**7+10095*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(2875-1200*logy*(
        -1+y)**2-6950*y+4675*y**2+1088*y**3-1825*y**4+674*y**5+15*y**6)
        -24*r**4*(-1+y)**3*(-275+1100*y-1650*y**2+610*y**3+560*y**4-538*y**5
        +139*y**6)+18*r**6*(-1+y)*(300+150*logy*(-1+y)**6-1650*y+3675*y**2
        -5070*y**3+4410*y**4-2220*y**5+563*y**6-48*y**7)+20*r**8*(710-4970*y
        +14910*y**2-27022*y**3+31885*y**4-24435*y**5+11627*y**6-3149*y**7+372*y**8))/(1800*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-2*MBhat**2*(-1+y)*(-6480*(-1+y)**7-10095*r**10*(-1+y)**7+5*r**2*(
        -1+y)**5*(2875-1200*logy*(-1+y)**2-6950*y+4675*y**2+1088*y**3-1825*y**4
        +674*y**5+15*y**6)+24*r**4*(-1+y)**3*(-275+1100*y-1650*y**2+610*y**3
        +560*y**4-538*y**5+139*y**6)+18*r**6*(-1+y)*(-300-150*logy*(-1+y)**6
        +1650*y-3675*y**2+5070*y**3-4410*y**4+2220*y**5-563*y**6+48*y**7)
        -20*r**8*(710-4970*y+14910*y**2-27022*y**3+31885*y**4-24435*y**5+11627*y**6
        -3149*y**7+372*y**8))+3*(4320*(-1+y)**8+2019*r**12*(-1+y)**8+r**2*(
        -1+y)**6*(-35219+6720*logy*(-1+y)**2+77158*y-45299*y**2+2240*y**3
        -2555*y**4+2710*y**5-875*y**6)+r**4*(-1+y)**4*(38125+19200*logy*(-1
        +y)**4-133300*y+161550*y**2-69300*y**3+6260*y**4-6332*y**5+5258*y**6
        -1372*y**7-25*y**8)+2*r**6*(-1+y)**2*(-5400-3450*logy*(-1+y)**6+28950*y
        -62025*y**2+65450*y**3-39515*y**4+16498*y**5-4811*y**6+566*y**7+47*y**8)
        +2*r**8*(-1+y)*(-3400+1350*logy*(-1+y)**7+22450*y-62625*y**2+94925*y**3
        -97895*y**4+73495*y**5-38123*y**6+12471*y**7-2533*y**8+275*y**9)+5*r**10*(
        -1049+8392*y-29372*y**2+58744*y**3-78876*y**4+74184*y**5-47808*y**6
        +19640*y**7-4687*y**8+496*y**9))+2*MBhat*(-12960*(-1+y)**8+15204*r**12*(
        -1+y)**8+r**2*(-1+y)**6*(65996-23280*logy*(-1+y)**2-155272*y+100916*y**2
        +3730*y**3-13580*y**4+3745*y**5-30*y**6+15*y**7)-12*r**4*(-1+y)**4*(4675
        +900*logy*(-1+y)**4-17800*y+24900*y**2-12550*y**3-1165*y**4+3252*y**5
        -1322*y**6+226*y**7)-6*r**6*(-1+y)**2*(800-4800*y+12000*y**2-22870*y**3
        +26170*y**4-13663*y**5-144*y**6+3710*y**7-1718*y**8+275*y**9)+15*r**10*(
        -2356+18848*y-65968*y**2+137382*y**3-185048*y**4+165133*y**5-96712*y**6
        +35708*y**7-7492*y**8+673*y**9)-20*r**8*(-1400+11200*y-39200*y**2
        +82330*y**3-111659*y**4+97306*y**5-51560*y**6+13762*y**7+49*y**8-1014*y**9
        +186*y**10)))/(3600*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (6480*(-1+y)**9+1346*r**14*(-1+y)**9+8*r**4*(-1+y)**5*(-13712+29760*logy*(
        -1+y)**4+84608*y-186432*y**2+183808*y**3-75712*y**4+7374*y**5-365*y**6
        +818*y**7-225*y**8)+r**2*(-1+y)**7*(-123646+18480*logy*(-1+y)**2+265772*y
        -151366*y**2+6160*y**3+1540*y**4-3728*y**5+3481*y**6-1058*y**7+85*y**8)
        +2*r**6*(-1+y)**3*(148975+23100*logy*(-1+y)**6-870750*y+2107575*y**2
        -2694600*y**3+1905450*y**4-708648*y**5+117603*y**6-6198*y**7+681*y**8
        -448*y**9)+4*r**8*(-1+y)**2*(20500-8400*logy*(-1+y)**7-135100*y+375900*y**2
        -567700*y**3+494200*y**4-253128*y**5+75153*y**6-10635*y**7-537*y**8
        +267*y**9)+4*r**12*(1376-12384*y+49536*y**2-115584*y**3+173376*y**4
        -183924*y**5+142287*y**6-78363*y**7+28017*y**8-5903*y**9+558*y**10)
        +r**10*(-1+y)*(15070+5400*logy*(-1+y)**8-115160*y+381460*y**2-712520*y**3
        +815050*y**4-655760*y**5+401329*y**6-179792*y**7+50842*y**8-8904*y**9
        +825*y**10)+MBhat**4*(-1+y)**2*(6480*(-1+y)**7+10095*r**10*(-1+y)**7
        -5*r**2*(-1+y)**5*(2875-1200*logy*(-1+y)**2-6950*y+4675*y**2+1088*y**3
        -1825*y**4+674*y**5+15*y**6)-24*r**4*(-1+y)**3*(-275+1100*y-1650*y**2
        +610*y**3+560*y**4-538*y**5+139*y**6)-18*r**6*(-1+y)*(-300-150*logy*(
        -1+y)**6+1650*y-3675*y**2+5070*y**3-4410*y**4+2220*y**5-563*y**6
        +48*y**7)+20*r**8*(710-4970*y+14910*y**2-27022*y**3+31885*y**4-24435*y**5
        +11627*y**6-3149*y**7+372*y**8))+2*MBhat**3*(-1+y)*(-12960*(-1+y)**8
        +15204*r**12*(-1+y)**8+r**2*(-1+y)**6*(65996-23280*logy*(-1+y)**2
        -155272*y+100916*y**2+3730*y**3-13580*y**4+3745*y**5-30*y**6+15*y**7)
        -12*r**4*(-1+y)**4*(4675+900*logy*(-1+y)**4-17800*y+24900*y**2-12550*y**3
        -1165*y**4+3252*y**5-1322*y**6+226*y**7)-6*r**6*(-1+y)**2*(800-4800*y
        +12000*y**2-22870*y**3+26170*y**4-13663*y**5-144*y**6+3710*y**7-1718*y**8
        +275*y**9)+15*r**10*(-2356+18848*y-65968*y**2+137382*y**3-185048*y**4
        +165133*y**5-96712*y**6+35708*y**7-7492*y**8+673*y**9)-20*r**8*(-1400
        +11200*y-39200*y**2+82330*y**3-111659*y**4+97306*y**5-51560*y**6+13762*y**7
        +49*y**8-1014*y**9+186*y**10))+4*MBhat*(-6480*(-1+y)**9+1810*r**14*(
        -1+y)**9-8*r**4*(-1+y)**5*(7120+11400*logy*(-1+y)**4-17080*y+2820*y**2
        +20920*y**3-15430*y**4+1119*y**5+862*y**6-169*y**7)+2*r**2*(-1+y)**7*(45393
        -8940*logy*(-1+y)**2-99726*y+58803*y**2-2980*y**3+1220*y**4-1555*y**5
        +180*y**6+45*y**7)-3*r**6*(-1+y)**3*(12150-4200*logy*(-1+y)**6-77100*y
        +205350*y**2-294800*y**3+233640*y**4-94889*y**5+15210*y**6+732*y**7
        -610*y**8+77*y**9)-r**8*(-1+y)**2*(6800-47600*y+142800*y**2-238000*y**3
        +189160*y**4-54909*y**5-1817*y**6-4498*y**7+6558*y**8-2369*y**9+275*y**10)
        -5*r**10*(1366-12294*y+49176*y**2-114744*y**3+184974*y**4-215955*y**5
        +177261*y**6-97434*y**7+33966*y**8-6949*y**9+633*y**10)+3*r**12*(2112
        -19008*y+76032*y**2-177408*y**3+277832*y**4-303363*y**5+230883*y**6
        -119172*y**7+39888*y**8-7797*y**9+673*y**10))+MBhat**2*(38880*(-1+y)**9
        +21784*r**14*(-1+y)**9+5*r**2*(-1+y)**7*(-69833+18720*logy*(-1+y)**2
        +158386*y-97913*y**2+2352*y**3+3153*y**4+1038*y**5-496*y**6+2*y**7
        -y**8)+3*r**4*(-1+y)**5*(122197+49440*logy*(-1+y)**4-439348*y+560142*y**2
        -261268*y**3+9582*y**4+11484*y**5+454*y**6-676*y**7+25*y**8)+6*r**6*(
        -1+y)**3*(-6500-6450*logy*(-1+y)**6+32550*y-62025*y**2+35330*y**3
        +16025*y**4-20238*y**5+1657*y**6+3934*y**7-1653*y**8+200*y**9)+2*r**8*(
        -1+y)**2*(27600+5250*logy*(-1+y)**7-198450*y+613725*y**2-1142185*y**3
        +1217635*y**4-658203*y**5+54131*y**6+151177*y**7-96915*y**8+25325*y**9
        -2480*y**10)+3*r**12*(21845-196605*y+786420*y**2-1881860*y**3+2946960*y**4
        -3130128*y**5+2269260*y**6-1102860*y**7+340875*y**8-59595*y**9+4344*y**10)
        -15*r**10*(5443-48987*y+195948*y**2-472908*y**3+741766*y**4-767110*y**5
        +513147*y**6-203987*y**7+32073*y**8+8903*y**9-4961*y**10+673*y**11)))/(1800*r**2*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (90720*(-1+y)**10+10095*r**16*(-1+y)**10+r**2*(-1+y)**8*(-3255295+436800*logy*(
        -1+y)**2+6947390*y-3910495*y**2+145600*y**3+36400*y**4+14560*y**5
        -57925*y**6+53978*y**7-14821*y**8+1968*y**9)+63*r**6*(-1+y)**4*(215035
        +294300*logy*(-1+y)**6-995910*y+1606875*y**2-671000*y**3-968250*y**4
        +1270200*y**5-514010*y**6+52256*y**7+4215*y**8+854*y**9-425*y**10)
        -105*r**10*(-1+y)**2*(28849+9900*logy*(-1+y)**8-220892*y+733522*y**2
        -1374644*y**3+1579705*y**4-1125164*y**5+489087*y**6-123996*y**7+15338*y**8
        +392*y**9-181*y**10)+21*r**4*(-1+y)**6*(-904503+608640*logy*(-1+y)**4
        +4226652*y-7557258*y**2+6255452*y**3-2172503*y**4+121728*y**5+27477*y**6
        -276*y**7+5470*y**8-1460*y**9+85*y**10)+7*r**8*(-1+y)**3*(-1628975
        +62700*logy*(-1+y)**7+11340125*y-33800925*y**2+55895975*y**3-55347350*y**4
        +32769510*y**5-10888310*y**6+1716878*y**7-45339*y**8-23491*y**9+307*y**10
        +75*y**11)+63*r**12*(-1+y)*(-4565+1500*logy*(-1+y)**9+39585*y-151590*y**2
        +335710*y**3-472065*y**4+434265*y**5-285765*y**6+145269*y**7-56226*y**8
        +13694*y**9-2061*y**10+165*y**11)+7*r**14*(-8491+84910*y-382095*y**2
        +1018920*y**3-1783110*y**4+2139732*y**5-1874082*y**6+1226880*y**7-588615*y**8
        +185830*y**9-34951*y**10+2976*y**11)-14*MBhat**6*(-1+y)**3*(-6480*(-1
        +y)**7-10095*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(2875-1200*logy*(-1
        +y)**2-6950*y+4675*y**2+1088*y**3-1825*y**4+674*y**5+15*y**6)+24*r**4*(
        -1+y)**3*(-275+1100*y-1650*y**2+610*y**3+560*y**4-538*y**5+139*y**6)
        +18*r**6*(-1+y)*(-300-150*logy*(-1+y)**6+1650*y-3675*y**2+5070*y**3
        -4410*y**4+2220*y**5-563*y**6+48*y**7)-20*r**8*(710-4970*y+14910*y**2
        -27022*y**3+31885*y**4-24435*y**5+11627*y**6-3149*y**7+372*y**8))+42*MBhat**5*(
        -1+y)**2*(-12960*(-1+y)**8+15204*r**12*(-1+y)**8+r**2*(-1+y)**6*(65996
        -23280*logy*(-1+y)**2-155272*y+100916*y**2+3730*y**3-13580*y**4+3745*y**5
        -30*y**6+15*y**7)-12*r**4*(-1+y)**4*(4675+900*logy*(-1+y)**4-17800*y
        +24900*y**2-12550*y**3-1165*y**4+3252*y**5-1322*y**6+226*y**7)-6*r**6*(
        -1+y)**2*(800-4800*y+12000*y**2-22870*y**3+26170*y**4-13663*y**5
        -144*y**6+3710*y**7-1718*y**8+275*y**9)+15*r**10*(-2356+18848*y-65968*y**2
        +137382*y**3-185048*y**4+165133*y**5-96712*y**6+35708*y**7-7492*y**8
        +673*y**9)-20*r**8*(-1400+11200*y-39200*y**2+82330*y**3-111659*y**4
        +97306*y**5-51560*y**6+13762*y**7+49*y**8-1014*y**9+186*y**10))-21*MBhat**4*(
        -1+y)*(-64800*(-1+y)**9-43568*r**14*(-1+y)**9+r**2*(-1+y)**7*(592673
        -167040*logy*(-1+y)**2-1352386*y+843233*y**2-16800*y**3-39195*y**4
        -2250*y**5+2335*y**6-20*y**7+10*y**8)-3*r**4*(-1+y)**5*(206269+79680*logy*(
        -1+y)**4-745396*y+958734*y**2-453236*y**3+12904*y**4+29300*y**5-4350*y**6
        +20*y**7+75*y**8)-6*r**6*(-1+y)**3*(-7600-9450*logy*(-1+y)**6+36150*y
        -62025*y**2+5210*y**3+71565*y**4-56974*y**5+8125*y**6+7302*y**7-3353*y**8
        +400*y**9)+2*r**8*(-1+y)**2*(-65400-6450*logy*(-1+y)**7+464250*y
        -1415325*y**2+2569145*y**3-2728955*y**4+1536891*y**5-222631*y**6-264941*y**7
        +186231*y**8-49825*y**9+4960*y**10)-3*r**12*(45709-411381*y+1645524*y**2
        -3933316*y**3+6148314*y**4-6514650*y**5+4708116*y**6-2278404*y**7+699921*y**8
        -121209*y**9+8688*y**10)+15*r**10*(11935-107415*y+429660*y**2-1033932*y**3
        +1621152*y**4-1687280*y**5+1148286*y**6-475422*y**7+88473*y**8+12623*y**9
        -9426*y**10+1346*y**11))+21*MBhat**2*(64800*(-1+y)**10+8558*r**16*(-1
        +y)**10+r**2*(-1+y)**8*(-1361318+251040*logy*(-1+y)**2+2973676*y
        -1737878*y**2+83680*y**3-6215*y**4+12958*y**5+9474*y**6-2082*y**7+105*y**8)
        +r**4*(-1+y)**6*(-414092+2180160*logy*(-1+y)**4+3836528*y-10115112*y**2
        +11103728*y**3-4901207*y**4+422220*y**5+72275*y**6+12280*y**7-3645*y**8
        -20*y**9+5*y**10)+4*r**6*(-1+y)**4*(515473+74760*logy*(-1+y)**6-3018078*y
        +7320915*y**2-9387420*y**3+6625860*y**4-2413044*y**5+366789*y**6-13530*y**7
        +2961*y**8-1876*y**9+150*y**10)+2*r**8*(-1+y)**3*(174075-72900*logy*(
        -1+y)**7-1145625*y+3181725*y**2-4792575*y**3+3944535*y**4-1730757*y**5
        +406771*y**6-79537*y**7+31374*y**8-13426*y**9+1840*y**10)+r**10*(-1
        +y)**2*(-35250+17400*logy*(-1+y)**8+299400*y-1117500*y**2+2397400*y**3
        -3901445*y**4+3968080*y**5-2191613*y**6+471304*y**7+115171*y**8-112952*y**9
        +29245*y**10-2480*y**11)+2*r**14*(-17654+176540*y-794430*y**2+2118480*y**3
        -3821055*y**4+4849110*y**5-4377723*y**6+2788440*y**7-1224855*y**8+351830*y**9
        -59075*y**10+4344*y**11)+r**12*(58868-588680*y+2649060*y**2-7064160*y**3
        +12966345*y**4-16975770*y**5+15666588*y**6-9973182*y**7+4238070*y**8
        -1121390*y**9+143984*y**10+2286*y**11-2019*y**12))+6*MBhat*(-90720*(
        -1+y)**10+9412*r**16*(-1+y)**10+28*r**4*(-1+y)**6*(220153-238140*logy*(
        -1+y)**4-1118752*y+2154408*y**2-1912552*y**3+716278*y**4-51696*y**5
        -5292*y**6-3540*y**7+285*y**8+60*y**9)-r**2*(-1+y)**8*(-2556172+398160*logy*(
        -1+y)**2+5510504*y-3153412*y**2+132720*y**3+33180*y**4-38325*y**5
        +34286*y**6-382*y**7-1294*y**8+255*y**9)+28*r**8*(-1+y)**3*(3775+28500*logy*(
        -1+y)**7-54925*y+264525*y**2-640375*y**3+889750*y**4-699768*y**5
        +298174*y**6-66748*y**7+8118*y**8-28*y**9+22*y**10)-21*r**6*(-1+y)**4*(419172
        +204240*logy*(-1+y)**6-2310792*y+5164260*y**2-5864480*y**3+3377160*y**4
        -755613*y**5-61298*y**6+30555*y**7+1480*y**8-899*y**9-30*y**10+5*y**11)
        -7*r**10*(-1+y)**2*(-36460+291680*y-1020880*y**2+2041760*y**3-2552200*y**4
        +1833863*y**5-671712*y**6+98721*y**7-11736*y**8+6557*y**9-2548*y**10
        +275*y**11)+21*r**14*(-2204+22040*y-99180*y**2+264480*y**3-462840*y**4
        +578151*y**5-526526*y**6+346115*y**7-157620*y**8+47365*y**9-8438*y**10
        +673*y**11)+84*r**12*(717-7170*y+32265*y**2-86040*y**3+150570*y**4
        -196126*y**5+199988*y**6-152002*y**7+79940*y**8-27420*y**9+5953*y**10
        -706*y**11+31*y**12))+2*MBhat**3*(-907200*(-1+y)**10+210960*r**16*(-1
        +y)**10-42*r**4*(-1+y)**6*(233644+275280*logy*(-1+y)**4-659296*y
        +438384*y**2+267774*y**3-330876*y**4+45155*y**5+11540*y**6-1960*y**7
        -60*y**8+15*y**9)+r**2*(-1+y)**8*(13178136-2966880*logy*(-1+y)**2
        -29323152*y+17628456*y**2-745150*y**3-115780*y**4-257740*y**5+14700*y**6
        +6945*y**7-30*y**8+15*y**9)-42*r**6*(-1+y)**4*(84280-33600*logy*(-1
        +y)**6-539280*y+1449000*y**2-2121735*y**3+1709850*y**4-677619*y**5
        +70212*y**6+35871*y**7-14742*y**8+1763*y**9)+56*r**8*(-1+y)**3*(-30950
        +216650*y-649950*y**2+1125505*y**3-1067635*y**4+490335*y**5-20578*y**6
        -98570*y**7+56394*y**8-13396*y**9+1195*y**10)-21*r**10*(-1+y)**2*(59800
        -478400*y+1674400*y**2-3537270*y**3+4339080*y**4-2975422*y**5+795576*y**6
        +416559*y**7-501348*y**8+211000*y**9-42620*y**10+3365*y**11)+14*r**14*(
        -53768+537680*y-2419560*y**2+6565875*y**3-11814390*y**4+14717577*y**5
        -12893748*y**6+7907010*y**7-3302100*y**8+883565*y**9-133286*y**10+8169*y**11)
        -42*r**12*(-27292+272920*y-1228140*y**2+3368910*y**3-6071760*y**4+7392615*y**5
        -6098490*y**6+3288789*y**7-1004670*y**8+50065*y**9+82790*y**10-28995*y**11
        +3258*y**12)))/(25200*r**2*(-1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-14*MBhat**8*(-1+y)**4*(-6480*(-1+y)**7-10095*r**10*(-1+y)**7+5*r**2*(
        -1+y)**5*(2875-1200*logy*(-1+y)**2-6950*y+4675*y**2+1088*y**3-1825*y**4
        +674*y**5+15*y**6)+24*r**4*(-1+y)**3*(-275+1100*y-1650*y**2+610*y**3
        +560*y**4-538*y**5+139*y**6)+18*r**6*(-1+y)*(-300-150*logy*(-1+y)**6
        +1650*y-3675*y**2+5070*y**3-4410*y**4+2220*y**5-563*y**6+48*y**7)
        -20*r**8*(710-4970*y+14910*y**2-27022*y**3+31885*y**4-24435*y**5+11627*y**6
        -3149*y**7+372*y**8))+56*MBhat**7*(-1+y)**3*(-12960*(-1+y)**8+15204*r**12*(
        -1+y)**8+r**2*(-1+y)**6*(65996-23280*logy*(-1+y)**2-155272*y+100916*y**2
        +3730*y**3-13580*y**4+3745*y**5-30*y**6+15*y**7)-12*r**4*(-1+y)**4*(4675
        +900*logy*(-1+y)**4-17800*y+24900*y**2-12550*y**3-1165*y**4+3252*y**5
        -1322*y**6+226*y**7)-6*r**6*(-1+y)**2*(800-4800*y+12000*y**2-22870*y**3
        +26170*y**4-13663*y**5-144*y**6+3710*y**7-1718*y**8+275*y**9)+15*r**10*(
        -2356+18848*y-65968*y**2+137382*y**3-185048*y**4+165133*y**5-96712*y**6
        +35708*y**7-7492*y**8+673*y**9)-20*r**8*(-1400+11200*y-39200*y**2
        +82330*y**3-111659*y**4+97306*y**5-51560*y**6+13762*y**7+49*y**8-1014*y**9
        +186*y**10))+84*MBhat**6*(-1+y)**2*(30240*(-1+y)**9+21784*r**14*(-1
        +y)**9+r**2*(-1+y)**7*(-278727+80160*logy*(-1+y)**2+637614*y-398967*y**2
        +7280*y**3+20875*y**4-230*y**5-730*y**6+10*y**7-5*y**8)+r**4*(-1+y)**5*(290341
        +109920*logy*(-1+y)**4-1051444*y+1357326*y**2-645204*y**3+16226*y**4
        +47116*y**5-9154*y**6+716*y**7+125*y**8)+2*r**6*(-1+y)**3*(-8700-12450*logy*(
        -1+y)**6+39750*y-62025*y**2-24910*y**3+127105*y**4-93710*y**5+14593*y**6
        +10670*y**7-5053*y**8+600*y**9)+2*r**8*(-1+y)**2*(34400+2550*logy*(
        -1+y)**7-243350*y+738975*y**2-1332035*y**3+1413425*y**4-805193*y**5
        +130377*y**6+126235*y**7-91849*y**8+24775*y**9-2480*y**10)+3*r**12*(23191
        -208719*y+834876*y**2-1994924*y**3+3116556*y**4-3299724*y**5+2382324*y**6
        -1151316*y**7+352989*y**8-60941*y**9+4344*y**10)-5*r**10*(18427-165843*y
        +663372*y**2-1594956*y**3+2500538*y**4-2607450*y**5+1783425*y**6-746857*y**7
        +144873*y**8+16343*y**9-13891*y**10+2019*y**11))+3*(30240*(-1+y)**11
        +2019*r**18*(-1+y)**11+r**2*(-1+y)**9*(-1785859+225120*logy*(-1+y)**2
        +3796838*y-2123539*y**2+75040*y**3+18760*y**4+7504*y**5+3752*y**6
        -22368*y**7+21367*y**8-5198*y**9+823*y**10)+14*r**6*(-1+y)**5*(-932054
        +2704620*logy*(-1+y)**6+8296944*y-28856220*y**2+51998060*y**3-52521645*y**4
        +29122518*y**5-7558373*y**6+374740*y**7+57319*y**8+16706*y**9+2645*y**10
        -880*y**11)+56*r**8*(-1+y)**4*(-581026+421320*logy*(-1+y)**7+3645862*y
        -9462966*y**2+12822370*y**3-9135820*y**4+2532252*y**5+630536*y**6-521768*y**7
        +63719*y**8+7227*y**9-811*y**10-175*y**11)+112*r**12*(-1+y)**2*(18593
        -5700*logy*(-1+y)**9-161637*y+620898*y**2-1380362*y**3+1950843*y**4
        -1807203*y**5+1085102*y**6-408454*y**7+90812*y**8-10238*y**9-92*y**10
        +58*y**11)+16*r**4*(-1+y)**7*(-1538241+719880*logy*(-1+y)**4+6872844*y
        -11749026*y**2+9272444*y**3-3037991*y**4+143976*y**5+23996*y**6+10064*y**7
        -475*y**8+3530*y**9-825*y**10+82*y**11)+14*r**10*(-1+y)**3*(632240
        -6300*logy*(-1+y)**8-5064220*y+17749970*y**2-35558740*y**3+44536625*y**4
        -35717500*y**5+17917550*y**6-5226260*y**7+752554*y**8-20632*y**9-10388*y**10
        +376*y**11+25*y**12)+14*r**14*(-1+y)*(8854+2700*logy*(-1+y)**10-85840*y
        +372780*y**2-953580*y**3+1587765*y**4-1791918*y**5+1379865*y**6-768692*y**7
        +332165*y**8-112450*y**9+23879*y**10-3128*y**11+220*y**12)+8*r**16*(2018
        -22198*y+110990*y**2-332970*y**3+665940*y**4-932316*y**5+932316*y**6
        -695180*y**7+393745*y**8-167215*y**9+47223*y**10-8013*y**11+620*y**12))
        +8*MBhat**5*(-1+y)*(-635040*(-1+y)**10+210960*r**16*(-1+y)**10-r**2*(
        -1+y)**8*(-9365124+2215920*logy*(-1+y)**2+20946168*y-12689004*y**2
        +494830*y**3+218260*y**4+127120*y**5+420*y**6-3165*y**7+30*y**8-15*y**9)
        -42*r**4*(-1+y)**6*(176684+184080*logy*(-1+y)**4-522656*y+415824*y**2
        +100414*y**3-207436*y**4+36203*y**5+4644*y**6-608*y**7-60*y**8+15*y**9)
        -42*r**6*(-1+y)**4*(47830-21000*logy*(-1+y)**6-307980*y+832950*y**2
        -1237335*y**3+1008930*y**4-392952*y**5+24582*y**6+33675*y**7-12912*y**8
        +1532*y**9)+14*r**8*(-1+y)**3*(-103400+723800*y-2171400*y**2+3788020*y**3
        -3703060*y**4+1796613*y**5-87763*y**6-407774*y**7+245250*y**8-60691*y**9
        +5605*y**10)-21*r**10*(-1+y)**2*(73460-587680*y+2056880*y**2-4302230*y**3
        +5423860*y**4-4050192*y**5+1493416*y**6+140059*y**7-438188*y**8+204670*y**9
        -42620*y**10+3365*y**11)+14*r**14*(-59198+591980*y-2663910*y**2+7217475*y**3
        -12954690*y**4+16085937*y**5-14034048*y**6+8558610*y**7-3546450*y**8
        +937865*y**9-138716*y**10+8169*y**11)-84*r**12*(-16814+168140*y-756630*y**2
        +2064615*y**3-3718740*y**4+4568100*y**5-3850614*y**6+2169477*y**7-740925*y**8
        +96560*y**9+28690*y**10-13488*y**11+1629*y**12))+2*MBhat**2*(1270080*(
        -1+y)**11+57183*r**18*(-1+y)**11+42*r**4*(-1+y)**7*(-5338893+4023840*logy*(
        -1+y)**4+25379412*y-46116798*y**2+38792212*y**3-13721893*y**4+829008*y**5
        +127271*y**6+47620*y**7+9202*y**8-1068*y**9-25*y**10)+r**2*(-1+y)**9*(
        -48968333+7506240*logy*(-1+y)**2+105442906*y-60227693*y**2+2502080*y**3
        +625520*y**4-265888*y**5+148414*y**6+223828*y**7-26261*y**8+3702*y**9
        +765*y**10)+14*r**8*(-1+y)**4*(-6724853+585060*logy*(-1+y)**7+46488911*y
        -137419023*y**2+224936285*y**3-219817010*y**4+127063986*y**5-40126082*y**6
        +5806346*y**7-264273*y**8-9529*y**9-16823*y**10+1425*y**11)+42*r**10*(
        -1+y)**3*(-353565-120000*logy*(-1+y)**8+2708520*y-8999820*y**2+16879640*y**3
        -19419550*y**4+13365560*y**5-5245865*y**6+1136008*y**7-152979*y**8+18544*y**9
        -11213*y**10+1640*y**11)+42*r**6*(-1+y)**5*(4578093+4858260*logy*(-1
        +y)**6-22610298*y+41950965*y**2-31643320*y**3-558810*y**4+14730960*y**5
        -7339368*y**6+814980*y**7+71523*y**8+4890*y**9-2925*y**10-60*y**11
        +10*y**12)-42*r**12*(-1+y)**2*(-1051-8700*logy*(-1+y)**9+18159*y
        -111786*y**2+365234*y**3-730551*y**4+1526319*y**5-1774875*y**6+1033755*y**7
        -261258*y**8+7462*y**9+21753*y**10-6297*y**11+496*y**12)+18*r**16*(17267
        -189937*y+949685*y**2-2849055*y**3+5698110*y**4-8222970*y**5+8731338*y**6
        -6815838*y**7+3838395*y**8-1518385*y**9+399245*y**10-62359*y**11+4344*y**12)
        -14*r**14*(46529-511819*y+2559095*y**2-7677285*y**3+15354570*y**4-22636158*y**5
        +25283184*y**6-20831136*y**7+12228645*y**8-4960655*y**9+1348897*y**10
        -220115*y**11+16248*y**12))+8*MBhat*(-90720*(-1+y)**11+4344*r**18*(
        -1+y)**11-r**2*(-1+y)**9*(-4389896+606480*logy*(-1+y)**2+9386272*y
        -5299616*y**2+202160*y**3+50540*y**4+20216*y**5-43568*y**6+32509*y**7
        +3934*y**8-1683*y**9+492*y**10)+21*r**10*(-1+y)**3*(41920+90000*logy*(
        -1+y)**8-245360*y+498760*y**2-157520*y**3-1063100*y**4+2110480*y**5
        -1782860*y**6+761187*y**7-175712*y**8+25694*y**9-120*y**10+71*y**11)
        -3*r**4*(-1+y)**7*(-12463536+7049280*logy*(-1+y)**4+56903424*y-99453696*y**2
        +80401024*y**3-27149536*y**4+1409856*y**5+271572*y**6+42197*y**7+46196*y**8
        -834*y**9-704*y**10+85*y**11)+42*r**6*(-1+y)**5*(-212220-1064400*logy*(
        -1+y)**6+208920*y+2670900*y**2-8883200*y**3+11984400*y**4-7986960*y**5
        +2403456*y**6-155395*y**7-26636*y**8-3400*y**9+280*y**10+95*y**11)
        -14*r**8*(-1+y)**4*(-2437496+877920*logy*(-1+y)**7+16184552*y-45480936*y**2
        +69656120*y**3-61974320*y**4+31039152*y**5-7358564*y**6+238355*y**7
        +138387*y**8-8938*y**9-2702*y**10-105*y**11+15*y**12)-21*r**12*(-1
        +y)**2*(21424-192816*y+771264*y**2-1799616*y**3+2699424*y**4-2699424*y**5
        +1668900*y**6-565387*y**7+82831*y**8-5594*y**9+574*y**10-451*y**11
        +55*y**12)+12*r**16*(2380-26180*y+130900*y**2-392700*y**3+785400*y**4
        -1099560*y**5+1140496*y**6-887876*y**7+512140*y**8-207845*y**9+56375*y**10
        -9163*y**11+673*y**12)+28*r**14*(-1514+16654*y-83270*y**2+249810*y**3
        -499620*y**4+699468*y**5-763374*y**6+692007*y**7-489966*y**8+244475*y**9
        -79474*y**10+16685*y**11-1974*y**12+93*y**13))+24*MBhat**3*(-211680*(
        -1+y)**11+11720*r**18*(-1+y)**11-r**2*(-1+y)**9*(-6266500+1083600*logy*(
        -1+y)**2+13616600*y-7891900*y**2+361200*y**3+36400*y**4+833*y**5
        +49742*y**6+4107*y**7-892*y**8+250*y**9)-28*r**8*(-1+y)**4*(-10115
        -33300*logy*(-1+y)**7+104105*y-428865*y**2+947875*y**3-1270000*y**4
        +967393*y**5-371677*y**6+53652*y**7+6936*y**8-5845*y**9+661*y**10)
        -r**4*(-1+y)**7*(-11434932+14661360*logy*(-1+y)**4+60401088*y-119924352*y**2
        +109272288*y**3-41879612*y**4+2970842*y**5+510384*y**6+126145*y**7+2780*y**8
        -2280*y**9-20*y**10+5*y**11)-7*r**6*(-1+y)**5*(2512564+1158480*logy*(
        -1+y)**6-13916904*y+31316820*y**2-35963360*y**3+21146840*y**4-5054659*y**5
        -199654*y**6+146293*y**7+13600*y**8-4525*y**9-450*y**10+75*y**11)+7*r**10*(
        -1+y)**3*(68940-551520*y+1930320*y**2-3860640*y**3+5080700*y**4-4044673*y**5
        +1740760*y**6-286668*y**7-71616*y**8+62891*y**9-15224*y**10+1330*y**11)
        -7*r**12*(-1+y)**2*(-23500+211500*y-846000*y**2+1974000*y**3-3268300*y**4
        +3452246*y**5-2213634*y**6+764799*y**7-43875*y**8-100380*y**9+48124*y**10
        -9413*y**11+673*y**12)+4*r**16*(13563-149193*y+745965*y**2-2237895*y**3
        +4578130*y**4-6661424*y**5+7022260*y**6-5375436*y**7+2957295*y**8-1135530*y**9
        +287012*y**10-42430*y**11+2723*y**12)-7*r**14*(15124-166364*y+831820*y**2
        -2495460*y**3+5198080*y**4-7746971*y**5+8249953*y**6-6246669*y**7+3311127*y**8
        -1175185*y**9+243023*y**10-14859*y**11-4343*y**12+724*y**13))+MBhat**4*(6350400*(
        -1+y)**11+568575*r**18*(-1+y)**11+r**2*(-1+y)**9*(-137219775+27384000*logy*(
        -1+y)**2+301823550*y-178295775*y**2+8261120*y**3+627830*y**4+1524404*y**5
        +632142*y**6-60816*y**7+2505*y**8+30*y**9-15*y**10)+84*r**4*(-1+y)**7*(
        -159068+2401440*logy*(-1+y)**4+3037712*y-9359448*y**2+11059152*y**3
        -5131663*y**4+486620*y**5+73605*y**6+10240*y**7-1985*y**8-60*y**9+15*y**10)
        +42*r**6*(-1+y)**5*(3969584+608880*logy*(-1+y)**6-23208624*y+56194920*y**2
        -71923920*y**3+50450965*y**4-17909374*y**5+2359999*y**6+86900*y**7-30125*y**8
        -8550*y**9+1425*y**10)-56*r**8*(-1+y)**4*(-429625+157500*logy*(-1+y)**7
        +2849875*y-7998375*y**2+12357125*y**3-10192325*y**4+3932391*y**5-68951*y**6
        -641323*y**7+324300*y**8-62132*y**9+4640*y**10)+84*r**10*(-1+y)**3*(42600
        +12750*logy*(-1+y)**8-328050*y+1097175*y**2-2278150*y**3+2036700*y**4
        -334078*y**5-988413*y**6+1197502*y**7-764498*y**8+269512*y**9-50980*y**10
        +4080*y**11)-84*r**12*(-1+y)**2*(-51100+459900*y-1839600*y**2+4560720*y**3
        -6533435*y**4+5542979*y**5-2354019*y**6-341733*y**7+1178259*y**8-757971*y**9
        +249575*y**10-42551*y**11+2896*y**12)+24*r**16*(96047-1056517*y+5282585*y**2
        -16052435*y**3+32729620*y**4-46983188*y**5+48530328*y**6-36211560*y**7
        +19266060*y**8-7066620*y**9+1671164*y**10-222244*y**11+11720*y**12)
        -14*r**14*(287008-3157088*y+15785440*y**2-48528960*y**3+99030765*y**4
        -139368495*y**5+137505291*y**6-94030689*y**7+41893203*y**8-9562465*y**9
        -782815*y**10+1231037*y**11-334908*y**12+32676*y**13)))/(25200*r**2*(-1
        +y)**11)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr7(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r7 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-720*(-1+y)**7-5655*r**10*(-1+y)**7-5*r**2*(-1+y)**5*(-35+1680*logy*(
        -1+y)**2+1750*y-2555*y**2+608*y**3+305*y**4-226*y**5-15*y**6)+24*r**4*(
        -1+y)**3*(125-500*y+750*y**2-310*y**3-60*y**4+38*y**5+11*y**6)-6*r**6*(
        -1+y)*(-300+450*logy*(-1+y)**6+2250*y-6975*y**2+13910*y**3-17030*y**4
        +12660*y**5-5339*y**6+944*y**7)-20*r**8*(70-490*y+1470*y**2-3518*y**3
        +6545*y**4-7215*y**5+4543*y**6-1561*y**7+228*y**8))/(1800*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-1440*(-1+y)**8-3393*r**12*(-1+y)**8-r**2*(-1+y)**6*(-131393+61440*logy*(
        -1+y)**2+324226*y-223553*y**2+20480*y**3+8615*y**4-430*y**5+375*y**6)
        +r**4*(-1+y)**4*(-116975-69600*logy*(-1+y)**4+398300*y-458250*y**2
        +166300*y**3+41180*y**4-33996*y**5+7474*y**6-1516*y**7+75*y**8)-6*r**6*(
        -1+y)**2*(600+2550*logy*(-1+y)**6-1050*y-5025*y**2+19450*y**3-20935*y**4
        +4602*y**5+7061*y**6-5666*y**7+1203*y**8)-2*r**8*(-1+y)*(-4600+4050*logy*(
        -1+y)**7+28150*y-70275*y**2+88775*y**3-43865*y**4-5535*y**5+19919*y**6
        -10763*y**7+549*y**8+525*y**9)-5*r**10*(-643+5144*y-18004*y**2+36008*y**3
        -51772*y**4+62168*y**5-51136*y**6+25640*y**7-7309*y**8+912*y**9)-2*MBhat**2*(
        -1+y)*(720*(-1+y)**7+5655*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(-35+1680*logy*(
        -1+y)**2+1750*y-2555*y**2+608*y**3+305*y**4-226*y**5-15*y**6)-24*r**4*(
        -1+y)**3*(125-500*y+750*y**2-310*y**3-60*y**4+38*y**5+11*y**6)+6*r**6*(
        -1+y)*(-300+450*logy*(-1+y)**6+2250*y-6975*y**2+13910*y**3-17030*y**4
        +12660*y**5-5339*y**6+944*y**7)+20*r**8*(70-490*y+1470*y**2-3518*y**3
        +6545*y**4-7215*y**5+4543*y**6-1561*y**7+228*y**8))-2*MBhat*(-1440*(
        -1+y)**8+7476*r**12*(-1+y)**8+r**2*(-1+y)**6*(32924-31920*logy*(-1
        +y)**2-97768*y+80804*y**2-14630*y**3+580*y**4-215*y**5-30*y**6+15*y**7)
        -4*r**4*(-1+y)**4*(4525+1500*logy*(-1+y)**4-16600*y+21900*y**2-14450*y**3
        +5645*y**4-1356*y**5-734*y**6+422*y**7)-6*r**6*(-1+y)**2*(1600-9600*y
        +24000*y**2-25830*y**3+7730*y**4+9513*y**5-10856*y**6+4690*y**7-1182*y**8
        +175*y**9)+5*r**10*(-1132+9056*y-31696*y**2+70154*y**3-110056*y**4
        +116951*y**5-82264*y**6+37076*y**7-9724*y**8+1131*y**9)-20*r**8*(280
        -2240*y+7840*y**2-16910*y**3+24769*y**4-26906*y**5+22360*y**6-13262*y**7
        +5101*y**8-1146*y**9+114*y**10)))/(3600*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-720*(-1+y)**9-754*r**14*(-1+y)**9-24*r**4*(-1+y)**5*(-121+10980*logy*(
        -1+y)**4+11464*y-39156*y**2+48064*y**3-22996*y**4+1742*y**5+1105*y**6
        -206*y**7+50*y**8)-r**2*(-1+y)**7*(-196534+63120*logy*(-1+y)**2+456188*y
        -291214*y**2+21040*y**3+5260*y**4+4048*y**5+29*y**6+278*y**7+65*y**8)
        -6*r**6*(-1+y)**3*(27725+14100*logy*(-1+y)**6-152250*y+338325*y**2
        -380600*y**3+214950*y**4-39384*y**5-15851*y**6+9366*y**7-2577*y**8
        +416*y**9)-4*r**8*(-1+y)**2*(-5700+4800*logy*(-1+y)**7+35100*y-88500*y**2
        +113900*y**3-71900*y**4+19428*y**5-4103*y**6+8285*y**7-6813*y**8+1383*y**9)
        -12*r**12*(118-1062*y+4248*y**2-9912*y**3+14868*y**4-16032*y**5+14781*y**6
        -9969*y**7+4221*y**8-1039*y**9+114*y**10)-3*r**10*(-1+y)*(3410+1800*logy*(
        -1+y)**8-25480*y+81980*y**2-147160*y**3+158750*y**4-98880*y**5+38607*y**6
        -8336*y**7-314*y**8-232*y**9+175*y**10)-MBhat**4*(-1+y)**2*(720*(-1
        +y)**7+5655*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(-35+1680*logy*(-1+y)**2
        +1750*y-2555*y**2+608*y**3+305*y**4-226*y**5-15*y**6)-24*r**4*(-1
        +y)**3*(125-500*y+750*y**2-310*y**3-60*y**4+38*y**5+11*y**6)+6*r**6*(
        -1+y)*(-300+450*logy*(-1+y)**6+2250*y-6975*y**2+13910*y**3-17030*y**4
        +12660*y**5-5339*y**6+944*y**7)+20*r**8*(70-490*y+1470*y**2-3518*y**3
        +6545*y**4-7215*y**5+4543*y**6-1561*y**7+228*y**8))-2*MBhat**3*(-1
        +y)*(-1440*(-1+y)**8+7476*r**12*(-1+y)**8+r**2*(-1+y)**6*(32924-31920*logy*(
        -1+y)**2-97768*y+80804*y**2-14630*y**3+580*y**4-215*y**5-30*y**6
        +15*y**7)-4*r**4*(-1+y)**4*(4525+1500*logy*(-1+y)**4-16600*y+21900*y**2
        -14450*y**3+5645*y**4-1356*y**5-734*y**6+422*y**7)-6*r**6*(-1+y)**2*(1600
        -9600*y+24000*y**2-25830*y**3+7730*y**4+9513*y**5-10856*y**6+4690*y**7
        -1182*y**8+175*y**9)+5*r**10*(-1132+9056*y-31696*y**2+70154*y**3-110056*y**4
        +116951*y**5-82264*y**6+37076*y**7-9724*y**8+1131*y**9)-20*r**8*(280
        -2240*y+7840*y**2-16910*y**3+24769*y**4-26906*y**5+22360*y**6-13262*y**7
        +5101*y**8-1146*y**9+114*y**10))-4*MBhat*(-720*(-1+y)**9+890*r**14*(
        -1+y)**9-2*r**2*(-1+y)**7*(-55557+21660*logy*(-1+y)**2+132774*y
        -88047*y**2+7220*y**3+2720*y**4+245*y**5+180*y**6+45*y**7)-24*r**4*(
        -1+y)**5*(3510+3600*logy*(-1+y)**4-10440*y+8460*y**2+1560*y**3-4390*y**4
        +1427*y**5-254*y**6+73*y**7)-r**6*(-1+y)**3*(19850+13800*logy*(-1
        +y)**6-105300*y+221850*y**2-226800*y**3+122280*y**4-46683*y**5+30270*y**6
        -18096*y**7+3330*y**8+19*y**9)+45*r**10*(34-306*y+1224*y**2-2856*y**3
        +4466*y**4-4585*y**5+3279*y**6-1926*y**7+874*y**8-231*y**9+27*y**10)
        -r**8*(-1+y)**2*(-4400+30800*y-92400*y**2+154000*y**3-120760*y**4
        +16659*y**5+55367*y**6-49502*y**7+16242*y**8-2581*y**9+175*y**10)+r**12*(1264
        -11376*y+45504*y**2-106176*y**3+170904*y**4-213561*y**5+191001*y**6
        -115884*y**7+45936*y**8-10759*y**9+1131*y**10))+MBhat**2*(-4320*(-1
        +y)**9-9016*r**14*(-1+y)**9-5*r**2*(-1+y)**7*(-62777+33120*logy*(-1
        +y)**2+158674*y-112457*y**2+13008*y**3+1737*y**4+702*y**5+104*y**6
        +2*y**7-y**8)-r**4*(-1+y)**5*(267959+137280*logy*(-1+y)**4-934556*y
        +1127274*y**2-521596*y**3+69154*y**4-35052*y**5+20738*y**6-1772*y**7
        +75*y**8)+6*r**6*(-1+y)**3*(-9900-3150*logy*(-1+y)**6+56250*y-131175*y**2
        +145630*y**3-74325*y**4+4662*y**5+9807*y**6-966*y**7-903*y**8+200*y**9)
        +2*r**8*(-1+y)**2*(-4800-5250*logy*(-1+y)**7+38850*y-134925*y**2
        +193465*y**3-105415*y**4-94953*y**5+186781*y**6-130873*y**7+55335*y**8
        -13625*y**9+1520*y**10)-3*r**12*(2205-19845*y+79380*y**2-200740*y**3
        +361440*y**4-451872*y**5+391020*y**6-231660*y**7+89955*y**8-20675*y**9
        +2136*y**10)+5*r**10*(-2119+19071*y-76284*y**2+186684*y**3-311838*y**4
        +401790*y**5-408351*y**6+305751*y**7-160029*y**8+55981*y**9-11787*y**10
        +1131*y**11)))/(1800*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-10080*(-1+y)**10-5655*r**16*(-1+y)**10-r**2*(-1+y)**8*(-5572895+1478400*logy*(
        -1+y)**2+12624190*y-7790495*y**2+492800*y**3+123200*y**4+49280*y**5
        +51415*y**6+6082*y**7+3271*y**8+1392*y**9)-21*r**4*(-1+y)**6*(-638847
        +651360*logy*(-1+y)**4+3206748*y-6112842*y**2+5377948*y**3-1995847*y**4
        +130272*y**5+11073*y**6+22876*y**7-3170*y**8+860*y**9+65*y**10)-105*r**10*(
        -1+y)**2*(8239+4500*logy*(-1+y)**8-61412*y+196942*y**2-351884*y**3
        +376855*y**4-238484*y**5+86877*y**6-17156*y**7+6078*y**8-4968*y**9
        +929*y**10)-21*r**6*(-1+y)**4*(614145+638100*logy*(-1+y)**6-3046770*y
        +5702625*y**2-4413000*y**3+119250*y**4+1866600*y**5-922870*y**6+33232*y**7
        +56805*y**8-10862*y**9+1325*y**10)-7*r**8*(-1+y)**3*(-714775+396300*logy*(
        -1+y)**7+4607125*y-12434325*y**2+17949775*y**3-14482150*y**4+5915190*y**5
        -436990*y**6-572378*y**7+237789*y**8-66059*y**9+8243*y**10+75*y**11)
        -21*r**12*(-1+y)*(-10695+4500*logy*(-1+y)**9+91755*y-346770*y**2
        +755130*y**3-1038195*y**4+924795*y**5-513595*y**6+193963*y**7-49302*y**8
        +4138*y**9-787*y**10+315*y**11)-7*r**14*(-2459+24590*y-110655*y**2
        +295080*y**3-516390*y**4+619668*y**5-538818*y**6+405120*y**7-232935*y**8
        +85670*y**9-18599*y**10+1824*y**11)-14*MBhat**6*(-1+y)**3*(720*(-1+y)**7
        +5655*r**10*(-1+y)**7+5*r**2*(-1+y)**5*(-35+1680*logy*(-1+y)**2+1750*y
        -2555*y**2+608*y**3+305*y**4-226*y**5-15*y**6)-24*r**4*(-1+y)**3*(125
        -500*y+750*y**2-310*y**3-60*y**4+38*y**5+11*y**6)+6*r**6*(-1+y)*(
        -300+450*logy*(-1+y)**6+2250*y-6975*y**2+13910*y**3-17030*y**4+12660*y**5
        -5339*y**6+944*y**7)+20*r**8*(70-490*y+1470*y**2-3518*y**3+6545*y**4
        -7215*y**5+4543*y**6-1561*y**7+228*y**8))-42*MBhat**5*(-1+y)**2*(-1440*(
        -1+y)**8+7476*r**12*(-1+y)**8+r**2*(-1+y)**6*(32924-31920*logy*(-1
        +y)**2-97768*y+80804*y**2-14630*y**3+580*y**4-215*y**5-30*y**6+15*y**7)
        -4*r**4*(-1+y)**4*(4525+1500*logy*(-1+y)**4-16600*y+21900*y**2-14450*y**3
        +5645*y**4-1356*y**5-734*y**6+422*y**7)-6*r**6*(-1+y)**2*(1600-9600*y
        +24000*y**2-25830*y**3+7730*y**4+9513*y**5-10856*y**6+4690*y**7-1182*y**8
        +175*y**9)+5*r**10*(-1132+9056*y-31696*y**2+70154*y**3-110056*y**4
        +116951*y**5-82264*y**6+37076*y**7-9724*y**8+1131*y**9)-20*r**8*(280
        -2240*y+7840*y**2-16910*y**3+24769*y**4-26906*y**5+22360*y**6-13262*y**7
        +5101*y**8-1146*y**9+114*y**10))-21*MBhat**4*(-1+y)*(7200*(-1+y)**9
        +18032*r**14*(-1+y)**9+r**2*(-1+y)**7*(-496377+269760*logy*(-1+y)**2
        +1262514*y-901017*y**2+109600*y**3+8755*y**4+7450*y**5+665*y**6+20*y**7
        -10*y**8)+r**4*(-1+y)**5*(418943+204960*logy*(-1+y)**4-1470812*y
        +1796298*y**2-876892*y**3+179488*y**4-104100*y**5+48950*y**6-5060*y**7
        +225*y**8)+6*r**6*(-1+y)**3*(19200+3750*logy*(-1+y)**6-111450*y+267375*y**2
        -310710*y**3+169585*y**4-13926*y**5-26675*y**6+7598*y**7+603*y**8-400*y**9)
        +2*r**8*(-1+y)**2*(14200+6450*logy*(-1+y)**7-105850*y+340125*y**2
        -475705*y**3+254695*y**4+195441*y**5-393481*y**6+272509*y**7-111219*y**8
        +26725*y**9-3040*y**10)+3*r**12*(5541-49869*y+199476*y**2-496484*y**3
        +865386*y**4-1046250*y**5+877044*y**6-504036*y**7+190089*y**8-42481*y**9
        +4272*y**10)-5*r**10*(-3595+32355*y-129420*y**2+319356*y**3-535896*y**4
        +689640*y**5-703398*y**6+534726*y**7-287109*y**8+103741*y**9-22662*y**10
        +2262*y**11))-6*MBhat*(-10080*(-1+y)**10+4628*r**16*(-1+y)**10-28*r**4*(
        -1+y)**6*(-108877+241260*logy*(-1+y)**4+676768*y-1497672*y**2+1480968*y**3
        -611502*y**4+41934*y**5+17788*y**6-760*y**7+615*y**8+90*y**9)-r**2*(
        -1+y)**8*(-3852988+1130640*logy*(-1+y)**2+8836616*y-5548948*y**2
        +376880*y**3+94220*y**4+59675*y**5+15694*y**6+4822*y**7+2074*y**8+195*y**9)
        +28*r**8*(-1+y)**3*(46475-25500*logy*(-1+y)**7-299825*y+810225*y**2
        -1171875*y**3+948750*y**4-418332*y**5+93226*y**6-24352*y**7+15882*y**8
        -2772*y**9+78*y**10)-21*r**6*(-1+y)**4*(262068+172560*logy*(-1+y)**6
        -1399848*y+2981940*y**2-3113120*y**3+1472040*y**4-51117*y**5-205522*y**6
        +62595*y**7-10280*y**8+1749*y**9-30*y**10+5*y**11)-7*r**10*(-1+y)**2*(9060
        -72480*y+253680*y**2-507360*y**3+634200*y**4-398313*y**5+59312*y**6
        +134429*y**7-118264*y**8+32293*y**9-4052*y**10+175*y**11)+7*r**14*(
        -1508+15080*y-67860*y**2+180960*y**3-316680*y**4+396837*y**5-404922*y**6
        +308505*y**7-162540*y**8+56855*y**9-11906*y**10+1131*y**11)+84*r**12*(
        -207+2070*y-9315*y**2+24840*y**3-43470*y**4+53816*y**5-42968*y**6
        +20652*y**7-7540*y**8+2470*y**9-303*y**10-64*y**11+19*y**12))-21*MBhat**2*(7200*(
        -1+y)**10+3542*r**16*(-1+y)**10+r**2*(-1+y)**8*(-1770342+593760*logy*(
        -1+y)**2+4134444*y-2660982*y**2+197920*y**3+61765*y**4+20902*y**5
        +5406*y**6+2142*y**7+345*y**8)+r**4*(-1+y)**6*(422292+2087040*logy*(
        -1+y)**4+397872*y-4770888*y**2+7354672*y**3-4018843*y**4+556380*y**5
        +30235*y**6+12680*y**7+2655*y**8-20*y**9+5*y**10)+12*r**6*(-1+y)**4*(84309
        +52080*logy*(-1+y)**6-453774*y+978195*y**2-1043860*y**3+539630*y**4
        -91152*y**5-7663*y**6-8590*y**7+3963*y**8-508*y**9+50*y**10)+2*r**8*(
        -1+y)**3*(-167425+45900*logy*(-1+y)**7+1126075*y-3217575*y**2+5041325*y**3
        -4457285*y**4+2097807*y**5-307721*y**6-128413*y**7+40326*y**8-6074*y**9
        +560*y**10)+r**10*(-1+y)**2*(2350+17400*logy*(-1+y)**8-1400*y-64700*y**2
        +291800*y**3-240505*y**4-140080*y**5+721763*y**6-762904*y**7+368879*y**8
        -113048*y**9+20005*y**10-1520*y**11)+2*r**14*(-2146+21460*y-96570*y**2
        +257520*y**3-478695*y**4+701190*y**5-751107*y**6+570360*y**7-301995*y**8
        +106270*y**9-22375*y**10+2136*y**11)-3*r**12*(2436-24360*y+109620*y**2
        -292320*y**3+535465*y**4-661850*y**5+623896*y**6-507094*y**7+317290*y**8
        -134430*y**9+36708*y**10-5738*y**11+377*y**12))-2*MBhat**3*(-100800*(
        -1+y)**10+69840*r**16*(-1+y)**10-r**2*(-1+y)**8*(-14340024+5802720*logy*(
        -1+y)**2+34482768*y-23044104*y**2+2058350*y**3+543620*y**4+127820*y**5
        +45780*y**6+8175*y**7+30*y**8-15*y**9)-42*r**4*(-1+y)**6*(260396+250320*logy*(
        -1+y)**4-791264*y+686256*y**2+23166*y**3-233884*y**4+44675*y**5+5140*y**6
        +1240*y**7-60*y**8+15*y**9)-42*r**6*(-1+y)**4*(72120+43200*logy*(-1
        +y)**6-389520*y+844200*y**2-872615*y**3+391850*y**4+16269*y**5-79612*y**6
        +25879*y**7-7158*y**8+987*y**9)-56*r**8*(-1+y)**3*(1150-8050*y+24150*y**2
        +4255*y**3-36385*y**4+37785*y**5-1978*y**6-14870*y**7+3594*y**8-796*y**9
        +145*y**10)-21*r**10*(-1+y)**2*(6200-49600*y+173600*y**2-220230*y**3
        +28520*y**4+440322*y**5-654376*y**6+496991*y**7-259252*y**8+90400*y**9
        -19180*y**10+1885*y**11)+14*r**14*(-712+7120*y-32040*y**2+113475*y**3
        -329910*y**4+620193*y**5-777252*y**6+667290*y**7-389700*y**8+148585*y**9
        -33454*y**10+3381*y**11)-42*r**12*(6372-63720*y+286740*y**2-779410*y**3
        +1424960*y**4-1922425*y**5+1975590*y**6-1523499*y**7+869370*y**8-360815*y**9
        +103990*y**10-18755*y**11+1602*y**12)))/(25200*r**2*(-1+y)**10)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-10080*(-1+y)**11-3393*r**18*(-1+y)**11-r**2*(-1+y)**9*(-9446273+2214240*logy*(
        -1+y)**2+21106786*y-12767633*y**2+738080*y**3+184520*y**4+73808*y**5
        +36904*y**6+46624*y**7+11149*y**8+2694*y**9+1581*y**10)-16*r**4*(-1
        +y)**7*(-3739987+2271360*logy*(-1+y)**4+17231308*y-30389682*y**2+24802508*y**3
        -8471987*y**4+454272*y**5+75712*y**6+3128*y**7+35445*y**8-3170*y**9
        +1145*y**10+174*y**11)-336*r**12*(-1+y)**2*(-4943+2100*logy*(-1+y)**9
        +42387*y-160098*y**2+348362*y**3-478443*y**4+425523*y**5-239582*y**6
        +82774*y**7-15042*y**8+2408*y**9-2008*y**10+342*y**11)-56*r**8*(-1
        +y)**4*(-902222+650040*logy*(-1+y)**7+5665514*y-14721402*y**2+19985390*y**3
        -14297540*y**4+4028244*y**5+932392*y**6-752296*y**7+25593*y**8+46469*y**9
        -8717*y**10+375*y**11)-42*r**6*(-1+y)**5*(95114+1865580*logy*(-1+y)**6
        +1294896*y-8833980*y**2+21106540*y**3-25157805*y**4+15659862*y**5-4475557*y**6
        +287060*y**7-13569*y**8+42994*y**9-6035*y**10+720*y**11)-14*r**10*(
        -1+y)**3*(914080+489900*logy*(-1+y)**8-6822740*y+21919990*y**2-39267580*y**3
        +42225875*y**4-26922100*y**5+8888650*y**6-402620*y**7-677762*y**8+224696*y**9
        -61036*y**10+5672*y**11+75*y**12)-14*r**14*(-1+y)*(22178+8100*logy*(
        -1+y)**10-213680*y+921060*y**2-2334660*y**3+3842655*y**4-4270986*y**5
        +3218955*y**6-1573404*y**7+537255*y**8-130550*y**9+12413*y**10-1416*y**11
        +420*y**12)-8*r**16*(1886-20746*y+103730*y**2-311190*y**3+622380*y**4
        -871332*y**5+871332*y**6-636660*y**7+404415*y**8-203105*y**9+66121*y**10
        -12851*y**11+1140*y**12)-14*MBhat**8*(-1+y)**4*(720*(-1+y)**7+5655*r**10*(
        -1+y)**7+5*r**2*(-1+y)**5*(-35+1680*logy*(-1+y)**2+1750*y-2555*y**2
        +608*y**3+305*y**4-226*y**5-15*y**6)-24*r**4*(-1+y)**3*(125-500*y
        +750*y**2-310*y**3-60*y**4+38*y**5+11*y**6)+6*r**6*(-1+y)*(-300+450*logy*(
        -1+y)**6+2250*y-6975*y**2+13910*y**3-17030*y**4+12660*y**5-5339*y**6
        +944*y**7)+20*r**8*(70-490*y+1470*y**2-3518*y**3+6545*y**4-7215*y**5
        +4543*y**6-1561*y**7+228*y**8))-56*MBhat**7*(-1+y)**3*(-1440*(-1+y)**8
        +7476*r**12*(-1+y)**8+r**2*(-1+y)**6*(32924-31920*logy*(-1+y)**2-97768*y
        +80804*y**2-14630*y**3+580*y**4-215*y**5-30*y**6+15*y**7)-4*r**4*(
        -1+y)**4*(4525+1500*logy*(-1+y)**4-16600*y+21900*y**2-14450*y**3
        +5645*y**4-1356*y**5-734*y**6+422*y**7)-6*r**6*(-1+y)**2*(1600-9600*y
        +24000*y**2-25830*y**3+7730*y**4+9513*y**5-10856*y**6+4690*y**7-1182*y**8
        +175*y**9)+5*r**10*(-1132+9056*y-31696*y**2+70154*y**3-110056*y**4
        +116951*y**5-82264*y**6+37076*y**7-9724*y**8+1131*y**9)-20*r**8*(280
        -2240*y+7840*y**2-16910*y**3+24769*y**4-26906*y**5+22360*y**6-13262*y**7
        +5101*y**8-1146*y**9+114*y**10))-28*MBhat**6*(-1+y)**2*(10080*(-1+y)**9
        +27048*r**14*(-1+y)**9+r**2*(-1+y)**7*(-678869+373920*logy*(-1+y)**2
        +1731658*y-1239749*y**2+154160*y**3+8825*y**4+11390*y**5+810*y**6
        +30*y**7-15*y**8)+r**4*(-1+y)**5*(569927+272640*logy*(-1+y)**4-2007068*y
        +2465322*y**2-1232188*y**3+289822*y**4-173148*y**5+77162*y**6-8348*y**7
        +375*y**8)+6*r**6*(-1+y)**3*(28500+4350*logy*(-1+y)**6-166650*y+403575*y**2
        -475790*y**3+264845*y**4-23190*y**5-43543*y**6+14230*y**7+303*y**8
        -600*y**9)+2*r**8*(-1+y)**2*(23600+7650*logy*(-1+y)**7-172850*y+545325*y**2
        -757945*y**3+403975*y**4+295929*y**5-600181*y**6+414145*y**7-167103*y**8
        +39825*y**9-4560*y**10)+9*r**12*(2959-26631*y+106524*y**2-264076*y**3
        +456444*y**4-546876*y**5+454356*y**6-258804*y**7+96741*y**8-21429*y**9
        +2136*y**10)-5*r**10*(-5071+45639*y-182556*y**2+452028*y**3-759954*y**4
        +977490*y**5-998445*y**6+763701*y**7-414189*y**8+151501*y**9-33537*y**10
        +3393*y**11))-8*MBhat**5*(-1+y)*(-70560*(-1+y)**10+69840*r**16*(-1
        +y)**10-r**2*(-1+y)**8*(-9673236+3983280*logy*(-1+y)**2+23329752*y
        -15648156*y**2+1451870*y**3+315140*y**4+107240*y**5+30660*y**6+4395*y**7
        +30*y**8-15*y**9)-42*r**4*(-1+y)**6*(176156+163920*logy*(-1+y)**4
        -540704*y+483216*y**2-14274*y**3-128524*y**4+10427*y**5+11236*y**6
        -512*y**7-60*y**8+15*y**9)-42*r**6*(-1+y)**4*(52270+29400*logy*(-1
        +y)**6-284220*y+622350*y**2-645815*y**3+269570*y**4+62952*y**5-109882*y**6
        +43975*y**7-10488*y**8+968*y**9)-14*r**8*(-1+y)**3*(17800-124600*y
        +373800*y**2-444980*y**3+216740*y**4+101163*y**5-174013*y**6+89026*y**7
        -34350*y**8+4559*y**9+55*y**10)-21*r**10*(-1+y)**2*(3140-25120*y+87920*y**2
        -48870*y**3-202060*y**4+622392*y**5-767416*y**6+557291*y**7-277612*y**8
        +92830*y**9-19180*y**10+1885*y**11)+14*r**14*(-3382+33820*y-152190*y**2
        +433875*y**3-890610*y**4+1293033*y**5-1337952*y**6+987690*y**7-509850*y**8
        +175285*y**9-36124*y**10+3381*y**11)-84*r**12*(2554-25540*y+114930*y**2
        -313865*y**3+573940*y**4-768980*y**5+785514*y**6-608307*y**7+353775*y**8
        -152060*y**9+46050*y**10-8812*y**11+801*y**12))-2*MBhat**2*(141120*(
        -1+y)**11+23667*r**18*(-1+y)**11+42*r**4*(-1+y)**7*(-3498197+3999360*logy*(
        -1+y)**4+17992148*y-34986942*y**2+31323348*y**3-11830197*y**4+750192*y**5
        +186459*y**6+50260*y**7+3778*y**8+2948*y**9+155*y**10)+r**2*(-1+y)**9*(
        -73504897+19911360*logy*(-1+y)**2+166921154*y-103371937*y**2+6637120*y**3
        +1659280*y**4+857248*y**5+438326*y**6+139652*y**7+46991*y**8+11838*y**9
        +585*y**10)+42*r**10*(-1+y)**3*(413715+90000*logy*(-1+y)**8-3219720*y
        +10909020*y**2-20978040*y**3+24962550*y**4-18339960*y**5+7884315*y**6
        -1492408*y**7-106671*y**8+47056*y**9-7537*y**10+760*y**11)+14*r**8*(
        -1+y)**4*(-4281997+3167940*logy*(-1+y)**7+26806039*y-69330327*y**2
        +93374965*y**3-65655490*y**4+17998914*y**5+4016582*y**6-2960846*y**7
        +68823*y**8+55279*y**9-12727*y**10+1425*y**11)+42*r**6*(-1+y)**5*(3400557
        +3622740*logy*(-1+y)**6-16780602*y+31083285*y**2-23330680*y**3-615690*y**4
        +11251440*y**5-5761312*y**6+716500*y**7+30627*y**8+9610*y**9-325*y**10
        -60*y**11+10*y**12)-42*r**12*(-1+y)**2*(5741-8700*logy*(-1+y)**9-42969*y
        +132726*y**2-205294*y**3+125241*y**4-174849*y**5+234265*y**6-430585*y**7
        +385878*y**8-150042*y**9+37597*y**10-5373*y**11+304*y**12)+14*r**14*(5759
        -63349*y+316745*y**2-950235*y**3+1900470*y**4-2811138*y**5+2758704*y**6
        -1901376*y**7+1218195*y**8-662105*y**9+231847*y**10-48005*y**11+4488*y**12)
        +6*r**16*(7249-79739*y+398695*y**2-1196085*y**3+2392170*y**4-3468990*y**5
        +4191726*y**6-3857226*y**7+2564265*y**8-1207195*y**9+382415*y**10-73213*y**11
        +6408*y**12))-8*MBhat*(-10080*(-1+y)**11+2136*r**18*(-1+y)**11-r**2*(
        -1+y)**9*(-7178984+1795920*logy*(-1+y)**2+16153888*y-9872864*y**2
        +598640*y**3+149660*y**4+59864*y**5+49168*y**6+20641*y**7+5686*y**8
        +2193*y**9+348*y**10)-42*r**6*(-1+y)**5*(361740+742800*logy*(-1+y)**6
        -1427640*y+1340700*y**2+1926400*y**3-5158800*y**4+4291920*y**5-1443424*y**6
        +73655*y**7+37164*y**8-2100*y**9+640*y**10-15*y**11)-r**4*(-1+y)**7*(
        -27989072+21658560*logy*(-1+y)**4+133614848*y-243739392*y**2+205810048*y**3
        -73111072*y**4+4331712*y**5+505484*y**6+489859*y**7+52812*y**8+10802*y**9
        +5632*y**10+195*y**11)+7*r**10*(-1+y)**3*(-703360-282000*logy*(-1+y)**8
        +5344880*y-17579080*y**2+32526160*y**3-36709700*y**4+25419760*y**5
        -10325620*y**6+2144189*y**7-242464*y**8+98218*y**9-13640*y**10+337*y**11)
        -14*r**8*(-1+y)**4*(-1068824+768480*logy*(-1+y)**7+6713288*y-17450184*y**2
        +23704280*y**3-16980080*y**4+4808688*y**5+1173484*y**6-1077865*y**7
        +204663*y**8-22462*y**9+1582*y**10-105*y**11+15*y**12)-21*r**12*(-1
        +y)**2*(-2704+24336*y-97344*y**2+227136*y**3-340704*y**4+340704*y**5
        -167660*y**6+18817*y**7+53139*y**8-46586*y**9+10766*y**10-1119*y**11
        +35*y**12)+4*r**16*(1780-19580*y+97900*y**2-293700*y**3+587400*y**4
        -822360*y**5+842352*y**6-722172*y**7+477780*y**8-221815*y**9+69245*y**10
        -13081*y**11+1131*y**12)+28*r**14*(574-6314*y+31570*y**2-94710*y**3
        +189420*y**4-265188*y**5+273354*y**6-184017*y**7+65766*y**8-12925*y**9
        +2234*y**10+545*y**11-366*y**12+57*y**13))-8*MBhat**3*(-70560*(-1+y)**11
        +11640*r**18*(-1+y)**11-r**2*(-1+y)**9*(-25548540+7652400*logy*(-1
        +y)**2+58749480*y-37027140*y**2+2550800*y**3+702800*y**4+326291*y**5
        +114954*y**6+37149*y**7+10836*y**8+1050*y**9)-28*r**8*(-1+y)**4*(-386005
        +198900*logy*(-1+y)**7+2503135*y-6813255*y**2+9963125*y**3-8114000*y**4
        +3381471*y**5-376619*y**6-163756*y**7+30192*y**8-7715*y**9+1067*y**10)
        -r**4*(-1+y)**7*(-15641004+42283920*logy*(-1+y)**4+104847936*y-241839744*y**2
        +245794336*y**3-104388964*y**4+8911014*y**5+1931888*y**6+162835*y**7
        +88980*y**8+5760*y**9-60*y**10+15*y**11)-21*r**6*(-1+y)**5*(1427396
        +1080720*logy*(-1+y)**6-7483656*y+15466980*y**2-15219040*y**3+6091960*y**4
        +686349*y**5-1129286*y**6+149357*y**7+13200*y**8+475*y**9-450*y**10
        +75*y**11)+7*r**10*(-1+y)**3*(-27020+216160*y-756560*y**2+1513120*y**3
        -2503100*y**4+2585869*y**5-1872280*y**6+748904*y**7-98752*y**8+36977*y**9
        -7528*y**10+410*y**11)-21*r**12*(-1+y)**2*(-3740+33660*y-134640*y**2
        +314160*y**3-320540*y**4+107694*y**5+322134*y**6-459249*y**7+284925*y**8
        -117420*y**9+31436*y**10-5037*y**11+377*y**12)+4*r**16*(1021-11231*y
        +56155*y**2-168465*y**3+386910*y**4-842688*y**5+1293180*y**6-1370292*y**7
        +1019865*y**8-525910*y**9+179564*y**10-36610*y**11+3381*y**12)-7*r**14*(
        -10772+118492*y-592460*y**2+1777380*y**3-3642240*y**4+5205183*y**5
        -5602029*y**6+4778697*y**7-3112971*y**8+1485605*y**9-506539*y**10+117047*y**11
        -16461*y**12+1068*y**13))+MBhat**4*(-705600*(-1+y)**11-140175*r**18*(
        -1+y)**11-r**2*(-1+y)**9*(-165586575+56616000*logy*(-1+y)**2+387789150*y
        -250510575*y**2+19268480*y**3+5504870*y**4+1908116*y**5+597198*y**6
        +181776*y**7+24345*y**8+30*y**9-15*y**10)-28*r**4*(-1+y)**7*(1749204
        +6679680*logy*(-1+y)**4-317136*y-12883656*y**2+21821744*y**3-12166961*y**4
        +1559940*y**5+143575*y**6+44720*y**7+3345*y**8-180*y**9+45*y**10)-56*r**8*(
        -1+y)**4*(-564875+148500*logy*(-1+y)**7+3805625*y-10897125*y**2+17341375*y**3
        -15923575*y**4+8287341*y**5-2043901*y**6+88327*y**7-20700*y**8+2468*y**9
        +640*y**10)-42*r**6*(-1+y)**5*(2011216+1371120*logy*(-1+y)**6-10696176*y
        +22627080*y**2-23120080*y**3+10376885*y**4-86526*y**5-1258449*y**6+150100*y**7
        +19875*y**8-8550*y**9+1425*y**10)+28*r**10*(-1+y)**3*(3800-38250*logy*(
        -1+y)**8-68650*y+393275*y**2-1739950*y**3+2693600*y**4-2421334*y**5
        +518011*y**6+601606*y**7-383894*y**8+138136*y**9-26240*y**10+1840*y**11)
        +84*r**12*(-1+y)**2*(11500-103500*y+414000*y**2-1120720*y**3+1952585*y**4
        -2602209*y**5+2319069*y**6-1436517*y**7+693711*y**8-266159*y**9+78175*y**10
        -15279*y**11+1424*y**12)-8*r**16*(-23191+255101*y-1275505*y**2+3726555*y**3
        -6851460*y**4+8435364*y**5-6898584*y**6+3391080*y**7-543180*y**8-459140*y**9
        +347908*y**10-101468*y**11+11640*y**12)+14*r**14*(-99008+1089088*y
        -5445440*y**2+16534560*y**3-33873315*y**4+50543745*y**5-56523621*y**6
        +47399919*y**7-29777253*y**8+13945415*y**9-4783735*y**10+1154693*y**11
        -178572*y**12+13524*y**13)))/(25200*r**2*(-1+y)**11)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr8(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r8 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-2985*r**8*(-1+y)**6+60*logy*(4+9*r**4)*(-1+y)**6+24*r**2*(-1+y)**2*(
        -155+620*y-930*y**2+670*y**3-175*y**4-42*y**5+24*y**6)+(-1+y)**4*(985
        -1730*y+625*y**2+800*y**3-955*y**4+278*y**5+45*y**6)+6*r**4*(120
        -630*y+1305*y**2-1370*y**3-150*y**4+1572*y**5-1087*y**6+240*y**7)
        -4*r**6*(-1250+7500*y-18750*y**2+27280*y**3-26715*y**4+16014*y**5
        -5315*y**6+756*y**7))/(360*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (8009-17615*r**2+10080*r**4+4240*r**6-6505*r**8+1791*r**10+60*logy*(64
        -28*r**2-33*r**4+27*r**6+24*MBhat*(-3+r**2)+2*MBhat**2*(4+9*r**4))*(
        -1+y)**7-59903*y+124985*r**2*y-68580*r**4*y-31300*r**6*y+45535*r**8*y
        -12537*r**10*y+193149*y**2-380835*r**2*y**2+198810*r**4*y**2+99570*r**6*y**2
        -136605*r**8*y**2+37611*r**10*y**2-348795*y**3+646485*r**2*y**3-317490*r**4*y**3
        -177290*r**6*y**3+227675*r**8*y**3-62685*r**10*y**3+382260*y**4-662130*r**2*y**4
        +307830*r**4*y**4+195350*r**6*y**4-239105*r**8*y**4+62685*r**10*y**4-255804*y**5
        +410334*r**2*y**5-184134*r**4*y**5-164550*r**6*y**5+182199*r**8*y**5-37611*r**10*y**5
        +98556*y**6-140322*r**2*y**6+60078*r**4*y**6+111814*r**6*y**6-89257*r**8*y**6
        +12537*r**10*y**6-19428*y**7+14862*r**2*y**7-4422*r**4*y**7-47710*r**6*y**7
        +24847*r**8*y**7-1791*r**10*y**7+4155*y**8+6321*r**2*y**8-2814*r**4*y**8
        +10818*r**6*y**8-3024*r**8*y**8-3965*y**9-2215*r**2*y**9+642*r**4*y**9-942*r**6*y**9
        +2495*y**10+85*r**2*y**10-873*y**11+45*r**2*y**11+144*y**12+2*MBhat**2*(
        -1+y)*(-2985*r**8*(-1+y)**6+24*r**2*(-1+y)**2*(-155+620*y-930*y**2
        +670*y**3-175*y**4-42*y**5+24*y**6)+(-1+y)**4*(985-1730*y+625*y**2
        +800*y**3-955*y**4+278*y**5+45*y**6)+6*r**4*(120-630*y+1305*y**2
        -1370*y**3-150*y**4+1572*y**5-1087*y**6+240*y**7)-4*r**6*(-1250+7500*y
        -18750*y**2+27280*y**3-26715*y**4+16014*y**5-5315*y**6+756*y**7))-6*MBhat*(1064*r**10*(
        -1+y)**7+(-1+y)**5*(-744+2208*y-1824*y**2+430*y**3-140*y**4+65*y**5
        +34*y**6+3*y**7)-4*r**2*(-1+y)**3*(60-180*y+150*y**2+125*y**3-720*y**4
        +802*y**5-322*y**6+37*y**7)-2*r**4*(-1+y)**2*(1100-5500*y+11000*y**2
        -12320*y**3+7500*y**4-3811*y**5+2063*y**6-861*y**7+157*y**8)+r**8*(2160
        -15120*y+45360*y**2-79410*y**3+92710*y**4-69601*y**5+32487*y**6-8621*y**7
        +995*y**8)-4*r**6*(-30+210*y-630*y**2+975*y**3+185*y**4-758*y**5
        -700*y**6+1295*y**7-673*y**8+126*y**9)))/(720*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-73750+47340*r**2+69350*r**4-36400*r**6-16650*r**8+12100*r**10-1990*r**12
        +300*logy*(24*MBhat**3*(-3+r**2)+MBhat**4*(4+9*r**4)+8*MBhat*(-27-49*r**2
        +18*r**4)+MBhat**2*(208+68*r**2-15*r**4+57*r**6)+2*(38+150*r**2-79*r**4
        -20*r**6+9*r**8))*(-1+y)**8+612800*y-288720*r**2*y-602200*r**4*y+279200*r**6*y
        +138600*r**8*y-96800*r**10*y+15920*r**12*y-2236000*y**2+650520*r**2*y**2
        +2297300*r**4*y**2-929200*r**6*y**2-506700*r**8*y**2+338800*r**10*y**2-55720*r**12*y**2
        +4684800*y**3-461040*r**2*y**3-5037000*r**4*y**3+1746400*r**6*y**3+1063800*r**8*y**3
        -677600*r**10*y**3+111440*r**12*y**3-6175200*y**4-683700*r**2*y**4+6959850*r**4*y**4
        -2015000*r**6*y**4-1405350*r**8*y**4+847000*r**10*y**4-139300*r**12*y**4
        +5257488*y**5+1808280*r**2*y**5-6255960*r**4*y**5+1479280*r**6*y**5+1206840*r**8*y**5
        -694808*r**10*y**5+111440*r**12*y**5-2828255*y**6-1746960*r**2*y**6+3640890*r**4*y**6
        -657060*r**6*y**6-788235*r**8*y**6+431860*r**10*y**6-55720*r**12*y**6+856792*y**7
        +858000*r**2*y**7-1306200*r**4*y**7+94560*r**6*y**7+437520*r**8*y**7-178592*r**10*y**7
        +15920*r**12*y**7-50306*y**8-189180*r**2*y**8+246000*r**4*y**8+64080*r**6*y**8
        -158550*r**8*y**8+42736*r**10*y**8-1990*r**12*y**8-93304*y**9+2640*r**2*y**9
        -9200*r**4*y**9-30000*r**6*y**9+31080*r**8*y**9-4536*r**10*y**9+67502*y**10
        +120*r**2*y**10-2990*r**4*y**10+4140*r**6*y**10-2355*r**8*y**10-29080*y**11
        +4320*r**2*y**11+160*r**4*y**11+7648*y**12-1980*r**2*y**12-1304*y**13+360*r**2*y**13
        +169*y**14+5*MBhat**4*(-1+y)**2*(-2985*r**8*(-1+y)**6+24*r**2*(-1+y)**2*(
        -155+620*y-930*y**2+670*y**3-175*y**4-42*y**5+24*y**6)+(-1+y)**4*(985
        -1730*y+625*y**2+800*y**3-955*y**4+278*y**5+45*y**6)+6*r**4*(120
        -630*y+1305*y**2-1370*y**3-150*y**4+1572*y**5-1087*y**6+240*y**7)
        -4*r**6*(-1250+7500*y-18750*y**2+27280*y**3-26715*y**4+16014*y**5
        -5315*y**6+756*y**7))-30*MBhat**3*(-1+y)*(1064*r**10*(-1+y)**7+(-1
        +y)**5*(-744+2208*y-1824*y**2+430*y**3-140*y**4+65*y**5+34*y**6+3*y**7)
        -4*r**2*(-1+y)**3*(60-180*y+150*y**2+125*y**3-720*y**4+802*y**5-322*y**6
        +37*y**7)-2*r**4*(-1+y)**2*(1100-5500*y+11000*y**2-12320*y**3+7500*y**4
        -3811*y**5+2063*y**6-861*y**7+157*y**8)+r**8*(2160-15120*y+45360*y**2
        -79410*y**3+92710*y**4-69601*y**5+32487*y**6-8621*y**7+995*y**8)-4*r**6*(
        -30+210*y-630*y**2+975*y**3+185*y**4-758*y**5-700*y**6+1295*y**7
        -673*y**8+126*y**9))-4*MBhat*(1900*r**12*(-1+y)**8+10*r**2*(-1+y)**4*(5995
        -21040*y+25680*y**2-11240*y**3-175*y**4+1164*y**5-266*y**6-4*y**7
        +30*y**8)+2*(-1+y)**6*(-22435+52970*y-34585*y**2+2700*y**3-30*y**4
        +1479*y**5-48*y**6+39*y**7+30*y**8)+15*r**4*(-1+y)**3*(1580-7180*y
        +12560*y**2-10160*y**3+4510*y**4-489*y**5+181*y**6-191*y**7+53*y**8)
        -5*r**6*(-1+y)**2*(-1880+11280*y-28200*y**2+37600*y**3-34350*y**4
        +13743*y**5-8118*y**6+4332*y**7-1284*y**8+157*y**9)-25*r**8*(-158+1264*y
        -4424*y**2+8848*y**3-11216*y**4+12391*y**5-10640*y**6+5158*y**7-1394*y**8
        +171*y**9)+3*r**10*(-2210+17680*y-61880*y**2+123760*y**3-159480*y**4
        +150849*y**5-96432*y**6+39492*y**7-9414*y**8+995*y**9))-5*MBhat**2*(3048*r**12*(
        -1+y)**8-(-1+y)**6*(-26471+65422*y-45191*y**2+3392*y**3+2663*y**4
        +338*y**5-54*y**6+186*y**7+3*y**8)+6*r**4*(-1+y)**3*(-1340+6550*y
        -12725*y**2+10545*y**3-7120*y**4+2646*y**5-1043*y**6+143*y**7+40*y**8)
        -r**2*(-1+y)**4*(38693-150692*y+217878*y**2-141172*y**3+47458*y**4
        -14820*y**5+5126*y**6-788*y**7+45*y**8)+3*r**10*(-1897+15176*y-53116*y**2
        +110056*y**3-155948*y**4+147784*y**5-93044*y**6+37528*y**7-8795*y**8
        +912*y**9)-2*r**6*(-1+y)**2*(-6800+42510*y-111405*y**2+171010*y**3
        -156015*y**4+125586*y**5-73103*y**6+30198*y**7-8109*y**8+1008*y**9)
        +r**8*(-6775+54200*y-189700*y**2+380840*y**3-458950*y**4+406952*y**5
        -314989*y**6+192800*y**7-84841*y**8+23448*y**9-2985*y**10)))/(1800*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (2223815+5787495*r**2-10613925*r**4+1806875*r**6+632625*r**8+283185*r**10
        -134995*r**12+14925*r**14+2100*logy*(272+2796*r**2+1485*r**4-1235*r**6
        -135*r**8+45*r**10+72*MBhat**5*(-3+r**2)+2*MBhat**6*(4+9*r**4)-16*MBhat**3*(139
        +276*r**2-63*r**4+39*r**6)+24*MBhat*(-53-367*r**2-36*r**4+40*r**6)
        +3*MBhat**4*(352+164*r**2+3*r**4+87*r**6)+6*MBhat**2*(396+1644*r**2
        -276*r**4+89*r**6+57*r**8))*(-1+y)**9-20585535*y-57959055*r**2*y+92406825*r**4*y
        -13668375*r**6*y-5410125*r**8*y-2643165*r**10*y+1214955*r**12*y-134325*r**14*y
        +84912540*y**2+258258420*r**2*y**2-355594050*r**4*y**2+43002750*r**6*y**2
        +20364750*r**8*y**2+10997910*r**10*y**2-4859820*r**12*y**2+537300*r**14*y**2
        -204983660*y**3-673062180*r**2*y**3+792297450*r**4*y**3-69217750*r**6*y**3
        -44115750*r**8*y**3-26795790*r**10*y**3+11339580*r**12*y**3-1253700*r**14*y**3
        +319470690*y**4+1132896870*r**2*y**4-1122957675*r**4*y**4+49363125*r**6*y**4
        +60220125*r**8*y**4+42178185*r**10*y**4-17009370*r**12*y**4+1880550*r**14*y**4
        -333864930*y**5-1280861190*r**2*y**5+1044371475*r**4*y**5+15993075*r**6*y**5
        -53075925*r**8*y**5-44559585*r**10*y**5+17009370*r**12*y**5-1880550*r**14*y**5
        +234632573*y**6+977154591*r**2*y**6-630414960*r**4*y**6-66221960*r**6*y**6
        +30408735*r**8*y**6+31564617*r**10*y**6-11443656*r**12*y**6+1253700*r**14*y**6
        -107893749*y**7-488884599*r**2*y**7+231479640*r**4*y**7+62504400*r**6*y**7
        -9978255*r**8*y**7-17005401*r**10*y**7+5998104*r**12*y**7-537300*r**14*y**7
        +31284291*y**8+147601251*r**2*y**8-41994855*r**4*y**8-30277695*r**6*y**8
        -1040130*r**8*y**8+8173494*r**10*y**8-2145591*r**12*y**8+134325*r**14*y**8
        -7773659*y**9-20408619*r**2*y**9+358995*r**4*y**9+7201355*r**6*y**9+2802870*r**8*y**9
        -2616306*r**10*y**9+450079*r**12*y**9-14925*r**14*y**9+5265246*y**10-1629054*r**2*y**10
        +490770*r**4*y**10-450870*r**6*y**10-915705*r**8*y**10+452529*r**10*y**10
        -42336*r**12*y**10-4446750*y**11+1649214*r**2*y**11+240030*r**4*y**11-38010*r**6*y**11
        +106785*r**8*y**11-29673*r**10*y**11+2416484*y**12-709296*r**2*y**12-80745*r**4*y**12
        +4655*r**6*y**12-773556*y**13+189504*r**2*y**13+11025*r**4*y**13-1575*r**6*y**13
        +119769*y**14-26901*r**2*y**14-3281*y**15+3549*r**2*y**15-288*y**16+70*MBhat**6*(
        -1+y)**3*(-2985*r**8*(-1+y)**6+24*r**2*(-1+y)**2*(-155+620*y-930*y**2
        +670*y**3-175*y**4-42*y**5+24*y**6)+(-1+y)**4*(985-1730*y+625*y**2
        +800*y**3-955*y**4+278*y**5+45*y**6)+6*r**4*(120-630*y+1305*y**2
        -1370*y**3-150*y**4+1572*y**5-1087*y**6+240*y**7)-4*r**6*(-1250+7500*y
        -18750*y**2+27280*y**3-26715*y**4+16014*y**5-5315*y**6+756*y**7))-630*MBhat**5*(
        -1+y)**2*(1064*r**10*(-1+y)**7+(-1+y)**5*(-744+2208*y-1824*y**2+430*y**3
        -140*y**4+65*y**5+34*y**6+3*y**7)-4*r**2*(-1+y)**3*(60-180*y+150*y**2
        +125*y**3-720*y**4+802*y**5-322*y**6+37*y**7)-2*r**4*(-1+y)**2*(1100
        -5500*y+11000*y**2-12320*y**3+7500*y**4-3811*y**5+2063*y**6-861*y**7
        +157*y**8)+r**8*(2160-15120*y+45360*y**2-79410*y**3+92710*y**4-69601*y**5
        +32487*y**6-8621*y**7+995*y**8)-4*r**6*(-30+210*y-630*y**2+975*y**3
        +185*y**4-758*y**5-700*y**6+1295*y**7-673*y**8+126*y**9))-105*MBhat**4*(
        -1+y)*(6096*r**12*(-1+y)**8-(-1+y)**6*(-44933+110986*y-76613*y**2
        +5504*y**3+4871*y**4+386*y**5+45*y**6+228*y**7+6*y**8)-r**2*(-1+y)**4*(59771
        -229244*y+324186*y**2-204604*y**3+72856*y**4-30276*y**5+11942*y**6
        -1796*y**7+45*y**8)+6*r**4*(-1+y)**3*(-1000+5030*y-10135*y**2+6875*y**3
        -6680*y**4+3938*y**5-2341*y**6+393*y**7+80*y**8)+3*r**10*(-4391+35128*y
        -122948*y**2+253544*y**3-353686*y**4+329000*y**5-202804*y**6+79832*y**7
        -18187*y**8+1824*y**9)-2*r**6*(-1+y)**2*(-11480+71490*y-186555*y**2
        +289630*y**3-266745*y**4+214182*y**5-127289*y**6+55458*y**7-15747*y**8
        +2016*y**9)+r**8*(-7045+56360*y-197260*y**2+397400*y**3-451120*y**4
        +392600*y**5-358522*y**6+271496*y**7-141811*y**8+43872*y**9-5970*y**10))
        -6*MBhat*(9880*r**14*(-1+y)**9+140*r**6*(-1+y)**3*(-10150+58500*y-139050*y**2
        +173400*y**3-118050*y**4+45813*y**5-6604*y**6+2532*y**7-1014*y**8+143*y**9)
        +(-1+y)**7*(-1598740+3642680*y-2266540*y**2+148400*y**3+37100*y**4
        -30331*y**5+63826*y**6+7528*y**7-1070*y**8+507*y**9)-105*r**4*(-1+y)**4*(40344
        -198840*y+390480*y**2-380880*y**3+183240*y**4-36001*y**5-795*y**6
        +336*y**7+16*y**8-15*y**9+3*y**10)+14*r**2*(-1+y)**5*(-99090+616560*y
        -1365240*y**2+1350560*y**3-557840*y**4+46407*y**5+3562*y**6+6962*y**7
        -678*y**8+207*y**9+30*y**10)-35*r**8*(-1+y)**2*(4300-30100*y+90300*y**2
        -150500*y**3+150500*y**4-111651*y**5+19997*y**6-10742*y**7+6174*y**8
        -1555*y**9+157*y**10)+21*r**12*(2440-21960*y+87840*y**2-204960*y**3
        +307440*y**4-311157*y**5+245133*y**6-135132*y**7+48708*y**8-10387*y**9
        +995*y**10)+42*r**10*(-1470+13230*y-52920*y**2+123480*y**3-185220*y**4
        +182727*y**5-156901*y**6+117720*y**7-53040*y**8+14485*y**9-2217*y**10
        +126*y**11))-3*MBhat**2*(41910*r**14*(-1+y)**9+420*r**4*(-1+y)**4*(9603
        -50775*y+108450*y**2-117650*y**3+64240*y**4-17684*y**5+1693*y**6-353*y**7
        +76*y**8)+70*r**6*(-1+y)**3*(12635-81150*y+218895*y**2-318560*y**3
        +228075*y**4-98646*y**5-19483*y**6+18276*y**7-4818*y**8+616*y**9)-(
        -1+y)**7*(-5367970+12399140*y-7862770*y**2+554400*y**3-9345*y**4+207354*y**5
        +97874*y**6-6910*y**7+4307*y**8+720*y**9)+105*r**2*(-1+y)**5*(-20692
        +17008*y+106008*y**2-202192*y**3+115841*y**4-12868*y**5-4273*y**6
        +328*y**7-69*y**8-52*y**9+y**10)+210*r**12*(574-5166*y+20664*y**2
        -48216*y**3+73563*y**4-85491*y**5+69804*y**6-39036*y**7+14319*y**8
        -3111*y**9+304*y**10)-105*r**8*(-1+y)**2*(1590-13410*y+48210*y**2
        -96310*y**3+140845*y**4-100235*y**5+95068*y**6-52492*y**7+17391*y**8
        -3553*y**9+336*y**10)-21*r**10*(-6420+57780*y-231120*y**2+539280*y**3
        -828715*y**4+668179*y**5-278829*y**6+115701*y**7-52089*y**8+22161*y**9
        -6923*y**10+995*y**11))-2*MBhat**3*(86040*r**14*(-1+y)**9+420*r**2*(-1
        +y)**5*(18124-61456*y+70104*y**2-24978*y**3-3783*y**4+2400*y**5+94*y**6
        -70*y**7+45*y**8)+210*r**4*(-1+y)**4*(10764-48780*y+84960*y**2-67445*y**3
        +37165*y**4-15044*y**5+10416*y**6-3987*y**7+591*y**8)+(-1+y)**7*(
        -6458280+15251760*y-9961080*y**2+638330*y**3+253400*y**4+303674*y**5
        -8988*y**6+8949*y**7+5790*y**8+45*y**9)+280*r**6*(-1+y)**3*(1555-6990*y
        +10455*y**2+6340*y**3-13185*y**4+24570*y**5-4783*y**6-1110*y**7+102*y**8
        +86*y**9)-105*r**8*(-1+y)**2*(12200-85400*y+256200*y**2-454170*y**3
        +489210*y**4-452886*y**5+280850*y**6-127055*y**7+43029*y**8-9533*y**9
        +995*y**10)+70*r**12*(1084-9756*y+39024*y**2-94773*y**3+175077*y**4
        -223146*y**5+196770*y**6-118620*y**7+46800*y**8-10915*y**9+1143*y**10)
        -84*r**10*(-7380+66420*y-265680*y**2+627250*y**3-940965*y**4+1024119*y**5
        -859476*y**6+536904*y**7-246501*y**8+80489*y**9-16890*y**10+1710*y**11)))/(25200*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-3888565-28662800*r**2+14974260*r**4+32280080*r**6-13874000*r**8-616560*r**10
        -315700*r**12+112240*r**14-8955*r**16+4200*logy*(212+3928*r**2+7851*r**4
        -1112*r**6-1675*r**8-96*r**10+27*r**12+48*MBhat**7*(-3+r**2)+MBhat**8*(4
        +9*r**4)-16*MBhat**5*(197+405*r**2-72*r**4+78*r**6)+MBhat**6*(992+520*r**2
        +42*r**4+234*r**6)+48*MBhat*(-29-409*r**2-503*r**4+161*r**6+20*r**8)
        -16*MBhat**3*(379+2627*r**2+411*r**4-147*r**6+120*r**8)+3*MBhat**4*(1880
        +7896*r**2-924*r**4+860*r**6+225*r**8)+2*MBhat**2*(1952+19980*r**2
        +12219*r**4-5267*r**6+930*r**8+189*r**10))*(-1+y)**10+39776050*y+303125600*r**2*y
        -116768400*r**4*y-327471200*r**6*y+131705000*r**8*y+5762400*r**10*y+3270400*r**12*y
        -1122400*r**14*y+89550*r**16*y-183444225*y**2-1446553200*r**2*y**2+360586800*r**4*y**2
        +1496972400*r**6*y**2-557497500*r**8*y**2-23914800*r**10*y**2-15283800*r**12*y**2
        +5050800*r**14*y**2-402975*r**16*y**2+502540600*y**3+4104939200*r**2*y**3
        -466951800*r**4*y**3-4061982400*r**6*y**3+1381135000*r**8*y**3+57724800*r**10*y**3
        +42457800*r**12*y**3-13468800*r**14*y**3+1074600*r**16*y**3-906158050*y**4
        -7678571600*r**2*y**4-172060350*r**4*y**4+7248581200*r**6*y**4-2205936250*r**8*y**4
        -88922400*r**10*y**4-77703150*r**12*y**4+23570400*r**14*y**4-1880550*r**16*y**4
        +1124786460*y**5+9907185120*r**2*y**5+1591388820*r**4*y**5-8894454240*r**6*y**5
        +2351653500*r**8*y**5+89772480*r**10*y**5+98006580*r**12*y**5-28284480*r**14*y**5
        +2256660*r**16*y**5-974718850*y**6-8948886800*r**2*y**6-2711073750*r**4*y**6
        +7608202000*r**6*y**6-1664241250*r**8*y**6-57876000*r**10*y**6-86434950*r**12*y**6
        +23570400*r**14*y**6-1880550*r**16*y**6+583595224*y**7+5608736864*r**2*y**7
        +2538049080*r**4*y**7-4486061440*r**6*y**7+737196040*r**8*y**7+22964256*r**10*y**7
        +52232376*r**12*y**7-13440000*r**14*y**7+1074600*r**16*y**7-231318240*y**8
        -2352003600*r**2*y**8-1444646280*r**4*y**8+1746263400*r**6*y**8-163932720*r**8*y**8
        -3242400*r**10*y**8-23938110*r**12*y**8+6085800*r**14*y**8-402975*r**16*y**8
        +52225480*y**9+609733280*r**2*y**9+482386800*r**4*y**9-402194800*r**6*y**9
        -4558400*r**8*y**9-5906880*r**10*y**9+10230220*r**12*y**9-1918000*r**14*y**9
        +89550*r**16*y**9+2830280*y**10-87380240*r**2*y**10-77340060*r**4*y**10
        +37971640*r**6*y**10+8857100*r**8*y**10+5639760*r**10*y**10-2953930*r**12*y**10
        +358120*r**14*y**10-8955*r**16*y**10-14558040*y**11+15583680*r**2*y**11
        +520800*r**4*y**11+2016000*r**6*y**11-428400*r**8*y**11-1545600*r**10*y**11
        +458640*r**12*y**11-30240*r**14*y**11+15407882*y**12-12314288*r**2*y**12
        +927570*r**4*y**12+82040*r**6*y**12-85190*r**8*y**12+160944*r**10*y**12-26376*r**12*y**12
        -10879820*y**13+7061600*r**2*y**13+41580*r**4*y**13-282800*r**6*y**13+10220*r**8*y**13
        +4988970*y**14-2375280*r**2*y**14-61950*r**4*y**14+88200*r**6*y**14-3150*r**8*y**14
        -1345400*y**15+396320*r**2*y**15+26880*r**4*y**15-10080*r**6*y**15+153965*y**16
        -13280*r**2*y**16+8694*y**17-576*r**2*y**17-2415*y**18+70*MBhat**8*(-1
        +y)**4*(-2985*r**8*(-1+y)**6+24*r**2*(-1+y)**2*(-155+620*y-930*y**2
        +670*y**3-175*y**4-42*y**5+24*y**6)+(-1+y)**4*(985-1730*y+625*y**2
        +800*y**3-955*y**4+278*y**5+45*y**6)+6*r**4*(120-630*y+1305*y**2
        -1370*y**3-150*y**4+1572*y**5-1087*y**6+240*y**7)-4*r**6*(-1250+7500*y
        -18750*y**2+27280*y**3-26715*y**4+16014*y**5-5315*y**6+756*y**7))-840*MBhat**7*(
        -1+y)**3*(1064*r**10*(-1+y)**7+(-1+y)**5*(-744+2208*y-1824*y**2+430*y**3
        -140*y**4+65*y**5+34*y**6+3*y**7)-4*r**2*(-1+y)**3*(60-180*y+150*y**2
        +125*y**3-720*y**4+802*y**5-322*y**6+37*y**7)-2*r**4*(-1+y)**2*(1100
        -5500*y+11000*y**2-12320*y**3+7500*y**4-3811*y**5+2063*y**6-861*y**7
        +157*y**8)+r**8*(2160-15120*y+45360*y**2-79410*y**3+92710*y**4-69601*y**5
        +32487*y**6-8621*y**7+995*y**8)-4*r**6*(-30+210*y-630*y**2+975*y**3
        +185*y**4-758*y**5-700*y**6+1295*y**7-673*y**8+126*y**9))+140*MBhat**6*(
        -1+y)**2*(-9144*r**12*(-1+y)**8+(-1+y)**6*(-63395+156550*y-108035*y**2
        +7616*y**3+7079*y**4+434*y**5+144*y**6+270*y**7+9*y**8)+r**2*(-1+y)**4*(80849
        -307796*y+430494*y**2-268036*y**3+98254*y**4-45732*y**5+18758*y**6
        -2804*y**7+45*y**8)-6*r**4*(-1+y)**3*(-660+3510*y-7545*y**2+3205*y**3
        -6240*y**4+5230*y**5-3639*y**6+643*y**7+120*y**8)-9*r**10*(-2295+18360*y
        -64260*y**2+132344*y**3-183808*y**4+170072*y**5-104188*y**6+40712*y**7
        -9193*y**8+912*y**9)+2*r**6*(-1+y)**2*(-16160+100470*y-261705*y**2
        +408250*y**3-377475*y**4+302778*y**5-181475*y**6+80718*y**7-23385*y**8
        +3024*y**9)+r**8*(7315-58520*y+204820*y**2-413960*y**3+443290*y**4
        -378248*y**5+402055*y**6-350192*y**7+198781*y**8-64296*y**9+8955*y**10))
        -8*MBhat**5*(-1+y)*(86040*r**14*(-1+y)**9+420*r**2*(-1+y)**5*(12129
        -40416*y+44424*y**2-13738*y**3-3608*y**4+1236*y**5+360*y**6-66*y**7
        +15*y**8)+210*r**4*(-1+y)**4*(6024-27240*y+47280*y**2-36965*y**3+23635*y**4
        -13577*y**5+9873*y**6-3414*y**7+432*y**8)+(-1+y)**7*(-4573740+10802280*y
        -7055940*y**2+411530*y**3+255920*y**4+179438*y**5-4956*y**6+5673*y**7
        +3270*y**8+45*y**9)+70*r**6*(-1+y)**3*(580+5880*y-42780*y**2+138160*y**3
        -155790*y**4+139509*y**5-43486*y**6+8556*y**7-3444*y**8+815*y**9)-105*r**8*(
        -1+y)**2*(10620-74340*y+223020*y**2-398870*y**3+432350*y**4-385836*y**5
        +241500*y**6-114825*y**7+41319*y**8-9533*y**9+995*y**10)+70*r**12*(2224
        -20016*y+80064*y**2-190533*y**3+318717*y**4-366786*y**5+292530*y**6
        -159660*y**7+57060*y**8-12055*y**9+1143*y**10)-42*r**10*(-8130+73170*y
        -292680*y**2+697580*y**3-1032210*y**4+1117251*y**5-977109*y**6+666036*y**7
        -346284*y**8+129751*y**9-30795*y**10+3420*y**11))-2*MBhat**2*(40005*r**16*(
        -1+y)**10+70*r**6*(-1+y)**4*(344041-1748226*y+3422505*y**2-2983240*y**3
        +657330*y**4+549768*y**5-440702*y**6+11184*y**7-6243*y**8+2498*y**9
        +45*y**10)-42*r**2*(-1+y)**6*(-1987675+9948700*y-18919050*y**2+16608700*y**3
        -6150175*y**4+420768*y**5+21505*y**6+51740*y**7+12210*y**8-420*y**9
        +417*y**10)+210*r**8*(-1+y)**3*(425+15625*y-111975*y**2+316825*y**3
        -479575*y**4+336153*y**5-176438*y**6-37726*y**7+27195*y**8-5805*y**9
        +576*y**10)-(-1+y)**8*(-31832365+71863130*y-44129965*y**2+2732800*y**3
        +683200*y**4-351680*y**5+573314*y**6+518780*y**7-7501*y**8-4194*y**9
        +1521*y**10)+210*r**4*(-1+y)**5*(659639-3053815*y+5496680*y**2-4682080*y**3
        +1730090*y**4-110326*y**5-49722*y**6-6006*y**7-153*y**8-555*y**9+114*y**10
        +6*y**11)+90*r**14*(-1759+17590*y-79155*y**2+211080*y**3-369390*y**4
        +441252*y**5-431550*y**6+307080*y**7-152775*y**8+50670*y**9-10083*y**10
        +912*y**11)-42*r**10*(-1+y)**2*(10595-65860*y+154910*y**2-133420*y**3
        -97825*y**4+645500*y**5-293217*y**6+507612*y**7-280302*y**8+78024*y**9
        -12945*y**10+1008*y**11)-14*r**12*(15775-157750*y+709875*y**2-1893000*y**3
        +3312750*y**4-4234644*y**5+2730864*y**6-415020*y**7-174465*y**8+149030*y**9
        -49751*y**10+6336*y**11))-24*MBhat*(1520*r**16*(-1+y)**10-(-1+y)**8*(1008620
        -2260840*y+1374020*y**2-81200*y**3-20300*y**4-8120*y**5+19054*y**6
        -27199*y**7-6240*y**8+1061*y**9+24*y**10)+35*r**8*(-1+y)**3*(34700
        -238100*y+697500*y**2-1128900*y**3+1086900*y**4-618540*y**5+207650*y**6
        -24371*y**7+7245*y**8-2237*y**9+233*y**10)-70*r**6*(-1+y)**4*(-24514
        +166404*y-473970*y**2+728560*y**3-643020*y**4+315168*y**5-77050*y**6
        +3345*y**7-388*y**8+22*y**10+3*y**11)-70*r**4*(-1+y)**5*(74898-314130*y
        +477360*y**2-276160*y**3-12820*y**4+62924*y**5-11176*y**6-1673*y**7
        -411*y**8-35*y**9-31*y**10+6*y**11)-7*r**10*(-1+y)**2*(-10660+85280*y
        -298480*y**2+596960*y**3-746200*y**4+596960*y**5-365122*y**6+31309*y**7
        -10152*y**8+7382*y**9-1674*y**10+157*y**11)+r**2*(-1+y)**6*(-4821020
        +22719680*y-40950720*y**2+34171680*y**3-11978520*y**4+687120*y**5+150094*y**6
        -46927*y**7+71508*y**8+3758*y**9-102*y**10+169*y**11)+20*r**14*(-543
        +5430*y-24435*y**2+65160*y**3-114030*y**4+136836*y**5-113694*y**6
        +76500*y**7-36975*y**8+11865*y**9-2281*y**10+199*y**11)+28*r**12*(715
        -7150*y+32175*y**2-85800*y**3+150150*y**4-180180*y**5+145427*y**6
        -102133*y**7+69020*y**8-28450*y**9+7230*y**10-1067*y**11+63*y**12))
        -3*MBhat**4*(30975*r**16*(-1+y)**10-70*r**4*(-1+y)**5*(-279408+1452480*y
        -3043560*y**2+3225960*y**3-1676345*y**4+387341*y**5+3706*y**6-2346*y**7
        -673*y**8+45*y**9)+280*r**6*(-1+y)**4*(19405-129330*y+362025*y**2
        -555960*y**3+438105*y**4-196426*y**5-6115*y**6+21560*y**7-5896*y**8
        +632*y**9)+140*r**2*(-1+y)**6*(-63184+15856*y+449976*y**2-774584*y**3
        +430721*y**4-47748*y**5-14975*y**6+1000*y**7-365*y**8-60*y**9+3*y**10)
        -(-1+y)**8*(-25438625+58773250*y-37282625*y**2+2397920*y**3+281890*y**4
        +1018332*y**5+330834*y**6-19424*y**7+13663*y**8+3170*y**9+15*y**10)
        +140*r**8*(-1+y)**3*(19600-130450*y+367725*y**2-609185*y**3+517265*y**4
        -360597*y**5+32376*y**6+39926*y**7-15518*y**8+2034*y**9+24*y**10)+40*r**14*(2689
        -26890*y+121005*y**2-324360*y**3+530040*y**4-572880*y**5+403788*y**6
        -167640*y**7+23430*y**8+12320*y**9-6490*y**10+956*y**11)-28*r**10*(
        -1+y)**2*(-33000+264000*y-924000*y**2+2028360*y**3-2666465*y**4+3290216*y**5
        -2475949*y**6+1329056*y**7-540079*y**8+161616*y**9-31915*y**10+3040*y**11)
        -70*r**12*(11232-112320*y+505440*y**2-1369600*y**3+2413955*y**4-3079774*y**5
        +2948103*y**6-2078568*y**7+1075605*y**8-404170*y**9+106725*y**10-18152*y**11
        +1524*y**12))-8*MBhat**3*(14340*r**16*(-1+y)**10+140*r**6*(-1+y)**4*(
        -69581+408666*y-995205*y**2+1282840*y**3-900645*y**4+352782*y**5-47635*y**6
        +10704*y**7-3162*y**8+356*y**9)-105*r**4*(-1+y)**5*(277084-1352540*y
        +2622880*y**2-2513280*y**3+1171120*y**4-214995*y**5-4241*y**6-624*y**7
        -332*y**8+135*y**9+9*y**10)+(-1+y)**8*(-11406720+25997040*y-16182120*y**2
        +1061200*y**3+91210*y**4+112693*y**5+314580*y**6+37455*y**7-4212*y**8
        +2214*y**9+180*y**10)+35*r**8*(-1+y)**3*(-58100+377900*y-1032900*y**2
        +1519900*y**3-1132990*y**4+485745*y**5+171581*y**6-107987*y**7+26391*y**8
        -5470*y**9+730*y**10)-r**2*(-1+y)**6*(10361460-63512640*y+139402560*y**2
        -137068640*y**3+56126690*y**4-4018224*y**5-1012984*y**6-427739*y**7
        +22626*y**8-13674*y**9-600*y**10+45*y**11)-21*r**10*(-1+y)**2*(-26820
        +214560*y-750960*y**2+1501920*y**3-2048590*y**4+1620768*y**5-1310628*y**6
        +691815*y**7-256242*y**8+69566*y**9-12144*y**10+995*y**11)+20*r**14*(
        -941+9410*y-42345*y**2+112920*y**3-195090*y**4+304542*y**5-342006*y**6
        +270870*y**7-148725*y**8+54005*y**9-11687*y**10+1143*y**11)-7*r**12*(25220
        -252200*y+1134900*y**2-3026400*y**3+5445060*y**4-6286113*y**5+5160030*y**6
        -3409485*y**7+1679760*y**8-600155*y**9+153158*y**10-26055*y**11+2280*y**12)))/(25200*(
        -1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr9(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r9 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (40*(-1+y)**6+110*r**10*(-1+y)**6-r**2*(-1+y)**4*(-210+180*logy*(
        -1+y)**2+600*y-480*y**2+360*y**3-485*y**4+198*y**5+5*y**6)-4*r**4*(
        -1+y)**2*(110-440*y+660*y**2-650*y**3+725*y**4-580*y**5+172*y**6)
        -r**6*(-1+y)*(560+180*logy*(-1+y)**5-2980*y+6410*y**2-7850*y**3
        +6415*y**4-3281*y**5+714*y**6)+2*r**8*(-240+1440*y-3600*y**2+4990*y**3
        -4165*y**4+2031*y**5-517*y**6+51*y**7))/(30*r**2*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (240*(-1+y)**7+198*r**12*(-1+y)**7+3*r**6*(-1+y)**2*(2060+120*logy*(
        -1+y)**5-10420*y+21140*y**2-21540*y**3+12795*y**4-7627*y**5+5021*y**6
        -1357*y**7)-6*r**2*(-1+y)**5*(-1152+480*logy*(-1+y)**2+2784*y-1872*y**2
        +160*y**3+195*y**4-197*y**5+65*y**6+5*y**7)+3*r**8*(-1+y)**2*(-840
        -540*logy*(-1+y)**5+4740*y-10830*y**2+12630*y**3-9125*y**4+4941*y**5
        -1217*y**6+29*y**7)-r**4*(-1+y)**3*(2030+8580*logy*(-1+y)**4+460*y
        -17850*y**2+29060*y**3-19295*y**4+11712*y**5-8860*y**6+2656*y**7+15*y**8)
        +r**10*(1660-11620*y+34860*y**2-58100*y**3+59825*y**4-39789*y**5+16219*y**6
        -3541*y**7+306*y**8)-6*MBhat**2*(-1+y)*(-40*(-1+y)**6-110*r**10*(-1
        +y)**6+r**2*(-1+y)**4*(-210+180*logy*(-1+y)**2+600*y-480*y**2+360*y**3
        -485*y**4+198*y**5+5*y**6)+4*r**4*(-1+y)**2*(110-440*y+660*y**2-650*y**3
        +725*y**4-580*y**5+172*y**6)+r**6*(-1+y)*(560+180*logy*(-1+y)**5
        -2980*y+6410*y**2-7850*y**3+6415*y**4-3281*y**5+714*y**6)-2*r**8*(
        -240+1440*y-3600*y**2+4990*y**3-4165*y**4+2031*y**5-517*y**6+51*y**7))
        +2*MBhat*(-240*(-1+y)**7+399*r**12*(-1+y)**7+r**4*(-1+y)**3*(5575
        +2040*logy*(-1+y)**4-20260*y+26310*y**2-17030*y**3+11915*y**4-10056*y**5
        +3988*y**6-514*y**7)+r**2*(-1+y)**5*(-3494+1860*logy*(-1+y)**2+8848*y
        -6284*y**2+1625*y**3-1210*y**4+356*y**5+84*y**6+3*y**7)-3*r**6*(-1
        +y)**2*(-890+600*logy*(-1+y)**5+3850*y-6200*y**2+5830*y**3-5795*y**4
        +5063*y**5-2157*y**6+318*y**7+29*y**8)+r**10*(1660-11620*y+34860*y**2
        -59825*y**3+64220*y**4-43197*y**5+17479*y**6-3817*y**7+330*y**8)-r**8*(2090
        -14630*y+43890*y**2-77200*y**3+90325*y**4-71727*y**5+36026*y**6-9608*y**7
        +681*y**8+153*y**9)))/(180*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (1200*(-1+y)**8+440*r**14*(-1+y)**8+20*r**8*(-1+y)**2*(-2330+300*logy*(
        -1+y)**6+14280*y-36600*y**2+50300*y**3-39225*y**4+17823*y**5-7248*y**6
        +4095*y**7-1005*y**8)-r**2*(-1+y)**6*(-78830+25800*logy*(-1+y)**2
        +183460*y-117530*y**2+8600*y**3+2150*y**4+3506*y**5-1975*y**6+152*y**7
        +107*y**8)-10*r**6*(-1+y)**3*(-20840+17400*logy*(-1+y)**5+86800*y
        -130100*y**2+72100*y**3+7450*y**4-20888*y**5+9793*y**6-6467*y**7+1972*y**8)
        -10*r**4*(-1+y)**4*(-17411+18060*logy*(-1+y)**4+87704*y-167676*y**2
        +147904*y**3-55036*y**4+2511*y**5+3748*y**6-2590*y**7+795*y**8+15*y**9)
        +5*r**10*(-1+y)*(-1250-2160*logy*(-1+y)**7+10910*y-40290*y**2+82270*y**3
        -101170*y**4+79860*y**5-45137*y**6+17831*y**7-3291*y**8+87*y**9)+2*r**12*(
        -2915+23320*y-81620*y**2+163240*y**3-204050*y**4+166813*y**5-91808*y**6
        +31996*y**7-6065*y**8+459*y**9)-30*MBhat**4*(-1+y)**2*(-40*(-1+y)**6
        -110*r**10*(-1+y)**6+r**2*(-1+y)**4*(-210+180*logy*(-1+y)**2+600*y
        -480*y**2+360*y**3-485*y**4+198*y**5+5*y**6)+4*r**4*(-1+y)**2*(110
        -440*y+660*y**2-650*y**3+725*y**4-580*y**5+172*y**6)+r**6*(-1+y)*(560
        +180*logy*(-1+y)**5-2980*y+6410*y**2-7850*y**3+6415*y**4-3281*y**5
        +714*y**6)-2*r**8*(-240+1440*y-3600*y**2+4990*y**3-4165*y**4+2031*y**5
        -517*y**6+51*y**7))+20*MBhat**3*(-1+y)*(-240*(-1+y)**7+399*r**12*(
        -1+y)**7+r**4*(-1+y)**3*(5575+2040*logy*(-1+y)**4-20260*y+26310*y**2
        -17030*y**3+11915*y**4-10056*y**5+3988*y**6-514*y**7)+r**2*(-1+y)**5*(
        -3494+1860*logy*(-1+y)**2+8848*y-6284*y**2+1625*y**3-1210*y**4+356*y**5
        +84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-890+600*logy*(-1+y)**5+3850*y
        -6200*y**2+5830*y**3-5795*y**4+5063*y**5-2157*y**6+318*y**7+29*y**8)
        +r**10*(1660-11620*y+34860*y**2-59825*y**3+64220*y**4-43197*y**5+17479*y**6
        -3817*y**7+330*y**8)-r**8*(2090-14630*y+43890*y**2-77200*y**3+90325*y**4
        -71727*y**5+36026*y**6-9608*y**7+681*y**8+153*y**9))+4*MBhat*(-1200*(
        -1+y)**8+475*r**14*(-1+y)**8+r**2*(-1+y)**6*(-53835+19500*logy*(-1
        +y)**2+127170*y-83085*y**2+6500*y**3+3830*y**4-721*y**5-593*y**6
        +349*y**7+25*y**8)+10*r**4*(-1+y)**4*(-4495+9480*logy*(-1+y)**4+27460*y
        -60150*y**2+59060*y**3-25175*y**4+4380*y**5-1594*y**6+418*y**7+60*y**8)
        -5*r**6*(-1+y)**3*(24505-6540*logy*(-1+y)**5-115985*y+215620*y**2
        -193820*y**3+83845*y**4-20459*y**5+9700*y**6-3599*y**7+481*y**8)-5*r**8*(
        -1+y)**2*(5600+2040*logy*(-1+y)**6-31560*y+72780*y**2-86840*y**3
        +58875*y**4-29136*y**5+14374*y**6-4635*y**7+693*y**8+29*y**9)+r**12*(
        -3290+26320*y-92120*y**2+184240*y**3-236255*y**4+204286*y**5-116228*y**6
        +41128*y**7-8111*y**8+660*y**9)-5*r**10*(-1655+13240*y-46340*y**2
        +92680*y**3-119120*y**4+106609*y**5-68504*y**6+29968*y**7-7805*y**8
        +927*y**9))+10*MBhat**2*(720*(-1+y)**8+460*r**14*(-1+y)**8+3*r**6*(
        -1+y)**3*(5500+960*logy*(-1+y)**5-28460*y+59320*y**2-60420*y**3
        +28075*y**4-4227*y**5+221*y**6+391*y**7-184*y**8)-r**2*(-1+y)**6*(
        -19948+8400*logy*(-1+y)**2+48296*y-32548*y**2+3682*y**3+1009*y**4
        -1214*y**5+442*y**6+64*y**7+y**8)-r**4*(-1+y)**4*(7675+24180*logy*(
        -1+y)**4-6520*y-38580*y**2+70210*y**3-35035*y**4+4032*y**5-3602*y**6
        +1574*y**7+30*y**8)+3*r**12*(-611+4888*y-17108*y**2+35010*y**3-46056*y**4
        +39732*y**5-22260*y**6+7728*y**7-1479*y**8+114*y**9)-r**8*(-1+y)**2*(
        -4180+2760*logy*(-1+y)**6+27840*y-77880*y**2+110740*y**3-82545*y**4
        +34458*y**5-9876*y**6+1662*y**7-543*y**8+204*y**9)+r**10*(700-5600*y
        +19600*y**2-45170*y**3+74650*y**4-81686*y**5+54271*y**6-19112*y**7
        +1345*y**8+1332*y**9-330*y**10)))/(900*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (5600*(-1+y)**9+1100*r**16*(-1+y)**9+35*r**10*(-1+y)**2*(7210+1800*logy*(
        -1+y)**7-52270*y+163110*y**2-284450*y**3+300200*y**4-192720*y**5
        +73351*y**6-23449*y**7+11905*y**8-2647*y**9)-r**2*(-1+y)**7*(-650110
        +184800*logy*(-1+y)**2+1485020*y-927310*y**2+61600*y**3+15400*y**4
        +6160*y**5+10269*y**6+1070*y**7-4107*y**8+328*y**9)-70*r**6*(-1+y)**4*(6795
        +74100*logy*(-1+y)**5-108075*y+401400*y**2-648400*y**3+509450*y**4
        -175990*y**5+11771*y**6+4805*y**7-3124*y**8+1200*y**9)-7*r**12*(-1
        +y)*(980+9000*logy*(-1+y)**8+1160*y-40060*y**2+164120*y**3-331150*y**4
        +390920*y**5-292057*y**6+156512*y**7-58444*y**8+9960*y**9-261*y**10)
        +70*r**8*(-1+y)**3*(-63050-33000*logy*(-1+y)**6+345300*y-764250*y**2
        +854000*y**3-475500*y**4+91200*y**5+18649*y**6-9722*y**7+5362*y**8
        -1674*y**9+5*y**10)-7*r**4*(-1+y)**5*(-508740+316200*logy*(-1+y)**4
        +2351160*y-4159140*y**2+3405160*y**3-1167490*y**4+63240*y**5+6667*y**6
        +13652*y**7-7268*y**8+2412*y**9+107*y**10)+14*r**14*(1495-13455*y
        +53820*y**2-125580*y**3+188370*y**4-188370*y**5+127624*y**6-59816*y**7
        +18189*y**8-3041*y**9+204*y**10)-140*MBhat**6*(-1+y)**3*(-40*(-1+y)**6
        -110*r**10*(-1+y)**6+r**2*(-1+y)**4*(-210+180*logy*(-1+y)**2+600*y
        -480*y**2+360*y**3-485*y**4+198*y**5+5*y**6)+4*r**4*(-1+y)**2*(110
        -440*y+660*y**2-650*y**3+725*y**4-580*y**5+172*y**6)+r**6*(-1+y)*(560
        +180*logy*(-1+y)**5-2980*y+6410*y**2-7850*y**3+6415*y**4-3281*y**5
        +714*y**6)-2*r**8*(-240+1440*y-3600*y**2+4990*y**3-4165*y**4+2031*y**5
        -517*y**6+51*y**7))+140*MBhat**5*(-1+y)**2*(-240*(-1+y)**7+399*r**12*(
        -1+y)**7+r**4*(-1+y)**3*(5575+2040*logy*(-1+y)**4-20260*y+26310*y**2
        -17030*y**3+11915*y**4-10056*y**5+3988*y**6-514*y**7)+r**2*(-1+y)**5*(
        -3494+1860*logy*(-1+y)**2+8848*y-6284*y**2+1625*y**3-1210*y**4+356*y**5
        +84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-890+600*logy*(-1+y)**5+3850*y
        -6200*y**2+5830*y**3-5795*y**4+5063*y**5-2157*y**6+318*y**7+29*y**8)
        +r**10*(1660-11620*y+34860*y**2-59825*y**3+64220*y**4-43197*y**5+17479*y**6
        -3817*y**7+330*y**8)-r**8*(2090-14630*y+43890*y**2-77200*y**3+90325*y**4
        -71727*y**5+36026*y**6-9608*y**7+681*y**8+153*y**9))+70*MBhat**4*(-1
        +y)*(1200*(-1+y)**8+920*r**14*(-1+y)**8-2*r**2*(-1+y)**6*(-16492+6960*logy*(
        -1+y)**2+39944*y-26932*y**2+3202*y**3+424*y**4-623*y**5+247*y**6
        +49*y**7+y**8)-3*r**4*(-1+y)**4*(4440+13260*logy*(-1+y)**4-4500*y
        -19770*y**2+37120*y**3-16925*y**4-1216*y**5+552*y**6+164*y**7+15*y**8)
        -3*r**6*(-1+y)**3*(-8940-1800*logy*(-1+y)**5+46500*y-97500*y**2+99300*y**3
        -43355*y**4+827*y**5+4579*y**6-2139*y**7+368*y**8)+12*r**12*(-322+2576*y
        -9016*y**2+18429*y**3-24183*y**4+20790*y**5-11592*y**6+3996*y**7-756*y**8
        +57*y**9)-r**8*(-1+y)**2*(-5840+3900*logy*(-1+y)**6+38940*y-109050*y**2
        +151100*y**3-99825*y**4+26718*y**5-1278*y**6-414*y**7-999*y**8+408*y**9)
        -3*r**10*(-1020+8160*y-28560*y**2+61100*y**3-89075*y**4+87662*y**5
        -54850*y**6+19328*y**7-2179*y**8-786*y**9+220*y**10))+2*MBhat*(-16800*(
        -1+y)**9+2470*r**16*(-1+y)**9+70*r**8*(-1+y)**3*(67475+15300*logy*(
        -1+y)**6-389550*y+927975*y**2-1160800*y**3+794100*y**4-273552*y**5
        +39725*y**6-8700*y**7+3117*y**8-450*y**9)+r**2*(-1+y)**7*(-1484580
        +449400*logy*(-1+y)**2+3418560*y-2158680*y**2+149800*y**3+37450*y**4
        +30184*y**5+15281*y**6-14113*y**7+737*y**8+321*y**9)-210*r**6*(-1+y)**4*(13793
        -32460*logy*(-1+y)**5-36505*y-8140*y**2+116340*y**3-139320*y**4+60745*y**5
        -7378*y**6+1184*y**7-541*y**8-35*y**9+y**10)+7*r**4*(-1+y)**5*(-802660
        +604800*logy*(-1+y)**4+3815440*y-6932760*y**2+5831440*y**3-2062660*y**4
        +112512*y**5+39337*y**6-1828*y**7-1938*y**8+2372*y**9+25*y**10)-35*r**10*(
        -1+y)**2*(-16620+4200*logy*(-1+y)**7+112140*y-321720*y**2+506800*y**3
        -470050*y**4+258726*y**5-93449*y**6+30446*y**7-7110*y**8+988*y**9+29*y**10)
        +14*r**14*(1795-16155*y+64620*y**2-150780*y**3+226170*y**4-230769*y**5
        +166131*y**6-81324*y**7+25281*y**8-4459*y**9+330*y**10)+21*r**12*(-4980
        +44820*y-179280*y**2+418320*y**3-627480*y**4+636784*y**5-458947*y**6
        +242439*y**7-90046*y**8+20734*y**9-2415*y**10+51*y**11))+MBhat**2*(84000*(
        -1+y)**9+12650*r**16*(-1+y)**9+70*r**8*(-1+y)**3*(-42435+7020*logy*(
        -1+y)**6+261630*y-675135*y**2+935280*y**3-731175*y**4+293010*y**5
        -43177*y**6+2220*y**7+2586*y**8-764*y**9)-70*r**6*(-1+y)**4*(-211285
        +169980*logy*(-1+y)**5+886445*y-1347940*y**2+781340*y**3+30335*y**4
        -163207*y**5+23480*y**6-2524*y**7+1496*y**8+60*y**9)-r**2*(-1+y)**7*(
        -5415500+1772400*logy*(-1+y)**2+12603400*y-8074100*y**2+590800*y**3
        +191905*y**4+144886*y**5-67641*y**6-5496*y**7+6246*y**8+300*y**9)-35*r**4*(
        -1+y)**5*(-347168+357360*logy*(-1+y)**4+1746032*y-3333768*y**2+2937232*y**3
        -1096945*y**4+77856*y**5+21632*y**6-9596*y**7+3795*y**8+212*y**9-2*y**10)
        -35*r**10*(-1+y)**2*(3540+10320*logy*(-1+y)**7-35100*y+141420*y**2
        -307940*y**3+379445*y**4-246843*y**5+83315*y**6-16721*y**7-852*y**8
        +252*y**9+204*y**10)+70*r**14*(1079-9711*y+38844*y**2-90636*y**3+139020*y**4
        -147924*y**5+108276*y**6-53244*y**7+16731*y**8-2999*y**9+228*y**10)
        -7*r**12*(7000-63000*y+252000*y**2-588000*y**3+972165*y**4-1285257*y**5
        +1266546*y**6-829254*y**7+338121*y**8-76309*y**9+5328*y**10+660*y**11))
        +2*MBhat**3*(-56000*(-1+y)**9+11910*r**16*(-1+y)**9-70*r**6*(-1+y)**4*(77787
        -23700*logy*(-1+y)**5-365235*y+671220*y**2-591170*y**3+238135*y**4
        -37289*y**5+10452*y**6-3348*y**7+408*y**8)+35*r**4*(-1+y)**5*(-61072
        +125040*logy*(-1+y)**4+369328*y-804072*y**2+784724*y**3-325901*y**4
        +42770*y**5-9534*y**6+2860*y**7+411*y**8+6*y**9)+r**2*(-1+y)**7*(-2472200
        +898800*logy*(-1+y)**2+5843200*y-3820400*y**2+307860*y**3+160475*y**4
        -27636*y**5-19873*y**6+10664*y**7+1100*y**8+10*y**9)-70*r**8*(-1+y)**3*(14485
        +4140*logy*(-1+y)**6-82770*y+194505*y**2-237500*y**3+159985*y**4
        -68426*y**5+24535*y**6-4296*y**7+10*y**8+152*y**9)+70*r**14*(639-5751*y
        +23004*y**2-54698*y**3+85372*y**4-90216*y**5+64974*y**6-31296*y**7
        +9549*y**8-1636*y**9+115*y**10)-35*r**10*(-1+y)**2*(7840-54880*y+164640*y**2
        -269660*y**3+270125*y**4-189449*y**5+97339*y**6-33935*y**7+8434*y**8
        -1634*y**9+220*y**10)-7*r**12*(-1000+9000*y-36000*y**2+56820*y**3
        -13965*y**4-39357*y**5+16998*y**6+40608*y**7-61677*y**8+39703*y**9
        -12840*y**10+1710*y**11)))/(4200*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (8400*(-1+y)**10+990*r**18*(-1+y)**10-84*r**12*(-1+y)**3*(-3860-2100*logy*(
        -1+y)**7+29120*y-94710*y**2+172550*y**3-190925*y**4+129255*y**5-50435*y**6
        +10499*y**7-6488*y**8+1634*y**9)-6*r**2*(-1+y)**8*(-252665+65100*logy*(
        -1+y)**2+570430*y-350315*y**2+21700*y**3+5425*y**4+2170*y**5+1085*y**6
        +1636*y**7+1888*y**8-1824*y**9+50*y**10)-210*r**6*(-1+y)**5*(102838
        +134940*logy*(-1+y)**5-649130*y+1635610*y**2-2085410*y**3+1380055*y**4
        -410951*y**5+22490*y**6+3458*y**7+1168*y**8-730*y**9+494*y**10)-420*r**8*(
        -1+y)**4*(41522+76440*logy*(-1+y)**6-172692*y+202410*y**2+112320*y**3
        -466440*y**4+415896*y**5-145756*y**6+11248*y**7+1825*y**8-1003*y**9
        +525*y**10-5*y**11)+70*r**10*(-1+y)**3*(281470-113700*logy*(-1+y)**7
        -1856590*y+5171820*y**2-7823800*y**3+6828925*y**4-3301455*y**5+702535*y**6
        +12371*y**7-20487*y**8+9041*y**9-2945*y**10+15*y**11)+42*r**14*(-1
        +y)*(1710-2700*logy*(-1+y)**9-12690*y+38610*y**2-57690*y**3+29835*y**4
        +38205*y**5-82170*y**6+70518*y**7-39412*y**8+14498*y**9-2312*y**10
        +58*y**11)+20*r**16*(-1274+12740*y-57330*y**2+152880*y**3-267540*y**4
        +321048*y**5-267540*y**6+154740*y**7-63105*y**8+17015*y**9-2543*y**10
        +153*y**11)-4*r**4*(-1+y)**6*(-3610580+1749300*logy*(-1+y)**4+16191620*y
        -27786030*y**2+22022620*y**3-7254955*y**4+349860*y**5+58310*y**6+14386*y**7
        +10251*y**8+4136*y**9-494*y**10+246*y**11)-210*MBhat**8*(-1+y)**4*(
        -40*(-1+y)**6-110*r**10*(-1+y)**6+r**2*(-1+y)**4*(-210+180*logy*(
        -1+y)**2+600*y-480*y**2+360*y**3-485*y**4+198*y**5+5*y**6)+4*r**4*(
        -1+y)**2*(110-440*y+660*y**2-650*y**3+725*y**4-580*y**5+172*y**6)
        +r**6*(-1+y)*(560+180*logy*(-1+y)**5-2980*y+6410*y**2-7850*y**3
        +6415*y**4-3281*y**5+714*y**6)-2*r**8*(-240+1440*y-3600*y**2+4990*y**3
        -4165*y**4+2031*y**5-517*y**6+51*y**7))+280*MBhat**7*(-1+y)**3*(-240*(
        -1+y)**7+399*r**12*(-1+y)**7+r**4*(-1+y)**3*(5575+2040*logy*(-1+y)**4
        -20260*y+26310*y**2-17030*y**3+11915*y**4-10056*y**5+3988*y**6-514*y**7)
        +r**2*(-1+y)**5*(-3494+1860*logy*(-1+y)**2+8848*y-6284*y**2+1625*y**3
        -1210*y**4+356*y**5+84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-890+600*logy*(
        -1+y)**5+3850*y-6200*y**2+5830*y**3-5795*y**4+5063*y**5-2157*y**6
        +318*y**7+29*y**8)+r**10*(1660-11620*y+34860*y**2-59825*y**3+64220*y**4
        -43197*y**5+17479*y**6-3817*y**7+330*y**8)-r**8*(2090-14630*y+43890*y**2
        -77200*y**3+90325*y**4-71727*y**5+36026*y**6-9608*y**7+681*y**8+153*y**9))
        -140*MBhat**6*(-1+y)**2*(-1680*(-1+y)**8-1380*r**14*(-1+y)**8+3*r**2*(
        -1+y)**6*(-15340+6480*logy*(-1+y)**2+37160*y-25060*y**2+3042*y**3
        +229*y**4-426*y**5+182*y**6+44*y**7+y**8)+r**4*(-1+y)**4*(18965+55380*logy*(
        -1+y)**4-20480*y-80040*y**2+152510*y**3-66515*y**4-11328*y**5+6914*y**6
        -590*y**7+60*y**8)+3*r**6*(-1+y)**3*(-12380-2640*logy*(-1+y)**5+64540*y
        -135680*y**2+138180*y**3-58635*y**4-2573*y**5+9379*y**6-3887*y**7+552*y**8)
        -9*r**12*(-655+5240*y-18340*y**2+37474*y**3-49136*y**4+42196*y**5
        -23492*y**6+8080*y**7-1523*y**8+114*y**9)+3*r**8*(-1+y)**2*(-2500+1680*logy*(
        -1+y)**6+16680*y-46740*y**2+63820*y**3-39035*y**4+6326*y**5+2440*y**6
        -830*y**7-485*y**8+204*y**9)+r**10*(-5420+43360*y-151760*y**2+321430*y**3
        -459800*y**4+444286*y**5-274829*y**6+96856*y**7-11729*y**8-3384*y**9
        +990*y**10))+12*MBhat**5*(-1+y)*(-39200*(-1+y)**9+11910*r**16*(-1+y)**9
        +70*r**6*(-1+y)**4*(-53282+17160*logy*(-1+y)**5+249250*y-455600*y**2
        +397350*y**3-154290*y**4+16830*y**5-752*y**6-251*y**7+73*y**8)+35*r**4*(
        -1+y)**5*(-43092+87120*logy*(-1+y)**4+259488*y-563472*y**2+548484*y**3
        -225201*y**4+25250*y**5-3158*y**6+1188*y**7+171*y**8+6*y**9)+r**2*(
        -1+y)**7*(-1718510+625800*logy*(-1+y)**2+4062820*y-2657210*y**2+216860*y**3
        +106855*y**4-17542*y**5-11571*y**6+5778*y**7+750*y**8+10*y**9)-70*r**8*(
        -1+y)**3*(8885+2100*logy*(-1+y)**6-51210*y+121725*y**2-150660*y**3
        +101110*y**4-39290*y**5+10161*y**6+339*y**7-683*y**8+123*y**9)+70*r**14*(734
        -6606*y+26424*y**2-62678*y**3+97342*y**4-102186*y**5+72954*y**6-34716*y**7
        +10404*y**8-1731*y**9+115*y**10)-35*r**10*(-1+y)**2*(4530-31710*y
        +95130*y**2-153810*y**3+147735*y**4-98621*y**5+51159*y**6-20179*y**7
        +6580*y**8-1634*y**9+220*y**10)-21*r**12*(1860-16740*y+66960*y**2
        -165300*y**3+275675*y**4-306813*y**5+219342*y**6-91368*y**7+12267*y**8
        +7387*y**9-3840*y**10+570*y**11))+3*MBhat**2*(78400*(-1+y)**10+4025*r**18*(
        -1+y)**10+70*r**10*(-1+y)**3*(58320+6300*logy*(-1+y)**7-414540*y
        +1265670*y**2-2153550*y**3+2208675*y**4-1374321*y**5+470714*y**6-66578*y**7
        +6823*y**8+1947*y**9-580*y**10)-140*r**8*(-1+y)**4*(439585+231300*logy*(
        -1+y)**6-2406210*y+5321625*y**2-5939000*y**3+3297750*y**4-620652*y**5
        -127585*y**6+31562*y**7+401*y**8+124*y**9+40*y**10)-r**2*(-1+y)**8*(
        -8987175+2553600*logy*(-1+y)**2+20527950*y-12817575*y**2+851200*y**3
        +212800*y**4+100912*y**5+138950*y**6-21708*y**7-30841*y**8+1646*y**9
        +321*y**10)-14*r**4*(-1+y)**6*(-3575250+2205000*logy*(-1+y)**4+16506000*y
        -29169000*y**2+23856000*y**3-8169000*y**4+440664*y**5+60995*y**6+54100*y**7
        -10220*y**8+2940*y**9+1091*y**10)-140*r**6*(-1+y)**5*(48070+520800*logy*(
        -1+y)**5-761150*y+2824300*y**2-4560300*y**3+3582150*y**4-1235334*y**5
        +85985*y**6+14495*y**7-955*y**8+1555*y**9+10*y**10-2*y**11)+60*r**16*(
        -555+5550*y-24975*y**2+66600*y**3-116550*y**4+142464*y**5-126630*y**6
        +79980*y**7-34695*y**8+9800*y**9-1607*y**10+114*y**11)-14*r**12*(-1
        +y)**2*(16450+27600*logy*(-1+y)**8-104000*y+253600*y**2-249600*y**3
        -74400*y**4+419232*y**5-349717*y**6+133352*y**7-37708*y**8-228*y**9
        +795*y**10+204*y**11)-28*r**14*(-1250+12500*y-56250*y**2+150000*y**3
        -262500*y**4+332808*y**5-346178*y**6+287940*y**7-167245*y**8+63040*y**9
        -14323*y**10+1458*y**11))+4*MBhat*(-16800*(-1+y)**10+1140*r**18*(-1
        +y)**10+r**2*(-1+y)**8*(-2446010+659400*logy*(-1+y)**2+5551420*y
        -3435110*y**2+219800*y**3+54950*y**4+21980*y**5+18298*y**6+26629*y**7
        -14036*y**8-3207*y**9+246*y**10)-35*r**10*(-1+y)**3*(293330-60000*logy*(
        -1+y)**7-1993310*y+5769930*y**2-9196550*y**3+8671550*y**4-4782930*y**5
        +1381364*y**6-150401*y**7+13269*y**8-4931*y**9+839*y**10)+70*r**8*(
        -1+y)**4*(366523+340860*logy*(-1+y)**6-1858278*y+3623115*y**2-3126520*y**3
        +640590*y**4+766344*y**5-465725*y**6+50889*y**7-957*y**8+2609*y**9
        +36*y**10-6*y**11)+105*r**6*(-1+y)**5*(138482+296760*logy*(-1+y)**5
        -989170*y+2720240*y**2-3709440*y**3+2596620*y**4-816084*y**5+50205*y**6
        +7557*y**7+454*y**8+440*y**9+317*y**10-5*y**11)-21*r**12*(-1+y)**2*(35690
        +6600*logy*(-1+y)**8-278920*y+949820*y**2-1838040*y**3+2205150*y**4
        -1671720*y**5+780663*y**6-231609*y**7+56410*y**8-9516*y**9+1203*y**10
        +29*y**11)+10*r**16*(-1589+15890*y-71505*y**2+190680*y**3-333690*y**4
        +400428*y**5-338898*y**6+208560*y**7-89415*y**8+24710*y**9-3923*y**10
        +264*y**11)+r**4*(-1+y)**6*(-18254270+9819600*logy*(-1+y)**4+82836680*y
        -143894220*y**2+115568680*y**3-38711770*y**4+1963920*y**5+318773*y**6
        +85189*y**7+102414*y**8-32206*y**9+11449*y**10+321*y**11)+7*r**14*(14110
        -141100*y+634950*y**2-1693200*y**3+2963100*y**4-3555720*y**5+2987241*y**6
        -1805298*y**7+807225*y**8-257980*y**9+52165*y**10-5646*y**11+153*y**12))
        +4*MBhat**3*(-117600*(-1+y)**10+5955*r**18*(-1+y)**10+210*r**8*(-1+y)**4*(152502
        +40440*logy*(-1+y)**6-874572*y+2065110*y**2-2551280*y**3+1714075*y**4
        -567122*y**5+61614*y**6-2062*y**7+183*y**8-68*y**9)+105*r**6*(-1+y)**5*(
        -189882+456840*logy*(-1+y)**5+492570*y+156960*y**2-1679760*y**3+1984575*y**4
        -858229*y**5+88418*y**6+254*y**7+3057*y**8+15*y**9+6*y**10)+3*r**2*(
        -1+y)**8*(-3428055+1037400*logy*(-1+y)**2+7893510*y-4984155*y**2
        +345800*y**3+87430*y**4+78736*y**5+15491*y**6-21371*y**7+60*y**8+769*y**9
        +25*y**10)-35*r**10*(-1+y)**3*(-72890+9000*logy*(-1+y)**7+501230*y
        -1472190*y**2+2390650*y**3-2322250*y**4+1344006*y**5-473293*y**6+115318*y**7
        -5982*y**8-2458*y**9+559*y**10)+r**4*(-1+y)**6*(-39511260+29542800*logy*(
        -1+y)**4+187587840*y-340467360*y**2+286063840*y**3-101027785*y**4
        +5519892*y**5+1732367*y**6+35362*y**7-25983*y**8+56352*y**9+1485*y**10
        -30*y**11)-21*r**12*(-1+y)**2*(-25740+205920*y-720720*y**2+1441440*y**3
        -1789335*y**4+1406048*y**5-744745*y**6+272714*y**7-60817*y**8+8742*y**9
        -1287*y**10+220*y**11)+20*r**16*(-1543+15430*y-69435*y**2+185160*y**3
        -330540*y**4+418026*y**5-375060*y**6+237090*y**7-103140*y**8+29245*y**9
        -4822*y**10+345*y**11)-21*r**14*(690-6900*y+31050*y**2-82800*y**3
        +125775*y**4-85208*y**5-5365*y**6+48990*y**7-38415*y**8+13670*y**9
        -817*y**10-860*y**11+190*y**12))+MBhat**4*(588000*(-1+y)**10+38325*r**18*(
        -1+y)**10-210*r**6*(-1+y)**5*(-487400+402000*logy*(-1+y)**5+2035000*y
        -3065000*y**2+1729060*y**3+125875*y**4-392431*y**5+44368*y**6+7072*y**7
        -909*y**8+165*y**9)-70*r**4*(-1+y)**6*(-1225038+1246440*logy*(-1+y)**4
        +6146592*y-11712768*y**2+10305268*y**3-3852967*y**4+294264*y**5+45854*y**6
        -10856*y**7+6663*y**8+468*y**9)-140*r**8*(-1+y)**4*(157255-15300*logy*(
        -1+y)**6-958830*y+2442975*y**2-3322760*y**3+2537565*y**4-983616*y**5
        +124529*y**6+2022*y**7-8172*y**8+1532*y**9)-r**2*(-1+y)**8*(-37556475
        +12306000*logy*(-1+y)**2+87418950*y-56015475*y**2+4046560*y**3+1555960*y**4
        +731416*y**5-348726*y**6-46116*y**7+35001*y**8+2490*y**9+15*y**10)
        -70*r**10*(-1+y)**3*(4600+27900*logy*(-1+y)**7-60100*y+277950*y**2
        -642230*y**3+800075*y**4-455085*y**5+68906*y**6+21250*y**7-16929*y**8
        +2099*y**9+364*y**10)-42*r**12*(-1+y)**2*(-7950+63600*y-222600*y**2
        +439820*y**3-628525*y**4+793132*y**5-718464*y**6+428760*y**7-171291*y**8
        +44688*y**9-7650*y**10+760*y**11)+20*r**16*(-6575+65750*y-295875*y**2
        +802020*y**3-1451100*y**4+1820070*y**5-1608432*y**6+997710*y**7-423270*y**8
        +115895*y**9-18140*y**10+1191*y**11)-70*r**14*(1288-12880*y+57960*y**2
        -144276*y**3+232215*y**4-290742*y**5+303435*y**6-255180*y**7+169200*y**8
        -85450*y**9+30490*y**10-6750*y**11+690*y**12)))/(6300*r**2*(-1+y)**10)
    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr10(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r10 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (60*(-1+y)**6-105*r**10*(-1+y)**6-r**2*(-1+y)**4*(115-120*logy*(-1
        +y)**2-350*y+295*y**2+140*y**3-325*y**4+122*y**5+5*y**6)-2*r**6*(
        -1+y)*(150+90*logy*(-1+y)**5-840*y+1905*y**2-2135*y**3+1600*y**4
        -869*y**5+207*y**6)-2*r**4*(-1+y)**2*(60-240*y+360*y**2-420*y**3
        +665*y**4-664*y**5+215*y**6)-2*r**8*(10-60*y+150*y**2+140*y**3-260*y**4
        +201*y**5-86*y**6+15*y**7))/(30*r**2*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (360*(-1+y)**7-189*r**12*(-1+y)**7-3*r**2*(-1+y)**5*(997-360*logy*(
        -1+y)**2-2354*y+1537*y**2-120*y**3+275*y**4-476*y**5+135*y**6+10*y**7)
        -6*r**6*(-1+y)**2*(-1200+90*logy*(-1+y)**5+5910*y-11595*y**2+11295*y**3
        -5505*y**4+2075*y**5-1489*y**6+449*y**7)+6*r**8*(-1+y)*(200-270*logy*(
        -1+y)**6-1470*y+4485*y**2-7330*y**3+5980*y**4-3344*y**5+1518*y**6
        -322*y**7+3*y**8)-r**4*(-1+y)**3*(-9125+3120*logy*(-1+y)**4+39620*y
        -65670*y**2+50020*y**3-17350*y**4+6228*y**5-5918*y**6+1892*y**7+15*y**8)
        -r**10*(-305+2135*y-6405*y**2+10675*y**3-5785*y**4+1821*y**5+133*y**6
        -379*y**7+90*y**8)-6*MBhat**2*(-1+y)*(-60*(-1+y)**6+105*r**10*(-1
        +y)**6+r**2*(-1+y)**4*(115-120*logy*(-1+y)**2-350*y+295*y**2+140*y**3
        -325*y**4+122*y**5+5*y**6)+2*r**6*(-1+y)*(150+90*logy*(-1+y)**5
        -840*y+1905*y**2-2135*y**3+1600*y**4-869*y**5+207*y**6)+2*r**4*(-1
        +y)**2*(60-240*y+360*y**2-420*y**3+665*y**4-664*y**5+215*y**6)+2*r**8*(10
        -60*y+150*y**2+140*y**3-260*y**4+201*y**5-86*y**6+15*y**7))-2*MBhat*(360*(
        -1+y)**7+714*r**12*(-1+y)**7-r**2*(-1+y)**5*(1519-1020*logy*(-1+y)**2
        -4058*y+3049*y**2+560*y**3-1420*y**4+275*y**5+84*y**6+3*y**7)+2*r**4*(
        -1+y)**3*(-365-300*logy*(-1+y)**4+1160*y-1140*y**2+1045*y**3-2635*y**4
        +3090*y**5-1118*y**6+107*y**7)+3*r**6*(-1+y)**2*(-740+600*logy*(-1
        +y)**5+3100*y-4700*y**2+2990*y**3-2320*y**4+2870*y**5-1449*y**6+267*y**7
        +6*y**8)+5*r**10*(313-2191*y+6573*y**2-11933*y**3+12860*y**4-8589*y**5
        +3451*y**6-745*y**7+63*y**8)-r**8*(520-3640*y+10920*y**2-23090*y**3
        +27260*y**4-15066*y**5+2341*y**6+1271*y**7-561*y**8+45*y**9)))/(180*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (900*(-1+y)**8-210*r**14*(-1+y)**8+r**2*(-1+y)**6*(-13300+3000*logy*(
        -1+y)**2+29600*y-17800*y**2+1000*y**3+250*y**4-2027*y**5+2949*y**6
        -657*y**7-45*y**8)-10*r**8*(-1+y)**2*(5210+360*logy*(-1+y)**6-30900*y
        +76170*y**2-99760*y**3+73020*y**4-26901*y**5+5207*y**6-2445*y**7+669*y**8)
        -10*r**6*(-1+y)**3*(-5015+11460*logy*(-1+y)**5+13615*y+1420*y**2
        -39620*y**3+48460*y**4-21383*y**5+3619*y**6-2048*y**7+718*y**8)+15*r**10*(
        -1+y)*(-20-360*logy*(-1+y)**7+500*y-2760*y**2+7120*y**3-10270*y**4
        +6815*y**5-3254*y**6+1386*y**7-270*y**8+3*y**9)-15*r**4*(-1+y)**4*(
        -7684+1680*logy*(-1+y)**4+32416*y-51984*y**2+38016*y**3-11184*y**4
        +101*y**5+810*y**6-794*y**7+250*y**8+5*y**9)-r**12*(700-5600*y+19600*y**2
        -39200*y**3+49000*y**4-24377*y**5+8116*y**6-956*y**7-488*y**8+135*y**9)
        -15*MBhat**4*(-1+y)**2*(-60*(-1+y)**6+105*r**10*(-1+y)**6+r**2*(-1
        +y)**4*(115-120*logy*(-1+y)**2-350*y+295*y**2+140*y**3-325*y**4
        +122*y**5+5*y**6)+2*r**6*(-1+y)*(150+90*logy*(-1+y)**5-840*y+1905*y**2
        -2135*y**3+1600*y**4-869*y**5+207*y**6)+2*r**4*(-1+y)**2*(60-240*y
        +360*y**2-420*y**3+665*y**4-664*y**5+215*y**6)+2*r**8*(10-60*y+150*y**2
        +140*y**3-260*y**4+201*y**5-86*y**6+15*y**7))-10*MBhat**3*(-1+y)*(360*(
        -1+y)**7+714*r**12*(-1+y)**7-r**2*(-1+y)**5*(1519-1020*logy*(-1+y)**2
        -4058*y+3049*y**2+560*y**3-1420*y**4+275*y**5+84*y**6+3*y**7)+2*r**4*(
        -1+y)**3*(-365-300*logy*(-1+y)**4+1160*y-1140*y**2+1045*y**3-2635*y**4
        +3090*y**5-1118*y**6+107*y**7)+3*r**6*(-1+y)**2*(-740+600*logy*(-1
        +y)**5+3100*y-4700*y**2+2990*y**3-2320*y**4+2870*y**5-1449*y**6+267*y**7
        +6*y**8)+5*r**10*(313-2191*y+6573*y**2-11933*y**3+12860*y**4-8589*y**5
        +3451*y**6-745*y**7+63*y**8)-r**8*(520-3640*y+10920*y**2-23090*y**3
        +27260*y**4-15066*y**5+2341*y**6+1271*y**7-561*y**8+45*y**9))-10*MBhat*(360*(
        -1+y)**8+170*r**14*(-1+y)**8+r**2*(-1+y)**6*(-3927+1260*logy*(-1
        +y)**2+9114*y-5817*y**2+420*y**3-594*y**4+852*y**5+19*y**6-74*y**7
        -5*y**8)-r**4*(-1+y)**4*(-23495+7260*logy*(-1+y)**4+101240*y-166380*y**2
        +125440*y**3-39775*y**4+4992*y**5-3194*y**6+764*y**7+120*y**8)+r**6*(
        -1+y)**3*(24550-8880*logy*(-1+y)**5-113870*y+205540*y**2-175940*y**3
        +66700*y**4-10322*y**5+5983*y**6-2243*y**7+250*y**8)+2*r**8*(-1+y)**2*(2630
        +1020*logy*(-1+y)**6-14760*y+33840*y**2-40020*y**3+23040*y**4-8283*y**5
        +3907*y**6-1260*y**7+213*y**8+3*y**9)+r**10*(-365+2920*y-10220*y**2
        +20440*y**3-16385*y**4+2614*y**5+1636*y**6-692*y**7+10*y**8+42*y**9)
        +r**12*(-443+3544*y-12404*y**2+24808*y**3-35951*y**4+32206*y**5-18704*y**6
        +6856*y**7-1424*y**8+126*y**9))-5*MBhat**2*(-1080*(-1+y)**8+1304*r**14*(
        -1+y)**8+r**2*(-1+y)**6*(7716-3600*logy*(-1+y)**2-19032*y+13116*y**2
        +138*y**3-936*y**4-1500*y**5+469*y**6+64*y**7+y**8)+2*r**4*(-1+y)**4*(
        -10442+5100*logy*(-1+y)**4+46868*y-80502*y**2+62773*y**3-18427*y**4
        +30*y**5-1388*y**6+641*y**7+15*y**8)+12*r**6*(-1+y)**3*(-1470-240*logy*(
        -1+y)**5+7590*y-15780*y**2+16345*y**3-7620*y**4+732*y**5+229*y**6
        -160*y**7+44*y**8)+4*r**8*(-1+y)**2*(-1580+690*logy*(-1+y)**6+10170*y
        -27495*y**2+41475*y**3-30120*y**4+9129*y**5+884*y**6-1773*y**7+465*y**8
        -15*y**9)+18*r**12*(-242+1936*y-6776*y**2+14101*y**3-18357*y**4+15540*y**5
        -8484*y**6+2832*y**7-507*y**8+34*y**9)-5*r**10*(-1196+9568*y-33488*y**2
        +70438*y**3-91940*y**4+75766*y**5-39179*y**6+11200*y**7-914*y**8-318*y**9
        +63*y**10)))/(450*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-70*MBhat**6*(-1+y)**3*(-60*(-1+y)**6+105*r**10*(-1+y)**6+r**2*(-1
        +y)**4*(115-120*logy*(-1+y)**2-350*y+295*y**2+140*y**3-325*y**4
        +122*y**5+5*y**6)+2*r**6*(-1+y)*(150+90*logy*(-1+y)**5-840*y+1905*y**2
        -2135*y**3+1600*y**4-869*y**5+207*y**6)+2*r**4*(-1+y)**2*(60-240*y
        +360*y**2-420*y**3+665*y**4-664*y**5+215*y**6)+2*r**8*(10-60*y+150*y**2
        +140*y**3-260*y**4+201*y**5-86*y**6+15*y**7))-70*MBhat**5*(-1+y)**2*(360*(
        -1+y)**7+714*r**12*(-1+y)**7-r**2*(-1+y)**5*(1519-1020*logy*(-1+y)**2
        -4058*y+3049*y**2+560*y**3-1420*y**4+275*y**5+84*y**6+3*y**7)+2*r**4*(
        -1+y)**3*(-365-300*logy*(-1+y)**4+1160*y-1140*y**2+1045*y**3-2635*y**4
        +3090*y**5-1118*y**6+107*y**7)+3*r**6*(-1+y)**2*(-740+600*logy*(-1
        +y)**5+3100*y-4700*y**2+2990*y**3-2320*y**4+2870*y**5-1449*y**6+267*y**7
        +6*y**8)+5*r**10*(313-2191*y+6573*y**2-11933*y**3+12860*y**4-8589*y**5
        +3451*y**6-745*y**7+63*y**8)-r**8*(520-3640*y+10920*y**2-23090*y**3
        +27260*y**4-15066*y**5+2341*y**6+1271*y**7-561*y**8+45*y**9))-35*MBhat**4*(
        -1+y)*(-1800*(-1+y)**8+2608*r**14*(-1+y)**8+r**2*(-1+y)**6*(12441
        -6120*logy*(-1+y)**2-31002*y+21621*y**2+636*y**3-2697*y**4-1572*y**5
        +533*y**6+98*y**7+2*y**8)+3*r**4*(-1+y)**4*(-10881+5760*logy*(-1+y)**4
        +49284*y-85446*y**2+67024*y**3-18786*y**4-2036*y**5+122*y**6+224*y**7
        +15*y**8)+6*r**6*(-1+y)**3*(-4680-1050*logy*(-1+y)**5+24450*y-51525*y**2
        +54085*y**3-24975*y**4+853*y**5+2405*y**6-1089*y**7+176*y**8)+2*r**8*(
        -1+y)**2*(-5720+1950*logy*(-1+y)**6+36270*y-96525*y**2+143910*y**3
        -102540*y**4+26484*y**5+8090*y**6-8058*y**7+1869*y**8-60*y**9)+9*r**12*(
        -989+7912*y-27692*y**2+57580*y**3-74898*y**4+63336*y**5-34524*y**6
        +11496*y**7-2049*y**8+136*y**9)+r**10*(11655-93240*y+326340*y**2-687300*y**3
        +902940*y**4-750054*y**5+390102*y**6-112512*y**7+9609*y**8+3090*y**9
        -630*y**10))-7*(-600*(-1+y)**9+75*r**16*(-1+y)**9+r**2*(-1+y)**7*(12135
        -1800*logy*(-1+y)**2-26070*y+14835*y**2-600*y**3-150*y**4-60*y**5
        +1365*y**6-1782*y**7+339*y**8+8*y**9)+5*r**10*(-1+y)**2*(-14445+900*logy*(
        -1+y)**7+100215*y-297495*y**2+489525*y**3-481650*y**4+282690*y**5
        -85491*y**6+11417*y**7-3601*y**8+875*y**9)+5*r**6*(-1+y)**4*(81465
        +89100*logy*(-1+y)**5-496425*y+1215600*y**2-1512600*y**3+979050*y**4
        -284910*y**5+14728*y**6+3760*y**7-2237*y**8+1065*y**9)+r**12*(-1+y)*(2115
        +4500*logy*(-1+y)**8-12420*y+25470*y**2-8940*y**3-51825*y**4+104460*y**5
        -61885*y**6+28652*y**7-12856*y**8+2376*y**9-27*y**10)+5*r**8*(-1+y)**3*(102225
        +69300*logy*(-1+y)**6-544050*y+1152225*y**2-1189800*y**3+545850*y**4
        -10440*y**5-66086*y**6+11920*y**7-3213*y**8+1214*y**9-5*y**10)+5*r**4*(
        -1+y)**5*(-38031+6480*logy*(-1+y)**4+158604*y-250866*y**2+180204*y**3
        -51531*y**4+1296*y**5-159*y**6+1292*y**7-1342*y**8+428*y**9+9*y**10)
        +5*r**14*(-81+729*y-2916*y**2+6804*y**3-10206*y**4+10206*y**5-4424*y**6
        +1336*y**7-189*y**8-39*y**9+12*y**10))-MBhat*(25200*(-1+y)**9+4420*r**16*(
        -1+y)**9+r**2*(-1+y)**7*(-431840+88200*logy*(-1+y)**2+951880*y-564140*y**2
        +29400*y**3+7350*y**4-40320*y**5+42077*y**6+8816*y**7-3793*y**8-270*y**9)
        +70*r**8*(-1+y)**3*(-110350-31800*logy*(-1+y)**6+630300*y-1480350*y**2
        +1814800*y**3-1202100*y**4+382422*y**5-37877*y**6+4494*y**7-1881*y**8
        +242*y**9)+70*r**6*(-1+y)**4*(-27494-120120*logy*(-1+y)**5+257590*y
        -815480*y**2+1215880*y**3-908240*y**4+302077*y**5-23546*y**6+2035*y**7
        -1496*y**8-105*y**9+3*y**10)-35*r**4*(-1+y)**5*(-162396+35280*logy*(
        -1+y)**4+684864*y-1097856*y**2+802464*y**3-235896*y**4+5466*y**5
        +5933*y**6-3968*y**7+354*y**8+454*y**9+5*y**10)+35*r**10*(-1+y)**2*(
        -16480+4200*logy*(-1+y)**7+111160*y-318780*y**2+501900*y**3-465150*y**4
        +225234*y**5-63107*y**6+17147*y**7-3003*y**8+453*y**9+6*y**10)+210*r**14*(66
        -594*y+2376*y**2-5544*y**3+8316*y**4-10101*y**5+7721*y**6-3924*y**7
        +1296*y**8-249*y**9+21*y**10)+7*r**12*(6380-57420*y+229680*y**2-535920*y**3
        +803880*y**4-659550*y**5+268251*y**6-59409*y**7+1236*y**8+3466*y**9
        -639*y**10+45*y**11))-5*MBhat**2*(-12600*(-1+y)**9+3586*r**16*(-1+y)**9
        +28*r**6*(-1+y)**4*(-38368+51540*logy*(-1+y)**5+140300*y-151750*y**2
        -20050*y**3+138560*y**4-76603*y**5+6554*y**6-85*y**7+257*y**8+15*y**9)
        +r**2*(-1+y)**7*(172114-47040*logy*(-1+y)**2-391268*y+242674*y**2
        -15680*y**3+10045*y**4-5936*y**5-14070*y**6+1878*y**7+633*y**8+30*y**9)
        +14*r**8*(-1+y)**3*(34065-2340*logy*(-1+y)**6-206730*y+523845*y**2
        -710160*y**3+548355*y**4-211200*y**5+24287*y**6+1050*y**7-1644*y**8
        +352*y**9)+7*r**4*(-1+y)**5*(-243088+66000*logy*(-1+y)**4+1038352*y
        -1689528*y**2+1258352*y**3-382913*y**4+18474*y**5+1117*y**6-4132*y**7
        +1821*y**8+106*y**9-y**10)-7*r**10*(-1+y)**2*(-3510-5160*logy*(-1
        +y)**7+29730*y-107250*y**2+214870*y**3-298465*y**4+198141*y**5-65734*y**6
        +8584*y**7+3075*y**8-1071*y**9+30*y**10)+14*r**14*(1076-9684*y+38736*y**2
        -90384*y**3+144501*y**4-153405*y**5+110796*y**6-53604*y**7+16461*y**8
        -2849*y**9+204*y**10)-7*r**12*(3848-34632*y+138528*y**2-323232*y**3
        +532143*y**4-592713*y**5+433491*y**6-213153*y**7+66087*y**8-10913*y**9
        +483*y**10+63*y**11))-5*MBhat**3*(16800*(-1+y)**9+9882*r**16*(-1+y)**9
        +14*r**6*(-1+y)**4*(78143-27300*logy*(-1+y)**5-363415*y+658580*y**2
        -566930*y**3+212635*y**4-20573*y**5+4922*y**6-1264*y**7+62*y**8)+r**2*(
        -1+y)**7*(-169944+62160*logy*(-1+y)**2+402048*y-263184*y**2+12320*y**3
        -8323*y**4+27356*y**5+1673*y**6-2284*y**7-220*y**8-2*y**9)-7*r**4*(
        -1+y)**5*(-152412+53280*logy*(-1+y)**4+662928*y-1100952*y**2+839628*y**3
        -265337*y**4+22114*y**5-11030*y**6+2724*y**7+411*y**8+6*y**9)+14*r**8*(
        -1+y)**3*(16625+4140*logy*(-1+y)**6-95610*y+226605*y**2-282140*y**3
        +168845*y**4-41742*y**5-3575*y**6+9628*y**7-3360*y**8+444*y**9)-7*r**10*(
        -1+y)**2*(-12480+87360*y-262080*y**2+457800*y**3-409805*y**4+201265*y**5
        -22439*y**6-33737*y**7+19336*y**8-3950*y**9+210*y**10)+14*r**14*(2861
        -25749*y+102996*y**2-246274*y**3+379316*y**4-392994*y**5+275688*y**6
        -127932*y**7+36873*y**8-5727*y**9+326*y**10)-7*r**12*(9292-83628*y
        +334512*y**2-810428*y**3+1247421*y**4-1269315*y**5+862926*y**6-369036*y**7
        +81639*y**8+79*y**9-4074*y**10+612*y**11)))/(2100*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (12600*(-1+y)**10-945*r**18*(-1+y)**10+3*r**2*(-1+y)**8*(-100725+8400*logy*(
        -1+y)**2+209850*y-113325*y**2+2800*y**3+700*y**4+280*y**5+140*y**6
        -9816*y**7+12017*y**8-2146*y**9+85*y**10)-210*r**6*(-1+y)**5*(249090
        +146700*logy*(-1+y)**5-1392150*y+3151050*y**2-3640050*y**3+2186775*y**4
        -584055*y**5+24450*y**6+3738*y**7+929*y**8-715*y**9+590*y**10)-84*r**12*(
        -1+y)**2*(31800+1800*logy*(-1+y)**8-252600*y+876900*y**2-1737000*y**3
        +2146050*y**4-1691640*y**5+829020*y**6-213552*y**7+22331*y**8-5026*y**9
        +1067*y**10)-420*r**8*(-1+y)**4*(22004+154080*logy*(-1+y)**6+22056*y
        -517380*y**2+1460240*y**3-1865580*y**4+1208472*y**5-355492*y**6+22480*y**7
        +2966*y**8-451*y**9+440*y**10-5*y**11)+42*r**14*(-1+y)*(2410-2700*logy*(
        -1+y)**9-18990*y+63810*y**2-116490*y**3+118035*y**4-49995*y**5-23370*y**6
        +17182*y**7-10663*y**8+5977*y**9-1078*y**10+12*y**11)-28*r**4*(-1+y)**6*(
        -259240+35400*logy*(-1+y)**4+1072360*y-1679340*y**2+1190360*y**3
        -332990*y**4+7080*y**5+1180*y**6-688*y**7+3777*y**8-4328*y**9+1457*y**10
        +12*y**11)+70*r**10*(-1+y)**3*(675080-316500*logy*(-1+y)**7-4409060*y
        +12119430*y**2-17983550*y**3+15214175*y**4-6913005*y**5+1196585*y**6
        +135025*y**7-40491*y**8+5197*y**9-2101*y**10+15*y**11)-20*r**16*(370
        -3700*y+16650*y**2-44400*y**3+77700*y**4-93240*y**5+77700*y**6-29520*y**7
        +7980*y**8-1135*y**9-134*y**10+45*y**11)-210*MBhat**8*(-1+y)**4*(-60*(
        -1+y)**6+105*r**10*(-1+y)**6+r**2*(-1+y)**4*(115-120*logy*(-1+y)**2
        -350*y+295*y**2+140*y**3-325*y**4+122*y**5+5*y**6)+2*r**6*(-1+y)*(150
        +90*logy*(-1+y)**5-840*y+1905*y**2-2135*y**3+1600*y**4-869*y**5
        +207*y**6)+2*r**4*(-1+y)**2*(60-240*y+360*y**2-420*y**3+665*y**4
        -664*y**5+215*y**6)+2*r**8*(10-60*y+150*y**2+140*y**3-260*y**4+201*y**5
        -86*y**6+15*y**7))-280*MBhat**7*(-1+y)**3*(360*(-1+y)**7+714*r**12*(
        -1+y)**7-r**2*(-1+y)**5*(1519-1020*logy*(-1+y)**2-4058*y+3049*y**2
        +560*y**3-1420*y**4+275*y**5+84*y**6+3*y**7)+2*r**4*(-1+y)**3*(-365
        -300*logy*(-1+y)**4+1160*y-1140*y**2+1045*y**3-2635*y**4+3090*y**5
        -1118*y**6+107*y**7)+3*r**6*(-1+y)**2*(-740+600*logy*(-1+y)**5+3100*y
        -4700*y**2+2990*y**3-2320*y**4+2870*y**5-1449*y**6+267*y**7+6*y**8)
        +5*r**10*(313-2191*y+6573*y**2-11933*y**3+12860*y**4-8589*y**5+3451*y**6
        -745*y**7+63*y**8)-r**8*(520-3640*y+10920*y**2-23090*y**3+27260*y**4
        -15066*y**5+2341*y**6+1271*y**7-561*y**8+45*y**9))-140*MBhat**6*(-1
        +y)**2*(-2520*(-1+y)**8+3912*r**14*(-1+y)**8+3*r**2*(-1+y)**6*(5722
        -2880*logy*(-1+y)**2-14324*y+10042*y**2+378*y**3-1486*y**4-548*y**5
        +199*y**6+44*y**7+y**8)+2*r**4*(-1+y)**4*(-22201+12180*logy*(-1+y)**4
        +100984*y-175836*y**2+138299*y**3-37931*y**4-6138*y**5+1754*y**6+31*y**7
        +30*y**8)+12*r**6*(-1+y)**3*(-3210-810*logy*(-1+y)**5+16860*y-35745*y**2
        +37740*y**3-17355*y**4+121*y**5+2176*y**6-929*y**7+132*y**8)+12*r**8*(
        -1+y)**2*(-1380+420*logy*(-1+y)**6+8700*y-23010*y**2+34145*y**3
        -24140*y**4+5785*y**5+2402*y**6-2095*y**7+468*y**8-15*y**9)+54*r**12*(
        -249+1992*y-6972*y**2+14493*y**3-18847*y**4+15932*y**5-8680*y**6
        +2888*y**7-514*y**8+34*y**9)+r**10*(17330-138640*y+485240*y**2-1022410*y**3
        +1346180*y**4-1121278*y**5+584309*y**6-169024*y**7+14648*y**8+4590*y**9
        -945*y**10))+60*MBhat**5*(-1+y)*(-11760*(-1+y)**9-9882*r**16*(-1+y)**9
        +14*r**6*(-1+y)**4*(-53593+18420*logy*(-1+y)**5+249545*y-453040*y**2
        +390990*y**3-145935*y**4+10251*y**5+1061*y**6-979*y**7+188*y**8)+r**2*(
        -1+y)**7*(114966-44520*logy*(-1+y)**2-274452*y+181746*y**2-6440*y**3
        +7*y**4-15428*y**5-1407*y**6+1248*y**7+150*y**8+2*y**9)+7*r**4*(-1
        +y)**5*(-105422+38760*logy*(-1+y)**4+460448*y-768192*y**2+588748*y**3
        -185787*y**4+12130*y**5-4642*y**6+1196*y**7+171*y**8+6*y**9)-14*r**8*(
        -1+y)**3*(11365+2100*logy*(-1+y)**6-66090*y+158925*y**2-202100*y**3
        +122765*y**4-25176*y**5-11389*y**6+12148*y**7-3786*y**8+438*y**9)+7*r**10*(
        -1+y)**2*(-11750+82250*y-246750*y**2+432250*y**3-402585*y**4+203257*y**5
        -23719*y**6-33633*y**7+19420*y**8-3950*y**9+210*y**10)-14*r**14*(3031
        -27279*y+109116*y**2-260554*y**3+400736*y**4-414414*y**5+289968*y**6
        -134052*y**7+38403*y**8-5897*y**9+326*y**10)+7*r**12*(10178-91602*y
        +366408*y**2-884852*y**3+1368939*y**4-1405629*y**5+964746*y**6-420156*y**7
        +98199*y**8-3021*y**9-3822*y**10+612*y**11))-6*MBhat**2*(-58800*(-1
        +y)**10+5705*r**18*(-1+y)**10+560*r**8*(-1+y)**4*(84432+49740*logy*(
        -1+y)**6-456852*y+992910*y**2-1075180*y**3+557685*y**4-73212*y**5
        -36594*y**6+6404*y**7+216*y**8-14*y**9+5*y**10)+r**2*(-1+y)**8*(1138255
        -201600*logy*(-1+y)**2-2478110*y+1440655*y**2-67200*y**3-16800*y**4
        +53088*y**5-11816*y**6-58820*y**7+511*y**8+2062*y**9+135*y**10)+28*r**4*(
        -1+y)**6*(-747050+154500*logy*(-1+y)**4+3142700*y-5023050*y**2+3657700*y**3
        -1068925*y**4+29388*y**5+8568*y**6+1448*y**7-4267*y**8+1548*y**9+260*y**10)
        +70*r**10*(-1+y)**3*(-62140+300*logy*(-1+y)**7+434680*y-1302990*y**2
        +2169550*y**3-2166925*y**4+1318287*y**5-419355*y**6+44351*y**7-1366*y**8
        -1246*y**9+264*y**10)+28*r**12*(-1+y)**2*(3290+6900*logy*(-1+y)**8
        -19420*y+40370*y**2-16340*y**3-76175*y**4+255364*y**5-183632*y**6
        +68188*y**7-16607*y**8+258*y**9+474*y**10-15*y**11)+140*r**6*(-1+y)**5*(228044
        +307320*logy*(-1+y)**5-1447540*y+3663380*y**2-4687780*y**3+3112190*y**4
        -928852*y**5+50579*y**6+7169*y**7+362*y**8+680*y**9+5*y**10-y**11)
        +40*r**16*(-746+7460*y-33570*y**2+89520*y**3-156660*y**4+203616*y**5
        -182112*y**6+114090*y**7-49095*y**8+13715*y**9-2213*y**10+153*y**11)
        -28*r**14*(-2300+23000*y-103500*y**2+276000*y**3-483000*y**4+658098*y**5
        -648095*y**6+424860*y**7-194410*y**8+59030*y**9-10523*y**10+840*y**11))
        -4*MBhat*(25200*(-1+y)**10+2040*r**18*(-1+y)**10+r**2*(-1+y)**8*(-553600
        +71400*logy*(-1+y)**2+1178600*y-660700*y**2+23800*y**3+5950*y**4
        +2380*y**5-39739*y**6+33224*y**7+12155*y**8-2826*y**9-84*y**10)+35*r**10*(
        -1+y)**3*(630920-159000*logy*(-1+y)**7-4257440*y+12215820*y**2-19246700*y**3
        +17855450*y**4-9600270*y**5+2615039*y**6-225587*y**7+3363*y**8-2561*y**9
        +446*y**10)-105*r**6*(-1+y)**5*(427892+340560*logy*(-1+y)**5-2480020*y
        +5811440*y**2-6946640*y**3+4324720*y**4-1205504*y**5+57491*y**6+7997*y**7
        +242*y**8+910*y**9+277*y**10-5*y**11)-140*r**8*(-1+y)**4*(250211+320820*logy*(
        -1+y)**6-1180446*y+1988655*y**2-1047440*y**3-818520*y**4+1289868*y**5
        -533926*y**6+47205*y**7+2364*y**8+1144*y**9+18*y**10-3*y**11)+42*r**12*(
        -1+y)**2*(18300+3300*logy*(-1+y)**8-143100*y+487650*y**2-944500*y**3
        +1134425*y**4-861340*y**5+355372*y**6-84599*y**7+16717*y**8-1362*y**9
        +174*y**10+3*y**11)-10*r**4*(-1+y)**6*(-1200212+207060*logy*(-1+y)**4
        +5007908*y-7925982*y**2+5698108*y**3-1631587*y**4+41412*y**5+3332*y**6
        +13243*y**7-9579*y**8-454*y**9+1768*y**10+27*y**11)+20*r**16*(-383
        +3830*y-17235*y**2+45960*y**3-80430*y**4+96516*y**5-101262*y**6+67440*y**7
        -30330*y**8+9050*y**9-1598*y**10+126*y**11)+35*r**14*(-1532+15320*y
        -68940*y**2+183840*y**3-321720*y**4+386064*y**5-279525*y**6+109650*y**7
        -25545*y**8+1640*y**9+925*y**10-186*y**11+9*y**12))-4*MBhat**3*(176400*(
        -1+y)**10+24705*r**18*(-1+y)**10-210*r**8*(-1+y)**4*(224648+63360*logy*(
        -1+y)**6-1284528*y+3021240*y**2-3711520*y**3+2469160*y**4-789790*y**5
        +69941*y**6+2078*y**7-939*y**8+90*y**9)-105*r**6*(-1+y)**5*(53142+540360*logy*(
        -1+y)**5-806070*y+2963040*y**2-4764240*y**3+3735405*y**4-1290751*y**5
        +97694*y**6+3308*y**7+2739*y**8+15*y**9+6*y**10)-3*r**2*(-1+y)**8*(958645
        -222600*logy*(-1+y)**2-2139890*y+1292545*y**2-74200*y**3+12915*y**4
        +27482*y**5-62524*y**6-17778*y**7+3990*y**8+750*y**9+25*y**10)+35*r**10*(
        -1+y)**3*(-85210+9000*logy*(-1+y)**7+587470*y-1730910*y**2+2821850*y**3
        -2802665*y**4+1481379*y**5-402728*y**6+40784*y**7+27249*y**8-9383*y**9
        +1064*y**10)-5*r**4*(-1+y)**6*(-8367360+2049600*logy*(-1+y)**4+35519040*y
        -57377760*y**2+42351040*y**3-12643996*y**4+395010*y**5+156737*y**6-78854*y**7
        +6786*y**8+10842*y**9+297*y**10-6*y**11)-21*r**12*(-1+y)**2*(32640
        -261120*y+913920*y**2-1827840*y**3+2491980*y**4-1878806*y**5+888133*y**6
        -217598*y**7-9104*y**8+19630*y**9-4205*y**10+210*y**11)+40*r**16*(-3104
        +31040*y-139680*y**2+372480*y**3-677880*y**4+846426*y**5-742938*y**6
        +456810*y**7-191475*y**8+51515*y**9-7841*y**10+489*y**11)-105*r**14*(
        -2490+24900*y-112050*y**2+298800*y**3-557239*y**4+713396*y**5-630103*y**6
        +392718*y**7-165915*y**8+43580*y**9-5623*y**10-42*y**11+68*y**12))
        -MBhat**4*(-882000*(-1+y)**10+223125*r**18*(-1+y)**10+280*r**4*(-1+y)**6*(
        -430215+126300*logy*(-1+y)**4+1847160*y-3023340*y**2+2268724*y**3
        -698626*y**4+36141*y**5-1489*y**6-2594*y**7+1602*y**8+117*y**9)+210*r**6*(
        -1+y)**5*(-385312+463440*logy*(-1+y)**5+1463120*y-1767640*y**2+226340*y**3
        +1036055*y**4-641995*y**5+52286*y**6+7034*y**7-973*y**8+165*y**9)+280*r**8*(
        -1+y)**4*(102950-7200*logy*(-1+y)**6-624900*y+1583850*y**2-2146720*y**3
        +1672800*y**4-670131*y**5+101284*y**6-12309*y**7+30*y**8+196*y**9)
        +r**2*(-1+y)**8*(11481675-3444000*logy*(-1+y)**2-26407350*y+16647675*y**2
        -990920*y**3+525070*y**4-631316*y**5-679518*y**6+45024*y**7+36555*y**8
        +2490*y**9+15*y**10)+140*r**10*(-1+y)**3*(-4000+13950*logy*(-1+y)**7
        +14050*y+6675*y**2-90805*y**3+395740*y**4-421782*y**5+293068*y**6
        -170092*y**7+64518*y**8-14642*y**9+1520*y**10)-84*r**12*(-1+y)**2*(16950
        -135600*y+474600*y**2-1037480*y**3+1058560*y**4-639074*y**5+37722*y**6
        +255408*y**7-201621*y**8+72860*y**9-12415*y**10+680*y**11)+20*r**16*(
        -50776+507760*y-2284920*y**2+6197280*y**3-11050620*y**4+13586706*y**5
        -11702712*y**6+7013370*y**7-2833830*y**8+720325*y**9-99208*y**10+4941*y**11)
        -70*r**14*(-25360+253600*y-1141200*y**2+3141276*y**3-5585325*y**4+6750258*y**5
        -5654025*y**6+3185148*y**7-1100925*y**8+160060*y**9+30887*y**10-16350*y**11
        +1956*y**12)))/(6300*r**2*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr11(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r11 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-20*(-1+y)**6-255*r**10*(-1+y)**6+r**2*(-1+y)**4*(275-120*logy*(
        -1+y)**2-670*y+455*y**2+300*y**3-565*y**4+186*y**5+5*y**6)+2*r**4*(
        -1+y)**2*(-120+480*y-720*y**2+180*y**3+805*y**4-920*y**5+283*y**6)
        +2*r**6*(-1+y)*(290+90*logy*(-1+y)**5-1540*y+3305*y**2-4095*y**3
        +3300*y**4-1641*y**5+351*y**6)-2*r**8*(-410+2460*y-6150*y**2+8700*y**3
        -7420*y**4+3711*y**5-986*y**6+105*y**7))/(30*r**2*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-120*(-1+y)**7-459*r**12*(-1+y)**7+r**2*(-1+y)**5*(1159-120*logy*(
        -1+y)**2-2438*y+1339*y**2-40*y**3+1145*y**4-1684*y**5+405*y**6+30*y**7)
        +6*r**6*(-1+y)**2*(-680+90*logy*(-1+y)**5+3310*y-6395*y**2+6095*y**3
        -3605*y**4+2623*y**5-2017*y**6+561*y**7)+r**10*(-2825+19775*y-59325*y**2
        +98875*y**3-104065*y**4+70869*y**5-29603*y**6+6749*y**7-630*y**8)+2*r**8*(
        -1+y)*(-1880+810*logy*(-1+y)**6+12090*y-32655*y**2+47590*y**3-43920*y**4
        +27648*y**5-11438*y**6+2286*y**7-81*y**8)+r**4*(-1+y)**3*(-3725+240*logy*(
        -1+y)**4+15140*y-23190*y**2+15940*y**3-6910*y**4+8388*y**5-8078*y**6
        +2276*y**7+15*y**8)-6*MBhat**2*(-1+y)*(20*(-1+y)**6+255*r**10*(-1
        +y)**6-r**2*(-1+y)**4*(275-120*logy*(-1+y)**2-670*y+455*y**2+300*y**3
        -565*y**4+186*y**5+5*y**6)-2*r**4*(-1+y)**2*(-120+480*y-720*y**2
        +180*y**3+805*y**4-920*y**5+283*y**6)-2*r**6*(-1+y)*(290+90*logy*(
        -1+y)**5-1540*y+3305*y**2-4095*y**3+3300*y**4-1641*y**5+351*y**6)
        +2*r**8*(-410+2460*y-6150*y**2+8700*y**3-7420*y**4+3711*y**5-986*y**6
        +105*y**7))-2*MBhat*(-120*(-1+y)**7+1050*r**12*(-1+y)**7+2*r**4*(-1
        +y)**3*(-365+420*logy*(-1+y)**4+1880*y-3660*y**2+1795*y**3+3485*y**4
        -4866*y**5+1858*y**6-199*y**7)+r**2*(-1+y)**5*(835-300*logy*(-1+y)**2
        -1970*y+1285*y**2+1280*y**3-2020*y**4+419*y**5+84*y**6+3*y**7)-3*r**6*(
        -1+y)**2*(-340+600*logy*(-1+y)**5+1100*y-700*y**2+30*y**3-1420*y**4
        +2354*y**5-889*y**6-21*y**7+54*y**8)+5*r**10*(763-5341*y+16023*y**2
        -27743*y**3+29840*y**4-20055*y**5+8113*y**6-1771*y**7+153*y**8)-r**8*(3800
        -26600*y+79800*y**2-140590*y**3+159640*y**4-117234*y**5+52175*y**6
        -11363*y**7+57*y**8+315*y**9)))/(180*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-300*(-1+y)**8-510*r**14*(-1+y)**8+3*r**2*(-1+y)**6*(-940+600*logy*(
        -1+y)**2+2480*y-1840*y**2+200*y**3+50*y**4+621*y**5-767*y**6+111*y**7
        +15*y**8)+30*r**8*(-1+y)**2*(1290+120*logy*(-1+y)**6-7620*y+18690*y**2
        -24320*y**3+17640*y**4-7399*y**5+2501*y**6-1223*y**7+291*y**8)+10*r**6*(
        -1+y)**3*(-95+4980*logy*(-1+y)**5-4505*y+21460*y**2-38060*y**3+31480*y**4
        -12023*y**5+3307*y**6-2612*y**7+814*y**8)+r**12*(5020-40160*y+140560*y**2
        -281120*y**3+351400*y**4-293513*y**5+165364*y**6-58844*y**7+11608*y**8
        -945*y**9)+5*r**10*(-1+y)*(1700+1080*logy*(-1+y)**7-12980*y+42720*y**2
        -78760*y**3+88210*y**4-66225*y**5+35102*y**6-12530*y**7+2214*y**8-81*y**9)
        +5*r**4*(-1+y)**4*(-6508+2640*logy*(-1+y)**4+28672*y-48288*y**2+37472*y**3
        -12008*y**4-369*y**5+3038*y**6-2846*y**7+750*y**8+15*y**9)-15*MBhat**4*(
        -1+y)**2*(20*(-1+y)**6+255*r**10*(-1+y)**6-r**2*(-1+y)**4*(275-120*logy*(
        -1+y)**2-670*y+455*y**2+300*y**3-565*y**4+186*y**5+5*y**6)-2*r**4*(
        -1+y)**2*(-120+480*y-720*y**2+180*y**3+805*y**4-920*y**5+283*y**6)
        -2*r**6*(-1+y)*(290+90*logy*(-1+y)**5-1540*y+3305*y**2-4095*y**3
        +3300*y**4-1641*y**5+351*y**6)+2*r**8*(-410+2460*y-6150*y**2+8700*y**3
        -7420*y**4+3711*y**5-986*y**6+105*y**7))-10*MBhat**3*(-1+y)*(-120*(
        -1+y)**7+1050*r**12*(-1+y)**7+2*r**4*(-1+y)**3*(-365+420*logy*(-1
        +y)**4+1880*y-3660*y**2+1795*y**3+3485*y**4-4866*y**5+1858*y**6-199*y**7)
        +r**2*(-1+y)**5*(835-300*logy*(-1+y)**2-1970*y+1285*y**2+1280*y**3
        -2020*y**4+419*y**5+84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-340+600*logy*(
        -1+y)**5+1100*y-700*y**2+30*y**3-1420*y**4+2354*y**5-889*y**6-21*y**7
        +54*y**8)+5*r**10*(763-5341*y+16023*y**2-27743*y**3+29840*y**4-20055*y**5
        +8113*y**6-1771*y**7+153*y**8)-r**8*(3800-26600*y+79800*y**2-140590*y**3
        +159640*y**4-117234*y**5+52175*y**6-11363*y**7+57*y**8+315*y**9))-10*MBhat*(
        -120*(-1+y)**8+250*r**14*(-1+y)**8+r**6*(-1+y)**3*(-13730+6240*logy*(
        -1+y)**5+62410*y-109220*y**2+88420*y**3-30980*y**4+6826*y**5-6887*y**6
        +2659*y**7-290*y**8)+r**2*(-1+y)**6*(-581+420*logy*(-1+y)**2+1582*y
        -1211*y**2+140*y**3+686*y**4-652*y**5-127*y**6+74*y**7+5*y**8)+3*r**4*(
        -1+y)**4*(-3365+1620*logy*(-1+y)**4+15080*y-25860*y**2+20480*y**3
        -7285*y**4+2008*y**5-1438*y**6+292*y**7+40*y**8)-2*r**8*(-1+y)**2*(2350
        +1020*logy*(-1+y)**6-13080*y+29640*y**2-34420*y**3+24000*y**4-12111*y**5
        +5511*y**6-1392*y**7+105*y**8+27*y**9)+r**12*(-1469+11752*y-41132*y**2
        +82264*y**3-106961*y**4+93010*y**5-52976*y**6+18808*y**7-3728*y**8
        +306*y**9)-3*r**10*(-995+7960*y-27860*y**2+55720*y**3-72755*y**4+65794*y**5
        -41156*y**6+16964*y**7-4126*y**8+454*y**9))-5*MBhat**2*(360*(-1+y)**8
        +1384*r**14*(-1+y)**8-r**2*(-1+y)**6*(292+240*logy*(-1+y)**2-344*y
        -68*y**2+1682*y**3-664*y**4-1684*y**5+469*y**6+64*y**7+y**8)-12*r**6*(
        -1+y)**3*(-1470-240*logy*(-1+y)**5+7590*y-15780*y**2+16245*y**3
        -8230*y**4+1790*y**5-365*y**6+44*y**7+14*y**8)-2*r**4*(-1+y)**4*(-5758
        +3900*logy*(-1+y)**4+26932*y-48198*y**2+37877*y**3-8483*y**4-2346*y**5
        -856*y**6+601*y**7+15*y**8)+18*r**12*(-306+2448*y-8568*y**2+17595*y**3
        -23027*y**4+19684*y**5-10892*y**6+3712*y**7-689*y**8+50*y**9)-4*r**8*(
        -1+y)**2*(-1320+690*logy*(-1+y)**6+8610*y-23595*y**2+32315*y**3
        -26790*y**4+16029*y**5-8414*y**6+3291*y**7-831*y**8+105*y**9)+r**10*(4900
        -39200*y+137200*y**2-291530*y**3+401020*y**4-355106*y**5+193753*y**6
        -55328*y**7+1246*y**8+3810*y**9-765*y**10)))/(450*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-1400*(-1+y)**9-1275*r**16*(-1+y)**9+r**2*(-1+y)**7*(-58575+21000*logy*(
        -1+y)**2+138150*y-90075*y**2+7000*y**3+1750*y**4+700*y**5+5523*y**6
        -5482*y**7-75*y**8+104*y**9)+35*r**6*(-1+y)**4*(31185+33900*logy*(
        -1+y)**5-189825*y+464400*y**2-577400*y**3+373450*y**4-108590*y**5
        +5544*y**6+2544*y**7-2781*y**8+1065*y**9)+35*r**10*(-1+y)**2*(-11565
        +900*logy*(-1+y)**7+80055*y-237015*y**2+388725*y**3-380850*y**4+222210*y**5
        -76179*y**6+18217*y**7-6137*y**8+1219*y**9)+7*r**12*(-1+y)*(-4525
        +4500*logy*(-1+y)**8+40700*y-160450*y**2+362900*y**3-516625*y**4
        +476300*y**5-305709*y**6+142380*y**7-45216*y**8+7128*y**9-243*y**10)
        +35*r**8*(-1+y)**3*(36025+33300*logy*(-1+y)**6-182850*y+357225*y**2
        -309800*y**3+65850*y**4+73560*y**5-47334*y**6+8784*y**7-3917*y**8+1342*y**9
        -5*y**10)+35*r**4*(-1+y)**5*(-14271+6480*logy*(-1+y)**4+63564*y-108306*y**2
        +85164*y**3-27771*y**4+1296*y**5-63*y**6+1228*y**7-1214*y**8+316*y**9
        +9*y**10)-35*r**14*(521-4689*y+18756*y**2-43764*y**3+65646*y**4-65646*y**5
        +45416*y**6-21784*y**7+6741*y**8-1169*y**9+84*y**10)-70*MBhat**6*(-1
        +y)**3*(20*(-1+y)**6+255*r**10*(-1+y)**6-r**2*(-1+y)**4*(275-120*logy*(
        -1+y)**2-670*y+455*y**2+300*y**3-565*y**4+186*y**5+5*y**6)-2*r**4*(
        -1+y)**2*(-120+480*y-720*y**2+180*y**3+805*y**4-920*y**5+283*y**6)
        -2*r**6*(-1+y)*(290+90*logy*(-1+y)**5-1540*y+3305*y**2-4095*y**3
        +3300*y**4-1641*y**5+351*y**6)+2*r**8*(-410+2460*y-6150*y**2+8700*y**3
        -7420*y**4+3711*y**5-986*y**6+105*y**7))-70*MBhat**5*(-1+y)**2*(-120*(
        -1+y)**7+1050*r**12*(-1+y)**7+2*r**4*(-1+y)**3*(-365+420*logy*(-1
        +y)**4+1880*y-3660*y**2+1795*y**3+3485*y**4-4866*y**5+1858*y**6-199*y**7)
        +r**2*(-1+y)**5*(835-300*logy*(-1+y)**2-1970*y+1285*y**2+1280*y**3
        -2020*y**4+419*y**5+84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-340+600*logy*(
        -1+y)**5+1100*y-700*y**2+30*y**3-1420*y**4+2354*y**5-889*y**6-21*y**7
        +54*y**8)+5*r**10*(763-5341*y+16023*y**2-27743*y**3+29840*y**4-20055*y**5
        +8113*y**6-1771*y**7+153*y**8)-r**8*(3800-26600*y+79800*y**2-140590*y**3
        +159640*y**4-117234*y**5+52175*y**6-11363*y**7+57*y**8+315*y**9))-35*MBhat**4*(
        -1+y)*(600*(-1+y)**8+2768*r**14*(-1+y)**8-r**2*(-1+y)**6*(-575+600*logy*(
        -1+y)**2+1750*y-1475*y**2+3404*y**3-2473*y**4-1684*y**5+533*y**6
        +98*y**7+2*y**8)-r**4*(-1+y)**4*(-19307+15360*logy*(-1+y)**4+92588*y
        -169602*y**2+135568*y**3-27022*y**4-17772*y**5+4654*y**6+128*y**7+45*y**8)
        -6*r**6*(-1+y)**3*(-5200-1050*logy*(-1+y)**5+27050*y-56725*y**2+58885*y**3
        -29315*y**4+4537*y**5+557*y**6-385*y**7+56*y**8)+9*r**12*(-1275+10200*y
        -35700*y**2+73236*y**3-95678*y**4+81592*y**5-44996*y**6+15256*y**7
        -2807*y**8+200*y**9)-2*r**8*(-1+y)**2*(-3400+1950*logy*(-1+y)**6+22350*y
        -61725*y**2+81670*y**3-63240*y**4+36468*y**5-22218*y**6+10878*y**7
        -3243*y**8+420*y**9)+r**10*(12625-101000*y+353500*y**2-741260*y**3
        +1004980*y**4-885146*y**5+487978*y**6-147008*y**7+9871*y**8+6990*y**9
        -1530*y**10))-MBhat*(-8400*(-1+y)**9+6500*r**16*(-1+y)**9+70*r**8*(
        -1+y)**3*(63650+21000*logy*(-1+y)**6-360900*y+839250*y**2-1014000*y**3
        +655500*y**4-202902*y**5+18901*y**6-3654*y**7+1809*y**8-234*y**9)+r**2*(
        -1+y)**7*(-258920+96600*logy*(-1+y)**2+614440*y-403820*y**2+32200*y**3
        +8050*y**4+27496*y**5-13293*y**6-14472*y**7+2169*y**8+270*y**9)-210*r**6*(
        -1+y)**4*(-6550-19800*logy*(-1+y)**5+52550*y-154600*y**2+220600*y**3
        -159800*y**4+51839*y**5-4366*y**6+1241*y**7-520*y**8-35*y**9+y**10)
        +35*r**4*(-1+y)**5*(-68804+31920*logy*(-1+y)**4+307136*y-524544*y**2
        +413536*y**3-135304*y**4+4986*y**5+5997*y**6-3504*y**7-246*y**8+454*y**9
        +5*y**10)+210*r**14*(266-2394*y+9576*y**2-22344*y**3+33516*y**4-34755*y**5
        +25207*y**6-12348*y**7+3852*y**8-683*y**9+51*y**10)-35*r**10*(-1+y)**2*(
        -15560+4200*logy*(-1+y)**7+104720*y-299460*y**2+469700*y**3-432950*y**4
        +253314*y**5-101019*y**6+31587*y**7-5619*y**8+453*y**9+54*y**10)+21*r**12*(
        -9020+81180*y-324720*y**2+757680*y**3-1136520*y**4+1174426*y**5-871065*y**6
        +461355*y**7-166060*y**8+36770*y**9-4131*y**10+105*y**11))+MBhat**2*(
        -21000*(-1+y)**9-19030*r**16*(-1+y)**9+420*r**6*(-1+y)**4*(-5904+10380*logy*(
        -1+y)**5+19140*y-12330*y**2-22270*y**3+37020*y**4-17257*y**5+1118*y**6
        -3*y**7+91*y**8+5*y**9)+70*r**8*(-1+y)**3*(28585-2340*logy*(-1+y)**6
        -173850*y+441645*y**2-600560*y**3+456915*y**4-189072*y**5+37439*y**6
        -6294*y**7+324*y**8+128*y**9)+r**2*(-1+y)**7*(-423510+168000*logy*(
        -1+y)**2+1015020*y-675510*y**2+56000*y**3+57785*y**4+14168*y**5-63126*y**6
        +1158*y**7+3165*y**8+150*y**9)+35*r**4*(-1+y)**5*(-111200+56400*logy*(
        -1+y)**4+501200*y-864600*y**2+689200*y**3-231505*y**4+18570*y**5
        +1005*y**6-5316*y**7+1821*y**8+106*y**9-y**10)+35*r**10*(-1+y)**2*(2150
        +5160*logy*(-1+y)**7-20210*y+78690*y**2-167270*y**3+176585*y**4-110781*y**5
        +48822*y**6-21336*y**7+6765*y**8-1545*y**9+210*y**10)-70*r**14*(1580
        -14220*y+56880*y**2-132720*y**3+205275*y**4-217539*y**5+157668*y**6
        -76572*y**7+23643*y**8-4127*y**9+300*y**10)+21*r**12*(7640-68760*y
        +275040*y**2-641760*y**3+1024375*y**4-1164897*y**5+922683*y**6-491897*y**7
        +166743*y**8-30977*y**9+1555*y**10+255*y**11))-5*MBhat**3*(-5600*(-1
        +y)**9+8262*r**16*(-1+y)**9-14*r**6*(-1+y)**4*(53087-19620*logy*(-1
        +y)**5-245815*y+442580*y**2-376930*y**3+140915*y**4-14677*y**5+5042*y**6
        -1744*y**7+182*y**8)+r**2*(-1+y)**7*(-56408+25200*logy*(-1+y)**2+138016*y
        -94208*y**2+15568*y**3+15603*y**4-20300*y**5-4697*y**6+2284*y**7+220*y**8
        +2*y**9)+7*r**4*(-1+y)**5*(-75180+44640*logy*(-1+y)**4+345360*y-607320*y**2
        +491340*y**3-164905*y**4+18338*y**5-11638*y**6+2628*y**7+411*y**8+6*y**9)
        +14*r**8*(-1+y)**3*(-15745-4140*logy*(-1+y)**6+90330*y-213405*y**2
        +262300*y**3-193245*y**4+95710*y**5-39281*y**6+10852*y**7-2024*y**8
        +148*y**9)+14*r**14*(2515-22635*y+90540*y**2-215390*y**3+333340*y**4
        -347886*y**5+246456*y**6-116004*y**7+34191*y**8-5529*y**9+346*y**10)
        -7*r**10*(-1+y)**2*(8400-58800*y+176400*y**2-274920*y**3+299485*y**4
        -255137*y**5+171175*y**6-84119*y**7+28336*y**8-5690*y**9+510*y**10)
        -7*r**12*(4804-43236*y+172944*y**2-426500*y**3+680883*y**4-707373*y**5
        +469986*y**6-179748*y**7+19377*y**8+14617*y**9-6654*y**10+900*y**11)))/(2100*r**2*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-4200*(-1+y)**10-2295*r**18*(-1+y)**10+r**2*(-1+y)**8*(-368905+109200*logy*(
        -1+y)**2+847010*y-532705*y**2+36400*y**3+9100*y**4+3640*y**5+1820*y**6
        +9416*y**7-5707*y**8-3114*y**9+105*y**10)+210*r**6*(-1+y)**5*(76554
        +55020*logy*(-1+y)**5-437790*y+1013130*y**2-1196530*y**3+735815*y**4
        -202183*y**5+9170*y**6+1930*y**7-23*y**8-835*y**9+510*y**10)+84*r**12*(
        -1+y)**2*(26420+1800*logy*(-1+y)**8-209560*y+726260*y**2-1435720*y**3
        +1769450*y**4-1390360*y**5+678380*y**6-198224*y**7+38287*y**8-9682*y**9
        +1599*y**10)+14*r**14*(-1+y)*(4490+8100*logy*(-1+y)**9-48510*y+230490*y**2
        -635010*y**3+1122615*y**4-1326735*y**5+1054590*y**6-599034*y**7+252381*y**8
        -72899*y**9+10386*y**10-324*y**11)+140*r**8*(-1+y)**4*(-7472+172560*logy*(
        -1+y)**6+217392*y-1061160*y**2+2277680*y**3-2571060*y**4+1546104*y**5
        -430244*y**6+25296*y**7+2538*y**8-1729*y**9+1320*y**10-15*y**11)-70*r**10*(
        -1+y)**3*(275360-158100*logy*(-1+y)**7-1769420*y+4754910*y**2-6818150*y**3
        +5434775*y**4-2154165*y**5+164705*y**6+141385*y**7-28683*y**8+6061*y**9
        -2293*y**10+15*y**11)+4*r**4*(-1+y)**6*(-994660+424200*logy*(-1+y)**4
        +4402840*y-7452660*y**2+5816840*y**3-1878410*y**4+84840*y**5+14140*y**6
        +6176*y**7+9771*y**8-14744*y**9+4451*y**10+156*y**11)-20*r**16*(-2242
        +22420*y-100890*y**2+269040*y**3-470820*y**4+564984*y**5-470820*y**6
        +277920*y**7-115980*y**8+31735*y**9-4906*y**10+315*y**11)-210*MBhat**8*(
        -1+y)**4*(20*(-1+y)**6+255*r**10*(-1+y)**6-r**2*(-1+y)**4*(275-120*logy*(
        -1+y)**2-670*y+455*y**2+300*y**3-565*y**4+186*y**5+5*y**6)-2*r**4*(
        -1+y)**2*(-120+480*y-720*y**2+180*y**3+805*y**4-920*y**5+283*y**6)
        -2*r**6*(-1+y)*(290+90*logy*(-1+y)**5-1540*y+3305*y**2-4095*y**3
        +3300*y**4-1641*y**5+351*y**6)+2*r**8*(-410+2460*y-6150*y**2+8700*y**3
        -7420*y**4+3711*y**5-986*y**6+105*y**7))-280*MBhat**7*(-1+y)**3*(-120*(
        -1+y)**7+1050*r**12*(-1+y)**7+2*r**4*(-1+y)**3*(-365+420*logy*(-1
        +y)**4+1880*y-3660*y**2+1795*y**3+3485*y**4-4866*y**5+1858*y**6-199*y**7)
        +r**2*(-1+y)**5*(835-300*logy*(-1+y)**2-1970*y+1285*y**2+1280*y**3
        -2020*y**4+419*y**5+84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-340+600*logy*(
        -1+y)**5+1100*y-700*y**2+30*y**3-1420*y**4+2354*y**5-889*y**6-21*y**7
        +54*y**8)+5*r**10*(763-5341*y+16023*y**2-27743*y**3+29840*y**4-20055*y**5
        +8113*y**6-1771*y**7+153*y**8)-r**8*(3800-26600*y+79800*y**2-140590*y**3
        +159640*y**4-117234*y**5+52175*y**6-11363*y**7+57*y**8+315*y**9))-140*MBhat**6*(
        -1+y)**2*(840*(-1+y)**8+4152*r**14*(-1+y)**8-r**2*(-1+y)**6*(-1442
        +960*logy*(-1+y)**2+3844*y-2882*y**2+5126*y**3-4282*y**4-1684*y**5
        +597*y**6+132*y**7+3*y**8)-2*r**4*(-1+y)**4*(-13549+11460*logy*(-1
        +y)**4+65656*y-121404*y**2+97691*y**3-18539*y**4-15426*y**5+5510*y**6
        -473*y**7+30*y**8)-12*r**6*(-1+y)**3*(-3730-810*logy*(-1+y)**5+19460*y
        -40945*y**2+42640*y**3-21085*y**4+2747*y**5+922*y**6-429*y**7+42*y**8)
        +54*r**12*(-323+2584*y-9044*y**2+18547*y**3-24217*y**4+20636*y**5
        -11368*y**6+3848*y**7-706*y**8+50*y**9)-4*r**8*(-1+y)**2*(-2080+1260*logy*(
        -1+y)**6+13740*y-38130*y**2+49355*y**3-36450*y**4+20439*y**5-13804*y**6
        +7587*y**7-2412*y**8+315*y**9)+r**10*(20350-162800*y+569800*y**2-1190990*y**3
        +1608940*y**4-1415186*y**5+782203*y**6-238688*y**7+18496*y**8+10170*y**9
        -2295*y**10))-60*MBhat**5*(-1+y)*(-3920*(-1+y)**9+8262*r**16*(-1+y)**9
        +42*r**6*(-1+y)**4*(-13119+4460*logy*(-1+y)**5+61135*y-111120*y**2
        +96170*y**3-36645*y**4+2617*y**5+615*y**6-305*y**7+36*y**8)+r**2*(
        -1+y)**7*(-48274+19320*logy*(-1+y)**2+115868*y-77254*y**2+13608*y**3
        +5999*y**4-11172*y**5-2919*y**6+1248*y**7+150*y**8+2*y**9)+7*r**4*(
        -1+y)**5*(-54990+34920*logy*(-1+y)**4+254880*y-452160*y**2+368460*y**3
        -121195*y**4+6290*y**5-3010*y**6+876*y**7+171*y**8+6*y**9)+14*r**8*(
        -1+y)**3*(-11045-2100*logy*(-1+y)**6+64170*y-154125*y**2+193460*y**3
        -145245*y**4+71488*y**5-28259*y**6+8068*y**7-1814*y**8+202*y**9)+14*r**14*(2765
        -24885*y+99540*y**2-236390*y**3+364840*y**4-379386*y**5+267456*y**6
        -125004*y**7+36441*y**8-5779*y**9+346*y**10)-7*r**10*(-1+y)**2*(2430
        -17010*y+51030*y**2-65970*y**3+71905*y**4-87953*y**5+91423*y**6-62087*y**7
        +25612*y**8-5690*y**9+510*y**10)-7*r**12*(7742-69678*y+278712*y**2
        -673292*y**3+1059333*y**4-1107315*y**5+761958*y**6-323316*y**7+64449*y**8
        +6549*y**9-6042*y**10+900*y**11))-6*MBhat**2*(19600*(-1+y)**10+6055*r**18*(
        -1+y)**10-560*r**8*(-1+y)**4*(40903+28560*logy*(-1+y)**6-216858*y
        +456465*y**2-465820*y**3+206565*y**4+3486*y**5-30491*y**6+4566*y**7
        +173*y**8-4*y**9+5*y**10)-70*r**10*(-1+y)**3*(-49980+300*logy*(-1
        +y)**7+349560*y-1047630*y**2+1743950*y**3-1741325*y**4+1029999*y**5
        -344035*y**6+57395*y**7-8906*y**8+462*y**9+100*y**10)-r**2*(-1+y)**8*(
        -1054465+336000*logy*(-1+y)**2+2444930*y-1558465*y**2+112000*y**3
        +28000*y**4+29344*y**5+26712*y**6-33932*y**7-9393*y**8+1414*y**9+135*y**10)
        -28*r**4*(-1+y)**6*(-385490+173700*logy*(-1+y)**4+1715660*y-2920890*y**2
        +2294660*y**3-747365*y**4+35076*y**5+6836*y**6+4256*y**7-4799*y**8
        +956*y**9+260*y**10)+140*r**6*(-1+y)**5*(-104288-154440*logy*(-1+y)**5
        +675880*y-1737860*y**2+2252660*y**3-1512430*y**4+455012*y**5-24255*y**6
        -2721*y**7+114*y**8-680*y**9-5*y**10+y**11)-28*r**12*(-1+y)**2*(4530
        +6900*logy*(-1+y)**8-29340*y+75090*y**2-85780*y**3+10625*y**4+18092*y**5
        -11376*y**6+9468*y**7-10563*y**8+3366*y**9-702*y**10+105*y**11)+40*r**16*(
        -1208+12080*y-54360*y**2+144960*y**3-253680*y**4+313740*y**5-278124*y**6
        +173850*y**7-74475*y**8+20675*y**9-3305*y**10+225*y**11)-28*r**14*(
        -3760+37600*y-169200*y**2+451200*y**3-789600*y**4+999054*y**5-947065*y**6
        +652980*y**7-313730*y**8+100090*y**9-19249*y**10+1680*y**11))-4*MBhat**3*(
        -58800*(-1+y)**10+20655*r**18*(-1+y)**10+70*r**8*(-1+y)**4*(426796
        +130320*logy*(-1+y)**6-2430456*y+5685180*y**2-6928640*y**3+4552200*y**4
        -1462602*y**5+137399*y**6+2946*y**7-1401*y**8+38*y**9)+105*r**6*(-1
        +y)**5*(39614+308520*logy*(-1+y)**5-506590*y+1784480*y**2-2812880*y**3
        +2181885*y**4-747951*y**5+52846*y**6+860*y**7+2675*y**8+15*y**9+6*y**10)
        +r**2*(-1+y)**8*(-2269065+760200*logy*(-1+y)**2+5298330*y-3409365*y**2
        +253400*y**3+94745*y**4+126350*y**5-71316*y**6-71478*y**7+4914*y**8
        +2250*y**9+75*y**10)+35*r**10*(-1+y)**3*(77170-9000*logy*(-1+y)**7
        -531190*y+1562070*y**2-2540450*y**3+2426105*y**4-1478067*y**5+594248*y**6
        -175568*y**7+28191*y**8-3145*y**9+136*y**10)+5*r**4*(-1+y)**6*(-4257576
        +2059680*logy*(-1+y)**4+19089984*y-32754336*y**2+25955584*y**3-8548996*y**4
        +405090*y**5+163121*y**6-71798*y**7-6318*y**8+10842*y**9+297*y**10
        -6*y**11)-21*r**12*(-1+y)**2*(-25640+205120*y-717920*y**2+1435840*y**3
        -1647780*y**4+1283190*y**5-743541*y**6+336798*y**7-117528*y**8+32066*y**9
        -5835*y**10+510*y**11)+40*r**16*(-3007+30070*y-135315*y**2+360840*y**3
        -647010*y**4+809970*y**5-715008*y**6+442830*y**7-187470*y**8+51175*y**9
        -7972*y**10+519*y**11)-35*r**14*(-4618+46180*y-207810*y**2+554160*y**3
        -1037115*y**4+1387812*y**5-1297323*y**6+836598*y**7-361935*y**8+95500*y**9
        -10771*y**10-978*y**11+300*y**12))-4*MBhat*(-8400*(-1+y)**10+3000*r**18*(
        -1+y)**10+35*r**10*(-1+y)**3*(-367240+103800*logy*(-1+y)**7+2466880*y
        -7037340*y**2+11002300*y**3-10094050*y**4+5329830*y**5-1431583*y**6
        +115171*y**7+1725*y**8+1729*y**9-382*y**10)+r**2*(-1+y)**8*(-590000
        +180600*logy*(-1+y)**2+1360600*y-860900*y**2+60200*y**3+15050*y**4
        +6020*y**5+14875*y**6+1720*y**7-13259*y**8-342*y**9+156*y**10)+105*r**6*(
        -1+y)**5*(161444+149520*logy*(-1+y)**5-956740*y+2287280*y**2-2785680*y**3
        +1766640*y**4-502848*y**5+26643*y**6+2429*y**7-982*y**8+630*y**9+277*y**10
        -5*y**11)+140*r**8*(-1+y)**4*(83075+150900*logy*(-1+y)**6-347550*y
        +416175*y**2+199600*y**3-904200*y**4+814380*y**5-285634*y**6+19605*y**7
        +1200*y**8+1144*y**9+18*y**10-3*y**11)-42*r**12*(-1+y)**2*(17500+3300*logy*(
        -1+y)**8-136700*y+465250*y**2-899700*y**3+1078425*y**4-816540*y**5
        +409440*y**6-141095*y**7+35893*y**8-4874*y**9+354*y**10+27*y**11)+2*r**4*(
        -1+y)**6*(-3173060+1371300*logy*(-1+y)**4+14063540*y-23837910*y**2
        +18634540*y**3-6029935*y**4+274260*y**5+50624*y**6+31159*y**7-11271*y**8
        -14086*y**9+6964*y**10+135*y**11)+20*r**16*(-1769+17690*y-79605*y**2
        +212280*y**3-371490*y**4+445788*y**5-383922*y**6+238320*y**7-102150*y**8
        +28310*y**9-4514*y**10+306*y**11)+7*r**14*(25660-256600*y+1154700*y**2
        -3079200*y**3+5388600*y**4-6466320*y**5+5527401*y**6-3467130*y**7+1584645*y**8
        -500920*y**9+99295*y**10-10446*y**11+315*y**12))+MBhat**4*(-294000*(
        -1+y)**10-154875*r**18*(-1+y)**10+280*r**4*(-1+y)**6*(-218781+117780*logy*(
        -1+y)**4+992904*y-1724916*y**2+1385108*y**3-466172*y**4+33141*y**5
        -389*y**6-3874*y**7+1602*y**8+117*y**9)+210*r**6*(-1+y)**5*(-218696
        +300720*logy*(-1+y)**5+792760*y-833720*y**2-164860*y**3+831775*y**4
        -461219*y**5+37710*y**6+6010*y**7-845*y**8+165*y**9)+280*r**8*(-1+y)**4*(83450
        -7200*logy*(-1+y)**6-507900*y+1291350*y**2-1751800*y**3+1314630*y**4
        -521823*y**5+82966*y**6-6141*y**7-2214*y**8+532*y**9)+r**2*(-1+y)**8*(
        -7258725+2604000*logy*(-1+y)**2+17121450*y-11164725*y**2+937720*y**3
        +782110*y**4-14084*y**5-622062*y**6-26544*y**7+36555*y**8+2490*y**9
        +15*y**10)-140*r**10*(-1+y)**3*(800-13950*logy*(-1+y)**7+8350*y-73875*y**2
        +228485*y**3-196220*y**4+2874*y**5+120856*y**6-92560*y**7+39246*y**8
        -8486*y**9+680*y**10)+84*r**12*(-1+y)**2*(-4850+38800*y-135800*y**2
        +210920*y**3-410360*y**4+668866*y**5-722098*y**6+520688*y**7-250711*y**8
        +76500*y**9-13325*y**10+1000*y**11)-20*r**16*(-34792+347920*y-1565640*y**2
        +4237200*y**3-7583940*y**4+9373518*y**5-8129352*y**6+4917990*y**7-2014650*y**8
        +523435*y**9-75064*y**10+4131*y**11)+70*r**14*(-10376+103760*y-466920*y**2
        +1308324*y**3-2382915*y**4+2878158*y**5-2314215*y**6+1168548*y**7-273915*y**8
        -57820*y**9+64225*y**10-18930*y**11+2076*y**12)))/(6300*r**2*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr12(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r12 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-60*(-1+y)**6-265*r**10*(-1+y)**6+2*r**4*(-1+y)**3*(-200+600*y-600*y**2
        +600*y**3-815*y**4+349*y**5)+5*r**2*(-1+y)**4*(-31+48*logy*(-1+y)**2
        +110*y-103*y**2+76*y**3-97*y**4+42*y**5+y**6)+2*r**6*(-1+y)*(390
        +90*logy*(-1+y)**5-2040*y+4305*y**2-5195*y**3+4105*y**4-1982*y**5
        +411*y**6)-2*r**8*(-430+2580*y-6450*y**2+9120*y**3-7770*y**4+3879*y**5
        -1028*y**6+109*y**7))/(30*r**2*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-360*(-1+y)**7-477*r**12*(-1+y)**7+r**4*(-1+y)**4*(-3845+8040*logy*(
        -1+y)**3+3495*y+8565*y**2-10895*y**3+5325*y**4-6279*y**5+2707*y**6
        +15*y**7)+r**2*(-1+y)**5*(-7723+3720*logy*(-1+y)**2+19166*y-13303*y**2
        +1240*y**3+1135*y**4-1040*y**5+435*y**6+30*y**7)+6*r**6*(-1+y)**2*(
        -1140-210*logy*(-1+y)**5+5910*y-12345*y**2+13045*y**3-8120*y**4
        +4682*y**5-2738*y**6+694*y**7)+r**10*(-2995+20965*y-62895*y**2+104825*y**3
        -110225*y**4+74901*y**5-31171*y**6+7069*y**7-654*y**8)+2*r**8*(-1+y)*(
        -2560+810*logy*(-1+y)**6+16170*y-42855*y**2+61190*y**3-54615*y**4
        +33426*y**5-13360*y**6+2568*y**7-84*y**8)+6*MBhat**2*(-1+y)*(-60*(
        -1+y)**6-265*r**10*(-1+y)**6+2*r**4*(-1+y)**3*(-200+600*y-600*y**2
        +600*y**3-815*y**4+349*y**5)+5*r**2*(-1+y)**4*(-31+48*logy*(-1+y)**2
        +110*y-103*y**2+76*y**3-97*y**4+42*y**5+y**6)+2*r**6*(-1+y)*(390
        +90*logy*(-1+y)**5-2040*y+4305*y**2-5195*y**3+4105*y**4-1982*y**5
        +411*y**6)-2*r**8*(-430+2580*y-6450*y**2+9120*y**3-7770*y**4+3879*y**5
        -1028*y**6+109*y**7))-2*MBhat*(-360*(-1+y)**7+1092*r**12*(-1+y)**7
        -6*r**4*(-1+y)**4*(930-360*logy*(-1+y)**3-2430*y+1890*y**2-865*y**3
        +1155*y**4-631*y**5+93*y**6)+r**2*(-1+y)**5*(-3727+2460*logy*(-1+y)**2
        +9914*y-7417*y**2+1750*y**3-1100*y**4+433*y**5+84*y**6+3*y**7)-3*r**6*(
        -1+y)**2*(-1060+600*logy*(-1+y)**5+4700*y-7900*y**2+7830*y**3-7050*y**4
        +5096*y**5-1781*y**6+133*y**7+56*y**8)+3*r**10*(1335-9345*y+28035*y**2
        -48525*y**3+52160*y**4-35021*y**5+14147*y**6-3081*y**7+265*y**8)-r**8*(4600
        -32200*y+96600*y**2-169550*y**3+192230*y**4-141636*y**5+63613*y**6
        -14341*y**7+357*y**8+327*y**9)))/(180*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-900*(-1+y)**8-530*r**14*(-1+y)**8+5*r**4*(-1+y)**5*(14594+18600*logy*(
        -1+y)**3-62382*y+90282*y**2-48694*y**3+4650*y**4+1947*y**5-1783*y**6
        +855*y**7+15*y**8)+r**2*(-1+y)**6*(-47330+16800*logy*(-1+y)**2+111460*y
        -72530*y**2+5600*y**3+1400*y**4+1487*y**5-580*y**6+299*y**7+44*y**8)
        +10*r**6*(-1+y)**3*(-9925+8340*logy*(-1+y)**5+41285*y-61720*y**2
        +33920*y**3+3890*y**4-10357*y**5+5372*y**6-3469*y**7+986*y**8)+10*r**8*(
        -1+y)**2*(2950-960*logy*(-1+y)**6-18660*y+49530*y**2-70840*y**3
        +57930*y**4-28665*y**5+11757*y**6-5169*y**7+1077*y**8)+r**12*(5330
        -42640*y+149240*y**2-298480*y**3+373100*y**4-311377*y**5+175052*y**6
        -62044*y**7+12170*y**8-981*y**9)+5*r**10*(-1+y)*(2470+1080*logy*(
        -1+y)**7-18370*y+58890*y**2-105710*y**3+115160*y**4-82815*y**5+42193*y**6
        -14455*y**7+2451*y**8-84*y**9)-15*MBhat**4*(-1+y)**2*(60*(-1+y)**6
        +265*r**10*(-1+y)**6-2*r**4*(-1+y)**3*(-200+600*y-600*y**2+600*y**3
        -815*y**4+349*y**5)-5*r**2*(-1+y)**4*(-31+48*logy*(-1+y)**2+110*y
        -103*y**2+76*y**3-97*y**4+42*y**5+y**6)-2*r**6*(-1+y)*(390+90*logy*(
        -1+y)**5-2040*y+4305*y**2-5195*y**3+4105*y**4-1982*y**5+411*y**6)
        +2*r**8*(-430+2580*y-6450*y**2+9120*y**3-7770*y**4+3879*y**5-1028*y**6
        +109*y**7))-10*MBhat**3*(-1+y)*(-360*(-1+y)**7+1092*r**12*(-1+y)**7
        -6*r**4*(-1+y)**4*(930-360*logy*(-1+y)**3-2430*y+1890*y**2-865*y**3
        +1155*y**4-631*y**5+93*y**6)+r**2*(-1+y)**5*(-3727+2460*logy*(-1+y)**2
        +9914*y-7417*y**2+1750*y**3-1100*y**4+433*y**5+84*y**6+3*y**7)-3*r**6*(
        -1+y)**2*(-1060+600*logy*(-1+y)**5+4700*y-7900*y**2+7830*y**3-7050*y**4
        +5096*y**5-1781*y**6+133*y**7+56*y**8)+3*r**10*(1335-9345*y+28035*y**2
        -48525*y**3+52160*y**4-35021*y**5+14147*y**6-3081*y**7+265*y**8)-r**8*(4600
        -32200*y+96600*y**2-169550*y**3+192230*y**4-141636*y**5+63613*y**6
        -14341*y**7+357*y**8+327*y**9))-2*MBhat*(-1800*(-1+y)**8+1300*r**14*(
        -1+y)**8+5*r**4*(-1+y)**5*(6775+19740*logy*(-1+y)**3-40065*y+69675*y**2
        -42965*y**3+6690*y**4-2286*y**5+1084*y**6+120*y**7)+r**2*(-1+y)**6*(
        -63815+25500*logy*(-1+y)**2+153130*y-102065*y**2+8500*y**3+3670*y**4
        -14*y**5-107*y**6+376*y**7+25*y**8)-15*r**6*(-1+y)**3*(8280-2280*logy*(
        -1+y)**5-39120*y+72540*y**2-64940*y**3+28100*y**4-7158*y**5+3345*y**6
        -1171*y**7+148*y**8)-10*r**8*(-1+y)**2*(3590+1020*logy*(-1+y)**6-20520*y
        +48240*y**2-59220*y**3+43440*y**4-22851*y**5+9621*y**6-2427*y**7+249*y**8
        +28*y**9)+3*r**12*(-2595+20760*y-72660*y**2+145320*y**3-188815*y**4
        +163958*y**5-93184*y**6+32984*y**7-6508*y**8+530*y**9)-5*r**10*(-3535
        +28280*y-98980*y**2+197960*y**3-257455*y**4+231812*y**5-144748*y**6
        +59492*y**7-14398*y**8+1572*y**9))-5*MBhat**2*(1080*(-1+y)**8+1440*r**14*(
        -1+y)**8-2*r**4*(-1+y)**5*(-5765+12360*logy*(-1+y)**3+4935*y+13605*y**2
        -14960*y**3+720*y**4-678*y**5+760*y**6+15*y**7)+6*r**6*(-1+y)**3*(2880
        +720*logy*(-1+y)**5-15120*y+32040*y**2-33250*y**3+16315*y**4-3483*y**5
        +699*y**6+19*y**7-64*y**8)-r**2*(-1+y)**6*(-22984+11040*logy*(-1+y)**2
        +57008*y-39544*y**2+4298*y**3+1346*y**4-880*y**5+511*y**6+64*y**7
        +y**8)+6*r**12*(-969+7752*y-27132*y**2+55697*y**3-72847*y**4+62216*y**5
        -34384*y**6+11696*y**7-2164*y**8+156*y**9)-2*r**8*(-1+y)**2*(-1540
        +1380*logy*(-1+y)**6+10620*y-30690*y**2+40950*y**3-30525*y**4+18612*y**5
        -11752*y**6+5220*y**7-1473*y**8+218*y**9)+r**10*(6040-48320*y+169120*y**2
        -356690*y**3+488020*y**4-432890*y**5+239197*y**6-71696*y**7+4336*y**8
        +3678*y**9-795*y**10)))/(450*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-4200*(-1+y)**9-1325*r**16*(-1+y)**9+7*r**4*(-1+y)**6*(249765+172200*logy*(
        -1+y)**3-921495*y+1179795*y**2-565465*y**3+43050*y**4+8610*y**5+4246*y**6
        -2098*y**7+1628*y**8+44*y**9)+r**2*(-1+y)**7*(-405235+121800*logy*(
        -1+y)**2+932270*y-587935*y**2+40600*y**3+10150*y**4+4060*y**5+3136*y**6
        +2644*y**7-486*y**8+96*y**9)+35*r**6*(-1+y)**4*(7905+74700*logy*(-1
        +y)**5-114225*y+415200*y**2-664200*y**3+518850*y**4-178470*y**5+11626*y**6
        +5494*y**7-3419*y**8+1215*y**9)+35*r**10*(-1+y)**2*(-5545-2700*logy*(
        -1+y)**7+41515*y-133995*y**2+242225*y**3-265850*y**4+178410*y**5
        -74822*y**6+24982*y**7-8778*y**8+1498*y**9)+7*r**12*(-1+y)*(-7395
        +4500*logy*(-1+y)**8+63660*y-240810*y**2+523620*y**3-717525*y**4
        +637020*y**5-387238*y**6+171236*y**7-51820*y**8+7824*y**9-252*y**10)
        +35*r**8*(-1+y)**3*(64975+36300*logy*(-1+y)**6-353550*y+774975*y**2
        -851800*y**3+457350*y**4-74040*y**5-26816*y**6+13492*y**7-6403*y**8
        +1662*y**9-5*y**10)-7*r**14*(2765-24885*y+99540*y**2-232260*y**3+348390*y**4
        -348390*y**5+240856*y**6-115304*y**7+35541*y**8-6129*y**9+436*y**10)
        -70*MBhat**6*(-1+y)**3*(60*(-1+y)**6+265*r**10*(-1+y)**6-2*r**4*(-1
        +y)**3*(-200+600*y-600*y**2+600*y**3-815*y**4+349*y**5)-5*r**2*(-1
        +y)**4*(-31+48*logy*(-1+y)**2+110*y-103*y**2+76*y**3-97*y**4+42*y**5
        +y**6)-2*r**6*(-1+y)*(390+90*logy*(-1+y)**5-2040*y+4305*y**2-5195*y**3
        +4105*y**4-1982*y**5+411*y**6)+2*r**8*(-430+2580*y-6450*y**2+9120*y**3
        -7770*y**4+3879*y**5-1028*y**6+109*y**7))-70*MBhat**5*(-1+y)**2*(-360*(
        -1+y)**7+1092*r**12*(-1+y)**7-6*r**4*(-1+y)**4*(930-360*logy*(-1
        +y)**3-2430*y+1890*y**2-865*y**3+1155*y**4-631*y**5+93*y**6)+r**2*(
        -1+y)**5*(-3727+2460*logy*(-1+y)**2+9914*y-7417*y**2+1750*y**3-1100*y**4
        +433*y**5+84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-1060+600*logy*(-1+y)**5
        +4700*y-7900*y**2+7830*y**3-7050*y**4+5096*y**5-1781*y**6+133*y**7
        +56*y**8)+3*r**10*(1335-9345*y+28035*y**2-48525*y**3+52160*y**4-35021*y**5
        +14147*y**6-3081*y**7+265*y**8)-r**8*(4600-32200*y+96600*y**2-169550*y**3
        +192230*y**4-141636*y**5+63613*y**6-14341*y**7+357*y**8+327*y**9))
        -35*MBhat**4*(-1+y)*(1800*(-1+y)**8+2880*r**14*(-1+y)**8-3*r**4*(-1
        +y)**5*(-6405+13800*logy*(-1+y)**3+5415*y+15285*y**2-16315*y**3-815*y**4
        +1189*y**5+111*y**6+15*y**7)-r**2*(-1+y)**6*(-38245+18360*logy*(-1
        +y)**2+94850*y-65785*y**2+7356*y**3+1557*y**4-720*y**5+587*y**6+98*y**7
        +2*y**8)-6*r**6*(-1+y)**3*(-4620-1230*logy*(-1+y)**5+24330*y-51735*y**2
        +53455*y**3-24510*y**4+2284*y**5+1340*y**6-732*y**7+128*y**8)-2*r**8*(
        -1+y)**2*(-520+1950*logy*(-1+y)**6+5070*y-18525*y**2+20710*y**3
        -6435*y**4+3798*y**5-10144*y**6+7872*y**7-2862*y**8+436*y**9)+3*r**12*(
        -4035+32280*y-112980*y**2+231692*y**3-302518*y**4+257768*y**5-141988*y**6
        +48056*y**7-8815*y**8+624*y**9)-3*r**10*(-5025+40200*y-140700*y**2
        +293700*y**3-397030*y**4+350302*y**5-194822*y**6+60544*y**7-5465*y**8
        -2234*y**9+530*y**10))+MBhat*(25200*(-1+y)**9-6760*r**16*(-1+y)**9
        -7*r**4*(-1+y)**6*(779520+657600*logy*(-1+y)**3-2996160*y+3982560*y**2
        -1985120*y**3+164400*y**4+38982*y**5-251*y**6+2661*y**7+2463*y**8+25*y**9)
        -r**2*(-1+y)**7*(-1836140+592200*logy*(-1+y)**2+4264480*y-2724440*y**2
        +197400*y**3+49350*y**4+23772*y**5+23079*y**6-3810*y**7+1845*y**8+264*y**9)
        +70*r**8*(-1+y)**3*(-75100-18000*logy*(-1+y)**6+432600*y-1027500*y**2
        +1280000*y**3-870000*y**4+300102*y**5-47013*y**6+9798*y**7-2853*y**8
        +386*y**9)+70*r**6*(-1+y)**4*(38032-101040*logy*(-1+y)**5-89120*y
        -74360*y**2+411160*y**3-458180*y**4+194449*y**5-23990*y**6+3841*y**7
        -1658*y**8-105*y**9+3*y**10)+35*r**10*(-1+y)**2*(-21980+4200*logy*(
        -1+y)**7+149660*y-434280*y**2+694400*y**3-657650*y**4+391002*y**5
        -161227*y**6+50113*y**7-9357*y**8+883*y**9+56*y**10)-14*r**14*(4240
        -38160*y+152640*y**2-356160*y**3+534240*y**4-553581*y**5+400869*y**6
        -195876*y**7+60894*y**8-10741*y**9+795*y**10)-7*r**12*(-31520+283680*y
        -1134720*y**2+2647680*y**3-3971520*y**4+4090446*y**5-3019755*y**6+1595817*y**7
        -572268*y**8+125702*y**9-13869*y**10+327*y**11))+MBhat**2*(-63000*(-1
        +y)**9-19800*r**16*(-1+y)**9+35*r**4*(-1+y)**6*(160824+192480*logy*(
        -1+y)**3-674952*y+963672*y**2-513704*y**3+50094*y**4+8448*y**5-1743*y**6
        +2097*y**7+105*y**8-y**9)+70*r**6*(-1+y)**4*(-107195+87420*logy*(-1
        +y)**5+448555*y-678560*y**2+387160*y**3+22285*y**4-84467*y**5+12676*y**6
        -1346*y**7+682*y**8+30*y**9)+140*r**8*(-1+y)**3*(11260-2640*logy*(
        -1+y)**6-70200*y+183420*y**2-257760*y**3+201975*y**4-82011*y**5+15460*y**6
        -2577*y**7-141*y**8+124*y**9)+r**2*(-1+y)**7*(-3309470+1167600*logy*(
        -1+y)**2+7786540*y-5060870*y**2+389200*y**3+100870*y**4+82390*y**5
        -6853*y**6+4406*y**7+3137*y**8+150*y**9)-210*r**14*(559-5031*y+20124*y**2
        -46956*y**3+72583*y**4-76839*y**5+55608*y**6-26952*y**7+8298*y**8-1442*y**9
        +104*y**10)+35*r**10*(-1+y)**2*(-570+5160*logy*(-1+y)**7-1170*y+21570*y**2
        -72070*y**3+76690*y**4-29268*y**5+6591*y**6-7287*y**7+3591*y**8-1175*y**9
        +218*y**10)+7*r**12*(28040-252360*y+1009440*y**2-2355360*y**3+3726750*y**4
        -4192500*y**5+3297921*y**6-1750299*y**7+592146*y**8-110624*y**9+6051*y**10
        +795*y**11))-MBhat**3*(-84000*(-1+y)**9+42990*r**16*(-1+y)**9+35*r**4*(
        -1+y)**6*(46260+132480*logy*(-1+y)**3-271260*y+469980*y**2-287944*y**3
        +37577*y**4-5065*y**5+3549*y**6+417*y**7+6*y**8)-70*r**6*(-1+y)**4*(79823
        -22980*logy*(-1+y)**5-376135*y+694820*y**2-616970*y**3+254155*y**4
        -44141*y**5+11582*y**6-3280*y**7+386*y**8)+r**2*(-1+y)**7*(-2966960
        +1184400*logy*(-1+y)**2+7118320*y-4743560*y**2+390880*y**3+169855*y**4
        +8036*y**5-3437*y**6+11756*y**7+1100*y**8+10*y**9)+70*r**8*(-1+y)**3*(
        -17225-4140*logy*(-1+y)**6+99210*y-235605*y**2+289540*y**3-209045*y**4
        +107606*y**5-46141*y**6+12628*y**7-2044*y**8+76*y**9)+70*r**14*(2675
        -24075*y+96300*y**2-228998*y**3+354172*y**4-369306*y**5+261324*y**6
        -122796*y**7+36099*y**8-5811*y**9+360*y**10)-35*r**10*(-1+y)**2*(6360
        -44520*y+133560*y**2-200560*y**3+214365*y**4-200313*y**5+150455*y**6
        -79247*y**7+27516*y**8-5626*y**9+530*y**10)-7*r**12*(29980-269820*y
        +1079280*y**2-2638460*y**3+4187295*y**4-4356873*y**5+2929362*y**6-1166508*y**7
        +170577*y**8+64297*y**9-33810*y**10+4680*y**11)))/(2100*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-12600*(-1+y)**10-2385*r**18*(-1+y)**10+r**2*(-1+y)**8*(-1937735+520800*logy*(
        -1+y)**2+4396270*y-2718935*y**2+173600*y**3+43400*y**4+17360*y**5
        +8680*y**6+1528*y**7+18439*y**8-4782*y**9+75*y**10)+4*r**4*(-1+y)**7*(3807500
        +1982400*logy*(-1+y)**3-13404900*y+16378500*y**2-7441900*y**3+495600*y**4
        +99120*y**5+33040*y**6+10776*y**7+7677*y**8+3313*y**9+144*y**10)+210*r**6*(
        -1+y)**5*(103586+139980*logy*(-1+y)**5-657910*y+1665770*y**2-2132370*y**3
        +1416135*y**4-423207*y**5+23330*y**6+3530*y**7+1443*y**8-845*y**9+526*y**10)
        +84*r**12*(-1+y)**2*(8540-6000*logy*(-1+y)**8-74320*y+284120*y**2
        -624240*y**3+864300*y**4-775440*y**5+443720*y**6-162952*y**7+47151*y**8
        -13886*y**9+1957*y**10)+14*r**14*(-1+y)*(9250+8100*logy*(-1+y)**9
        -91350*y+401850*y**2-1034850*y**3+1722375*y**4-1926495*y**5+1454430*y**6
        -771786*y**7+304599*y**8-83221*y**9+11334*y**10-336*y**11)+140*r**8*(
        -1+y)**4*(106484+250080*logy*(-1+y)**6-388824*y+221820*y**2+954640*y**3
        -1966380*y**4+1536792*y**5-506212*y**6+35112*y**7+8616*y**8-3713*y**9
        +1530*y**10-15*y**11)-70*r**10*(-1+y)**3*(334180-146100*logy*(-1+y)**7
        -2193160*y+6068130*y**2-9090850*y**3+7812475*y**4-3664785*y**5+710245*y**6
        +53309*y**7-38427*y**8+12677*y**9-2909*y**10+15*y**11)-20*r**16*(-2378
        +23780*y-107010*y**2+285360*y**3-499380*y**4+599256*y**5-499380*y**6
        +294600*y**7-122730*y**8+33455*y**9-5144*y**10+327*y**11)+210*MBhat**8*(
        -1+y)**4*(-60*(-1+y)**6-265*r**10*(-1+y)**6+2*r**4*(-1+y)**3*(-200
        +600*y-600*y**2+600*y**3-815*y**4+349*y**5)+5*r**2*(-1+y)**4*(-31
        +48*logy*(-1+y)**2+110*y-103*y**2+76*y**3-97*y**4+42*y**5+y**6)
        +2*r**6*(-1+y)*(390+90*logy*(-1+y)**5-2040*y+4305*y**2-5195*y**3
        +4105*y**4-1982*y**5+411*y**6)-2*r**8*(-430+2580*y-6450*y**2+9120*y**3
        -7770*y**4+3879*y**5-1028*y**6+109*y**7))-280*MBhat**7*(-1+y)**3*(-360*(
        -1+y)**7+1092*r**12*(-1+y)**7-6*r**4*(-1+y)**4*(930-360*logy*(-1
        +y)**3-2430*y+1890*y**2-865*y**3+1155*y**4-631*y**5+93*y**6)+r**2*(
        -1+y)**5*(-3727+2460*logy*(-1+y)**2+9914*y-7417*y**2+1750*y**3-1100*y**4
        +433*y**5+84*y**6+3*y**7)-3*r**6*(-1+y)**2*(-1060+600*logy*(-1+y)**5
        +4700*y-7900*y**2+7830*y**3-7050*y**4+5096*y**5-1781*y**6+133*y**7
        +56*y**8)+3*r**10*(1335-9345*y+28035*y**2-48525*y**3+52160*y**4-35021*y**5
        +14147*y**6-3081*y**7+265*y**8)-r**8*(4600-32200*y+96600*y**2-169550*y**3
        +192230*y**4-141636*y**5+63613*y**6-14341*y**7+357*y**8+327*y**9))
        -140*MBhat**6*(-1+y)**2*(2520*(-1+y)**8+4320*r**14*(-1+y)**8-2*r**4*(
        -1+y)**5*(-13450+29040*logy*(-1+y)**3+11310*y+32250*y**2-33985*y**3
        -3165*y**4+4245*y**5-427*y**6+30*y**7)-r**2*(-1+y)**6*(-53506+25680*logy*(
        -1+y)**2+132692*y-92026*y**2+10414*y**3+1768*y**4-560*y**5+663*y**6
        +132*y**7+3*y**8)-6*r**6*(-1+y)**3*(-6360-1740*logy*(-1+y)**5+33540*y
        -71430*y**2+73660*y**3-32705*y**4+1085*y**5+3379*y**6-1445*y**7+192*y**8)
        +18*r**12*(-1022+8176*y-28616*y**2+58665*y**3-76557*y**4+65184*y**5
        -35868*y**6+12120*y**7-2217*y**8+156*y**9)-2*r**8*(-1+y)**2*(500+2520*logy*(
        -1+y)**6-480*y-6360*y**2+470*y**3+17655*y**4-11016*y**5-8536*y**6
        +10524*y**7-4251*y**8+654*y**9)+r**10*(24110-192880*y+675080*y**2
        -1405510*y**3+1894160*y**4-1668922*y**5+929735*y**6-291568*y**7+28454*y**8
        +9726*y**9-2385*y**10))-12*MBhat**2*(29400*(-1+y)**10+3150*r**18*(-1
        +y)**10-7*r**4*(-1+y)**7*(1834670+1233600*logy*(-1+y)**3-6737610*y
        +8588010*y**2-4096270*y**3+308400*y**4+58824*y**5+25581*y**6+1553*y**7
        +2855*y**8+547*y**9)+35*r**10*(-1+y)**3*(34640+4800*logy*(-1+y)**7
        -247280*y+758640*y**2-1298000*y**3+1340000*y**4-821208*y**5+272433*y**6
        -43997*y**7+7473*y**8+93*y**9-184*y**10)-70*r**8*(-1+y)**4*(233245
        +128100*logy*(-1+y)**6-1271370*y+2794125*y**2-3085000*y**3+1673250*y**4
        -285708*y**5-78245*y**6+19454*y**7-347*y**8+16*y**9+20*y**10)-r**2*(
        -1+y)**8*(-2833250+848400*logy*(-1+y)**2+6514900*y-4105850*y**2+282800*y**3
        +70700*y**4+22568*y**5+35343*y**6+9336*y**7-2107*y**8+594*y**9+66*y**10)
        +70*r**6*(-1+y)**5*(-27092-273360*logy*(-1+y)**5+408820*y-1501040*y**2
        +2412240*y**3-1889520*y**4+650442*y**5-45634*y**6-7570*y**7+167*y**8
        -725*y**9-5*y**10+y**11)-7*r**12*(-1+y)**2*(14750+13800*logy*(-1+y)**8
        -104200*y+309500*y**2-490200*y**3+419550*y**4-289344*y**5+199435*y**6
        -81584*y**7+7954*y**8+1112*y**9-841*y**10+218*y**11)+10*r**16*(-2569
        +25690*y-115605*y**2+308280*y**3-539490*y**4+666792*y**5-590394*y**6
        +368400*y**7-157455*y**8+43570*y**9-6931*y**10+468*y**11)-14*r**14*(
        -4540+45400*y-204300*y**2+544800*y**3-953400*y**4+1197042*y**5-1121701*y**6
        +766800*y**7-365735*y**8+115790*y**9-22062*y**10+1906*y**11))-12*MBhat**5*(
        -1+y)*(-58800*(-1+y)**9+42990*r**16*(-1+y)**9+35*r**4*(-1+y)**6*(32710
        +93000*logy*(-1+y)**3-191130*y+330630*y**2-202014*y**3+24197*y**4
        -493*y**5+1381*y**6+177*y**7+6*y**8)+70*r**6*(-1+y)**4*(-54983+16140*logy*(
        -1+y)**5+258775*y-477200*y**2+422150*y**3-169855*y**4+22667*y**5
        -1547*y**6-233*y**7+58*y**8)+r**2*(-1+y)**7*(-2073550+827400*logy*(
        -1+y)**2+4974500*y-3314650*y**2+271880*y**3+118475*y**4+8232*y**5
        -1939*y**6+6492*y**7+750*y**8+10*y**9)+70*r**8*(-1+y)**3*(-10045-2100*logy*(
        -1+y)**6+58170*y-139125*y**2+171100*y**3-122165*y**4+61904*y**5-26899*y**6
        +7774*y**7-1546*y**8+132*y**9)+70*r**14*(2935-26415*y+105660*y**2
        -250838*y**3+386932*y**4-402066*y**5+283164*y**6-132156*y**7+38439*y**8
        -6071*y**9+360*y**10)-35*r**10*(-1+y)**2*(-710+4970*y-14910*y**2+46890*y**3
        -53095*y**4-4149*y**5+57123*y**6-53595*y**7+24372*y**8-5626*y**9+530*y**10)
        -7*r**12*(45550-409950*y+1639800*y**2-3946340*y**3+6192105*y**4-6473511*y**5
        +4472214*y**6-1923516*y**7+407529*y**8+22069*y**9-30630*y**10+4680*y**11))
        -4*MBhat*(-25200*(-1+y)**10+3120*r**18*(-1+y)**10+6*r**4*(-1+y)**7*(3178380
        +1845900*logy*(-1+y)**3-11381040*y+14149890*y**2-6562530*y**3+461475*y**4
        +92295*y**5+27125*y**6+16441*y**7+1462*y**8+2318*y**9+44*y**10)+r**2*(
        -1+y)**8*(-3108040+877800*logy*(-1+y)**2+7093880*y-4424740*y**2+292600*y**3
        +73150*y**4+29260*y**5+8771*y**6+32522*y**7-703*y**8-1044*y**9+144*y**10)
        -105*r**10*(-1+y)**3*(122040-27000*logy*(-1+y)**7-827280*y+2387340*y**2
        -3789900*y**3+3553650*y**4-1943190*y**5+561021*y**6-67799*y**7+6243*y**8
        -1393*y**9+228*y**10)+105*r**6*(-1+y)**5*(144956+311280*logy*(-1+y)**5
        -1036060*y+2850320*y**2-3887920*y**3+2722160*y**4-855712*y**5+52499*y**6
        +8489*y**7+346*y**8+570*y**9+309*y**10-5*y**11)+140*r**8*(-1+y)**4*(181867
        +190740*logy*(-1+y)**6-900462*y+1678935*y**2-1284880*y**3+9960*y**4
        +568236*y**5-286160*y**6+31428*y**7-360*y**8+1151*y**9+18*y**10-3*y**11)
        -42*r**12*(-1+y)**2*(24380+3300*logy*(-1+y)**8-191740*y+657890*y**2
        -1284980*y**3+1560025*y**4-1201820*y**5+603658*y**6-211052*y**7+53765*y**8
        -7858*y**9+644*y**10+28*y**11)+60*r**16*(-627+6270*y-28215*y**2+75240*y**3
        -131670*y**4+158004*y**5-135982*y**6+84280*y**7-36030*y**8+9950*y**9
        -1578*y**10+106*y**11)+7*r**14*(29540-295400*y+1329300*y**2-3544800*y**3
        +6203400*y**4-7444080*y**5+6346065*y**6-3962442*y**7+1806945*y**8-569240*y**9
        +111935*y**10-11550*y**11+327*y**12))-4*MBhat**3*(-176400*(-1+y)**10
        +21495*r**18*(-1+y)**10+70*r**8*(-1+y)**4*(487634+127080*logy*(-1+y)**6
        -2798724*y+6615570*y**2-8185360*y**3+5503680*y**4-1837146*y**5+223435*y**6
        -12114*y**7+243*y**8-158*y**9)+r**4*(-1+y)**7*(39287820+32667600*logy*(
        -1+y)**3-150531060*y+199532460*y**2-99178420*y**3+8076180*y**4+1889202*y**5
        +240709*y**6+87411*y**7+59193*y**8+1455*y**9-30*y**10)+105*r**6*(-1
        +y)**5*(-196622+474840*logy*(-1+y)**5+508270*y+170560*y**2-1753360*y**3
        +2066165*y**4-892263*y**5+92222*y**6+2024*y**7+2647*y**8+15*y**9+6*y**10)
        -35*r**10*(-1+y)**3*(-86510+9000*logy*(-1+y)**7+596570*y-1758210*y**2
        +2867350*y**3-2741545*y**4+1614615*y**5-645028*y**6+191734*y**7-29985*y**8
        +2675*y**9+34*y**10)+r**2*(-1+y)**8*(-12838845+4128600*logy*(-1+y)**2
        +29806290*y-19031745*y**2+1376200*y**3+316225*y**4+226618*y**5+120204*y**6
        -12804*y**7+6174*y**8+2208*y**9+75*y**10)-21*r**12*(-1+y)**2*(-21820
        +174560*y-610960*y**2+1221920*y**3-1373200*y**4+1008078*y**5-579069*y**6
        +274662*y**7-101532*y**8+29214*y**9-5583*y**10+530*y**11)+20*r**16*(
        -6403+64030*y-288135*y**2+768360*y**3-1376970*y**4+1722252*y**5-1518510*y**6
        +939000*y**7-396675*y**8+107950*y**9-16735*y**10+1080*y**11)-7*r**14*(
        -28510+285100*y-1282950*y**2+3421200*y**3-6333015*y**4+8372604*y**5
        -7770975*y**6+4992150*y**7-2158155*y**8+573640*y**9-68419*y**10-4230*y**11
        +1560*y**12))+MBhat**4*(-882000*(-1+y)**10-161175*r**18*(-1+y)**10+420*r**4*(
        -1+y)**7*(190756+226080*logy*(-1+y)**3-798348*y+1137468*y**2-606012*y**3
        +59753*y**4+8759*y**5+295*y**6+1231*y**7+78*y**8)+210*r**6*(-1+y)**5*(
        -506688+407760*logy*(-1+y)**5+2125680*y-3231960*y**2+1876820*y**3
        +64655*y**4-383843*y**5+49498*y**6+5902*y**7-1069*y**8+165*y**9)+280*r**8*(
        -1+y)**4*(75385-11700*logy*(-1+y)**6-464010*y+1195125*y**2-1648340*y**3
        +1254135*y**4-469281*y**5+54743*y**6+2979*y**7-4698*y**8+812*y**9)
        +r**2*(-1+y)**8*(-46253625+16296000*logy*(-1+y)**2+108803250*y-70697625*y**2
        +5323640*y**3+1630370*y**4+995204*y**5-15414*y**6+28056*y**7+36639*y**8
        +2490*y**9+15*y**10)-420*r**10*(-1+y)**3*(300-4650*logy*(-1+y)**7
        +2550*y-23925*y**2+76475*y**3-68745*y**4-14283*y**5+59092*y**6-41196*y**7
        +16127*y**8-3149*y**9+204*y**10)+84*r**12*(-1+y)**2*(-500+4000*y-14000*y**2
        -35560*y**3-86845*y**4+424594*y**5-617518*y**6+502520*y**7-254212*y**8
        +78616*y**9-13685*y**10+1040*y**11)-60*r**16*(-12442+124420*y-559890*y**2
        +1514600*y**3-2709140*y**4+3345594*y**5-2898336*y**6+1750770*y**7-715650*y**8
        +185305*y**9-26412*y**10+1433*y**11)+70*r**14*(-13264+132640*y-596880*y**2
        +1656564*y**3-2999895*y**4+3629622*y**5-2949675*y**6+1536420*y**7-412065*y**8
        -28880*y**9+62783*y**10-19530*y**11+2160*y**12)))/(6300*r**2*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr13(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r13 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (120*(-1+y)**5+675*r**10*(-1+y)**5-r**2*(-1+y)**3*(55+240*logy*(-1
        +y)**2+130*y-305*y**2+1940*y**3-2905*y**4+1046*y**5+15*y**6)+10*r**8*(224
        -1120*y+2240*y**2-2498*y**3+1540*y**4-479*y**5+57*y**6)-6*r**6*(310
        +90*logy*(-1+y)**5-1640*y+3505*y**2-4435*y**3+3905*y**4-2212*y**5
        +519*y**6)-12*r**4*(-30+150*y-300*y**2+635*y**3-1465*y**4+1947*y**5
        -1217*y**6+280*y**7))/(90*r**2*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (240*(-1+y)**6+405*r**12*(-1+y)**6-r**2*(-1+y)**4*(-6175+2640*logy*(
        -1+y)**2+14990*y-10135*y**2+880*y**3+2065*y**4-2468*y**5+765*y**6
        +30*y**7)+5*r**10*(-539+3234*y-8085*y**2+10780*y**3-8955*y**4+4404*y**5
        -1133*y**6+114*y**7)-6*r**6*(-1+y)*(-1540+90*logy*(-1+y)**5+7610*y
        -14995*y**2+14695*y**3-8490*y**4+5308*y**5-3849*y**6+1069*y**7)-2*r**8*(
        -2200+810*logy*(-1+y)**6+14010*y-37455*y**2+53990*y**3-48795*y**4
        +30942*y**5-13783*y**6+2886*y**7-75*y**8)-r**4*(-1+y)**2*(-715+8760*logy*(
        -1+y)**4+11620*y-34950*y**2+40820*y**3-24860*y**4+18516*y**5-15154*y**6
        +4420*y**7+15*y**8)-2*MBhat**2*(-1+y)*(-120*(-1+y)**5-675*r**10*(-1
        +y)**5+r**2*(-1+y)**3*(55+240*logy*(-1+y)**2+130*y-305*y**2+1940*y**3
        -2905*y**4+1046*y**5+15*y**6)-10*r**8*(224-1120*y+2240*y**2-2498*y**3
        +1540*y**4-479*y**5+57*y**6)+6*r**6*(310+90*logy*(-1+y)**5-1640*y
        +3505*y**2-4435*y**3+3905*y**4-2212*y**5+519*y**6)+12*r**4*(-30+150*y
        -300*y**2+635*y**3-1465*y**4+1947*y**5-1217*y**6+280*y**7))+2*MBhat*(
        -240*(-1+y)**6+903*r**12*(-1+y)**6+r**4*(-1+y)**2*(4625+3180*logy*(
        -1+y)**4-15320*y+16620*y**2-11410*y**3+17845*y**4-19380*y**5+7646*y**6
        -914*y**7)+r**2*(-1+y)**4*(-3323+1680*logy*(-1+y)**2+8326*y-5843*y**2
        +2780*y**3-2905*y**4+785*y**5+129*y**6+3*y**7)+5*r**10*(-637+3822*y
        -9555*y**2+13610*y**3-11400*y**4+5505*y**5-1390*y**6+135*y**7)+r**8*(2360
        -14160*y+35400*y**2-55030*y**3+57465*y**4-37914*y**5+12812*y**6-1128*y**7
        -285*y**8)-3*r**6*(-1+y)*(-380+1140*logy*(-1+y)**5+760*y+1330*y**2
        -2480*y**3-2715*y**4+6669*y**5-3461*y**6+515*y**7+50*y**8)))/(180*r**2*(
        -1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (120*(-1+y)**7+90*r**14*(-1+y)**7-r**2*(-1+y)**5*(-7322+2400*logy*(
        -1+y)**2+17044*y-10922*y**2+800*y**3+200*y**4+563*y**5-525*y**6
        +129*y**7+9*y**8)+r**12*(962-6734*y+20202*y**2-33670*y**3+33670*y**4
        -22227*y**5+9065*y**6-1979*y**7+171*y**8)-2*r**8*(-1+y)*(5920+360*logy*(
        -1+y)**6-35160*y+86820*y**2-113960*y**3+83670*y**4-34953*y**5+11728*y**6
        -6393*y**7+1608*y**8)-2*r**6*(-1+y)**2*(-9245+12180*logy*(-1+y)**5
        +34045*y-37640*y**2-2960*y**3+31930*y**4-20261*y**5+7354*y**6-5297*y**7
        +1642*y**8)-r**4*(-1+y)**3*(-21770+18600*logy*(-1+y)**4+105680*y
        -195720*y**2+167680*y**3-60520*y**4+1851*y**5+6004*y**6-4834*y**7+1470*y**8
        +15*y**9)+r**10*(-1990-1080*logy*(-1+y)**7+15010*y-48810*y**2+88910*y**3
        -98360*y**4+71727*y**5-37396*y**6+14176*y**7-2622*y**8+75*y**9)-MBhat**4*(
        -1+y)**2*(-120*(-1+y)**5-675*r**10*(-1+y)**5+r**2*(-1+y)**3*(55+240*logy*(
        -1+y)**2+130*y-305*y**2+1940*y**3-2905*y**4+1046*y**5+15*y**6)-10*r**8*(224
        -1120*y+2240*y**2-2498*y**3+1540*y**4-479*y**5+57*y**6)+6*r**6*(310
        +90*logy*(-1+y)**5-1640*y+3505*y**2-4435*y**3+3905*y**4-2212*y**5
        +519*y**6)+12*r**4*(-30+150*y-300*y**2+635*y**3-1465*y**4+1947*y**5
        -1217*y**6+280*y**7))+2*MBhat**3*(-1+y)*(-240*(-1+y)**6+903*r**12*(
        -1+y)**6+r**4*(-1+y)**2*(4625+3180*logy*(-1+y)**4-15320*y+16620*y**2
        -11410*y**3+17845*y**4-19380*y**5+7646*y**6-914*y**7)+r**2*(-1+y)**4*(
        -3323+1680*logy*(-1+y)**2+8326*y-5843*y**2+2780*y**3-2905*y**4+785*y**5
        +129*y**6+3*y**7)+5*r**10*(-637+3822*y-9555*y**2+13610*y**3-11400*y**4
        +5505*y**5-1390*y**6+135*y**7)+r**8*(2360-14160*y+35400*y**2-55030*y**3
        +57465*y**4-37914*y**5+12812*y**6-1128*y**7-285*y**8)-3*r**6*(-1+y)*(
        -380+1140*logy*(-1+y)**5+760*y+1330*y**2-2480*y**3-2715*y**4+6669*y**5
        -3461*y**6+515*y**7+50*y**8))+2*MBhat*(-240*(-1+y)**7+215*r**14*(-1
        +y)**7+r**6*(-1+y)**2*(-34805+12540*logy*(-1+y)**5+161485*y-291620*y**2
        +249820*y**3-99320*y**4+23746*y**5-16205*y**6+6181*y**7-722*y**8)+r**2*(
        -1+y)**5*(-10942+3840*logy*(-1+y)**2+25724*y-16702*y**2+1280*y**3
        +1190*y**4-616*y**5-109*y**6+122*y**7+5*y**8)+r**4*(-1+y)**3*(-17905
        +22980*logy*(-1+y)**4+94600*y-187860*y**2+171200*y**3-69155*y**4
        +13680*y**5-6730*y**6+1672*y**7+210*y**8)+r**12*(1283-8981*y+26943*y**2
        -44905*y**3+48280*y**4-32802*y**5+13454*y**6-3002*y**7+270*y**8)-2*r**10*(1145
        -8015*y+24045*y**2-40075*y**3+44260*y**4-34539*y**5+18545*y**6-5813*y**7
        +807*y**8)-2*r**8*(-1+y)*(3470+2100*logy*(-1+y)**6-18720*y+40500*y**2
        -43500*y**3+26250*y**4-13479*y**5+9462*y**6-3582*y**7+534*y**8+25*y**9))
        +MBhat**2*(720*(-1+y)**7+1160*r**14*(-1+y)**7-r**2*(-1+y)**5*(-21116
        +8400*logy*(-1+y)**2+50632*y-33716*y**2+4930*y**3+400*y**4-2210*y**5
        +853*y**6+82*y**7+y**8)-6*r**4*(-1+y)**3*(-270+5340*logy*(-1+y)**4
        +6420*y-20310*y**2+22835*y**3-8305*y**4-322*y**5-592*y**6+395*y**7
        +5*y**8)+18*r**12*(242-1694*y+5082*y**2-8845*y**3+9450*y**4-6272*y**5
        +2492*y**6-528*y**7+43*y**8)-12*r**6*(-1+y)**2*(-2880-420*logy*(-1
        +y)**5+14820*y-30690*y**2+31075*y**3-14410*y**4+1591*y**5+392*y**6
        -257*y**7+71*y**8)-4*r**8*(-1+y)*(-3080+690*logy*(-1+y)**6+19170*y
        -49995*y**2+66435*y**3-47820*y**4+18342*y**5-4590*y**6+954*y**7-351*y**8
        +95*y**9)-3*r**10*(660-4620*y+13860*y**2-28250*y**3+36810*y**4-28882*y**5
        +12229*y**6-1547*y**7-725*y**8+225*y**9)))/(90*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (8400*(-1+y)**8+3375*r**16*(-1+y)**8+35*r**14*(-1493+11944*y-41804*y**2
        +83608*y**3-104510*y**4+83608*y**5-45752*y**6+15928*y**7-3017*y**8
        +228*y**9)-r**2*(-1+y)**6*(-847895+243600*logy*(-1+y)**2+1939390*y
        -1213295*y**2+81200*y**3+20300*y**4+8120*y**5+19327*y**6-8954*y**7
        -173*y**8+300*y**9)-105*r**6*(-1+y)**3*(33375+88500*logy*(-1+y)**5
        -255375*y+732000*y**2-1027000*y**3+734750*y**4-235450*y**5+13820*y**6
        +6500*y**7-5533*y**8+2145*y**9)-105*r**10*(-1+y)*(-15255+900*logy*(
        -1+y)**7+105885*y-314505*y**2+517875*y**3-510000*y**4+299700*y**5
        -101677*y**6+23611*y**7-9731*y**8+2177*y**9)-21*r**12*(-5255+4500*logy*(
        -1+y)**8+46540*y-180890*y**2+403780*y**3-567725*y**4+517180*y**5
        -321857*y**6+145996*y**7-49016*y**8+8112*y**9-225*y**10)-35*r**8*(-1
        +y)**2*(240125+168900*logy*(-1+y)**6-1271850*y+2672925*y**2-2719400*y**3
        +1195050*y**4+28680*y**5-181378*y**6+47736*y**7-26139*y**8+8506*y**9
        -15*y**10)-105*r**4*(-1+y)**4*(-53181+29880*logy*(-1+y)**4+242604*y
        -423666*y**2+342204*y**3-115431*y**4+5976*y**5+497*y**6+1972*y**7-1700*y**8
        +620*y**9+9*y**10)-70*MBhat**6*(-1+y)**3*(-120*(-1+y)**5-675*r**10*(
        -1+y)**5+r**2*(-1+y)**3*(55+240*logy*(-1+y)**2+130*y-305*y**2+1940*y**3
        -2905*y**4+1046*y**5+15*y**6)-10*r**8*(224-1120*y+2240*y**2-2498*y**3
        +1540*y**4-479*y**5+57*y**6)+6*r**6*(310+90*logy*(-1+y)**5-1640*y
        +3505*y**2-4435*y**3+3905*y**4-2212*y**5+519*y**6)+12*r**4*(-30+150*y
        -300*y**2+635*y**3-1465*y**4+1947*y**5-1217*y**6+280*y**7))+210*MBhat**5*(
        -1+y)**2*(-240*(-1+y)**6+903*r**12*(-1+y)**6+r**4*(-1+y)**2*(4625
        +3180*logy*(-1+y)**4-15320*y+16620*y**2-11410*y**3+17845*y**4-19380*y**5
        +7646*y**6-914*y**7)+r**2*(-1+y)**4*(-3323+1680*logy*(-1+y)**2+8326*y
        -5843*y**2+2780*y**3-2905*y**4+785*y**5+129*y**6+3*y**7)+5*r**10*(
        -637+3822*y-9555*y**2+13610*y**3-11400*y**4+5505*y**5-1390*y**6+135*y**7)
        +r**8*(2360-14160*y+35400*y**2-55030*y**3+57465*y**4-37914*y**5+12812*y**6
        -1128*y**7-285*y**8)-3*r**6*(-1+y)*(-380+1140*logy*(-1+y)**5+760*y
        +1330*y**2-2480*y**3-2715*y**4+6669*y**5-3461*y**6+515*y**7+50*y**8))
        -105*MBhat**4*(-1+y)*(-1200*(-1+y)**7-2320*r**14*(-1+y)**7+r**2*(-1
        +y)**5*(-36057+14160*logy*(-1+y)**2+86274*y-57297*y**2+8980*y**3
        -1265*y**4-1952*y**5+941*y**6+134*y**7+2*y**8)+5*r**4*(-1+y)**3*(-505
        +11064*logy*(-1+y)**4+13084*y-41754*y**2+46640*y**3-14960*y**4-4476*y**5
        +1610*y**6+64*y**7+9*y**8)-9*r**12*(1013-7091*y+21273*y**2-36955*y**3
        +39375*y**4-26033*y**5+10283*y**6-2157*y**7+172*y**8)+6*r**6*(-1+y)**2*(
        -9980-1770*logy*(-1+y)**5+51670*y-107765*y**2+109605*y**3-49150*y**4
        +1056*y**5+5417*y**6-2097*y**7+284*y**8)+2*r**8*(-1+y)*(-10120+1950*logy*(
        -1+y)**6+62670*y-162525*y**2+211750*y**3-142485*y**4+42426*y**5-4577*y**6
        +930*y**7-1329*y**8+380*y**9)+r**10*(6655-46585*y+139755*y**2-263825*y**3
        +319535*y**4-240087*y**5+101059*y**6-15517*y**7-3780*y**8+1350*y**9))
        +3*MBhat**2*(42000*(-1+y)**8+15950*r**16*(-1+y)**8+70*r**8*(-1+y)**2*(
        -49625+6180*logy*(-1+y)**6+303930*y-778365*y**2+1068720*y**3-823455*y**4
        +332496*y**5-48807*y**6+1494*y**7+1752*y**8-460*y**9)-420*r**6*(-1
        +y)**3*(-21680+22380*logy*(-1+y)**5+86020*y-116090*y**2+41490*y**3
        +34540*y**4-27511*y**5+2422*y**6-151*y**7+235*y**8+5*y**9)-r**2*(-1
        +y)**6*(-2768230+873600*logy*(-1+y)**2+6410060*y-4078630*y**2+291200*y**3
        +112805*y**4+75488*y**5-61726*y**6+5918*y**7+4565*y**8+150*y**9)+70*r**14*(
        -1292+10336*y-36176*y**2+72352*y**3-95375*y**4+82852*y**5-46844*y**6
        +16480*y**7-3221*y**8+258*y**9)+35*r**4*(-1+y)**4*(266104-216240*logy*(
        -1+y)**4-1280656*y+2353464*y**2-2001456*y**3+721149*y**4-50634*y**5
        -10833*y**6+7932*y**7-3453*y**8-178*y**9+y**10)-35*r**10*(-1+y)*(12790
        +5160*logy*(-1+y)**7-94690*y+302130*y**2-539670*y**3+550485*y**4
        -318369*y**5+90318*y**6-12768*y**7-297*y**8-199*y**9+190*y**10)-21*r**12*(
        -3920+31360*y-109760*y**2+219520*y**3-325755*y**4+347562*y**5-248061*y**6
        +110456*y**7-27217*y**8+2230*y**9+225*y**10))+3*MBhat*(-16800*(-1+y)**8
        +5590*r**16*(-1+y)**8+70*r**8*(-1+y)**2*(115575+33900*logy*(-1+y)**6
        -659550*y+1547175*y**2-1893400*y**3+1250550*y**4-404166*y**5+48177*y**6
        -13350*y**7+5307*y**8-638*y**9)+70*r**14*(-706+5648*y-19768*y**2+39536*y**3
        -49420*y**4+42497*y**5-24136*y**6+8516*y**7-1672*y**8+135*y**9)+r**2*(
        -1+y)**6*(-1407200+420000*logy*(-1+y)**2+3234400*y-2037200*y**2+140000*y**3
        +35000*y**4+37268*y**5+5411*y**6-14836*y**7+3527*y**8+270*y**9)-70*r**6*(
        -1+y)**3*(17738-138360*logy*(-1+y)**5+49670*y-445240*y**2+906440*y**3
        -799120*y**4+300377*y**5-31864*y**6+7265*y**7-3013*y**8-240*y**9+3*y**10)
        +35*r**4*(-1+y)**4*(-205138+131640*logy*(-1+y)**4+952192*y-1691568*y**2
        +1390992*y**3-479388*y**4+23922*y**5+10893*y**6-3312*y**7-12*y**8+838*y**9
        +5*y**10)-35*r**10*(-1+y)*(-23920+9600*logy*(-1+y)**7+157840*y-439920*y**2
        +666000*y**3-582000*y**4+303648*y**5-107331*y**6+42939*y**7-11727*y**8
        +1621*y**9+50*y**10)+7*r**12*(21850-174800*y+611800*y**2-1223600*y**3
        +1529500*y**4-1318214*y**5+822241*y**6-365084*y**7+99076*y**8-13134*y**9
        +285*y**10))+5*MBhat**3*(-33600*(-1+y)**8+20250*r**16*(-1+y)**8-42*r**6*(
        -1+y)**3*(122145-40620*logy*(-1+y)**5-570105*y+1038660*y**2-899930*y**3
        +347965*y**4-42383*y**5+12396*y**6-4398*y**7+450*y**8)+21*r**4*(-1
        +y)**4*(-126336+163440*logy*(-1+y)**4+668784*y-1330056*y**2+1209484*y**3
        -468421*y**4+56066*y**5-17294*y**6+5076*y**7+771*y**8+6*y**9)+r**2*(
        -1+y)**6*(-1602348+551040*logy*(-1+y)**2+3755736*y-2428908*y**2+201152*y**3
        +120407*y**4-52024*y**5-12663*y**6+11136*y**7+786*y**8+6*y**9)-14*r**8*(
        -1+y)**2*(75655+29700*logy*(-1+y)**6-424230*y+971475*y**2-1131380*y**3
        +717735*y**4-265770*y**5+93521*y**6-18240*y**7+966*y**8+428*y**9)+14*r**14*(
        -5711+45688*y-159908*y**2+329686*y**3-430430*y**4+366268*y**5-201404*y**6
        +67984*y**7-12413*y**8+870*y**9)-21*r**12*(-1768+14144*y-49504*y**2
        +118100*y**3-171355*y**4+147902*y**5-71008*y**6+9488*y**7+8669*y**8
        -4770*y**9+774*y**10)-21*r**10*(-11460+91680*y-320880*y**2+622360*y**3
        -769415*y**4+668214*y**5-437982*y**6+221622*y**7-83079*y**8+22850*y**9
        -4360*y**10+450*y**11)))/(6300*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (8400*(-1+y)**9+2025*r**18*(-1+y)**9-r**2*(-1+y)**7*(-1207055+319200*logy*(
        -1+y)**2+2733310*y-1685855*y**2+106400*y**3+26600*y**4+10640*y**5
        +5320*y**6+5584*y**7+9067*y**8-5766*y**9+75*y**10)+20*r**16*(2132-19188*y
        +76752*y**2-179088*y**3+268632*y**4-268632*y**5+179088*y**6-83652*y**7
        +25398*y**8-4247*y**9+285*y**10)-210*r**6*(-1+y)**4*(141234+139020*logy*(
        -1+y)**5-845190*y+2037930*y**2-2501330*y**3+1598215*y**4-458663*y**5
        +23170*y**6+4014*y**7+631*y**8-1485*y**9+994*y**10)-168*r**12*(-1+y)*(16010
        +900*logy*(-1+y)**8-127180*y+441530*y**2-874660*y**3+1080725*y**4
        -851980*y**5+417590*y**6-120408*y**7+21929*y**8-7044*y**9+1388*y**10)
        -70*r**14*(1034+1620*logy*(-1+y)**9-10926*y+50994*y**2-138426*y**3
        +241659*y**4-282483*y**5+222342*y**6-122622*y**7+50163*y**8-15317*y**9
        +2298*y**10-60*y**11)+70*r**10*(-1+y)**2*(437620-225300*logy*(-1+y)**7
        -2838040*y+7725570*y**2-11298850*y**3+9327475*y**4-4019385*y**5+551245*y**6
        +152177*y**7-40581*y**8+15071*y**9-5117*y**10+15*y**11)+140*r**8*(-1
        +y)**3*(-74960-312000*logy*(-1+y)**6+137760*y+591600*y**2-2348800*y**3
        +3321600*y**4-2264640*y**5+689440*y**6-45252*y**7-6798*y**8+5195*y**9
        -2760*y**10+15*y**11)-40*r**4*(-1+y)**5*(-322778+146580*logy*(-1+y)**4
        +1437692*y-2449698*y**2+1926292*y**3-628153*y**4+29316*y**5+4886*y**6
        +2164*y**7-39*y**8-799*y**9+850*y**10+15*y**11)-70*MBhat**8*(-1+y)**4*(
        -120*(-1+y)**5-675*r**10*(-1+y)**5+r**2*(-1+y)**3*(55+240*logy*(-1
        +y)**2+130*y-305*y**2+1940*y**3-2905*y**4+1046*y**5+15*y**6)-10*r**8*(224
        -1120*y+2240*y**2-2498*y**3+1540*y**4-479*y**5+57*y**6)+6*r**6*(310
        +90*logy*(-1+y)**5-1640*y+3505*y**2-4435*y**3+3905*y**4-2212*y**5
        +519*y**6)+12*r**4*(-30+150*y-300*y**2+635*y**3-1465*y**4+1947*y**5
        -1217*y**6+280*y**7))+280*MBhat**7*(-1+y)**3*(-240*(-1+y)**6+903*r**12*(
        -1+y)**6+r**4*(-1+y)**2*(4625+3180*logy*(-1+y)**4-15320*y+16620*y**2
        -11410*y**3+17845*y**4-19380*y**5+7646*y**6-914*y**7)+r**2*(-1+y)**4*(
        -3323+1680*logy*(-1+y)**2+8326*y-5843*y**2+2780*y**3-2905*y**4+785*y**5
        +129*y**6+3*y**7)+5*r**10*(-637+3822*y-9555*y**2+13610*y**3-11400*y**4
        +5505*y**5-1390*y**6+135*y**7)+r**8*(2360-14160*y+35400*y**2-55030*y**3
        +57465*y**4-37914*y**5+12812*y**6-1128*y**7-285*y**8)-3*r**6*(-1+y)*(
        -380+1140*logy*(-1+y)**5+760*y+1330*y**2-2480*y**3-2715*y**4+6669*y**5
        -3461*y**6+515*y**7+50*y**8))-140*MBhat**6*(-1+y)**2*(-1680*(-1+y)**7
        -3480*r**14*(-1+y)**7+r**2*(-1+y)**5*(-50998+19920*logy*(-1+y)**2
        +121916*y-80878*y**2+13030*y**3-2930*y**4-1694*y**5+1029*y**6+186*y**7
        +3*y**8)+2*r**4*(-1+y)**3*(-1715+39300*logy*(-1+y)**4+46160*y-147840*y**2
        +164695*y**3-49885*y**4-21414*y**5+9826*y**6-865*y**7+30*y**8)-54*r**12*(257
        -1799*y+5397*y**2-9370*y**3+9975*y**4-6587*y**5+2597*y**6-543*y**7
        +43*y**8)+12*r**6*(-1+y)**2*(-7100-1350*logy*(-1+y)**5+36850*y-77075*y**2
        +78530*y**3-34740*y**4-535*y**5+5025*y**6-1840*y**7+213*y**8)+4*r**8*(
        -1+y)*(-7040+1260*logy*(-1+y)**6+43500*y-112530*y**2+145315*y**3
        -94665*y**4+24084*y**5+13*y**6-24*y**7-978*y**8+285*y**9)+r**10*(11330
        -79310*y+237930*y**2-442900*y**3+528640*y**4-393528*y**5+165431*y**6
        -26393*y**7-5385*y**8+2025*y**9))+2*MBhat**2*(117600*(-1+y)**9+15225*r**18*(
        -1+y)**9+210*r**10*(-1+y)**2*(75560+5700*logy*(-1+y)**7-534620*y
        +1623810*y**2-2746250*y**3+2796125*y**4-1706781*y**5+581461*y**6-80563*y**7
        +6104*y**8+1072*y**9-318*y**10)-280*r**8*(-1+y)**3*(497195+300300*logy*(
        -1+y)**6-2682870*y+5806275*y**2-6240200*y**3+3178650*y**4-368175*y**5
        -242680*y**6+39969*y**7+834*y**8+532*y**9+30*y**10)+60*r**16*(2002
        -18018*y+72072*y**2-168168*y**3+252252*y**4-266742*y**5+194250*y**6
        -94860*y**7+29520*y**8-5215*y**9+387*y**10)-r**2*(-1+y)**7*(-12540535
        +3511200*logy*(-1+y)**2+28592270*y-17807335*y**2+1170400*y**3+292600*y**4
        +121324*y**5+222152*y**6-48988*y**7-31651*y**8+5838*y**9+405*y**10)
        -84*r**4*(-1+y)**5*(-1096450+592500*logy*(-1+y)**4+4978300*y-8652450*y**2
        +6953300*y**3-2330825*y**4+120954*y**5+13604*y**6+13744*y**7-4331*y**8
        +2014*y**9+460*y**10)-28*r**14*(6400-57600*y+230400*y**2-537600*y**3
        +806400*y**4-928683*y**5+812742*y**6-499443*y**7+199917*y**8-47713*y**9
        +5100*y**10)+420*r**6*(-1+y)**4*(-118136-356280*logy*(-1+y)**5+946960*y
        -2784620*y**2+3972220*y**3-2876810*y**4+929449*y**5-56948*y**6-8669*y**7
        +628*y**8-1355*y**9-32*y**10+y**11)-84*r**12*(-1+y)*(-7510+6900*logy*(
        -1+y)**8+66980*y-262030*y**2+588460*y**3-832175*y**4+705454*y**5
        -349412*y**6+81220*y**7-9353*y**8-1201*y**9+152*y**10+95*y**11))+4*MBhat*(
        -16800*(-1+y)**9+2580*r**18*(-1+y)**9+r**2*(-1+y)**7*(-2108060+571200*logy*(
        -1+y)**2+4787320*y-2964860*y**2+190400*y**3+47600*y**4+19040*y**5
        +12103*y**6+25660*y**7-11879*y**8-834*y**9+150*y**10)+20*r**16*(1571
        -14139*y+56556*y**2-131964*y**3+197946*y**4-197946*y**5+141624*y**6
        -68976*y**7+21294*y**8-3716*y**9+270*y**10)-35*r**10*(-1+y)**2*(571480
        -148800*logy*(-1+y)**7-3851560*y+11033880*y**2-17348200*y**3+16046200*y**4
        -8586120*y**5+2356483*y**6-212911*y**7+18615*y**8-8869*y**9+1162*y**10)
        +105*r**6*(-1+y)**4*(260696+360480*logy*(-1+y)**5-1663960*y+4229120*y**2
        -5430720*y**3+3616560*y**4-1083792*y**5+62043*y**6+7397*y**7-712*y**8
        +1080*y**9+565*y**10-5*y**11)+140*r**8*(-1+y)**3*(213653+270060*logy*(
        -1+y)**6-1011858*y+1719465*y**2-942320*y**3-643560*y**4+1067604*y**5
        -447001*y**6+38883*y**7-1101*y**8+2290*y**9+108*y**10-3*y**11)-42*r**12*(
        -1+y)*(27820+8700*logy*(-1+y)**8-213860*y+713710*y**2-1346220*y**3
        +1560975*y**4-1126980*y**5+508209*y**6-148979*y**7+43003*y**8-8710*y**9
        +1087*y**10+25*y**11)+2*r**4*(-1+y)**5*(-9683060+4710300*logy*(-1+y)**4
        +43442540*y-74584410*y**2+59143540*y**3-19496185*y**4+942060*y**5+179039*y**6
        +13519*y**7+30309*y**8-10306*y**9+12739*y**10+135*y**11)+7*r**14*(-21160
        +190440*y-761760*y**2+1777440*y**3-2666160*y**4+2666160*y**5-1887261*y**6
        +981909*y**7-370086*y**8+87334*y**9-10581*y**10+285*y**11))+20*MBhat**5*(
        -1+y)*(-23520*(-1+y)**8+20250*r**16*(-1+y)**8+42*r**6*(-1+y)**3*(
        -87340+28080*logy*(-1+y)**5+408620*y-747040*y**2+650110*y**3-248645*y**4
        +18637*y**5+3809*y**6-1783*y**7+272*y**8)+21*r**4*(-1+y)**4*(-90526
        +117480*logy*(-1+y)**4+479584*y-954336*y**2+867084*y**3-330111*y**4
        +28706*y**5-3834*y**6+1732*y**7+351*y**8+6*y**9)+r**2*(-1+y)**6*(-1142784
        +389760*logy*(-1+y)**2+2675328*y-1727424*y**2+147392*y**3+70427*y**4
        -26152*y**5-8085*y**6+6012*y**7+576*y**8+6*y**9)-14*r**8*(-1+y)**2*(54835
        +17100*logy*(-1+y)**6-311910*y+728475*y**2-870380*y**3+560235*y**4
        -184896*y**5+36749*y**6+3252*y**7-2238*y**8+278*y**9)+28*r**14*(-3178
        +25424*y-88984*y**2+182903*y**3-237790*y**4+201194*y**5-109732*y**6
        +36572*y**7-6529*y**8+435*y**9)-21*r**12*(-4334+34672*y-121352*y**2
        +261796*y**3-357725*y**4+310066*y**5-163520*y**6+42400*y**7+2125*y**8
        -4230*y**9+774*y**10)-21*r**10*(-6880+55040*y-192640*y**2+365880*y**3
        -432075*y**4+353018*y**5-225646*y**6+124190*y**7-56599*y**8+19622*y**9
        -4360*y**10+450*y**11))+5*MBhat**4*(117600*(-1+y)**9+24675*r**18*(-1
        +y)**9+56*r**8*(-1+y)**3*(-155150+21600*logy*(-1+y)**6+952500*y-2446050*y**2
        +3365410*y**3-2597220*y**4+1055403*y**5-155740*y**6+4593*y**7+4662*y**8
        -808*y**9)-42*r**6*(-1+y)**4*(-661088+638160*logy*(-1+y)**5+2667280*y
        -3739160*y**2+1616560*y**3+768255*y**4-753375*y**5+71654*y**6+9606*y**7
        -57*y**8+165*y**9)-56*r**4*(-1+y)**5*(-490569+396420*logy*(-1+y)**4
        +2358696*y-4330884*y**2+3681164*y**3-1325951*y**4+94503*y**5+11818*y**6
        -4492*y**7+2970*y**8+225*y**9)-r**2*(-1+y)**7*(-7976685+2486400*logy*(
        -1+y)**2+18439770*y-11706285*y**2+816704*y**3+357686*y**4+144956*y**5
        -112014*y**6+912*y**7+10863*y**8+570*y**9+3*y**10)+28*r**10*(-1+y)**2*(
        -28300-13950*logy*(-1+y)**7+212050*y-684975*y**2+1248755*y**3-1253540*y**4
        +659352*y**5-94124*y**6-39922*y**7+24603*y**8-4409*y**9+110*y**10)
        +4*r**16*(25276-227484*y+909936*y**2-2171484*y**3+3358656*y**4-3500406*y**5
        +2475354*y**6-1162116*y**7+341154*y**8-54781*y**9+3375*y**10)-14*r**14*(3824
        -34416*y+137664*y**2-371064*y**3+599391*y**4-586587*y**5+334824*y**6
        -65556*y**7-51291*y**8+44209*y**9-14082*y**10+1740*y**11)-168*r**12*(705
        -6345*y+25380*y**2-54172*y**3+87721*y**4-123192*y**5+137147*y**6-111252*y**7
        +63504*y**8-25131*y**9+6629*y**10-1080*y**11+86*y**12))+4*MBhat**3*(
        -117600*(-1+y)**9+16875*r**18*(-1+y)**9+70*r**8*(-1+y)**3*(791675+218700*logy*(
        -1+y)**6-4531350*y+10672275*y**2-13136200*y**3+8764620*y**4-2875422*y**5
        +288937*y**6-6858*y**7+1413*y**8-290*y**9)+105*r**6*(-1+y)**4*(-137954
        +681480*logy*(-1+y)**5+8290*y+1687120*y**2-3958720*y**3+3687865*y**4
        -1427875*y**5+126622*y**6-28*y**7+4981*y**8+285*y**9+6*y**10)+r**2*(
        -1+y)**7*(-10272465+3007200*logy*(-1+y)**2+23552130*y-14783265*y**2
        +1002400*y**3+244825*y**4+266854*y**5+24486*y**6-70674*y**7+9144*y**8
        +2970*y**9+75*y**10)-35*r**10*(-1+y)**2*(-144590+36000*logy*(-1+y)**7
        +976130*y-2802390*y**2+4418650*y**3-4071925*y**4+2236431*y**5-706834*y**6
        +167230*y**7-14649*y**8-901*y**9+448*y**10)+10*r**16*(9317-83853*y
        +335412*y**2-782628*y**3+1222242*y**4-1295658*y**5+936852*y**6-453708*y**7
        +139527*y**8-24203*y**9+1740*y**10)+5*r**4*(-1+y)**5*(-11035914+6974520*logy*(
        -1+y)**4+51118176*y-90626304*y**2+74366576*y**3-25541510*y**4+1285410*y**5
        +412489*y**6-29182*y**7+1578*y**8+19914*y**9+549*y**10-6*y**11)-35*r**14*(1802
        -16218*y+64872*y**2-151368*y**3+280275*y**4-360105*y**5+303270*y**6
        -167472*y**7+56673*y**8-8867*y**9-432*y**10+258*y**11)-21*r**12*(35290
        -317610*y+1270440*y**2-2964360*y**3+4326490*y**4-4166944*y**5+2764467*y**6
        -1310163*y**7+458112*y**8-115348*y**9+23231*y**10-4055*y**11+450*y**12)))/(6300*r**2*(
        -1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr14(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r14 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (60*(-1+y)**6+630*r**10*(-1+y)**6-6*r**4*(-1+y)**2*(60-240*y+360*y**2
        -260*y**3+105*y**4-20*y**5+y**6)-r**2*(-1+y)**4*(-130+300*logy*(-1
        +y)**2+560*y-580*y**2+100*y**3+115*y**4-74*y**5+15*y**6)-3*r**6*(
        -1+y)*(520+180*logy*(-1+y)**5-2780*y+6010*y**2-7110*y**3+5095*y**4
        -2039*y**5+352*y**6)+2*r**8*(-1010+6060*y-15150*y**2+21310*y**3-18375*y**4
        +9390*y**5-2573*y**6+288*y**7))/(90*r**2*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (120*(-1+y)**7+378*r**12*(-1+y)**7-4*r**2*(-1+y)**5*(-208+210*logy*(
        -1+y)**2+626*y-523*y**2+70*y**3+25*y**4+37*y**5-33*y**6+9*y**7)
        -3*r**6*(-1+y)**2*(500-720*logy*(-1+y)**5-1780*y+1760*y**2+640*y**3
        -2715*y**4+2691*y**5-1147*y**6+195*y**7)+r**4*(-1+y)**3*(-3370+1020*logy*(
        -1+y)**4+14500*y-23790*y**2+17900*y**3-5045*y**4-744*y**5+452*y**6
        +40*y**7-15*y**8)+r**8*(-1+y)*(2840-1620*logy*(-1+y)**6-18660*y
        +51510*y**2-76780*y**3+70485*y**4-43902*y**5+16868*y**6-3246*y**7+165*y**8)
        +r**10*(2300-16100*y+48300*y**2-80500*y**3+83935*y**4-57765*y**5+24731*y**6
        -5837*y**7+576*y**8)-2*MBhat**2*(-1+y)*(-60*(-1+y)**6-630*r**10*(-1
        +y)**6+6*r**4*(-1+y)**2*(60-240*y+360*y**2-260*y**3+105*y**4-20*y**5
        +y**6)+r**2*(-1+y)**4*(-130+300*logy*(-1+y)**2+560*y-580*y**2+100*y**3
        +115*y**4-74*y**5+15*y**6)+3*r**6*(-1+y)*(520+180*logy*(-1+y)**5
        -2780*y+6010*y**2-7110*y**3+5095*y**4-2039*y**5+352*y**6)-2*r**8*(
        -1010+6060*y-15150*y**2+21310*y**3-18375*y**4+9390*y**5-2573*y**6
        +288*y**7))+2*MBhat*(-120*(-1+y)**7+777*r**12*(-1+y)**7+r**2*(-1+y)**5*(
        -877+840*logy*(-1+y)**2+2594*y-2137*y**2+295*y**3+160*y**4-50*y**5
        +24*y**6+3*y**7)+3*r**4*(-1+y)**3*(395+120*logy*(-1+y)**4-1460*y
        +1950*y**2-1170*y**3+355*y**4-8*y**5-76*y**6+38*y**7)-3*r**6*(-1+y)**2*(
        -570+2850*y-5700*y**2+6050*y**3-3365*y**4+49*y**5+942*y**6-431*y**7
        +55*y**8)+3*r**10*(935-6545*y+19635*y**2-33870*y**3+36690*y**4-24969*y**5
        +10283*y**6-2309*y**7+210*y**8)+r**8*(-3550+24850*y-74550*y**2+128420*y**3
        -139895*y**4+95913*y**5-37741*y**6+6205*y**7+636*y**8-288*y**9)))/(180*r**2*(
        -1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (600*(-1+y)**8+840*r**14*(-1+y)**8+20*r**8*(-1+y)**2*(2380+1620*logy*(
        -1+y)**6-12660*y+26790*y**2-27620*y**3+12615*y**4+816*y**5-4295*y**6
        +1896*y**7-282*y**8)-10*r**6*(-1+y)**3*(-560-5640*logy*(-1+y)**5+8440*y
        -30980*y**2+49780*y**3-38990*y**4+12778*y**5+799*y**6-1043*y**7+64*y**8)
        -r**2*(-1+y)**6*(-11230+6600*logy*(-1+y)**2+29060*y-21130*y**2+2200*y**3
        +550*y**4+40*y**5+1107*y**6-654*y**7+117*y**8)+10*r**4*(-1+y)**4*(
        -5789+1140*logy*(-1+y)**4+24296*y-38724*y**2+28096*y**3-8164*y**4
        +462*y**5-398*y**6+116*y**7+87*y**8-18*y**9)+5*r**10*(-1+y)*(-2290
        -2160*logy*(-1+y)**7+18190*y-62130*y**2+118670*y**3-137570*y**4+103770*y**5
        -56473*y**6+19711*y**7-3483*y**8+165*y**9)+2*r**12*(-4115+32920*y
        -115220*y**2+230440*y**3-288050*y**4+237730*y**5-134984*y**6+49192*y**7
        -10037*y**8+864*y**9)-10*MBhat**4*(-1+y)**2*(-60*(-1+y)**6-630*r**10*(
        -1+y)**6+6*r**4*(-1+y)**2*(60-240*y+360*y**2-260*y**3+105*y**4-20*y**5
        +y**6)+r**2*(-1+y)**4*(-130+300*logy*(-1+y)**2+560*y-580*y**2+100*y**3
        +115*y**4-74*y**5+15*y**6)+3*r**6*(-1+y)*(520+180*logy*(-1+y)**5
        -2780*y+6010*y**2-7110*y**3+5095*y**4-2039*y**5+352*y**6)-2*r**8*(
        -1010+6060*y-15150*y**2+21310*y**3-18375*y**4+9390*y**5-2573*y**6
        +288*y**7))+20*MBhat**3*(-1+y)*(-120*(-1+y)**7+777*r**12*(-1+y)**7
        +r**2*(-1+y)**5*(-877+840*logy*(-1+y)**2+2594*y-2137*y**2+295*y**3
        +160*y**4-50*y**5+24*y**6+3*y**7)+3*r**4*(-1+y)**3*(395+120*logy*(
        -1+y)**4-1460*y+1950*y**2-1170*y**3+355*y**4-8*y**5-76*y**6+38*y**7)
        -3*r**6*(-1+y)**2*(-570+2850*y-5700*y**2+6050*y**3-3365*y**4+49*y**5
        +942*y**6-431*y**7+55*y**8)+3*r**10*(935-6545*y+19635*y**2-33870*y**3
        +36690*y**4-24969*y**5+10283*y**6-2309*y**7+210*y**8)+r**8*(-3550+24850*y
        -74550*y**2+128420*y**3-139895*y**4+95913*y**5-37741*y**6+6205*y**7
        +636*y**8-288*y**9))+4*MBhat*(-600*(-1+y)**8+925*r**14*(-1+y)**8+5*r**4*(
        -1+y)**4*(4195+60*logy*(-1+y)**4-16720*y+24960*y**2-16520*y**3+3665*y**4
        +816*y**5-314*y**6-40*y**7+30*y**8)+r**2*(-1+y)**6*(-9820+6000*logy*(
        -1+y)**2+25640*y-18820*y**2+2000*y**3+350*y**4+929*y**5-258*y**6
        +9*y**7+30*y**8)+15*r**6*(-1+y)**3*(-25-780*logy*(-1+y)**5+905*y
        -3760*y**2+6360*y**3-5445*y**4+2511*y**5-281*y**6-140*y**7+43*y**8)
        -5*r**8*(-1+y)**2*(3900-23400*y+58500*y**2-78000*y**3+61485*y**4-30684*y**5
        +7292*y**6+219*y**7-447*y**8+55*y**9)+9*r**12*(-595+4760*y-16660*y**2
        +33320*y**3-43000*y**4+37694*y**5-21812*y**6+7912*y**7-1619*y**8+140*y**9)
        -5*r**10*(-2600+20800*y-72800*y**2+145600*y**3-186905*y**4+164527*y**5
        -99356*y**6+38536*y**7-8687*y**8+885*y**9))+10*MBhat**2*(360*(-1+y)**8
        +916*r**14*(-1+y)**8-r**2*(-1+y)**6*(-4364+3120*logy*(-1+y)**2+11848*y
        -9044*y**2+980*y**3+503*y**4+158*y**5-92*y**6+46*y**7+y**8)-r**4*(
        -1+y)**4*(7121+1020*logy*(-1+y)**4-27464*y+39156*y**2-24964*y**3
        +6811*y**4-756*y**5+410*y**6-128*y**7+30*y**8)+3*r**6*(-1+y)**3*(-580
        +1200*logy*(-1+y)**5+1700*y-400*y**2-3280*y**3+5205*y**4-5077*y**5
        +2731*y**6-843*y**7+112*y**8)+3*r**12*(-1153+9224*y-32284*y**2+66188*y**3
        -87244*y**4+75404*y**5-42364*y**6+14768*y**7-2845*y**8+222*y**9)-r**8*(
        -1+y)**2*(-300+2760*logy*(-1+y)**6+4560*y-19680*y**2+36200*y**3
        -43155*y**4+47442*y**5-34820*y**6+15438*y**7-3789*y**8+384*y**9)+r**10*(2900
        -23200*y+81200*y**2-171100*y**3+235310*y**4-204700*y**5+103103*y**6
        -21904*y**7-4741*y**8+3762*y**9-630*y**10)))/(900*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-140*MBhat**6*(-1+y)**3*(-60*(-1+y)**6-630*r**10*(-1+y)**6+6*r**4*(
        -1+y)**2*(60-240*y+360*y**2-260*y**3+105*y**4-20*y**5+y**6)+r**2*(
        -1+y)**4*(-130+300*logy*(-1+y)**2+560*y-580*y**2+100*y**3+115*y**4
        -74*y**5+15*y**6)+3*r**6*(-1+y)*(520+180*logy*(-1+y)**5-2780*y
        +6010*y**2-7110*y**3+5095*y**4-2039*y**5+352*y**6)-2*r**8*(-1010+6060*y
        -15150*y**2+21310*y**3-18375*y**4+9390*y**5-2573*y**6+288*y**7))+420*MBhat**5*(
        -1+y)**2*(-120*(-1+y)**7+777*r**12*(-1+y)**7+r**2*(-1+y)**5*(-877
        +840*logy*(-1+y)**2+2594*y-2137*y**2+295*y**3+160*y**4-50*y**5+24*y**6
        +3*y**7)+3*r**4*(-1+y)**3*(395+120*logy*(-1+y)**4-1460*y+1950*y**2
        -1170*y**3+355*y**4-8*y**5-76*y**6+38*y**7)-3*r**6*(-1+y)**2*(-570
        +2850*y-5700*y**2+6050*y**3-3365*y**4+49*y**5+942*y**6-431*y**7+55*y**8)
        +3*r**10*(935-6545*y+19635*y**2-33870*y**3+36690*y**4-24969*y**5+10283*y**6
        -2309*y**7+210*y**8)+r**8*(-3550+24850*y-74550*y**2+128420*y**3-139895*y**4
        +95913*y**5-37741*y**6+6205*y**7+636*y**8-288*y**9))-210*MBhat**4*(
        -1+y)*(-600*(-1+y)**8-1832*r**14*(-1+y)**8+2*r**2*(-1+y)**6*(-3948
        +2700*logy*(-1+y)**2+10596*y-7998*y**2+840*y**3+453*y**4+84*y**5
        -26*y**6+28*y**7+y**8)+3*r**4*(-1+y)**4*(3624+1020*logy*(-1+y)**4
        -13476*y+18174*y**2-10676*y**3+2859*y**4-752*y**5+424*y**6-72*y**7
        +15*y**8)-3*r**6*(-1+y)**3*(-660+1680*logy*(-1+y)**5+1620*y+960*y**2
        -5920*y**3+7695*y**4-7463*y**5+4315*y**6-1491*y**7+224*y**8)-12*r**12*(
        -608+4864*y-17024*y**2+34858*y**3-45827*y**4+39466*y**5-22064*y**6
        +7636*y**7-1454*y**8+111*y**9)+r**8*(-1+y)**2*(2240+3900*logy*(-1
        +y)**6-9540*y+12150*y**2-4380*y**3-15825*y**4+50982*y**5-52772*y**6
        +27630*y**7-7413*y**8+768*y**9)+3*r**10*(-2700+21600*y-75600*y**2
        +157000*y**3-211685*y**4+183700*y**5-96234*y**6+24792*y**7+1023*y**8
        -2316*y**9+420*y**10))-7*(-1200*(-1+y)**9-900*r**16*(-1+y)**9-30*r**6*(
        -1+y)**4*(9255+12900*logy*(-1+y)**5-59175*y+150600*y**2-193600*y**3
        +129050*y**4-38710*y**5+2523*y**6-731*y**7+538*y**8+10*y**9)+r**2*(
        -1+y)**7*(-47930+20400*logy*(-1+y)**2+116260*y-78530*y**2+6800*y**3
        +1700*y**4+680*y**5-1013*y**6+3466*y**7-1433*y**8+120*y**9)+15*r**10*(
        -1+y)**2*(17370-9000*logy*(-1+y)**7-112590*y+306270*y**2-447450*y**3
        +368700*y**4-158220*y**5+16787*y**6+18915*y**7-8491*y**8+1109*y**9)
        +3*r**12*(-1+y)*(-4940+9000*logy*(-1+y)**8+48520*y-205820*y**2+495640*y**3
        -745550*y**4+722440*y**5-468999*y**6+229232*y**7-73492*y**8+11864*y**9
        -495*y**10)-10*r**8*(-1+y)**3*(20150+36600*logy*(-1+y)**6-84300*y
        +100950*y**2+48400*y**3-219300*y**4+197520*y**5-66715*y**6-3714*y**7
        +4920*y**8-326*y**9+15*y**10)+3*r**4*(-1+y)**5*(78780-5400*logy*(-1
        +y)**4-320520*y+491580*y**2-338520*y**3+90030*y**4-1080*y**5-1163*y**6
        +1212*y**7+712*y**8-908*y**9+117*y**10)-2*r**14*(6455-58095*y+232380*y**2
        -542220*y**3+813330*y**4-813330*y**5+555072*y**6-267648*y**7+84717*y**8
        -15173*y**9+1152*y**10))+6*MBhat*(-8400*(-1+y)**9+4810*r**16*(-1+y)**9
        +70*r**8*(-1+y)**3*(-6275-8100*logy*(-1+y)**6+29550*y-49575*y**2
        +25600*y**3+21300*y**4-34740*y**5+18714*y**6-1986*y**7-822*y**8+154*y**9)
        +r**2*(-1+y)**7*(-280400+126000*logy*(-1+y)**2+686800*y-469400*y**2
        +42000*y**3+10500*y**4-2772*y**5+17304*y**6-2421*y**7-1122*y**8+351*y**9)
        -70*r**6*(-1+y)**4*(3817+11460*logy*(-1+y)**5-30545*y+89740*y**2
        -127940*y**3+92620*y**4-29153*y**5-191*y**6+919*y**7+97*y**8-15*y**9
        +3*y**10)+14*r**4*(-1+y)**5*(56030+3600*logy*(-1+y)**4-220520*y+323580*y**2
        -208520*y**3+48530*y**4-534*y**5+1791*y**6+636*y**7-714*y**8+66*y**9
        +15*y**10)-35*r**10*(-1+y)**2*(-12760+89320*y-267960*y**2+446600*y**3
        -446600*y**4+277512*y**5-115320*y**6+26049*y**7-1521*y**8-415*y**9
        +55*y**10)+14*r**14*(2905-26145*y+104580*y**2-244020*y**3+366030*y**4
        -375669*y**5+274491*y**6-136764*y**7+43641*y**8-7999*y**9+630*y**10)
        +14*r**12*(-11350+102150*y-408600*y**2+953400*y**3-1430100*y**4+1456422*y**5
        -1056723*y**6+549351*y**7-190779*y**8+40621*y**9-4536*y**10+144*y**11))
        +3*MBhat**2*(42000*(-1+y)**9+25190*r**16*(-1+y)**9+70*r**6*(-1+y)**4*(7447
        +19020*logy*(-1+y)**5-56255*y+160060*y**2-223460*y**3+161995*y**4
        -56447*y**5+3076*y**6+268*y**7+496*y**8-60*y**9)+70*r**8*(-1+y)**3*(24035
        +11220*logy*(-1+y)**6-132990*y+298815*y**2-342320*y**3+205815*y**4
        -53334*y**5-19119*y**6+14472*y**7-3066*y**8+252*y**9)-r**2*(-1+y)**7*(
        -1115180+546000*logy*(-1+y)**2+2776360*y-1934180*y**2+182000*y**3
        +23135*y**4+58310*y**5+21581*y**6-10672*y**7+2486*y**8+360*y**9)-35*r**4*(
        -1+y)**5*(58128+12240*logy*(-1+y)**4-220272*y+305928*y**2-179472*y**3
        +30561*y**4+5268*y**5+772*y**6-124*y**7-147*y**8+80*y**9-2*y**10)
        -35*r**10*(-1+y)**2*(-10300+10320*logy*(-1+y)**7+61780*y-149220*y**2
        +176460*y**3-114885*y**4+41319*y**5+43501*y**6-47779*y**7+19428*y**8
        -4128*y**9+384*y**10)+70*r**14*(1981-17829*y+71316*y**2-166404*y**3
        +256032*y**4-273672*y**5+201180*y**6-99540*y**7+31545*y**8-5725*y**9
        +444*y**10)-7*r**12*(27160-244440*y+977760*y**2-2281440*y**3+3591915*y**4
        -4107195*y**5+3295398*y**6-1731342*y**7+565443*y**8-96667*y**9+2148*y**10
        +1260*y**11))+4*MBhat**3*(-42000*(-1+y)**9+36450*r**16*(-1+y)**9+210*r**6*(
        -1+y)**4*(-1155-2940*logy*(-1+y)**5+8715*y-24780*y**2+34155*y**3
        -24350*y**4+8791*y**5+55*y**6-723*y**7+132*y**8)+105*r**4*(-1+y)**5*(11562
        +3240*logy*(-1+y)**4-43008*y+58032*y**2-32750*y**3+5535*y**4+619*y**5
        +310*y**6-96*y**7+33*y**8+3*y**9)+r**2*(-1+y)**7*(-826680+462000*logy*(
        -1+y)**2+2115360*y-1519680*y**2+144130*y**3+47215*y**4+48601*y**5
        -7497*y**6+1296*y**7+1440*y**8+15*y**9)+70*r**8*(-1+y)**3*(-15115+2340*logy*(
        -1+y)**6+93030*y-239595*y**2+330410*y**3-272295*y**4+150534*y**5
        -50966*y**6+11556*y**7-2379*y**8+340*y**9)-105*r**10*(-1+y)**2*(4800
        -33600*y+100800*y**2-164770*y**3+172365*y**4-137326*y**5+80782*y**6
        -36003*y**7+11849*y**8-2407*y**9+210*y**10)+35*r**14*(3946-35514*y
        +142056*y**2-337890*y**3+526890*y**4-555786*y**5+399378*y**6-191772*y**7
        +58248*y**8-9907*y**9+687*y**10)-21*r**12*(3230-29070*y+116280*y**2
        -296750*y**3+499225*y**4-512226*y**5+288009*y**6-45366*y**7-54846*y**8
        +43419*y**9-13570*y**10+1665*y**11)))/(12600*r**2*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-70*MBhat**8*(-1+y)**4*(-60*(-1+y)**6-630*r**10*(-1+y)**6+6*r**4*(-1
        +y)**2*(60-240*y+360*y**2-260*y**3+105*y**4-20*y**5+y**6)+r**2*(
        -1+y)**4*(-130+300*logy*(-1+y)**2+560*y-580*y**2+100*y**3+115*y**4
        -74*y**5+15*y**6)+3*r**6*(-1+y)*(520+180*logy*(-1+y)**5-2780*y
        +6010*y**2-7110*y**3+5095*y**4-2039*y**5+352*y**6)-2*r**8*(-1010+6060*y
        -15150*y**2+21310*y**3-18375*y**4+9390*y**5-2573*y**6+288*y**7))+280*MBhat**7*(
        -1+y)**3*(-120*(-1+y)**7+777*r**12*(-1+y)**7+r**2*(-1+y)**5*(-877
        +840*logy*(-1+y)**2+2594*y-2137*y**2+295*y**3+160*y**4-50*y**5+24*y**6
        +3*y**7)+3*r**4*(-1+y)**3*(395+120*logy*(-1+y)**4-1460*y+1950*y**2
        -1170*y**3+355*y**4-8*y**5-76*y**6+38*y**7)-3*r**6*(-1+y)**2*(-570
        +2850*y-5700*y**2+6050*y**3-3365*y**4+49*y**5+942*y**6-431*y**7+55*y**8)
        +3*r**10*(935-6545*y+19635*y**2-33870*y**3+36690*y**4-24969*y**5+10283*y**6
        -2309*y**7+210*y**8)+r**8*(-3550+24850*y-74550*y**2+128420*y**3-139895*y**4
        +95913*y**5-37741*y**6+6205*y**7+636*y**8-288*y**9))+140*MBhat**6*(
        -1+y)**2*(840*(-1+y)**8+2748*r**14*(-1+y)**8-r**2*(-1+y)**6*(-11428
        +7680*logy*(-1+y)**2+30536*y-22948*y**2+2380*y**3+1309*y**4+178*y**5
        -12*y**6+66*y**7+3*y**8)-r**4*(-1+y)**4*(14623+5100*logy*(-1+y)**4
        -53392*y+69888*y**2-39092*y**3+10343*y**4-3756*y**5+2134*y**6-304*y**7
        +60*y**8)+3*r**6*(-1+y)**3*(-740+2160*logy*(-1+y)**5+1540*y+2320*y**2
        -8560*y**3+10185*y**4-9849*y**5+5899*y**6-2139*y**7+336*y**8)+9*r**12*(
        -1237+9896*y-34636*y**2+70892*y**3-93124*y**4+80108*y**5-44716*y**6
        +15440*y**7-2929*y**8+222*y**9)-r**8*(-1+y)**2*(4780+5040*logy*(-1
        +y)**6-23640*y+43980*y**2-44960*y**3+11505*y**4+54522*y**5-70724*y**6
        +39822*y**7-11037*y**8+1152*y**9)+r**10*(13300-106400*y+372400*y**2
        -770900*y**3+1034800*y**4-897500*y**5+474301*y**6-126848*y**7-1397*y**8
        +10134*y**9-1890*y**10))-10*(-420*(-1+y)**10-189*r**18*(-1+y)**10
        +r**2*(-1+y)**8*(-30293+10500*logy*(-1+y)**2+71086*y-46043*y**2+3500*y**3
        +875*y**4+350*y**5+175*y**6-776*y**7+1732*y**8-576*y**9+12*y**10)
        -21*r**6*(-1+y)**5*(23682+16260*logy*(-1+y)**5-134670*y+309990*y**2
        -364190*y**3+222745*y**4-60809*y**5+2710*y**6+522*y**7-336*y**8+200*y**9
        +12*y**10)+42*r**12*(-1+y)**2*(-4496-1980*logy*(-1+y)**8+33988*y
        -111038*y**2+203596*y**3-226775*y**4+153700*y**5-58370*y**6+7974*y**7
        +3444*y**8-1576*y**9+183*y**10)+7*r**14*(-1+y)*(230+1620*logy*(-1
        +y)**9-3690*y+22050*y**2-70890*y**3+140355*y**4-181179*y**5+154806*y**6
        -89718*y**7+40392*y**8-12068*y**9+1794*y**10-66*y**11)+14*r**4*(-1
        +y)**6*(7840+1140*logy*(-1+y)**4-30220*y+43050*y**2-26420*y**3+5465*y**4
        +228*y**5+38*y**6+34*y**7-144*y**8+236*y**9-95*y**10+6*y**11)-7*r**10*(
        -1+y)**3*(-29890+39900*logy*(-1+y)**7+169330*y-368340*y**2+334600*y**3
        +14525*y**4-288015*y**5+235655*y**6-71249*y**7-4629*y**8+4675*y**9
        -277*y**10+15*y**11)-14*r**8*(-1+y)**4*(12874+38280*logy*(-1+y)**6
        -38964*y-17430*y**2+214640*y**3-352380*y**4+255792*y**5-80912*y**6
        +5832*y**7-1257*y**8+950*y**9-63*y**10+18*y**11)-2*r**16*(-1864+18640*y
        -83880*y**2+223680*y**3-391440*y**4+469728*y**5-391440*y**6+227700*y**7
        -95325*y**8+26650*y**9-4249*y**10+288*y**11))+8*MBhat**5*(-1+y)*(-29400*(
        -1+y)**9+36450*r**16*(-1+y)**9+105*r**6*(-1+y)**4*(-2235-3540*logy*(
        -1+y)**5+14715*y-38280*y**2+49230*y**3-32365*y**4+10049*y**5+953*y**6
        -1026*y**7+135*y**8)+105*r**4*(-1+y)**5*(7367+3180*logy*(-1+y)**4
        -26288*y+33072*y**2-16230*y**3+1870*y**4-197*y**5+624*y**6-56*y**7
        +3*y**8+3*y**9)+r**2*(-1+y)**7*(-620460+336000*logy*(-1+y)**2+1576920*y
        -1124460*y**2+102130*y**3+39865*y**4+29092*y**5-2079*y**6+1107*y**7
        +810*y**8+15*y**9)+35*r**8*(-1+y)**3*(-18530+4680*logy*(-1+y)**6+115860*y
        -303690*y**2+426820*y**3-360135*y**4+209016*y**5-80056*y**6+23769*y**7
        -6099*y**8+845*y**9)-105*r**10*(-1+y)**2*(2200-15400*y+46200*y**2
        -73770*y**3+76460*y**4-68704*y**5+50048*y**6-28201*y**7+10964*y**8
        -2407*y**9+210*y**10)+35*r**14*(4501-40509*y+162036*y**2-384510*y**3
        +596820*y**4-625716*y**5+445998*y**6-211752*y**7+63243*y**8-10462*y**9
        +687*y**10)-21*r**12*(8585-77265*y+309060*y**2-746570*y**3+1186105*y**4
        -1238472*y**5+823563*y**6-312882*y**7+30933*y**8+27588*y**9-12310*y**10
        +1665*y**11))+MBhat**2*(117600*(-1+y)**10+24045*r**18*(-1+y)**10+140*r**8*(
        -1+y)**4*(51947+78540*logy*(-1+y)**6-233142*y+347235*y**2-70280*y**3
        -339990*y**4+375540*y**5-153199*y**6+318*y**7+3039*y**8+1492*y**9-120*y**10)
        +210*r**10*(-1+y)**3*(-46840+15300*logy*(-1+y)**7+312580*y-884190*y**2
        +1366550*y**3-1232675*y**4+640617*y**5-176926*y**6-11518*y**7+16741*y**8
        -2719*y**9+140*y**10)-42*r**4*(-1+y)**6*(230730+108600*logy*(-1+y)**4
        -814320*y+1004280*y**2-452320*y**3+4480*y**4+21480*y**5-679*y**6+8396*y**7
        +196*y**8-764*y**9+201*y**10)-r**2*(-1+y)**8*(-5817955+2251200*logy*(
        -1+y)**2+13887110*y-9194755*y**2+750400*y**3+187600*y**4-31808*y**5
        +161798*y**6+103148*y**7-34765*y**8-66*y**9+1053*y**10)+420*r**6*(-1
        +y)**5*(27686+31680*logy*(-1+y)**5-170110*y+419420*y**2-525020*y**3
        +341710*y**4-98966*y**5+2673*y**6+907*y**7-23*y**8+15*y**9+26*y**10
        +2*y**11)-42*r**12*(-1+y)**2*(45990+27600*logy*(-1+y)**8-340320*y
        +1080720*y**2-1903840*y**3+1993400*y**4-1300096*y**5+596337*y**6-61000*y**7
        -57316*y**8+24068*y**9-4407*y**10+384*y**11)+60*r**16*(-3023+30230*y
        -136035*y**2+362760*y**3-634830*y**4+778680*y**5-696318*y**6+442140*y**7
        -193275*y**8+55160*y**9-9179*y**10+666*y**11)-28*r**14*(-12970+129700*y
        -583650*y**2+1556400*y**3-2723700*y**4+3389400*y**5-3230094*y**6+2294340*y**7
        -1111305*y**8+354280*y**9-68395*y**10+5994*y**11))+4*MBhat*(-8400*(
        -1+y)**10+2220*r**18*(-1+y)**10+105*r**10*(-1+y)**3*(11270-11400*logy*(
        -1+y)**7-67490*y+162570*y**2-191150*y**3+91400*y**4+24960*y**5-50247*y**6
        +25490*y**7-2398*y**8-926*y**9+121*y**10)+r**2*(-1+y)**8*(-507910+184800*logy*(
        -1+y)**2+1200620*y-785110*y**2+61600*y**3+15400*y**4+6160*y**5-9037*y**6
        +21269*y**7-97*y**8-2265*y**9+210*y**10)-210*r**6*(-1+y)**5*(14487
        +12660*logy*(-1+y)**5-85095*y+201840*y**2-244040*y**3+153670*y**4
        -43394*y**5+2316*y**6-289*y**7+212*y**8+35*y**9-9*y**10+3*y**11)-70*r**8*(
        -1+y)**4*(28921+38820*logy*(-1+y)**6-134706*y+220305*y**2-99640*y**3
        -119370*y**4+164208*y**5-65225*y**6+1161*y**7+987*y**8+329*y**9+24*y**10
        +6*y**11)-21*r**12*(-1+y)**2*(30790-246320*y+862120*y**2-1724240*y**3
        +2155300*y**4-1724240*y**5+883473*y**6-312199*y**7+63344*y**8-4548*y**9
        -335*y**10+55*y**11)+3*r**4*(-1+y)**6*(401690+121800*logy*(-1+y)**4
        -1484960*y+1983840*y**2-1078960*y**3+147940*y**4+24360*y**5+4865*y**6
        -7303*y**7+14092*y**8-3638*y**9-363*y**10+117*y**11)+30*r**16*(-859
        +8590*y-38655*y**2+103080*y**3-180390*y**4+216468*y**5-184142*y**6
        +115040*y**7-50175*y**8+14230*y**9-2347*y**10+168*y**11)+14*r**14*(10495
        -104950*y+472275*y**2-1259400*y**3+2203950*y**4-2644740*y**5+2231235*y**6
        -1370634*y**7+620415*y**8-192130*y**9+37330*y**10-3990*y**11+144*y**12))
        +4*MBhat**3*(-58800*(-1+y)**10+12150*r**18*(-1+y)**10+70*r**8*(-1+y)**4*(
        -26920-28800*logy*(-1+y)**6+132720*y-245400*y**2+183200*y**3+4875*y**4
        -88614*y**5+59863*y**6-1812*y**7-4920*y**8+688*y**9)+210*r**6*(-1+y)**5*(
        -9224-11520*logy*(-1+y)**5+57640*y-144080*y**2+182480*y**3-120770*y**4
        +37009*y**5-1839*y**6+517*y**7-164*y**8-60*y**9+3*y**10)+r**2*(-1
        +y)**8*(-2304150+966000*logy*(-1+y)**2+5574300*y-3753150*y**2+322000*y**3
        +52045*y**4+42658*y**5+78099*y**6-4125*y**7-3294*y**8+1407*y**9+90*y**10)
        +35*r**10*(-1+y)**3*(37120+14400*logy*(-1+y)**7-274240*y+873120*y**2
        -1556000*y**3+1671845*y**4-1117659*y**5+532415*y**6-148730*y**7+22371*y**8
        -2797*y**9+355*y**10)+r**4*(-1+y)**6*(3080520+1730400*logy*(-1+y)**4
        -10591680*y+12426720*y**2-4823680*y**3-544955*y**4+318192*y**5+121072*y**6
        +55382*y**7-10863*y**8+4272*y**9+330*y**10-30*y**11)-21*r**12*(-1+y)**2*(
        -25400+203200*y-711200*y**2+1422400*y**3-1731805*y**4+1340108*y**5
        -797494*y**6+345266*y**7-112759*y**8+29686*y**9-5262*y**10+420*y**11)
        +20*r**16*(-3152+31520*y-141840*y**2+378240*y**3-675990*y**4+854490*y**5
        -765114*y**6+482550*y**7-209295*y**8+59105*y**9-9689*y**10+687*y**11)
        -14*r**14*(-3320+33200*y-149400*y**2+398400*y**3-762375*y**4+1111083*y**5
        -1116375*y**6+726945*y**7-303660*y**8+69680*y**9-1523*y**10-3210*y**11
        +555*y**12))+MBhat**4*(294000*(-1+y)**10+80325*r**18*(-1+y)**10+210*r**6*(
        -1+y)**5*(24584+27120*logy*(-1+y)**5-150040*y+367880*y**2-455960*y**3
        +293165*y**4-85393*y**5-3208*y**6+1288*y**7+1129*y**8-165*y**9)-210*r**4*(
        -1+y)**6*(51442+27240*logy*(-1+y)**4-178528*y+213312*y**2-88428*y**3
        -5083*y**4+5804*y**5+1954*y**6+304*y**7+39*y**8+24*y**9)+140*r**8*(
        -1+y)**4*(74995+17100*logy*(-1+y)**6-432870*y+1030875*y**2-1288640*y**3
        +896745*y**4-334590*y**5+3713*y**6+28836*y**7-5676*y**8+212*y**9)-r**2*(
        -1+y)**8*(-8727675+4074000*logy*(-1+y)**2+21529350*y-14838675*y**2
        +1308440*y**3+247100*y**4+418040*y**5+109746*y**6-30612*y**7+11301*y**8
        +2370*y**9+15*y**10)+210*r**10*(-1+y)**3*(20800-9300*logy*(-1+y)**7
        -136300*y+376350*y**2-562130*y**3+521205*y**4-343091*y**5+127394*y**6
        -31890*y**7+9559*y**8-2629*y**9+332*y**10)+60*r**16*(-4931+49310*y
        -221895*y**2+601100*y**3-1084580*y**4+1355004*y**5-1191540*y**6+734340*y**7
        -308790*y**8+83470*y**9-12802*y**10+810*y**11)-42*r**12*(-1+y)**2*(
        -23850+190800*y-667800*y**2+1303780*y**3-1772755*y**4+1953680*y**5
        -1516828*y**6+862184*y**7-362779*y**8+105968*y**9-18860*y**10+1480*y**11)
        -70*r**14*(-376+3760*y-16920*y**2+68760*y**3-161745*y**4+162018*y**5
        -3165*y**6-168288*y**7+214380*y**8-145100*y**9+58862*y**10-13560*y**11
        +1374*y**12)))/(6300*r**2*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr15(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r15 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-240*(-1+y)**6+195*r**10*(-1+y)**6+6*r**6*(-1+y)**2*(-260+90*logy*(
        -1+y)**4+1130*y-1875*y**2+1730*y**3-1455*y**4+472*y**5)-r**2*(-1
        +y)**4*(-245+240*logy*(-1+y)**2+730*y-605*y**2-1480*y**3+2255*y**4
        -766*y**5+15*y**6)+24*r**4*(-1+y)**2*(35-140*y+210*y**2-270*y**3
        +480*y**4-440*y**5+134*y**6)+4*r**8*(130-780*y+1950*y**2-2540*y**3
        +1275*y**4-78*y**5-197*y**6+60*y**7))/(180*r**2*(-1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-480*(-1+y)**7+117*r**12*(-1+y)**7+r**2*(-1+y)**5*(-2837+960*logy*(
        -1+y)**2+6634*y-4277*y**2+320*y**3+2075*y**4-2518*y**5+795*y**6)
        +6*r**6*(-1+y)**2*(-2960+510*logy*(-1+y)**5+14290*y-27305*y**2+25605*y**3
        -12340*y**4+5826*y**5-4179*y**6+1207*y**7)+3*r**4*(-1+y)**3*(-4135
        +4320*logy*(-1+y)**4+20860*y-39930*y**2+35260*y**3-14700*y**4+6052*y**5
        -4782*y**6+1524*y**7-5*y**8)+2*r**8*(-1+y)*(-1520+810*logy*(-1+y)**6
        +9930*y-27255*y**2+40390*y**3-35745*y**4+21726*y**5-10361*y**6+2082*y**7
        +33*y**8)+3*r**10*(-295+2065*y-6195*y**2+10325*y**3-10535*y**4+4881*y**5
        -847*y**6-199*y**7+80*y**8)+2*MBhat**2*(-1+y)*(-240*(-1+y)**6+195*r**10*(
        -1+y)**6+6*r**6*(-1+y)**2*(-260+90*logy*(-1+y)**4+1130*y-1875*y**2
        +1730*y**3-1455*y**4+472*y**5)-r**2*(-1+y)**4*(-245+240*logy*(-1+y)**2
        +730*y-605*y**2-1480*y**3+2255*y**4-766*y**5+15*y**6)+24*r**4*(-1
        +y)**2*(35-140*y+210*y**2-270*y**3+480*y**4-440*y**5+134*y**6)+4*r**8*(130
        -780*y+1950*y**2-2540*y**3+1275*y**4-78*y**5-197*y**6+60*y**7))+6*MBhat*(160*(
        -1+y)**7+28*r**12*(-1+y)**7+r**2*(-1+y)**5*(162-324*y+162*y**2-730*y**3
        +910*y**4-213*y**5-32*y**6+y**7)+4*r**4*(-1+y)**3*(-315-180*logy*(
        -1+y)**4+1080*y-1260*y**2+910*y**3-1355*y**4+1372*y**5-536*y**6
        +68*y**7)-2*r**6*(-1+y)**2*(680-540*logy*(-1+y)**5-2860*y+4370*y**2
        -3460*y**3+4220*y**4-4849*y**5+2489*y**6-529*y**7+11*y**8)+5*r**10*(
        -54+378*y-1134*y**2+1932*y**3-1720*y**4+847*y**5-161*y**6-29*y**7
        +13*y**8)-4*r**8*(-180+1260*y-3780*y**2+6550*y**3-7655*y**4+6679*y**5
        -4105*y**6+1473*y**7-252*y**8+10*y**9)))/(360*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-1200*(-1+y)**8+130*r**14*(-1+y)**8+20*r**8*(-1+y)**3*(-6260+960*logy*(
        -1+y)**5+30340*y-58280*y**2+55080*y**3-25140*y**4+4512*y**5-2781*y**6
        +993*y**7)+r**2*(-1+y)**6*(-17870+6000*logy*(-1+y)**2+41740*y-26870*y**2
        +2000*y**3+500*y**4+3596*y**5-3667*y**6+986*y**7+65*y**8)+40*r**4*(
        -1+y)**4*(-5927+2940*logy*(-1+y)**4+26648*y-45852*y**2+36448*y**3
        -12052*y**4+384*y**5+772*y**6-604*y**7+210*y**8)+10*r**6*(-1+y)**3*(
        -13475+24780*logy*(-1+y)**5+42595*y-23240*y**2-59360*y**3+91630*y**4
        -43886*y**5+9919*y**6-5639*y**7+1888*y**8)+5*r**10*(-1+y)*(1130+1080*logy*(
        -1+y)**7-8990*y+30750*y**2-58810*y**3+68260*y**4-50520*y**5+24239*y**6
        -10025*y**7+1773*y**8+33*y**9)+4*r**12*(430-3440*y+12040*y**2-24080*y**3
        +30100*y**4-25124*y**5+9997*y**6-1712*y**7-191*y**8+90*y**9)+5*MBhat**4*(
        -240*(-1+y)**8+195*r**10*(-1+y)**8+6*r**6*(-1+y)**4*(-260+90*logy*(
        -1+y)**4+1130*y-1875*y**2+1730*y**3-1455*y**4+472*y**5)-r**2*(-1
        +y)**6*(-245+240*logy*(-1+y)**2+730*y-605*y**2-1480*y**3+2255*y**4
        -766*y**5+15*y**6)+24*r**4*(-1+y)**4*(35-140*y+210*y**2-270*y**3
        +480*y**4-440*y**5+134*y**6)+4*r**8*(-1+y)**2*(130-780*y+1950*y**2
        -2540*y**3+1275*y**4-78*y**5-197*y**6+60*y**7))+30*MBhat**3*(-1+y)*(160*(
        -1+y)**7+28*r**12*(-1+y)**7+r**2*(-1+y)**5*(162-324*y+162*y**2-730*y**3
        +910*y**4-213*y**5-32*y**6+y**7)+4*r**4*(-1+y)**3*(-315-180*logy*(
        -1+y)**4+1080*y-1260*y**2+910*y**3-1355*y**4+1372*y**5-536*y**6
        +68*y**7)-2*r**6*(-1+y)**2*(680-540*logy*(-1+y)**5-2860*y+4370*y**2
        -3460*y**3+4220*y**4-4849*y**5+2489*y**6-529*y**7+11*y**8)+5*r**10*(
        -54+378*y-1134*y**2+1932*y**3-1720*y**4+847*y**5-161*y**6-29*y**7
        +13*y**8)-4*r**8*(-180+1260*y-3780*y**2+6550*y**3-7655*y**4+6679*y**5
        -4105*y**6+1473*y**7-252*y**8+10*y**9))+20*MBhat*(240*(-1+y)**8+10*r**14*(
        -1+y)**8-2*r**2*(-1+y)**6*(-1064+360*logy*(-1+y)**2+2488*y-1604*y**2
        +120*y**3+318*y**4-225*y**5-18*y**6+33*y**7)-2*r**4*(-1+y)**4*(-9365
        +6180*logy*(-1+y)**4+43640*y-77820*y**2+64240*y**3-22930*y**4+3342*y**5
        -1472*y**6+428*y**7+45*y**8)+3*r**6*(-1+y)**3*(8230-3240*logy*(-1
        +y)**5-37910*y+67720*y**2-56920*y**3+20800*y**4-3817*y**5+2589*y**6
        -1043*y**7+135*y**8)+r**8*(-1+y)**2*(4360+2160*logy*(-1+y)**6-24000*y
        +53520*y**2-60560*y**3+36120*y**4-18237*y**5+13602*y**6-5424*y**7+990*y**8
        -11*y**9)+3*r**12*(74-592*y+2072*y**2-4144*y**3+5412*y**4-3921*y**5
        +1680*y**6-324*y**7-18*y**8+13*y**9)+r**10*(-1000+8000*y-28000*y**2
        +56000*y**3-71680*y**4+62087*y**5-39868*y**6+19262*y**7-5488*y**8+687*y**9))
        -5*MBhat**2*(1440*(-1+y)**8+168*r**14*(-1+y)**8+6*r**6*(-1+y)**3*(6180
        +90*logy*(-1+y)**5-30990*y+62205*y**2-61585*y**3+26520*y**4-3230*y**5
        +375*y**6+269*y**7-176*y**8)+r**2*(-1+y)**6*(6065-1920*logy*(-1+y)**2
        -14050*y+8945*y**2-3064*y**3-97*y**4+2534*y**5-872*y**6-38*y**7+y**8)
        -r**4*(-1+y)**4*(-23917+30720*logy*(-1+y)**4+126388*y-251022*y**2
        +223028*y**3-74702*y**4+5076*y**5-6454*y**6+2884*y**7+15*y**8)-3*r**12*(735
        -5880*y+20580*y**2-42088*y**3+51944*y**4-40936*y**5+19964*y**6-5416*y**7
        +569*y**8+24*y**9)+2*r**8*(-1+y)**2*(1640-1050*logy*(-1+y)**6-10890*y
        +30375*y**2-42630*y**3+19425*y**4+11034*y**5-12139*y**6+6606*y**7-1701*y**8
        +80*y**9)+r**10*(4415-35320*y+123620*y**2-254080*y**3+338630*y**4
        -319732*y**5+215351*y**6-95296*y**7+25997*y**8-3780*y**9+195*y**10)))/(900*r**2*(
        -1+y)**8)

    if (dQ2 == 0 and dr == 0 and i==3):
        return (2100*logy*(64+1704*r**2+8805*r**4+6305*r**6+225*r**8+45*r**10+108*MBhat**5*r**2*(
        -2+3*r**2)+2*MBhat**6*(-4+9*r**4)+3*MBhat**4*(48+808*r**2-69*r**4
        +43*r**6)+8*MBhat**3*(-52-960*r**2-765*r**4+144*r**6)+6*MBhat**2*(84
        +1776*r**2+3400*r**4+25*r**6+29*r**8)+12*MBhat*(-24-574*r**2-1931*r**4
        -530*r**6+45*r**8))+(16800+408655*r**2+9581355*r**4+12053475*r**6
        -18559625*r**8-3521175*r**10+40845*r**12-19355*r**14-975*r**16-151200*y
        -3812295*r**2*y-89810595*r**4*y-126971775*r**6*y+153796125*r**8*y+31218075*r**10*y
        -462105*r**12*y+174195*r**14*y+8775*r**16*y+604800*y**2+15853980*r**2*y**2
        +375345180*r**4*y**2+591094350*r**6*y**2-555602250*r**8*y**2-122746050*r**10*y**2
        +2273670*r**12*y**2-696780*r**14*y**2-35100*r**16*y**2-1411200*y**3-38605420*r**2*y**3
        -918746220*r**4*y**3-1601106150*r**6*y**3+1137519250*r**8*y**3+280737450*r**10*y**3
        -6439230*r**12*y**3+1625820*r**14*y**3+81900*r**16*y**3+2116800*y**4+60730530*r**2*y**4
        +1453265730*r**4*y**4+2789959725*r**6*y**4-1428228375*r**8*y**4-411183675*r**10*y**4
        +11643345*r**12*y**4-2438730*r**14*y**4-122850*r**16*y**4-2116800*y**5-64117410*r**2*y**5
        -1543441410*r**4*y**5-3255920325*r**6*y**5+1094567775*r**8*y**5+399276675*r**10*y**5
        -14024745*r**12*y**5+2438730*r**14*y**5+122850*r**16*y**5+1411200*y**6+45534517*r**2*y**6
        +1104174855*r**4*y**6+2558947440*r**6*y**6-451741360*r**8*y**6-256405065*r**10*y**6
        +11586393*r**12*y**6-1722840*r**14*y**6-81900*r**16*y**6-604800*y**7-20882397*r**2*y**7
        -516699855*r**4*y**7-1318147320*r**6*y**7+33836040*r**8*y**7+105485625*r**10*y**7
        -6477513*r**12*y**7+595560*r**14*y**7+35100*r**16*y**7+151200*y**8+4990203*r**2*y**8
        +147195615*r**4*y**8+410168745*r**6*y**8+55288485*r**8*y**8-27485850*r**10*y**8
        +2558682*r**12*y**8-94815*r**14*y**8-8775*r**16*y**8-16800*y**9+1007933*r**2*y**9
        -23894535*r**4*y**9-58256205*r**6*y**9-26460665*r**8*y**9+6210750*r**10*y**9
        -811398*r**12*y**9-6265*r**14*y**9+975*r**16*y**9-2248722*r**2*y**10+5634090*r**4*y**10
        -5288430*r**6*y**10+7438410*r**8*y**10-1888425*r**10*y**10+109977*r**12*y**10
        +3360*r**14*y**10+1846194*r**2*y**11-4204410*r**4*y**11+4801230*r**6*y**11
        -2220330*r**8*y**11+301665*r**10*y**11+2079*r**12*y**11-953708*r**2*y**12
        +2135700*r**4*y**12-1589385*r**6*y**12+365995*r**8*y**12+282492*r**2*y**13
        -613620*r**4*y**13+254625*r**6*y**13+525*r**8*y**13-30063*r**2*y**14+76755*r**4*y**14
        -6073*r**2*y**15+1365*r**4*y**15+1584*r**2*y**16+70*MBhat**6*(-1+y)**3*(
        -240*(-1+y)**6+195*r**10*(-1+y)**6+6*r**6*(-1+y)**2*(-260+1130*y
        -1875*y**2+1730*y**3-1455*y**4+472*y**5)-r**2*(-1+y)**4*(-245+730*y
        -605*y**2-1480*y**3+2255*y**4-766*y**5+15*y**6)+24*r**4*(-1+y)**2*(35
        -140*y+210*y**2-270*y**3+480*y**4-440*y**5+134*y**6)+4*r**8*(130
        -780*y+1950*y**2-2540*y**3+1275*y**4-78*y**5-197*y**6+60*y**7))+630*MBhat**5*(
        -1+y)**2*(160*(-1+y)**7+28*r**12*(-1+y)**7+r**2*(-1+y)**5*(162-324*y
        +162*y**2-730*y**3+910*y**4-213*y**5-32*y**6+y**7)+4*r**4*(-1+y)**3*(
        -315+1080*y-1260*y**2+910*y**3-1355*y**4+1372*y**5-536*y**6+68*y**7)
        -2*r**6*(-1+y)**2*(680-2860*y+4370*y**2-3460*y**3+4220*y**4-4849*y**5
        +2489*y**6-529*y**7+11*y**8)+5*r**10*(-54+378*y-1134*y**2+1932*y**3
        -1720*y**4+847*y**5-161*y**6-29*y**7+13*y**8)-4*r**8*(-180+1260*y
        -3780*y**2+6550*y**3-7655*y**4+6679*y**5-4105*y**6+1473*y**7-252*y**8
        +10*y**9))-105*MBhat**4*(-1+y)*(2400*(-1+y)**8+336*r**14*(-1+y)**8
        +r**2*(-1+y)**6*(9293-21466*y+13613*y**2-5808*y**3+1881*y**4+2550*y**5
        -949*y**6-76*y**7+2*y**8)-r**4*(-1+y)**4*(-35429+190196*y-382254*y**2
        +340276*y**3-105304*y**4-8004*y**5+1438*y**6+1196*y**7+45*y**8)-6*r**6*(
        -1+y)**3*(-9400+47690*y-97105*y**2+97565*y**3-40700*y**4+634*y**5
        +3429*y**6-1745*y**7+352*y**8)-3*r**12*(1431-11448*y+40068*y**2-81992*y**3
        +101158*y**4-79688*y**5+38836*y**6-10520*y**7+1099*y**8+48*y**9)+2*r**8*(
        -1+y)**2*(1760-11850*y+33495*y**2-44870*y**3+3105*y**4+43794*y**5
        -34639*y**6+15294*y**7-3369*y**8+160*y**9)+r**10*(9715-77720*y+272020*y**2
        -557720*y**3+739840*y**4-685712*y**5+447886*y**6-192536*y**7+51157*y**8
        -7320*y**9+390*y**10))+6*MBhat*(16800*(-1+y)**9+260*r**16*(-1+y)**9
        -140*r**4*(-1+y)**5*(-40919+180896*y-305784*y**2+238296*y**3-76794*y**4
        +3102*y**5+1586*y**6-458*y**7+69*y**8+114*y**9)+140*r**8*(-1+y)**3*(
        -50875+289350*y-675675*y**2+821400*y**3-536550*y**4+166446*y**5-16838*y**6
        +3702*y**7-1512*y**8+192*y**9)-r**2*(-1+y)**7*(-300810+702420*y-452010*y**2
        +33600*y**3+8400*y**4+27531*y**5-8008*y**6-8422*y**7+3824*y**8+195*y**9)
        -35*r**6*(-1+y)**4*(18014-321790*y+1222880*y**2-1995280*y**3+1576940*y**4
        -546217*y**5+48281*y**6-3796*y**7+2870*y**8+255*y**9+3*y**10)-35*r**10*(
        -1+y)**2*(14370-95190*y+266670*y**2-406650*y**3+359400*y**4-180819*y**5
        +61319*y**6-31562*y**7+9654*y**8-1523*y**9+11*y**10)+35*r**14*(-274
        +2466*y-9864*y**2+23016*y**3-34524*y**4+36603*y**5-22239*y**6+8316*y**7
        -1494*y**8-29*y**9+39*y**10)+28*r**12*(2395-21555*y+86220*y**2-201180*y**3
        +301770*y**4-308502*y**5+216750*y**6-110790*y**7+44490*y**8-10720*y**9
        +1107*y**10+15*y**11))-10*MBhat**3*(-33600*(-1+y)**9+4176*r**16*(-1
        +y)**9-42*r**6*(-1+y)**4*(73822-338510*y+600520*y**2-499795*y**3+175065*y**4
        -22002*y**5+12572*y**6-4289*y**7+457*y**8)-r**2*(-1+y)**7*(258024-603408*y
        +389064*y**2-42854*y**3-57554*y**4+44296*y**5+5040*y**6-5919*y**7-132*y**8
        +3*y**9)+42*r**4*(-1+y)**5*(-58424+272096*y-484944*y**2+399686*y**3
        -140204*y**4+15719*y**5-4804*y**6+1424*y**7+168*y**8+3*y**9)-56*r**8*(
        -1+y)**3*(6430-34260*y+72690*y**2-76715*y**3+35175*y**4-13845*y**5
        +16502*y**6-6039*y**7+693*y**8+89*y**9)+14*r**14*(2498-22482*y+89928*y**2
        -213297*y**3+321573*y**4-324198*y**5+218778*y**6-95652*y**7+24858*y**8
        -3077*y**9+63*y**10)+21*r**10*(-1+y)**2*(2120-14840*y+44520*y**2-76730*y**3
        +85520*y**4-64658*y**5+13460*y**6-591*y**7+1459*y**8-765*y**9+65*y**10)
        +42*r**12*(-1912+17208*y-68832*y**2+163530*y**3-250754*y**4+265851*y**5
        -197919*y**6+99696*y**7-32634*y**8+6361*y**9-613*y**10+18*y**11))-21*MBhat**2*(12000*(
        -1+y)**9+330*r**16*(-1+y)**9-r**2*(-1+y)**7*(-148710+347820*y-224310*y**2
        +16800*y**3+15285*y**4+7998*y**5-13106*y**6+2278*y**7+745*y**8)-20*r**6*(
        -1+y)**4*(-76019+278095*y-301190*y**2-38810*y**3+275440*y**4-152012*y**5
        +18073*y**6-2513*y**7+1066*y**8+30*y**9)-10*r**8*(-1+y)**3*(82525-493650*y
        +1229625*y**2-1632000*y**3+1220265*y**4-459474*y**5+60083*y**6-6468*y**7
        -762*y**8+576*y**9)-5*r**4*(-1+y)**5*(-427100+1921520*y-3308520*y**2
        +2631920*y**3-875135*y**4+49980*y**5+13423*y**6-7864*y**7+3795*y**8
        +140*y**9+y**10)-10*r**14*(-710+6390*y-25560*y**2+59640*y**3-92925*y**4
        +91413*y**5-59892*y**6+24948*y**7-5877*y**8+533*y**9+24*y**10)+5*r**10*(
        -1+y)**2*(-7210+53950*y-174030*y**2+314410*y**3-335515*y**4+170637*y**5
        +15476*y**6-29108*y**7+13575*y**8-2825*y**9+80*y**10)+r**12*(-20140
        +181260*y-725040*y**2+1691760*y**3-2626875*y**4+2788707*y**5-2182053*y**6
        +1290477*y**7-502713*y**8+117457*y**9-13035*y**10+195*y**11)))/(r**2*(
        -1+y)**9))/12600
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-16800*(-1+y)**10+585*r**18*(-1+y)**10+210*r**10*(-1+y)**4*(352080
        +174900*logy*(-1+y)**6-1937580*y+4319250*y**2-4884500*y**3+2788875*y**4
        -590850*y**5-76425*y**6+23300*y**7-5077*y**8+2242*y**9+5*y**10)+336*r**12*(
        -1+y)**2*(17755+2100*logy*(-1+y)**8-139940*y+481390*y**2-943180*y**3
        +1149575*y**4-890260*y**5+425530*y**6-114132*y**7+15376*y**8-4656*y**9
        +967*y**10)+210*r**6*(-1+y)**5*(381998+276540*logy*(-1+y)**5-2186530*y
        +5064410*y**2-5986210*y**3+3684455*y**4-1013431*y**5+46090*y**6+7710*y**7
        +2299*y**8-1335*y**9+1120*y**10)+280*r**8*(-1+y)**4*(71918+373560*logy*(
        -1+y)**6-57948*y-975810*y**2+3168880*y**3-4244460*y**4+2818464*y**5
        -843304*y**6+55536*y**7+8763*y**8-2654*y**9+1515*y**10)+r**2*(-1+y)**8*(
        -563305+184800*logy*(-1+y)**2+1311410*y-840505*y**2+61600*y**3+15400*y**4
        +6160*y**5+3080*y**6+23792*y**7-11899*y**8-1098*y**9+2085*y**10)+120*r**16*(134
        -1340*y+6030*y**2-16080*y**3+28140*y**4-33768*y**5+28140*y**6-17240*y**7
        +5255*y**8-770*y**9-33*y**10+20*y**11)+48*r**4*(-1+y)**6*(-401285+126000*logy*(
        -1+y)**4+1731140*y-2848710*y**2+2151140*y**3-663785*y**4+25200*y**5
        +4200*y**6+696*y**7+2641*y**8-2124*y**9+1136*y**10+66*y**11)+14*r**14*(
        -1+y)*(-370+8100*logy*(-1+y)**9-4770*y+55530*y**2-226770*y**3+510255*y**4
        -714375*y**5+646350*y**6-405918*y**7+142587*y**8-50423*y**9+7452*y**10
        +132*y**11)+70*MBhat**8*(-1+y)**4*(-240*(-1+y)**6+195*r**10*(-1+y)**6
        +6*r**6*(-1+y)**2*(-260+90*logy*(-1+y)**4+1130*y-1875*y**2+1730*y**3
        -1455*y**4+472*y**5)-r**2*(-1+y)**4*(-245+240*logy*(-1+y)**2+730*y
        -605*y**2-1480*y**3+2255*y**4-766*y**5+15*y**6)+24*r**4*(-1+y)**2*(35
        -140*y+210*y**2-270*y**3+480*y**4-440*y**5+134*y**6)+4*r**8*(130
        -780*y+1950*y**2-2540*y**3+1275*y**4-78*y**5-197*y**6+60*y**7))+840*MBhat**7*(
        -1+y)**3*(160*(-1+y)**7+28*r**12*(-1+y)**7+r**2*(-1+y)**5*(162-324*y
        +162*y**2-730*y**3+910*y**4-213*y**5-32*y**6+y**7)+4*r**4*(-1+y)**3*(
        -315-180*logy*(-1+y)**4+1080*y-1260*y**2+910*y**3-1355*y**4+1372*y**5
        -536*y**6+68*y**7)-2*r**6*(-1+y)**2*(680-540*logy*(-1+y)**5-2860*y
        +4370*y**2-3460*y**3+4220*y**4-4849*y**5+2489*y**6-529*y**7+11*y**8)
        +5*r**10*(-54+378*y-1134*y**2+1932*y**3-1720*y**4+847*y**5-161*y**6
        -29*y**7+13*y**8)-4*r**8*(-180+1260*y-3780*y**2+6550*y**3-7655*y**4
        +6679*y**5-4105*y**6+1473*y**7-252*y**8+10*y**9))-140*MBhat**6*(-1
        +y)**2*(3360*(-1+y)**8+504*r**14*(-1+y)**8+6*r**6*(-1+y)**3*(12620
        +1290*logy*(-1+y)**5-64390*y+132005*y**2-133545*y**3+54880*y**4+1962*y**5
        -7233*y**6+3221*y**7-528*y**8)+r**2*(-1+y)**6*(12521-3840*logy*(-1
        +y)**2-28882*y+18281*y**2-8552*y**3+3859*y**4+2566*y**5-1026*y**6
        -114*y**7+3*y**8)-3*r**4*(-1+y)**4*(-15647+22080*logy*(-1+y)**4+84668*y
        -171162*y**2+152508*y**3-45302*y**4-7028*y**5+3110*y**6-164*y**7+25*y**8)
        -9*r**12*(709-5672*y+19852*y**2-40632*y**3+50124*y**4-39480*y**5+19236*y**6
        -5208*y**7+543*y**8+24*y**9)+2*r**8*(-1+y)**2*(1880-1530*logy*(-1
        +y)**6-12810*y+36615*y**2-47110*y**3-13215*y**4+76554*y**5-57139*y**6
        +23982*y**7-5037*y**8+240*y**9)+3*r**10*(5005-40040*y+140140*y**2
        -287120*y**3+380350*y**4-350564*y**5+226807*y**6-96592*y**7+25439*y**8
        -3620*y**9+195*y**10))-40*MBhat**5*(-1+y)*(-23520*(-1+y)**9+4176*r**16*(
        -1+y)**9-42*r**6*(-1+y)**4*(49132-20880*logy*(-1+y)**5-224780*y
        +397360*y**2-329035*y**3+112665*y**4-10551*y**5+4805*y**6-1160*y**7
        +52*y**8)+r**2*(-1+y)**7*(-168648+57120*logy*(-1+y)**2+394416*y-254328*y**2
        +32774*y**3+30842*y**4-25396*y**5-3528*y**6+3147*y**7+132*y**8-3*y**9)
        +42*r**4*(-1+y)**5*(-39694+26040*logy*(-1+y)**4+184816*y-329304*y**2
        +271206*y**3-94344*y**4+9035*y**5-1860*y**6+568*y**7+78*y**8+3*y**9)
        -14*r**8*(-1+y)**3*(12640+10800*logy*(-1+y)**6-65040*y+130200*y**2
        -125180*y**3+32340*y**4-669*y**5+25202*y**6-7884*y**7-198*y**8+389*y**9)
        +14*r**14*(2468-22212*y+88848*y**2-210777*y**3+317793*y**4-320418*y**5
        +216258*y**6-94572*y**7+24588*y**8-3047*y**9+63*y**10)+21*r**10*(-1
        +y)**2*(4120-28840*y+86520*y**2-146730*y**3+158880*y**4-115472*y**5
        +42382*y**6-10193*y**7+2833*y**8-765*y**9+65*y**10)+84*r**12*(-1067
        +9603*y-38412*y**2+91089*y**3-139711*y**4+146925*y**5-107361*y**6
        +52854*y**7-16776*y**8+3134*y**9-287*y**10+9*y**11))-2*MBhat**2*(235200*(
        -1+y)**10+2205*r**18*(-1+y)**10-70*r**8*(-1+y)**4*(3461569+2200980*logy*(
        -1+y)**6-18568434*y+39818145*y**2-42085960*y**3+20559570*y**4-1560552*y**5
        -1926518*y**6+337248*y**7-18171*y**8+5138*y**9+285*y**10)-210*r**10*(
        -1+y)**3*(-154295+6000*logy*(-1+y)**7+1074065*y-3201195*y**2+5293325*y**3
        -5240825*y**4+3114603*y**5-994620*y**6+111780*y**7-10937*y**8-221*y**9
        +400*y**10)-r**2*(-1+y)**8*(-4992305+1680000*logy*(-1+y)**2+11664610*y
        -7512305*y**2+560000*y**3+140000*y**4+177464*y**5+195622*y**6-152852*y**7
        -3713*y**8+16974*y**9+585*y**10)-42*r**4*(-1+y)**6*(-3018005+1118400*logy*(
        -1+y)**4+13190420*y-22022430*y**2+16918420*y**3-5348005*y**4+222624*y**5
        +35979*y**6+28884*y**7-9886*y**8+6084*y**9+955*y**10)-210*r**6*(-1
        +y)**5*(675695+1131300*logy*(-1+y)**5-4509775*y+11847800*y**2-15618800*y**3
        +10637650*y**4-3250394*y**5+186654*y**6+34534*y**7-41*y**8+2625*y**9
        +98*y**10+2*y**11)-30*r**16*(2315-23150*y+104175*y**2-277800*y**3
        +486150*y**4-612612*y**5+501270*y**6-281400*y**7+102555*y**8-21470*y**9
        +1751*y**10+72*y**11)+42*r**12*(-1+y)**2*(14835-8700*logy*(-1+y)**8
        -127380*y+480630*y**2-1042460*y**3+1424875*y**4-1216868*y**5+575379*y**6
        +39148*y**7-62258*y**8+24096*y**9-4237*y**10+80*y**11)-14*r**14*(-19735
        +197350*y-888075*y**2+2368200*y**3-4144350*y**4+5171328*y**5-4556100*y**6
        +2986560*y**7-1550535*y**8+535630*y**9-110653*y**10+10380*y**11))+24*MBhat*(5600*(
        -1+y)**10+40*r**18*(-1+y)**10-r**2*(-1+y)**8*(-151560+50400*logy*(
        -1+y)**2+353520*y-227160*y**2+16800*y**3+4200*y**4+1680*y**5+5740*y**6
        +1115*y**7-3078*y**8+851*y**9+132*y**10)-70*r**6*(-1+y)**5*(161112
        +156960*logy*(-1+y)**5-962520*y+2317440*y**2-2840640*y**3+1812720*y**4
        -519504*y**5+27004*y**6+3937*y**7+469*y**8+345*y**9+141*y**10)+35*r**10*(
        -1+y)**3*(186960-50400*logy*(-1+y)**7-1258320*y+3598560*y**2-5644800*y**3
        +5203800*y**4-2769480*y**5+744276*y**6-63407*y**7+4805*y**8-1913*y**9
        +239*y**10)+7*r**12*(-1+y)**2*(34920+10800*logy*(-1+y)**8-268560*y
        +896760*y**2-1692720*y**3+1964700*y**4-1420560*y**5+619880*y**6-165835*y**7
        +65036*y**8-15578*y**9+2128*y**10-11*y**11)-70*r**8*(-1+y)**4*(138072
        +181440*logy*(-1+y)**6-646992*y+1073160*y**2-523680*y**3-514440*y**4
        +750096*y**5-304352*y**6+27155*y**7+1170*y**8+596*y**9+54*y**10+y**11)
        +20*r**16*(108-1080*y+4860*y**2-12960*y**3+22680*y**4-27216*y**5+24304*y**6
        -12700*y**7+4200*y**8-695*y**9-2*y**10+13*y**11)-r**4*(-1+y)**6*(-4548840
        +1537200*logy*(-1+y)**4+19732560*y-32673240*y**2+24856560*y**3-7751340*y**4
        +307440*y**5+47096*y**6+28961*y**7+1616*y**8-1794*y**9+5956*y**10+65*y**11)
        +28*r**14*(-780+7800*y-35100*y**2+93600*y**3-163800*y**4+196560*y**5
        -167488*y**6+99865*y**7-42485*y**8+14610*y**9-3060*y**10+273*y**11
        +5*y**12))-8*MBhat**3*(-117600*(-1+y)**10+3480*r**18*(-1+y)**10+140*r**8*(
        -1+y)**4*(321343+102060*logy*(-1+y)**6-1825998*y+4258815*y**2-5168120*y**3
        +3375150*y**4-1046511*y**5+95282*y**6-9462*y**7+2652*y**8-271*y**9)
        +r**2*(-1+y)**8*(-1879530+638400*logy*(-1+y)**2+4397460*y-2837130*y**2
        +212800*y**3+80920*y**4+141715*y**5-44436*y**6-39663*y**7+13134*y**8
        +1770*y**9)+105*r**6*(-1+y)**5*(23050+507000*logy*(-1+y)**5-622250*y
        +2512000*y**2-4202000*y**3+3374120*y**4-1185641*y**5+100501*y**6+2820*y**7
        +2230*y**8+195*y**9+15*y**10)-105*r**10*(-1+y)**3*(-22030+9000*logy*(
        -1+y)**7+145210*y-404130*y**2+610550*y**3-539560*y**4+258909*y**5
        -68181*y**6+35691*y**7-8337*y**8+800*y**9+78*y**10)+15*r**4*(-1+y)**6*(
        -2558732+1070160*logy*(-1+y)**4+11305088*y-19097952*y**2+14872288*y**3
        -4786552*y**4+197498*y**5+76468*y**6-7307*y**7+2768*y**8+3408*y**9
        +80*y**10+y**11)+60*r**16*(-675+6750*y-30375*y**2+81000*y**3-145810*y**4
        +176050*y**5-148302*y**6+86130*y**7-33165*y**8+7755*y**9-891*y**10
        +21*y**11)+21*r**12*(-1+y)**2*(2180-17440*y+61040*y**2-122080*y**3
        +129880*y**4-72130*y**5-69724*y**6+18617*y**7-6952*y**8+4784*y**9-1320*y**10
        +65*y**11)+35*r**14*(3278-32780*y+147510*y**2-393360*y**3+710160*y**4
        -881379*y**5+794166*y**6-531375*y**7+247230*y**8-76445*y**9+14228*y**10
        -1245*y**11+12*y**12))-3*MBhat**4*(392000*(-1+y)**10+28875*r**18*(-1
        +y)**10-70*r**6*(-1+y)**5*(-690112+912240*logy*(-1+y)**5+2538320*y
        -2796040*y**2-227920*y**3+2371295*y**4-1299043*y**5+116170*y**6+6670*y**7
        +535*y**8+285*y**9)-280*r**8*(-1+y)**4*(94725+300*logy*(-1+y)**6-568050*y
        +1419225*y**2-1882600*y**3+1400945*y**4-511842*y**5+44649*y**6+7456*y**7
        -3876*y**8+568*y**9)-r**2*(-1+y)**8*(-4423925+1512000*logy*(-1+y)**2
        +10359850*y-6691925*y**2+516880*y**3+536690*y**4+96684*y**5-288918*y**6
        +31264*y**7+19955*y**8+250*y**9-5*y**10)-140*r**4*(-1+y)**6*(-482308
        +239040*logy*(-1+y)**4+2168272*y-3730488*y**2+2966384*y**3-988911*y**4
        +62236*y**5+7809*y**6-2656*y**7+2047*y**8+132*y**9+3*y**10)+140*r**10*(
        -1+y)**3*(-20600-2550*logy*(-1+y)**7+146750*y-449175*y**2+755615*y**3
        -753205*y**4+382781*y**5-4736*y**6-57470*y**7+25864*y**8-4324*y**9
        +100*y**10)+28*r**12*(-1+y)**2*(-27100+216800*y-758800*y**2+1533760*y**3
        -1857305*y**4+1457848*y**5-484829*y**6-26336*y**7+79297*y**8-22840*y**9
        +1565*y**10+80*y**11)+40*r**16*(-5529+55290*y-248805*y**2+671600*y**3
        -1184540*y**4+1436736*y**5-1215144*y**6+709680*y**7-275700*y**8+65520*y**9
        -7828*y**10+232*y**11)-70*r**14*(-7984+79840*y-359280*y**2+972136*y**3
        -1716925*y**4+2110346*y**5-1838921*y**6+1114872*y**7-455235*y**8+115830*y**9
        -15059*y**10+296*y**11+84*y**12)))/(12600*r**2*(-1+y)**10)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr16(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r16 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (-240*(-1+y)**5+1275*r**10*(-1+y)**5-r**2*(-1+y)**3*(-845+240*logy*(
        -1+y)**2+1930*y-1205*y**2-3520*y**3+5375*y**4-1774*y**5+15*y**6)
        +20*r**8*(22-110*y+220*y**2-484*y**3+485*y**4-241*y**5+48*y**6)+6*r**6*(380
        +90*logy*(-1+y)**5-1990*y+4205*y**2-5665*y**3+6165*y**4-4071*y**5
        +1024*y**6)+24*r**4*(-35+175*y-350*y**2+770*y**3-1770*y**4+2262*y**5
        -1355*y**6+303*y**7))/(360*r**2*(-1+y)**5)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-480*(-1+y)**6+765*r**12*(-1+y)**6+r**2*(-1+y)**4*(-3325+1920*logy*(
        -1+y)**2+8570*y-6205*y**2+640*y**3+4315*y**4-5462*y**5+1515*y**6)
        +5*r**10*(-23+138*y-345*y**2+460*y**3-2235*y**4+2262*y**5-1049*y**6
        +192*y**7)+6*r**6*(-1+y)*(-4040+510*logy*(-1+y)**5+19690*y-38105*y**2
        +36405*y**3-20360*y**4+13366*y**5-9251*y**6+2487*y**7)+r**4*(-1+y)**2*(
        -15725+13920*logy*(-1+y)**4+76820*y-143070*y**2+123220*y**3-58300*y**4
        +40092*y**5-31946*y**6+9212*y**7-15*y**8)+2*r**8*(-2680+810*logy*(
        -1+y)**6+16890*y-44655*y**2+63590*y**3-53805*y**4+35166*y**5-17677*y**6
        +3546*y**7+105*y**8)+2*MBhat**2*(-1+y)*(-240*(-1+y)**5+1275*r**10*(
        -1+y)**5-r**2*(-1+y)**3*(-845+240*logy*(-1+y)**2+1930*y-1205*y**2
        -3520*y**3+5375*y**4-1774*y**5+15*y**6)+20*r**8*(22-110*y+220*y**2
        -484*y**3+485*y**4-241*y**5+48*y**6)+6*r**6*(380+90*logy*(-1+y)**5
        -1990*y+4205*y**2-5665*y**3+6165*y**4-4071*y**5+1024*y**6)+24*r**4*(
        -35+175*y-350*y**2+770*y**3-1770*y**4+2262*y**5-1355*y**6+303*y**7))
        +2*MBhat*(480*(-1+y)**6+1848*r**12*(-1+y)**6+r**2*(-1+y)**4*(1532
        -960*logy*(-1+y)**2-4024*y+2972*y**2-5150*y**3+6280*y**4-1475*y**5
        -186*y**6+3*y**7)+8*r**4*(-1+y)**2*(-395-660*logy*(-1+y)**4+920*y
        -60*y**2+625*y**3-4435*y**4+5046*y**5-1982*y**6+245*y**7)+5*r**10*(
        -556+3336*y-8340*y**2+13010*y**3-12000*y**4+6585*y**5-1990*y**6+255*y**7)
        -6*r**6*(-1+y)*(80-1080*logy*(-1+y)**5+680*y-4060*y**2+3930*y**3
        +5950*y**4-11579*y**5+6495*y**6-1387*y**7+35*y**8)-8*r**8*(-200+1200*y
        -3000*y**2+3325*y**3+1035*y**4-4848*y**5+3385*y**6-897*y**7+60*y**8)))/(720*r**2*(
        -1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (-240*(-1+y)**7+170*r**14*(-1+y)**7+r**2*(-1+y)**5*(-3374+1680*logy*(
        -1+y)**2+8428*y-5894*y**2+560*y**3+140*y**4+1424*y**5-1607*y**6
        +334*y**7+13*y**8)+4*r**12*(14-98*y+294*y**2-490*y**3+490*y**4-1446*y**5
        +1169*y**6-455*y**7+72*y**8)+24*r**4*(-1+y)**3*(-1775+780*logy*(-1
        +y)**4+7880*y-13380*y**2+10480*y**3-3400*y**4-24*y**5+531*y**6-436*y**7
        +130*y**8)+4*r**8*(-1+y)*(8100+960*logy*(-1+y)**6-47640*y+116220*y**2
        -150160*y**3+107820*y**4-42288*y**5+14047*y**6-7650*y**7+1911*y**8)
        +2*r**6*(-1+y)**2*(-7915+26700*logy*(-1+y)**5+12875*y+41000*y**2
        -130000*y**3+131750*y**4-57442*y**5+18851*y**6-12391*y**7+3704*y**8)
        +3*r**10*(710+360*logy*(-1+y)**7-5330*y+17250*y**2-31270*y**3+34420*y**4
        -22412*y**5+10673*y**6-4655*y**7+819*y**8+35*y**9)+MBhat**4*(-1+y)**2*(
        -240*(-1+y)**5+1275*r**10*(-1+y)**5-r**2*(-1+y)**3*(-845+240*logy*(
        -1+y)**2+1930*y-1205*y**2-3520*y**3+5375*y**4-1774*y**5+15*y**6)
        +20*r**8*(22-110*y+220*y**2-484*y**3+485*y**4-241*y**5+48*y**6)+6*r**6*(380
        +90*logy*(-1+y)**5-1990*y+4205*y**2-5665*y**3+6165*y**4-4071*y**5
        +1024*y**6)+24*r**4*(-35+175*y-350*y**2+770*y**3-1770*y**4+2262*y**5
        -1355*y**6+303*y**7))+2*MBhat**3*(-1+y)*(480*(-1+y)**6+1848*r**12*(
        -1+y)**6+r**2*(-1+y)**4*(1532-960*logy*(-1+y)**2-4024*y+2972*y**2
        -5150*y**3+6280*y**4-1475*y**5-186*y**6+3*y**7)+8*r**4*(-1+y)**2*(
        -395-660*logy*(-1+y)**4+920*y-60*y**2+625*y**3-4435*y**4+5046*y**5
        -1982*y**6+245*y**7)+5*r**10*(-556+3336*y-8340*y**2+13010*y**3-12000*y**4
        +6585*y**5-1990*y**6+255*y**7)-6*r**6*(-1+y)*(80-1080*logy*(-1+y)**5
        +680*y-4060*y**2+3930*y**3+5950*y**4-11579*y**5+6495*y**6-1387*y**7
        +35*y**8)-8*r**8*(-200+1200*y-3000*y**2+3325*y**3+1035*y**4-4848*y**5
        +3385*y**6-897*y**7+60*y**8))+4*MBhat*(240*(-1+y)**7+220*r**14*(-1
        +y)**7-2*r**2*(-1+y)**5*(-1544+720*logy*(-1+y)**2+3808*y-2624*y**2
        +240*y**3+660*y**4-531*y**5-54*y**6+57*y**7)-4*r**4*(-1+y)**3*(-5845
        +3540*logy*(-1+y)**4+26920*y-47460*y**2+38720*y**3-14180*y**4+3198*y**5
        -1804*y**6+442*y**7+45*y**8)+r**12*(388-2716*y+8148*y**2-13580*y**3
        +17420*y**4-14049*y**5+7063*y**6-2029*y**7+255*y**8)+r**6*(-1+y)**2*(31820
        -14160*logy*(-1+y)**5-144940*y+254480*y**2-207280*y**3+75920*y**4
        -23815*y**5+19835*y**6-7693*y**7+953*y**8)+r**10*(160-1120*y+3360*y**2
        -5600*y**3+3920*y**4-8079*y**5+11857*y**6-5917*y**7+1059*y**8)+r**8*(
        -1+y)*(5440+4320*logy*(-1+y)**6-28320*y+57840*y**2-55520*y**3+27240*y**4
        -25233*y**5+28038*y**6-12804*y**7+2394*y**8-35*y**9))+MBhat**2*(-1440*(
        -1+y)**7+2520*r**14*(-1+y)**7-r**2*(-1+y)**5*(12473-6240*logy*(-1
        +y)**2-31186*y+21833*y**2-7600*y**3+935*y**4+5114*y**5-1640*y**6
        -74*y**7+y**8)+r**4*(-1+y)**3*(-38725+45120*logy*(-1+y)**4+200020*y
        -390270*y**2+332180*y**3-95030*y**4-3804*y**5-8518*y**6+4996*y**7+15*y**8)
        +6*r**6*(-1+y)**2*(-12460-810*logy*(-1+y)**5+63110*y-128245*y**2
        +125105*y**3-50920*y**4+1262*y**5+4329*y**6-2085*y**7+480*y**8)+3*r**12*(1769
        -12383*y+37149*y**2-67035*y**3+74445*y**4-52059*y**5+22281*y**6-5295*y**7
        +528*y**8)+r**10*(-5785+40495*y-121485*y**2+206555*y**3-166655*y**4
        +25953*y**5+51464*y**6-40712*y**7+12165*y**8-1275*y**9)-2*r**8*(-1
        +y)*(12560-1050*logy*(-1+y)**6-76410*y+194175*y**2-247350*y**3+129405*y**4
        +14034*y**5-49735*y**6+27582*y**7-6261*y**8+320*y**9)))/(360*r**2*(-1
        +y)**7)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-16800*(-1+y)**8+6375*r**16*(-1+y)**8+35*r**14*(-143+1144*y-4004*y**2
        +8008*y**3-10010*y**4+8008*y**5-13328*y**6+8536*y**7-2795*y**8+384*y**9)
        +r**2*(-1+y)**6*(-208255+134400*logy*(-1+y)**2+550910*y-409855*y**2
        +44800*y**3+11200*y**4+4480*y**5+70007*y**6-71374*y**7+8327*y**8+1440*y**9)
        +105*r**6*(-1+y)**3*(154275+140100*logy*(-1+y)**5-911475*y+2173200*y**2
        -2640200*y**3+1670350*y**4-474170*y**5+20708*y**6+16116*y**7-12621*y**8
        +4585*y**9)+105*r**10*(-1+y)*(-41935+4500*logy*(-1+y)**7+289045*y
        -851385*y**2+1387475*y**3-1348100*y**4+777360*y**5-245101*y**6+51879*y**7
        -22535*y**8+5217*y**9)+105*r**4*(-1+y)**4*(-48819+13920*logy*(-1+y)**4
        +209196*y-341634*y**2+255596*y**3-77819*y**4+2784*y**5-1075*y**6+4908*y**7
        -4442*y**8+1388*y**9+13*y**10)+35*r**8*(-1+y)**2*(492475+414300*logy*(
        -1+y)**6-2540550*y+5108475*y**2-4739800*y**3+1483350*y**4+649560*y**5
        -537314*y**6+137616*y**7-68457*y**8+20390*y**9+15*y**10)+21*r**12*(
        -4345+4500*logy*(-1+y)**8+39260*y-155410*y**2+352820*y**3-504025*y**4
        +466220*y**5-252377*y**6+92796*y**7-38046*y**8+6152*y**9+315*y**10)
        +70*MBhat**6*(-1+y)**3*(-240*(-1+y)**5+1275*r**10*(-1+y)**5-r**2*(-1
        +y)**3*(-845+240*logy*(-1+y)**2+1930*y-1205*y**2-3520*y**3+5375*y**4
        -1774*y**5+15*y**6)+20*r**8*(22-110*y+220*y**2-484*y**3+485*y**4
        -241*y**5+48*y**6)+6*r**6*(380+90*logy*(-1+y)**5-1990*y+4205*y**2
        -5665*y**3+6165*y**4-4071*y**5+1024*y**6)+24*r**4*(-35+175*y-350*y**2
        +770*y**3-1770*y**4+2262*y**5-1355*y**6+303*y**7))+210*MBhat**5*(-1
        +y)**2*(480*(-1+y)**6+1848*r**12*(-1+y)**6+r**2*(-1+y)**4*(1532-960*logy*(
        -1+y)**2-4024*y+2972*y**2-5150*y**3+6280*y**4-1475*y**5-186*y**6
        +3*y**7)+8*r**4*(-1+y)**2*(-395-660*logy*(-1+y)**4+920*y-60*y**2
        +625*y**3-4435*y**4+5046*y**5-1982*y**6+245*y**7)+5*r**10*(-556+3336*y
        -8340*y**2+13010*y**3-12000*y**4+6585*y**5-1990*y**6+255*y**7)-6*r**6*(
        -1+y)*(80-1080*logy*(-1+y)**5+680*y-4060*y**2+3930*y**3+5950*y**4
        -11579*y**5+6495*y**6-1387*y**7+35*y**8)-8*r**8*(-200+1200*y-3000*y**2
        +3325*y**3+1035*y**4-4848*y**5+3385*y**6-897*y**7+60*y**8))+105*MBhat**4*(
        -1+y)*(-2400*(-1+y)**7+5040*r**14*(-1+y)**7+r**2*(-1+y)**5*(-21621
        +10560*logy*(-1+y)**2+53802*y-37461*y**2+14560*y**3-6185*y**4-4766*y**5
        +1765*y**6+148*y**7-2*y**8)+15*r**4*(-1+y)**3*(-4115+5088*logy*(-1
        +y)**4+21548*y-42498*y**2+36076*y**3-8784*y**4-3180*y**5+994*y**6
        +52*y**7+3*y**8)+6*r**6*(-1+y)**2*(-20880-2130*logy*(-1+y)**5+106530*y
        -218385*y**2+213805*y**3-81480*y**4-10842*y**5+17909*y**6-6657*y**7
        +960*y**8)+3*r**12*(3793-26551*y+79653*y**2-142995*y**3+157815*y**4
        -109473*y**5+46347*y**6-10845*y**7+1056*y**8)-2*r**8*(-1+y)*(22440
        -1290*logy*(-1+y)**6-135930*y+343695*y**2-431110*y**3+205005*y**4
        +63234*y**5-117147*y**6+58710*y**7-12417*y**8+640*y**9)-3*r**10*(3895
        -27265*y+81795*y**2-139045*y**3+115595*y**4-24797*y**5-28791*y**6
        +25073*y**7-7790*y**8+850*y**9))+21*MBhat**2*(-12000*(-1+y)**8+4950*r**16*(
        -1+y)**8+r**2*(-1+y)**6*(-197830+88800*logy*(-1+y)**2+484460*y-331030*y**2
        +29600*y**3+32165*y**4+8198*y**5-28546*y**6+3038*y**7+1145*y**8)+100*r**6*(
        -1+y)**3*(-13999+25584*logy*(-1+y)**5+44411*y-24862*y**2-60418*y**3
        +93044*y**4-41140*y**5+3497*y**6-469*y**7+362*y**8+6*y**9)+10*r**14*(
        -1258+10064*y-35224*y**2+70448*y**3-99715*y**4+93128*y**5-57652*y**6
        +22808*y**7-5227*y**8+528*y**9)+10*r**8*(-1+y)**2*(127565-6180*logy*(
        -1+y)**6-771570*y+1947465*y**2-2627520*y**3+1980705*y**4-729666*y**5
        +74595*y**6+12492*y**7-7986*y**8+1600*y**9)+r**12*(15380-123040*y
        +430640*y**2-861280*y**3+1235285*y**4-913768*y**5+164339*y**6+137936*y**7
        -95857*y**8+21720*y**9-1275*y**10)+5*r**10*(-1+y)*(34890+3480*logy*(
        -1+y)**7-247710*y+755310*y**2-1283210*y**3+1295195*y**4-623445*y**5
        +24228*y**6+93276*y**7-47271*y**8+9137*y**9-320*y**10)+5*r**4*(-1+y)**4*(
        -459804+215040*logy*(-1+y)**4+2054256*y-3511464*y**2+2771056*y**3
        -920479*y**4+64428*y**5+13119*y**6-17016*y**7+7059*y**8+284*y**9+y**10))
        +6*MBhat*(16800*(-1+y)**8+5720*r**16*(-1+y)**8-280*r**4*(-1+y)**4*(
        -16985+6300*logy*(-1+y)**4+74240*y-123960*y**2+95240*y**3-30110*y**4
        +813*y**5+1368*y**6-690*y**7+15*y**8+105*y**9)-r**2*(-1+y)**6*(-272180
        +134400*logy*(-1+y)**2+678760*y-473780*y**2+44800*y**3+11200*y**4
        +55699*y**5-29386*y**6-19090*y**7+5462*y**8+195*y**9)+280*r**8*(-1
        +y)**2*(-32125-10500*logy*(-1+y)**6+182250*y-424125*y**2+513000*y**3
        -332250*y**4+102843*y**5-13496*y**6+5064*y**7-1932*y**8+231*y**9)+35*r**14*(
        -380+3040*y-10640*y**2+21280*y**3-26600*y**4+28273*y**5-19544*y**6
        +8644*y**7-2228*y**8+255*y**9)-105*r**6*(-1+y)**3*(30836+80880*logy*(
        -1+y)**5-235060*y+672320*y**2-941920*y**3+673160*y**4-217583*y**5
        +21971*y**6-5284*y**7+2056*y**8+175*y**9+y**10)+168*r**12*(-135+1080*y
        -3780*y**2+7560*y**3-9450*y**4+5478*y**5-3120*y**6+3342*y**7-1376*y**8
        +181*y**9+10*y**10)-35*r**10*(-1+y)*(21220-10800*logy*(-1+y)**7-137740*y
        +375420*y**2-550100*y**3+455600*y**4-202203*y**5+79041*y**6-54504*y**7
        +20232*y**8-3401*y**9+35*y**10))+10*MBhat**3*(33600*(-1+y)**8+23040*r**16*(
        -1+y)**8+42*r**6*(-1+y)**3*(105100-43920*logy*(-1+y)**5-481580*y
        +853360*y**2-702315*y**3+240955*y**4-29492*y**5+17312*y**6-6413*y**7
        +673*y**8)+r**2*(-1+y)**6*(461472-208320*logy*(-1+y)**2-1131264*y
        +773952*y**2-105686*y**3-102704*y**4+98476*y**5+12852*y**6-10203*y**7
        -258*y**8+3*y**9)-42*r**4*(-1+y)**4*(-76424+48480*logy*(-1+y)**4+354176*y
        -628224*y**2+513286*y**3-178344*y**4+22823*y**5-9812*y**6+2648*y**7
        +348*y**8+3*y**9)+56*r**8*(-1+y)**2*(14320+8640*logy*(-1+y)**6-77280*y
        +167280*y**2-173435*y**3+71640*y**4-10743*y**5+5660*y**6+1419*y**7
        -1848*y**8+467*y**9)+14*r**14*(-4012+32096*y-112336*y**2+236327*y**3
        -315280*y**4+276038*y**5-157864*y**6+56444*y**7-11308*y**8+945*y**9)
        -42*r**12*(-1304+10432*y-36512*y**2+77526*y**3-87304*y**4+44141*y**5
        +4592*y**6-21268*y**7+13112*y**8-3643*y**9+396*y**10)-105*r**10*(1088
        -8704*y+30464*y**2-58966*y**3+56764*y**4-15710*y**5-19836*y**6+28311*y**7
        -19572*y**8+7488*y**9-1412*y**10+85*y**11)))/(25200*r**2*(-1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (-16800*(-1+y)**9+3825*r**18*(-1+y)**9+40*r**16*(142-1278*y+5112*y**2
        -11928*y**3+17892*y**4-17892*y**5+11928*y**6-13512*y**7+7083*y**8-1987*y**9
        +240*y**10)+336*r**12*(-1+y)*(21715+2100*logy*(-1+y)**8-171620*y
        +592270*y**2-1164940*y**3+1426775*y**4-1112020*y**5+536410*y**6-143308*y**7
        +21884*y**8-7784*y**9+1668*y**10)+r**2*(-1+y)**7*(13135+117600*logy*(
        -1+y)**2+91330*y-163265*y**2+39200*y**3+9800*y**4+3920*y**5+1960*y**6
        +51328*y**7-47171*y**8-282*y**9+1725*y**10)+210*r**6*(-1+y)**4*(310142
        +129660*logy*(-1+y)**5-1680370*y+3684890*y**2-4117090*y**3+2382695*y**4
        -606199*y**5+21610*y**6+4214*y**7+2387*y**8-3815*y**9+2016*y**10)+280*r**8*(
        -1+y)**3*(-56690+319800*logy*(-1+y)**6+659940*y-2609250*y**2+5078000*y**3
        -5407500*y**4+3122400*y**5-840200*y**6+46680*y**7+11739*y**8-6874*y**9
        +2955*y**10)-80*r**4*(-1+y)**5*(18971+8400*logy*(-1+y)**4-67484*y
        +84426*y**2-39484*y**3+1471*y**4+1680*y**5+280*y**6+608*y**7-2775*y**8
        +3340*y**9-1123*y**10-36*y**11)+70*r**10*(-1+y)**2*(-1075040+584700*logy*(
        -1+y)**7+6940580*y-18775290*y**2+27199250*y**3-22083125*y**4+9156975*y**5
        -1005875*y**6-439831*y**7+121539*y**8-44965*y**9+12967*y**10+15*y**11)
        +70*r**14*(326+1620*logy*(-1+y)**9-4554*y+25506*y**2-78954*y**3+152451*y**4
        -193275*y**5+162870*y**6-77454*y**7+22311*y**8-9419*y**9+1452*y**10
        +84*y**11)+70*MBhat**8*(-1+y)**4*(-240*(-1+y)**5+1275*r**10*(-1+y)**5
        -r**2*(-1+y)**3*(-845+240*logy*(-1+y)**2+1930*y-1205*y**2-3520*y**3
        +5375*y**4-1774*y**5+15*y**6)+20*r**8*(22-110*y+220*y**2-484*y**3
        +485*y**4-241*y**5+48*y**6)+6*r**6*(380+90*logy*(-1+y)**5-1990*y
        +4205*y**2-5665*y**3+6165*y**4-4071*y**5+1024*y**6)+24*r**4*(-35+175*y
        -350*y**2+770*y**3-1770*y**4+2262*y**5-1355*y**6+303*y**7))+280*MBhat**7*(
        -1+y)**3*(480*(-1+y)**6+1848*r**12*(-1+y)**6+r**2*(-1+y)**4*(1532
        -960*logy*(-1+y)**2-4024*y+2972*y**2-5150*y**3+6280*y**4-1475*y**5
        -186*y**6+3*y**7)+8*r**4*(-1+y)**2*(-395-660*logy*(-1+y)**4+920*y
        -60*y**2+625*y**3-4435*y**4+5046*y**5-1982*y**6+245*y**7)+5*r**10*(
        -556+3336*y-8340*y**2+13010*y**3-12000*y**4+6585*y**5-1990*y**6+255*y**7)
        -6*r**6*(-1+y)*(80-1080*logy*(-1+y)**5+680*y-4060*y**2+3930*y**3
        +5950*y**4-11579*y**5+6495*y**6-1387*y**7+35*y**8)-8*r**8*(-200+1200*y
        -3000*y**2+3325*y**3+1035*y**4-4848*y**5+3385*y**6-897*y**7+60*y**8))
        +140*MBhat**6*(-1+y)**2*(-3360*(-1+y)**7+7560*r**14*(-1+y)**7+r**2*(
        -1+y)**5*(-30769+14880*logy*(-1+y)**2+76418*y-53089*y**2+21520*y**3
        -11435*y**4-4418*y**5+1890*y**6+222*y**7-3*y**8)+r**4*(-1+y)**3*(-84725
        +107520*logy*(-1+y)**4+446420*y-884670*y**2+750100*y**3-168490*y**4
        -91596*y**5+38338*y**6-3436*y**7+75*y**8)+9*r**12*(1939-13573*y+40719*y**2
        -72985*y**3+80395*y**4-55629*y**5+23471*y**6-5465*y**7+528*y**8)+6*r**6*(
        -1+y)**2*(-29300-3450*logy*(-1+y)**5+149950*y-308525*y**2+302505*y**3
        -112040*y**4-22946*y**5+31489*y**6-11229*y**7+1440*y**8)+r**10*(-17585
        +123095*y-369285*y**2+627715*y**3-526915*y**4+122829*y**5+121282*y**6
        -109726*y**7+34575*y**8-3825*y**9)-2*r**8*(-1+y)*(32320-1530*logy*(
        -1+y)**6-195450*y+493215*y**2-614870*y**3+280605*y**4+112434*y**5
        -184559*y**6+89838*y**7-18573*y**8+960*y**9))+2*MBhat**2*(-235200*(-1
        +y)**9+33075*r**18*(-1+y)**9+70*r**8*(-1+y)**3*(3714265+2651700*logy*(
        -1+y)**6-19633890*y+41129625*y**2-41581000*y**3+17927250*y**4+808104*y**5
        -2703350*y**6+383568*y**7-11883*y**8+8786*y**9+285*y**10)+r**2*(-1
        +y)**7*(-3613985+1982400*logy*(-1+y)**2+9210370*y-6587585*y**2+660800*y**3
        +165200*y**4+365456*y**5+219478*y**6-384188*y**7-33617*y**8+21006*y**9
        +585*y**10)+210*r**10*(-1+y)**2*(-209895-6000*logy*(-1+y)**7+1475265*y
        -4446795*y**2+7453325*y**3-7505825*y**4+4546287*y**5-1445968*y**6+150528*y**7
        +3475*y**8-5917*y**9+1120*y**10)+30*r**16*(3533-31797*y+127188*y**2
        -296772*y**3+445158*y**4-515718*y**5+410676*y**6-223524*y**7+79641*y**8
        -16769*y**9+1584*y**10)+42*r**4*(-1+y)**5*(-2070965+643200*logy*(-1
        +y)**4+8927060*y-14676990*y**2+11071060*y**3-3410965*y**4+123456*y**5
        +28411*y**6+29076*y**7-27934*y**8+9396*y**9+1755*y**10)+14*r**14*(-10145
        +91305*y-365220*y**2+852180*y**3-1278270*y**4+1640358*y**5-1204782*y**6
        +281478*y**7+51093*y**8-45877*y**9+8040*y**10)+42*r**12*(-1+y)*(-71795
        +8700*logy*(-1+y)**8+583060*y-2075510*y**2+4232220*y**3-5412075*y**4
        +4520772*y**5-1986771*y**6+165428*y**7+140762*y**8-69144*y**9+12013*y**10
        -320*y**11)+210*r**6*(-1+y)**4*(1001431+1050180*logy*(-1+y)**5-6057335*y
        +14740120*y**2-18240720*y**3+11745810*y**4-3391974*y**5+173226*y**6
        +33214*y**7-3929*y**8+5325*y**9+206*y**10+2*y**11))+8*MBhat*(16800*(
        -1+y)**9+2640*r**18*(-1+y)**9+20*r**16*(412-3708*y+14832*y**2-34608*y**3
        +51912*y**4-51912*y**5+46368*y**6-27612*y**7+10728*y**8-2467*y**9+255*y**10)
        -210*r**6*(-1+y)**4*(177744+112320*logy*(-1+y)**5-1001040*y+2282880*y**2
        -2657280*y**3+1609440*y**4-434208*y**5+19520*y**6+3285*y**7-669*y**8
        +475*y**9+285*y**10)-r**2*(-1+y)**7*(-154960+134400*logy*(-1+y)**2
        +444320*y-356560*y**2+44800*y**3+11200*y**4+4480*y**5+36680*y**6-10903*y**7
        -20674*y**8+2937*y**9+360*y**10)+35*r**10*(-1+y)**2*(698080-192000*logy*(
        -1+y)**7-4694560*y+13411680*y**2-21008800*y**3+19328800*y**4-10253280*y**5
        +2744536*y**6-264625*y**7+41871*y**8-15571*y**9+1789*y**10)+21*r**12*(
        -1+y)*(54240+21600*logy*(-1+y)**8-412320*y+1356720*y**2-2511840*y**3
        +2837400*y**4-1967520*y**5+765520*y**6-203883*y**7+96144*y**8-28914*y**9
        +4408*y**10-35*y**11)-70*r**8*(-1+y)**3*(282152+572640*logy*(-1+y)**6
        -1120272*y+1082760*y**2+1419520*y**3-3927840*y**4+3289056*y**5-1118080*y**6
        +95409*y**7-3582*y**8+4372*y**9+342*y**10+3*y**11)+28*r**14*(920-8280*y
        +33120*y**2-77280*y**3+115920*y**4-115920*y**5+56844*y**6-16911*y**7
        +14469*y**8-5111*y**9+489*y**10+60*y**11)-r**4*(-1+y)**5*(-5577760
        +1192800*logy*(-1+y)**4+23503840*y-37641360*y**2+27479840*y**3-8062760*y**4
        +238560*y**5+12208*y**6+121043*y**7-62772*y**8-30302*y**9+29348*y**10
        +195*y**11))+40*MBhat**5*(-1+y)*(23520*(-1+y)**8+23040*r**16*(-1+y)**8
        -42*r**6*(-1+y)**3*(-73280+29760*logy*(-1+y)**5+336640*y-598880*y**2
        +495035*y**3-165035*y**4+5677*y**5+2523*y**6-1280*y**7+280*y**8)+r**2*(
        -1+y)**6*(331776-147840*logy*(-1+y)**2-811392*y+553536*y**2-85526*y**3
        -47264*y**4+53872*y**5+8316*y**6-5415*y**7-258*y**8+3*y**9)-42*r**4*(
        -1+y)**4*(-53044+34320*logy*(-1+y)**4+246496*y-438384*y**2+358406*y**3
        -121624*y**4+10031*y**5-2596*y**6+880*y**7+168*y**8+3*y**9)+14*r**14*(
        -4672+37376*y-130816*y**2+273287*y**3-361480*y**4+312998*y**5-176344*y**6
        +61724*y**7-11968*y**8+945*y**9)+14*r**8*(-1+y)**2*(40960+21600*logy*(
        -1+y)**6-224160*y+495600*y**2-527180*y**3+204840*y**4+32727*y**5
        -61474*y**6+44088*y**7-14574*y**8+1973*y**9)-84*r**12*(-846+6768*y
        -23688*y**2+49627*y**3-59152*y**4+37805*y**5-8260*y**6-6088*y**7+5414*y**8
        -1694*y**9+198*y**10)-21*r**10*(5120-40960*y+143360*y**2-276910*y**3
        +264780*y**4-54552*y**5-139052*y**6+177103*y**7-111812*y**8+39558*y**9
        -7060*y**10+425*y**11))+5*MBhat**4*(-235200*(-1+y)**9+58275*r**18*(-1
        +y)**9+42*r**6*(-1+y)**4*(-710416+1174320*logy*(-1+y)**5+2377760*y
        -1819720*y**2-2085000*y**3+3926215*y**4-1828451*y**5+143066*y**6+14374*y**7
        +2047*y**8+285*y**9)+56*r**8*(-1+y)**3*(415775-27900*logy*(-1+y)**6
        -2522550*y+6390075*y**2-8654560*y**3+6588555*y**4-2494950*y**5+300475*y**6
        +17976*y**7-15372*y**8+2176*y**9)+r**2*(-1+y)**7*(-4071075+1780800*logy*(
        -1+y)**2+9922950*y-6742275*y**2+641312*y**3+620942*y**4-11452*y**5
        -373674*y**6+17424*y**7+19077*y**8+294*y**9-3*y**10)+28*r**4*(-1+y)**5*(
        -1630572+778560*logy*(-1+y)**4+7300848*y-12508392*y**2+9896240*y**3
        -3291965*y**4+232356*y**5+13915*y**6-19840*y**7+11613*y**8+828*y**9
        +9*y**10)+28*r**10*(-1+y)**2*(116600+7650*logy*(-1+y)**7-823850*y
        +2498325*y**2-4230025*y**3+4418335*y**4-2431815*y**5+521560*y**6-22162*y**7
        -2016*y**8-7592*y**9+2240*y**10)+8*r**16*(19109-171981*y+687924*y**2
        -1663956*y**3+2611224*y**4-2772504*y**5+2009616*y**6-976464*y**7+301716*y**8
        -52724*y**9+3840*y**10)-14*r**14*(7744-69696*y+278784*y**2-720720*y**3
        +991305*y**4-674625*y**5+55524*y**6+340524*y**7-329661*y**8+152789*y**9
        -37092*y**10+3780*y**11)-84*r**12*(1100-9900*y+39600*y**2-89648*y**3
        +212933*y**4-365325*y**5+382137*y**6-294513*y**7+194139*y**8-96691*y**9
        +31315*y**10-5499*y**11+352*y**12))+8*MBhat**3*(117600*(-1+y)**9+19200*r**18*(
        -1+y)**9+140*r**8*(-1+y)**3*(-422330-131400*logy*(-1+y)**6+2402580*y
        -5612250*y**2+6826000*y**3-4465620*y**4+1389951*y**5-134266*y**6+11196*y**7
        -3438*y**8+377*y**9)-r**2*(-1+y)**7*(-2109780+974400*logy*(-1+y)**2
        +5193960*y-3571380*y**2+324800*y**3+158480*y**4+232883*y**5-145362*y**6
        -91275*y**7+16944*y**8+2490*y**9)-105*r**6*(-1+y)**4*(162916+565680*logy*(
        -1+y)**5-1380260*y+4174720*y**2-6060320*y**3+4448640*y**4-1463001*y**5
        +118173*y**6-1196*y**7+4552*y**8+465*y**9+15*y**10)+20*r**16*(2834
        -25506*y+102024*y**2-238056*y**3+386484*y**4-426258*y**5+323862*y**6
        -167148*y**7+55962*y**8-10943*y**9+945*y**10)+35*r**10*(-1+y)**2*(-137540
        +54000*logy*(-1+y)**7+908780*y-2537340*y**2+3850900*y**3-3376240*y**4
        +1510791*y**5-319453*y**6+69739*y**7+6915*y**8-6730*y**9+1378*y**10)
        -5*r**4*(-1+y)**5*(-7067448+2718240*logy*(-1+y)**4+30988032*y-51918528*y**2
        +40048832*y**3-12739184*y**4+484974*y**5+289408*y**6-95785*y**7+4020*y**8
        +19296*y**9+492*y**10+3*y**11)-35*r**14*(2036-18324*y+73296*y**2-171024*y**3
        +295488*y**4-297837*y**5+151701*y**6-26460*y**7-16920*y**8+12145*y**9
        -3021*y**10+264*y**11)-21*r**12*(-18680+168120*y-672480*y**2+1569120*y**3
        -2386000*y**4+2170338*y**5-968538*y**6+43647*y**7+202017*y**8-153408*y**9
        +54424*y**10-8985*y**11+425*y**12)))/(25200*r**2*(-1+y)**9)

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr17(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r17 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (505*r**8*(-1+y)**6+60*logy*(4+3*r**4)*(-1+y)**6-(-1+y)**4*(385-1010*y
        +745*y**2-355*y**4+198*y**5+5*y**6)-8*r**2*(-1+y)**2*(-115+460*y
        -690*y**2+570*y**3-265*y**4+38*y**5+8*y**6)+6*r**4*(-100+630*y-1665*y**2
        +2490*y**3-2250*y**4+1212*y**5-365*y**6+48*y**7)+4*r**6*(-110+660*y
        -1650*y**2+2700*y**3-2935*y**4+1878*y**5-663*y**6+100*y**7))/(120*(
        -1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (-2851+605*r**2+720*r**4+880*r**6+1555*r**8-909*r**10+60*logy*(-8
        -56*r**2-69*r**4+27*r**6-8*MBhat*(5+9*r**2)+6*MBhat**2*(4+3*r**4))*(
        -1+y)**7+20437*y-875*r**2*y-900*r**4*y-7780*r**6*y-10885*r**8*y+6363*r**10*y
        -62991*y**2-9135*r**2*y**2-11790*r**4*y**2+29010*r**6*y**2+32655*r**8*y**2
        -19089*r**10*y**2+108345*y**3+38745*r**2*y**3+48630*r**4*y**3-59690*r**6*y**3
        -54425*r**8*y**3+31815*r**10*y**3-112620*y**4-64170*r**2*y**4-96210*r**4*y**4
        +73430*r**6*y**4+64715*r**8*y**4-31815*r**10*y**4+70644*y**5+48390*r**2*y**5
        +118866*r**4*y**5-51078*r**6*y**5-56589*r**8*y**5+19089*r**10*y**5-22452*y**6
        -9162*r**2*y**6-92466*r**4*y**6+17566*r**6*y**6+30163*r**8*y**6-6363*r**10*y**6
        -3924*y**7-7194*r**2*y**7+42282*r**4*y**7-1606*r**6*y**7-9109*r**8*y**7
        +909*r**10*y**7+10935*y**8+1629*r**2*y**8-10110*r**4*y**8-1014*r**6*y**8
        +1200*r**8*y**8-8225*y**9+1717*r**2*y**9+978*r**4*y**9+282*r**6*y**9+3227*y**10
        -535*r**2*y**10-525*y**11-15*r**2*y**11-6*MBhat**2*(-1+y)*(-505*r**8*(
        -1+y)**6+(-1+y)**4*(385-1010*y+745*y**2-355*y**4+198*y**5+5*y**6)
        +8*r**2*(-1+y)**2*(-115+460*y-690*y**2+570*y**3-265*y**4+38*y**5
        +8*y**6)-6*r**4*(-100+630*y-1665*y**2+2490*y**3-2250*y**4+1212*y**5
        -365*y**6+48*y**7)-4*r**6*(-110+660*y-1650*y**2+2700*y**3-2935*y**4
        +1878*y**5-663*y**6+100*y**7))+2*MBhat*(2100*r**10*(-1+y)**7+(-1+y)**5*(2140
        -5480*y+3940*y**2-310*y**3-940*y**4+461*y**5-6*y**6+3*y**7)-12*r**2*(
        -1+y)**3*(255-840*y+900*y**2-510*y**3+95*y**4+164*y**5-118*y**6
        +30*y**7)-6*r**4*(-1+y)**2*(640-3200*y+6400*y**2-8370*y**3+7300*y**4
        -4209*y**5+1559*y**6-359*y**7+47*y**8)+15*r**8*(228-1596*y+4788*y**2
        -8666*y**3+10110*y**4-7483*y**5+3437*y**6-895*y**7+101*y**8)-4*r**6*(
        -400+2800*y-8400*y**2+14270*y**3-15695*y**4+12999*y**5-8521*y**6
        +3925*y**7-1128*y**8+150*y**9)))/(720*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (45690+106040*r**2-94450*r**4-58400*r**6+2950*r**8-2840*r**10+1010*r**12
        +300*logy*(-8*MBhat**3*(5+9*r**2)+3*MBhat**4*(4+3*r**4)+8*MBhat*(7
        +52*r**2+21*r**4)+MBhat**2*(8-24*r**2-129*r**4+35*r**6)+2*(-18-196*r**2
        -163*r**4-56*r**6+9*r**8))*(-1+y)**8-376320*y-965920*r**2*y+657800*r**4*y
        +433600*r**6*y-18200*r**8*y+22720*r**10*y-8080*r**12*y+1360320*y**2
        +3851120*r**2*y**2-1911100*r**4*y**2-1383200*r**6*y**2+42100*r**8*y**2-79520*r**10*y**2
        +28280*r**12*y**2-2821440*y**3-8799840*r**2*y**3+2909400*r**4*y**3+2452800*r**6*y**3
        -33800*r**8*y**3+159040*r**10*y**3-56560*r**12*y**3+3678000*y**4+12646200*r**2*y**4
        -2267550*r**4*y**4-2595600*r**6*y**4-33350*r**8*y**4-198800*r**10*y**4+70700*r**12*y**4
        -3093048*y**5-11770800*r**2*y**5+480600*r**4*y**5+1565280*r**6*y**5+90160*r**8*y**5
        +183088*r**10*y**5-56560*r**12*y**5+1644829*y**6+7014720*r**2*y**6+542610*r**4*y**6
        -356860*r**6*y**6-55295*r**8*y**6-132044*r**10*y**6+28280*r**12*y**6-509864*y**7
        -2529840*r**2*y**7-349080*r**4*y**7-162640*r**6*y**7-80*r**8*y**7+59584*r**10*y**7
        -8080*r**12*y**7+77998*y**8+480120*r**2*y**8-18240*r**4*y**8+138680*r**6*y**8
        +9010*r**8*y**8-15548*r**10*y**8+1010*r**12*y**8-18920*y**9-25520*r**2*y**9
        +62800*r**4*y**9-37200*r**6*y**9-4200*r**8*y**9+1800*r**10*y**9+25798*y**10
        -12560*r**2*y**10-12710*r**4*y**10+3540*r**6*y**10+705*r**8*y**10-19544*y**11
        +8080*r**2*y**11-80*r**4*y**11+7984*y**12-1800*r**2*y**12-1568*y**13+85*y**14
        -15*MBhat**4*(-1+y)**2*(-505*r**8*(-1+y)**6+(-1+y)**4*(385-1010*y
        +745*y**2-355*y**4+198*y**5+5*y**6)+8*r**2*(-1+y)**2*(-115+460*y
        -690*y**2+570*y**3-265*y**4+38*y**5+8*y**6)-6*r**4*(-100+630*y-1665*y**2
        +2490*y**3-2250*y**4+1212*y**5-365*y**6+48*y**7)-4*r**6*(-110+660*y
        -1650*y**2+2700*y**3-2935*y**4+1878*y**5-663*y**6+100*y**7))+20*MBhat*(250*r**12*(
        -1+y)**8+2*(-1+y)**6*(-1875+4170*y-2505*y**2+140*y**3-16*y**4-7*y**5
        +36*y**6+9*y**7)+8*r**2*(-1+y)**4*(-65+1040*y-3120*y**2+3640*y**3
        -1540*y**4+57*y**5-8*y**6+14*y**7)+3*r**4*(-1+y)**3*(-530+1810*y
        -1520*y**2-1280*y**3+4720*y**4-3941*y**5+1213*y**6-191*y**7+7*y**8)
        +r**8*(-650+5200*y-18200*y**2+36400*y**3-43730*y**4+30619*y**5-12872*y**6
        +3742*y**7-524*y**8+15*y**9)-r**6*(-1+y)**2*(-3680+22080*y-55200*y**2
        +73600*y**3-62280*y**4+35121*y**5-14602*y**6+3984*y**7-630*y**8+47*y**9)
        +3*r**10*(-200+1600*y-5600*y**2+11200*y**3-15336*y**4+14895*y**5-9576*y**6
        +3948*y**7-948*y**8+101*y**9))+10*MBhat**3*(-1+y)*(2100*r**10*(-1+y)**7
        +(-1+y)**5*(2140-5480*y+3940*y**2-310*y**3-940*y**4+461*y**5-6*y**6
        +3*y**7)-12*r**2*(-1+y)**3*(255-840*y+900*y**2-510*y**3+95*y**4+164*y**5
        -118*y**6+30*y**7)-6*r**4*(-1+y)**2*(640-3200*y+6400*y**2-8370*y**3
        +7300*y**4-4209*y**5+1559*y**6-359*y**7+47*y**8)+15*r**8*(228-1596*y
        +4788*y**2-8666*y**3+10110*y**4-7483*y**5+3437*y**6-895*y**7+101*y**8)
        -4*r**6*(-400+2800*y-8400*y**2+14270*y**3-15695*y**4+12999*y**5-8521*y**6
        +3925*y**7-1128*y**8+150*y**9))+5*MBhat**2*(2744*r**12*(-1+y)**8-(-1
        +y)**6*(-3015+5550*y-2295*y**2-688*y**3+191*y**4-814*y**5+496*y**6
        -2*y**7+y**8)+3*r**2*(-1+y)**4*(-1103+3932*y-4938*y**2+892*y**3-298*y**4
        +1132*y**5+238*y**6-148*y**7+5*y**8)+6*r**4*(-1+y)**3*(-300+2790*y
        -8805*y**2+9185*y**3-4820*y**4+850*y**5+685*y**6-361*y**7+56*y**8)
        -2*r**6*(-1+y)**2*(-320+870*y+975*y**2+9290*y**3-34575*y**4+52722*y**5
        -39223*y**6+17022*y**7-4041*y**8+400*y**9)+3*r**10*(-1905+15240*y
        -53340*y**2+112024*y**3-152792*y**4+137368*y**5-81284*y**6+30424*y**7
        -6503*y**8+600*y**9)-3*r**8*(-275+2200*y-7700*y**2+16760*y**3-17390*y**4
        -4264*y**5+27917*y**6-28592*y**7+15079*y**8-4240*y**9+505*y**10)))/(1800*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (-530595-3650535*r**2+1187025*r**4+2183125*r**6+840875*r**8-37905*r**10
        +10535*r**12-2525*r**14+2100*logy*(-56-1048*r**2-2025*r**4-765*r**6
        -165*r**8+15*r**10-8*MBhat**5*(5+9*r**2)+MBhat**6*(8+6*r**4)+16*MBhat**3*(9
        +71*r**2+28*r**4)+MBhat**4*(24+8*r**2-189*r**4+43*r**6)+8*MBhat*(27
        +379*r**2+415*r**4+95*r**6)+2*MBhat**2*(-148-1528*r**2-852*r**4-243*r**6
        +29*r**8))*(-1+y)**9+4892955*y+35055615*r**2*y-6430725*r**4*y-18041625*r**6*y
        -7221375*r**8*y+309645*r**10*y-94815*r**12*y+22725*r**14*y-20101020*y**2
        -150126060*r**2*y**2+6586650*r**4*y**2+64937250*r**6*y**2+27326250*r**8*y**2
        -1096830*r**10*y**2+379260*r**12*y**2-90900*r**14*y**2+48313580*y**3+376703740*r**2*y**3
        +35661150*r**4*y**3-132242250*r**6*y**3-59603250*r**8*y**3+2181270*r**10*y**3
        -884940*r**12*y**3+212100*r**14*y**3-74939970*y**4-611272410*r**2*y**4-142794225*r**4*y**4
        +164626875*r**6*y**4+82128375*r**8*y**4-2610405*r**10*y**4+1327410*r**12*y**4
        -318150*r**14*y**4+77903490*y**5+666732570*r**2*y**5+249957225*r**4*y**5
        -124143075*r**6*y**5-73396575*r**8*y**5+1816605*r**10*y**5-1327410*r**12*y**5
        +318150*r**14*y**5-54409397*y**6-490678335*r**2*y**6-256111800*r**4*y**6
        +49411320*r**6*y**6+41418545*r**8*y**6-659673*r**10*y**6+993720*r**12*y**6
        -212100*r**14*y**6+24757597*y**7+236560135*r**2*y**7+161545440*r**4*y**7
        -3296720*r**6*y**7-12990145*r**8*y**7+241353*r**10*y**7-607880*r**12*y**7
        +90900*r**14*y**7-6800543*y**8-68733595*r**2*y**8-60918165*r**4*y**8-4293625*r**6*y**8
        +1026130*r**8*y**8-210042*r**10*y**8+237195*r**12*y**8-22725*r**14*y**8+999527*y**9
        +9479155*r**2*y**9+12408585*r**4*y**9+367325*r**6*y**9+671930*r**8*y**9+85638*r**10*y**9
        -54355*r**12*y**9+2525*r**14*y**9-101598*y**10-25130*r**2*y**10-1203090*r**4*y**10
        +656950*r**6*y**10-219695*r**8*y**10-22617*r**10*y**10+5600*r**12*y**10-35714*y**11
        +21770*r**2*y**11+117810*r**4*y**11-170870*r**6*y**11+18935*r**8*y**11+2961*r**10*y**11
        +117348*y**12-111300*r**2*y**12+3045*r**4*y**12+5145*r**6*y**12-100212*y**13
        +56980*r**2*y**13-8925*r**4*y**13+175*r**6*y**13+42943*y**14-13195*r**2*y**14
        -9047*y**15+595*r**2*y**15+656*y**16-70*MBhat**6*(-1+y)**3*(-505*r**8*(
        -1+y)**6+(-1+y)**4*(385-1010*y+745*y**2-355*y**4+198*y**5+5*y**6)
        +8*r**2*(-1+y)**2*(-115+460*y-690*y**2+570*y**3-265*y**4+38*y**5
        +8*y**6)-6*r**4*(-100+630*y-1665*y**2+2490*y**3-2250*y**4+1212*y**5
        -365*y**6+48*y**7)-4*r**6*(-110+660*y-1650*y**2+2700*y**3-2935*y**4
        +1878*y**5-663*y**6+100*y**7))+70*MBhat**5*(-1+y)**2*(2100*r**10*(-1
        +y)**7+(-1+y)**5*(2140-5480*y+3940*y**2-310*y**3-940*y**4+461*y**5
        -6*y**6+3*y**7)-12*r**2*(-1+y)**3*(255-840*y+900*y**2-510*y**3+95*y**4
        +164*y**5-118*y**6+30*y**7)-6*r**4*(-1+y)**2*(640-3200*y+6400*y**2
        -8370*y**3+7300*y**4-4209*y**5+1559*y**6-359*y**7+47*y**8)+15*r**8*(228
        -1596*y+4788*y**2-8666*y**3+10110*y**4-7483*y**5+3437*y**6-895*y**7
        +101*y**8)-4*r**6*(-400+2800*y-8400*y**2+14270*y**3-15695*y**4+12999*y**5
        -8521*y**6+3925*y**7-1128*y**8+150*y**9))+35*MBhat**4*(-1+y)*(5488*r**12*(
        -1+y)**8-(-1+y)**6*(-3179+4918*y-1019*y**2-1536*y**3+417*y**4-1026*y**5
        +467*y**6-4*y**7+2*y**8)+r**2*(-1+y)**4*(-6013+24532*y-37758*y**2
        +17492*y**3-4208*y**4+3372*y**5+1406*y**6-308*y**7+45*y**8)+6*r**4*(
        -1+y)**3*(-720+5490*y-15705*y**2+14165*y**3-3920*y**4-2466*y**5+2729*y**6
        -885*y**7+112*y**8)-2*r**6*(-1+y)**2*(-1080+5190*y-9105*y**2+37370*y**3
        -87075*y**4+113058*y**5-79615*y**6+33678*y**7-7941*y**8+800*y**9)+3*r**10*(
        -4113+32904*y-115164*y**2+241016*y**3-326794*y**4+291704*y**5-171052*y**6
        +63272*y**7-13309*y**8+1200*y**9)-5*r**8*(-641+5128*y-17948*y**2+37528*y**3
        -44696*y**4+19144*y**5+16150*y**6-26456*y**7+16033*y**8-4848*y**9+606*y**10))
        +2*MBhat*(6500*r**14*(-1+y)**9+140*r**2*(-1+y)**5*(-28243+135712*y
        -249048*y**2+211512*y**3-75618*y**4+4992*y**5+520*y**6+176*y**7+57*y**8
        +12*y**9)+140*r**6*(-1+y)**3*(7725-40650*y+84525*y**2-84200*y**3+34650*y**4
        +9498*y**5-13868*y**6+4308*y**7-522*y**8+34*y**9)-(-1+y)**7*(989860
        -2206520*y+1330060*y**2-75600*y**3-18900*y**4+2667*y**5-7378*y**6
        -6430*y**7-1294*y**8+255*y**9)-35*r**4*(-1+y)**4*(99700-398900*y+548800*y**2
        -216800*y**3-140600*y**4+116707*y**5-8351*y**6-2588*y**7-260*y**8-15*y**9
        +3*y**10)-35*r**8*(-1+y)**2*(12300-86100*y+258300*y**2-430500*y**3
        +430500*y**4-273975*y**5+110777*y**6-34340*y**7+6996*y**8-845*y**9
        +47*y**10)+35*r**12*(628-5652*y+22608*y**2-52752*y**3+79128*y**4-86121*y**5
        +70665*y**6-39420*y**7+14400*y**8-3115*y**9+303*y**10)+28*r**10*(1505
        -13545*y+54180*y**2-126420*y**3+189630*y**4-181104*y**5+97824*y**6
        -24426*y**7+714*y**8+2314*y**9-747*y**10+75*y**11))+10*MBhat**3*(8016*r**14*(
        -1+y)**9-14*r**4*(-1+y)**4*(2776-7160*y-2480*y**2+18715*y**3-32135*y**4
        +20128*y**5-2276*y**6-575*y**7+127*y**8)+(-1+y)**7*(-147400+325040*y
        -192760*y**2+6174*y**3+4676*y**4-1652*y**5+980*y**6+463*y**7-2*y**8
        +y**9)-14*r**2*(-1+y)**5*(-92-16672*y+59088*y**2-74682*y**3+34988*y**4
        -4345*y**5+1524*y**6-280*y**7-12*y**8+3*y**9)+56*r**6*(-1+y)**3*(2470
        -14820*y+37050*y**2-45665*y**3+31720*y**4-10993*y**5+821*y**6+1163*y**7
        -463*y**8+57*y**9)+14*r**12*(1352-12168*y+48672*y**2-117453*y**3+186557*y**4
        -200970*y**5+148386*y**6-73884*y**7+23616*y**8-4339*y**9+343*y**10)
        -7*r**8*(-1+y)**2*(-3880+27160*y-81480*y**2+113910*y**3-50210*y**4
        -69692*y**5+107372*y**6-74823*y**7+28933*y**8-5955*y**9+505*y**10)
        -14*r**10*(500-4500*y+18000*y**2-44646*y**3+62418*y**4-34137*y**5
        -21519*y**6+50400*y**7-40590*y**8+17965*y**9-4341*y**10+450*y**11))
        +7*MBhat**2*(5390*r**14*(-1+y)**9+(-1+y)**7*(383930-856660*y+517130*y**2
        -29600*y**3-1655*y**4-5762*y**5-606*y**6-2082*y**7+105*y**8)+60*r**4*(
        -1+y)**4*(13729-60125*y+98950*y**2-70550*y**3+10910*y**4+7290*y**5
        -969*y**6+21*y**7-106*y**8+10*y**9)+10*r**6*(-1+y)**3*(-24575+132870*y
        -288435*y**2+311680*y**3-203355*y**4+80022*y**5-26465*y**6+9852*y**7
        -3306*y**8+432*y**9)+5*r**2*(-1+y)**5*(132420-713040*y+1436280*y**2
        -1324240*y**3+508165*y**4-34788*y**5-7513*y**6+2008*y**7-729*y**8-4*y**9
        +y**10)+30*r**12*(490-4410*y+17640*y**2-41160*y**3+65625*y**4-74193*y**5
        +57876*y**6-30804*y**7+10701*y**8-2189*y**9+200*y**10)-5*r**8*(-1+y)**2*(7190
        -46850*y+128370*y**2-189590*y**3+214745*y**4-207063*y**5+182272*y**6
        -92248*y**7+29847*y**8-5473*y**9+400*y**10)-3*r**10*(-1060+9540*y
        -38160*y**2+89040*y**3-98525*y**4+33509*y**5-17031*y**6+48559*y**7
        -40071*y**8+18319*y**9-4625*y**10+505*y**11)))/(8400*(-1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (2996735+39085520*r**2+31550820*r**4-50879920*r**6-17416000*r**8-5505360*r**10
        +189980*r**12-26320*r**14+4545*r**16+4200*logy*(-148-4240*r**2-15693*r**4
        -11672*r**6-2735*r**8-456*r**10+27*r**12-16*MBhat**7*(5+9*r**2)+3*MBhat**8*(4
        +3*r**4)+48*MBhat**5*(11+90*r**2+35*r**4)+2*MBhat**6*(40+40*r**2-249*r**4
        +51*r**6)+16*MBhat**3*(173+2375*r**2+2235*r**4+333*r**6)+16*MBhat*(55
        +1260*r**2+3234*r**4+1340*r**6+225*r**8)+MBhat**4*(-1880-19440*r**2
        -9252*r**4-2100*r**6+255*r**8)+6*MBhat**2*(-360-6456*r**2-10625*r**4
        -2263*r**6-400*r**8+29*r**10))*(-1+y)**10-30588950*y-408663200*r**2*y
        -381418800*r**4*y+459776800*r**6*y+162673000*r**8*y+53138400*r**10*y
        -1786400*r**12*y+263200*r**14*y-45450*r**16*y+140758275*y**2+1928024400*r**2*y**2
        +2045937600*r**4*y**2-1823883600*r**6*y**2-674593500*r**8*y**2-229546800*r**10*y**2
        +7471800*r**12*y**2-1184400*r**14*y**2+204525*r**16*y**2-384679400*y**3
        -5408518400*r**2*y**3-6444492600*r**4*y**3+4128353600*r**6*y**3+1626611000*r**8*y**3
        +583396800*r**10*y**3-18223800*r**12*y**3+3158400*r**14*y**3-545400*r**16*y**3
        +691836950*y**4+9999147200*r**2*y**4+13255180050*r**4*y**4-5753946800*r**6*y**4
        -2501959250*r**8*y**4-963488400*r**10*y**4+28489650*r**12*y**4-5527200*r**14*y**4
        +954450*r**16*y**4-856311540*y**5-12746912640*r**2*y**5-18674461260*r**4*y**5
        +4845795360*r**6*y**5+2519897100*r**8*y**5+1075747680*r**10*y**5-29424780*r**12*y**5
        +6632640*r**14*y**5-1145340*r**16*y**5+739700150*y**6+11370363200*r**2*y**6
        +18330296250*r**4*y**6-1979222000*r**6*y**6-1617460250*r**8*y**6-816018000*r**10*y**6
        +19757850*r**12*y**6-5527200*r**14*y**6+954450*r**16*y**6-441315944*y**7
        -7031659904*r**2*y**7-12451307400*r**4*y**7-341313280*r**6*y**7+581840840*r**8*y**7
        +408039744*r**10*y**7-8386056*r**12*y**7+3480000*r**14*y**7-545400*r**16*y**7
        +174668640*y**8+2904444720*r**2*y**8+5655510840*r**4*y**8+870496200*r**6*y**8
        -53697840*r**8*y**8-122520720*r**10*y**8+2948610*r**12*y**8-1846200*r**14*y**8
        +204525*r**16*y**8-41365880*y**9-735783680*r**2*y**9-1581123600*r**4*y**9
        -451822000*r**6*y**9-33930400*r**8*y**9+15919680*r**10*y**9-1341620*r**12*y**9
        +634000*r**14*y**9-45450*r**16*y**9+3542600*y**10+93579440*r**2*y**10+217839300*r**4*y**10
        +107446360*r**6*y**10+5611900*r**8*y**10+1569120*r**10*y**10+369110*r**12*y**10
        -129400*r**14*y**10+4545*r**16*y**10+1605000*y**11-3870720*r**2*y**11-1780800*r**4*y**11
        -12112800*r**6*y**11+3368400*r**8*y**11-792960*r**10*y**11-72240*r**12*y**11
        +12000*r**14*y**11-1251502*y**12+992768*r**2*y**12-1666350*r**4*y**12+1557080*r**6*y**12
        -976990*r**8*y**12+60816*r**10*y**12+7896*r**12*y**12+337540*y**13-29120*r**2*y**13
        -133140*r**4*y**13-215600*r**6*y**13+30940*r**8*y**13+278130*y**14-371520*r**2*y**14
        +106050*r**4*y**14-29400*r**6*y**14+1050*r**8*y**14-329720*y**15+225280*r**2*y**15
        -36960*r**4*y**15+148025*y**16-57280*r**2*y**16-31554*y**17+3936*r**2*y**17
        +2445*y**18-210*MBhat**8*(-1+y)**4*(-505*r**8*(-1+y)**6+(-1+y)**4*(385
        -1010*y+745*y**2-355*y**4+198*y**5+5*y**6)+8*r**2*(-1+y)**2*(-115
        +460*y-690*y**2+570*y**3-265*y**4+38*y**5+8*y**6)-6*r**4*(-100+630*y
        -1665*y**2+2490*y**3-2250*y**4+1212*y**5-365*y**6+48*y**7)-4*r**6*(
        -110+660*y-1650*y**2+2700*y**3-2935*y**4+1878*y**5-663*y**6+100*y**7))
        +280*MBhat**7*(-1+y)**3*(2100*r**10*(-1+y)**7+(-1+y)**5*(2140-5480*y
        +3940*y**2-310*y**3-940*y**4+461*y**5-6*y**6+3*y**7)-12*r**2*(-1+y)**3*(255
        -840*y+900*y**2-510*y**3+95*y**4+164*y**5-118*y**6+30*y**7)-6*r**4*(
        -1+y)**2*(640-3200*y+6400*y**2-8370*y**3+7300*y**4-4209*y**5+1559*y**6
        -359*y**7+47*y**8)+15*r**8*(228-1596*y+4788*y**2-8666*y**3+10110*y**4
        -7483*y**5+3437*y**6-895*y**7+101*y**8)-4*r**6*(-400+2800*y-8400*y**2
        +14270*y**3-15695*y**4+12999*y**5-8521*y**6+3925*y**7-1128*y**8+150*y**9))
        +140*MBhat**6*(-1+y)**2*(8232*r**12*(-1+y)**8-(-1+y)**6*(-3343+4286*y
        +257*y**2-2384*y**3+643*y**4-1238*y**5+438*y**6-6*y**7+3*y**8)+r**2*(
        -1+y)**4*(-8717+37268*y-60702*y**2+32308*y**3-7522*y**4+3348*y**5
        +2098*y**6-172*y**7+75*y**8)+6*r**4*(-1+y)**3*(-1140+8190*y-22605*y**2
        +19145*y**3-3020*y**4-5782*y**5+4773*y**6-1409*y**7+168*y**8)+9*r**10*(
        -2107+16856*y-58996*y**2+123336*y**3-166932*y**4+148680*y**5-86940*y**6
        +32040*y**7-6705*y**8+600*y**9)-2*r**6*(-1+y)**2*(-1840+9510*y-19185*y**2
        +65450*y**3-139575*y**4+173394*y**5-120007*y**6+50334*y**7-11841*y**8
        +1200*y**9)+r**8*(5585-44680*y+156380*y**2-325000*y**3+394790*y**4
        -204232*y**5-77749*y**6+178784*y**7-115093*y**8+35760*y**9-4545*y**10))
        +120*MBhat**5*(-1+y)*(8016*r**14*(-1+y)**9-14*r**4*(-1+y)**4*(1186-1730*y
        -7040*y**2+14875*y**3-17975*y**4+8305*y**5+1363*y**6-1148*y**7+148*y**8)
        +(-1+y)**7*(-94900+208280*y-122620*y**2+2254*y**3+5124*y**4-1456*y**5
        -28*y**6+211*y**7-2*y**8+y**9)-14*r**2*(-1+y)**5*(-612-8352*y+34128*y**2
        -45562*y**3+22668*y**4-3889*y**5+1460*y**6-168*y**7-12*y**8+3*y**9)
        +14*r**6*(-1+y)**3*(6200-37200*y+93000*y**2-109060*y**3+64600*y**4
        -8851*y**5-11318*y**6+8636*y**7-2482*y**8+275*y**9)+14*r**12*(1602
        -14418*y+57672*y**2-138453*y**3+218057*y**4-232470*y**5+169386*y**6
        -82884*y**7+25866*y**8-4589*y**9+343*y**10)-7*r**8*(-1+y)**2*(-2580
        +18060*y-54180*y**2+68410*y**3-8250*y**4-88970*y**5+113838*y**6-75841*y**7
        +28963*y**8-5955*y**9+505*y**10)-28*r**10*(550-4950*y+19800*y**2-47523*y**3
        +71013*y**4-62415*y**5+25947*y**6+4914*y**7-12951*y**8+7409*y**9-2019*y**10
        +225*y**11))+6*MBhat**2*(12005*r**16*(-1+y)**10-14*r**2*(-1+y)**6*(-3048835
        +14132140*y-25071810*y**2+20588140*y**3-7083835*y**4+399408*y**5+58073*y**6
        +26108*y**7+2878*y**8+1068*y**9+25*y**10)+70*r**6*(-1+y)**4*(-296309
        +1642074*y-3697845*y**2+4251560*y**3-2509770*y**4+547176*y**5+65318*y**6
        -16976*y**7-2833*y**8-890*y**9+95*y**10)+(-1+y)**8*(6820865-15153730*y
        +9088865*y**2-504000*y**3-126000*y**4-7392*y**5-41846*y**6-33508*y**7
        -13383*y**8+1234*y**9+255*y**10)+70*r**8*(-1+y)**3*(58025-382175*y
        +1062525*y**2-1602875*y**3+1392875*y**4-735021*y**5+226904*y**6-53680*y**7
        +10063*y**8-3057*y**9+376*y**10)+70*r**4*(-1+y)**5*(357415-1149575*y
        +705400*y**2+1419600*y**3-2303550*y**4+1083858*y**5-107718*y**6-11034*y**7
        +159*y**8-595*y**9-10*y**10+2*y**11)-14*r**12*(2085-20850*y+93825*y**2
        -250200*y**3+437850*y**4-453948*y**5+208020*y**6-18860*y**7-1315*y**8
        +5170*y**9-2057*y**10+280*y**11)-14*r**10*(-1+y)**2*(-26965+207020*y
        -689770*y**2+1298340*y**3-1501125*y**4+1208172*y**5-729081*y**6+462252*y**7
        -173610*y**8+44368*y**9-6761*y**10+400*y**11)+30*r**14*(-1415+14150*y
        -63675*y**2+169800*y**3-297150*y**4+379092*y**5-363342*y**6+247320*y**7
        -117495*y**8+37110*y**9-7011*y**10+600*y**11))+8*MBhat*(3000*r**16*(
        -1+y)**10+210*r**4*(-1+y)**5*(2292-140820*y+605040*y**2-1036240*y**3
        +841520*y**4-297664*y**5+23128*y**6+2655*y**7+475*y**8+75*y**9+19*y**10)
        +105*r**8*(-1+y)**3*(-27600+175200*y-462600*y**2+645000*y**3-487500*y**4
        +166500*y**5+22224*y**6-35431*y**7+10281*y**8-845*y**9+51*y**10)-(
        -1+y)**8*(2153800-4769600*y+2846800*y**2-154000*y**3-38500*y**4-15400*y**5
        +5488*y**6-12179*y**7-8834*y**8-1347*y**9+492*y**10)-70*r**6*(-1+y)**4*(
        -276200+1496400*y-3258600*y**2+3540800*y**3-1851600*y**4+258240*y**5
        +101428*y**6-11187*y**7-3738*y**8-448*y**9-18*y**10+3*y**11)-21*r**10*(
        -1+y)**2*(-30800+246400*y-862400*y**2+1724800*y**3-2156000*y**4+1724800*y**5
        -889828*y**6+279927*y**7-68440*y**8+10658*y**9-1004*y**10+47*y**11)
        -3*r**2*(-1+y)**6*(6730800-30451200*y+52732800*y**2-42211200*y**3+14080800*y**4
        -705600*y**5-134764*y**6-20859*y**7-18204*y**8-5314*y**9-704*y**10
        +85*y**11)+60*r**14*(-228+2280*y-10260*y**2+27360*y**3-47880*y**4
        +57456*y**5-51632*y**6+36500*y**7-17880*y**8+5825*y**9-1138*y**10+101*y**11)
        +28*r**12*(-1450+14500*y-65250*y**2+174000*y**3-304500*y**4+365400*y**5
        -292254*y**6+132855*y**7-23055*y**8-2870*y**9+3410*y**10-861*y**11
        +75*y**12))+8*MBhat**3*(20040*r**16*(-1+y)**10-140*r**6*(-1+y)**4*(-28139
        +148854*y-312195*y**2+316360*y**3-158280*y**4+249*y**5+29108*y**6
        -5496*y**7-1428*y**8+227*y**9)-(-1+y)**8*(6388740-14230680*y+8568540*y**2
        -484400*y**3-79520*y**4-48797*y**5-48342*y**6-16575*y**7-2676*y**8
        +750*y**9)-105*r**4*(-1+y)**5*(231740-979900*y+1512800*y**2-916800*y**3
        +3160*y**4+169935*y**5-26227*y**6+844*y**7-868*y**8-75*y**9+15*y**10)
        +35*r**8*(-1+y)**3*(-48500+339500*y-1018500*y**2+1697500*y**3-1581880*y**4
        +840303*y**5-192169*y**6-437*y**7+22419*y**8-6706*y**9+670*y**10)-5*r**2*(
        -1+y)**6*(4683060-22722240*y+42063360*y**2-36022240*y**3+12962716*y**4
        -790986*y**5-168896*y**6-19177*y**7+1668*y**8-1368*y**9-12*y**10+3*y**11)
        -21*r**10*(-1+y)**2*(6060-48480*y+169680*y**2-339360*y**3+346220*y**4
        -75978*y**5-271800*y**6+266793*y**7-142644*y**8+44344*y**9-7420*y**10
        +505*y**11)+20*r**14*(-2885+28850*y-129825*y**2+346200*y**3-633990*y**4
        +829626*y**5-770658*y**6+508290*y**7-233055*y**8+70655*y**9-12725*y**10
        +1029*y**11)-35*r**12*(20-200*y+900*y**2-2400*y**3-18288*y**4+55725*y**5
        -27354*y**6-40191*y**7+58680*y**8-38945*y**9+14966*y**10-3213*y**11
        +300*y**12))+MBhat**4*(291375*r**16*(-1+y)**10+280*r**6*(-1+y)**4*(-58325
        +318450*y-701625*y**2+734200*y**3-467025*y**4+164082*y**5-26881*y**6
        -2712*y**7-636*y**8+272*y**9)+210*r**4*(-1+y)**5*(361392-1621920*y
        +2781240*y**2-2182520*y**3+572615*y**4+73741*y**5-2614*y**6-3866*y**7
        -1873*y**8+285*y**9)+420*r**2*(-1+y)**6*(129780-713520*y+1459080*y**2
        -1363760*y**3+533925*y**4-42900*y**5-5927*y**6+2048*y**7-397*y**8-12*y**9
        +3*y**10)-(-1+y)**8*(-34820625+77537250*y-46664625*y**2+2396800*y**3
        +464170*y**4+538636*y**5+19698*y**6+60816*y**7-2505*y**8-30*y**9+15*y**10)
        +420*r**8*(-1+y)**3*(-9800+66050*y-189225*y**2+255525*y**3-205895*y**4
        +89535*y**5-27312*y**6-15494*y**7+12944*y**8-3768*y**9+440*y**10)+120*r**14*(
        -6139+61390*y-276255*y**2+755440*y**3-1377460*y**4+1741152*y**5-1553832*y**6
        +976080*y**7-421140*y**8+118080*y**9-19156*y**10+1336*y**11)-84*r**10*(
        -1+y)**2*(13500-108000*y+378000*y**2-659200*y**3+413395*y**4+381656*y**5
        -824413*y**6+784448*y**7-435751*y**8+144560*y**9-26355*y**10+2000*y**11)
        -70*r**12*(-5120+51200*y-230400*y**2+651360*y**3-1070265*y**4+846426*y**5
        +38619*y**6-809832*y**7+931545*y**8-576610*y**9+213817*y**10-44856*y**11
        +4116*y**12)))/(25200*(-1+y)**10)
    

    return 0. 

@jit(float64(int64, float64, float64, float64, int64, int64), cache=True, nopython=True)
def Xr18(i,y,r,MBhat,dQ2,dr):
    """ 1/mb^5 level functions of r18 for MX moments  """

    logy = math.log((1-y)/r**2)

    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return (40*(-1+y)**6-485*r**10*(-1+y)**6-4*r**4*(-1+y)**3*(-70+210*y-210*y**2
        +560*y**3-775*y**4+299*y**5)+5*r**2*(-1+y)**4*(-23+46*y-23*y**2-112*y**3
        +157*y**4-50*y**5+y**6)-2*r**6*(-1+y)*(220+90*logy*(-1+y)**5-1190*y
        +2605*y**2-3505*y**3+3615*y**4-2301*y**5+568*y**6)-4*r**8*(-100+600*y
        -1500*y**2+2510*y**3-2705*y**4+1716*y**5-601*y**6+90*y**7))/(60*r**2*(
        -1+y)**6)
    if (dQ2 == 0 and dr == 0 and i==1):
        return (240*(-1+y)**7-873*r**12*(-1+y)**7-3*r**2*(-1+y)**5*(-131+240*logy*(
        -1+y)**2+502*y-491*y**2+80*y**3+665*y**4-850*y**5+265*y**6)+r**4*(
        -1+y)**4*(-11785-6960*logy*(-1+y)**3+42315*y-52755*y**2+24545*y**3
        -9495*y**4+11301*y**5-4717*y**6+15*y**7)-6*r**6*(-1+y)**2*(-2320+510*logy*(
        -1+y)**5+11090*y-20905*y**2+19205*y**3-10530*y**4+7200*y**5-5075*y**6
        +1359*y**7)+r**10*(-1175+8225*y-24675*y**2+41125*y**3-52255*y**4+47433*y**5
        -25991*y**6+8033*y**7-1080*y**8)-6*r**8*(-1+y)*(-200+270*logy*(-1
        +y)**6+1470*y-4485*y**2+7330*y**3-6625*y**4+4734*y**5-2709*y**6+526*y**7
        +39*y**8)-6*MBhat**2*(-1+y)*(-40*(-1+y)**6+485*r**10*(-1+y)**6+4*r**4*(
        -1+y)**3*(-70+210*y-210*y**2+560*y**3-775*y**4+299*y**5)-5*r**2*(
        -1+y)**4*(-23+46*y-23*y**2-112*y**3+157*y**4-50*y**5+y**6)+2*r**6*(
        -1+y)*(220+90*logy*(-1+y)**5-1190*y+2605*y**2-3505*y**3+3615*y**4
        -2301*y**5+568*y**6)+4*r**8*(-100+600*y-1500*y**2+2510*y**3-2705*y**4
        +1716*y**5-601*y**6+90*y**7))-2*MBhat*(240*(-1+y)**7+2142*r**12*(-1
        +y)**7+6*r**4*(-1+y)**4*(125-540*logy*(-1+y)**3+165*y-975*y**2-515*y**3
        +2050*y**4-1062*y**5+136*y**6)+r**2*(-1+y)**5*(548-600*logy*(-1+y)**2
        -1696*y+1448*y**2-2390*y**3+2710*y**4-647*y**5-96*y**6+3*y**7)-6*r**6*(
        -1+y)**2*(-40-540*logy*(-1+y)**5+740*y-2830*y**2+2220*y**3+3885*y**4
        -6990*y**5+3855*y**6-855*y**7+39*y**8)+15*r**10*(268-1876*y+5628*y**2
        -10122*y**3+11510*y**4-8267*y**5+3661*y**6-911*y**7+97*y**8)+r**8*(
        -2080+14560*y-43680*y**2+72680*y**3-58850*y**4+3558*y**5+29366*y**6
        -20786*y**7+5772*y**8-540*y**9)))/(360*r**2*(-1+y)**7)
    if (dQ2 == 0 and dr == 0 and i==2):
        return (600*(-1+y)**8-970*r**14*(-1+y)**8-20*r**4*(-1+y)**5*(4481+420*logy*(
        -1+y)**3-13863*y+14493*y**2-5251*y**3+105*y**4+579*y**5-862*y**6
        +420*y**7)-r**2*(-1+y)**6*(9390-18780*y+9390*y**2+2904*y**5-3643*y**6
        +974*y**7+65*y**8)-40*r**8*(-1+y)**2*(2405+480*logy*(-1+y)**6-13950*y
        +33435*y**2-42180*y**3+29235*y**4-10878*y**5+3574*y**6-2139*y**7+543*y**8)
        -10*r**6*(-1+y)**3*(1625+13740*logy*(-1+y)**5-21865*y+78080*y**2
        -123880*y**3+96290*y**4-36046*y**5+10385*y**6-6529*y**7+1976*y**8)
        -5*r**10*(-1+y)*(-410+1080*logy*(-1+y)**7+1790*y-1590*y**2-4910*y**3
        +14360*y**4-11208*y**5+6487*y**6-4993*y**7+897*y**8+117*y**9)-4*r**12*(
        -535+4280*y-14980*y**2+29960*y**3-37450*y**4+36854*y**5-27607*y**6
        +12812*y**7-3424*y**8+405*y**9)-15*MBhat**4*(-1+y)**2*(-40*(-1+y)**6
        +485*r**10*(-1+y)**6+4*r**4*(-1+y)**3*(-70+210*y-210*y**2+560*y**3
        -775*y**4+299*y**5)-5*r**2*(-1+y)**4*(-23+46*y-23*y**2-112*y**3+157*y**4
        -50*y**5+y**6)+2*r**6*(-1+y)*(220+90*logy*(-1+y)**5-1190*y+2605*y**2
        -3505*y**3+3615*y**4-2301*y**5+568*y**6)+4*r**8*(-100+600*y-1500*y**2
        +2510*y**3-2705*y**4+1716*y**5-601*y**6+90*y**7))-10*MBhat**3*(-1+y)*(240*(
        -1+y)**7+2142*r**12*(-1+y)**7+6*r**4*(-1+y)**4*(125-540*logy*(-1
        +y)**3+165*y-975*y**2-515*y**3+2050*y**4-1062*y**5+136*y**6)+r**2*(
        -1+y)**5*(548-600*logy*(-1+y)**2-1696*y+1448*y**2-2390*y**3+2710*y**4
        -647*y**5-96*y**6+3*y**7)-6*r**6*(-1+y)**2*(-40-540*logy*(-1+y)**5
        +740*y-2830*y**2+2220*y**3+3885*y**4-6990*y**5+3855*y**6-855*y**7
        +39*y**8)+15*r**10*(268-1876*y+5628*y**2-10122*y**3+11510*y**4-8267*y**5
        +3661*y**6-911*y**7+97*y**8)+r**8*(-2080+14560*y-43680*y**2+72680*y**3
        -58850*y**4+3558*y**5+29366*y**6-20786*y**7+5772*y**8-540*y**9))-20*MBhat*(120*(
        -1+y)**8+255*r**14*(-1+y)**8-6*r**4*(-1+y)**5*(2235+780*logy*(-1
        +y)**3-7485*y+8655*y**2-3665*y**3+525*y**4-395*y**5+173*y**6+15*y**7)
        -r**2*(-1+y)**6*(299+300*logy*(-1+y)**2-298*y-151*y**2+100*y**3
        +544*y**4-482*y**5-18*y**6+66*y**7)+3*r**6*(-1+y)**3*(5125-2820*logy*(
        -1+y)**5-22805*y+38560*y**2-29160*y**3+9300*y**4-3341*y**5+3491*y**6
        -1357*y**7+163*y**8)+r**8*(-1+y)**2*(2240+2160*logy*(-1+y)**6-11280*y
        +21720*y**2-18160*y**3+7380*y**4-13041*y**5+17300*y**6-7920*y**7+1500*y**8
        -39*y**9)+3*r**12*(-222+1776*y-6216*y**2+12432*y**3-17072*y**4+16183*y**5
        -10136*y**6+4060*y**7-944*y**8+97*y**9)+3*r**10*(105-840*y+2940*y**2
        -5880*y**3+8645*y**4-7939*y**5+2708*y**6+890*y**7-778*y**8+149*y**9))
        -5*MBhat**2*(-720*(-1+y)**8+2968*r**14*(-1+y)**8+3*r**4*(-1+y)**5*(10151
        +7200*logy*(-1+y)**3-37653*y+48453*y**2-20231*y**3-485*y**4-545*y**5
        +881*y**6+5*y**7)-r**2*(-1+y)**6*(1865-2400*logy*(-1+y)**2-6130*y
        +5465*y**2-3296*y**3+487*y**4+2158*y**5-872*y**6-38*y**7+y**8)+6*r**6*(
        -1+y)**3*(-7620-90*logy*(-1+y)**5+38190*y-76605*y**2+72905*y**3
        -28880*y**4+490*y**5+2485*y**6-1153*y**7+260*y**8)-2*r**8*(-1+y)**2*(8080
        -1050*logy*(-1+y)**6-49530*y+126975*y**2-163030*y**3+83685*y**4+15822*y**5
        -39831*y**6+21954*y**7-5205*y**8+360*y**9)+3*r**12*(-2505+20040*y
        -70140*y**2+146408*y**3-196024*y**4+172256*y**5-98980*y**6+35624*y**7
        -7207*y**8+612*y**9)-15*r**10*(-535+4280*y-14980*y**2+30792*y**3-37830*y**4
        +25900*y**5-7151*y**6-2416*y**7+2699*y**8-856*y**9+97*y**10)))/(900*r**2*(
        -1+y)**8)
    if (dQ2 == 0 and dr == 0 and i==3):
        return (2800*(-1+y)**9-2425*r**16*(-1+y)**9+r**2*(-1+y)**7*(-180415+25200*logy*(
        -1+y)**2+386030*y-218215*y**2+8400*y**3+2100*y**4+840*y**5-7889*y**6
        +9986*y**7-1765*y**8-472*y**9)+35*r**4*(-1+y)**6*(12033+17040*logy*(
        -1+y)**3-53139*y+78699*y**2-43273*y**3+4260*y**4+852*y**5-255*y**6
        +1277*y**7-793*y**8-13*y**9)-35*r**6*(-1+y)**4*(113055+38100*logy*(
        -1+y)**5-603375*y+1302000*y**2-1429000*y**3+809750*y**4-200050*y**5
        +4268*y**6+7164*y**7-6189*y**8+2425*y**9)-35*r**10*(-1+y)**2*(-25555
        +4500*logy*(-1+y)**7+174385*y-507405*y**2+814175*y**3-774800*y**4
        +433380*y**5-128625*y**6+24827*y**7-12699*y**8+3037*y**9)-35*r**8*(
        -1+y)**3*(69525+75300*logy*(-1+y)**6-341850*y+628725*y**2-461800*y**3
        -30150*y**4+237960*y**5-119446*y**6+26272*y**7-12603*y**8+3682*y**9
        +5*y**10)-35*r**14*(233-2097*y+8388*y**2-19572*y**3+29358*y**4-29358*y**5
        +23352*y**6-14728*y**7+5877*y**8-1373*y**9+144*y**10)-7*r**12*(-1+y)*(5155
        +4500*logy*(-1+y)**8-36740*y+110590*y**2-179180*y**3+160975*y**4
        -65780*y**5+23743*y**6-8580*y**7-8958*y**8+1784*y**9+351*y**10)-70*MBhat**6*(
        -1+y)**3*(-40*(-1+y)**6+485*r**10*(-1+y)**6+4*r**4*(-1+y)**3*(-70
        +210*y-210*y**2+560*y**3-775*y**4+299*y**5)-5*r**2*(-1+y)**4*(-23
        +46*y-23*y**2-112*y**3+157*y**4-50*y**5+y**6)+2*r**6*(-1+y)*(220
        +90*logy*(-1+y)**5-1190*y+2605*y**2-3505*y**3+3615*y**4-2301*y**5
        +568*y**6)+4*r**8*(-100+600*y-1500*y**2+2510*y**3-2705*y**4+1716*y**5
        -601*y**6+90*y**7))-70*MBhat**5*(-1+y)**2*(240*(-1+y)**7+2142*r**12*(
        -1+y)**7+6*r**4*(-1+y)**4*(125-540*logy*(-1+y)**3+165*y-975*y**2
        -515*y**3+2050*y**4-1062*y**5+136*y**6)+r**2*(-1+y)**5*(548-600*logy*(
        -1+y)**2-1696*y+1448*y**2-2390*y**3+2710*y**4-647*y**5-96*y**6+3*y**7)
        -6*r**6*(-1+y)**2*(-40-540*logy*(-1+y)**5+740*y-2830*y**2+2220*y**3
        +3885*y**4-6990*y**5+3855*y**6-855*y**7+39*y**8)+15*r**10*(268-1876*y
        +5628*y**2-10122*y**3+11510*y**4-8267*y**5+3661*y**6-911*y**7+97*y**8)
        +r**8*(-2080+14560*y-43680*y**2+72680*y**3-58850*y**4+3558*y**5+29366*y**6
        -20786*y**7+5772*y**8-540*y**9))-35*MBhat**4*(-1+y)*(-1200*(-1+y)**8
        +5936*r**14*(-1+y)**8+r**4*(-1+y)**5*(49121+36240*logy*(-1+y)**3-183603*y
        +237963*y**2-96841*y**3-12405*y**4+8031*y**5+569*y**6+45*y**7)+r**2*(
        -1+y)**6*(-3337+4080*logy*(-1+y)**2+10754*y-9457*y**2+6352*y**3
        -2969*y**4-1766*y**5+949*y**6+76*y**7-2*y**8)+30*r**6*(-1+y)**3*(-2584
        -138*logy*(-1+y)**5+13058*y-26461*y**2+25321*y**3-9446*y**4-1244*y**5
        +2009*y**6-733*y**7+104*y**8)-2*r**8*(-1+y)**2*(15560-1290*logy*(-1
        +y)**6-94650*y+240495*y**2-304070*y**3+147495*y**4+45846*y**5-87789*y**6
        +45486*y**7-10293*y**8+720*y**9)+3*r**12*(-5301+42408*y-148428*y**2
        +309112*y**3-412418*y**4+360808*y**5-206108*y**6+73576*y**7-14705*y**8
        +1224*y**9)+r**10*(17225-137800*y+482300*y**2-989560*y**3+1228280*y**4
        -876688*y**5+287954*y**6+38456*y**7-71857*y**8+24600*y**9-2910*y**10))
        -2*MBhat*(8400*(-1+y)**9+6630*r**16*(-1+y)**9-210*r**4*(-1+y)**6*(4873
        -1860*logy*(-1+y)**3-12759*y+9969*y**2-1463*y**3-465*y**4+127*y**5
        -329*y**6+127*y**7+76*y**8)+r**2*(-1+y)**7*(-287680+29400*logy*(-1
        +y)**2+604760*y-331780*y**2+9800*y**3+2450*y**4-18319*y**5+13230*y**6
        +7302*y**7-3768*y**8-195*y**9)+70*r**8*(-1+y)**3*(-70425-26100*logy*(
        -1+y)**6+396450*y-912825*y**2+1086600*y**3-684450*y**4+200796*y**5
        -24340*y**6+11568*y**7-4290*y**8+496*y**9)-35*r**6*(-1+y)**4*(110432
        +101760*logy*(-1+y)**5-653920*y+1562240*y**2-1901440*y**3+1205120*y**4
        -347725*y**5+29237*y**6-7288*y**7+3230*y**8+255*y**9+3*y**10)-105*r**10*(
        -1+y)**2*(3120-1800*logy*(-1+y)**7-20040*y+53820*y**2-77100*y**3
        +61350*y**4-24363*y**5+11279*y**6-11044*y**7+4216*y**8-711*y**9+13*y**10)
        +35*r**14*(688-6192*y+24768*y**2-57792*y**3+86688*y**4-95193*y**5
        +76209*y**6-41436*y**7+14724*y**8-3091*y**9+291*y**10)+14*r**12*(-745
        +6705*y-26820*y**2+62580*y**3-93870*y**4+130134*y**5-137658*y**6+69642*y**7
        -8778*y**8-1298*y**9-27*y**10+135*y**11))-35*MBhat**2*(-1200*(-1+y)**9
        +1166*r**16*(-1+y)**9+r**2*(-1+y)**7*(16562+480*logy*(-1+y)**2-32644*y
        +15842*y**2+160*y**3+1993*y**4+430*y**5-2410*y**6+518*y**7+149*y**8)
        +r**4*(-1+y)**6*(197900+29760*logy*(-1+y)**3-623460*y+668100*y**2
        -252460*y**3+12555*y**4+447*y**5-3080*y**6+3936*y**7+141*y**8+y**9)
        +4*r**6*(-1+y)**4*(-8411+66000*logy*(-1+y)**5-23945*y+212890*y**2
        -432890*y**3+376900*y**4-134672*y**5+10405*y**6-1001*y**7+874*y**8
        +30*y**9)+2*r**8*(-1+y)**3*(83325+1500*logy*(-1+y)**6-498450*y+1241625*y**2
        -1648000*y**3+1208985*y**4-424386*y**5+32563*y**6+10380*y**7-5226*y**8
        +984*y**9)+r**10*(-1+y)**2*(20490+3480*logy*(-1+y)**7-146910*y+452910*y**2
        -779210*y**3+803795*y**4-385005*y**5-4444*y**6+73660*y**7-36483*y**8
        +7317*y**9-360*y**10)+2*r**14*(1930-17370*y+69480*y**2-162120*y**3
        +257355*y**4-283563*y**5+214788*y**6-110412*y**7+36783*y**8-7147*y**9
        +612*y**10)-r**12*(5428-48852*y+195408*y**2-455952*y**3+730425*y**4
        -785217*y**5+514839*y**6-183999*y**7+23031*y**8+7721*y**9-3123*y**10
        +291*y**11))-10*MBhat**3*(5600*(-1+y)**9+9192*r**16*(-1+y)**9-14*r**4*(
        -1+y)**6*(44538+16200*logy*(-1+y)**3-149814*y+174114*y**2-73028*y**3
        +6050*y**4-2437*y**5+1563*y**6+171*y**7+3*y**8)+14*r**6*(-1+y)**4*(52298
        -26280*logy*(-1+y)**5-235210*y+404720*y**2-313235*y**3+92105*y**4
        -5554*y**5+7146*y**6-2779*y**7+269*y**8)+r**2*(-1+y)**7*(-11572-15120*logy*(
        -1+y)**2+8024*y+11108*y**2-10402*y**3-12614*y**4+14056*y**5+616*y**6
        -1973*y**7-44*y**8+y**9)+28*r**8*(-1+y)**3*(5270+2880*logy*(-1+y)**6
        -28740*y+63210*y**2-65730*y**3+25940*y**4-1074*y**5-879*y**6+2508*y**7
        -1227*y**8+222*y**9)+14*r**14*(1958-17622*y+70488*y**2-169197*y**3
        +265013*y**4-280602*y**5+202650*y**6-97980*y**7+30060*y**8-5195*y**9
        +371*y**10)-7*r**10*(-1+y)**2*(-6900+48300*y-144900*y**2+235210*y**3
        -165080*y**4+7974*y**5+73408*y**6-70843*y**7+31581*y**8-6715*y**9+485*y**10)
        -14*r**12*(2398-21582*y+86328*y**2-208254*y**3+312088*y**4-287415*y**5
        +151647*y**6-26646*y**7-20061*y**8+15344*y**9-4306*y**10+459*y**11)))/(4200*r**2*(
        -1+y)**9)
    if (dQ2 == 0 and dr == 0 and i==4):
        return (8400*(-1+y)**10-4365*r**18*(-1+y)**10+210*r**6*(-1+y)**5*(-129374
        +46980*logy*(-1+y)**5+599890*y-1082330*y**2+925730*y**3-345415*y**4
        +22103*y**5+7830*y**6+530*y**7+497*y**8+1595*y**9-1080*y**10)+3*r**2*(
        -1+y)**8*(-421025+67200*logy*(-1+y)**2+909250*y-521825*y**2+22400*y**3
        +5600*y**4+2240*y**5+1120*y**6-4432*y**7+5789*y**8+38*y**9-555*y**10)
        +8*r**4*(-1+y)**7*(1838630+949200*logy*(-1+y)**3-6465090*y+7888890*y**2
        -3578830*y**3+237300*y**4+47460*y**5+15820*y**6+10644*y**7+6813*y**8
        -6953*y**9-354*y**10)-840*r**8*(-1+y)**4*(-38986+43080*logy*(-1+y)**6
        +276996*y-821730*y**2+1311040*y**3-1198680*y**4+608712*y**5-144532*y**6
        +6004*y**7+1869*y**8-1148*y**9+505*y**10)-168*r**12*(-1+y)**2*(26850
        +4200*logy*(-1+y)**8-210600*y+720300*y**2-1401400*y**3+1692950*y**4
        -1295560*y**5+608580*y**6-152872*y**7+19631*y**8-8806*y**9+1977*y**10)
        -70*r**10*(-1+y)**3*(-560920+335100*logy*(-1+y)**7+3591340*y-9601170*y**2
        +13656250*y**3-10724125*y**4+4088775*y**5-190075*y**6-310343*y**7+71175*y**8
        -26201*y**9+7079*y**10+15*y**11)-42*r**14*(-1+y)*(-4630+2700*logy*(
        -1+y)**9+38970*y-143730*y**2+302970*y**3-397755*y**4+329715*y**5
        -163110*y**6+63366*y**7-22419*y**8-1249*y**9+516*y**10+156*y**11)-40*r**16*(
        -524+5240*y-23580*y**2+62880*y**3-110040*y**4+132048*y**5-110040*y**6
        +73260*y**7-39765*y**8+13880*y**9-2873*y**10+270*y**11)-210*MBhat**8*(
        -1+y)**4*(-40*(-1+y)**6+485*r**10*(-1+y)**6+4*r**4*(-1+y)**3*(-70
        +210*y-210*y**2+560*y**3-775*y**4+299*y**5)-5*r**2*(-1+y)**4*(-23
        +46*y-23*y**2-112*y**3+157*y**4-50*y**5+y**6)+2*r**6*(-1+y)*(220
        +90*logy*(-1+y)**5-1190*y+2605*y**2-3505*y**3+3615*y**4-2301*y**5
        +568*y**6)+4*r**8*(-100+600*y-1500*y**2+2510*y**3-2705*y**4+1716*y**5
        -601*y**6+90*y**7))-280*MBhat**7*(-1+y)**3*(240*(-1+y)**7+2142*r**12*(
        -1+y)**7+6*r**4*(-1+y)**4*(125-540*logy*(-1+y)**3+165*y-975*y**2
        -515*y**3+2050*y**4-1062*y**5+136*y**6)+r**2*(-1+y)**5*(548-600*logy*(
        -1+y)**2-1696*y+1448*y**2-2390*y**3+2710*y**4-647*y**5-96*y**6+3*y**7)
        -6*r**6*(-1+y)**2*(-40-540*logy*(-1+y)**5+740*y-2830*y**2+2220*y**3
        +3885*y**4-6990*y**5+3855*y**6-855*y**7+39*y**8)+15*r**10*(268-1876*y
        +5628*y**2-10122*y**3+11510*y**4-8267*y**5+3661*y**6-911*y**7+97*y**8)
        +r**8*(-2080+14560*y-43680*y**2+72680*y**3-58850*y**4+3558*y**5+29366*y**6
        -20786*y**7+5772*y**8-540*y**9))-140*MBhat**6*(-1+y)**2*(-1680*(-1+y)**8
        +8904*r**14*(-1+y)**8+r**4*(-1+y)**5*(67789+50880*logy*(-1+y)**3-254247*y
        +330567*y**2-132989*y**3-23355*y**4+17697*y**5-1505*y**6+75*y**7)-3*r**2*(
        -1+y)**6*(1603-1920*logy*(-1+y)**2-5126*y+4483*y**2-3136*y**3+1817*y**4
        +458*y**5-342*y**6-38*y**7+y**8)+6*r**6*(-1+y)**3*(-18220-1290*logy*(
        -1+y)**5+92390*y-188005*y**2+180305*y**3-65580*y**4-12930*y**5+17605*y**6
        -6177*y**7+780*y**8)-6*r**8*(-1+y)**2*(7680-510*logy*(-1+y)**6-46590*y
        +118005*y**2-148370*y**3+70435*y**4+25290*y**5-45249*y**6+23006*y**7
        -5127*y**8+360*y**9)+9*r**12*(-2699+21592*y-75572*y**2+157272*y**3
        -209604*y**4+183120*y**5-104412*y**6+37176*y**7-7401*y**8+612*y**9)
        +r**10*(26425-211400*y+739900*y**2-1517240*y**3+1889110*y**4-1364876*y**5
        +468643*y**6+40672*y**7-103229*y**8+36360*y**9-4365*y**10))-120*MBhat**5*(
        -1+y)*(3920*(-1+y)**9+9192*r**16*(-1+y)**9-14*r**4*(-1+y)**6*(31128
        +11520*logy*(-1+y)**3-104904*y+122184*y**2-51038*y**3+2900*y**4-67*y**5
        +525*y**6+81*y**7+3*y**8)-14*r**6*(-1+y)**4*(-36923+17820*logy*(-1
        +y)**5+166795*y-289040*y**2+225755*y**3-64205*y**4-4469*y**5+3327*y**6
        -1292*y**7+220*y**8)+r**2*(-1+y)**7*(-7386-10920*logy*(-1+y)**2+3852*y
        +8994*y**2-9002*y**3-4998*y**4+7308*y**5+364*y**6-1049*y**7-44*y**8
        +y**9)+14*r**8*(-1+y)**3*(8300+3600*logy*(-1+y)**6-46200*y+104700*y**2
        -113300*y**3+44500*y**4+10893*y**5-19058*y**6+12936*y**7-3954*y**8
        +483*y**9)+14*r**14*(2213-19917*y+79668*y**2-190617*y**3+297143*y**4
        -312732*y**5+224070*y**6-107160*y**7+32355*y**8-5450*y**9+371*y**10)
        -7*r**10*(-1+y)**2*(-7530+52710*y-158130*y**2+257260*y**3-194900*y**4
        +25788*y**5+74974*y**6-74617*y**7+32475*y**8-6715*y**9+485*y**10)-14*r**12*(3064
        -27576*y+110304*y**2-264198*y**3+400600*y**4-387180*y**5+230604*y**6
        -69234*y**7-5049*y**8+12221*y**9-4015*y**10+459*y**11))-6*MBhat**2*(
        -39200*(-1+y)**10+12985*r**18*(-1+y)**10-14*r**4*(-1+y)**7*(182475
        +504000*logy*(-1+y)**3-1051425*y+1807425*y**2-1106475*y**3+126000*y**4
        +29352*y**5+3699*y**6+4711*y**7-7127*y**8-955*y**9)+70*r**8*(-1+y)**4*(561227
        +504540*logy*(-1+y)**6-2862822*y+5643435*y**2-5001880*y**3+1228710*y**4
        +1016880*y**5-658354*y**6+76072*y**7-2409*y**8+1286*y**9+95*y**10)
        +r**2*(-1+y)**8*(2416765-336000*logy*(-1+y)**2-5169530*y+2920765*y**2
        -112000*y**3-28000*y**4+19880*y**5+20370*y**6-55316*y**7+997*y**8+5474*y**9
        +195*y**10)+70*r**10*(-1+y)**3*(-148735+6000*logy*(-1+y)**7+1035145*y
        -3084435*y**2+5098725*y**3-5046225*y**4+2978019*y**5-904764*y**6+75080*y**7
        +5775*y**8-4089*y**9+724*y**10)+14*r**12*(-1+y)**2*(-39635+8700*logy*(
        -1+y)**8+325780*y-1175030*y**2+2431260*y**3-3160875*y**4+2744940*y**5
        -1232035*y**6+66964*y**7+108898*y**8-51988*y**9+9381*y**10-360*y**11)
        +70*r**6*(-1+y)**5*(773335+327300*logy*(-1+y)**5-4193975*y+9206200*y**2
        -10297200*y**3+5966850*y**4-1517442*y**5+46390*y**6+14318*y**7-865*y**8
        +2625*y**9+98*y**10+2*y**11)+14*r**14*(7275-72750*y+327375*y**2-873000*y**3
        +1527750*y**4-1992552*y**5+1901220*y**6-1171240*y**7+428515*y**8-90570*y**9
        +7737*y**10+240*y**11)+10*r**16*(-5605+56050*y-252225*y**2+672600*y**3
        -1177050*y**4+1499652*y**5-1394778*y**6+918480*y**7-420165*y**8+127030*y**9
        -22801*y**10+1836*y**11))-8*MBhat*(8400*(-1+y)**10+3060*r**18*(-1+y)**10
        +r**2*(-1+y)**8*(-845300+130200*logy*(-1+y)**2+1820800*y-1040600*y**2
        +43400*y**3+10850*y**4+4340*y**5-8372*y**6+4771*y**7+8710*y**8-2445*y**9
        -354*y**10)+15*r**4*(-1+y)**7*(352416+253680*logy*(-1+y)**3-1310928*y
        +1691448*y**2-817496*y**3+63420*y**4+12684*y**5+5614*y**6+2043*y**7
        -753*y**8-1179*y**9-13*y**10)-210*r**6*(-1+y)**5*(105450+11400*logy*(
        -1+y)**5-538650*y+1105800*y**2-1143800*y**3+600400*y**4-131480*y**5
        +2292*y**6-183*y**7-246*y**8+300*y**9+141*y**10)+105*r**10*(-1+y)**3*(138080
        -41400*logy*(-1+y)**7-925160*y+2630580*y**2-4094500*y**3+3732250*y**4
        -1949550*y**5+506292*y**6-44991*y**7+8649*y**8-2933*y**9+323*y**10)
        +21*r**12*(-1+y)**2*(24800+10800*logy*(-1+y)**8-187600*y+613400*y**2
        -1126000*y**3+1256300*y**4-853840*y**5+303522*y**6-78243*y**7+57156*y**8
        -18090*y**9+2754*y**10-39*y**11)-70*r**8*(-1+y)**4*(29108+285360*logy*(
        -1+y)**6+110712*y-1132860*y**2+2937280*y**3-3629760*y**4+2307984*y**5
        -671470*y**6+48477*y**7-1152*y**8+2056*y**9+162*y**10+3*y**11)+60*r**16*(
        -250+2500*y-11250*y**2+30000*y**3-52500*y**4+63000*y**5-57344*y**6
        +39500*y**7-18840*y**8+5965*y**9-1130*y**10+97*y**11)+70*r**14*(82
        -820*y+3690*y**2-9840*y**3+17220*y**4-20664*y**5+27990*y**6-33030*y**7
        +19065*y**8-4150*y**9+562*y**10-132*y**11+27*y**12))-8*MBhat**3*(58800*(
        -1+y)**10+22980*r**18*(-1+y)**10+210*r**8*(-1+y)**4*(-160313-57060*logy*(
        -1+y)**6+904818*y-2090865*y**2+2502520*y**3-1589620*y**4+463830*y**5
        -33026*y**6+2364*y**7-750*y**8+62*y**9)-3*r**2*(-1+y)**8*(642800-63000*logy*(
        -1+y)**2-1348600*y+737300*y**2-21000*y**3+3080*y**4+23849*y**5-22428*y**6
        -10421*y**7+4630*y**8+590*y**9)-5*r**4*(-1+y)**7*(1751466-397320*logy*(
        -1+y)**3-4857078*y+4261098*y**2-1023046*y**3-107604*y**4+15582*y**5
        -26642*y**6+21375*y**7+10467*y**8+243*y**9+3*y**10)-105*r**6*(-1+y)**5*(234824
        +246720*logy*(-1+y)**5-1420840*y+3458480*y**2-4280880*y**3+2758700*y**4
        -806605*y**5+52813*y**6+1436*y**7+2198*y**8+195*y**9+15*y**10)+35*r**10*(
        -1+y)**3*(-69220+27000*logy*(-1+y)**7+457540*y-1278120*y**2+1941200*y**3
        -1686500*y**4+696699*y**5-96659*y**6+12605*y**7+16605*y**8-6884*y**9
        +1034*y**10)-21*r**12*(-1+y)**2*(17850-142800*y+499800*y**2-999600*y**3
        +1293910*y**4-834962*y**5+156376*y**6+99409*y**7-110438*y**8+45120*y**9
        -8350*y**10+485*y**11)+10*r**16*(-8627+86270*y-388215*y**2+1035240*y**3
        -1884330*y**4+2409960*y**5-2179842*y**6+1392540*y**7-614025*y**8+177260*y**9
        -29969*y**10+2226*y**11)-105*r**14*(-1304+13040*y-58680*y**2+156480*y**3
        -291024*y**4+370983*y**5-308970*y**6+163899*y**7-50490*y**8+4125*y**9
        +2838*y**10-999*y**11+102*y**12))-MBhat**4*(-588000*(-1+y)**10+354375*r**18*(
        -1+y)**10+420*r**4*(-1+y)**7*(239228+39840*logy*(-1+y)**3-757524*y
        +817284*y**2-312460*y**3+14991*y**4+555*y**5+366*y**6+2182*y**7+135*y**8
        +3*y**9)+630*r**6*(-1+y)**5*(-39088+204560*logy*(-1+y)**5-9120*y
        +529640*y**2-1211200*y**3+1105165*y**4-409593*y**5+32558*y**6+1882*y**7
        +221*y**8+95*y**9)+280*r**8*(-1+y)**4*(272975+900*logy*(-1+y)**6-1636950*y
        +4089675*y**2-5458900*y**3+4067235*y**4-1506678*y**5+185803*y**6-2640*y**7
        -5172*y**8+952*y**9)+r**2*(-1+y)**8*(7808025+336000*logy*(-1+y)**2
        -15280050*y+7304025*y**2+201040*y**3+947590*y**4-141932*y**5-739746*y**6
        +134448*y**7+59865*y**8+750*y**9-15*y**10)+420*r**10*(-1+y)**3*(17600
        +2550*logy*(-1+y)**7-125750*y+386175*y**2-673215*y**3+755325*y**4
        -455721*y**5+130568*y**6-38826*y**7+14722*y**8-4678*y**9+700*y**10)
        +120*r**16*(-9787+97870*y-440415*y**2+1198660*y**3-2161180*y**4+2694552*y**5
        -2363424*y**6+1451640*y**7-607560*y**8+163100*y**9-24736*y**10+1532*y**11)
        -84*r**12*(-1+y)**2*(13100-104800*y+366800*y**2-753840*y**3+495675*y**4
        +198912*y**5-635721*y**6+704736*y**7-437517*y**8+156900*y**9-29385*y**10
        +2040*y**11)-70*r**14*(-21568+215680*y-970560*y**2+2682072*y**3-4698225*y**4
        +5299266*y**5-3800013*y**6+1480296*y**7+23445*y**8-360290*y**9+191525*y**10
        -46080*y**11+4452*y**12)))/(12600*r**2*(-1+y)**10)

    return 0. 

"Here we define the 1/mb^4 and 1/mb^5 corrections to the centralized MX-moments in the historical basis"

def moment_1_KIN_MS_HO(elcut, par, hqe, flagmb4 = 1, flagmb5 = 1):
    """ 1/mb^4 and 1/mb^5 corrections to the central moment of the MX spectrum. Moment n. 1 """
    y=2*elcut/par.mbkin
    r=par.mcMS/par.mbkin
    elcuthat = elcut/par.mbkin
    MBhat = par.MB/par.mbkin

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    m1=hqe.m1/par.mbkin**4
    m2=hqe.m2/par.mbkin**4
    m3=hqe.m3/par.mbkin**4
    m4=hqe.m4/par.mbkin**4
    m5=hqe.m5/par.mbkin**4
    m6=hqe.m6/par.mbkin**4
    m7=hqe.m7/par.mbkin**4
    m8=hqe.m8/par.mbkin**4
    m9=hqe.m9/par.mbkin**4
    r1=hqe.r1/par.mbkin**5
    r2=hqe.r2/par.mbkin**5
    r3=hqe.r3/par.mbkin**5
    r4=hqe.r4/par.mbkin**5
    r5=hqe.r5/par.mbkin**5
    r6=hqe.r6/par.mbkin**5
    r7=hqe.r7/par.mbkin**5
    r8=hqe.r8/par.mbkin**5
    r9=hqe.r9/par.mbkin**5
    r10=hqe.r10/par.mbkin**5
    r11=hqe.r11/par.mbkin**5
    r12=hqe.r12/par.mbkin**5
    r13=hqe.r13/par.mbkin**5
    r14=hqe.r14/par.mbkin**5
    r15=hqe.r15/par.mbkin**5
    r16=hqe.r16/par.mbkin**5
    r17=hqe.r17/par.mbkin**5
    r18=hqe.r18/par.mbkin**5

    res = 0
    
    if flagmb4 == 1:
        res += (-((X(1,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm1(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m1
        res += (-((X(1,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm2(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m2
        res += (-((X(1,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm3(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m3
        res += (-((X(1,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm4(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m4
        res += (-((X(1,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm5(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m5
        res += (-((X(1,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm6(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m6
        res += (-((X(1,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm7(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m7
        res += (-((X(1,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm8(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m8
        res += (-((X(1,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xm9(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m9
        res += ((X(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        -(Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi**2
        res += ((X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3-(XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG**2
        res += ((2*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*mupi

    if flagmb5 == 1:
        res += (-((X(1,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr1(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r1
        res += (-((X(1,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr2(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r2
        res += (-((X(1,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr3(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r3
        res += (-((X(1,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr4(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r4
        res += (-((X(1,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr5(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r5
        res += (-((X(1,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr6(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r6
        res += (-((X(1,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr7(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r7
        res += (-((X(1,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr8(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r8
        res += (-((X(1,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr9(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r9
        res += (-((X(1,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr10(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r10
        res += (-((X(1,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr11(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r11
        res += (-((X(1,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr12(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r12
        res += (-((X(1,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr13(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r13
        res += (-((X(1,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr14(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r14
        res += (-((X(1,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr15(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r15
        res += (-((X(1,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr16(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r16
        res += (-((X(1,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr17(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r17
        res += (-((X(1,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)
        +Xr18(1,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r18
        res += ((2*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoD
        res += ((2*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoD
        res += ((2*X(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoLS
        res += ((2*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoLS

    return res*par.mbkin**2

def moment_2_KIN_MS_HO(elcut, par, hqe, flagmb4 = 1, flagmb5 = 1):
    """ 1/mb^4 and 1/mb^5 corrections to the central moment of the MX spectrum. Moment n. 2 """
    y=2*elcut/par.mbkin
    r=par.mcMS/par.mbkin
    elcuthat = elcut/par.mbkin
    MBhat = par.MB/par.mbkin

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    m1=hqe.m1/par.mbkin**4
    m2=hqe.m2/par.mbkin**4
    m3=hqe.m3/par.mbkin**4
    m4=hqe.m4/par.mbkin**4
    m5=hqe.m5/par.mbkin**4
    m6=hqe.m6/par.mbkin**4
    m7=hqe.m7/par.mbkin**4
    m8=hqe.m8/par.mbkin**4
    m9=hqe.m9/par.mbkin**4
    r1=hqe.r1/par.mbkin**5
    r2=hqe.r2/par.mbkin**5
    r3=hqe.r3/par.mbkin**5
    r4=hqe.r4/par.mbkin**5
    r5=hqe.r5/par.mbkin**5
    r6=hqe.r6/par.mbkin**5
    r7=hqe.r7/par.mbkin**5
    r8=hqe.r8/par.mbkin**5
    r9=hqe.r9/par.mbkin**5
    r10=hqe.r10/par.mbkin**5
    r11=hqe.r11/par.mbkin**5
    r12=hqe.r12/par.mbkin**5
    r13=hqe.r13/par.mbkin**5
    r14=hqe.r14/par.mbkin**5
    r15=hqe.r15/par.mbkin**5
    r16=hqe.r16/par.mbkin**5
    r17=hqe.r17/par.mbkin**5
    r18=hqe.r18/par.mbkin**5

    res = 0
    
    if flagmb4 == 1:
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm1(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m1
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm2(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m2
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm3(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m3
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm4(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m4
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm5(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m5
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm6(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m6
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm7(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m7
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm8(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m8
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xm9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm9(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m9
        res += ((-3*X(1,y,r,MBhat,0,0)**2*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(X(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -Xpi(1,y,r,MBhat,0,0)**2/X(0,y,r,MBhat,0,0)**2-(Xpi(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi**2
        res += ((-3*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -XG(1,y,r,MBhat,0,0)**2/X(0,y,r,MBhat,0,0)**2-(XG(0,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG**2
        res += ((-6*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(4*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(2*XG(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*mupi

    if flagmb5 == 1:
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr1(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r1
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr2(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r2
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr3(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r3
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr4(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r4
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr5(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r5
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr6(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r6
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr7(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r7
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr8(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r8
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(2*X(1,y,r,MBhat,0,0)*Xr9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr9(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r9
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr10(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr10(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r10
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr11(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr11(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r11
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr12(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr12(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r12
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr13(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr13(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r13
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr14(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr14(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r14
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr15(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr15(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r15
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr16(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr16(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r16
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr17(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr17(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r17
        res += ((2*X(1,y,r,MBhat,0,0)**2*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(2,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(2*X(1,y,r,MBhat,0,0)*Xr18(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr18(2,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r18
        res += ((-6*X(1,y,r,MBhat,0,0)**2*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(4*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(2*XD(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoD
        res += ((-6*X(1,y,r,MBhat,0,0)**2*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(4*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(2*XD(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoD
        res += ((-6*X(1,y,r,MBhat,0,0)**2*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XLS(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(4*X(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(2*XLS(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XLS(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoLS
        res += ((-6*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(4*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(4*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(2*XG(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoLS

    return res*par.mbkin**4

def moment_3_KIN_MS_HO(elcut, par, hqe, flagmb4 = 1, flagmb5 = 1):
    """ 1/mb^4 and 1/mb^5 corrections to the central moment of the MX spectrum. Moment n. 3 """
    y=2*elcut/par.mbkin
    r=par.mcMS/par.mbkin
    elcuthat = elcut/par.mbkin
    MBhat = par.MB/par.mbkin

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    m1=hqe.m1/par.mbkin**4
    m2=hqe.m2/par.mbkin**4
    m3=hqe.m3/par.mbkin**4
    m4=hqe.m4/par.mbkin**4
    m5=hqe.m5/par.mbkin**4
    m6=hqe.m6/par.mbkin**4
    m7=hqe.m7/par.mbkin**4
    m8=hqe.m8/par.mbkin**4
    m9=hqe.m9/par.mbkin**4
    r1=hqe.r1/par.mbkin**5
    r2=hqe.r2/par.mbkin**5
    r3=hqe.r3/par.mbkin**5
    r4=hqe.r4/par.mbkin**5
    r5=hqe.r5/par.mbkin**5
    r6=hqe.r6/par.mbkin**5
    r7=hqe.r7/par.mbkin**5
    r8=hqe.r8/par.mbkin**5
    r9=hqe.r9/par.mbkin**5
    r10=hqe.r10/par.mbkin**5
    r11=hqe.r11/par.mbkin**5
    r12=hqe.r12/par.mbkin**5
    r13=hqe.r13/par.mbkin**5
    r14=hqe.r14/par.mbkin**5
    r15=hqe.r15/par.mbkin**5
    r16=hqe.r16/par.mbkin**5
    r17=hqe.r17/par.mbkin**5
    r18=hqe.r18/par.mbkin**5

    res = 0
    
    if flagmb4 == 1:
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm1(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm1(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m1
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm2(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm2(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m2
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm3(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm3(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m3
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm4(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm4(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m4
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm5(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm5(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m5
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm6(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm6(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m6
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm7(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm7(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m7
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm8(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm8(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m8
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xm9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xm9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xm9(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm9(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m9
        res += ((12*X(1,y,r,MBhat,0,0)**3*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**5
        -(9*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(X(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*Xpi(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(Xpi(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi**2
        res += ((12*X(1,y,r,MBhat,0,0)**3*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**5
        -(9*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XG(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG**2
        res += ((24*X(1,y,r,MBhat,0,0)**3*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XG(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XG(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*mupi

    if flagmb5 == 1:
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr1(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr1(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r1
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr2(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr2(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r2
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr3(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr3(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r3
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr4(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr4(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r4
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr5(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr5(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r5
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr6(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr6(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r6
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr7(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr7(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r7
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr8(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr8(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r8
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(6*X(1,y,r,MBhat,0,0)**2*Xr9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr9(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr9(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r9
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr10(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr10(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr10(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr10(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r10
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr11(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr11(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr11(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr11(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r11
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr12(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr12(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr12(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr12(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r12
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr13(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr13(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr13(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr13(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r13
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr14(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr14(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr14(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr14(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r14
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr15(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr15(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr15(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr15(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r15
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr16(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr16(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr16(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr16(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r16
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr17(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr17(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr17(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr17(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r17
        res += ((-6*X(1,y,r,MBhat,0,0)**3*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(3,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr18(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*X(2,y,r,MBhat,0,0)*Xr18(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(3*X(1,y,r,MBhat,0,0)*Xr18(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr18(3,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r18
        res += ((24*X(1,y,r,MBhat,0,0)**3*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(3,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XD(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XD(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XD(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoD
        res += ((24*X(1,y,r,MBhat,0,0)**3*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(3,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XD(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(18*X(1,y,r,MBhat,0,0)**2*XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XD(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XD(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoD
        res += ((24*X(1,y,r,MBhat,0,0)**3*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(3,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XLS(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XLS(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XLS(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XLS(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoLS
        res += ((24*X(1,y,r,MBhat,0,0)**3*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(6*X(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(3,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XG(2,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(3*XG(1,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*XLS(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoLS

    return res*par.mbkin**6

def moment_4_KIN_MS_HO(elcut, par, hqe, flagmb4 = 1, flagmb5 = 1):
    """ 1/mb^4 and 1/mb^5 corrections to the central moment of the MX spectrum. Moment n. 4 """
    y=2*elcut/par.mbkin
    r=par.mcMS/par.mbkin
    elcuthat = elcut/par.mbkin
    MBhat = par.MB/par.mbkin

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    m1=hqe.m1/par.mbkin**4
    m2=hqe.m2/par.mbkin**4
    m3=hqe.m3/par.mbkin**4
    m4=hqe.m4/par.mbkin**4
    m5=hqe.m5/par.mbkin**4
    m6=hqe.m6/par.mbkin**4
    m7=hqe.m7/par.mbkin**4
    m8=hqe.m8/par.mbkin**4
    m9=hqe.m9/par.mbkin**4
    r1=hqe.r1/par.mbkin**5
    r2=hqe.r2/par.mbkin**5
    r3=hqe.r3/par.mbkin**5
    r4=hqe.r4/par.mbkin**5
    r5=hqe.r5/par.mbkin**5
    r6=hqe.r6/par.mbkin**5
    r7=hqe.r7/par.mbkin**5
    r8=hqe.r8/par.mbkin**5
    r9=hqe.r9/par.mbkin**5
    r10=hqe.r10/par.mbkin**5
    r11=hqe.r11/par.mbkin**5
    r12=hqe.r12/par.mbkin**5
    r13=hqe.r13/par.mbkin**5
    r14=hqe.r14/par.mbkin**5
    r15=hqe.r15/par.mbkin**5
    r16=hqe.r16/par.mbkin**5
    r17=hqe.r17/par.mbkin**5
    r18=hqe.r18/par.mbkin**5

    res = 0
    
    if flagmb4 == 1:
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm1(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm1(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm1(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m1
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm2(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm2(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm2(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m2
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm3(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm3(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm3(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m3
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm4(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm4(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm4(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m4
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm5(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm5(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm5(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m5
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm6(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm6(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm6(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m6
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm7(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm7(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm7(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m7
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm8(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm8(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm8(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m8
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xm9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xm9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xm9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xm9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xm9(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xm9(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xm9(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*m9
        res += ((-30*X(1,y,r,MBhat,0,0)**4*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**6
        +(36*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**5
        -(12*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(X(4,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*Xpi(1,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*Xpi(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*Xpi(1,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(Xpi(0,y,r,MBhat,0,0)*Xpi(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi**2
        res += ((-30*X(1,y,r,MBhat,0,0)**4*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**6
        +(36*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**5
        -(12*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(X(4,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(1,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**4
        +(6*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)**2)/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XG(1,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*XG(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG**2
        res += ((-60*X(1,y,r,MBhat,0,0)**4*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**6
        +(72*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(24*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(4,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(4,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(48*X(1,y,r,MBhat,0,0)**3*XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(36*X(1,y,r,MBhat,0,0)**2*XG(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XG(3,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XG(1,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*Xpi(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*mupi

    if flagmb5 == 1:
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr1(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr1(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr1(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr1(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr1(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r1
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr2(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr2(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr2(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr2(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr2(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r2
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr3(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr3(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr3(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr3(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr3(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r3
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr4(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr4(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr4(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr4(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr4(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r4
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr5(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr5(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr5(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr5(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr5(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r5
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr6(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr6(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr6(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr6(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr6(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r6
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr7(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr7(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr7(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr7(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr7(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r7
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr8(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr8(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr8(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr8(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr8(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r8
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr9(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2-(12*X(1,y,r,MBhat,0,0)**3*Xr9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr9(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr9(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr9(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr9(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r9
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr10(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr10(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr10(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr10(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr10(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr10(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr10(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r10
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr11(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr11(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr11(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr11(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr11(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr11(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr11(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r11
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr12(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr12(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr12(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr12(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr12(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr12(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr12(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r12
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr13(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr13(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr13(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr13(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr13(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr13(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr13(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r13
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr14(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr14(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr14(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr14(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr14(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr14(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr14(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r14
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr15(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr15(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr15(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr15(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr15(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr15(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr15(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r15
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr16(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr16(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr16(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr16(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr16(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr16(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr16(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r16
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr17(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr17(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr17(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr17(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr17(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr17(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr17(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r17
        res += ((12*X(1,y,r,MBhat,0,0)**4*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(18*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(X(4,y,r,MBhat,0,0)*Xr18(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(12*X(1,y,r,MBhat,0,0)**3*Xr18(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*Xr18(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(3,y,r,MBhat,0,0)*Xr18(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(6*X(1,y,r,MBhat,0,0)**2*Xr18(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*X(1,y,r,MBhat,0,0)*Xr18(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +Xr18(4,y,r,MBhat,0,0)/X(0,y,r,MBhat,0,0))*r18
        res += ((-60*X(1,y,r,MBhat,0,0)**4*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**6
        +(72*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(24*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(4,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XD(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*XD(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(4,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(48*X(1,y,r,MBhat,0,0)**3*XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(36*X(1,y,r,MBhat,0,0)**2*XD(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(2,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XD(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XD(3,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XD(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XD(1,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*Xpi(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoD
        res += ((-60*X(1,y,r,MBhat,0,0)**4*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**6
        +(72*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(24*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(4,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XD(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*XD(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XD(4,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2+(48*X(1,y,r,MBhat,0,0)**3*XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(36*X(1,y,r,MBhat,0,0)**2*XD(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(2,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XD(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XD(3,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XD(0,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*XD(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*XD(0,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XD(1,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XD(0,y,r,MBhat,0,0)*XG(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoD
        res += ((-60*X(1,y,r,MBhat,0,0)**4*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**6
        +(72*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(24*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(4,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XLS(2,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*XLS(3,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XLS(4,y,r,MBhat,0,0)*Xpi(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(48*X(1,y,r,MBhat,0,0)**3*XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(36*X(1,y,r,MBhat,0,0)**2*XLS(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(2,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XLS(3,y,r,MBhat,0,0)*Xpi(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XLS(0,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0)*Xpi(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XLS(1,y,r,MBhat,0,0)*Xpi(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XLS(0,y,r,MBhat,0,0)*Xpi(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*mupi*rhoLS
        res += ((-60*X(1,y,r,MBhat,0,0)**4*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**6
        +(72*X(1,y,r,MBhat,0,0)**2*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(24*X(1,y,r,MBhat,0,0)*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(2*X(4,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(48*X(1,y,r,MBhat,0,0)**3*XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(18*X(1,y,r,MBhat,0,0)**2*XG(2,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(1,y,r,MBhat,0,0)*XG(3,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(XG(4,y,r,MBhat,0,0)*XLS(0,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        +(48*X(1,y,r,MBhat,0,0)**3*XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**5
        -(36*X(1,y,r,MBhat,0,0)*X(2,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(8*X(3,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(36*X(1,y,r,MBhat,0,0)**2*XG(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(2,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(12*X(1,y,r,MBhat,0,0)*XG(2,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XG(3,y,r,MBhat,0,0)*XLS(1,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(18*X(1,y,r,MBhat,0,0)**2*XG(0,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**4
        +(12*X(1,y,r,MBhat,0,0)*XG(1,y,r,MBhat,0,0)*XLS(2,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        +(8*X(1,y,r,MBhat,0,0)*XG(0,y,r,MBhat,0,0)*XLS(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**3
        -(4*XG(1,y,r,MBhat,0,0)*XLS(3,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2
        -(XG(0,y,r,MBhat,0,0)*XLS(4,y,r,MBhat,0,0))/X(0,y,r,MBhat,0,0)**2)*muG*rhoLS

    return res*par.mbkin**8