from numba import jit, int64, float64
from kolya import parameters
from kolya import NLOpartonic as nlo
from kolya import NNLOpartonic as nnlo
from kolya import NLOpw as nlopw
from kolya import schemechange_KINMS as kin
import math

@jit(float64(int64, int64, float64, float64, int64, int64), cache=True, nopython=True)
def X(i,j,elcuthat,r,dEl,dr):
    """ Tree level functions (partonic) for mix moments and their derivatives """
    y  = 2*elcuthat
    logy = math.log((1-y)/r**2)
    # tree level function
    if (dEl == 0 and dr == 0 and i==0 and j == 0):
        return     (1-8*r**2-6*r**4+12*logy*r**4+4*r**6-r**8-(2*r**6)/(-1+y)**2-(6*r**4*(1
            +r**2))/(-1+y)-2*r**4*(-3+r**2)*y+2*(-1+r**2)*y**3+y**4)
    if (dEl == 0 and dr == 0 and i==0 and j == 1):
        return     (3*logy*r**4*(5+r**2)+(13-135*r**2-280*r**4+160*r**6-35*r**8+7*r**10)/20
            -r**8/(2*(-1+y)**3)-(r**6*(3+r**2))/(-1+y)**2-(r**4*(6+11*r**2))/(
            -1+y)+(r**4*(18-4*r**2+r**4)*y)/2-(r**4*(-3+r**2)*y**2)/2-(-1+r**2)**2*y**3
            -((-1+r**2)*y**4)/4+y**5/10)
    if (dEl == 0 and dr == 0 and i==0 and j == 2):
        return     (2*logy*r**4*(9+5*r**2)+(13-171*r**2-720*r**4+300*r**6-45*r**8+24*r**10
            -4*r**12)/30-(3*r**10)/(20*(-1+y)**4)-(r**8*(5+r**2))/(4*(-1+y)**3)
            +(-16*r**6-13*r**8)/(4*(-1+y)**2)-(2*r**4*(3+9*r**2+r**4))/(-1+y)
            -(3*r**4*(-80-5*r**4+r**6)*y)/20+(r**4*(12-4*r**2+r**4)*y**2)/4+((
            -3+9*r**2-6*r**4+2*r**6)*y**3)/6-((-1+r**2)*y**5)/20+y**6/60)
    if (dEl == 0 and dr == 0 and i==0 and j == 3):
        return     ((3*logy*r**4*(14+15*r**2+r**4))/2+(166-2716*r**2-19880*r**4+3290*r**6
            +1015*r**8+595*r**10-210*r**12+30*r**14)/560-r**12/(20*(-1+y)**5)-(3*r**10*(7
            +r**2))/(40*(-1+y)**4)+(-90*r**8-47*r**10)/(40*(-1+y)**3)+(-80*r**6
            -117*r**8-9*r**10)/(16*(-1+y)**2)-(3*(16*r**4+72*r**6+23*r**8))/(8*(
            -1+y))+(r**4*(300+110*r**2+15*r**4-6*r**6+r**8)*y)/20-(3*r**4*(-120
            +20*r**2-15*r**4+3*r**6)*y**2)/80-((2-8*r**2+2*r**4-4*r**6+r**8)*y**3)/8
            +((-1+3*r**2)*y**4)/16-((-1+r**2)*y**6)/80+y**7/280)
    if (dEl == 0 and dr == 0 and i==0 and j == 4):
        return     ((3*logy*r**4*(32+56*r**2+11*r**4))/4+(233-4672*r**2-53928*r**4-10640*r**6
            +10220*r**8+1512*r**10-686*r**12+200*r**14-25*r**16)/1120-r**14/(56*(
            -1+y)**6)-(r**12*(9+r**2))/(40*(-1+y)**5)+(-96*r**10-37*r**12)/(80*(
            -1+y)**4)+(-35*r**8-34*r**10-2*r**12)/(10*(-1+y)**3)+(-240*r**6-550*r**8
            -123*r**10)/(40*(-1+y)**2)+(-48*r**4-304*r**6-189*r**8-9*r**10)/(8*(
            -1+y))-(r**4*(-1008-896*r**2-91*r**4+21*r**6-7*r**8+r**10)*y)/56
            +(r**4*(120+20*r**2+15*r**4-6*r**6+r**8)*y**2)/20+((-5+25*r**2+35*r**4
            +25*r**6-10*r**8+2*r**10)*y**3)/40+((-1+4*r**2+3*r**4)*y**4)/16+((
            -1+3*r**2)*y**5)/40-((-1+r**2)*y**7)/280+y**8/1120)
    if (dEl == 0 and dr == 0 and i==1 and j == 0):
        return     (18*logy*r**4*(1+r**2)-(3*(-1+15*r**2+100*r**4-20*r**6-5*r**8+r**10))/10
            -r**8/(-1+y)**3-(4*r**6*(1+r**2))/(-1+y)**2-(2*r**4*(3+11*r**2+3*r**4))/(
            -1+y)-r**4*(-12-4*r**2+r**4)*y-r**4*(-3+r**2)*y**2+((-1+r**2)*y**4)/2
            +y**5/5)
    if (dEl == 0 and dr == 0 and i==1 and j == 1):
        return     (logy*r**4*(21+32*r**2+3*r**4)+(13-240*r**2-2445*r**4-120*r**6+405*r**8
            -30*r**10+5*r**12)/60-(3*r**10)/(10*(-1+y)**4)-(r**8*(2+r**2))/(-1
            +y)**3+(-10*r**6-17*r**8-2*r**10)/(2*(-1+y)**2)-(r**4*(6+31*r**2+15*r**4))/(
            -1+y)+(r**4*(75+55*r**2-5*r**4+r**6)*y)/5+(9*r**4*y**2)/2-(r**4*(
            -3+r**2)*y**3)/3-((-1+r**2)**2*y**4)/4+y**6/30)
    if (dEl == 0 and dr == 0 and i==1 and j == 2):
        return     (logy*r**4*(24+53*r**2+13*r**4)+(134-2996*r**2-44268*r**4-18690*r**6
            +12705*r**8-147*r**10+154*r**12-22*r**14)/840-r**12/(10*(-1+y)**5)
            -(3*r**10*(3+r**2))/(10*(-1+y)**4)+(-65*r**8-67*r**10-5*r**12)/(20*(
            -1+y)**3)+(-48*r**6-123*r**8-35*r**10)/(8*(-1+y)**2)+(-24*r**4-168*r**6
            -133*r**8-8*r**10)/(4*(-1+y))-(r**4*(-360-460*r**2-15*r**4-6*r**6
            +r**8)*y)/20+(r**4*(240+100*r**2-5*r**4+r**6)*y**2)/40+(r**4*(24-4*r**2
            +r**4)*y**3)/12+((-1+3*r**2)*y**4)/8-((-1+r**2)**2*y**5)/20-((-1+r**2)*y**6)/120
            +y**7/140)
    if (dEl == 0 and dr == 0 and i==1 and j == 3):
        return     ((3*logy*r**4*(18+55*r**2+25*r**4+r**6))/2+(67-1790*r**2-36764*r**4
            -33810*r**6+12495*r**8+1932*r**10+119*r**12-40*r**14+5*r**16)/560-r**14/(28*(
            -1+y)**6)+(-4*r**12-r**14)/(10*(-1+y)**5)+(-75*r**10-55*r**12-3*r**14)/(40*(
            -1+y)**4)+(-190*r**8-315*r**10-63*r**12)/(40*(-1+y)**3)+(-560*r**6
            -2015*r**8-1032*r**10-45*r**12)/(80*(-1+y)**2)+(-48*r**4-440*r**6-525*r**8
            -87*r**10)/(8*(-1+y))+(r**4*(1470+2905*r**2+560*r**4+21*r**6-7*r**8
            +r**10)*y)/70-(r**4*(-600-580*r**2-15*r**4-6*r**6+r**8)*y**2)/80-(r**4*(
            -120-20*r**2-5*r**4+r**6)*y**3)/40+((-1+4*r**2+9*r**4)*y**4)/16+((
            -1+3*r**2)*y**5)/20-((-1+r**2)**2*y**6)/80-((-1+r**2)*y**7)/280+y**8/560)
    if (dEl == 0 and dr == 0 and i==2 and j == 0):
        return     (8*logy*r**4*(3+8*r**2+3*r**4)-(2*(-1+24*r**2+420*r**4+300*r**6-135*r**8
            -6*r**10+r**12))/15-(3*r**10)/(5*(-1+y)**4)-(3*r**8*(1+r**2))/(-1+y)**3
            -(r**6*(6+17*r**2+6*r**4))/(-1+y)**2-(2*r**4*(1+r**2)*(3+20*r**2+3*r**4))/(
            -1+y)-(3*r**4*(-30-50*r**2-5*r**4+r**6)*y)/5-r**4*(-6-4*r**2+r**4)*y**2
            -(2*r**4*(-3+r**2)*y**3)/3+((-1+r**2)*y**5)/5+y**6/15)
    if (dEl == 0 and dr == 0 and i==2 and j == 1):
        return     (logy*r**4*(27+95*r**2+55*r**4+3*r**6)+(43-1225*r**2-28833*r**4-34965*r**6
            +9030*r**8+2898*r**10-91*r**12+13*r**14)/420-r**12/(5*(-1+y)**5)-(3*(5*r**10
            +3*r**12))/(10*(-1+y)**4)+(-45*r**8-77*r**10-15*r**12)/(10*(-1+y)**3)
            +(-28*r**6-109*r**8-65*r**10-4*r**12)/(4*(-1+y)**2)+(-12*r**4-118*r**6
            -165*r**8-38*r**10)/(2*(-1+y))+(r**4*(210+500*r**2+135*r**4-6*r**6
            +r**8)*y)/10+(r**4*(150+190*r**2-5*r**4+r**6)*y**2)/20-(r**4*(-18-4*r**2
            +r**4)*y**3)/6-(r**4*(-3+r**2)*y**4)/4-((-1+r**2)**2*y**5)/10+((-1+r**2)*y**6)/60
            +y**7/70)
    if (dEl == 0 and dr == 0 and i==2 and j == 2):
        return     (logy*r**4*(30+136*r**2+115*r**4+16*r**6)+(67-2240*r**2-69020*r**4
            -127008*r**6+6090*r**8+18648*r**10+56*r**12+56*r**14-7*r**16)/840-r**14/(14*(
            -1+y)**6)-(r**12*(7+3*r**2))/(10*(-1+y)**5)-(r**10*(57+67*r**2+9*r**4))/(20*(
            -1+y)**4)+(-125*r**8-313*r**10-125*r**12-5*r**14)/(20*(-1+y)**3)+(
            -80*r**6-410*r**8-368*r**10-55*r**12)/(10*(-1+y)**2)-(r**4*(6+74*r**2
            +140*r**4+53*r**6+2*r**8))/(-1+y)-(3*r**4*(-1120-3640*r**2-1785*r**4
            -49*r**6-7*r**8+r**10)*y)/140+3*r**4*(3+6*r**2+r**4)*y**2+(r**4*(120
            +100*r**2-5*r**4+r**6)*y**3)/30+(3*r**4*y**4)/2-((1-3*r**2-3*r**4+r**6)*y**5)/20
            -((-1+r**2)**2*y**6)/30+y**8/280)
    if (dEl == 0 and dr == 0 and i==3 and j == 0):
        return     (30*logy*r**4*(1+5*r**2+5*r**4+r**6)+(1-35*r**2-1183*r**4-2555*r**6
            -210*r**8+434*r**10+7*r**12-r**14)/14-(2*r**12)/(5*(-1+y)**5)-(12*r**10*(1
            +r**2))/(5*(-1+y)**4)+(-30*r**8-77*r**10-30*r**12)/(5*(-1+y)**3)+(
            -16*r**6-87*r**8-87*r**10-16*r**12)/(2*(-1+y)**2)-(3*r**4*(2+26*r**2
            +55*r**4+26*r**6+2*r**8))/(-1+y)-(2*r**4*(-60-220*r**2-135*r**4-6*r**6
            +r**8)*y)/5-(3*r**4*(-30-70*r**2-15*r**4+3*r**6)*y**2)/10-r**4*(-4-4*r**2
            +r**4)*y**3-(r**4*(-3+r**2)*y**4)/2+((-1+r**2)*y**6)/10+y**7/35)
    if (dEl == 0 and dr == 0 and i==3 and j == 1):
        return     (3*logy*r**4*(11+68*r**2+90*r**4+28*r**6+r**8)+(8-324*r**2-13804*r**4
            -40292*r**6-12635*r**8+8260*r**10+1015*r**12-16*r**14+2*r**16)/140
            -r**14/(7*(-1+y)**6)-(2*r**12*(3+2*r**2))/(5*(-1+y)**5)+(-42*r**10
            -73*r**12-18*r**14)/(10*(-1+y)**4)+(-80*r**8-281*r**10-187*r**12-20*r**14)/(10*(
            -1+y)**3)+(-180*r**6-1235*r**8-1692*r**10-525*r**12-20*r**14)/(20*(
            -1+y)**2)+(-12*r**4-190*r**6-513*r**8-339*r**10-46*r**12)/(2*(-1+y))
            +(r**4*(945+4445*r**2+3955*r**4+567*r**6-14*r**8+2*r**10)*y)/35+(r**4*(210
            +680*r**2+315*r**4-6*r**6+r**8)*y**2)/20-(r**4*(-50-90*r**2-5*r**4
            +r**6)*y**3)/10-(r**4*(-9-4*r**2+r**4)*y**4)/4-(r**4*(-3+r**2)*y**5)/5
            -((-1+r**2)**2*y**6)/20+((-1+r**2)*y**7)/70+y**8/140)
    if (dEl == 0 and dr == 0 and i==4 and j == 0):
        return     (36*logy*r**4*(1+8*r**2+15*r**4+8*r**6+r**8)-(3*(-1+48*r**2+2688*r**4
            +11144*r**6+8120*r**8-1680*r**10-1050*r**12-8*r**14+r**16))/70-(2*r**14)/(7*(
            -1+y)**6)-(2*r**12*(1+r**2))/(-1+y)**5+(-30*r**10-73*r**12-30*r**14)/(5*(
            -1+y)**4)-(2*(25*r**8+117*r**10+117*r**12+25*r**14))/(5*(-1+y)**3)
            -(2*(25*r**6+220*r**8+423*r**10+220*r**12+25*r**14))/(5*(-1+y)**2)
            -(2*r**4*(1+r**2)*(3+56*r**2+157*r**4+56*r**6+3*r**8))/(-1+y)-(2*r**4*(
            -105-665*r**2-910*r**4-294*r**6-7*r**8+r**10)*y)/7-(4*r**4*(-15-70*r**2
            -60*r**4-6*r**6+r**8)*y**2)/5-(6*r**4*(-5-15*r**2-5*r**4+r**6)*y**3)/5
            -r**4*(-3-4*r**2+r**4)*y**4-(2*r**4*(-3+r**2)*y**5)/5+(2*(-1+r**2)*y**7)/35
            +y**8/70)

    # derivative wrt Elcuthat
    if (dEl == 1 and dr == 0 and i==0 and j == 0):
        return     (-4*r**4*(-3+r**2)+(8*r**6)/(-1+y)**3+(12*r**4*(1+r**2))/(-1+y)**2+(24*r**4)/(
            -1+y)+12*(-1+r**2)*y**2+8*y**3)
    if (dEl == 1 and dr == 0 and i==0 and j == 1):
        return     (r**4*(18-4*r**2+r**4)+(3*r**8)/(-1+y)**4+(4*r**6*(3+r**2))/(-1+y)**3
            +(2*r**4*(6+11*r**2))/(-1+y)**2+(6*r**4*(5+r**2))/(-1+y)-2*r**4*(
            -3+r**2)*y-6*(-1+r**2)**2*y**2-2*(-1+r**2)*y**3+y**4)
    if (dEl == 1 and dr == 0 and i==0 and j == 2):
        return     ((-3*r**4*(-80-5*r**4+r**6))/10+(6*r**10)/(5*(-1+y)**5)+(3*r**8*(5+r**2))/(2*(
            -1+y)**4)+(r**6*(16+13*r**2))/(-1+y)**3+(4*r**4*(3+9*r**2+r**4))/(
            -1+y)**2+(4*r**4*(9+5*r**2))/(-1+y)+r**4*(12-4*r**2+r**4)*y+(-3
            +9*r**2-6*r**4+2*r**6)*y**2-((-1+r**2)*y**4)/2+y**5/5)
    if (dEl == 1 and dr == 0 and i==0 and j == 3):
        return     ((r**4*(300+110*r**2+15*r**4-6*r**6+r**8))/10+r**12/(2*(-1+y)**6)+(3*r**10*(7
            +r**2))/(5*(-1+y)**5)+(3*(90*r**8+47*r**10))/(20*(-1+y)**4)+(80*r**6
            +117*r**8+9*r**10)/(4*(-1+y)**3)+(3*(16*r**4+72*r**6+23*r**8))/(4*(
            -1+y)**2)+(3*r**4*(1+r**2)*(14+r**2))/(-1+y)-(3*r**4*(-120+20*r**2
            -15*r**4+3*r**6)*y)/20-(3*(2-8*r**2+2*r**4-4*r**6+r**8)*y**2)/4+((
            -1+3*r**2)*y**3)/2-(3*(-1+r**2)*y**5)/20+y**6/20)
    if (dEl == 1 and dr == 0 and i==0 and j == 4):
        return     (-1/28*(r**4*(-1008-896*r**2-91*r**4+21*r**6-7*r**8+r**10))+(3*r**14)/(14*(
            -1+y)**7)+(r**12*(9+r**2))/(4*(-1+y)**6)+(96*r**10+37*r**12)/(10*(
            -1+y)**5)+(3*(35*r**8+34*r**10+2*r**12))/(5*(-1+y)**4)+(240*r**6+550*r**8
            +123*r**10)/(10*(-1+y)**3)+(48*r**4+304*r**6+189*r**8+9*r**10)/(4*(
            -1+y)**2)+(3*(32*r**4+56*r**6+11*r**8))/(2*(-1+y))+(r**4*(120+20*r**2
            +15*r**4-6*r**6+r**8)*y)/5+(3*(-5+25*r**2+35*r**4+25*r**6-10*r**8
            +2*r**10)*y**2)/20+((-1+4*r**2+3*r**4)*y**3)/2+((-1+3*r**2)*y**4)/4
            -((-1+r**2)*y**6)/20+y**7/70)
    if (dEl == 1 and dr == 0 and i==1 and j == 0):
        return     (-2*r**4*(-12-4*r**2+r**4)+(6*r**8)/(-1+y)**4+(16*r**6*(1+r**2))/(-1
            +y)**3+(4*r**4*(3+11*r**2+3*r**4))/(-1+y)**2+(36*r**4*(1+r**2))/(-1
            +y)-4*r**4*(-3+r**2)*y+4*(-1+r**2)*y**3+2*y**4)
    if (dEl == 1 and dr == 0 and i==1 and j == 1):
        return     ((2*r**4*(75+55*r**2-5*r**4+r**6))/5+(12*r**10)/(5*(-1+y)**5)+(6*r**8*(2
            +r**2))/(-1+y)**4+(2*r**6*(10+17*r**2+2*r**4))/(-1+y)**3+(2*r**4*(6
            +31*r**2+15*r**4))/(-1+y)**2+(2*r**4*(21+32*r**2+3*r**4))/(-1+y)
            +18*r**4*y-2*r**4*(-3+r**2)*y**2-2*(-1+r**2)**2*y**3+(2*y**5)/5)
    if (dEl == 1 and dr == 0 and i==1 and j == 2):
        return     (-1/10*(r**4*(-360-460*r**2-15*r**4-6*r**6+r**8))+r**12/(-1+y)**6+(12*r**10*(3
            +r**2))/(5*(-1+y)**5)+(3*(65*r**8+67*r**10+5*r**12))/(10*(-1+y)**4)
            +(48*r**6+123*r**8+35*r**10)/(2*(-1+y)**3)+(24*r**4+168*r**6+133*r**8
            +8*r**10)/(2*(-1+y)**2)+(2*r**4*(24+53*r**2+13*r**4))/(-1+y)+(r**4*(240
            +100*r**2-5*r**4+r**6)*y)/10+(r**4*(24-4*r**2+r**4)*y**2)/2+(-1+3*r**2)*y**3
            -((-1+r**2)**2*y**4)/2-((-1+r**2)*y**5)/10+y**6/10)
    if (dEl == 1 and dr == 0 and i==1 and j == 3):
        return     ((r**4*(1470+2905*r**2+560*r**4+21*r**6-7*r**8+r**10))/35+(3*r**14)/(7*(
            -1+y)**7)+(r**12*(4+r**2))/(-1+y)**6+(75*r**10+55*r**12+3*r**14)/(5*(
            -1+y)**5)+(3*(190*r**8+315*r**10+63*r**12))/(20*(-1+y)**4)+(560*r**6
            +2015*r**8+1032*r**10+45*r**12)/(20*(-1+y)**3)+(48*r**4+440*r**6+525*r**8
            +87*r**10)/(4*(-1+y)**2)+(3*r**4*(2+r**2)*(9+23*r**2+r**4))/(-1+y)
            -(r**4*(-600-580*r**2-15*r**4-6*r**6+r**8)*y)/20-(3*r**4*(-120-20*r**2
            -5*r**4+r**6)*y**2)/20+((-1+4*r**2+9*r**4)*y**3)/2+((-1+3*r**2)*y**4)/2
            -(3*(-1+r**2)**2*y**5)/20-((-1+r**2)*y**6)/20+y**7/35)
    if (dEl == 1 and dr == 0 and i==2 and j == 0):
        return     ((-6*r**4*(-30-50*r**2-5*r**4+r**6))/5+(24*r**10)/(5*(-1+y)**5)+(18*r**8*(1
            +r**2))/(-1+y)**4+(4*r**6*(6+17*r**2+6*r**4))/(-1+y)**3+(4*r**4*(1
            +r**2)*(3+20*r**2+3*r**4))/(-1+y)**2+(16*r**4*(3+8*r**2+3*r**4))/(
            -1+y)-4*r**4*(-6-4*r**2+r**4)*y-4*r**4*(-3+r**2)*y**2+2*(-1+r**2)*y**4
            +(4*y**5)/5)
    if (dEl == 1 and dr == 0 and i==2 and j == 1):
        return     ((r**4*(210+500*r**2+135*r**4-6*r**6+r**8))/5+(2*r**12)/(-1+y)**6+(12*(5*r**10
            +3*r**12))/(5*(-1+y)**5)+(3*(45*r**8+77*r**10+15*r**12))/(5*(-1+y)**4)
            +(r**6*(28+109*r**2+65*r**4+4*r**6))/(-1+y)**3+(r**4*(12+118*r**2
            +165*r**4+38*r**6))/(-1+y)**2+(2*r**4*(27+95*r**2+55*r**4+3*r**6))/(
            -1+y)+(r**4*(150+190*r**2-5*r**4+r**6)*y)/5-r**4*(-18-4*r**2+r**4)*y**2
            -2*r**4*(-3+r**2)*y**3-(-1+r**2)**2*y**4+((-1+r**2)*y**5)/5+y**6/5)
    if (dEl == 1 and dr == 0 and i==2 and j == 2):
        return     ((-3*r**4*(-1120-3640*r**2-1785*r**4-49*r**6-7*r**8+r**10))/70+(6*r**14)/(7*(
            -1+y)**7)+(r**12*(7+3*r**2))/(-1+y)**6+(2*r**10*(57+67*r**2+9*r**4))/(5*(
            -1+y)**5)+(3*(125*r**8+313*r**10+125*r**12+5*r**14))/(10*(-1+y)**4)
            +(2*(80*r**6+410*r**8+368*r**10+55*r**12))/(5*(-1+y)**3)+(2*r**4*(6
            +74*r**2+140*r**4+53*r**6+2*r**8))/(-1+y)**2+(2*r**4*(30+136*r**2
            +115*r**4+16*r**6))/(-1+y)+12*r**4*(3+6*r**2+r**4)*y+(r**4*(120+100*r**2
            -5*r**4+r**6)*y**2)/5+12*r**4*y**3-((1-3*r**2-3*r**4+r**6)*y**4)/2-(2*(
            -1+r**2)**2*y**5)/5+(2*y**7)/35)
    if (dEl == 1 and dr == 0 and i==3 and j == 0):
        return     ((-4*r**4*(-60-220*r**2-135*r**4-6*r**6+r**8))/5+(4*r**12)/(-1+y)**6
            +(96*r**10*(1+r**2))/(5*(-1+y)**5)+(6*(30*r**8+77*r**10+30*r**12))/(5*(
            -1+y)**4)+(2*r**6*(1+r**2)*(16+71*r**2+16*r**4))/(-1+y)**3+(6*r**4*(2
            +26*r**2+55*r**4+26*r**6+2*r**8))/(-1+y)**2+(60*r**4*(1+r**2)*(1+4*r**2
            +r**4))/(-1+y)-(6*r**4*(-30-70*r**2-15*r**4+3*r**6)*y)/5-6*r**4*(
            -4-4*r**2+r**4)*y**2-4*r**4*(-3+r**2)*y**3+(6*(-1+r**2)*y**5)/5+(2*y**6)/5)
    if (dEl == 1 and dr == 0 and i==3 and j == 1):
        return     ((2*r**4*(945+4445*r**2+3955*r**4+567*r**6-14*r**8+2*r**10))/35+(12*r**14)/(7*(
            -1+y)**7)+(4*r**12*(3+2*r**2))/(-1+y)**6+(4*r**10*(42+73*r**2+18*r**4))/(5*(
            -1+y)**5)+(3*(80*r**8+281*r**10+187*r**12+20*r**14))/(5*(-1+y)**4)
            +(180*r**6+1235*r**8+1692*r**10+525*r**12+20*r**14)/(5*(-1+y)**3)+(r**4*(12
            +190*r**2+513*r**4+339*r**6+46*r**8))/(-1+y)**2+(6*r**4*(11+68*r**2
            +90*r**4+28*r**6+r**8))/(-1+y)+(r**4*(210+680*r**2+315*r**4-6*r**6
            +r**8)*y)/5-(3*r**4*(-50-90*r**2-5*r**4+r**6)*y**2)/5-2*r**4*(-9-4*r**2
            +r**4)*y**3-2*r**4*(-3+r**2)*y**4-(3*(-1+r**2)**2*y**5)/5+((-1+r**2)*y**6)/5
            +(4*y**7)/35)
    if (dEl == 1 and dr == 0 and i==4 and j == 0):
        return     ((-4*r**4*(-105-665*r**2-910*r**4-294*r**6-7*r**8+r**10))/7+(24*r**14)/(7*(
            -1+y)**7)+(20*r**12*(1+r**2))/(-1+y)**6+(8*(30*r**10+73*r**12+30*r**14))/(5*(
            -1+y)**5)+(12*(25*r**8+117*r**10+117*r**12+25*r**14))/(5*(-1+y)**4)
            +(8*(25*r**6+220*r**8+423*r**10+220*r**12+25*r**14))/(5*(-1+y)**3)
            +(4*r**4*(1+r**2)*(3+56*r**2+157*r**4+56*r**6+3*r**8))/(-1+y)**2+(72*r**4*(1
            +8*r**2+15*r**4+8*r**6+r**8))/(-1+y)-(16*r**4*(-15-70*r**2-60*r**4
            -6*r**6+r**8)*y)/5-(36*r**4*(-5-15*r**2-5*r**4+r**6)*y**2)/5-8*r**4*(
            -3-4*r**2+r**4)*y**3-4*r**4*(-3+r**2)*y**4+(4*(-1+r**2)*y**6)/5+(8*y**7)/35)

    # derivative wrt r
    if (dEl == 0 and dr == 1 and i==0 and j == 0):
        return     (48*logy*r**3-8*r*(2+6*r**2-3*r**4+r**6)-(12*r**5)/(-1+y)**2-(12*r**3*(2
            +3*r**2))/(-1+y)-12*r**3*(-2+r**2)*y+4*r*y**3)
    if (dEl == 0 and dr == 1 and i==0 and j == 1):
        return     (6*logy*r**3*(10+3*r**2)+(r*(-27-172*r**2+84*r**4-28*r**6+7*r**8))/2
            -(4*r**7)/(-1+y)**3-(2*r**5*(9+4*r**2))/(-1+y)**2-(6*r**3*(4+11*r**2))/(
            -1+y)+4*r**3*(9-3*r**2+r**4)*y-3*r**3*(-2+r**2)*y**2-4*r*(-1+r**2)*y**3
            -(r*y**4)/2)
    if (dEl == 0 and dr == 1 and i==0 and j == 2):
        return     (12*logy*r**3*(6+5*r**2)-(r*(57+660*r**2-200*r**4+60*r**6-40*r**8+8*r**10))/5
            -(3*r**9)/(2*(-1+y)**4)-(5*(4*r**7+r**9))/(2*(-1+y)**3)-(2*r**5*(12
            +13*r**2))/(-1+y)**2-(4*r**3*(6+27*r**2+4*r**4))/(-1+y)-(3*r**3*(
            -32-4*r**4+r**6)*y)/2+2*r**3*(6-3*r**2+r**4)*y**2+r*(3-4*r**2+2*r**4)*y**3
            -(r*y**5)/10)
    if (dEl == 0 and dr == 1 and i==0 and j == 3):
        return     (3*logy*r**3*(28+45*r**2+4*r**4)+(r*(-388-7360*r**2-390*r**4+460*r**6
            +425*r**8-180*r**10+30*r**12))/40-(3*r**11)/(5*(-1+y)**5)-(3*(35*r**9
            +6*r**11))/(20*(-1+y)**4)+(-72*r**7-47*r**9)/(4*(-1+y)**3)-(3*(80*r**5
            +156*r**7+15*r**9))/(8*(-1+y)**2)-(3*r**3*(8+54*r**2+23*r**4))/(-1
            +y)+(3*r**3*(100+55*r**2+10*r**4-5*r**6+r**8)*y)/5-(9*r**3*(-16+4*r**2
            -4*r**4+r**6)*y**2)/8-r*(-2+r**2-3*r**4+r**6)*y**3+(3*r*y**4)/8-(r*y**6)/40)
    if (dEl == 0 and dr == 1 and i==0 and j == 4):
        return     (6*logy*r**3*(16+42*r**2+11*r**4)-(r*(1168+33684*r**2+19740*r**4-7910*r**6
            -1890*r**8+1029*r**10-350*r**12+50*r**14))/140-r**13/(4*(-1+y)**6)
            +(-54*r**11-7*r**13)/(20*(-1+y)**5)-(3*(80*r**9+37*r**11))/(20*(-1+y)**4)
            -(2*(70*r**7+85*r**9+6*r**11))/(5*(-1+y)**3)+(-144*r**5-440*r**7-123*r**9)/(4*(
            -1+y)**2)-(3*(32*r**3+304*r**5+252*r**7+15*r**9))/(4*(-1+y))-(r**3*(
            -288-384*r**2-52*r**4+15*r**6-6*r**8+r**10)*y)/4+(3*r**3*(40+10*r**2
            +10*r**4-5*r**6+r**8)*y**2)/5+(r*(5+14*r**2+15*r**4-8*r**6+2*r**8)*y**3)/4
            +(r*(2+3*r**2)*y**4)/4+(3*r*y**5)/20-(r*y**7)/140)
    if (dEl == 0 and dr == 1 and i==1 and j == 0):
        return     (36*logy*r**3*(2+3*r**2)-3*r*(3+52*r**2-4*r**6+r**8)-(8*r**7)/(-1+y)**3
            -(8*r**5*(3+4*r**2))/(-1+y)**2-(12*r**3*(2+11*r**2+4*r**4))/(-1+y)
            -8*r**3*(-6-3*r**2+r**4)*y-6*r**3*(-2+r**2)*y**2+r*y**4)
    if (dEl == 0 and dr == 1 and i==1 and j == 1):
        return     (12*logy*r**3*(7+16*r**2+2*r**4)+r*(-8-205*r**2-76*r**4+48*r**6-5*r**8
            +r**10)-(3*r**9)/(-1+y)**4-(2*r**7*(8+5*r**2))/(-1+y)**3-(2*r**5*(15
            +34*r**2+5*r**4))/(-1+y)**2-(6*r**3*(4+31*r**2+20*r**4))/(-1+y)+2*r**3*(30
            +33*r**2-4*r**4+r**6)*y+18*r**3*y**2-2*r**3*(-2+r**2)*y**3-r*(-1+r**2)*y**4)
    if (dEl == 0 and dr == 1 and i==1 and j == 2):
        return     (2*logy*r**3*(48+159*r**2+52*r**4)-(r*(428+15528*r**2+14370*r**4-5700*r**6
            +105*r**8-132*r**10+22*r**12))/60-(6*r**11)/(5*(-1+y)**5)-(9*(5*r**9
            +2*r**11))/(5*(-1+y)**4)+(-52*r**7-67*r**9-6*r**11)/(2*(-1+y)**3)+(
            -144*r**5-492*r**7-175*r**9)/(4*(-1+y)**2)-(2*r**3*(12+126*r**2+133*r**4
            +10*r**6))/(-1+y)-(3*r**3*(-120-230*r**2-10*r**4-5*r**6+r**8)*y)/5
            +(r**3*(96+60*r**2-4*r**4+r**6)*y**2)/4+(2*r**3*(12-3*r**2+r**4)*y**3)/3
            +(3*r*y**4)/4-(r*(-1+r**2)*y**5)/5-(r*y**6)/60)
    if (dEl == 0 and dr == 1 and i==1 and j == 3):
        return     (3*logy*r**3*(36+165*r**2+100*r**4+5*r**6)+(r*(-895-44324*r**2-73815*r**4
            +14490*r**6+4410*r**8+357*r**10-140*r**12+20*r**14))/140-r**13/(2*(
            -1+y)**6)-(r**11*(24+7*r**2))/(5*(-1+y)**5)-(3*(125*r**9+110*r**11
            +7*r**13))/(20*(-1+y)**4)+(-760*r**7-1575*r**9-378*r**11)/(20*(-1+y)**3)
            +(-168*r**5-806*r**7-516*r**9-27*r**11)/(4*(-1+y)**2)-(3*(32*r**3+440*r**5
            +700*r**7+145*r**9))/(4*(-1+y))+(r**3*(420+1245*r**2+320*r**4+15*r**6
            -6*r**8+r**10)*y)/5-(3*r**3*(-200-290*r**2-10*r**4-5*r**6+r**8)*y**2)/20
            -(r**3*(-48-12*r**2-4*r**4+r**6)*y**3)/4+(r*(2+9*r**2)*y**4)/4+(3*r*y**5)/10
            -(r*(-1+r**2)*y**6)/20-(r*y**7)/140)
    if (dEl == 0 and dr == 1 and i==2 and j == 0):
        return     (96*logy*r**3*(1+4*r**2+2*r**4)-(8*r*(4+170*r**2+230*r**4-60*r**6-5*r**8
            +r**10))/5-(6*r**9)/(-1+y)**4-(6*r**7*(4+5*r**2))/(-1+y)**3-(4*r**5*(9
            +34*r**2+15*r**4))/(-1+y)**2-(4*r**3*(6+69*r**2+92*r**4+15*r**6))/(
            -1+y)-6*r**3*(-12-30*r**2-4*r**4+r**6)*y-8*r**3*(-3-3*r**2+r**4)*y**2
            -4*r**3*(-2+r**2)*y**3+(2*r*y**5)/5)
    if (dEl == 0 and dr == 1 and i==2 and j == 1):
        return     (2*logy*r**3*(54+285*r**2+220*r**4+15*r**6)+(r*(-175-9858*r**2-20685*r**4
            +1860*r**6+1890*r**8-78*r**10+13*r**12))/30-(12*r**11)/(5*(-1+y)**5)
            -(3*(25*r**9+18*r**11))/(5*(-1+y)**4)-(r**7*(36+77*r**2+18*r**4))/(
            -1+y)**3+(-84*r**5-436*r**7-325*r**9-24*r**11)/(2*(-1+y)**2)-(2*r**3*(12
            +177*r**2+330*r**4+95*r**6))/(-1+y)+(6*r**3*(70+250*r**2+90*r**4
            -5*r**6+r**8)*y)/5+(r**3*(60+114*r**2-4*r**4+r**6)*y**2)/2-(4*r**3*(
            -9-3*r**2+r**4)*y**3)/3-(3*r**3*(-2+r**2)*y**4)/2-(2*r*(-1+r**2)*y**5)/5
            +(r*y**6)/30)
    if (dEl == 0 and dr == 1 and i==2 and j == 2):
        return     (8*logy*r**3*(15+102*r**2+115*r**4+20*r**6)-(2*r*(40+2915*r**2+8844*r**4
            +1290*r**6-1425*r**8-6*r**10-7*r**12+r**14))/15-r**13/(-1+y)**6-(21*r**11*(2
            +r**2))/(5*(-1+y)**5)-(3*(95*r**9+134*r**11+21*r**13))/(10*(-1+y)**4)
            +(-100*r**7-313*r**9-150*r**11-7*r**13)/(2*(-1+y)**3)-(2*r**5*(24+164*r**2
            +184*r**4+33*r**6))/(-1+y)**2-(2*r**3*(12+222*r**2+560*r**4+265*r**6
            +12*r**8))/(-1+y)-(3*r**3*(-320-1560*r**2-1020*r**4-35*r**6-6*r**8
            +r**10)*y)/10+12*r**3*(3+9*r**2+2*r**4)*y**2+(r**3*(48+60*r**2-4*r**4
            +r**6)*y**3)/3+6*r**3*y**4-(3*r*(-1-2*r**2+r**4)*y**5)/10-(2*r*(-1+r**2)*y**6)/15)
    if (dEl == 0 and dr == 1 and i==3 and j == 0):
        return     (60*logy*r**3*(2+15*r**2+20*r**4+5*r**6)-r*(5+398*r**2+1395*r**4+420*r**6
            -250*r**8-6*r**10+r**12)-(24*r**11)/(5*(-1+y)**5)-(24*(5*r**9+6*r**11))/(5*(
            -1+y)**4)-(2*r**7*(24+77*r**2+36*r**4))/(-1+y)**3-(3*r**5*(16+116*r**2
            +145*r**4+32*r**6))/(-1+y)**2-(12*r**3*(2+39*r**2+110*r**4+65*r**6
            +6*r**8))/(-1+y)-(24*r**3*(-20-110*r**2-90*r**4-5*r**6+r**8)*y)/5
            -9*r**3*(-4-14*r**2-4*r**4+r**6)*y**2-8*r**3*(-2-3*r**2+r**4)*y**3-3*r**3*(
            -2+r**2)*y**4+(r*y**6)/5)
    if (dEl == 0 and dr == 1 and i==3 and j == 1):
        return     (12*logy*r**3*(11+102*r**2+180*r**4+70*r**6+3*r**8)+(r*(-162-16114*r**2
            -74718*r**4-44170*r**6+14770*r**8+2835*r**10-56*r**12+8*r**14))/35
            -(2*r**13)/(-1+y)**6-(8*r**11*(9+7*r**2))/(5*(-1+y)**5)-(6*(35*r**9
            +73*r**11+21*r**13))/(5*(-1+y)**4)+(-320*r**7-1405*r**9-1122*r**11
            -140*r**13)/(5*(-1+y)**3)-(r**5*(54+494*r**2+846*r**4+315*r**6+14*r**8))/(
            -1+y)**2-(3*r**3*(8+190*r**2+684*r**4+565*r**6+92*r**8))/(-1+y)
            +(2*r**3*(270+1905*r**2+2260*r**4+405*r**6-12*r**8+2*r**10)*y)/5+(3*r**3*(70
            +340*r**2+210*r**4-5*r**6+r**8)*y**2)/5-r**3*(-20-54*r**2-4*r**4+r**6)*y**3
            -r**3*(-9-6*r**2+2*r**4)*y**4-(6*r**3*(-2+r**2)*y**5)/5-(r*(-1+r**2)*y**6)/5
            +(r*y**7)/35)
    if (dEl == 0 and dr == 1 and i==4 and j == 0):
        return     (144*logy*r**3*(1+12*r**2+30*r**4+20*r**6+3*r**8)-(12*r*(12+1554*r**2
            +10038*r**4+11270*r**6-420*r**8-1365*r**10-14*r**12+2*r**14))/35-(4*r**13)/(
            -1+y)**6-(4*r**11*(6+7*r**2))/(-1+y)**5-(12*(25*r**9+73*r**11+35*r**13))/(5*(
            -1+y)**4)-(4*(100*r**7+585*r**9+702*r**11+175*r**13))/(5*(-1+y)**3)
            -(4*r**5*(15+176*r**2+423*r**4+264*r**6+35*r**8))/(-1+y)**2-(12*r**3*(2
            +59*r**2+284*r**4+355*r**6+118*r**8+7*r**10))/(-1+y)-4*r**3*(-30
            -285*r**2-520*r**4-210*r**6-6*r**8+r**10)*y-(48*r**3*(-5-35*r**2-40*r**4
            -5*r**6+r**8)*y**2)/5-12*r**3*(-2-9*r**2-4*r**4+r**6)*y**3-4*r**3*(-3
            -6*r**2+2*r**4)*y**4-(12*r**3*(-2+r**2)*y**5)/5+(4*r*y**7)/35)

    # second derivative wrt Elcuthat
    if (dEl == 2 and dr == 0 and i==0 and j == 0):
        return     ((-48*r**6)/(-1+y)**4-(48*r**4*(1+r**2))/(-1+y)**3-(48*r**4)/(-1+y)**2
            +48*(-1+r**2)*y+48*y**2)
    if (dEl == 2 and dr == 0 and i==0 and j == 1):
        return     (-4*r**4*(-3+r**2)-(24*r**8)/(-1+y)**5-(24*r**6*(3+r**2))/(-1+y)**4
            -(8*r**4*(6+11*r**2))/(-1+y)**3-(12*r**4*(5+r**2))/(-1+y)**2-24*(
            -1+r**2)**2*y-12*(-1+r**2)*y**2+8*y**3)
    if (dEl == 2 and dr == 0 and i==0 and j == 2):
        return     (2*r**4*(12-4*r**2+r**4)-(12*r**10)/(-1+y)**6-(12*r**8*(5+r**2))/(-1
            +y)**5-(6*r**6*(16+13*r**2))/(-1+y)**4-(16*r**4*(3+9*r**2+r**4))/(
            -1+y)**3-(8*r**4*(9+5*r**2))/(-1+y)**2+4*(-3+9*r**2-6*r**4+2*r**6)*y
            -4*(-1+r**2)*y**3+2*y**4)
    if (dEl == 2 and dr == 0 and i==0 and j == 3):
        return     ((-3*r**4*(-120+20*r**2-15*r**4+3*r**6))/10-(6*r**12)/(-1+y)**7-(6*r**10*(7
            +r**2))/(-1+y)**6-(6*(90*r**8+47*r**10))/(5*(-1+y)**5)-(3*(80*r**6
            +117*r**8+9*r**10))/(2*(-1+y)**4)-(3*r**4*(16+72*r**2+23*r**4))/(-1
            +y)**3-(6*r**4*(1+r**2)*(14+r**2))/(-1+y)**2-3*(2-8*r**2+2*r**4
            -4*r**6+r**8)*y+3*(-1+3*r**2)*y**2-(3*(-1+r**2)*y**4)/2+(3*y**5)/5)
    if (dEl == 2 and dr == 0 and i==0 and j == 4):
        return     ((2*r**4*(120+20*r**2+15*r**4-6*r**6+r**8))/5-(3*r**14)/(-1+y)**8-(3*r**12*(9
            +r**2))/(-1+y)**7-(r**10*(96+37*r**2))/(-1+y)**6-(24*(35*r**8+34*r**10
            +2*r**12))/(5*(-1+y)**5)-(3*(240*r**6+550*r**8+123*r**10))/(5*(-1+y)**4)
            -(r**4*(48+304*r**2+189*r**4+9*r**6))/(-1+y)**3-(3*r**4*(32+56*r**2
            +11*r**4))/(-1+y)**2+(3*(-5+25*r**2+35*r**4+25*r**6-10*r**8+2*r**10)*y)/5
            +3*(-1+4*r**2+3*r**4)*y**2+2*(-1+3*r**2)*y**3-(3*(-1+r**2)*y**5)/5
            +y**6/5)
    if (dEl == 2 and dr == 0 and i==1 and j == 0):
        return     (-8*r**4*(-3+r**2)-(48*r**8)/(-1+y)**5-(96*r**6*(1+r**2))/(-1+y)**4
            -(16*r**4*(3+11*r**2+3*r**4))/(-1+y)**3-(72*r**4*(1+r**2))/(-1+y)**2
            +24*(-1+r**2)*y**2+16*y**3)
    if (dEl == 2 and dr == 0 and i==1 and j == 1):
        return     (36*r**4-(24*r**10)/(-1+y)**6-(48*r**8*(2+r**2))/(-1+y)**5-(12*r**6*(10
            +17*r**2+2*r**4))/(-1+y)**4-(8*r**4*(6+31*r**2+15*r**4))/(-1+y)**3
            -(4*r**4*(21+32*r**2+3*r**4))/(-1+y)**2-8*r**4*(-3+r**2)*y-12*(-1
            +r**2)**2*y**2+4*y**4)
    if (dEl == 2 and dr == 0 and i==1 and j == 2):
        return     ((r**4*(240+100*r**2-5*r**4+r**6))/5-(12*r**12)/(-1+y)**7-(24*r**10*(3
            +r**2))/(-1+y)**6-(12*(65*r**8+67*r**10+5*r**12))/(5*(-1+y)**5)-(3*r**6*(48
            +123*r**2+35*r**4))/(-1+y)**4-(2*r**4*(24+168*r**2+133*r**4+8*r**6))/(
            -1+y)**3-(4*r**4*(24+53*r**2+13*r**4))/(-1+y)**2+2*r**4*(24-4*r**2
            +r**4)*y+6*(-1+3*r**2)*y**2-4*(-1+r**2)**2*y**3-(-1+r**2)*y**4+(6*y**5)/5)
    if (dEl == 2 and dr == 0 and i==1 and j == 3):
        return     (-1/10*(r**4*(-600-580*r**2-15*r**4-6*r**6+r**8))-(6*r**14)/(-1+y)**8
            -(12*r**12*(4+r**2))/(-1+y)**7-(2*r**10*(75+55*r**2+3*r**4))/(-1+y)**6
            -(6*(190*r**8+315*r**10+63*r**12))/(5*(-1+y)**5)-(3*(560*r**6+2015*r**8
            +1032*r**10+45*r**12))/(10*(-1+y)**4)-(r**4*(48+440*r**2+525*r**4+87*r**6))/(
            -1+y)**3-(6*r**4*(2+r**2)*(9+23*r**2+r**4))/(-1+y)**2-(3*r**4*(-120
            -20*r**2-5*r**4+r**6)*y)/5+3*(-1+4*r**2+9*r**4)*y**2+4*(-1+3*r**2)*y**3
            -(3*(-1+r**2)**2*y**4)/2-(3*(-1+r**2)*y**5)/5+(2*y**6)/5)
    if (dEl == 2 and dr == 0 and i==2 and j == 0):
        return     (-8*r**4*(-6-4*r**2+r**4)-(48*r**10)/(-1+y)**6-(144*r**8*(1+r**2))/(
            -1+y)**5-(24*r**6*(6+17*r**2+6*r**4))/(-1+y)**4-(16*r**4*(1+r**2)*(3
            +20*r**2+3*r**4))/(-1+y)**3-(32*r**4*(3+8*r**2+3*r**4))/(-1+y)**2
            -16*r**4*(-3+r**2)*y+16*(-1+r**2)*y**3+8*y**4)
    if (dEl == 2 and dr == 0 and i==2 and j == 1):
        return     ((2*r**4*(150+190*r**2-5*r**4+r**6))/5-(24*r**12)/(-1+y)**7-(24*r**10*(5
            +3*r**2))/(-1+y)**6-(24*(45*r**8+77*r**10+15*r**12))/(5*(-1+y)**5)
            -(6*r**6*(28+109*r**2+65*r**4+4*r**6))/(-1+y)**4-(4*r**4*(12+118*r**2
            +165*r**4+38*r**6))/(-1+y)**3-(4*r**4*(27+95*r**2+55*r**4+3*r**6))/(
            -1+y)**2-4*r**4*(-18-4*r**2+r**4)*y-12*r**4*(-3+r**2)*y**2-8*(-1
            +r**2)**2*y**3+2*(-1+r**2)*y**4+(12*y**5)/5)
    if (dEl == 2 and dr == 0 and i==2 and j == 2):
        return     (24*r**4*(3+6*r**2+r**4)-(12*r**14)/(-1+y)**8-(12*r**12*(7+3*r**2))/(
            -1+y)**7-(4*r**10*(57+67*r**2+9*r**4))/(-1+y)**6-(12*(125*r**8+313*r**10
            +125*r**12+5*r**14))/(5*(-1+y)**5)-(12*(80*r**6+410*r**8+368*r**10
            +55*r**12))/(5*(-1+y)**4)-(8*r**4*(6+74*r**2+140*r**4+53*r**6+2*r**8))/(
            -1+y)**3-(4*r**4*(30+136*r**2+115*r**4+16*r**6))/(-1+y)**2+(4*r**4*(120
            +100*r**2-5*r**4+r**6)*y)/5+72*r**4*y**2-4*(1-3*r**2-3*r**4+r**6)*y**3
            -4*(-1+r**2)**2*y**4+(4*y**6)/5)
    if (dEl == 2 and dr == 0 and i==3 and j == 0):
        return     ((-12*r**4*(-30-70*r**2-15*r**4+3*r**6))/5-(48*r**12)/(-1+y)**7-(192*r**10*(1
            +r**2))/(-1+y)**6-(48*(30*r**8+77*r**10+30*r**12))/(5*(-1+y)**5)-(12*r**6*(1
            +r**2)*(16+71*r**2+16*r**4))/(-1+y)**4-(24*r**4*(2+26*r**2+55*r**4
            +26*r**6+2*r**8))/(-1+y)**3-(120*r**4*(1+r**2)*(1+4*r**2+r**4))/(-1
            +y)**2-24*r**4*(-4-4*r**2+r**4)*y-24*r**4*(-3+r**2)*y**2+12*(-1+r**2)*y**4
            +(24*y**5)/5)
    if (dEl == 2 and dr == 0 and i==3 and j == 1):
        return     ((2*r**4*(210+680*r**2+315*r**4-6*r**6+r**8))/5-(24*r**14)/(-1+y)**8
            -(48*r**12*(3+2*r**2))/(-1+y)**7-(8*r**10*(42+73*r**2+18*r**4))/(-1
            +y)**6-(24*(80*r**8+281*r**10+187*r**12+20*r**14))/(5*(-1+y)**5)-(6*(180*r**6
            +1235*r**8+1692*r**10+525*r**12+20*r**14))/(5*(-1+y)**4)-(4*r**4*(12
            +190*r**2+513*r**4+339*r**6+46*r**8))/(-1+y)**3-(12*r**4*(11+68*r**2
            +90*r**4+28*r**6+r**8))/(-1+y)**2-(12*r**4*(-50-90*r**2-5*r**4+r**6)*y)/5
            -12*r**4*(-9-4*r**2+r**4)*y**2-16*r**4*(-3+r**2)*y**3-6*(-1+r**2)**2*y**4
            +(12*(-1+r**2)*y**5)/5+(8*y**6)/5)
    if (dEl == 2 and dr == 0 and i==4 and j == 0):
        return     ((-32*r**4*(-15-70*r**2-60*r**4-6*r**6+r**8))/5-(48*r**14)/(-1+y)**8
            -(240*r**12*(1+r**2))/(-1+y)**7-(16*r**10*(30+73*r**2+30*r**4))/(-1
            +y)**6-(96*(25*r**8+117*r**10+117*r**12+25*r**14))/(5*(-1+y)**5)-(48*(25*r**6
            +220*r**8+423*r**10+220*r**12+25*r**14))/(5*(-1+y)**4)-(16*r**4*(1
            +r**2)*(3+56*r**2+157*r**4+56*r**6+3*r**8))/(-1+y)**3-(144*r**4*(1
            +8*r**2+15*r**4+8*r**6+r**8))/(-1+y)**2-(144*r**4*(-5-15*r**2-5*r**4
            +r**6)*y)/5-48*r**4*(-3-4*r**2+r**4)*y**2-32*r**4*(-3+r**2)*y**3+(48*(
            -1+r**2)*y**5)/5+(16*y**6)/5)

    # second derivative wrt r
    if (dEl == 0 and dr == 2 and i==0 and j == 0):
        return     (144*logy*r**2-8*(2+30*r**2-15*r**4+7*r**6)-(60*r**4)/(-1+y)**2-(36*r**2*(2
            +5*r**2))/(-1+y)-12*r**2*(-6+5*r**2)*y+4*y**3)
    if (dEl == 0 and dr == 2 and i==0 and j == 1):
        return     (90*logy*r**2*(2+r**2)+(-27-756*r**2+348*r**4-196*r**6+63*r**8)/2-(28*r**6)/(
            -1+y)**3-(2*r**4*(45+28*r**2))/(-1+y)**2-(6*r**2*(12+55*r**2))/(-1
            +y)+4*r**2*(27-15*r**2+7*r**4)*y-3*r**2*(-6+5*r**2)*y**2-4*(-1+3*r**2)*y**3
            -y**4/2)
    if (dEl == 0 and dr == 2 and i==0 and j == 2):
        return     (12*logy*r**2*(18+25*r**2)+(-57-2700*r**2+400*r**4-420*r**6+360*r**8
            -88*r**10)/5-(27*r**8)/(2*(-1+y)**4)-(5*(28*r**6+9*r**8))/(2*(-1+y)**3)
            -(2*r**4*(60+91*r**2))/(-1+y)**2-(4*r**2*(18+135*r**2+28*r**4))/(-1
            +y)-(3*r**2*(-96-28*r**4+9*r**6)*y)/2+2*r**2*(18-15*r**2+7*r**4)*y**2
            +(3-12*r**2+10*r**4)*y**3-y**5/10)
    if (dEl == 0 and dr == 2 and i==0 and j == 3):
        return     (3*logy*r**2*(84+225*r**2+28*r**4)+(-388-28800*r**2-12750*r**4+2260*r**6
            +3825*r**8-1980*r**10+390*r**12)/40-(33*r**10)/(5*(-1+y)**5)-(9*(105*r**8
            +22*r**10))/(20*(-1+y)**4)-(9*(56*r**6+47*r**8))/(4*(-1+y)**3)-(3*(400*r**4
            +1092*r**6+135*r**8))/(8*(-1+y)**2)-(3*r**2*(24+270*r**2+161*r**4))/(
            -1+y)+(3*r**2*(300+275*r**2+70*r**4-45*r**6+11*r**8)*y)/5-(9*r**2*(
            -48+20*r**2-28*r**4+9*r**6)*y**2)/8-(-2+3*r**2-15*r**4+7*r**6)*y**3
            +(3*y**4)/8-y**6/40)
    if (dEl == 0 and dr == 2 and i==0 and j == 4):
        return     (6*logy*r**2*(48+210*r**2+77*r**4)+(-1168-127932*r**2-169260*r**4+36890*r**6
            +17010*r**8-11319*r**10+4550*r**12-750*r**14)/140-(13*r**12)/(4*(-1
            +y)**6)+(-594*r**10-91*r**12)/(20*(-1+y)**5)-(3*(720*r**8+407*r**10))/(20*(
            -1+y)**4)-(2*(490*r**6+765*r**8+66*r**10))/(5*(-1+y)**3)+(-720*r**4
            -3080*r**6-1107*r**8)/(4*(-1+y)**2)-(3*(96*r**2+1520*r**4+1764*r**6
            +135*r**8))/(4*(-1+y))-(r**2*(-864-1920*r**2-364*r**4+135*r**6-66*r**8
            +13*r**10)*y)/4+(3*r**2*(120+50*r**2+70*r**4-45*r**6+11*r**8)*y**2)/5
            +((5+42*r**2+75*r**4-56*r**6+18*r**8)*y**3)/4+((2+9*r**2)*y**4)/4
            +(3*y**5)/20-y**7/140)
    if (dEl == 0 and dr == 2 and i==1 and j == 0):
        return     (108*logy*r**2*(2+5*r**2)-3*(3+204*r**2+72*r**4-28*r**6+9*r**8)-(56*r**6)/(
            -1+y)**3-(8*r**4*(15+28*r**2))/(-1+y)**2-(12*r**2*(6+55*r**2+28*r**4))/(
            -1+y)-8*r**2*(-18-15*r**2+7*r**4)*y-6*r**2*(-6+5*r**2)*y**2+y**4)
    if (dEl == 0 and dr == 2 and i==1 and j == 1):
        return     (-8-783*r**2-764*r**4+288*r**6-45*r**8+11*r**10+12*logy*r**2*(21+80*r**2
            +14*r**4)-(27*r**8)/(-1+y)**4-(2*r**6*(56+45*r**2))/(-1+y)**3-(2*r**4*(75
            +238*r**2+45*r**4))/(-1+y)**2-(6*r**2*(12+155*r**2+140*r**4))/(-1+y)
            +2*r**2*(90+165*r**2-28*r**4+9*r**6)*y+54*r**2*y**2-2*r**2*(-6+5*r**2)*y**3
            -(-1+3*r**2)*y**4)
    if (dEl == 0 and dr == 2 and i==1 and j == 2):
        return     (2*logy*r**2*(144+795*r**2+364*r**4)+(-428-58104*r**2-110010*r**4+27420*r**6
            -945*r**8+1452*r**10-286*r**12)/60-(66*r**10)/(5*(-1+y)**5)-(9*(45*r**8
            +22*r**10))/(5*(-1+y)**4)+(-364*r**6-603*r**8-66*r**10)/(2*(-1+y)**3)
            -(3*(240*r**4+1148*r**6+525*r**8))/(4*(-1+y)**2)-(2*r**2*(36+630*r**2
            +931*r**4+90*r**6))/(-1+y)-(3*r**2*(-360-1150*r**2-70*r**4-45*r**6
            +11*r**8)*y)/5+(r**2*(288+300*r**2-28*r**4+9*r**6)*y**2)/4+(2*r**2*(36
            -15*r**2+7*r**4)*y**3)/3+(3*y**4)/4-((-1+3*r**2)*y**5)/5-y**6/60)
    if (dEl == 0 and dr == 2 and i==1 and j == 3):
        return     (3*logy*r**2*(108+825*r**2+700*r**4+45*r**6)+(-895-163212*r**2-507675*r**4
            +17430*r**6+35490*r**8+3927*r**10-1820*r**12+300*r**14)/140-(13*r**12)/(2*(
            -1+y)**6)-(r**10*(264+91*r**2))/(5*(-1+y)**5)-(3*(1125*r**8+1210*r**10
            +91*r**12))/(20*(-1+y)**4)-(7*(760*r**6+2025*r**8+594*r**10))/(20*(-1
            +y)**3)+(-840*r**4-5642*r**6-4644*r**8-297*r**10)/(4*(-1+y)**2)-(3*(96*r**2
            +2200*r**4+4900*r**6+1305*r**8))/(4*(-1+y))+(r**2*(1260+6225*r**2
            +2240*r**4+135*r**6-66*r**8+13*r**10)*y)/5-(3*r**2*(-600-1450*r**2
            -70*r**4-45*r**6+11*r**8)*y**2)/20-(r**2*(-144-60*r**2-28*r**4+9*r**6)*y**3)/4
            +((2+27*r**2)*y**4)/4+(3*y**5)/10-((-1+3*r**2)*y**6)/20-y**7/140)
    if (dEl == 0 and dr == 2 and i==2 and j == 0):
        return     (96*logy*r**2*(3+20*r**2+14*r**4)-(8*(4+630*r**2+1630*r**4-180*r**6
            -45*r**8+11*r**10))/5-(54*r**8)/(-1+y)**4-(6*r**6*(28+45*r**2))/(-1
            +y)**3-(4*r**4*(45+238*r**2+135*r**4))/(-1+y)**2-(4*r**2*(18+345*r**2
            +644*r**4+135*r**6))/(-1+y)-6*r**2*(-36-150*r**2-28*r**4+9*r**6)*y
            -8*r**2*(-9-15*r**2+7*r**4)*y**2-4*r**2*(-6+5*r**2)*y**3+(2*y**5)/5)
    if (dEl == 0 and dr == 2 and i==2 and j == 1):
        return     (2*logy*r**2*(162+1425*r**2+1540*r**4+135*r**6)+(-175-36054*r**2-137625*r**4
            -13380*r**6+15210*r**8-858*r**10+169*r**12)/30-(132*r**10)/(5*(-1+y)**5)
            -(27*(25*r**8+22*r**10))/(5*(-1+y)**4)-(9*r**6*(28+77*r**2+22*r**4))/(
            -1+y)**3+(-420*r**4-3052*r**6-2925*r**8-264*r**10)/(2*(-1+y)**2)-(6*r**2*(12
            +295*r**2+770*r**4+285*r**6))/(-1+y)+(6*r**2*(210+1250*r**2+630*r**4
            -45*r**6+11*r**8)*y)/5+(r**2*(180+570*r**2-28*r**4+9*r**6)*y**2)/2-(4*r**2*(
            -27-15*r**2+7*r**4)*y**3)/3-(3*r**2*(-6+5*r**2)*y**4)/2-(2*(-1+3*r**2)*y**5)/5
            +y**6/30)
    if (dEl == 0 and dr == 2 and i==2 and j == 2):
        return     (40*logy*r**2*(9+102*r**2+161*r**4+36*r**6)-(2*(40+10545*r**2+56460*r**4
            +22830*r**6-10425*r**8-66*r**10-91*r**12+15*r**14))/15-(13*r**12)/(
            -1+y)**6-(21*r**10*(22+13*r**2))/(5*(-1+y)**5)-(3*(855*r**8+1474*r**10
            +273*r**12))/(10*(-1+y)**4)+(-700*r**6-2817*r**8-1650*r**10-91*r**12)/(2*(
            -1+y)**3)-(2*r**4*(120+1148*r**2+1656*r**4+363*r**6))/(-1+y)**2-(2*r**2*(36
            +1110*r**2+3920*r**4+2385*r**6+132*r**8))/(-1+y)-(3*r**2*(-960-7800*r**2
            -7140*r**4-315*r**6-66*r**8+13*r**10)*y)/10+12*r**2*(9+45*r**2+14*r**4)*y**2
            +(r**2*(144+300*r**2-28*r**4+9*r**6)*y**3)/3+18*r**2*y**4-(3*(-1-6*r**2
            +5*r**4)*y**5)/10-(2*(-1+3*r**2)*y**6)/15)
    if (dEl == 0 and dr == 2 and i==3 and j == 0):
        return     (-5-1434*r**2-8775*r**4-5340*r**6+1650*r**8+66*r**10-13*r**12+60*logy*r**2*(6
            +75*r**2+140*r**4+45*r**6)-(264*r**10)/(5*(-1+y)**5)-(72*(15*r**8+22*r**10))/(5*(
            -1+y)**4)-(6*r**6*(56+231*r**2+132*r**4))/(-1+y)**3-(3*r**4*(80+812*r**2
            +1305*r**4+352*r**6))/(-1+y)**2-(12*r**2*(6+195*r**2+770*r**4+585*r**6
            +66*r**8))/(-1+y)-(24*r**2*(-60-550*r**2-630*r**4-45*r**6+11*r**8)*y)/5
            -9*r**2*(-12-70*r**2-28*r**4+9*r**6)*y**2-8*r**2*(-6-15*r**2+7*r**4)*y**3
            -3*r**2*(-6+5*r**2)*y**4+y**6/5)
    if (dEl == 0 and dr == 2 and i==3 and j == 1):
        return     (36*logy*r**2*(11+170*r**2+420*r**4+210*r**6+11*r**8)+(-162-57582*r**2
            -459270*r**4-460390*r**6+74130*r**8+28665*r**10-728*r**12+120*r**14)/35
            -(26*r**12)/(-1+y)**6-(8*r**10*(99+91*r**2))/(5*(-1+y)**5)-(6*(315*r**8
            +803*r**10+273*r**12))/(5*(-1+y)**4)+(-2240*r**6-12645*r**8-12342*r**10
            -1820*r**12)/(5*(-1+y)**3)-(r**4*(270+3458*r**2+7614*r**4+3465*r**6
            +182*r**8))/(-1+y)**2-(3*r**2*(24+950*r**2+4788*r**4+5085*r**6+1012*r**8))/(
            -1+y)+(2*r**2*(810+9525*r**2+15820*r**4+3645*r**6-132*r**8+26*r**10)*y)/5
            +(3*r**2*(210+1700*r**2+1470*r**4-45*r**6+11*r**8)*y**2)/5-r**2*(-60
            -270*r**2-28*r**4+9*r**6)*y**3-r**2*(-27-30*r**2+14*r**4)*y**4-(6*r**2*(
            -6+5*r**2)*y**5)/5-((-1+3*r**2)*y**6)/5+y**7/35)
    if (dEl == 0 and dr == 2 and i==4 and j == 0):
        return     (432*logy*r**2*(1+20*r**2+70*r**4+60*r**6+11*r**8)-(12*(12+5502*r**2
            +60270*r**4+104090*r**6+13020*r**8-12495*r**10-182*r**12+30*r**14))/35
            -(52*r**12)/(-1+y)**6-(4*r**10*(66+91*r**2))/(-1+y)**5-(12*(225*r**8
            +803*r**10+455*r**12))/(5*(-1+y)**4)-(4*(700*r**6+5265*r**8+7722*r**10
            +2275*r**12))/(5*(-1+y)**3)-(4*r**4*(75+1232*r**2+3807*r**4+2904*r**6
            +455*r**8))/(-1+y)**2-(12*r**2*(6+295*r**2+1988*r**4+3195*r**6+1298*r**8
            +91*r**10))/(-1+y)-4*r**2*(-90-1425*r**2-3640*r**4-1890*r**6-66*r**8
            +13*r**10)*y-(48*r**2*(-15-175*r**2-280*r**4-45*r**6+11*r**8)*y**2)/5
            -12*r**2*(-6-45*r**2-28*r**4+9*r**6)*y**3-4*r**2*(-9-30*r**2+14*r**4)*y**4
            -(12*r**2*(-6+5*r**2)*y**5)/5+(4*y**7)/35)

    # second derivative wrt Elcuthat and r
    if (dEl == 1 and dr == 1 and i==0 and j == 0):
        return     (-24*r**3*(-2+r**2)+(48*r**5)/(-1+y)**3+(24*r**3*(2+3*r**2))/(-1+y)**2
            +(96*r**3)/(-1+y)+24*r*y**2)
    if (dEl == 1 and dr == 1 and i==0 and j == 1):
        return     (8*r**3*(9-3*r**2+r**4)+(24*r**7)/(-1+y)**4+(8*r**5*(9+4*r**2))/(-1
            +y)**3+(12*r**3*(4+11*r**2))/(-1+y)**2+(12*r**3*(10+3*r**2))/(-1+y)
            -12*r**3*(-2+r**2)*y-24*r*(-1+r**2)*y**2-4*r*y**3)
    if (dEl == 1 and dr == 1 and i==0 and j == 2):
        return     (-3*r**3*(-32-4*r**4+r**6)+(12*r**9)/(-1+y)**5+(15*r**7*(4+r**2))/(-1
            +y)**4+(8*r**5*(12+13*r**2))/(-1+y)**3+(8*r**3*(6+27*r**2+4*r**4))/(
            -1+y)**2+(24*r**3*(6+5*r**2))/(-1+y)+8*r**3*(6-3*r**2+r**4)*y+6*r*(3
            -4*r**2+2*r**4)*y**2-r*y**4)
    if (dEl == 1 and dr == 1 and i==0 and j == 3):
        return     ((6*r**3*(100+55*r**2+10*r**4-5*r**6+r**8))/5+(6*r**11)/(-1+y)**6+(6*(35*r**9
            +6*r**11))/(5*(-1+y)**5)+(3*(72*r**7+47*r**9))/(2*(-1+y)**4)+(3*(80*r**5
            +156*r**7+15*r**9))/(2*(-1+y)**3)+(6*r**3*(8+54*r**2+23*r**4))/(-1
            +y)**2+(6*r**3*(28+45*r**2+4*r**4))/(-1+y)-(9*r**3*(-16+4*r**2-4*r**4
            +r**6)*y)/2-6*r*(-2+r**2-3*r**4+r**6)*y**2+3*r*y**3-(3*r*y**5)/10)
    if (dEl == 1 and dr == 1 and i==0 and j == 4):
        return     (-1/2*(r**3*(-288-384*r**2-52*r**4+15*r**6-6*r**8+r**10))+(3*r**13)/(
            -1+y)**7+(54*r**11+7*r**13)/(2*(-1+y)**6)+(6*(80*r**9+37*r**11))/(5*(
            -1+y)**5)+(12*(70*r**7+85*r**9+6*r**11))/(5*(-1+y)**4)+(r**5*(144
            +440*r**2+123*r**4))/(-1+y)**3+(3*(32*r**3+304*r**5+252*r**7+15*r**9))/(2*(
            -1+y)**2)+(12*r**3*(16+42*r**2+11*r**4))/(-1+y)+(12*r**3*(40+10*r**2
            +10*r**4-5*r**6+r**8)*y)/5+(3*r*(5+14*r**2+15*r**4-8*r**6+2*r**8)*y**2)/2
            +2*r*(2+3*r**2)*y**3+(3*r*y**4)/2-(r*y**6)/10)
    if (dEl == 1 and dr == 1 and i==1 and j == 0):
        return     (-16*r**3*(-6-3*r**2+r**4)+(48*r**7)/(-1+y)**4+(32*r**5*(3+4*r**2))/(
            -1+y)**3+(24*r**3*(2+11*r**2+4*r**4))/(-1+y)**2+(72*r**3*(2+3*r**2))/(
            -1+y)-24*r**3*(-2+r**2)*y+8*r*y**3)
    if (dEl == 1 and dr == 1 and i==1 and j == 1):
        return     (4*r**3*(30+33*r**2-4*r**4+r**6)+(24*r**9)/(-1+y)**5+(12*r**7*(8+5*r**2))/(
            -1+y)**4+(8*r**5*(15+34*r**2+5*r**4))/(-1+y)**3+(12*r**3*(4+31*r**2
            +20*r**4))/(-1+y)**2+(24*r**3*(7+16*r**2+2*r**4))/(-1+y)+72*r**3*y
            -12*r**3*(-2+r**2)*y**2-8*r*(-1+r**2)*y**3)
    if (dEl == 1 and dr == 1 and i==1 and j == 2):
        return     ((-6*r**3*(-120-230*r**2-10*r**4-5*r**6+r**8))/5+(12*r**11)/(-1+y)**6
            +(72*(5*r**9+2*r**11))/(5*(-1+y)**5)+(3*r**7*(52+67*r**2+6*r**4))/(
            -1+y)**4+(r**5*(144+492*r**2+175*r**4))/(-1+y)**3+(4*r**3*(12+126*r**2
            +133*r**4+10*r**6))/(-1+y)**2+(4*r**3*(48+159*r**2+52*r**4))/(-1+y)
            +r**3*(96+60*r**2-4*r**4+r**6)*y+4*r**3*(12-3*r**2+r**4)*y**2+6*r*y**3
            -2*r*(-1+r**2)*y**4-(r*y**5)/5)
    if (dEl == 1 and dr == 1 and i==1 and j == 3):
        return     ((2*r**3*(420+1245*r**2+320*r**4+15*r**6-6*r**8+r**10))/5+(6*r**13)/(
            -1+y)**7+(2*r**11*(24+7*r**2))/(-1+y)**6+(6*(125*r**9+110*r**11+7*r**13))/(5*(
            -1+y)**5)+(3*(760*r**7+1575*r**9+378*r**11))/(10*(-1+y)**4)+(r**5*(168
            +806*r**2+516*r**4+27*r**6))/(-1+y)**3+(3*(32*r**3+440*r**5+700*r**7
            +145*r**9))/(2*(-1+y)**2)+(6*r**3*(36+165*r**2+100*r**4+5*r**6))/(-1
            +y)-(3*r**3*(-200-290*r**2-10*r**4-5*r**6+r**8)*y)/5-(3*r**3*(-48
            -12*r**2-4*r**4+r**6)*y**2)/2+2*r*(2+9*r**2)*y**3+3*r*y**4-(3*r*(-1
            +r**2)*y**5)/5-(r*y**6)/10)
    if (dEl == 1 and dr == 1 and i==2 and j == 0):
        return     (-12*r**3*(-12-30*r**2-4*r**4+r**6)+(48*r**9)/(-1+y)**5+(36*r**7*(4
            +5*r**2))/(-1+y)**4+(16*r**5*(9+34*r**2+15*r**4))/(-1+y)**3+(8*r**3*(6
            +69*r**2+92*r**4+15*r**6))/(-1+y)**2+(192*r**3*(1+4*r**2+2*r**4))/(
            -1+y)-32*r**3*(-3-3*r**2+r**4)*y-24*r**3*(-2+r**2)*y**2+4*r*y**4)
    if (dEl == 1 and dr == 1 and i==2 and j == 1):
        return     ((12*r**3*(70+250*r**2+90*r**4-5*r**6+r**8))/5+(24*r**11)/(-1+y)**6
            +(24*(25*r**9+18*r**11))/(5*(-1+y)**5)+(6*r**7*(36+77*r**2+18*r**4))/(
            -1+y)**4+(2*r**5*(84+436*r**2+325*r**4+24*r**6))/(-1+y)**3+(4*r**3*(12
            +177*r**2+330*r**4+95*r**6))/(-1+y)**2+(4*r**3*(54+285*r**2+220*r**4
            +15*r**6))/(-1+y)+2*r**3*(60+114*r**2-4*r**4+r**6)*y-8*r**3*(-9-3*r**2
            +r**4)*y**2-12*r**3*(-2+r**2)*y**3-4*r*(-1+r**2)*y**4+(2*r*y**5)/5)
    if (dEl == 1 and dr == 1 and i==2 and j == 2):
        return     ((-3*r**3*(-320-1560*r**2-1020*r**4-35*r**6-6*r**8+r**10))/5+(12*r**13)/(
            -1+y)**7+(42*r**11*(2+r**2))/(-1+y)**6+(12*(95*r**9+134*r**11+21*r**13))/(5*(
            -1+y)**5)+(3*r**7*(100+313*r**2+150*r**4+7*r**6))/(-1+y)**4+(8*r**5*(24
            +164*r**2+184*r**4+33*r**6))/(-1+y)**3+(4*r**3*(12+222*r**2+560*r**4
            +265*r**6+12*r**8))/(-1+y)**2+(16*r**3*(15+102*r**2+115*r**4+20*r**6))/(
            -1+y)+48*r**3*(3+9*r**2+2*r**4)*y+2*r**3*(48+60*r**2-4*r**4+r**6)*y**2
            +48*r**3*y**3-3*r*(-1-2*r**2+r**4)*y**4-(8*r*(-1+r**2)*y**5)/5)
    if (dEl == 1 and dr == 1 and i==3 and j == 0):
        return     ((-48*r**3*(-20-110*r**2-90*r**4-5*r**6+r**8))/5+(48*r**11)/(-1+y)**6
            +(192*(5*r**9+6*r**11))/(5*(-1+y)**5)+(12*r**7*(24+77*r**2+36*r**4))/(
            -1+y)**4+(12*r**5*(16+116*r**2+145*r**4+32*r**6))/(-1+y)**3+(24*r**3*(2
            +39*r**2+110*r**4+65*r**6+6*r**8))/(-1+y)**2+(120*r**3*(2+15*r**2
            +20*r**4+5*r**6))/(-1+y)-36*r**3*(-4-14*r**2-4*r**4+r**6)*y-48*r**3*(
            -2-3*r**2+r**4)*y**2-24*r**3*(-2+r**2)*y**3+(12*r*y**5)/5)
    if (dEl == 1 and dr == 1 and i==3 and j == 1):
        return     ((4*r**3*(270+1905*r**2+2260*r**4+405*r**6-12*r**8+2*r**10))/5+(24*r**13)/(
            -1+y)**7+(16*r**11*(9+7*r**2))/(-1+y)**6+(48*(35*r**9+73*r**11+21*r**13))/(5*(
            -1+y)**5)+(6*(320*r**7+1405*r**9+1122*r**11+140*r**13))/(5*(-1+y)**4)
            +(4*r**5*(54+494*r**2+846*r**4+315*r**6+14*r**8))/(-1+y)**3+(6*r**3*(8
            +190*r**2+684*r**4+565*r**6+92*r**8))/(-1+y)**2+(24*r**3*(11+102*r**2
            +180*r**4+70*r**6+3*r**8))/(-1+y)+(12*r**3*(70+340*r**2+210*r**4
            -5*r**6+r**8)*y)/5-6*r**3*(-20-54*r**2-4*r**4+r**6)*y**2-8*r**3*(-9
            -6*r**2+2*r**4)*y**3-12*r**3*(-2+r**2)*y**4-(12*r*(-1+r**2)*y**5)/5+(2*r*y**6)/5)
    if (dEl == 1 and dr == 1 and i==4 and j == 0):
        return     (-8*r**3*(-30-285*r**2-520*r**4-210*r**6-6*r**8+r**10)+(48*r**13)/(-1
            +y)**7+(40*r**11*(6+7*r**2))/(-1+y)**6+(96*(25*r**9+73*r**11+35*r**13))/(5*(
            -1+y)**5)+(24*(100*r**7+585*r**9+702*r**11+175*r**13))/(5*(-1+y)**4)
            +(16*r**5*(15+176*r**2+423*r**4+264*r**6+35*r**8))/(-1+y)**3+(24*r**3*(2
            +59*r**2+284*r**4+355*r**6+118*r**8+7*r**10))/(-1+y)**2+(288*r**3*(1
            +12*r**2+30*r**4+20*r**6+3*r**8))/(-1+y)-(192*r**3*(-5-35*r**2-40*r**4
            -5*r**6+r**8)*y)/5-72*r**3*(-2-9*r**2-4*r**4+r**6)*y**2-32*r**3*(-3
            -6*r**2+2*r**4)*y**3-24*r**3*(-2+r**2)*y**4+(8*r*y**6)/5)
    return 0. 

@jit(float64(int64, int64, float64, float64, int64, int64), cache=True, nopython=True)
def Xpi(i,j,elcuthat,r,dEl,dr):
    """ Tree level functions (mupi) for mix moments and their derivatives """
    y  = 2*elcuthat
    logy = math.log((1-y)/r**2)
    # tree level function
    if (dEl == 0 and dr == 0 and i==0 and j == 0):
        return     (-6*logy*r**4+(-1+8*r**2-2*r**4-4*r**6+r**8)/2-(2*r**6)/(-1+y)**4
            -(2*(3*r**4+10*r**6))/(3*(-1+y)**3)+(-21*r**4-20*r**6)/(3*(-1+y)**2)
            -(6*r**4)/(-1+y)+(4*r**4*(-3+r**2)*y)/3+(5*y**4)/6)
    if (dEl == 0 and dr == 0 and i==0 and j == 1):
        return     (-1/3*(r**4*(18+2*r**2+r**4))-r**8/(-1+y)**5+(-12*r**6-11*r**8)/(4*(
            -1+y)**4)+(-24*r**4-120*r**6-25*r**8)/(12*(-1+y)**3)-(3*(5*r**4+8*r**6))/(2*(
            -1+y)**2)+(-69*r**4-34*r**6)/(6*(-1+y))-(r**4*(30-4*r**2+3*r**4)*y)/12
            +(r**4*(-3+r**2)*y**2)/3+((-1-2*r**2+3*r**4)*y**3)/6+y**4/3+y**5/12)
    if (dEl == 0 and dr == 0 and i==0 and j == 2):
        return     ((logy*r**4*(27+7*r**2))/3+(47-585*r**2-3240*r**4+660*r**6-435*r**8
            +162*r**10-20*r**12)/180-r**10/(2*(-1+y)**6)+(-25*r**8-12*r**10)/(10*(
            -1+y)**5)+(-32*r**6-59*r**8-6*r**10)/(8*(-1+y)**4)+(-24*r**4-168*r**6
            -91*r**8)/(12*(-1+y)**3)+(-192*r**4-496*r**6-73*r**8)/(24*(-1+y)**2)
            +(-17*r**4-19*r**6)/(-1+y)+(r**4*(6-4*r**2+r**4)*y)/3-(r**4*(12-4*r**2
            +3*r**4)*y**2)/24-((3-3*r**2+2*r**6)*y**3)/9-((-1+r**2)*y**4)/6+y**5/15
            +y**6/72)
    if (dEl == 0 and dr == 0 and i==0 and j == 3):
        return     (logy*r**4*(21+13*r**2)+(316-4788*r**2-33810*r**4+5670*r**6-3675*r**8
            +2457*r**10-742*r**12+90*r**14)/840-r**12/(4*(-1+y)**7)+(-42*r**10
            -13*r**12)/(24*(-1+y)**6)+(-540*r**8-564*r**10-35*r**12)/(120*(-1+y)**5)
            +(-400*r**6-1155*r**8-342*r**10)/(80*(-1+y)**4)+(-96*r**4-896*r**6
            -897*r**8-72*r**10)/(48*(-1+y)**3)+(-408*r**4-1580*r**6-639*r**8)/(48*(
            -1+y)**2)+(-360*r**4-708*r**6-71*r**8)/(16*(-1+y))+(r**4*(1140-170*r**2
            +165*r**4-48*r**6+5*r**8)*y)/120+(r**4*(6-4*r**2+r**4)*y**2)/4+((-18
            +48*r**2-42*r**4+4*r**6+3*r**8)*y**3)/48-((-1+3*r**2)*y**4)/24-((-1
            +r**2)*y**5)/20+y**6/60+y**7/336)
    if (dEl == 0 and dr == 0 and i==0 and j == 4):
        return     ((logy*r**4*(288+328*r**2+23*r**4))/8+(2749-50176*r**2-504840*r**4-22960*r**6
            -9380*r**8+36792*r**10-17766*r**12+4680*r**14-525*r**16)/6720-r**14/(8*(
            -1+y)**8)+(-9*r**12-2*r**14)/(8*(-1+y)**7)+(-1344*r**10-945*r**12
            -40*r**14)/(336*(-1+y)**6)+(-280*r**8-480*r**10-93*r**12)/(40*(-1+y)**5)
            +(-2880*r**6-11760*r**8-6756*r**10-353*r**12)/(480*(-1+y)**4)+(-240*r**4
            -2880*r**6-4555*r**8-1038*r**10)/(120*(-1+y)**3)+(-144*r**4-784*r**6
            -599*r**8-36*r**10)/(16*(-1+y)**2)+(-672*r**4-2056*r**6-591*r**8)/(24*(
            -1+y))-(r**4*(-8400-2240*r**2-945*r**4+483*r**6-133*r**8+15*r**10)*y)/420
            +(r**4*(600-260*r**2+165*r**4-48*r**6+5*r**8)*y**2)/120-((2-8*r**2
            +6*r**4-4*r**6+r**8)*y**3)/6-((5-12*r**2+9*r**4)*y**4)/96-((-1+3*r**2)*y**5)/60
            -((-1+r**2)*y**6)/60+y**7/210+y**8/1344)
    if (dEl == 0 and dr == 0 and i==1 and j == 0):
        return     (-9*logy*r**4*(1+r**2)+(3*(-1+15*r**2+60*r**4-60*r**6-5*r**8+r**10))/20
            -(2*r**8)/(-1+y)**5+(-8*r**6-17*r**8)/(2*(-1+y)**4)-(2*(3*r**4+25*r**6
            +20*r**8))/(3*(-1+y)**3)+(-24*r**4-74*r**6-25*r**8)/(3*(-1+y)**2)-(9*(r**4
            +r**6))/(-1+y)+(2*r**4*(-12-4*r**2+r**4)*y)/3+(r**4*(-3+r**2)*y**2)/2
            +(5*(-1+r**2)*y**4)/12+(2*y**5)/5)
    if (dEl == 0 and dr == 0 and i==1 and j == 1):
        return     (-1/15*(r**4*(120+170*r**2+15*r**4+3*r**6))-r**10/(-1+y)**6+(-20*r**8
            -19*r**10)/(5*(-1+y)**5)+(-50*r**6-165*r**8-51*r**10)/(10*(-1+y)**4)
            +(-12*r**4-128*r**6-157*r**8-15*r**10)/(6*(-1+y)**3)+(-51*r**4-217*r**6
            -118*r**8)/(6*(-1+y)**2)+(-87*r**4-187*r**6-42*r**8)/(6*(-1+y))-(r**4*(105
            +85*r**2+5*r**4+3*r**6)*y)/30-(r**4*(3+2*r**2)*y**2)/3+(r**4*(-3+r**2)*y**3)/6
            +((-1+r**2)*y**4)/3+(2*y**5)/15+y**6/15)
    if (dEl == 0 and dr == 0 and i==1 and j == 2):
        return     ((logy*r**4*(72+137*r**2+17*r**4))/6+(446-9604*r**2-177660*r**4-125370*r**6
            +13125*r**8-4095*r**10+1330*r**12-110*r**14)/5040-r**12/(2*(-1+y)**7)
            +(-12*r**10-7*r**12)/(4*(-1+y)**6)+(-65*r**8-117*r**10-21*r**12)/(10*(
            -1+y)**5)+(-240*r**6-1100*r**8-692*r**10-35*r**12)/(40*(-1+y)**4)
            +(-240*r**4-3200*r**6-5695*r**8-1429*r**10)/(120*(-1+y)**3)+(-432*r**4
            -2472*r**6-2169*r**8-175*r**10)/(48*(-1+y)**2)+(-120*r**4-403*r**6
            -178*r**8)/(6*(-1+y))+(4*r**4*(15+10*r**2-5*r**4+r**6)*y)/15-(r**4*(
            -80+100*r**2-5*r**4+r**6)*y**2)/80-(r**6*(4+r**2)*y**3)/18-((13-23*r**2
            +16*r**4)*y**4)/48-((-37+5*r**2)*y**6)/720+y**7/70)
    if (dEl == 0 and dr == 0 and i==1 and j == 3):
        return     (logy*r**4*(27+70*r**2+20*r**4)+(115-2920*r**2-62706*r**4-58380*r**6
            +9555*r**8-798*r**10+665*r**12-164*r**14+15*r**16)/840-r**14/(4*(-1
            +y)**8)+(-56*r**12-23*r**14)/(28*(-1+y)**7)+(-525*r**10-623*r**12-76*r**14)/(84*(
            -1+y)**6)+(-285*r**8-768*r**10-308*r**12-10*r**14)/(30*(-1+y)**5)
            +(-560*r**6-3365*r**8-3381*r**10-518*r**12)/(80*(-1+y)**4)+(-480*r**4
            -7840*r**6-19235*r**8-8949*r**10-420*r**12)/(240*(-1+y)**3)+(-2280*r**4
            -17020*r**6-22475*r**8-4737*r**10)/(240*(-1+y)**2)+(-408*r**4-1940*r**6
            -1415*r**8-85*r**10)/(16*(-1+y))+(r**4*(6090+8645*r**2-350*r**4+357*r**6
            -77*r**8+5*r**10)*y)/420+(r**4*(90+10*r**2-5*r**4+r**6)*y**2)/20+(r**4*(360
            -220*r**2+25*r**4+3*r**6)*y**3)/240-((5-14*r**2+9*r**4)*y**4)/24-((3
            -5*r**2+4*r**4)*y**5)/40-((-1+r**2)*y**6)/60-((-29+5*r**2)*y**7)/1680
            +y**8/280)
    if (dEl == 0 and dr == 0 and i==2 and j == 0):
        return     (-4*logy*r**4*(3+8*r**2+3*r**4)+(-1+24*r**2+300*r**4-20*r**6-255*r**8
            -6*r**10+r**12)/15-(2*r**10)/(-1+y)**6-(2*(15*r**8+26*r**10))/(5*(-1
            +y)**5)+(-60*r**6-305*r**8-216*r**10)/(10*(-1+y)**4)+(-6*r**4-88*r**6
            -183*r**8-66*r**10)/(3*(-1+y)**3)+(-18*r**4-108*r**6-113*r**8-20*r**10)/(2*(
            -1+y)**2)-(4*(3*r**4+8*r**6+3*r**8))/(-1+y)+(2*r**4*(-30-50*r**2
            -5*r**4+r**6)*y)/5+(r**4*(-6-4*r**2+r**4)*y**2)/2+(2*(-1+r**2)*y**5)/5
            +(7*y**6)/30)
    if (dEl == 0 and dr == 0 and i==2 and j == 1):
        return     ((-2*r**4*(75+255*r**2+140*r**4+9*r**6+r**8))/15-r**12/(-1+y)**7+(
            -30*r**10-29*r**12)/(6*(-1+y)**6)+(-90*r**8-252*r**10-91*r**12)/(10*(
            -1+y)**5)+(-140*r**6-920*r**8-1024*r**10-161*r**12)/(20*(-1+y)**4)
            +(-120*r**4-2120*r**6-5760*r**8-3182*r**10-175*r**12)/(60*(-1+y)**3)
            +(-114*r**4-872*r**6-1259*r**8-350*r**10)/(12*(-1+y)**2)-(5*(42*r**4
            +188*r**6+155*r**8+20*r**10))/(12*(-1+y))-(r**4*(90+220*r**2+75*r**4
            +6*r**6+r**8)*y)/20-(r**4*(30+70*r**2+25*r**4+r**6)*y**2)/30+(r**4*(6
            -12*r**2+r**4)*y**3)/12-((17-22*r**2+5*r**4)*y**5)/60+((1+r**2)*y**6)/30
            +y**7/20)
    if (dEl == 0 and dr == 0 and i==2 and j == 2):
        return     ((5*logy*r**4*(18+76*r**2+55*r**4+4*r**6))/6+(215-7000*r**2-263620*r**4
            -587160*r**6-144270*r**8+14952*r**10-1904*r**12+592*r**14-35*r**16)/5040
            -r**14/(2*(-1+y)**8)+(-49*r**12-32*r**14)/(14*(-1+y)**7)+(-114*r**10
            -205*r**12-48*r**14)/(12*(-1+y)**6)+(-125*r**8-486*r**10-334*r**12
            -32*r**14)/(10*(-1+y)**5)+(-320*r**6-2625*r**8-4116*r**10-1322*r**12
            -40*r**14)/(40*(-1+y)**4)+(-60*r**4-1260*r**6-4380*r**8-3544*r**10
            -515*r**12)/(30*(-1+y)**3)+(-600*r**4-5740*r**6-11230*r**8-5008*r**10
            -255*r**12)/(60*(-1+y)**2)+(-138*r**4-868*r**6-1075*r**8-256*r**10)/(6*(
            -1+y))+(r**4*(30+80*r**2+15*r**4-6*r**6+r**8)*y)/5+(r**4*(25+20*r**2
            -20*r**4+r**6)*y**2)/10-(r**4*(-90+40*r**2+10*r**4+r**6)*y**3)/45-(r**4*(
            -3+2*r**2)*y**4)/4-((-1+r**2)**2*y**5)/5-((11-16*r**2+5*r**4)*y**6)/180
            +y**7/35+y**8/80)
    if (dEl == 0 and dr == 0 and i==3 and j == 0):
        return     (-15*logy*r**4*(1+5*r**2+5*r**4+r**6)+(-1+35*r**2+903*r**4+1155*r**6
            -1190*r**8-714*r**10-7*r**12+r**14)/28-(2*r**12)/(-1+y)**7+(-24*r**10
            -37*r**12)/(3*(-1+y)**6)-(2*(90*r**8+363*r**10+238*r**12))/(15*(-1+y)**5)
            +(-80*r**6-705*r**8-1212*r**10-434*r**12)/(10*(-1+y)**4)+(-60*r**4
            -1340*r**6-5055*r**8-4749*r**10-980*r**12)/(30*(-1+y)**3)-(5*(24*r**4
            +232*r**6+477*r**8+255*r**10+28*r**12))/(12*(-1+y)**2)-(15*(r**4+5*r**6
            +5*r**8+r**10))/(-1+y)+(4*r**4*(-60-220*r**2-135*r**4-6*r**6+r**8)*y)/15
            +(3*r**4*(-30-70*r**2-15*r**4+3*r**6)*y**2)/20-(5*r**4*(-3+r**2)*y**4)/12
            +(7*(-1+r**2)*y**6)/20+(16*y**7)/105)
    if (dEl == 0 and dr == 0 and i==3 and j == 1):
        return     ((-2*r**4*(126+763*r**2+980*r**4+294*r**6+14*r**8+r**10))/21-r**14/(
            -1+y)**8+(-42*r**12-41*r**14)/(7*(-1+y)**7)-(2*(147*r**10+378*r**12
            +148*r**14))/(21*(-1+y)**6)+(-80*r**8-423*r**10-451*r**12-88*r**14)/(5*(
            -1+y)**5)+(-540*r**6-5745*r**8-12885*r**10-7274*r**12-696*r**14)/(60*(
            -1+y)**4)+(-120*r**4-3120*r**6-14405*r**8-17751*r**10-5592*r**12-200*r**14)/(60*(
            -1+y)**3)+(-210*r**4-2470*r**6-6515*r**8-4779*r**10-810*r**12)/(20*(
            -1+y)**2)+(-246*r**4-1838*r**6-3105*r**8-1383*r**10-116*r**12)/(12*(
            -1+y))-(r**4*(1155+5495*r**2+5145*r**4+1029*r**6+70*r**8+6*r**10)*y)/210
            -(r**4*(30+140*r**2+135*r**4+30*r**6+r**8)*y**2)/30+(r**4*(90+50*r**2
            -95*r**4+3*r**6)*y**3)/60-(2*r**4*(-3+r**2)*y**4)/3-(r**4*(-3+r**2)*y**5)/6
            -((7-10*r**2+3*r**4)*y**6)/30+((-5+21*r**2)*y**7)/420+(4*y**8)/105)
    if (dEl == 0 and dr == 0 and i==4 and j == 0):
        return     (-18*logy*r**4*(1+8*r**2+15*r**4+8*r**6+r**8)+(3*(-1+48*r**2+2128*r**4
            +6664*r**6-280*r**8-6160*r**10-1610*r**12-8*r**14+r**16))/140-(2*r**14)/(
            -1+y)**8-(10*(7*r**12+10*r**14))/(7*(-1+y)**7)+(-420*r**10-1477*r**12
            -920*r**14)/(21*(-1+y)**6)-(2*(150*r**8+1032*r**10+1585*r**12+560*r**14))/(15*(
            -1+y)**5)+(-100*r**6-1330*r**8-4008*r**10-3497*r**12-760*r**14)/(10*(
            -1+y)**4)-(2*(15*r**4+470*r**6+2755*r**8+4728*r**10+2537*r**12+340*r**14))/(15*(
            -1+y)**3)+(-33*r**4-464*r**6-1555*r**8-1632*r**10-533*r**12-40*r**14)/(3*(
            -1+y)**2)-(18*(r**4+8*r**6+15*r**8+8*r**10+r**12))/(-1+y)+(4*r**4*(
            -105-665*r**2-910*r**4-294*r**6-7*r**8+r**10)*y)/21+(2*r**4*(-15-70*r**2
            -60*r**4-6*r**6+r**8)*y**2)/5-(5*r**4*(-3-4*r**2+r**4)*y**4)/6-(4*r**4*(
            -3+r**2)*y**5)/5+(32*(-1+r**2)*y**7)/105+(3*y**8)/28)

    # derivative wrt Elcuthat
    if (dEl == 1 and dr == 0 and i==0 and j == 0):
        return     ((8*r**4*(-3+r**2))/3+(16*r**6)/(-1+y)**5+(4*(3*r**4+10*r**6))/(-1+y)**4
            +(4*(21*r**4+20*r**6))/(3*(-1+y)**3)+(12*r**4)/(-1+y)**2-(12*r**4)/(
            -1+y)+(20*y**3)/3)
    if (dEl == 1 and dr == 0 and i==0 and j == 1):
        return     (-1/6*(r**4*(30-4*r**2+3*r**4))+(10*r**8)/(-1+y)**6+(2*(12*r**6+11*r**8))/(
            -1+y)**5+(24*r**4+120*r**6+25*r**8)/(2*(-1+y)**4)+(6*(5*r**4+8*r**6))/(
            -1+y)**3+(69*r**4+34*r**6)/(3*(-1+y)**2)+(4*r**4*(-3+r**2)*y)/3+(
            -1-2*r**2+3*r**4)*y**2+(8*y**3)/3+(5*y**4)/6)
    if (dEl == 1 and dr == 0 and i==0 and j == 2):
        return     ((2*r**4*(6-4*r**2+r**4))/3+(6*r**10)/(-1+y)**7+(25*r**8+12*r**10)/(
            -1+y)**6+(32*r**6+59*r**8+6*r**10)/(-1+y)**5+(24*r**4+168*r**6+91*r**8)/(2*(
            -1+y)**4)+(192*r**4+496*r**6+73*r**8)/(6*(-1+y)**3)+(2*(17*r**4+19*r**6))/(
            -1+y)**2+(2*(27*r**4+7*r**6))/(3*(-1+y))-(r**4*(12-4*r**2+3*r**4)*y)/6
            -(2*(3-3*r**2+2*r**6)*y**2)/3-(4*(-1+r**2)*y**3)/3+(2*y**4)/3+y**5/6)
    if (dEl == 1 and dr == 0 and i==0 and j == 3):
        return     ((r**4*(1140-170*r**2+165*r**4-48*r**6+5*r**8))/60+(7*r**12)/(2*(-1+y)**8)
            +(42*r**10+13*r**12)/(2*(-1+y)**7)+(540*r**8+564*r**10+35*r**12)/(12*(
            -1+y)**6)+(400*r**6+1155*r**8+342*r**10)/(10*(-1+y)**5)+(96*r**4+896*r**6
            +897*r**8+72*r**10)/(8*(-1+y)**4)+(408*r**4+1580*r**6+639*r**8)/(12*(
            -1+y)**3)+(360*r**4+708*r**6+71*r**8)/(8*(-1+y)**2)+(2*r**4*(21+13*r**2))/(
            -1+y)+r**4*(6-4*r**2+r**4)*y+((-18+48*r**2-42*r**4+4*r**6+3*r**8)*y**2)/8
            -((-1+3*r**2)*y**3)/3-((-1+r**2)*y**4)/2+y**5/5+y**6/24)
    if (dEl == 1 and dr == 0 and i==0 and j == 4):
        return     (-1/210*(r**4*(-8400-2240*r**2-945*r**4+483*r**6-133*r**8+15*r**10))
            +(2*r**14)/(-1+y)**9+(7*(9*r**12+2*r**14))/(4*(-1+y)**8)+(1344*r**10
            +945*r**12+40*r**14)/(28*(-1+y)**7)+(280*r**8+480*r**10+93*r**12)/(4*(
            -1+y)**6)+(2880*r**6+11760*r**8+6756*r**10+353*r**12)/(60*(-1+y)**5)
            +(240*r**4+2880*r**6+4555*r**8+1038*r**10)/(20*(-1+y)**4)+(144*r**4
            +784*r**6+599*r**8+36*r**10)/(4*(-1+y)**3)+(672*r**4+2056*r**6+591*r**8)/(12*(
            -1+y)**2)+(288*r**4+328*r**6+23*r**8)/(4*(-1+y))+(r**4*(600-260*r**2
            +165*r**4-48*r**6+5*r**8)*y)/30-(2-8*r**2+6*r**4-4*r**6+r**8)*y**2
            -((5-12*r**2+9*r**4)*y**3)/12-((-1+3*r**2)*y**4)/6-((-1+r**2)*y**5)/5
            +y**6/15+y**7/84)
    if (dEl == 1 and dr == 0 and i==1 and j == 0):
        return     ((4*r**4*(-12-4*r**2+r**4))/3+(20*r**8)/(-1+y)**6+(4*(8*r**6+17*r**8))/(
            -1+y)**5+(4*(3*r**4+25*r**6+20*r**8))/(-1+y)**4+(4*(24*r**4+74*r**6
            +25*r**8))/(3*(-1+y)**3)+(18*(r**4+r**6))/(-1+y)**2-(18*(r**4+r**6))/(
            -1+y)+2*r**4*(-3+r**2)*y+(10*(-1+r**2)*y**3)/3+4*y**4)
    if (dEl == 1 and dr == 0 and i==1 and j == 1):
        return     (-1/15*(r**4*(105+85*r**2+5*r**4+3*r**6))+(12*r**10)/(-1+y)**7+(2*(20*r**8
            +19*r**10))/(-1+y)**6+(4*(50*r**6+165*r**8+51*r**10))/(5*(-1+y)**5)
            +(12*r**4+128*r**6+157*r**8+15*r**10)/(-1+y)**4+(2*(51*r**4+217*r**6
            +118*r**8))/(3*(-1+y)**3)+(87*r**4+187*r**6+42*r**8)/(3*(-1+y)**2)
            -(4*r**4*(3+2*r**2)*y)/3+r**4*(-3+r**2)*y**2+(8*(-1+r**2)*y**3)/3+(4*y**4)/3
            +(4*y**5)/5)
    if (dEl == 1 and dr == 0 and i==1 and j == 2):
        return     ((8*r**4*(15+10*r**2-5*r**4+r**6))/15+(7*r**12)/(-1+y)**8+(3*(12*r**10
            +7*r**12))/(-1+y)**7+(65*r**8+117*r**10+21*r**12)/(-1+y)**6+(240*r**6
            +1100*r**8+692*r**10+35*r**12)/(5*(-1+y)**5)+(240*r**4+3200*r**6+5695*r**8
            +1429*r**10)/(20*(-1+y)**4)+(432*r**4+2472*r**6+2169*r**8+175*r**10)/(12*(
            -1+y)**3)+(120*r**4+403*r**6+178*r**8)/(3*(-1+y)**2)+(72*r**4+137*r**6
            +17*r**8)/(3*(-1+y))-(r**4*(-80+100*r**2-5*r**4+r**6)*y)/20-(r**6*(4
            +r**2)*y**2)/3-((13-23*r**2+16*r**4)*y**3)/6-((-37+5*r**2)*y**5)/60
            +y**6/5)
    if (dEl == 1 and dr == 0 and i==1 and j == 3):
        return     ((r**4*(6090+8645*r**2-350*r**4+357*r**6-77*r**8+5*r**10))/210+(4*r**14)/(
            -1+y)**9+(56*r**12+23*r**14)/(2*(-1+y)**8)+(525*r**10+623*r**12+76*r**14)/(7*(
            -1+y)**7)+(285*r**8+768*r**10+308*r**12+10*r**14)/(3*(-1+y)**6)+(560*r**6
            +3365*r**8+3381*r**10+518*r**12)/(10*(-1+y)**5)+(480*r**4+7840*r**6
            +19235*r**8+8949*r**10+420*r**12)/(40*(-1+y)**4)+(2280*r**4+17020*r**6
            +22475*r**8+4737*r**10)/(60*(-1+y)**3)+(408*r**4+1940*r**6+1415*r**8
            +85*r**10)/(8*(-1+y)**2)+(2*(27*r**4+70*r**6+20*r**8))/(-1+y)+(r**4*(90
            +10*r**2-5*r**4+r**6)*y)/5+(r**4*(360-220*r**2+25*r**4+3*r**6)*y**2)/40
            -((5-14*r**2+9*r**4)*y**3)/3-((3-5*r**2+4*r**4)*y**4)/4-((-1+r**2)*y**5)/5
            -((-29+5*r**2)*y**6)/120+(2*y**7)/35)
    if (dEl == 1 and dr == 0 and i==2 and j == 0):
        return     ((4*r**4*(-30-50*r**2-5*r**4+r**6))/5+(24*r**10)/(-1+y)**7+(4*(15*r**8
            +26*r**10))/(-1+y)**6+(4*(60*r**6+305*r**8+216*r**10))/(5*(-1+y)**5)
            +(2*(6*r**4+88*r**6+183*r**8+66*r**10))/(-1+y)**4+(2*(18*r**4+108*r**6
            +113*r**8+20*r**10))/(-1+y)**3+(8*(3*r**4+8*r**6+3*r**8))/(-1+y)**2
            -(8*(3*r**4+8*r**6+3*r**8))/(-1+y)+2*r**4*(-6-4*r**2+r**4)*y+4*(
            -1+r**2)*y**4+(14*y**5)/5)
    if (dEl == 1 and dr == 0 and i==2 and j == 1):
        return     (-1/10*(r**4*(90+220*r**2+75*r**4+6*r**6+r**8))+(14*r**12)/(-1+y)**8
            +(2*(30*r**10+29*r**12))/(-1+y)**7+(90*r**8+252*r**10+91*r**12)/(-1
            +y)**6+(2*(140*r**6+920*r**8+1024*r**10+161*r**12))/(5*(-1+y)**5)+(120*r**4
            +2120*r**6+5760*r**8+3182*r**10+175*r**12)/(10*(-1+y)**4)+(114*r**4
            +872*r**6+1259*r**8+350*r**10)/(3*(-1+y)**3)+(5*(42*r**4+188*r**6+155*r**8
            +20*r**10))/(6*(-1+y)**2)-(2*r**4*(30+70*r**2+25*r**4+r**6)*y)/15+(r**4*(6
            -12*r**2+r**4)*y**2)/2-((17-22*r**2+5*r**4)*y**4)/6+(2*(1+r**2)*y**5)/5
            +(7*y**6)/10)
    if (dEl == 1 and dr == 0 and i==2 and j == 2):
        return     ((2*r**4*(30+80*r**2+15*r**4-6*r**6+r**8))/5+(8*r**14)/(-1+y)**9+(49*r**12
            +32*r**14)/(-1+y)**8+(114*r**10+205*r**12+48*r**14)/(-1+y)**7+(125*r**8
            +486*r**10+334*r**12+32*r**14)/(-1+y)**6+(320*r**6+2625*r**8+4116*r**10
            +1322*r**12+40*r**14)/(5*(-1+y)**5)+(60*r**4+1260*r**6+4380*r**8+3544*r**10
            +515*r**12)/(5*(-1+y)**4)+(600*r**4+5740*r**6+11230*r**8+5008*r**10
            +255*r**12)/(15*(-1+y)**3)+(138*r**4+868*r**6+1075*r**8+256*r**10)/(3*(
            -1+y)**2)+(5*(18*r**4+76*r**6+55*r**8+4*r**10))/(3*(-1+y))+(2*r**4*(25
            +20*r**2-20*r**4+r**6)*y)/5-(2*r**4*(-90+40*r**2+10*r**4+r**6)*y**2)/15
            -2*r**4*(-3+2*r**2)*y**3-2*(-1+r**2)**2*y**4-((11-16*r**2+5*r**4)*y**5)/15
            +(2*y**6)/5+y**7/5)
    if (dEl == 1 and dr == 0 and i==3 and j == 0):
        return     ((8*r**4*(-60-220*r**2-135*r**4-6*r**6+r**8))/15+(28*r**12)/(-1+y)**8
            +(4*(24*r**10+37*r**12))/(-1+y)**7+(4*(90*r**8+363*r**10+238*r**12))/(3*(
            -1+y)**6)+(4*(80*r**6+705*r**8+1212*r**10+434*r**12))/(5*(-1+y)**5)
            +(60*r**4+1340*r**6+5055*r**8+4749*r**10+980*r**12)/(5*(-1+y)**4)+(5*(24*r**4
            +232*r**6+477*r**8+255*r**10+28*r**12))/(3*(-1+y)**3)+(30*(r**4+5*r**6
            +5*r**8+r**10))/(-1+y)**2-(30*(r**4+5*r**6+5*r**8+r**10))/(-1+y)
            +(3*r**4*(-30-70*r**2-15*r**4+3*r**6)*y)/5-(10*r**4*(-3+r**2)*y**3)/3
            +(21*(-1+r**2)*y**5)/5+(32*y**6)/15)
    if (dEl == 1 and dr == 0 and i==3 and j == 1):
        return     (-1/105*(r**4*(1155+5495*r**2+5145*r**4+1029*r**6+70*r**8+6*r**10))+(16*r**14)/(
            -1+y)**9+(2*(42*r**12+41*r**14))/(-1+y)**8+(8*(147*r**10+378*r**12
            +148*r**14))/(7*(-1+y)**7)+(2*(80*r**8+423*r**10+451*r**12+88*r**14))/(
            -1+y)**6+(2*(540*r**6+5745*r**8+12885*r**10+7274*r**12+696*r**14))/(15*(
            -1+y)**5)+(120*r**4+3120*r**6+14405*r**8+17751*r**10+5592*r**12+200*r**14)/(10*(
            -1+y)**4)+(210*r**4+2470*r**6+6515*r**8+4779*r**10+810*r**12)/(5*(
            -1+y)**3)+(246*r**4+1838*r**6+3105*r**8+1383*r**10+116*r**12)/(6*(
            -1+y)**2)-(2*r**4*(30+140*r**2+135*r**4+30*r**6+r**8)*y)/15+(r**4*(90
            +50*r**2-95*r**4+3*r**6)*y**2)/10-(16*r**4*(-3+r**2)*y**3)/3-(5*r**4*(
            -3+r**2)*y**4)/3-(2*(7-10*r**2+3*r**4)*y**5)/5+((-5+21*r**2)*y**6)/30
            +(64*y**7)/105)
    if (dEl == 1 and dr == 0 and i==4 and j == 0):
        return     ((8*r**4*(-105-665*r**2-910*r**4-294*r**6-7*r**8+r**10))/21+(32*r**14)/(
            -1+y)**9+(20*(7*r**12+10*r**14))/(-1+y)**8+(4*(420*r**10+1477*r**12
            +920*r**14))/(7*(-1+y)**7)+(4*(150*r**8+1032*r**10+1585*r**12+560*r**14))/(3*(
            -1+y)**6)+(4*(100*r**6+1330*r**8+4008*r**10+3497*r**12+760*r**14))/(5*(
            -1+y)**5)+(4*(15*r**4+470*r**6+2755*r**8+4728*r**10+2537*r**12+340*r**14))/(5*(
            -1+y)**4)+(4*(33*r**4+464*r**6+1555*r**8+1632*r**10+533*r**12+40*r**14))/(3*(
            -1+y)**3)+(36*(r**4+8*r**6+15*r**8+8*r**10+r**12))/(-1+y)**2-(36*(r**4
            +8*r**6+15*r**8+8*r**10+r**12))/(-1+y)+(8*r**4*(-15-70*r**2-60*r**4
            -6*r**6+r**8)*y)/5-(20*r**4*(-3-4*r**2+r**4)*y**3)/3-8*r**4*(-3+r**2)*y**4
            +(64*(-1+r**2)*y**6)/15+(12*y**7)/7)

    # derivative wrt r
    if (dEl == 0 and dr == 1 and i==0 and j == 0):
        return     (4*r*(2+2*r**2-3*r**4+r**6)-(12*r**5)/(-1+y)**4-(8*(r**3+5*r**5))/(
            -1+y)**3-(4*(7*r**3+10*r**5))/(-1+y)**2-(24*r**3)/(-1+y)+8*r**3*(
            -2+r**2)*y-24*r**3*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 1):
        return     ((-4*r**3*(18+3*r**2+2*r**4))/3-(8*r**7)/(-1+y)**5-(2*(9*r**5+11*r**7))/(
            -1+y)**4-(2*(12*r**3+90*r**5+25*r**7))/(3*(-1+y)**3)-(6*(5*r**3+12*r**5))/(
            -1+y)**2-(2*(23*r**3+17*r**5))/(-1+y)-2*r**3*(5-r**2+r**4)*y+2*r**3*(
            -2+r**2)*y**2+(2*r*(-1+3*r**2)*y**3)/3)
    if (dEl == 0 and dr == 1 and i==0 and j == 2):
        return     (-1/6*(r*(39+540*r**2-104*r**4+116*r**6-54*r**8+8*r**10))-(5*r**9)/(
            -1+y)**6-(4*(5*r**7+3*r**9))/(-1+y)**5+(-48*r**5-118*r**7-15*r**9)/(2*(
            -1+y)**4)-(2*(12*r**3+126*r**5+91*r**7))/(3*(-1+y)**3)+(-96*r**3-372*r**5
            -73*r**7)/(3*(-1+y)**2)-(2*(34*r**3+57*r**5))/(-1+y)+(8*r**3*(3-3*r**2
            +r**4)*y)/3-r**3*(2-r**2+r**4)*y**2-(2*r*(-1+2*r**4)*y**3)/3-(r*y**4)/3
            +2*r**3*(18+7*r**2)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 3):
        return     ((r*(-228-4060*r**2+290*r**4-700*r**6+585*r**8-212*r**10+30*r**12))/20
            -(3*r**11)/(-1+y)**7+(-35*r**9-13*r**11)/(2*(-1+y)**6)+(-72*r**7-94*r**9
            -7*r**11)/(2*(-1+y)**5)-(3*(40*r**5+154*r**7+57*r**9))/(4*(-1+y)**4)
            +(-16*r**3-224*r**5-299*r**7-30*r**9)/(2*(-1+y)**3)+(-68*r**3-395*r**5
            -213*r**7)/(2*(-1+y)**2)+(-180*r**3-531*r**5-71*r**7)/(2*(-1+y))+(r**3*(76
            -17*r**2+22*r**4-8*r**6+r**8)*y)/2+2*r**3*(3-3*r**2+r**4)*y**2+(r*(4
            -7*r**2+r**4+r**6)*y**3)/2-(r*y**4)/4-(r*y**5)/10+6*r**3*(14+13*r**2)*math.log((1
            -y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 4):
        return     (-1/120*(r*(1792+44700*r**2+12300*r**4+2030*r**6-6570*r**8+3807*r**10
            -1170*r**12+150*r**14))-(7*r**13)/(4*(-1+y)**8)+(-27*r**11-7*r**13)/(2*(
            -1+y)**7)-(5*(96*r**9+81*r**11+4*r**13))/(12*(-1+y)**6)+(-560*r**7
            -1200*r**9-279*r**11)/(10*(-1+y)**5)+(-1440*r**5-7840*r**7-5630*r**9
            -353*r**11)/(40*(-1+y)**4)+(-48*r**3-864*r**5-1822*r**7-519*r**9)/(6*(
            -1+y)**3)+(-72*r**3-588*r**5-599*r**7-45*r**9)/(2*(-1+y)**2)+(-112*r**3
            -514*r**5-197*r**7)/(-1+y)-(r**3*(-800-320*r**2-180*r**4+115*r**6
            -38*r**8+5*r**10)*y)/10+(r**3*(40-26*r**2+22*r**4-8*r**6+r**8)*y**2)/2
            -(4*r*(-2+3*r**2-3*r**4+r**6)*y**3)/3-(r*(-2+3*r**2)*y**4)/8-(r*y**5)/10
            -(r*y**6)/30+r**3*(144+246*r**2+23*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 0):
        return     ((3*r*(3+36*r**2-24*r**4-4*r**6+r**8))/2-(16*r**7)/(-1+y)**5-(4*(6*r**5
            +17*r**7))/(-1+y)**4-(4*(6*r**3+75*r**5+80*r**7))/(3*(-1+y)**3)-(4*(24*r**3
            +111*r**5+50*r**7))/(3*(-1+y)**2)-(18*(2*r**3+3*r**5))/(-1+y)+(16*r**3*(
            -6-3*r**2+r**4)*y)/3+3*r**3*(-2+r**2)*y**2+(5*r*y**4)/6-18*r**3*(2
            +3*r**2)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 1):
        return     (-2*r**3*(16+34*r**2+4*r**4+r**6)-(10*r**9)/(-1+y)**6-(2*(16*r**7+19*r**9))/(
            -1+y)**5-(3*(10*r**5+44*r**7+17*r**9))/(-1+y)**4+(-24*r**3-384*r**5
            -628*r**7-75*r**9)/(3*(-1+y)**3)+(-102*r**3-651*r**5-472*r**7)/(3*(
            -1+y)**2)+(-58*r**3-187*r**5-56*r**7)/(-1+y)-(r**3*(42+51*r**2+4*r**4
            +3*r**6)*y)/3-4*r**3*(1+r**2)*y**2+r**3*(-2+r**2)*y**3+(2*r*y**4)/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 2):
        return     (-1/360*(r*(1372+59400*r**2+70170*r**4-5460*r**6+2925*r**8-1140*r**10
            +110*r**12))-(6*r**11)/(-1+y)**7-(3*(10*r**9+7*r**11))/(-1+y)**6+(
            -260*r**7-585*r**9-126*r**11)/(5*(-1+y)**5)+(-72*r**5-440*r**7-346*r**9
            -21*r**11)/(2*(-1+y)**4)+(-96*r**3-1920*r**5-4556*r**7-1429*r**9)/(12*(
            -1+y)**3)+(-864*r**3-7416*r**5-8676*r**7-875*r**9)/(24*(-1+y)**2)
            +(-240*r**3-1209*r**5-712*r**7)/(3*(-1+y))+(8*r**3*(6+6*r**2-4*r**4
            +r**6)*y)/3-(r**3*(-32+60*r**2-4*r**4+r**6)*y**2)/8-(4*r**5*(3+r**2)*y**3)/9
            -(r*(-23+32*r**2)*y**4)/24-(r*y**6)/72+(r**3*(144+411*r**2+68*r**4)*math.log((1
            -y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 3):
        return     ((r*(-1460-74046*r**2-116970*r**4+10710*r**6-1995*r**8+1995*r**10-574*r**12
            +60*r**14))/210-(7*r**13)/(2*(-1+y)**8)+(-48*r**11-23*r**13)/(2*(-1
            +y)**7)+(-375*r**9-534*r**11-76*r**13)/(6*(-1+y)**6)-(2*(570*r**7+1920*r**9
            +924*r**11+35*r**13))/(15*(-1+y)**5)+(-1680*r**5-13460*r**7-16905*r**9
            -3108*r**11)/(40*(-1+y)**4)+(-192*r**3-4704*r**5-15388*r**7-8949*r**9
            -504*r**11)/(24*(-1+y)**3)+(-912*r**3-10212*r**5-17980*r**7-4737*r**9)/(24*(
            -1+y)**2)+(-816*r**3-5820*r**5-5660*r**7-425*r**9)/(8*(-1+y))+(r**3*(1740
            +3705*r**2-200*r**4+255*r**6-66*r**8+5*r**10)*y)/30+(r**3*(36+6*r**2
            -4*r**4+r**6)*y**2)/2+(r**3*(144-132*r**2+20*r**4+3*r**6)*y**3)/24-(r*(
            -7+9*r**2)*y**4)/6-(r*(-5+8*r**2)*y**5)/20-(r*y**6)/30-(r*y**7)/168
            +4*r**3*(27+105*r**2+40*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 0):
        return     ((4*r*(4+130*r**2+70*r**4-140*r**6-5*r**8+r**10))/5-(20*r**9)/(-1+y)**6
            -(8*(6*r**7+13*r**9))/(-1+y)**5-(4*(9*r**5+61*r**7+54*r**9))/(-1+y)**4
            -(4*(2*r**3+44*r**5+122*r**7+55*r**9))/(-1+y)**3-(4*(9*r**3+81*r**5
            +113*r**7+25*r**9))/(-1+y)**2-(48*(r**3+4*r**5+2*r**7))/(-1+y)+4*r**3*(
            -12-30*r**2-4*r**4+r**6)*y+4*r**3*(-3-3*r**2+r**4)*y**2+(4*r*y**5)/5
            -48*r**3*(1+4*r**2+2*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 1):
        return     ((-4*r**3*(150+765*r**2+560*r**4+45*r**6+6*r**8))/15-(12*r**11)/(-1+y)**7
            -(2*(25*r**9+29*r**11))/(-1+y)**6-(6*(60*r**7+210*r**9+91*r**11))/(5*(
            -1+y)**5)+(-210*r**5-1840*r**7-2560*r**9-483*r**11)/(5*(-1+y)**4)
            +(-24*r**3-636*r**5-2304*r**7-1591*r**9-105*r**11)/(3*(-1+y)**3)+(
            -114*r**3-1308*r**5-2518*r**7-875*r**9)/(3*(-1+y)**2)-(10*(21*r**3+141*r**5
            +155*r**7+25*r**9))/(3*(-1+y))-(3*r**3*(30+110*r**2+50*r**4+5*r**6
            +r**8)*y)/5-(r**3*(12+42*r**2+20*r**4+r**6)*y**2)/3+(2*r**3*(3-9*r**2
            +r**4)*y**3)/3-(r*(-11+5*r**2)*y**5)/15+(r*y**6)/15)
    if (dEl == 0 and dr == 1 and i==2 and j == 2):
        return     (-1/45*(r*(125+10765*r**2+37155*r**4+14430*r**6-1035*r**8+204*r**10
            -74*r**12+5*r**14))-(7*r**13)/(-1+y)**8-(2*(21*r**11+16*r**13))/(-1
            +y)**7+(-95*r**9-205*r**11-56*r**13)/(-1+y)**6-(2*(250*r**7+1215*r**9
            +1002*r**11+112*r**13))/(5*(-1+y)**5)+(-240*r**5-2625*r**7-5145*r**9
            -1983*r**11-70*r**13)/(5*(-1+y)**4)-(2*(12*r**3+378*r**5+1752*r**7
            +1772*r**9+309*r**11))/(3*(-1+y)**3)+(-120*r**3-1722*r**5-4492*r**7
            -2504*r**9-153*r**11)/(3*(-1+y)**2)-(4*(69*r**3+651*r**5+1075*r**7
            +320*r**9))/(3*(-1+y))+(12*r**3*(10+40*r**2+10*r**4-5*r**6+r**8)*y)/5
            +r**3*(10+12*r**2-16*r**4+r**6)*y**2-(2*r**3*(-36+24*r**2+8*r**4+r**6)*y**3)/9
            -3*r**3*(-1+r**2)*y**4-(4*r*(-1+r**2)*y**5)/5-(r*(-8+5*r**2)*y**6)/45
            +(20*r**3*(9+57*r**2+55*r**4+5*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==3 and j == 0):
        return     ((r*(5+318*r**2+795*r**4-380*r**6-450*r**8-6*r**10+r**12))/2-(24*r**11)/(
            -1+y)**7-(4*(20*r**9+37*r**11))/(-1+y)**6-(4*(120*r**7+605*r**9+476*r**11))/(5*(
            -1+y)**5)-(12*(20*r**5+235*r**7+505*r**9+217*r**11))/(5*(-1+y)**4)
            +(-8*r**3-268*r**5-1348*r**7-1583*r**9-392*r**11)/(-1+y)**3-(5*(16*r**3
            +232*r**5+636*r**7+425*r**9+56*r**11))/(2*(-1+y)**2)-(30*(2*r**3+15*r**5
            +20*r**7+5*r**9))/(-1+y)+(16*r**3*(-20-110*r**2-90*r**4-5*r**6+r**8)*y)/5
            +(9*r**3*(-4-14*r**2-4*r**4+r**6)*y**2)/2-(5*r**3*(-2+r**2)*y**4)/2+(7*r*y**6)/10
            -30*r**3*(2+15*r**2+20*r**4+5*r**6)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==3 and j == 1):
        return     ((-4*r**3*(36+327*r**2+560*r**4+210*r**6+12*r**8+r**10))/3-(14*r**13)/(
            -1+y)**8-(2*(36*r**11+41*r**13))/(-1+y)**7-(4*(105*r**9+324*r**11
            +148*r**13))/(3*(-1+y)**6)-(2*(320*r**7+2115*r**9+2706*r**11+616*r**13))/(5*(
            -1+y)**5)+(-540*r**5-7660*r**7-21475*r**9-14548*r**11-1624*r**13)/(10*(
            -1+y)**4)+(-240*r**3-9360*r**5-57620*r**7-88755*r**9-33552*r**11-1400*r**13)/(30*(
            -1+y)**3)+(-84*r**3-1482*r**5-5212*r**7-4779*r**9-972*r**11)/(2*(-1
            +y)**2)+(-164*r**3-1838*r**5-4140*r**7-2305*r**9-232*r**11)/(2*(-1
            +y))-(r**3*(110+785*r**2+980*r**4+245*r**6+20*r**8+2*r**10)*y)/5
            -(2*r**3*(10+70*r**2+90*r**4+25*r**6+r**8)*y**2)/5+(r**3*(36+30*r**2
            -76*r**4+3*r**6)*y**3)/6-4*r**3*(-2+r**2)*y**4-r**3*(-2+r**2)*y**5-(2*r*(
            -5+3*r**2)*y**6)/15+(r*y**7)/10)
    if (dEl == 0 and dr == 1 and i==4 and j == 0):
        return     ((6*r*(12+1274*r**2+6678*r**4+2870*r**6-6020*r**8-2205*r**10-14*r**12
            +2*r**14))/35-(28*r**13)/(-1+y)**8-(40*(3*r**11+5*r**13))/(-1+y)**7
            -(4*(150*r**9+633*r**11+460*r**13))/(3*(-1+y)**6)-(8*(60*r**7+516*r**9
            +951*r**11+392*r**13))/(3*(-1+y)**5)-(2*(150*r**5+2660*r**7+10020*r**9
            +10491*r**11+2660*r**13))/(5*(-1+y)**4)-(8*(15*r**3+705*r**5+5510*r**7
            +11820*r**9+7611*r**11+1190*r**13))/(15*(-1+y)**3)-(4*(33*r**3+696*r**5
            +3110*r**7+4080*r**9+1599*r**11+140*r**13))/(3*(-1+y)**2)-(72*(r**3
            +12*r**5+30*r**7+20*r**9+3*r**11))/(-1+y)+(8*r**3*(-30-285*r**2-520*r**4
            -210*r**6-6*r**8+r**10)*y)/3+(24*r**3*(-5-35*r**2-40*r**4-5*r**6+r**8)*y**2)/5
            -(10*r**3*(-3-6*r**2+2*r**4)*y**4)/3-(24*r**3*(-2+r**2)*y**5)/5+(64*r*y**7)/105
            -72*r**3*(1+12*r**2+30*r**4+20*r**6+3*r**8)*math.log((1-y)/r**2))
    return 0. 

@jit(float64(int64, int64, float64, float64, int64, int64), cache=True, nopython=True)
def XG(i,j,elcuthat,r,dEl,dr):
    """ Tree level functions (muG) for mix moments and their derivatives """
    y  = 2*elcuthat
    logy = math.log((1-y)/r**2)
    # tree level function
    if (dEl == 0 and dr == 0 and i==0 and j == 0):
        return     (6*logy*r**4+(-9-54*r**4+52*r**6-15*r**8)/6+(10*r**6)/(3*(-1+y)**3)
            +(9*r**4+20*r**6)/(3*(-1+y)**2)+(2*(-2*r**2+3*r**4))/(-1+y)-(2*r**2*(6
            -9*r**2+5*r**4)*y)/3-4*r**2*y**2-(4*y**3)/3-(5*y**4)/6)
    if (dEl == 0 and dr == 0 and i==0 and j == 1):
        return     (2*logy*r**2*(1+r**2)+(-9-7*r**2-13*r**4+67*r**6-30*r**8+7*r**10)/6
            +(5*r**8)/(4*(-1+y)**4)+(52*r**6+25*r**8)/(12*(-1+y)**3)+(12*r**4+65*r**6)/(6*(
            -1+y)**2)+(-24*r**2+51*r**4+34*r**6)/(6*(-1+y))+(r**2*(-8+26*r**2
            -12*r**4+5*r**6)*y)/4-(r**2*(18-21*r**2+5*r**4)*y**2)/6-((5-6*r**2
            +5*r**4)*y**3)/6-y**4/2-y**5/12)
    if (dEl == 0 and dr == 0 and i==0 and j == 2):
        return     (-1/3*(logy*r**2*(-12+9*r**2+7*r**4))+(-257-735*r**2+1980*r**4+1740*r**6
            -1245*r**8+522*r**10-100*r**12)/180+r**10/(2*(-1+y)**5)+(23*r**8+6*r**10)/(8*(
            -1+y)**4)+(60*r**6+77*r**8)/(12*(-1+y)**3)+(24*r**4+392*r**6+73*r**8)/(24*(
            -1+y)**2)+(-12*r**2+27*r**4+53*r**6)/(3*(-1+y))-(r**4*(-24+14*r**2
            -10*r**4+3*r**6)*y)/6+(r**2*(-16+36*r**2-20*r**4+5*r**6)*y**2)/8+((
            -9+15*r**2-18*r**4+10*r**6)*y**3)/18+((-2+r**2)*y**4)/6-y**5/10-y**6/72)
    if (dEl == 0 and dr == 0 and i==0 and j == 3):
        return     (-1/2*(logy*r**2*(-12+15*r**2+23*r**4))+(-2228-14364*r**2+50190*r**4
            +16870*r**6-13965*r**8+7707*r**10-2786*r**12+450*r**14)/1680+(5*r**12)/(24*(
            -1+y)**6)+(198*r**10+35*r**12)/(120*(-1+y)**5)+(380*r**8+273*r**10)/(80*(
            -1+y)**4)+(256*r**6+657*r**8+72*r**10)/(48*(-1+y)**3)+(1076*r**6+597*r**8)/(48*(
            -1+y)**2)+(-64*r**2+120*r**4+604*r**6+71*r**8)/(16*(-1+y))+(r**2*(240
            -310*r**4+195*r**6-108*r**8+25*r**10)*y)/120-(r**2*(8-30*r**2+26*r**4
            -14*r**6+3*r**8)*y**2)/8-((14-24*r**2+30*r**4-44*r**6+15*r**8)*y**3)/48
            +((-11+9*r**2)*y**4)/48+((-2+r**2)*y**5)/20-y**6/40-y**7/336)
    if (dEl == 0 and dr == 0 and i==0 and j == 4):
        return     (-1/8*(logy*r**2*(-64+80*r**2+264*r**4+23*r**6))+(-8181-96320*r**2+350280*r**4
            +167440*r**6-85540*r**8+39144*r**10-20146*r**12+6280*r**14-875*r**16)/6720
            +(5*r**14)/(56*(-1+y)**7)+(301*r**12+40*r**14)/(336*(-1+y)**6)+(426*r**10
            +211*r**12)/(120*(-1+y)**5)+(3240*r**8+4560*r**10+353*r**12)/(480*(-1
            +y)**4)+(640*r**6+2910*r**8+947*r**10)/(120*(-1+y)**3)+(-48*r**4+1360*r**6
            +1557*r**8+108*r**10)/(48*(-1+y)**2)+(-96*r**2+96*r**4+1588*r**6+561*r**8)/(24*(
            -1+y))-(r**2*(-1120+1120*r**2+2100*r**4-385*r**6+301*r**8-133*r**10
            +25*r**12)*y)/280+(r**4*(240-400*r**2+315*r**4-138*r**6+25*r**8)*y**2)/120
            +((-1+2*r**2-2*r**4+6*r**6-4*r**8+r**10)*y**3)/6+((-5+4*r**2+3*r**4)*y**4)/32
            +((-11+9*r**2)*y**5)/120+((-2+r**2)*y**6)/60-y**7/140-y**8/1344)
    if (dEl == 0 and dr == 0 and i==1 and j == 0):
        return     (logy*r**2*(4-3*r**2+9*r**4)+(-15-55*r**2+48*r**4+24*r**6+49*r**8
            -9*r**10)/12+(5*r**8)/(2*(-1+y)**4)+(16*r**6+25*r**8)/(3*(-1+y)**3)
            +(3*r**4+57*r**6+25*r**8)/(3*(-1+y)**2)+(-12*r**2+15*r**4+59*r**6)/(3*(
            -1+y))-(5*r**6*(-4+r**2)*y)/3-(r**2*(12-3*r**2+5*r**4)*y**2)/6-(8*r**2*y**3)/3
            -((7+5*r**2)*y**4)/12-y**5/3)
    if (dEl == 0 and dr == 0 and i==1 and j == 1):
        return     ((-2*logy*r**2*(-9+9*r**2+2*r**4))/3+(-106-855*r**2+2070*r**4+420*r**6
            +540*r**8-144*r**10+25*r**12)/90+r**10/(-1+y)**5+(3*(3*r**8+2*r**10))/(2*(
            -1+y)**4)+(34*r**6+97*r**8+15*r**10)/(6*(-1+y)**3)+(147*r**6+118*r**8)/(6*(
            -1+y)**2)+(-8*r**2+7*r**4+83*r**6+14*r**8)/(2*(-1+y))+(r**2*(12
            -15*r**2+49*r**4-11*r**6+3*r**8)*y)/6+(r**2*(-6+3*r**2+8*r**4)*y**2)/6
            -(r**2*(36-27*r**2+5*r**4)*y**3)/18-((1+r**2)*y**4)/3-(4*y**5)/15-y**6/18)
    if (dEl == 0 and dr == 0 and i==1 and j == 2):
        return     (-1/6*(logy*r**2*(-48+42*r**2+147*r**4+17*r**6))+(-5526-78260*r**2+223440*r**4
            +119070*r**6+15645*r**8-11823*r**10+3570*r**12-550*r**14)/5040+(5*r**12)/(12*(
            -1+y)**6)+(7*(12*r**10+5*r**12))/(30*(-1+y)**5)+(265*r**8+381*r**10
            +35*r**12)/(40*(-1+y)**4)+(136*r**6+653*r**8+257*r**10)/(24*(-1+y)**3)
            +(-48*r**4+1432*r**6+2033*r**8+175*r**10)/(48*(-1+y)**2)+(-48*r**2
            +846*r**6+371*r**8)/(12*(-1+y))-(r**2*(-120+150*r**2-100*r**4+45*r**6
            -22*r**8+5*r**10)*y)/30+(r**4*(-8+44*r**2-9*r**4+r**6)*y**2)/16+(r**2*(
            -48+60*r**2-12*r**4+5*r**6)*y**3)/36+((-9-21*r**2+16*r**4)*y**4)/48
            -y**5/5+((-57+5*r**2)*y**6)/720-y**7/84)
    if (dEl == 0 and dr == 0 and i==1 and j == 3):
        return     (-1/2*(logy*r**2*(-20+9*r**2+130*r**4+40*r**6))+(-1705-37840*r**2+107982*r**4
            +130620*r**6-7875*r**8-5418*r**10+1939*r**12-552*r**14+75*r**16)/1680
            +(5*r**14)/(28*(-1+y)**7)+(133*r**12+40*r**14)/(84*(-1+y)**6)+(327*r**10
            +309*r**12+20*r**14)/(60*(-1+y)**5)+(2100*r**8+5109*r**10+1309*r**12)/(240*(
            -1+y)**4)+(1280*r**6+9955*r**8+7377*r**10+420*r**12)/(240*(-1+y)**3)
            +(-96*r**4+1644*r**6+3871*r**8+933*r**10)/(48*(-1+y)**2)+(-192*r**2
            -264*r**4+5060*r**6+4305*r**8+255*r**10)/(48*(-1+y))+(r**2*(2520-2520*r**2
            -5075*r**4-910*r**6+315*r**8-133*r**10+25*r**12)*y)/420-(r**2*(-120
            +210*r**2-260*r**4+105*r**6-36*r**8+5*r**10)*y**2)/120-(r**2*(32-48*r**2
            -4*r**4-11*r**6+3*r**8)*y**3)/48+((-5-22*r**2+27*r**4)*y**4)/48+((
            -17-9*r**2+12*r**4)*y**5)/120+((-5+r**2)*y**6)/60+((-41+5*r**2)*y**7)/1680
            -y**8/336)
    if (dEl == 0 and dr == 0 and i==2 and j == 0):
        return     (4*logy*r**2*(-1+r**2)**2*(2+3*r**2)+(-3-50*r**2+112*r**4+32*r**6+35*r**8
            +8*r**10-r**12)/3+(2*r**10)/(-1+y)**5+(13*r**8+18*r**10)/(2*(-1+y)**4)
            +(18*r**6+92*r**8+45*r**10)/(3*(-1+y)**3)+(-6*r**4+188*r**6+329*r**8
            +60*r**10)/(6*(-1+y)**2)+(4*(-3*r**2-3*r**4+53*r**6+29*r**8))/(3*(-1
            +y))-r**2*(-4+6*r**2-10*r**4-7*r**6+r**8)*y-(r**4*(18-28*r**2+5*r**4)*y**2)/6
            -(4*r**2*(1+r**2)*y**3)/3-2*r**2*y**4-((1+r**2)*y**5)/3-y**6/6)
    if (dEl == 0 and dr == 0 and i==2 and j == 1):
        return     ((-10*logy*r**2*(-3+17*r**4+2*r**6))/3+(-590-14945*r**2+34335*r**4+44940*r**6
            +3885*r**8+2772*r**10-490*r**12+65*r**14)/630+(5*r**12)/(6*(-1+y)**6)
            +(46*r**10+35*r**12)/(10*(-1+y)**5)+(175*r**8+429*r**10+105*r**12)/(20*(
            -1+y)**4)+(68*r**6+544*r**8+460*r**10+35*r**12)/(12*(-1+y)**3)+(
            -24*r**4+422*r**6+1150*r**8+371*r**10)/(12*(-1+y)**2)+(-48*r**2-114*r**4
            +1260*r**6+1285*r**8+100*r**10)/(12*(-1+y))+(r**2*(120-110*r**2-100*r**4
            +195*r**6-26*r**8+5*r**10)*y)/20+(r**2*(12-42*r**2+62*r**4+9*r**6
            +r**8)*y**2)/12-(r**2*(24+54*r**2-76*r**4+5*r**6)*y**3)/36+(r**2*(-3
            +r**2)*y**4)/2+((-11-30*r**2+5*r**4)*y**5)/60-((29+5*r**2)*y**6)/180
            -y**7/28)
    if (dEl == 0 and dr == 0 and i==2 and j == 2):
        return     (-1/6*(logy*r**2*(-72-54*r**2+700*r**4+405*r**6+20*r**8))+(-4405-159096*r**2
            +326788*r**4+956760*r**6+75810*r**8-504*r**10-6104*r**12+1392*r**14
            -175*r**16)/5040+(5*r**14)/(14*(-1+y)**7)+(77*r**12+40*r**14)/(28*(-1
            +y)**6)+(81*r**10+124*r**12+20*r**14)/(10*(-1+y)**5)+(435*r**8+1640*r**10
            +846*r**12+40*r**14)/(40*(-1+y)**4)+(150*r**6+1885*r**8+2504*r**10
            +480*r**12)/(30*(-1+y)**3)+(-36*r**4+448*r**6+1878*r**8+1034*r**10
            +51*r**12)/(12*(-1+y)**2)+(-24*r**2-102*r**4+848*r**6+1425*r**8+286*r**10)/(6*(
            -1+y))-(r**2*(-560+140*r**2+2450*r**4-140*r**6+77*r**8-28*r**10
            +5*r**12)*y)/70-(r**2*(-20+35*r**2-10*r**4-25*r**6+2*r**8)*y**2)/10
            +(r**4*(-36+56*r**2-3*r**4+r**6)*y**3)/18+(r**2*(-4+r**2+2*r**4)*y**4)/4
            +((-1-6*r**2+3*r**4)*y**5)/10+((-23-18*r**2+5*r**4)*y**6)/180-(2*y**7)/35
            -y**8/112)
    if (dEl == 0 and dr == 0 and i==3 and j == 0):
        return     (3*logy*r**2*(4+5*r**2-35*r**4-15*r**6+5*r**8)+(-345-13713*r**2+21315*r**4
            +76195*r**6+4830*r**8+9030*r**10+833*r**12-75*r**14)/420+(5*r**12)/(3*(
            -1+y)**6)+(4*(27*r**10+35*r**12))/(15*(-1+y)**5)+(110*r**8+417*r**10
            +210*r**12)/(10*(-1+y)**4)+(32*r**6+405*r**8+591*r**10+140*r**12)/(6*(
            -1+y)**3)+(-36*r**4+452*r**6+2115*r**8+1413*r**10+140*r**12)/(12*(
            -1+y)**2)+(-4*r**2-21*r**4+135*r**6+265*r**8+63*r**10)/(-1+y)-(2*r**2*(
            -60+220*r**4-195*r**6-54*r**8+5*r**10)*y)/15-(r**2*(-8+18*r**2-6*r**4
            -35*r**6+3*r**8)*y**2)/4+(4*r**4*(-3+2*r**2)*y**3)/3+(r**2*(-12-27*r**2
            +5*r**4)*y**4)/12-(8*r**2*y**5)/5-((13+15*r**2)*y**6)/60-(2*y**7)/21)
    if (dEl == 0 and dr == 0 and i==3 and j == 1):
        return     (-2*logy*r**2*(-7-16*r**2+90*r**4+100*r**6+7*r**8)+(-162-8648*r**2
            +9331*r**4+80360*r**6+22575*r**8+665*r**10+763*r**12-96*r**14+10*r**16)/210
            +(5*r**14)/(7*(-1+y)**7)+(2*(49*r**12+40*r**14))/(21*(-1+y)**6)+(2*(87*r**10
            +200*r**12+60*r**14))/(15*(-1+y)**5)+(780*r**8+4263*r**10+3725*r**12
            +480*r**14)/(60*(-1+y)**4)+(52*r**6+1049*r**8+2197*r**10+886*r**12
            +40*r**14)/(12*(-1+y)**3)+(-48*r**4+442*r**6+3138*r**8+3081*r**10+535*r**12)/(12*(
            -1+y)**2)+(-48*r**2-366*r**4+1994*r**6+6015*r**8+2589*r**10+116*r**12)/(12*(
            -1+y))+(r**2*(700+665*r**2-5215*r**4-805*r**6+791*r**8-70*r**10
            +10*r**12)*y)/70+(r**2*(36-30*r**2-100*r**4+135*r**6+6*r**8+r**10)*y**2)/12
            -(r**2*(-8+46*r**2-26*r**4-33*r**6+r**8)*y**3)/12+(r**2*(-1-5*r**2
            +4*r**4)*y**4)/2+(r**2*(-36-3*r**2+5*r**4)*y**5)/30+((-7-30*r**2+5*r**4)*y**6)/60
            -((3+r**2)*y**7)/28-y**8/42)
    if (dEl == 0 and dr == 0 and i==4 and j == 0):
        return     (2*logy*r**2*(8+33*r**2-120*r**4-245*r**6-48*r**8+9*r**10)+(-291-21536*r**2
            -2184*r**4+276360*r**6+176960*r**8-1680*r**10+13314*r**12+664*r**14
            -45*r**16)/420+(10*r**14)/(7*(-1+y)**7)+(161*r**12+200*r**14)/(21*(-1
            +y)**6)+(2*(120*r**10+393*r**12+200*r**14))/(15*(-1+y)**5)+(450*r**8
            +3408*r**10+4555*r**12+1200*r**14)/(30*(-1+y)**4)+(2*(25*r**6+855*r**8
            +2616*r**10+1795*r**12+250*r**14))/(15*(-1+y)**3)+(-15*r**4+96*r**6
            +1183*r**8+1824*r**10+645*r**12+40*r**14)/(3*(-1+y)**2)+(2*(-6*r**2
            -69*r**4+256*r**6+1365*r**8+1032*r**10+139*r**12))/(3*(-1+y))-(2*r**2*(
            -126-315*r**2+1295*r**4+910*r**6-504*r**8-77*r**10+5*r**12)*y)/21
            -(2*r**2*(-30-15*r**2+190*r**4-120*r**6-96*r**8+5*r**10)*y**2)/15+(4*r**2*(1
            -3*r**2-3*r**4+7*r**6)*y**3)/3+(r**4*(-27+4*r**2+5*r**4)*y**4)/6+(2*r**2*(
            -6-21*r**2+5*r**4)*y**5)/15-(4*r**2*y**6)/3-(4*(4+5*r**2)*y**7)/105-(5*y**8)/84)

    # derivative wrt Elcuthat
    if (dEl == 1 and dr == 0 and i==0 and j == 0):
        return     ((-4*r**2*(6-9*r**2+5*r**4))/3-(20*r**6)/(-1+y)**4-(4*(9*r**4+20*r**6))/(3*(
            -1+y)**3)-(4*(-2*r**2+3*r**4))/(-1+y)**2+(12*r**4)/(-1+y)-16*r**2*y
            -8*y**2-(20*y**3)/3)
    if (dEl == 1 and dr == 0 and i==0 and j == 1):
        return     ((r**2*(-8+26*r**2-12*r**4+5*r**6))/2-(10*r**8)/(-1+y)**5+(-52*r**6
            -25*r**8)/(2*(-1+y)**4)-(2*(12*r**4+65*r**6))/(3*(-1+y)**3)+(24*r**2
            -51*r**4-34*r**6)/(3*(-1+y)**2)+(4*(r**2+r**4))/(-1+y)-(2*r**2*(18
            -21*r**2+5*r**4)*y)/3-(5-6*r**2+5*r**4)*y**2-4*y**3-(5*y**4)/6)
    if (dEl == 1 and dr == 0 and i==0 and j == 2):
        return     (-1/3*(r**4*(-24+14*r**2-10*r**4+3*r**6))-(5*r**10)/(-1+y)**6+(-23*r**8
            -6*r**10)/(-1+y)**5+(-60*r**6-77*r**8)/(2*(-1+y)**4)+(-24*r**4-392*r**6
            -73*r**8)/(6*(-1+y)**3)-(2*(-12*r**2+27*r**4+53*r**6))/(3*(-1+y)**2)
            -(2*(-12*r**2+9*r**4+7*r**6))/(3*(-1+y))+(r**2*(-16+36*r**2-20*r**4
            +5*r**6)*y)/2+((-9+15*r**2-18*r**4+10*r**6)*y**2)/3+(4*(-2+r**2)*y**3)/3
            -y**4-y**5/6)
    if (dEl == 1 and dr == 0 and i==0 and j == 3):
        return     ((r**2*(240-310*r**4+195*r**6-108*r**8+25*r**10))/60-(5*r**12)/(2*(-1
            +y)**7)+(-198*r**10-35*r**12)/(12*(-1+y)**6)+(-380*r**8-273*r**10)/(10*(
            -1+y)**5)+(-256*r**6-657*r**8-72*r**10)/(8*(-1+y)**4)+(-1076*r**6
            -597*r**8)/(12*(-1+y)**3)+(64*r**2-120*r**4-604*r**6-71*r**8)/(8*(-1
            +y)**2)+(12*r**2-15*r**4-23*r**6)/(-1+y)-(r**2*(8-30*r**2+26*r**4
            -14*r**6+3*r**8)*y)/2-((14-24*r**2+30*r**4-44*r**6+15*r**8)*y**2)/8
            +((-11+9*r**2)*y**3)/6+((-2+r**2)*y**4)/2-(3*y**5)/10-y**6/24)
    if (dEl == 1 and dr == 0 and i==0 and j == 4):
        return     (-1/140*(r**2*(-1120+1120*r**2+2100*r**4-385*r**6+301*r**8-133*r**10
            +25*r**12))-(5*r**14)/(4*(-1+y)**8)+(-301*r**12-40*r**14)/(28*(-1+y)**7)
            +(-426*r**10-211*r**12)/(12*(-1+y)**6)+(-3240*r**8-4560*r**10-353*r**12)/(60*(
            -1+y)**5)+(-640*r**6-2910*r**8-947*r**10)/(20*(-1+y)**4)+(48*r**4
            -1360*r**6-1557*r**8-108*r**10)/(12*(-1+y)**3)+(96*r**2-96*r**4-1588*r**6
            -561*r**8)/(12*(-1+y)**2)+(64*r**2-80*r**4-264*r**6-23*r**8)/(4*(-1
            +y))+(r**4*(240-400*r**2+315*r**4-138*r**6+25*r**8)*y)/30+(-1+2*r**2
            -2*r**4+6*r**6-4*r**8+r**10)*y**2+((-5+4*r**2+3*r**4)*y**3)/4+((-11
            +9*r**2)*y**4)/12+((-2+r**2)*y**5)/5-y**6/10-y**7/84)
    if (dEl == 1 and dr == 0 and i==1 and j == 0):
        return     ((-10*r**6*(-4+r**2))/3-(20*r**8)/(-1+y)**5-(2*(16*r**6+25*r**8))/(-1
            +y)**4-(4*(3*r**4+57*r**6+25*r**8))/(3*(-1+y)**3)-(2*(-12*r**2+15*r**4
            +59*r**6))/(3*(-1+y)**2)+(2*(4*r**2-3*r**4+9*r**6))/(-1+y)-(2*r**2*(12
            -3*r**2+5*r**4)*y)/3-16*r**2*y**2-(2*(7+5*r**2)*y**3)/3-(10*y**4)/3)
    if (dEl == 1 and dr == 0 and i==1 and j == 1):
        return     ((r**2*(12-15*r**2+49*r**4-11*r**6+3*r**8))/3-(10*r**10)/(-1+y)**6-(12*(3*r**8
            +2*r**10))/(-1+y)**5+(-34*r**6-97*r**8-15*r**10)/(-1+y)**4-(2*(147*r**6
            +118*r**8))/(3*(-1+y)**3)+(8*r**2-7*r**4-83*r**6-14*r**8)/(-1+y)**2
            -(4*(-9*r**2+9*r**4+2*r**6))/(3*(-1+y))+(2*r**2*(-6+3*r**2+8*r**4)*y)/3
            -(r**2*(36-27*r**2+5*r**4)*y**2)/3-(8*(1+r**2)*y**3)/3-(8*y**4)/3-(2*y**5)/3)
    if (dEl == 1 and dr == 0 and i==1 and j == 2):
        return     (-1/15*(r**2*(-120+150*r**2-100*r**4+45*r**6-22*r**8+5*r**10))-(5*r**12)/(
            -1+y)**7-(7*(12*r**10+5*r**12))/(3*(-1+y)**6)+(-265*r**8-381*r**10
            -35*r**12)/(5*(-1+y)**5)+(-136*r**6-653*r**8-257*r**10)/(4*(-1+y)**4)
            +(48*r**4-1432*r**6-2033*r**8-175*r**10)/(12*(-1+y)**3)+(48*r**2-846*r**6
            -371*r**8)/(6*(-1+y)**2)+(48*r**2-42*r**4-147*r**6-17*r**8)/(3*(-1
            +y))+(r**4*(-8+44*r**2-9*r**4+r**6)*y)/4+(r**2*(-48+60*r**2-12*r**4
            +5*r**6)*y**2)/6+((-9-21*r**2+16*r**4)*y**3)/6-2*y**4+((-57+5*r**2)*y**5)/60
            -y**6/6)
    if (dEl == 1 and dr == 0 and i==1 and j == 3):
        return     ((r**2*(2520-2520*r**2-5075*r**4-910*r**6+315*r**8-133*r**10+25*r**12))/210
            -(5*r**14)/(2*(-1+y)**8)+(-133*r**12-40*r**14)/(7*(-1+y)**7)+(-327*r**10
            -309*r**12-20*r**14)/(6*(-1+y)**6)+(-2100*r**8-5109*r**10-1309*r**12)/(30*(
            -1+y)**5)+(-1280*r**6-9955*r**8-7377*r**10-420*r**12)/(40*(-1+y)**4)
            +(96*r**4-1644*r**6-3871*r**8-933*r**10)/(12*(-1+y)**3)+(192*r**2+264*r**4
            -5060*r**6-4305*r**8-255*r**10)/(24*(-1+y)**2)+(20*r**2-9*r**4-130*r**6
            -40*r**8)/(-1+y)-(r**2*(-120+210*r**2-260*r**4+105*r**6-36*r**8+5*r**10)*y)/30
            -(r**2*(32-48*r**2-4*r**4-11*r**6+3*r**8)*y**2)/8+((-5-22*r**2+27*r**4)*y**3)/6
            +((-17-9*r**2+12*r**4)*y**4)/12+((-5+r**2)*y**5)/5+((-41+5*r**2)*y**6)/120
            -y**7/21)
    if (dEl == 1 and dr == 0 and i==2 and j == 0):
        return     (-2*r**2*(-4+6*r**2-10*r**4-7*r**6+r**8)-(20*r**10)/(-1+y)**6-(4*(13*r**8
            +18*r**10))/(-1+y)**5-(2*(18*r**6+92*r**8+45*r**10))/(-1+y)**4-(2*(
            -6*r**4+188*r**6+329*r**8+60*r**10))/(3*(-1+y)**3)-(8*(-3*r**2-3*r**4
            +53*r**6+29*r**8))/(3*(-1+y)**2)+(8*(2*r**2-r**4-4*r**6+3*r**8))/(
            -1+y)-(2*r**4*(18-28*r**2+5*r**4)*y)/3-8*r**2*(1+r**2)*y**2-16*r**2*y**3
            -(10*(1+r**2)*y**4)/3-2*y**5)
    if (dEl == 1 and dr == 0 and i==2 and j == 1):
        return     ((r**2*(120-110*r**2-100*r**4+195*r**6-26*r**8+5*r**10))/10-(10*r**12)/(
            -1+y)**7+(-46*r**10-35*r**12)/(-1+y)**6-(2*(175*r**8+429*r**10+105*r**12))/(5*(
            -1+y)**5)+(-68*r**6-544*r**8-460*r**10-35*r**12)/(2*(-1+y)**4)+(24*r**4
            -422*r**6-1150*r**8-371*r**10)/(3*(-1+y)**3)+(48*r**2+114*r**4-1260*r**6
            -1285*r**8-100*r**10)/(6*(-1+y)**2)-(20*(-3*r**2+17*r**6+2*r**8))/(3*(
            -1+y))+(r**2*(12-42*r**2+62*r**4+9*r**6+r**8)*y)/3-(r**2*(24+54*r**2
            -76*r**4+5*r**6)*y**2)/6+4*r**2*(-3+r**2)*y**3+((-11-30*r**2+5*r**4)*y**4)/6
            -((29+5*r**2)*y**5)/15-y**6/2)
    if (dEl == 1 and dr == 0 and i==2 and j == 2):
        return     (-1/35*(r**2*(-560+140*r**2+2450*r**4-140*r**6+77*r**8-28*r**10+5*r**12))
            -(5*r**14)/(-1+y)**8-(3*(77*r**12+40*r**14))/(7*(-1+y)**7)+(-81*r**10
            -124*r**12-20*r**14)/(-1+y)**6+(-435*r**8-1640*r**10-846*r**12-40*r**14)/(5*(
            -1+y)**5)+(-150*r**6-1885*r**8-2504*r**10-480*r**12)/(5*(-1+y)**4)
            +(36*r**4-448*r**6-1878*r**8-1034*r**10-51*r**12)/(3*(-1+y)**3)+(24*r**2
            +102*r**4-848*r**6-1425*r**8-286*r**10)/(3*(-1+y)**2)+(72*r**2+54*r**4
            -700*r**6-405*r**8-20*r**10)/(3*(-1+y))-(2*r**2*(-20+35*r**2-10*r**4
            -25*r**6+2*r**8)*y)/5+(r**4*(-36+56*r**2-3*r**4+r**6)*y**2)/3+2*r**2*(
            -4+r**2+2*r**4)*y**3+(-1-6*r**2+3*r**4)*y**4+((-23-18*r**2+5*r**4)*y**5)/15
            -(4*y**6)/5-y**7/7)
    if (dEl == 1 and dr == 0 and i==3 and j == 0):
        return     ((-4*r**2*(-60+220*r**4-195*r**6-54*r**8+5*r**10))/15-(20*r**12)/(-1
            +y)**7-(8*(27*r**10+35*r**12))/(3*(-1+y)**6)-(4*(110*r**8+417*r**10
            +210*r**12))/(5*(-1+y)**5)+(-32*r**6-405*r**8-591*r**10-140*r**12)/(
            -1+y)**4+(36*r**4-452*r**6-2115*r**8-1413*r**10-140*r**12)/(3*(-1
            +y)**3)-(2*(-4*r**2-21*r**4+135*r**6+265*r**8+63*r**10))/(-1+y)**2
            +(6*(4*r**2+5*r**4-35*r**6-15*r**8+5*r**10))/(-1+y)-r**2*(-8+18*r**2
            -6*r**4-35*r**6+3*r**8)*y+8*r**4*(-3+2*r**2)*y**2+(2*r**2*(-12-27*r**2
            +5*r**4)*y**3)/3-16*r**2*y**4-((13+15*r**2)*y**5)/5-(4*y**6)/3)
    if (dEl == 1 and dr == 0 and i==3 and j == 1):
        return     ((r**2*(700+665*r**2-5215*r**4-805*r**6+791*r**8-70*r**10+10*r**12))/35
            -(10*r**14)/(-1+y)**8-(8*(49*r**12+40*r**14))/(7*(-1+y)**7)-(4*(87*r**10
            +200*r**12+60*r**14))/(3*(-1+y)**6)-(2*(780*r**8+4263*r**10+3725*r**12
            +480*r**14))/(15*(-1+y)**5)+(-52*r**6-1049*r**8-2197*r**10-886*r**12
            -40*r**14)/(2*(-1+y)**4)+(48*r**4-442*r**6-3138*r**8-3081*r**10-535*r**12)/(3*(
            -1+y)**3)+(48*r**2+366*r**4-1994*r**6-6015*r**8-2589*r**10-116*r**12)/(6*(
            -1+y)**2)-(4*(-7*r**2-16*r**4+90*r**6+100*r**8+7*r**10))/(-1+y)
            +(r**2*(36-30*r**2-100*r**4+135*r**6+6*r**8+r**10)*y)/3-(r**2*(-8
            +46*r**2-26*r**4-33*r**6+r**8)*y**2)/2+4*r**2*(-1-5*r**2+4*r**4)*y**3
            +(r**2*(-36-3*r**2+5*r**4)*y**4)/3+((-7-30*r**2+5*r**4)*y**5)/5-((3
            +r**2)*y**6)/2-(8*y**7)/21)
    if (dEl == 1 and dr == 0 and i==4 and j == 0):
        return     ((-4*r**2*(-126-315*r**2+1295*r**4+910*r**6-504*r**8-77*r**10+5*r**12))/21
            -(20*r**14)/(-1+y)**8-(4*(161*r**12+200*r**14))/(7*(-1+y)**7)-(4*(120*r**10
            +393*r**12+200*r**14))/(3*(-1+y)**6)-(4*(450*r**8+3408*r**10+4555*r**12
            +1200*r**14))/(15*(-1+y)**5)-(4*(25*r**6+855*r**8+2616*r**10+1795*r**12
            +250*r**14))/(5*(-1+y)**4)-(4*(-15*r**4+96*r**6+1183*r**8+1824*r**10
            +645*r**12+40*r**14))/(3*(-1+y)**3)-(4*(-6*r**2-69*r**4+256*r**6+1365*r**8
            +1032*r**10+139*r**12))/(3*(-1+y)**2)+(4*(8*r**2+33*r**4-120*r**6-245*r**8
            -48*r**10+9*r**12))/(-1+y)-(8*r**2*(-30-15*r**2+190*r**4-120*r**6
            -96*r**8+5*r**10)*y)/15+8*r**2*(1-3*r**2-3*r**4+7*r**6)*y**2+(4*r**4*(
            -27+4*r**2+5*r**4)*y**3)/3+(4*r**2*(-6-21*r**2+5*r**4)*y**4)/3-16*r**2*y**5
            -(8*(4+5*r**2)*y**6)/15-(20*y**7)/21)

    # derivative wrt r
    if (dEl == 0 and dr == 1 and i==0 and j == 0):
        return     (-4*r**3*(12-13*r**2+5*r**4)+(20*r**5)/(-1+y)**3+(4*(3*r**3+10*r**5))/(
            -1+y)**2+(8*(-r+3*r**3))/(-1+y)-4*r*(2-6*r**2+5*r**4)*y-8*r*y**2
            +24*r**3*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 1):
        return     ((r*(-19-38*r**2+201*r**4-120*r**6+35*r**8))/3+(10*r**7)/(-1+y)**4
            +(2*(39*r**5+25*r**7))/(3*(-1+y)**3)+(8*r**3+65*r**5)/(-1+y)**2+(2*(
            -4*r+17*r**3+17*r**5))/(-1+y)+2*r*(-2+13*r**2-9*r**4+5*r**6)*y
            -r*(6-14*r**2+5*r**4)*y**2-(2*r*(-3+5*r**2)*y**3)/3+4*r*(1+2*r**2)*math.log((1
            -y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 2):
        return     (-1/6*(r*(97-300*r**2-376*r**4+332*r**6-174*r**8+40*r**10))+(5*r**9)/(
            -1+y)**5+(46*r**7+15*r**9)/(2*(-1+y)**4)+(2*(45*r**5+77*r**7))/(3*(
            -1+y)**3)+(12*r**3+294*r**5+73*r**7)/(3*(-1+y)**2)+(2*(-4*r+18*r**3
            +53*r**5))/(-1+y)-(r**3*(-48+42*r**2-40*r**4+15*r**6)*y)/3+r*(-4
            +18*r**2-15*r**4+5*r**6)*y**2+(r*(5-12*r**2+10*r**4)*y**3)/3+(r*y**4)/3
            -2*r*(-4+6*r**2+7*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 3):
        return     ((r*(-1164+5380*r**2+3330*r**4-2660*r**6+1835*r**8-796*r**10+150*r**12))/40
            +(5*r**11)/(2*(-1+y)**6)+(33*r**9+7*r**11)/(2*(-1+y)**5)+(304*r**7
            +273*r**9)/(8*(-1+y)**4)+(64*r**5+219*r**7+30*r**9)/(2*(-1+y)**3)+(269*r**5
            +199*r**7)/(2*(-1+y)**2)+(-16*r+60*r**3+453*r**5+71*r**7)/(2*(-1+y))
            +(r*(8-31*r**4+26*r**6-18*r**8+5*r**10)*y)/2-(r*(8-60*r**2+78*r**4
            -56*r**6+15*r**8)*y**2)/4-(r*(-2+5*r**2-11*r**4+5*r**6)*y**3)/2+(3*r*y**4)/8
            +(r*y**5)/10-3*r*(-4+10*r**2+23*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 4):
        return     (-1/120*(r*(5360-27420*r**2-25860*r**4+11530*r**6-6990*r**8+4317*r**10
            -1570*r**12+250*r**14))+(5*r**13)/(4*(-1+y)**7)+(129*r**11+20*r**13)/(12*(
            -1+y)**6)+(355*r**9+211*r**11)/(10*(-1+y)**5)+(2160*r**7+3800*r**9
            +353*r**11)/(40*(-1+y)**4)+(384*r**5+2328*r**7+947*r**9)/(12*(-1+y)**3)
            +(-8*r**3+340*r**5+519*r**7+45*r**9)/(2*(-1+y)**2)+(-8*r+16*r**3
            +397*r**5+187*r**7)/(-1+y)-(r*(-160+320*r**2+900*r**4-220*r**6+215*r**8
            -114*r**10+25*r**12)*y)/20+(r**3*(16-40*r**2+42*r**4-23*r**6+5*r**8)*y**2)/2
            +(r*(2-4*r**2+18*r**4-16*r**6+5*r**8)*y**3)/3+(r*(2+3*r**2)*y**4)/8
            +(3*r*y**5)/20+(r*y**6)/30-r*(-16+40*r**2+198*r**4+23*r**6)*math.log((1
            -y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 0):
        return     (-1/6*(r*(103-132*r**2+36*r**4-196*r**6+45*r**8))+(20*r**7)/(-1+y)**4
            +(8*(12*r**5+25*r**7))/(3*(-1+y)**3)+(2*(6*r**3+171*r**5+100*r**7))/(3*(
            -1+y)**2)+(2*(-4*r+10*r**3+59*r**5))/(-1+y)-(40*r**5*(-3+r**2)*y)/3
            -r*(4-2*r**2+5*r**4)*y**2-(16*r*y**3)/3-(5*r*y**4)/6+2*r*(4-6*r**2
            +27*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 1):
        return     ((r*(-93+312*r**2+92*r**4+144*r**6-48*r**8+10*r**10))/3+(10*r**9)/(
            -1+y)**5+(6*(6*r**7+5*r**9))/(-1+y)**4+(102*r**5+388*r**7+75*r**9)/(3*(
            -1+y)**3)+(441*r**5+472*r**7)/(3*(-1+y)**2)+(-8*r+14*r**3+249*r**5
            +56*r**7)/(-1+y)+(r*(12-30*r**2+147*r**4-44*r**6+15*r**8)*y)/3+2*r*(
            -1+r**2+4*r**4)*y**2-(r*(12-18*r**2+5*r**4)*y**3)/3-(2*r*y**4)/3-4*r*(
            -3+6*r**2+2*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 2):
        return     (-1/72*(r*(3388-13776*r**2-13734*r**4-2196*r**6+1689*r**8-612*r**10
            +110*r**12))+(5*r**11)/(-1+y)**6+(14*(2*r**9+r**11))/(-1+y)**5+(212*r**7
            +381*r**9+42*r**11)/(4*(-1+y)**4)+(408*r**5+2612*r**7+1285*r**9)/(12*(
            -1+y)**3)+(-96*r**3+4296*r**5+8132*r**7+875*r**9)/(24*(-1+y)**2)+(
            -24*r+1269*r**5+742*r**7)/(3*(-1+y))-(2*r*(-12+30*r**2-30*r**4+18*r**6
            -11*r**8+3*r**10)*y)/3+(r**3*(-16+132*r**2-36*r**4+5*r**6)*y**2)/8+(2*r*(
            -12+30*r**2-9*r**4+5*r**6)*y**3)/9+(r*(-21+32*r**2)*y**4)/24+(r*y**6)/72
            -(r*(-48+84*r**2+441*r**4+68*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 3):
        return     ((r*(-27320+111762*r**2+250530*r**4+1050*r**6-13545*r**8+5817*r**10
            -1932*r**12+300*r**14))/420+(5*r**13)/(2*(-1+y)**7)+(57*r**11+20*r**13)/(3*(
            -1+y)**6)+(1635*r**9+1854*r**11+140*r**13)/(30*(-1+y)**5)+(2800*r**7
            +8515*r**9+2618*r**11)/(40*(-1+y)**4)+(768*r**5+7964*r**7+7377*r**9
            +504*r**11)/(24*(-1+y)**3)+(-192*r**3+4932*r**5+15484*r**7+4665*r**9)/(24*(
            -1+y)**2)+(-64*r-176*r**3+5060*r**5+5740*r**7+425*r**9)/(8*(-1+y))
            +(r*(360-720*r**2-2175*r**4-520*r**6+225*r**8-114*r**10+25*r**12)*y)/30
            -(r*(-4+14*r**2-26*r**4+14*r**6-6*r**8+r**10)*y**2)/2-(r*(32-96*r**2
            -12*r**4-44*r**6+15*r**8)*y**3)/24+(r*(-11+27*r**2)*y**4)/12+(r*(-3
            +8*r**2)*y**5)/20+(r*y**6)/30+(r*y**7)/168-2*r*(-10+9*r**2+195*r**4
            +80*r**6)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 0):
        return     ((-4*r*(37-118*r**2-72*r**4-52*r**6-20*r**8+3*r**10))/3+(20*r**9)/(
            -1+y)**5+(2*(26*r**7+45*r**9))/(-1+y)**4+(2*(54*r**5+368*r**7+225*r**9))/(3*(
            -1+y)**3)+(4*(-3*r**3+141*r**5+329*r**7+75*r**9))/(3*(-1+y)**2)+(8*(
            -3*r-6*r**3+159*r**5+116*r**7))/(3*(-1+y))-2*r*(-4+12*r**2-30*r**4
            -28*r**6+5*r**8)*y-(4*r**3*(9-21*r**2+5*r**4)*y**2)/3-(8*r*(1+2*r**2)*y**3)/3
            -4*r*y**4-(2*r*y**5)/3+16*r*(1-r**2-6*r**4+6*r**6)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 1):
        return     ((r*(-607+1962*r**2+4872*r**4+564*r**6+396*r**8-84*r**10+13*r**12))/9
            +(10*r**11)/(-1+y)**6+(2*(23*r**9+21*r**11))/(-1+y)**5+(140*r**7+429*r**9
            +126*r**11)/(2*(-1+y)**4)+(102*r**5+1088*r**7+1150*r**9+105*r**11)/(3*(
            -1+y)**3)+(-48*r**3+1266*r**5+4600*r**7+1855*r**9)/(6*(-1+y)**2)+(2*(
            -12*r-57*r**3+945*r**5+1285*r**7+125*r**9))/(3*(-1+y))+r*(12-22*r**2
            -30*r**4+78*r**6-13*r**8+3*r**10)*y+(r*(12-84*r**2+186*r**4+36*r**6
            +5*r**8)*y**2)/6-(2*r*(6+27*r**2-57*r**4+5*r**6)*y**3)/9+r*(-3+2*r**2)*y**4
            +(r*(-3+r**2)*y**5)/3-(r*y**6)/18-(20*r*(-3+51*r**4+8*r**6)*math.log((1
            -y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==2 and j == 2):
        return     (-1/45*(r*(3921-10861*r**2-61755*r**4-11490*r**6-255*r**8+654*r**10
            -174*r**12+25*r**14))+(5*r**13)/(-1+y)**7+(33*r**11+20*r**13)/(-1+y)**6
            +(405*r**9+744*r**11+140*r**13)/(5*(-1+y)**5)+(435*r**7+2050*r**9+1269*r**11
            +70*r**13)/(5*(-1+y)**4)+(2*(45*r**5+754*r**7+1252*r**9+288*r**11))/(3*(
            -1+y)**3)+(-36*r**3+672*r**5+3756*r**7+2585*r**9+153*r**11)/(3*(-1
            +y)**2)+(2*(-12*r-102*r**3+1272*r**5+2850*r**7+715*r**9))/(3*(-1+y))
            -(r*(-80+40*r**2+1050*r**4-80*r**6+55*r**8-24*r**10+5*r**12)*y)/5
            -2*r*(-2+7*r**2-3*r**4-10*r**6+r**8)*y**2+(r**3*(-72+168*r**2-12*r**4
            +5*r**6)*y**3)/9+r*(-2+r**2+3*r**4)*y**4+(6*r*(-1+r**2)*y**5)/5+(r*(
            -9+5*r**2)*y**6)/45-(4*r*(-18-27*r**2+525*r**4+405*r**6+25*r**8)*math.log((1
            -y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==3 and j == 0):
        return     (-1/10*(r*(893-1730*r**2-12985*r**4-1820*r**6-1850*r**8-238*r**10+25*r**12))
            +(20*r**11)/(-1+y)**6+(8*(9*r**9+14*r**11))/(-1+y)**5+(88*r**7+417*r**9
            +252*r**11)/(-1+y)**4+(32*r**5+540*r**7+985*r**9+280*r**11)/(-1+y)**3
            +(-24*r**3+452*r**5+2820*r**7+2355*r**9+280*r**11)/(2*(-1+y)**2)+(2*(
            -4*r-42*r**3+405*r**5+1060*r**7+315*r**9))/(-1+y)-8*r*(-2+22*r**4
            -26*r**6-9*r**8+r**10)*y-(r*(-8+36*r**2-18*r**4-140*r**6+15*r**8)*y**2)/2
            +16*r**3*(-1+r**2)*y**3+(r*(-4-18*r**2+5*r**4)*y**4)/2-(16*r*y**5)/5
            -(r*y**6)/2+6*r*(4+10*r**2-105*r**4-60*r**6+25*r**8)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==3 and j == 1):
        return     ((r*(-11588+11942*r**2+278880*r**4+132300*r**6+6265*r**8+4578*r**10
            -672*r**12+80*r**14))/105+(10*r**13)/(-1+y)**7+(8*(21*r**11+20*r**13))/(3*(
            -1+y)**6)+(4*(29*r**9+80*r**11+28*r**13))/(-1+y)**5+(208*r**7+1421*r**9
            +1490*r**11+224*r**13)/(2*(-1+y)**4)+(156*r**5+4196*r**7+10985*r**9
            +5316*r**11+280*r**13)/(6*(-1+y)**3)+(-32*r**3+442*r**5+4184*r**7+5135*r**9
            +1070*r**11)/(2*(-1+y)**2)+(-16*r-244*r**3+1994*r**5+8020*r**7+4315*r**9
            +232*r**11)/(2*(-1+y))+r*(20+38*r**2-447*r**4-92*r**6+113*r**8-12*r**10
            +2*r**12)*y+r*(6-10*r**2-50*r**4+90*r**6+5*r**8+r**10)*y**2-(r*(
            -8+92*r**2-78*r**4-132*r**6+5*r**8)*y**3)/6+r*(-1-10*r**2+12*r**4)*y**4
            +(r*(-12-2*r**2+5*r**4)*y**5)/5+(r*(-3+r**2)*y**6)/3-(r*y**7)/14-4*r*(
            -7-32*r**2+270*r**4+400*r**6+35*r**8)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==4 and j == 0):
        return     ((-2*r*(7064+8022*r**2-232470*r**4-228410*r**6-7980*r**8-18081*r**10
            -1162*r**12+90*r**14))/105+(20*r**13)/(-1+y)**7+(4*(69*r**11+100*r**13))/(3*(
            -1+y)**6)+(8*(300*r**9+1179*r**11+700*r**13))/(15*(-1+y)**5)+(2*(60*r**7
            +568*r**9+911*r**11+280*r**13))/(-1+y)**4+(4*(15*r**5+684*r**7+2616*r**9
            +2154*r**11+350*r**13))/(3*(-1+y)**3)+(4*(-15*r**3+144*r**5+2366*r**7
            +4560*r**9+1935*r**11+140*r**13))/(3*(-1+y)**2)+(8*(-r-23*r**3+128*r**5
            +910*r**7+860*r**9+139*r**11))/(-1+y)-(4*r*(-18-90*r**2+555*r**4
            +520*r**6-360*r**8-66*r**10+5*r**12)*y)/3-8*r*(-1-r**2+19*r**4-16*r**6
            -16*r**8+r**10)*y**2+(8*r*(1-6*r**2-9*r**4+28*r**6)*y**3)/3+(2*r**3*(
            -27+6*r**2+10*r**4)*y**4)/3+(4*r*(-2-14*r**2+5*r**4)*y**5)/5-(8*r*y**6)/3
            -(8*r*y**7)/21+8*r*(4+33*r**2-180*r**4-490*r**6-120*r**8+27*r**10)*math.log((1
            -y)/r**2))
    return 0. 

@jit(float64(int64, int64, float64, float64, int64, int64), cache=True, nopython=True)
def XD(i,j,elcuthat,r,dEl,dr):
    """ Tree level functions (rhoD) for mix moments and their derivatives """
    y  = 2*elcuthat
    logy = math.log((1-y)/r**2)
    # tree level function
    if (dEl == 0 and dr == 0 and i==0 and j == 0):
        return     (-2*logy*(4+3*r**4)+(77-104*r**2+22*r**4-4*r**6-5*r**8)/6+(8*r**6)/(3*(
            -1+y)**5)+(2*(r**4+3*r**6))/(-1+y)**4+(2*(3*r**4+4*r**6))/(3*(-1+y)**3)
            +(-8*r**2-17*r**4-4*r**6)/(3*(-1+y)**2)-(2*(8*r**2+9*r**4))/(3*(-1
            +y))-(8*(3+r**4)*y)/3+(4*(-3+2*r**2)*y**2)/3+(8*y**3)/3+y**4/6)
    if (dEl == 0 and dr == 0 and i==0 and j == 1):
        return     ((8*logy*(-3+r**2))/3+(2*(339-755*r**2+225*r**4-45*r**6-55*r**8+21*r**10))/45
            +(5*r**8)/(3*(-1+y)**6)+(2*(6*r**6+5*r**8))/(3*(-1+y)**5)+(24*r**4
            +104*r**6+15*r**8)/(12*(-1+y)**4)+(20*r**4+48*r**6-5*r**8)/(12*(-1
            +y)**3)+(-16*r**2-51*r**4-4*r**6)/(6*(-1+y)**2)+(-40*r**2-101*r**4
            +4*r**6)/(6*(-1+y))+((-96+16*r**2-34*r**4-8*r**6+5*r**8)*y)/12-(4*(3
            -2*r**2+r**4)*y**2)/3+((21-20*r**2+3*r**4)*y**3)/18+y**5/60)
    if (dEl == 0 and dr == 0 and i==0 and j == 2):
        return     ((-2*logy*(12-8*r**2-21*r**4+2*r**6))/3+(618-1936*r**2+420*r**4-76*r**6
            -134*r**8+117*r**10-28*r**12)/36+r**10/(-1+y)**7+(25*r**8+11*r**10)/(6*(
            -1+y)**6)+(160*r**6+275*r**8+18*r**10)/(30*(-1+y)**5)+(240*r**4+1440*r**6
            +635*r**8-18*r**10)/(120*(-1+y)**4)+(48*r**4+184*r**6+15*r**8)/(36*(
            -1+y)**3)+(-64*r**2-320*r**4-88*r**6-5*r**8)/(24*(-1+y)**2)-(2*(12*r**2
            +55*r**4+2*r**6))/(3*(-1+y))-(2*(60-20*r**2+10*r**6-10*r**8+3*r**10)*y)/15
            +((-96+64*r**2-40*r**4-8*r**6+5*r**8)*y**2)/24-(8*r**2*y**3)/9-((1
            +r**2)*y**4)/12+y**6/360)
    if (dEl == 0 and dr == 0 and i==0 and j == 3):
        return     (4*logy*(-2+2*r**2+11*r**4)+(2012-8246*r**2-840*r**4+630*r**6-490*r**8
            +623*r**10-322*r**12+60*r**14)/105+(7*r**12)/(12*(-1+y)**8)+(7*r**10
            +2*r**12)/(2*(-1+y)**7)+(180*r**8+180*r**10+7*r**12)/(24*(-1+y)**6)
            +(800*r**6+2210*r**8+524*r**10-7*r**12)/(120*(-1+y)**5)+(480*r**4+3840*r**6
            +3215*r**8+126*r**10)/(240*(-1+y)**4)+(240*r**4+1240*r**6+605*r**8
            -42*r**10)/(240*(-1+y)**3)+(-128*r**2-968*r**4-716*r**6-15*r**8)/(48*(
            -1+y)**2)+(-448*r**2-3336*r**4-804*r**6+15*r**8)/(48*(-1+y))+((
            -192+96*r**2+236*r**4-50*r**6+48*r**8-31*r**10+7*r**12)*y)/24-((120
            -80*r**2+30*r**4+30*r**6-35*r**8+9*r**10)*y**2)/30-((38+28*r**2+10*r**4
            -20*r**6+5*r**8)*y**3)/48-y**4/3-((1+r**2)*y**5)/40+y**7/1680)
    if (dEl == 0 and dr == 0 and i==0 and j == 4):
        return     ((logy*(-192+256*r**2+2424*r**4+432*r**6-15*r**8))/24+(60645-311520*r**2
            -220920*r**4+86400*r**6-13980*r**8+22176*r**10-18390*r**12+7240*r**14
            -1125*r**16)/2880+r**14/(3*(-1+y)**9)+(63*r**12+13*r**14)/(24*(-1+y)**8)
            +(1344*r**10+917*r**12+24*r**14)/(168*(-1+y)**7)+(3920*r**8+6664*r**10
            +1043*r**12-8*r**14)/(336*(-1+y)**6)+(960*r**6+3840*r**8+1884*r**10
            +49*r**12)/(120*(-1+y)**5)+(960*r**4+9920*r**6+12920*r**8+2188*r**10
            -49*r**12)/(480*(-1+y)**4)+(240*r**4+1200*r**6+1535*r**8+54*r**10)/(360*(
            -1+y)**3)+(-640*r**2-6960*r**4-10000*r**6-955*r**8-18*r**10)/(240*(
            -1+y)**2)+(-256*r**2-2864*r**4-1648*r**6-15*r**8)/(24*(-1+y))-((1680
            -1120*r**2-7070*r**4-385*r**8+448*r**10-217*r**12+40*r**14)*y)/210
            +((-96+64*r**2+52*r**4-32*r**6+58*r**8-35*r**10+7*r**12)*y**2)/24
            +((-120-40*r**2+30*r**4+60*r**6-55*r**8+12*r**10)*y**3)/90-((61-16*r**2
            +9*r**4)*y**4)/96-(2*y**5)/15-((1+r**2)*y**6)/120+y**8/6720)
    if (dEl == 0 and dr == 0 and i==1 and j == 0):
        return     ((logy*(-24-8*r**2+69*r**4-27*r**6))/3+(729-2023*r**2-96*r**4-72*r**6
            -23*r**8-9*r**10)/36+(10*r**8)/(3*(-1+y)**6)+(2*(8*r**6+17*r**8))/(3*(
            -1+y)**5)+(12*r**4+92*r**6+75*r**8)/(6*(-1+y)**4)+(2*(2*r**4+12*r**6
            +5*r**8))/(3*(-1+y)**3)+(-8*r**2-57*r**4-39*r**6-5*r**8)/(3*(-1+y)**2)
            +(-8*r**2-57*r**4-9*r**6)/(-1+y)-(8*(3+2*r**2+r**6)*y)/3+((-24
            -8*r**2-3*r**4+r**6)*y**2)/6+(8*(-3+2*r**2)*y**3)/9+((11+r**2)*y**4)/12)
    if (dEl == 0 and dr == 0 and i==1 and j == 1):
        return     ((8*logy*(-3+25*r**4))/3+(2*(491-1830*r**2-1130*r**4+400*r**6-60*r**8
            -16*r**10+5*r**12))/45+(2*r**10)/(-1+y)**7+(20*r**8+19*r**10)/(3*(-1
            +y)**6)+(20*r**6+66*r**8+19*r**10)/(3*(-1+y)**5)+(12*r**4+116*r**6
            +141*r**8+9*r**10)/(6*(-1+y)**4)+(18*r**4+142*r**6+123*r**8-9*r**10)/(18*(
            -1+y)**3)+(-16*r**2-155*r**4-175*r**6-6*r**8)/(6*(-1+y)**2)+(-56*r**2
            -577*r**4-253*r**6+6*r**8)/(6*(-1+y))+((-48-24*r**2+103*r**4-25*r**6
            -3*r**8+r**10)*y)/6-(2*(6+2*r**2-r**4+r**6)*y**2)/3+((-48+24*r**2
            -11*r**4+r**6)*y**3)/18+y**5/15)
    if (dEl == 0 and dr == 0 and i==1 and j == 2):
        return     ((logy*(-48+16*r**2+842*r**4+223*r**6-11*r**8))/6+(16802-79780*r**2
            -115040*r**4+38750*r**6-2995*r**8-583*r**10+570*r**12-110*r**14)/720
            +(7*r**12)/(6*(-1+y)**8)+(12*r**10+7*r**12)/(2*(-1+y)**7)+(130*r**8
            +238*r**10+39*r**12)/(12*(-1+y)**6)+(80*r**6+370*r**8+222*r**10+7*r**12)/(10*(
            -1+y)**5)+(240*r**4+2880*r**6+4905*r**8+1055*r**10-21*r**12)/(120*(
            -1+y)**4)+(48*r**4+424*r**6+707*r**8+33*r**10)/(72*(-1+y)**3)+(-128*r**2
            -1664*r**4-3056*r**6-401*r**8-11*r**10)/(48*(-1+y)**2)+(-64*r**2-914*r**4
            -757*r**6-11*r**8)/(6*(-1+y))-((120+40*r**2-770*r**4+15*r**8-10*r**10
            +2*r**12)*y)/15+((-192-64*r**2+280*r**4-60*r**6-r**8+r**10)*y**2)/48
            -(2*(12-4*r**2+r**4+r**6)*y**3)/9-((31-5*r**2+8*r**4)*y**4)/48-y**5/15
            -((-5+r**2)*y**6)/720)
    if (dEl == 0 and dr == 0 and i==1 and j == 3):
        return     ((2*logy*(-12+8*r**2+387*r**4+225*r**6))/3+(31236-182484*r**2-472626*r**4
            +101010*r**6+13125*r**8-1701*r**10+1722*r**12-778*r**14+120*r**16)/1260
            +(2*r**14)/(3*(-1+y)**9)+(56*r**12+23*r**14)/(12*(-1+y)**8)+(1050*r**10
            +1274*r**12+141*r**14)/(84*(-1+y)**7)+(95*r**8+265*r**10+101*r**12
            +2*r**14)/(6*(-1+y)**6)+(560*r**6+3435*r**8+3315*r**10+416*r**12-4*r**14)/(60*(
            -1+y)**5)+(480*r**4+7040*r**6+15855*r**8+6573*r**10+144*r**12)/(240*(
            -1+y)**4)+(240*r**4+840*r**6+5075*r**8+2529*r**10-144*r**12)/(720*(
            -1+y)**3)+(-128*r**2-2184*r**4-6068*r**6-2023*r**8-21*r**10)/(48*(
            -1+y)**2)+(-576*r**2-11016*r**4-14780*r**6-1665*r**8+21*r**10)/(48*(
            -1+y))+((-480-80*r**2+6750*r**4+1665*r**6-135*r**8+63*r**10-30*r**12
            +5*r**14)*y)/60-((120+40*r**2-540*r**4+10*r**8-6*r**10+r**12)*y**2)/30
            -((384-64*r**2-264*r**4+84*r**6-31*r**8+3*r**10)*y**3)/144+((-13
            +r**2)*y**4)/12-((37-9*r**2+6*r**4)*y**5)/120-y**6/30-((-1+r**2)*y**7)/1680)
    if (dEl == 0 and dr == 0 and i==2 and j == 0):
        return     ((-4*logy*(6+4*r**2-131*r**4-56*r**6+9*r**8))/3+(1119-4780*r**2-11860*r**4
            +3100*r**6-335*r**8-26*r**10-5*r**12)/45+(4*r**10)/(-1+y)**7+(2*(5*r**8
            +9*r**10))/(-1+y)**6+(2*(20*r**6+105*r**8+76*r**10))/(5*(-1+y)**5)
            +(12*r**4+164*r**6+359*r**8+132*r**10)/(6*(-1+y)**4)+(6*r**4+76*r**6
            +183*r**8+36*r**10)/(9*(-1+y)**3)+(-16*r**2-242*r**4-564*r**6-149*r**8
            -12*r**10)/(6*(-1+y)**2)-(4*(8*r**2+139*r**4+164*r**6+9*r**8))/(3*(
            -1+y))-(8*(3+4*r**2-24*r**4+r**8)*y)/3+((-24-32*r**2+42*r**4
            -12*r**6+r**8)*y**2)/6+(8*(-3-2*r**2+r**4)*y**3)/9+(2*(-3+2*r**2)*y**4)/3
            +(8*y**5)/15-y**6/18)
    if (dEl == 0 and dr == 0 and i==2 and j == 1):
        return     ((8*logy*(-3-r**2+117*r**4+95*r**6))/3+(2*(4117-22071*r**2-85008*r**4
            +8540*r**6+4620*r**8-462*r**10-49*r**12+13*r**14))/315+(7*r**12)/(3*(
            -1+y)**8)+(10*(r**10+r**12))/(-1+y)**7+(90*r**8+264*r**10+95*r**12)/(6*(
            -1+y)**6)+(280*r**6+1910*r**8+2168*r**10+315*r**12)/(30*(-1+y)**5)
            +(120*r**4+1960*r**6+5390*r**8+3056*r**10+105*r**12)/(60*(-1+y)**4)
            +(12*r**4+128*r**6+664*r**8+384*r**10-21*r**12)/(36*(-1+y)**3)+(-32*r**2
            -614*r**4-2008*r**6-923*r**8-16*r**10)/(12*(-1+y)**2)+(-144*r**2-3226*r**4
            -5560*r**6-1005*r**8+16*r**10)/(12*(-1+y))+((-480-560*r**2+8310*r**4
            +3080*r**6-335*r**8-24*r**10+5*r**12)*y)/60-(2*(6+8*r**2-34*r**4
            +r**8)*y**2)/3+((-96-80*r**2+114*r**4-24*r**6+r**8)*y**3)/36+(2*(-3
            +r**2)*y**4)/3-((11-8*r**2+r**4)*y**5)/60+(2*y**6)/45-y**7/84)
    if (dEl == 0 and dr == 0 and i==2 and j == 2):
        return     ((logy*(-48+3080*r**4+3850*r**6+455*r**8-14*r**10))/6+(137941-899500*r**2
            -5021240*r**4-625660*r**6+712110*r**8-30240*r**10-868*r**12+1540*r**14
            -245*r**16)/5040+(4*r**14)/(3*(-1+y)**9)+(49*r**12+33*r**14)/(6*(-1
            +y)**8)+(798*r**10+1505*r**12+348*r**14)/(42*(-1+y)**7)+(1750*r**8
            +7210*r**10+5005*r**12+432*r**14)/(84*(-1+y)**6)+(320*r**6+2745*r**8
            +4368*r**10+1346*r**12+24*r**14)/(30*(-1+y)**5)+(240*r**4+4640*r**6
            +15615*r**8+12604*r**10+1618*r**12-24*r**14)/(120*(-1+y)**4)+(-88*r**6
            -3*r**8+281*r**10+9*r**12)/(18*(-1+y)**3)+(-32*r**2-768*r**4-3412*r**6
            -2581*r**8-187*r**10-3*r**12)/(12*(-1+y)**2)+(-80*r**2-2264*r**4-5414*r**6
            -1905*r**8-14*r**10)/(6*(-1+y))-(2*(140+140*r**2-4480*r**4-3360*r**6
            +14*r**10-7*r**12+r**14)*y)/35+((-48-64*r**2+616*r**4+186*r**6-19*r**8
            +r**10)*y**2)/12-(2*(12+12*r**2-44*r**4+r**8)*y**3)/9-((8-4*r**4+r**6)*y**4)/4
            +(2*(-5+r**2)*y**5)/15-((14-7*r**2+r**4)*y**6)/180-y**8/336)
    if (dEl == 0 and dr == 0 and i==3 and j == 0):
        return     (logy*(-8-8*r**2+585*r**4+925*r**6+165*r**8-15*r**10)+(11897-71323*r**2
            -526715*r**4-166775*r**6+98490*r**8-5642*r**10-217*r**12-25*r**14)/420
            +(14*r**12)/(3*(-1+y)**8)+(2*(8*r**10+13*r**12))/(-1+y)**7+(60*r**8
            +258*r**10+175*r**12)/(3*(-1+y)**6)+(2*(80*r**6+745*r**8+1348*r**10
            +490*r**12))/(15*(-1+y)**5)+(60*r**4+1260*r**6+4940*r**8+5079*r**10
            +1050*r**12)/(30*(-1+y)**4)+(-80*r**6+495*r**8+1257*r**10+140*r**12)/(30*(
            -1+y)**3)+(-32*r**2-836*r**4-4164*r**6-3895*r**8-505*r**10-28*r**12)/(12*(
            -1+y)**2)+(-40*r**2-1269*r**4-3609*r**6-1745*r**8-45*r**10)/(3*(-1
            +y))-(8*(3+6*r**2-110*r**4-110*r**6+r**10)*y)/3+((-240-560*r**2
            +3510*r**4+1470*r**6-185*r**8+9*r**10)*y**2)/60+(8*(-1-2*r**2+4*r**4)*y**3)/3
            -((24+24*r**2-19*r**4+r**6)*y**4)/12+(8*(-3+2*r**2)*y**5)/15-((-5
            +r**2)*y**6)/12-(8*y**7)/105)
    if (dEl == 0 and dr == 0 and i==3 and j == 1):
        return     ((8*logy*(-3-2*r**2+336*r**4+714*r**6+245*r**8))/3+(2*(4632-33452*r**2
            -325059*r**4-205506*r**6+73220*r**8+6888*r**10-483*r**12-26*r**14+6*r**16))/315
            +(8*r**14)/(3*(-1+y)**9)+(42*r**12+43*r**14)/(3*(-1+y)**8)+(588*r**10
            +1610*r**12+645*r**14)/(21*(-1+y)**7)+(5*(112*r**8+637*r**10+707*r**12
            +136*r**14))/(21*(-1+y)**6)+(36*r**6+407*r**8+954*r**10+551*r**12+48*r**14)/(3*(
            -1+y)**5)+(120*r**4+2920*r**6+13415*r**8+17413*r**10+5780*r**12+120*r**14)/(60*(
            -1+y)**4)+(-60*r**4-2820*r**6-6625*r**8+5157*r**10+2820*r**12-120*r**14)/(180*(
            -1+y)**3)+(-160*r**2-5070*r**4-32350*r**6-42685*r**8-10119*r**10-100*r**12)/(60*(
            -1+y)**2)+(-176*r**2-6802*r**4-24882*r**6-17755*r**8-1745*r**10+20*r**12)/(12*(
            -1+y))+((-1680-3080*r**2+102725*r**4+152705*r**6+23135*r**8-1477*r**10
            -70*r**12+10*r**14)*y)/210-((60+140*r**2-1690*r**4-1430*r**6+5*r**8
            +11*r**10)*y**2)/15+((-480-1040*r**2+4770*r**4+1410*r**6-155*r**8+3*r**10)*y**3)/180
            +(2*(-3-4*r**2+7*r**4)*y**4)/3-((48-8*r**2-11*r**4+r**6)*y**5)/30
            +((-9+5*r**2)*y**6)/45-((-17+5*r**2)*y**7)/420-(2*y**8)/105)
    if (dEl == 0 and dr == 0 and i==4 and j == 0):
        return     ((-2*logy*(12+16*r**2-2157*r**4-6840*r**6-4495*r**8-456*r**10+27*r**12))/3
            +(39213-312176*r**2-4693416*r**4-6027000*r**6+392840*r**8+671664*r**10
            -25158*r**12-584*r**14-45*r**16)/1260+(16*r**14)/(3*(-1+y)**9)+(2*(35*r**12
            +53*r**14))/(3*(-1+y)**8)+(2*(420*r**10+1589*r**12+1032*r**14))/(21*(
            -1+y)**7)+(100*r**8+748*r**10+1217*r**12+440*r**14)/(3*(-1+y)**6)
            +(2*(100*r**6+1430*r**8+4584*r**10+4223*r**12+920*r**14))/(15*(-1+y)**5)
            +(60*r**4+1780*r**6+10530*r**8+19560*r**10+11853*r**12+1560*r**14)/(30*(
            -1+y)**4)+(2*(-15*r**4-720*r**6-2965*r**8-240*r**10+1713*r**12+120*r**14))/(45*(
            -1+y)**3)+(-8*r**2-321*r**4-2752*r**6-5503*r**8-2640*r**10-197*r**12
            -8*r**14)/(3*(-1+y)**2)-(2*(24*r**2+1203*r**4+6152*r**6+7209*r**8+1896*r**10
            +27*r**12))/(3*(-1+y))-(8*(3+8*r**2-315*r**4-760*r**6-305*r**8+r**12)*y)/3
            +(2*(-30-100*r**2+1605*r**4+2590*r**6+410*r**8-30*r**10+r**12)*y**2)/15
            -(8*(3+10*r**2-66*r**4-54*r**6+r**8)*y**3)/9-((12+32*r**2-75*r**4
            -12*r**6+r**8)*y**4)/6+(8*(-3-4*r**2+3*r**4)*y**5)/15+(4*(-3+2*r**2)*y**6)/9
            -(8*(-5+2*r**2)*y**7)/105-y**8/12)

    # derivative wrt Elcuthat
    if (dEl == 1 and dr == 0 and i==0 and j == 0):
        return     ((-16*(3+r**4))/3-(80*r**6)/(3*(-1+y)**6)-(16*(r**4+3*r**6))/(-1+y)**5
            -(4*(3*r**4+4*r**6))/(-1+y)**4+(4*(8*r**2+17*r**4+4*r**6))/(3*(-1+y)**3)
            +(4*(8*r**2+9*r**4))/(3*(-1+y)**2)-(4*(4+3*r**4))/(-1+y)+(16*(-3
            +2*r**2)*y)/3+16*y**2+(4*y**3)/3)
    if (dEl == 1 and dr == 0 and i==0 and j == 1):
        return     ((-96+16*r**2-34*r**4-8*r**6+5*r**8)/6-(20*r**8)/(-1+y)**7-(20*(6*r**6
            +5*r**8))/(3*(-1+y)**6)-(2*(24*r**4+104*r**6+15*r**8))/(3*(-1+y)**5)
            +(-20*r**4-48*r**6+5*r**8)/(2*(-1+y)**4)+(2*(16*r**2+51*r**4+4*r**6))/(3*(
            -1+y)**3)+(40*r**2+101*r**4-4*r**6)/(3*(-1+y)**2)+(16*(-3+r**2))/(3*(
            -1+y))-(16*(3-2*r**2+r**4)*y)/3+((21-20*r**2+3*r**4)*y**2)/3+y**4/6)
    if (dEl == 1 and dr == 0 and i==0 and j == 2):
        return     ((-4*(60-20*r**2+10*r**6-10*r**8+3*r**10))/15-(14*r**10)/(-1+y)**8
            -(2*(25*r**8+11*r**10))/(-1+y)**7+(-160*r**6-275*r**8-18*r**10)/(3*(
            -1+y)**6)+(-240*r**4-1440*r**6-635*r**8+18*r**10)/(15*(-1+y)**5)+(
            -48*r**4-184*r**6-15*r**8)/(6*(-1+y)**4)+(64*r**2+320*r**4+88*r**6
            +5*r**8)/(6*(-1+y)**3)+(4*(12*r**2+55*r**4+2*r**6))/(3*(-1+y)**2)-(4*(12
            -8*r**2-21*r**4+2*r**6))/(3*(-1+y))+((-96+64*r**2-40*r**4-8*r**6
            +5*r**8)*y)/6-(16*r**2*y**2)/3-(2*(1+r**2)*y**3)/3+y**5/30)
    if (dEl == 1 and dr == 0 and i==0 and j == 3):
        return     ((-192+96*r**2+236*r**4-50*r**6+48*r**8-31*r**10+7*r**12)/12-(28*r**12)/(3*(
            -1+y)**9)-(7*(7*r**10+2*r**12))/(-1+y)**8+(-180*r**8-180*r**10-7*r**12)/(2*(
            -1+y)**7)+(-800*r**6-2210*r**8-524*r**10+7*r**12)/(12*(-1+y)**6)+(
            -480*r**4-3840*r**6-3215*r**8-126*r**10)/(30*(-1+y)**5)+(-240*r**4-1240*r**6
            -605*r**8+42*r**10)/(40*(-1+y)**4)+(128*r**2+968*r**4+716*r**6+15*r**8)/(12*(
            -1+y)**3)+(448*r**2+3336*r**4+804*r**6-15*r**8)/(24*(-1+y)**2)+(8*(
            -2+2*r**2+11*r**4))/(-1+y)-(2*(120-80*r**2+30*r**4+30*r**6-35*r**8
            +9*r**10)*y)/15-((38+28*r**2+10*r**4-20*r**6+5*r**8)*y**2)/8-(8*y**3)/3
            -((1+r**2)*y**4)/4+y**6/120)
    if (dEl == 1 and dr == 0 and i==0 and j == 4):
        return     ((-1680+1120*r**2+7070*r**4+385*r**8-448*r**10+217*r**12-40*r**14)/105
            -(6*r**14)/(-1+y)**10-(2*(63*r**12+13*r**14))/(3*(-1+y)**9)+(-1344*r**10
            -917*r**12-24*r**14)/(12*(-1+y)**8)+(-3920*r**8-6664*r**10-1043*r**12
            +8*r**14)/(28*(-1+y)**7)+(-960*r**6-3840*r**8-1884*r**10-49*r**12)/(12*(
            -1+y)**6)+(-960*r**4-9920*r**6-12920*r**8-2188*r**10+49*r**12)/(60*(
            -1+y)**5)+(-240*r**4-1200*r**6-1535*r**8-54*r**10)/(60*(-1+y)**4)
            +(640*r**2+6960*r**4+10000*r**6+955*r**8+18*r**10)/(60*(-1+y)**3)+(256*r**2
            +2864*r**4+1648*r**6+15*r**8)/(12*(-1+y)**2)+(-192+256*r**2+2424*r**4
            +432*r**6-15*r**8)/(12*(-1+y))+((-96+64*r**2+52*r**4-32*r**6+58*r**8
            -35*r**10+7*r**12)*y)/6+((-120-40*r**2+30*r**4+60*r**6-55*r**8+12*r**10)*y**2)/15
            -((61-16*r**2+9*r**4)*y**3)/12-(4*y**4)/3-((1+r**2)*y**5)/10+y**7/420)
    if (dEl == 1 and dr == 0 and i==1 and j == 0):
        return     ((-16*(3+2*r**2+r**6))/3-(40*r**8)/(-1+y)**7-(20*(8*r**6+17*r**8))/(3*(
            -1+y)**6)-(4*(12*r**4+92*r**6+75*r**8))/(3*(-1+y)**5)-(4*(2*r**4+12*r**6
            +5*r**8))/(-1+y)**4+(4*(8*r**2+57*r**4+39*r**6+5*r**8))/(3*(-1+y)**3)
            +(2*(8*r**2+57*r**4+9*r**6))/(-1+y)**2-(2*(24+8*r**2-69*r**4+27*r**6))/(3*(
            -1+y))+(2*(-24-8*r**2-3*r**4+r**6)*y)/3+(16*(-3+2*r**2)*y**2)/3
            +(2*(11+r**2)*y**3)/3)
    if (dEl == 1 and dr == 0 and i==1 and j == 1):
        return     ((-48-24*r**2+103*r**4-25*r**6-3*r**8+r**10)/3-(28*r**10)/(-1+y)**8
            -(4*(20*r**8+19*r**10))/(-1+y)**7-(10*(20*r**6+66*r**8+19*r**10))/(3*(
            -1+y)**6)-(4*(12*r**4+116*r**6+141*r**8+9*r**10))/(3*(-1+y)**5)+(
            -18*r**4-142*r**6-123*r**8+9*r**10)/(3*(-1+y)**4)+(2*(16*r**2+155*r**4
            +175*r**6+6*r**8))/(3*(-1+y)**3)+(56*r**2+577*r**4+253*r**6-6*r**8)/(3*(
            -1+y)**2)+(16*(-3+25*r**4))/(3*(-1+y))-(8*(6+2*r**2-r**4+r**6)*y)/3
            +((-48+24*r**2-11*r**4+r**6)*y**2)/3+(2*y**4)/3)
    if (dEl == 1 and dr == 0 and i==1 and j == 2):
        return     ((-2*(120+40*r**2-770*r**4+15*r**8-10*r**10+2*r**12))/15-(56*r**12)/(3*(
            -1+y)**9)-(7*(12*r**10+7*r**12))/(-1+y)**8+(-130*r**8-238*r**10-39*r**12)/(
            -1+y)**7+(-80*r**6-370*r**8-222*r**10-7*r**12)/(-1+y)**6+(-240*r**4
            -2880*r**6-4905*r**8-1055*r**10+21*r**12)/(15*(-1+y)**5)+(-48*r**4
            -424*r**6-707*r**8-33*r**10)/(12*(-1+y)**4)+(128*r**2+1664*r**4+3056*r**6
            +401*r**8+11*r**10)/(12*(-1+y)**3)+(64*r**2+914*r**4+757*r**6+11*r**8)/(3*(
            -1+y)**2)+(-48+16*r**2+842*r**4+223*r**6-11*r**8)/(3*(-1+y))+((
            -192-64*r**2+280*r**4-60*r**6-r**8+r**10)*y)/12-(4*(12-4*r**2+r**4
            +r**6)*y**2)/3-((31-5*r**2+8*r**4)*y**3)/6-(2*y**4)/3-((-5+r**2)*y**5)/60)
    if (dEl == 1 and dr == 0 and i==1 and j == 3):
        return     ((-480-80*r**2+6750*r**4+1665*r**6-135*r**8+63*r**10-30*r**12+5*r**14)/30
            -(12*r**14)/(-1+y)**10-(4*(56*r**12+23*r**14))/(3*(-1+y)**9)+(-1050*r**10
            -1274*r**12-141*r**14)/(6*(-1+y)**8)-(2*(95*r**8+265*r**10+101*r**12
            +2*r**14))/(-1+y)**7+(-560*r**6-3435*r**8-3315*r**10-416*r**12+4*r**14)/(6*(
            -1+y)**6)+(-480*r**4-7040*r**6-15855*r**8-6573*r**10-144*r**12)/(30*(
            -1+y)**5)+(-240*r**4-840*r**6-5075*r**8-2529*r**10+144*r**12)/(120*(
            -1+y)**4)+(128*r**2+2184*r**4+6068*r**6+2023*r**8+21*r**10)/(12*(-1
            +y)**3)+(576*r**2+11016*r**4+14780*r**6+1665*r**8-21*r**10)/(24*(-1
            +y)**2)+(4*(-12+8*r**2+387*r**4+225*r**6))/(3*(-1+y))-(2*(120+40*r**2
            -540*r**4+10*r**8-6*r**10+r**12)*y)/15-((384-64*r**2-264*r**4+84*r**6
            -31*r**8+3*r**10)*y**2)/24+(2*(-13+r**2)*y**3)/3-((37-9*r**2+6*r**4)*y**4)/12
            -(2*y**5)/5-((-1+r**2)*y**6)/120)
    if (dEl == 1 and dr == 0 and i==2 and j == 0):
        return     ((-16*(3+4*r**2-24*r**4+r**8))/3-(56*r**10)/(-1+y)**8-(24*(5*r**8+9*r**10))/(
            -1+y)**7-(4*(20*r**6+105*r**8+76*r**10))/(-1+y)**6-(4*(12*r**4+164*r**6
            +359*r**8+132*r**10))/(3*(-1+y)**5)-(2*(6*r**4+76*r**6+183*r**8+36*r**10))/(3*(
            -1+y)**4)+(2*(16*r**2+242*r**4+564*r**6+149*r**8+12*r**10))/(3*(-1
            +y)**3)+(8*(8*r**2+139*r**4+164*r**6+9*r**8))/(3*(-1+y)**2)-(8*(6
            +4*r**2-131*r**4-56*r**6+9*r**8))/(3*(-1+y))+(2*(-24-32*r**2+42*r**4
            -12*r**6+r**8)*y)/3+(16*(-3-2*r**2+r**4)*y**2)/3+(16*(-3+2*r**2)*y**3)/3
            +(16*y**4)/3-(2*y**5)/3)
    if (dEl == 1 and dr == 0 and i==2 and j == 1):
        return     ((-480-560*r**2+8310*r**4+3080*r**6-335*r**8-24*r**10+5*r**12)/30-(112*r**12)/(3*(
            -1+y)**9)-(140*(r**10+r**12))/(-1+y)**8-(2*(90*r**8+264*r**10+95*r**12))/(
            -1+y)**7+(-280*r**6-1910*r**8-2168*r**10-315*r**12)/(3*(-1+y)**6)
            -(2*(120*r**4+1960*r**6+5390*r**8+3056*r**10+105*r**12))/(15*(-1+y)**5)
            +(-12*r**4-128*r**6-664*r**8-384*r**10+21*r**12)/(6*(-1+y)**4)+(32*r**2
            +614*r**4+2008*r**6+923*r**8+16*r**10)/(3*(-1+y)**3)+(144*r**2+3226*r**4
            +5560*r**6+1005*r**8-16*r**10)/(6*(-1+y)**2)+(16*(-3-r**2+117*r**4
            +95*r**6))/(3*(-1+y))-(8*(6+8*r**2-34*r**4+r**8)*y)/3+((-96-80*r**2
            +114*r**4-24*r**6+r**8)*y**2)/6+(16*(-3+r**2)*y**3)/3-((11-8*r**2
            +r**4)*y**4)/6+(8*y**5)/15-y**6/6)
    if (dEl == 1 and dr == 0 and i==2 and j == 2):
        return     ((-4*(140+140*r**2-4480*r**4-3360*r**6+14*r**10-7*r**12+r**14))/35
            -(24*r**14)/(-1+y)**10-(8*(49*r**12+33*r**14))/(3*(-1+y)**9)+(-798*r**10
            -1505*r**12-348*r**14)/(3*(-1+y)**8)+(-1750*r**8-7210*r**10-5005*r**12
            -432*r**14)/(7*(-1+y)**7)+(-320*r**6-2745*r**8-4368*r**10-1346*r**12
            -24*r**14)/(3*(-1+y)**6)+(-240*r**4-4640*r**6-15615*r**8-12604*r**10
            -1618*r**12+24*r**14)/(15*(-1+y)**5)+(88*r**6+3*r**8-281*r**10-9*r**12)/(3*(
            -1+y)**4)+(32*r**2+768*r**4+3412*r**6+2581*r**8+187*r**10+3*r**12)/(3*(
            -1+y)**3)+(80*r**2+2264*r**4+5414*r**6+1905*r**8+14*r**10)/(3*(-1
            +y)**2)+(-48+3080*r**4+3850*r**6+455*r**8-14*r**10)/(3*(-1+y))+((
            -48-64*r**2+616*r**4+186*r**6-19*r**8+r**10)*y)/3-(4*(12+12*r**2
            -44*r**4+r**8)*y**2)/3-2*(8-4*r**4+r**6)*y**3+(4*(-5+r**2)*y**4)/3
            -((14-7*r**2+r**4)*y**5)/15-y**7/21)
    if (dEl == 1 and dr == 0 and i==3 and j == 0):
        return     ((-16*(3+6*r**2-110*r**4-110*r**6+r**10))/3-(224*r**12)/(3*(-1+y)**9)
            -(28*(8*r**10+13*r**12))/(-1+y)**8-(4*(60*r**8+258*r**10+175*r**12))/(
            -1+y)**7-(4*(80*r**6+745*r**8+1348*r**10+490*r**12))/(3*(-1+y)**6)
            -(4*(60*r**4+1260*r**6+4940*r**8+5079*r**10+1050*r**12))/(15*(-1+y)**5)
            +(80*r**6-495*r**8-1257*r**10-140*r**12)/(5*(-1+y)**4)+(32*r**2+836*r**4
            +4164*r**6+3895*r**8+505*r**10+28*r**12)/(3*(-1+y)**3)+(2*(40*r**2
            +1269*r**4+3609*r**6+1745*r**8+45*r**10))/(3*(-1+y)**2)-(2*(8+8*r**2
            -585*r**4-925*r**6-165*r**8+15*r**10))/(-1+y)+((-240-560*r**2+3510*r**4
            +1470*r**6-185*r**8+9*r**10)*y)/15+16*(-1-2*r**2+4*r**4)*y**2-(2*(24
            +24*r**2-19*r**4+r**6)*y**3)/3+(16*(-3+2*r**2)*y**4)/3-(-5+r**2)*y**5
            -(16*y**6)/15)
    if (dEl == 1 and dr == 0 and i==3 and j == 1):
        return     ((-1680-3080*r**2+102725*r**4+152705*r**6+23135*r**8-1477*r**10-70*r**12
            +10*r**14)/105-(48*r**14)/(-1+y)**10-(16*(42*r**12+43*r**14))/(3*(-1
            +y)**9)-(2*(588*r**10+1610*r**12+645*r**14))/(3*(-1+y)**8)-(20*(112*r**8
            +637*r**10+707*r**12+136*r**14))/(7*(-1+y)**7)-(10*(36*r**6+407*r**8
            +954*r**10+551*r**12+48*r**14))/(3*(-1+y)**6)-(2*(120*r**4+2920*r**6
            +13415*r**8+17413*r**10+5780*r**12+120*r**14))/(15*(-1+y)**5)+(60*r**4
            +2820*r**6+6625*r**8-5157*r**10-2820*r**12+120*r**14)/(30*(-1+y)**4)
            +(160*r**2+5070*r**4+32350*r**6+42685*r**8+10119*r**10+100*r**12)/(15*(
            -1+y)**3)+(176*r**2+6802*r**4+24882*r**6+17755*r**8+1745*r**10-20*r**12)/(6*(
            -1+y)**2)+(16*(-3-2*r**2+336*r**4+714*r**6+245*r**8))/(3*(-1+y))
            -(4*(60+140*r**2-1690*r**4-1430*r**6+5*r**8+11*r**10)*y)/15+((-480
            -1040*r**2+4770*r**4+1410*r**6-155*r**8+3*r**10)*y**2)/30+(16*(-3-4*r**2
            +7*r**4)*y**3)/3-((48-8*r**2-11*r**4+r**6)*y**4)/3+(4*(-9+5*r**2)*y**5)/15
            -((-17+5*r**2)*y**6)/30-(32*y**7)/105)
    if (dEl == 1 and dr == 0 and i==4 and j == 0):
        return     ((-16*(3+8*r**2-315*r**4-760*r**6-305*r**8+r**12))/3-(96*r**14)/(-1
            +y)**10-(32*(35*r**12+53*r**14))/(3*(-1+y)**9)-(4*(420*r**10+1589*r**12
            +1032*r**14))/(3*(-1+y)**8)-(4*(100*r**8+748*r**10+1217*r**12+440*r**14))/(
            -1+y)**7-(4*(100*r**6+1430*r**8+4584*r**10+4223*r**12+920*r**14))/(3*(
            -1+y)**6)-(4*(60*r**4+1780*r**6+10530*r**8+19560*r**10+11853*r**12
            +1560*r**14))/(15*(-1+y)**5)-(4*(-15*r**4-720*r**6-2965*r**8-240*r**10
            +1713*r**12+120*r**14))/(15*(-1+y)**4)+(4*(8*r**2+321*r**4+2752*r**6
            +5503*r**8+2640*r**10+197*r**12+8*r**14))/(3*(-1+y)**3)+(4*(24*r**2
            +1203*r**4+6152*r**6+7209*r**8+1896*r**10+27*r**12))/(3*(-1+y)**2)
            -(4*(12+16*r**2-2157*r**4-6840*r**6-4495*r**8-456*r**10+27*r**12))/(3*(
            -1+y))+(8*(-30-100*r**2+1605*r**4+2590*r**6+410*r**8-30*r**10+r**12)*y)/15
            -(16*(3+10*r**2-66*r**4-54*r**6+r**8)*y**2)/3-(4*(12+32*r**2-75*r**4
            -12*r**6+r**8)*y**3)/3+(16*(-3-4*r**2+3*r**4)*y**4)/3+(16*(-3+2*r**2)*y**5)/3
            -(16*(-5+2*r**2)*y**6)/15-(4*y**7)/3)

    # derivative wrt r
    if (dEl == 0 and dr == 1 and i==0 and j == 0):
        return     ((-4*(-12+26*r**2-20*r**4+3*r**6+5*r**8))/(3*r)+(16*r**5)/(-1+y)**5
            +(4*(2*r**3+9*r**5))/(-1+y)**4+(8*(r**3+2*r**5))/(-1+y)**3-(4*(4*r
            +17*r**3+6*r**5))/(3*(-1+y)**2)-(8*(4*r+9*r**3))/(3*(-1+y))-(32*r**3*y)/3
            +(16*r*y**2)/3-24*r**3*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 1):
        return     ((4*(36-163*r**2+90*r**4-27*r**6-44*r**8+21*r**10))/(9*r)+(40*r**7)/(3*(
            -1+y)**6)+(8*(9*r**5+10*r**7))/(3*(-1+y)**5)+(2*(4*r**3+26*r**5+5*r**7))/(
            -1+y)**4-(2*(-10*r**3-36*r**5+5*r**7))/(3*(-1+y)**3)-(2*(8*r+51*r**3
            +6*r**5))/(3*(-1+y)**2)+(2*(-20*r-101*r**3+6*r**5))/(3*(-1+y))+(2*r*(4
            -17*r**2-6*r**4+5*r**6)*y)/3-(16*r*(-1+r**2)*y**2)/3+(2*r*(-10+3*r**2)*y**3)/9
            +(16*r*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==0 and j == 2):
        return     (-1/18*(-288+2128*r**2-336*r**4+180*r**6+536*r**8-585*r**10+168*r**12)/r
            +(10*r**9)/(-1+y)**7+(5*(20*r**7+11*r**9))/(3*(-1+y)**6)+(2*(48*r**5
            +110*r**7+9*r**9))/(3*(-1+y)**5)+(48*r**3+432*r**5+254*r**7-9*r**9)/(6*(
            -1+y)**4)+(2*(8*r**3+46*r**5+5*r**7))/(3*(-1+y)**3)+(-16*r-160*r**3
            -66*r**5-5*r**7)/(3*(-1+y)**2)-(8*(6*r+55*r**3+3*r**5))/(3*(-1+y))
            -(4*r*(-4+6*r**4-8*r**6+3*r**8)*y)/3+(r*(16-20*r**2-6*r**4+5*r**6)*y**2)/3
            -(16*r*y**3)/9-(r*y**4)/6-(8*r*(-4-21*r**2+3*r**4)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==0 and j == 3):
        return     ((2*(120-1298*r**2-900*r**4+270*r**6-280*r**8+445*r**10-276*r**12+60*r**14))/(15*r)
            +(7*r**11)/(-1+y)**8+(35*r**9+12*r**11)/(-1+y)**7+(120*r**7+150*r**9
            +7*r**11)/(2*(-1+y)**6)+(1200*r**5+4420*r**7+1310*r**9-21*r**11)/(30*(
            -1+y)**5)+(96*r**3+1152*r**5+1286*r**7+63*r**9)/(12*(-1+y)**4)+(48*r**3
            +372*r**5+242*r**7-21*r**9)/(12*(-1+y)**3)+(-32*r-484*r**3-537*r**5
            -15*r**7)/(6*(-1+y)**2)+(-112*r-1668*r**3-603*r**5+15*r**7)/(6*(-1
            +y))+(r*(96+472*r**2-150*r**4+192*r**6-155*r**8+42*r**10)*y)/12
            -(r*(-16+12*r**2+18*r**4-28*r**6+9*r**8)*y**2)/3-(r*(7+5*r**2-15*r**4
            +5*r**6)*y**3)/6-(r*y**5)/20+16*r*(1+11*r**2)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 4):
        return     (-1/72*(-1152+17112*r**2+36636*r**4-10368*r**6+2706*r**8-5544*r**10
            +5517*r**12-2534*r**14+450*r**16)/r+(14*r**13)/(3*(-1+y)**9)+(7*(54*r**11
            +13*r**13))/(12*(-1+y)**8)+(160*r**9+131*r**11+4*r**13)/(2*(-1+y)**7)
            +(1120*r**7+2380*r**9+447*r**11-4*r**13)/(12*(-1+y)**6)+(480*r**5+2560*r**7
            +1570*r**9+49*r**11)/(10*(-1+y)**5)+(960*r**3+14880*r**5+25840*r**7
            +5470*r**9-147*r**11)/(120*(-1+y)**4)+(48*r**3+360*r**5+614*r**7+27*r**9)/(18*(
            -1+y)**3)+(-64*r-1392*r**3-3000*r**5-382*r**7-9*r**9)/(12*(-1+y)**2)
            +(-64*r-1432*r**3-1236*r**5-15*r**7)/(3*(-1+y))-(2*r*(-80-1010*r**2
            -110*r**6+160*r**8-93*r**10+20*r**12)*y)/15+(r*(64+104*r**2-96*r**4
            +232*r**6-175*r**8+42*r**10)*y**2)/12+(4*r*(-2+3*r**2+9*r**4-11*r**6
            +3*r**8)*y**3)/9-(r*(-8+9*r**2)*y**4)/24-(r*y**6)/60-(r*(-64-1212*r**2
            -324*r**4+15*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 0):
        return     (-1/18*(-288+1927*r**2+1020*r**4-108*r**6+92*r**8+45*r**10)/r+(80*r**7)/(3*(
            -1+y)**6)+(16*(6*r**5+17*r**7))/(3*(-1+y)**5)+(4*(2*r**3+23*r**5+25*r**7))/(
            -1+y)**4+(16*(r**3+9*r**5+5*r**7))/(3*(-1+y)**3)-(2*(8*r+114*r**3
            +117*r**5+20*r**7))/(3*(-1+y)**2)-(2*(8*r+114*r**3+27*r**5))/(-1+y)
            -(16*r*(2+3*r**4)*y)/3+(r*(-8-6*r**2+3*r**4)*y**2)/3+(32*r*y**3)/9
            +(r*y**4)/6-(2*r*(8-138*r**2+81*r**4)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 1):
        return     ((8*(18-183*r**2-376*r**4+120*r**6-24*r**8-8*r**10+3*r**12))/(9*r)
            +(20*r**9)/(-1+y)**7+(10*(16*r**7+19*r**9))/(3*(-1+y)**6)+(2*(60*r**5
            +264*r**7+95*r**9))/(3*(-1+y)**5)+(8*r**3+116*r**5+188*r**7+15*r**9)/(
            -1+y)**4+(12*r**3+142*r**5+164*r**7-15*r**9)/(3*(-1+y)**3)+(-16*r
            -310*r**3-525*r**5-24*r**7)/(3*(-1+y)**2)+(-56*r-1154*r**3-759*r**5
            +24*r**7)/(3*(-1+y))+(r*(-24+206*r**2-75*r**4-12*r**6+5*r**8)*y)/3
            -(4*r*(2-2*r**2+3*r**4)*y**2)/3+(r*(24-22*r**2+3*r**4)*y**3)/9+(800*r**3*math.log((1
            -y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 2):
        return     (-1/72*(-1152+16340*r**2+66224*r**4-17898*r**6+2132*r**8+583*r**10-684*r**12
            +154*r**14)/r+(14*r**11)/(-1+y)**8+(6*(10*r**9+7*r**11))/(-1+y)**7
            +(260*r**7+595*r**9+117*r**11)/(3*(-1+y)**6)+(2*(120*r**5+740*r**7
            +555*r**9+21*r**11))/(5*(-1+y)**5)+(480*r**3+8640*r**5+19620*r**7+5275*r**9
            -126*r**11)/(60*(-1+y)**4)+(96*r**3+1272*r**5+2828*r**7+165*r**9)/(36*(
            -1+y)**3)+(-128*r-3328*r**3-9168*r**5-1604*r**7-55*r**9)/(24*(-1
            +y)**2)+(-64*r-1828*r**3-2271*r**5-44*r**7)/(3*(-1+y))-(4*r*(20
            -770*r**2+30*r**6-25*r**8+6*r**10)*y)/15+(r*(-64+560*r**2-180*r**4
            -4*r**6+5*r**8)*y**2)/24-(4*r*(-4+2*r**2+3*r**4)*y**3)/9-(r*(-5+16*r**2)*y**4)/24
            -(r*y**6)/360-(r*(-16-1684*r**2-669*r**4+44*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 3):
        return     ((10080-189204*r**2-1270332*r**4+114030*r**6+52500*r**8-8505*r**10+10332*r**12
            -5446*r**14+960*r**16)/(630*r)+(28*r**13)/(3*(-1+y)**9)+(7*(48*r**11
            +23*r**13))/(6*(-1+y)**8)+(250*r**9+364*r**11+47*r**13)/(2*(-1+y)**7)
            +(380*r**7+1325*r**9+606*r**11+14*r**13)/(3*(-1+y)**6)+(1680*r**5+13740*r**7
            +16575*r**9+2496*r**11-28*r**13)/(30*(-1+y)**5)+(320*r**3+7040*r**5
            +21140*r**7+10955*r**9+288*r**11)/(40*(-1+y)**4)+(480*r**3+2520*r**5
            +20300*r**7+12645*r**9-864*r**11)/(360*(-1+y)**3)+(-128*r-4368*r**3
            -18204*r**5-8092*r**7-105*r**9)/(24*(-1+y)**2)+(-192*r-7344*r**3-14780*r**5
            -2220*r**7+35*r**9)/(8*(-1+y))+(r*(-16+2700*r**2+999*r**4-108*r**6
            +63*r**8-36*r**10+7*r**12)*y)/6-(2*r*(20-540*r**2+20*r**6-15*r**8
            +3*r**10)*y**2)/15-(r*(-64-528*r**2+252*r**4-124*r**6+15*r**8)*y**3)/72
            +(r*y**4)/6-(r*(-3+4*r**2)*y**5)/20-(r*y**7)/840+(4*r*(8+774*r**2
            +675*r**4)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==2 and j == 0):
        return     ((-4*(-36+454*r**2+3158*r**4-594*r**6+80*r**8+13*r**10+3*r**12))/(9*r)
            +(40*r**9)/(-1+y)**7+(20*(4*r**7+9*r**9))/(-1+y)**6+(16*(3*r**5+21*r**7
            +19*r**9))/(-1+y)**5+(4*(6*r**3+123*r**5+359*r**7+165*r**9))/(3*(-1
            +y)**4)+(8*(r**3+19*r**5+61*r**7+15*r**9))/(3*(-1+y)**3)-(4*(4*r
            +121*r**3+423*r**5+149*r**7+15*r**9))/(3*(-1+y)**2)-(16*(4*r+139*r**3
            +246*r**5+18*r**7))/(3*(-1+y))-(64*r*(1-12*r**2+r**6)*y)/3+(4*r*(
            -8+21*r**2-9*r**4+r**6)*y**2)/3+(32*r*(-1+r**2)*y**3)/9+(8*r*y**4)/3
            -(16*r*(2-131*r**2-84*r**4+18*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==2 and j == 1):
        return     ((4*(180-3093*r**2-31308*r**4-2040*r**6+2640*r**8-330*r**10-42*r**12
            +13*r**14))/(45*r)+(28*r**11)/(-1+y)**8+(20*(5*r**9+6*r**11))/(-1+y)**7
            +(10*(12*r**7+44*r**9+19*r**11))/(-1+y)**6+(2*(84*r**5+764*r**7+1084*r**9
            +189*r**11))/(3*(-1+y)**5)+(24*r**3+588*r**5+2156*r**7+1528*r**9+63*r**11)/(3*(
            -1+y)**4)+(12*r**3+192*r**5+1328*r**7+960*r**9-63*r**11)/(9*(-1+y)**3)
            -(2*(8*r+307*r**3+1506*r**5+923*r**7+20*r**9))/(3*(-1+y)**2)+(2*(
            -36*r-1613*r**3-4170*r**5-1005*r**7+20*r**9))/(3*(-1+y))+(r*(-56
            +1662*r**2+924*r**4-134*r**6-12*r**8+3*r**10)*y)/3-(16*r*(2-17*r**2
            +r**6)*y**2)/3+(2*r*(-20+57*r**2-18*r**4+r**6)*y**3)/9+(4*r*y**4)/3
            -(r*(-4+r**2)*y**5)/15+(16*r*(-1+234*r**2+285*r**4)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==2 and j == 2):
        return     (-1/90*(-1440+32125*r**2+451060*r**4+182535*r**6-88080*r**8+4980*r**10
            +186*r**12-385*r**14+70*r**16)/r+(56*r**13)/(3*(-1+y)**9)+(7*(14*r**11
            +11*r**13))/(-1+y)**8+(2*(95*r**9+215*r**11+58*r**13))/(-1+y)**7+(500*r**7
            +2575*r**9+2145*r**11+216*r**13)/(3*(-1+y)**6)+(4*(80*r**5+915*r**7
            +1820*r**9+673*r**11+14*r**13))/(5*(-1+y)**5)+(120*r**3+3480*r**5+15615*r**7
            +15755*r**9+2427*r**11-42*r**13)/(15*(-1+y)**4)+(-264*r**5-12*r**7
            +1405*r**9+54*r**11)/(9*(-1+y)**3)+(-32*r-1536*r**3-10236*r**5-10324*r**7
            -935*r**9-18*r**11)/(6*(-1+y)**2)-(2*(40*r+2264*r**3+8121*r**5+3810*r**7
            +35*r**9))/(3*(-1+y))-(4*r*(20-1280*r**2-1440*r**4+10*r**8-6*r**10
            +r**12)*y)/5+(r*(-64+1232*r**2+558*r**4-76*r**6+5*r**8)*y**2)/6-(16*r*(3
            -22*r**2+r**6)*y**3)/9-(r**3*(-8+3*r**2)*y**4)/2+(4*r*y**5)/15-(r*(-7
            +2*r**2)*y**6)/90-(70*r**3*(-88-165*r**2-26*r**4+r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==3 and j == 0):
        return     (-1/30*(-480+9709*r**2+185590*r**4+126975*r**6-46380*r**8+3130*r**10
            +186*r**12+25*r**14)/r+(56*r**11)/(-1+y)**8+(8*(20*r**9+39*r**11))/(
            -1+y)**7+(20*(8*r**7+43*r**9+35*r**11))/(-1+y)**6+(16*(12*r**5+149*r**7
            +337*r**9+147*r**11))/(3*(-1+y)**5)+(24*r**3+756*r**5+3952*r**7+5079*r**9
            +1260*r**11)/(3*(-1+y)**4)+(-16*r**5+132*r**7+419*r**9+56*r**11)/(-1
            +y)**3+(-32*r-1672*r**3-12492*r**5-15580*r**7-2525*r**9-168*r**11)/(6*(
            -1+y)**2)-(2*(40*r+2538*r**3+10827*r**5+6980*r**7+225*r**9))/(3*(
            -1+y))-(16*r*(6-220*r**2-330*r**4+5*r**8)*y)/3+(r*(-112+1404*r**2
            +882*r**4-148*r**6+9*r**8)*y**2)/6+(32*r*(-1+4*r**2)*y**3)/3-(r*(24
            -38*r**2+3*r**4)*y**4)/6+(32*r*y**5)/15-(r*y**6)/6-2*r*(8-1170*r**2
            -2775*r**4-660*r**6+75*r**8)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==3 and j == 1):
        return     ((8*(630-16306*r**2-395619*r**4-458199*r**6+94990*r**8+17220*r**10-1449*r**12
            -91*r**14+24*r**16))/(315*r)+(112*r**13)/(3*(-1+y)**9)+(14*(36*r**11
            +43*r**13))/(3*(-1+y)**8)+(10*(28*r**9+92*r**11+43*r**13))/(-1+y)**7
            +(10*(64*r**7+455*r**9+606*r**11+136*r**13))/(3*(-1+y)**6)+(4*(54*r**5
            +814*r**7+2385*r**9+1653*r**11+168*r**13))/(3*(-1+y)**5)+(48*r**3+1752*r**5
            +10732*r**7+17413*r**9+6936*r**11+168*r**13)/(6*(-1+y)**4)+(-24*r**3
            -1692*r**5-5300*r**7+5157*r**9+3384*r**11-168*r**13)/(18*(-1+y)**3)
            +(-32*r-2028*r**3-19410*r**5-34148*r**7-10119*r**9-120*r**11)/(6*(
            -1+y)**2)+(-176*r-13604*r**3-74646*r**5-71020*r**7-8725*r**9+120*r**11)/(6*(
            -1+y))+(r*(-88+5870*r**2+13089*r**4+2644*r**6-211*r**8-12*r**10
            +2*r**12)*y)/3-(2*r*(28-676*r**2-858*r**4+4*r**6+11*r**8)*y**2)/3+(r*(
            -208+1908*r**2+846*r**4-124*r**6+3*r**8)*y**3)/18+(8*r*(-2+7*r**2)*y**4)/3
            -(r*(-8-22*r**2+3*r**4)*y**5)/15+(2*r*y**6)/9-(r*y**7)/42+(32*r*(-1
            +336*r**2+1071*r**4+490*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==4 and j == 0):
        return     ((-2*(-2520+74684*r**2+2799678*r**4+5956650*r**6+551110*r**8-743820*r**10
            +32067*r**12+1022*r**14+90*r**16))/(315*r)+(224*r**13)/(3*(-1+y)**9)
            +(28*(30*r**11+53*r**13))/(3*(-1+y)**8)+(8*(50*r**9+227*r**11+172*r**13))/(
            -1+y)**7+(4*(200*r**7+1870*r**9+3651*r**11+1540*r**13))/(3*(-1+y)**6)
            +(8*(150*r**5+2860*r**7+11460*r**9+12669*r**11+3220*r**13))/(15*(-1
            +y)**5)+(2*(20*r**3+890*r**5+7020*r**7+16300*r**9+11853*r**11+1820*r**13))/(5*(
            -1+y)**4)+(8*(-15*r**3-1080*r**5-5930*r**7-600*r**9+5139*r**11+420*r**13))/(45*(
            -1+y)**3)-(4*(4*r+321*r**3+4128*r**5+11006*r**7+6600*r**9+591*r**11
            +28*r**13))/(3*(-1+y)**2)-(8*(4*r+401*r**3+3076*r**5+4806*r**7+1580*r**9
            +27*r**11))/(-1+y)-(32*r*(4-315*r**2-1140*r**4-610*r**6+3*r**10)*y)/3
            +(8*r*(-50+1605*r**2+3885*r**4+820*r**6-75*r**8+3*r**10)*y**2)/15-(32*r*(5
            -66*r**2-81*r**4+2*r**6)*y**3)/9-(2*r*(16-75*r**2-18*r**4+2*r**6)*y**4)/3
            +(32*r*(-2+3*r**2)*y**5)/15+(16*r*y**6)/9-(32*r*y**7)/105-(8*r*(8-2157*r**2
            -10260*r**4-8990*r**6-1140*r**8+81*r**10)*math.log((1-y)/r**2))/3)
    return 0. 

@jit(float64(int64, int64, float64, float64, int64, int64), cache=True, nopython=True)
def XLS(i,j,elcuthat,r,dEl,dr):
    """ Tree level functions (rhoLS) for mix moments and their derivatives """
    y  = 2*elcuthat
    logy = math.log((1-y)/r**2)
    # tree level function
    if (dEl == 0 and dr == 0 and i==0 and j == 0):
        return     (-6*logy*r**4+(3-8*r**2+14*r**4-20*r**6+5*r**8)/2+(2*r**6)/(-1+y)**4
            +(2*(3*r**4+4*r**6))/(3*(-1+y)**3)+(3*r**4-4*r**6)/(3*(-1+y)**2)-(6*r**4)/(
            -1+y)+(8*r**4*(-3+r**2)*y)/3+y**4/6)
    if (dEl == 0 and dr == 0 and i==0 and j == 1):
        return     ((21-105*r**2+105*r**4-165*r**6+95*r**8-21*r**10)/15+r**8/(-1+y)**5
            +(12*r**6+5*r**8)/(4*(-1+y)**4)+(24*r**4+56*r**6-5*r**8)/(12*(-1+y)**3)
            +(15*r**4-4*r**6)/(6*(-1+y)**2)+(-39*r**4+4*r**6)/(6*(-1+y))-(r**4*(22
            -16*r**2+5*r**4)*y)/4+(2*r**4*(-3+r**2)*y**2)/3+((-3-4*r**2+3*r**4)*y**3)/6
            +y**5/60)
    if (dEl == 0 and dr == 0 and i==0 and j == 2):
        return     ((-4*logy*r**4*(-6+r**2))/3+(44-324*r**2+84*r**4-240*r**6+288*r**8
            -147*r**10+28*r**12)/36+r**10/(2*(-1+y)**6)+(25*r**8+6*r**10)/(10*(
            -1+y)**5)+(160*r**6+165*r**8-6*r**10)/(40*(-1+y)**4)+(24*r**4+96*r**6
            +5*r**8)/(12*(-1+y)**3)+(96*r**4+32*r**6-5*r**8)/(24*(-1+y)**2)-(2*(9*r**4
            +2*r**6))/(3*(-1+y))+(r**6*(40-35*r**2+9*r**4)*y)/15-(r**4*(16-16*r**2
            +5*r**4)*y**2)/8-(2*(3-3*r**4+2*r**6)*y**3)/9-((1+r**2)*y**4)/12+y**6/360)
    if (dEl == 0 and dr == 0 and i==0 and j == 3):
        return     (18*logy*r**4+(144-1428*r**2-1330*r**4+630*r**6+805*r**8-861*r**10
            +364*r**12-60*r**14)/140+r**12/(4*(-1+y)**7)+(7*(6*r**10+r**12))/(24*(
            -1+y)**6)+(540*r**8+348*r**10-7*r**12)/(120*(-1+y)**5)+(400*r**6+765*r**8
            +42*r**10)/(80*(-1+y)**4)+(480*r**4+3040*r**6+1005*r**8-42*r**10)/(240*(
            -1+y)**3)+(264*r**4+284*r**6-15*r**8)/(48*(-1+y)**2)+(-72*r**4-100*r**6
            +5*r**8)/(16*(-1+y))-(r**4*(-204+2*r**2+54*r**4-33*r**6+7*r**8)*y)/24
            +(r**6*(40-35*r**2+9*r**4)*y**2)/20+((-10+12*r**2+2*r**4-12*r**6+5*r**8)*y**3)/16
            -y**4/6-((1+r**2)*y**5)/40+y**7/1680)
    if (dEl == 0 and dr == 0 and i==0 and j == 4):
        return     ((-5*logy*r**4*(-48-16*r**2+r**4))/8+(815-10400*r**2-28920*r**4+19200*r**6
            +1580*r**8-5904*r**10+4390*r**12-1560*r**14+225*r**16)/960+r**14/(8*(
            -1+y)**8)+(63*r**12+8*r**14)/(56*(-1+y)**7)+(1344*r**10+623*r**12
            -8*r**14)/(336*(-1+y)**6)+(840*r**8+1032*r**10+49*r**12)/(120*(-1+y)**5)
            +(2880*r**6+8880*r**8+2148*r**10-49*r**12)/(480*(-1+y)**4)+(240*r**4
            +2240*r**6+1645*r**8+18*r**10)/(120*(-1+y)**3)+(1680*r**4+3440*r**6
            +285*r**8-18*r**10)/(240*(-1+y)**2)+(-48*r**4-320*r**6-15*r**8)/(24*(
            -1+y))+(r**4*(1400-105*r**4+119*r**6-56*r**8+10*r**10)*y)/70-(r**4*(
            -96-16*r**2+54*r**4-33*r**6+7*r**8)*y**2)/24-((15-35*r**2+20*r**6
            -20*r**8+6*r**10)*y**3)/30-((7-8*r**2+3*r**4)*y**4)/32-y**5/15-((1
            +r**2)*y**6)/120+y**8/6720)
    if (dEl == 0 and dr == 0 and i==1 and j == 0):
        return     (-(logy*r**2*(4-3*r**2+9*r**4))+(15+7*r**2-84*r**4-12*r**6-61*r**8
            +9*r**10)/12+(2*r**8)/(-1+y)**5+(8*r**6+11*r**8)/(2*(-1+y)**4)+(2*(3*r**4
            +15*r**6+5*r**8))/(3*(-1+y)**3)+(12*r**4+6*r**6-5*r**8)/(3*(-1+y)**2)
            +(-5*r**4-9*r**6)/(-1+y)+(4*r**2*(-3-3*r**2-6*r**4+r**6)*y)/3+(r**2*(
            -4-7*r**2+r**4)*y**2)/2-(4*r**2*y**3)/3+((-9+r**2)*y**4)/12)
    if (dEl == 0 and dr == 0 and i==1 and j == 1):
        return     (4*logy*r**2*(-1+3*r**2)+(16+5*r**2-345*r**4+155*r**6-95*r**8+33*r**10
            -5*r**12)/15+r**10/(-1+y)**6+(20*r**8+13*r**10)/(5*(-1+y)**5)+(10*r**6
            +23*r**8+3*r**10)/(2*(-1+y)**4)+(12*r**4+88*r**6+51*r**8-3*r**10)/(6*(
            -1+y)**3)+(33*r**4+49*r**6-6*r**8)/(6*(-1+y)**2)+(-15*r**4-83*r**6
            +6*r**8)/(6*(-1+y))-(r**2*(24-27*r**2+37*r**4-15*r**6+3*r**8)*y)/6
            -(r**2*(6+3*r**2+r**4)*y**2)/3+(r**2*(-8-3*r**2+r**4)*y**3)/6-(2*y**4)/3
            -(2*y**5)/15)
    if (dEl == 0 and dr == 0 and i==1 and j == 2):
        return     (-1/6*(logy*r**2*(24-132*r**2-113*r**4+11*r**6))+(650+340*r**2-32460*r**4
            +14850*r**6-3465*r**8+2067*r**10-770*r**12+110*r**14)/720+r**12/(2*(
            -1+y)**7)+(12*r**10+5*r**12)/(4*(-1+y)**6)+(65*r**8+87*r**10+7*r**12)/(10*(
            -1+y)**5)+(240*r**6+840*r**8+284*r**10-7*r**12)/(40*(-1+y)**4)+(48*r**4
            +496*r**6+489*r**8+11*r**10)/(24*(-1+y)**3)+(336*r**4+880*r**6+107*r**8
            -11*r**10)/(48*(-1+y)**2)+(6*r**4-117*r**6-11*r**8)/(6*(-1+y))+(r**2*(
            -120+450*r**2+20*r**4+45*r**6-32*r**8+6*r**10)*y)/30-(r**2*(96-144*r**2
            +52*r**4-15*r**6+3*r**8)*y**2)/48-(r**2*(12-3*r**2-r**4+r**6)*y**3)/9
            -((25-3*r**2+8*r**4)*y**4)/48-(7*y**5)/30-((23+r**2)*y**6)/720)
    if (dEl == 0 and dr == 0 and i==1 and j == 3):
        return     (logy*r**2*(-4+33*r**2+55*r**4)+(640+732*r**2-62034*r**4+15750*r**6
            +7035*r**8+1239*r**10-1372*r**12+458*r**14-60*r**16)/840+r**14/(4*(
            -1+y)**8)+(56*r**12+17*r**14)/(28*(-1+y)**7)+(75*r**10+69*r**12+4*r**14)/(12*(
            -1+y)**6)+(285*r**8+618*r**10+144*r**12-2*r**14)/(30*(-1+y)**5)+(560*r**6
            +2795*r**8+1765*r**10+48*r**12)/(80*(-1+y)**4)+(160*r**4+2240*r**6
            +3435*r**8+545*r**10-16*r**12)/(80*(-1+y)**3)+(408*r**4+1620*r**6+613*r**8
            -21*r**10)/(48*(-1+y)**2)+(88*r**4-380*r**6-195*r**8+7*r**10)/(16*(
            -1+y))-(r**2*(48-330*r**2-237*r**4+14*r**6+12*r**8-6*r**10+r**12)*y)/12
            +(r**2*(-40+170*r**2+10*r**4+5*r**6-5*r**8+r**10)*y**2)/20+(r**2*(-64
            +120*r**2-20*r**4-7*r**6+3*r**8)*y**3)/48+((-9+r**2)*y**4)/24-((11
            -3*r**2+2*r**4)*y**5)/40-y**6/12-((15+r**2)*y**7)/1680)
    if (dEl == 0 and dr == 0 and i==2 and j == 0):
        return     (-4*logy*r**2*(-1+r**2)**2*(2+3*r**2)+(15+190*r**2-620*r**4+80*r**6
            -115*r**8-52*r**10+5*r**12)/15+(2*r**10)/(-1+y)**6+(2*(3*r**8+4*r**10))/(
            -1+y)**5+(60*r**6+235*r**8+108*r**10)/(10*(-1+y)**4)+(6*r**4+68*r**6
            +87*r**8+12*r**10)/(3*(-1+y)**3)+(14*r**4+44*r**6+7*r**8-4*r**10)/(2*(
            -1+y)**2)-(4*(-r**4+6*r**6+3*r**8))/(-1+y)+(4*r**2*(-10-10*r**4
            -10*r**6+r**8)*y)/5+(r**2*(-8-6*r**2-12*r**4+r**6)*y**2)/2-(8*r**2*(1
            +r**2)*y**3)/3-2*r**2*y**4-(4*y**5)/5-y**6/30)
    if (dEl == 0 and dr == 0 and i==2 and j == 1):
        return     (4*logy*r**2*(-2+3*r**2+15*r**4)+(90+1428*r**2-6972*r**4-1540*r**6
            +1785*r**8-525*r**10+119*r**12-13*r**14)/105+r**12/(-1+y)**7+(30*r**10
            +23*r**12)/(6*(-1+y)**6)+(90*r**8+204*r**10+49*r**12)/(10*(-1+y)**5)
            +(28*r**6+152*r**8+116*r**10+7*r**12)/(4*(-1+y)**4)+(120*r**4+1800*r**6
            +3290*r**8+836*r**10-35*r**12)/(60*(-1+y)**3)+(102*r**4+472*r**6+231*r**8
            -16*r**10)/(12*(-1+y)**2)+(114*r**4-280*r**6-315*r**8+16*r**10)/(12*(
            -1+y))-(r**2*(160-210*r**2-400*r**4+145*r**6-36*r**8+5*r**10)*y)/20
            -(r**2*(60-30*r**2+20*r**4+r**8)*y**2)/15+(r**2*(-32-6*r**2-8*r**4
            +r**6)*y**3)/12-2*r**2*y**4-((35+12*r**2+r**4)*y**5)/60-y**6/5-y**7/140)
    if (dEl == 0 and dr == 0 and i==2 and j == 2):
        return     (-1/6*(logy*r**2*(48-120*r**2-770*r**4-225*r**6+14*r**8))+(3707+74004*r**2
            -480200*r**4-349020*r**6+306810*r**8-27216*r**10+7924*r**12-2220*r**14
            +245*r**16)/5040+r**14/(2*(-1+y)**8)+(49*r**12+26*r**14)/(14*(-1+y)**7)
            +(266*r**10+399*r**12+64*r**14)/(28*(-1+y)**6)+(125*r**8+416*r**10
            +206*r**12+8*r**14)/(10*(-1+y)**5)+(320*r**6+2315*r**8+2672*r**10+446*r**12
            -8*r**14)/(40*(-1+y)**4)+(12*r**4+232*r**6+585*r**8+257*r**10+3*r**12)/(6*(
            -1+y)**3)+(600*r**4+3800*r**6+3225*r**8+199*r**10-15*r**12)/(60*(-1
            +y)**2)+(96*r**4-94*r**6-315*r**8-14*r**10)/(6*(-1+y))+(r**2*(-280
            +770*r**2+2450*r**4+35*r**6+35*r**8-21*r**10+3*r**12)*y)/35+(r**2*(
            -80+160*r**2+210*r**4-25*r**6+r**8)*y**2)/20-(2*r**2*(60-60*r**2-10*r**4
            +r**8)*y**3)/45-(r**2*(8-4*r**2+r**4)*y**4)/4-(2*(1+r**2)*y**5)/5-((50
            +3*r**2+r**4)*y**6)/180-(2*y**7)/35-y**8/560)
    if (dEl == 0 and dr == 0 and i==3 and j == 0):
        return     (-3*logy*r**2*(4+5*r**2-35*r**4-15*r**6+5*r**8)+(115+4011*r**2-7805*r**4
            -16765*r**6+7490*r**8-2030*r**10-371*r**12+25*r**14)/140+(2*r**12)/(
            -1+y)**7+(24*r**10+31*r**12)/(3*(-1+y)**6)+(2*(90*r**8+309*r**10+154*r**12))/(15*(
            -1+y)**5)+(80*r**6+615*r**8+804*r**10+182*r**12)/(10*(-1+y)**4)+(60*r**4
            +1220*r**6+3465*r**8+1971*r**10+140*r**12)/(30*(-1+y)**3)+(120*r**4
            +848*r**6+855*r**8+69*r**10-28*r**12)/(12*(-1+y)**2)-(3*(-7*r**4+r**6
            +25*r**8+5*r**10))/(-1+y)+(4*r**2*(-45-30*r**2+190*r**4-45*r**6-30*r**8
            +2*r**10)*y)/15+(3*r**2*(-40-30*r**2+10*r**4-55*r**6+3*r**8)*y**2)/20
            -4*r**2*(1+r**2+r**4)*y**3-(r**2*(36+33*r**2+r**4)*y**4)/12-(12*r**2*y**5)/5
            -((15+r**2)*y**6)/20-(4*y**7)/105)
    if (dEl == 0 and dr == 0 and i==3 and j == 1):
        return     (12*logy*r**2*(-1-r**2+17*r**4+15*r**6)+(75+3177*r**2-8043*r**4-26544*r**6
            +9170*r**8+2730*r**10-462*r**12+73*r**14-6*r**16)/105+r**14/(-1+y)**8
            +(6*r**12+5*r**14)/(-1+y)**7+(2*(147*r**10+329*r**12+100*r**14))/(21*(
            -1+y)**6)+(240*r**8+1131*r**10+955*r**12+120*r**14)/(15*(-1+y)**5)
            +(540*r**6+5295*r**8+9501*r**10+3596*r**12+120*r**14)/(60*(-1+y)**4)
            +(120*r**4+3040*r**6+11195*r**8+9045*r**10+1276*r**12-40*r**14)/(60*(
            -1+y)**3)+(690*r**4+6290*r**6+9195*r**8+2253*r**10-100*r**12)/(60*(
            -1+y)**2)+(354*r**4+338*r**6-1305*r**8-543*r**10+20*r**12)/(12*(-1
            +y))-(r**2*(840-35*r**2-9695*r**4-3605*r**6+595*r**8-98*r**10+10*r**12)*y)/70
            -(r**2*(180-30*r**2-890*r**4+45*r**6-3*r**8+2*r**10)*y**2)/30+(r**2*(
            -240-30*r**2+210*r**4-55*r**6+3*r**8)*y**3)/60-r**2*(3+r**2)*y**4-(r**2*(72
            -3*r**2+r**4)*y**5)/30-((15+13*r**2)*y**6)/30-((31+r**2)*y**7)/140-y**8/105)
    if (dEl == 0 and dr == 0 and i==4 and j == 0):
        return     (-2*logy*r**2*(8+33*r**2-120*r**4-245*r**6-48*r**8+9*r**10)+(291+19856*r**2
            -336*r**4-207480*r**6-29960*r**8+67200*r**10-9114*r**12-904*r**14+45*r**16)/420
            +(2*r**14)/(-1+y)**8+(2*(35*r**12+44*r**14))/(7*(-1+y)**7)+(420*r**10
            +1323*r**12+680*r**14)/(21*(-1+y)**6)+(2*(150*r**8+948*r**10+1215*r**12
            +320*r**14))/(15*(-1+y)**5)+(100*r**6+1270*r**8+3232*r**10+2103*r**12
            +280*r**14)/(10*(-1+y)**4)+(2*(15*r**4+480*r**6+2405*r**8+3012*r**10
            +963*r**12+40*r**14))/(15*(-1+y)**3)+(39*r**4+480*r**6+1057*r**8+528*r**10
            +27*r**12-8*r**14)/(3*(-1+y)**2)-(2*(-23*r**4-64*r**6+63*r**8+92*r**10
            +9*r**12))/(-1+y)+(8*r**2*(-42-105*r**2+525*r**4+560*r**6-42*r**8
            -21*r**10+r**12)*y)/21+(2*r**2*(-20-35*r**2+130*r**4+40*r**6-26*r**8
            +r**10)*y**2)/5-(8*r**2*(2+3*r**2-3*r**4+2*r**6)*y**3)/3-(r**2*(24+33*r**2
            +12*r**4+r**6)*y**4)/6-(16*r**2*(1+r**2)*y**5)/5-(8*r**2*y**6)/3-(8*(9
            +r**2)*y**7)/105-y**8/28)

    # derivative wrt Elcuthat
    if (dEl == 1 and dr == 0 and i==0 and j == 0):
        return     ((16*r**4*(-3+r**2))/3-(16*r**6)/(-1+y)**5-(4*(3*r**4+4*r**6))/(-1+y)**4
            +(4*(-3*r**4+4*r**6))/(3*(-1+y)**3)+(12*r**4)/(-1+y)**2-(12*r**4)/(
            -1+y)+(4*y**3)/3)
    if (dEl == 1 and dr == 0 and i==0 and j == 1):
        return     (-1/2*(r**4*(22-16*r**2+5*r**4))-(10*r**8)/(-1+y)**6-(2*(12*r**6+5*r**8))/(
            -1+y)**5+(-24*r**4-56*r**6+5*r**8)/(2*(-1+y)**4)+(2*(-15*r**4+4*r**6))/(3*(
            -1+y)**3)+(39*r**4-4*r**6)/(3*(-1+y)**2)+(8*r**4*(-3+r**2)*y)/3+(
            -3-4*r**2+3*r**4)*y**2+y**4/6)
    if (dEl == 1 and dr == 0 and i==0 and j == 2):
        return     ((2*r**6*(40-35*r**2+9*r**4))/15-(6*r**10)/(-1+y)**7+(-25*r**8-6*r**10)/(
            -1+y)**6+(-160*r**6-165*r**8+6*r**10)/(5*(-1+y)**5)+(-24*r**4-96*r**6
            -5*r**8)/(2*(-1+y)**4)+(-96*r**4-32*r**6+5*r**8)/(6*(-1+y)**3)+(4*(9*r**4
            +2*r**6))/(3*(-1+y)**2)-(8*(-6*r**4+r**6))/(3*(-1+y))-(r**4*(16-16*r**2
            +5*r**4)*y)/2-(4*(3-3*r**4+2*r**6)*y**2)/3-(2*(1+r**2)*y**3)/3+y**5/30)
    if (dEl == 1 and dr == 0 and i==0 and j == 3):
        return     (-1/12*(r**4*(-204+2*r**2+54*r**4-33*r**6+7*r**8))-(7*r**12)/(2*(-1+y)**8)
            -(7*(6*r**10+r**12))/(2*(-1+y)**7)+(-540*r**8-348*r**10+7*r**12)/(12*(
            -1+y)**6)+(-400*r**6-765*r**8-42*r**10)/(10*(-1+y)**5)+(-480*r**4
            -3040*r**6-1005*r**8+42*r**10)/(40*(-1+y)**4)+(-264*r**4-284*r**6+15*r**8)/(12*(
            -1+y)**3)+(72*r**4+100*r**6-5*r**8)/(8*(-1+y)**2)+(36*r**4)/(-1+y)
            +(r**6*(40-35*r**2+9*r**4)*y)/5+(3*(-10+12*r**2+2*r**4-12*r**6+5*r**8)*y**2)/8
            -(4*y**3)/3-((1+r**2)*y**4)/4+y**6/120)
    if (dEl == 1 and dr == 0 and i==0 and j == 4):
        return     ((r**4*(1400-105*r**4+119*r**6-56*r**8+10*r**10))/35-(2*r**14)/(-1+y)**9
            +(-63*r**12-8*r**14)/(4*(-1+y)**8)+(-1344*r**10-623*r**12+8*r**14)/(28*(
            -1+y)**7)+(-840*r**8-1032*r**10-49*r**12)/(12*(-1+y)**6)+(-2880*r**6
            -8880*r**8-2148*r**10+49*r**12)/(60*(-1+y)**5)+(-240*r**4-2240*r**6
            -1645*r**8-18*r**10)/(20*(-1+y)**4)+(-1680*r**4-3440*r**6-285*r**8
            +18*r**10)/(60*(-1+y)**3)+(48*r**4+320*r**6+15*r**8)/(12*(-1+y)**2)
            -(5*(-48*r**4-16*r**6+r**8))/(4*(-1+y))-(r**4*(-96-16*r**2+54*r**4
            -33*r**6+7*r**8)*y)/6-((15-35*r**2+20*r**6-20*r**8+6*r**10)*y**2)/5
            -((7-8*r**2+3*r**4)*y**3)/4-(2*y**4)/3-((1+r**2)*y**5)/10+y**7/420)
    if (dEl == 1 and dr == 0 and i==1 and j == 0):
        return     ((8*r**2*(-3-3*r**2-6*r**4+r**6))/3-(20*r**8)/(-1+y)**6-(4*(8*r**6+11*r**8))/(
            -1+y)**5-(4*(3*r**4+15*r**6+5*r**8))/(-1+y)**4+(4*(-12*r**4-6*r**6
            +5*r**8))/(3*(-1+y)**3)+(2*(5*r**4+9*r**6))/(-1+y)**2-(2*(4*r**2-3*r**4
            +9*r**6))/(-1+y)+2*r**2*(-4-7*r**2+r**4)*y-8*r**2*y**2+(2*(-9+r**2)*y**3)/3)
    if (dEl == 1 and dr == 0 and i==1 and j == 1):
        return     (-1/3*(r**2*(24-27*r**2+37*r**4-15*r**6+3*r**8))-(12*r**10)/(-1+y)**7
            -(2*(20*r**8+13*r**10))/(-1+y)**6-(4*(10*r**6+23*r**8+3*r**10))/(-1
            +y)**5+(-12*r**4-88*r**6-51*r**8+3*r**10)/(-1+y)**4+(2*(-33*r**4-49*r**6
            +6*r**8))/(3*(-1+y)**3)+(15*r**4+83*r**6-6*r**8)/(3*(-1+y)**2)+(8*(
            -r**2+3*r**4))/(-1+y)-(4*r**2*(6+3*r**2+r**4)*y)/3+r**2*(-8-3*r**2
            +r**4)*y**2-(16*y**3)/3-(4*y**4)/3)
    if (dEl == 1 and dr == 0 and i==1 and j == 2):
        return     ((r**2*(-120+450*r**2+20*r**4+45*r**6-32*r**8+6*r**10))/15-(7*r**12)/(
            -1+y)**8-(3*(12*r**10+5*r**12))/(-1+y)**7+(-65*r**8-87*r**10-7*r**12)/(
            -1+y)**6+(-240*r**6-840*r**8-284*r**10+7*r**12)/(5*(-1+y)**5)+(-48*r**4
            -496*r**6-489*r**8-11*r**10)/(4*(-1+y)**4)+(-336*r**4-880*r**6-107*r**8
            +11*r**10)/(12*(-1+y)**3)+(-6*r**4+117*r**6+11*r**8)/(3*(-1+y)**2)
            +(-24*r**2+132*r**4+113*r**6-11*r**8)/(3*(-1+y))-(r**2*(96-144*r**2
            +52*r**4-15*r**6+3*r**8)*y)/12-(2*r**2*(12-3*r**2-r**4+r**6)*y**2)/3
            -((25-3*r**2+8*r**4)*y**3)/6-(7*y**4)/3-((23+r**2)*y**5)/60)
    if (dEl == 1 and dr == 0 and i==1 and j == 3):
        return     (-1/6*(r**2*(48-330*r**2-237*r**4+14*r**6+12*r**8-6*r**10+r**12))-(4*r**14)/(
            -1+y)**9+(-56*r**12-17*r**14)/(2*(-1+y)**8)+(-75*r**10-69*r**12-4*r**14)/(
            -1+y)**7+(-285*r**8-618*r**10-144*r**12+2*r**14)/(3*(-1+y)**6)+(
            -560*r**6-2795*r**8-1765*r**10-48*r**12)/(10*(-1+y)**5)+(3*(-160*r**4
            -2240*r**6-3435*r**8-545*r**10+16*r**12))/(40*(-1+y)**4)+(-408*r**4
            -1620*r**6-613*r**8+21*r**10)/(12*(-1+y)**3)+(-88*r**4+380*r**6+195*r**8
            -7*r**10)/(8*(-1+y)**2)+(2*(-4*r**2+33*r**4+55*r**6))/(-1+y)+(r**2*(
            -40+170*r**2+10*r**4+5*r**6-5*r**8+r**10)*y)/5+(r**2*(-64+120*r**2
            -20*r**4-7*r**6+3*r**8)*y**2)/8+((-9+r**2)*y**3)/3-((11-3*r**2+2*r**4)*y**4)/4
            -y**5-((15+r**2)*y**6)/120)
    if (dEl == 1 and dr == 0 and i==2 and j == 0):
        return     ((8*r**2*(-10-10*r**4-10*r**6+r**8))/5-(24*r**10)/(-1+y)**7-(20*(3*r**8
            +4*r**10))/(-1+y)**6-(4*(60*r**6+235*r**8+108*r**10))/(5*(-1+y)**5)
            -(2*(6*r**4+68*r**6+87*r**8+12*r**10))/(-1+y)**4+(2*(-14*r**4-44*r**6
            -7*r**8+4*r**10))/(-1+y)**3+(8*(-r**4+6*r**6+3*r**8))/(-1+y)**2-(8*(2*r**2
            -r**4-4*r**6+3*r**8))/(-1+y)+2*r**2*(-8-6*r**2-12*r**4+r**6)*y-16*r**2*(1
            +r**2)*y**2-16*r**2*y**3-8*y**4-(2*y**5)/5)
    if (dEl == 1 and dr == 0 and i==2 and j == 1):
        return     (-1/10*(r**2*(160-210*r**2-400*r**4+145*r**6-36*r**8+5*r**10))-(14*r**12)/(
            -1+y)**8-(2*(30*r**10+23*r**12))/(-1+y)**7+(-90*r**8-204*r**10-49*r**12)/(
            -1+y)**6-(2*(28*r**6+152*r**8+116*r**10+7*r**12))/(-1+y)**5+(-120*r**4
            -1800*r**6-3290*r**8-836*r**10+35*r**12)/(10*(-1+y)**4)+(-102*r**4
            -472*r**6-231*r**8+16*r**10)/(3*(-1+y)**3)+(-114*r**4+280*r**6+315*r**8
            -16*r**10)/(6*(-1+y)**2)+(8*(-2*r**2+3*r**4+15*r**6))/(-1+y)-(4*r**2*(60
            -30*r**2+20*r**4+r**8)*y)/15+(r**2*(-32-6*r**2-8*r**4+r**6)*y**2)/2
            -16*r**2*y**3-((35+12*r**2+r**4)*y**4)/6-(12*y**5)/5-y**6/10)
    if (dEl == 1 and dr == 0 and i==2 and j == 2):
        return     ((2*r**2*(-280+770*r**2+2450*r**4+35*r**6+35*r**8-21*r**10+3*r**12))/35
            -(8*r**14)/(-1+y)**9+(-49*r**12-26*r**14)/(-1+y)**8-(3*(266*r**10+399*r**12
            +64*r**14))/(7*(-1+y)**7)+(-125*r**8-416*r**10-206*r**12-8*r**14)/(
            -1+y)**6+(-320*r**6-2315*r**8-2672*r**10-446*r**12+8*r**14)/(5*(-1
            +y)**5)+(-12*r**4-232*r**6-585*r**8-257*r**10-3*r**12)/(-1+y)**4+(
            -600*r**4-3800*r**6-3225*r**8-199*r**10+15*r**12)/(15*(-1+y)**3)+(
            -96*r**4+94*r**6+315*r**8+14*r**10)/(3*(-1+y)**2)+(-48*r**2+120*r**4
            +770*r**6+225*r**8-14*r**10)/(3*(-1+y))+(r**2*(-80+160*r**2+210*r**4
            -25*r**6+r**8)*y)/5-(4*r**2*(60-60*r**2-10*r**4+r**8)*y**2)/15-2*r**2*(8
            -4*r**2+r**4)*y**3-4*(1+r**2)*y**4-((50+3*r**2+r**4)*y**5)/15-(4*y**6)/5
            -y**7/35)
    if (dEl == 1 and dr == 0 and i==3 and j == 0):
        return     ((8*r**2*(-45-30*r**2+190*r**4-45*r**6-30*r**8+2*r**10))/15-(28*r**12)/(
            -1+y)**8-(4*(24*r**10+31*r**12))/(-1+y)**7-(4*(90*r**8+309*r**10+154*r**12))/(3*(
            -1+y)**6)-(4*(80*r**6+615*r**8+804*r**10+182*r**12))/(5*(-1+y)**5)
            +(-60*r**4-1220*r**6-3465*r**8-1971*r**10-140*r**12)/(5*(-1+y)**4)
            +(-120*r**4-848*r**6-855*r**8-69*r**10+28*r**12)/(3*(-1+y)**3)+(6*(
            -7*r**4+r**6+25*r**8+5*r**10))/(-1+y)**2-(6*(4*r**2+5*r**4-35*r**6
            -15*r**8+5*r**10))/(-1+y)+(3*r**2*(-40-30*r**2+10*r**4-55*r**6+3*r**8)*y)/5
            -24*r**2*(1+r**2+r**4)*y**2-(2*r**2*(36+33*r**2+r**4)*y**3)/3-24*r**2*y**4
            -(3*(15+r**2)*y**5)/5-(8*y**6)/15)
    if (dEl == 1 and dr == 0 and i==3 and j == 1):
        return     (-1/35*(r**2*(840-35*r**2-9695*r**4-3605*r**6+595*r**8-98*r**10+10*r**12))
            -(16*r**14)/(-1+y)**9-(14*(6*r**12+5*r**14))/(-1+y)**8-(8*(147*r**10
            +329*r**12+100*r**14))/(7*(-1+y)**7)-(2*(240*r**8+1131*r**10+955*r**12
            +120*r**14))/(3*(-1+y)**6)-(2*(540*r**6+5295*r**8+9501*r**10+3596*r**12
            +120*r**14))/(15*(-1+y)**5)+(-120*r**4-3040*r**6-11195*r**8-9045*r**10
            -1276*r**12+40*r**14)/(10*(-1+y)**4)+(-690*r**4-6290*r**6-9195*r**8
            -2253*r**10+100*r**12)/(15*(-1+y)**3)+(-354*r**4-338*r**6+1305*r**8
            +543*r**10-20*r**12)/(6*(-1+y)**2)+(24*(-r**2-r**4+17*r**6+15*r**8))/(
            -1+y)-(2*r**2*(180-30*r**2-890*r**4+45*r**6-3*r**8+2*r**10)*y)/15
            +(r**2*(-240-30*r**2+210*r**4-55*r**6+3*r**8)*y**2)/10-8*r**2*(3+r**2)*y**3
            -(r**2*(72-3*r**2+r**4)*y**4)/3-(2*(15+13*r**2)*y**5)/5-((31+r**2)*y**6)/10
            -(16*y**7)/105)
    if (dEl == 1 and dr == 0 and i==4 and j == 0):
        return     ((16*r**2*(-42-105*r**2+525*r**4+560*r**6-42*r**8-21*r**10+r**12))/21
            -(32*r**14)/(-1+y)**9-(4*(35*r**12+44*r**14))/(-1+y)**8-(4*(420*r**10
            +1323*r**12+680*r**14))/(7*(-1+y)**7)-(4*(150*r**8+948*r**10+1215*r**12
            +320*r**14))/(3*(-1+y)**6)-(4*(100*r**6+1270*r**8+3232*r**10+2103*r**12
            +280*r**14))/(5*(-1+y)**5)-(4*(15*r**4+480*r**6+2405*r**8+3012*r**10
            +963*r**12+40*r**14))/(5*(-1+y)**4)+(4*(-39*r**4-480*r**6-1057*r**8
            -528*r**10-27*r**12+8*r**14))/(3*(-1+y)**3)+(4*(-23*r**4-64*r**6+63*r**8
            +92*r**10+9*r**12))/(-1+y)**2-(4*(8*r**2+33*r**4-120*r**6-245*r**8
            -48*r**10+9*r**12))/(-1+y)+(8*r**2*(-20-35*r**2+130*r**4+40*r**6
            -26*r**8+r**10)*y)/5-16*r**2*(2+3*r**2-3*r**4+2*r**6)*y**2-(4*r**2*(24
            +33*r**2+12*r**4+r**6)*y**3)/3-32*r**2*(1+r**2)*y**4-32*r**2*y**5-(16*(9
            +r**2)*y**6)/15-(4*y**7)/7)

    # derivative wrt r
    if (dEl == 0 and dr == 1 and i==0 and j == 0):
        return     (4*r*(-2+10*r**2-15*r**4+5*r**6)+(12*r**5)/(-1+y)**4+(8*(r**3+2*r**5))/(
            -1+y)**3-(4*(-r**3+2*r**5))/(-1+y)**2-(24*r**3)/(-1+y)+16*r**3*(
            -2+r**2)*y-24*r**3*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 1):
        return     ((-2*r*(21-42*r**2+99*r**4-76*r**6+21*r**8))/3+(8*r**7)/(-1+y)**5+(2*(9*r**5
            +5*r**7))/(-1+y)**4-(2*(-12*r**3-42*r**5+5*r**7))/(3*(-1+y)**3)-(2*(
            -5*r**3+2*r**5))/(-1+y)**2+(2*(-13*r**3+2*r**5))/(-1+y)-2*r**3*(11
            -12*r**2+5*r**4)*y+4*r**3*(-2+r**2)*y**2+(2*r*(-2+3*r**2)*y**3)/3)
    if (dEl == 0 and dr == 1 and i==0 and j == 2):
        return     ((r*(-108-40*r**2-224*r**4+384*r**6-245*r**8+56*r**10))/6+(5*r**9)/(
            -1+y)**6+(2*(10*r**7+3*r**9))/(-1+y)**5-(3*(-16*r**5-22*r**7+r**9))/(2*(
            -1+y)**4)+(2*(12*r**3+72*r**5+5*r**7))/(3*(-1+y)**3)+(48*r**3+24*r**5
            -5*r**7)/(3*(-1+y)**2)-(8*(3*r**3+r**5))/(-1+y)+(2*r**5*(24-28*r**2
            +9*r**4)*y)/3-r**3*(8-12*r**2+5*r**4)*y**2-(8*r**3*(-1+r**2)*y**3)/3
            -(r*y**4)/6-8*r**3*(-4+r**2)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 3):
        return     (-1/10*(r*(204+740*r**2-270*r**4-460*r**6+615*r**8-312*r**10+60*r**12))
            +(3*r**11)/(-1+y)**7+(7*(5*r**9+r**11))/(2*(-1+y)**6)+(360*r**7+290*r**9
            -7*r**11)/(10*(-1+y)**5)+(3*(40*r**5+102*r**7+7*r**9))/(4*(-1+y)**4)
            +(32*r**3+304*r**5+134*r**7-7*r**9)/(4*(-1+y)**3)+(44*r**3+71*r**5
            -5*r**7)/(2*(-1+y)**2)+(-36*r**3-75*r**5+5*r**7)/(2*(-1+y))-(r**3*(
            -136+2*r**2+72*r**4-55*r**6+14*r**8)*y)/4+(r**5*(24-28*r**2+9*r**4)*y**2)/2
            +(r*(3+r**2-9*r**4+5*r**6)*y**3)/2-(r*y**5)/20+72*r**3*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==0 and j == 4):
        return     ((r*(-520-4332*r**2+2400*r**4+346*r**6-1476*r**8+1317*r**10-546*r**12
            +90*r**14))/24+(7*r**13)/(4*(-1+y)**8)+(27*r**11+4*r**13)/(2*(-1+y)**7)
            +(480*r**9+267*r**11-4*r**13)/(12*(-1+y)**6)+(560*r**7+860*r**9+49*r**11)/(10*(
            -1+y)**5)+(1440*r**5+5920*r**7+1790*r**9-49*r**11)/(40*(-1+y)**4)
            +(48*r**3+672*r**5+658*r**7+9*r**9)/(6*(-1+y)**3)+(112*r**3+344*r**5
            +38*r**7-3*r**9)/(4*(-1+y)**2)+(-8*r**3-80*r**5-5*r**7)/(-1+y)+(r**3*(400
            -60*r**4+85*r**6-48*r**8+10*r**10)*y)/5-(r**3*(-64-16*r**2+72*r**4
            -55*r**6+14*r**8)*y**2)/4-(r*(-7+12*r**4-16*r**6+6*r**8)*y**3)/3-(r*(
            -4+3*r**2)*y**4)/8-(r*y**6)/60-5*r**3*(-24-12*r**2+r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 0):
        return     ((r*(55-204*r**2+72*r**4-244*r**6+45*r**8))/6+(16*r**7)/(-1+y)**5+(4*(6*r**5
            +11*r**7))/(-1+y)**4+(4*(6*r**3+45*r**5+20*r**7))/(3*(-1+y)**3)-(4*(
            -12*r**3-9*r**5+10*r**7))/(3*(-1+y)**2)-(2*(10*r**3+27*r**5))/(-1+y)
            +(8*r*(-3-6*r**2-18*r**4+4*r**6)*y)/3+r*(-4-14*r**2+3*r**4)*y**2
            -(8*r*y**3)/3+(r*y**4)/6-2*r*(4-6*r**2+27*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 1):
        return     ((-2*r*(-13+174*r**2-93*r**4+76*r**6-33*r**8+6*r**10))/3+(10*r**9)/(
            -1+y)**6+(2*(16*r**7+13*r**9))/(-1+y)**5+(30*r**5+92*r**7+15*r**9)/(
            -1+y)**4+(8*r**3+88*r**5+68*r**7-5*r**9)/(-1+y)**3+(22*r**3+49*r**5
            -8*r**7)/(-1+y)**2+(-10*r**3-83*r**5+8*r**7)/(-1+y)-r*(8-18*r**2
            +37*r**4-20*r**6+5*r**8)*y-2*r*(2+2*r**2+r**4)*y**2+(r*(-8-6*r**2
            +3*r**4)*y**3)/3+8*r*(-1+6*r**2)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==1 and j == 2):
        return     ((r*(644-16152*r**2+6198*r**4-2508*r**6+2067*r**8-924*r**10+154*r**12))/72
            +(6*r**11)/(-1+y)**7+(15*(2*r**9+r**11))/(-1+y)**6+(260*r**7+435*r**9
            +42*r**11)/(5*(-1+y)**5)+(360*r**5+1680*r**7+710*r**9-21*r**11)/(10*(
            -1+y)**4)+(96*r**3+1488*r**5+1956*r**7+55*r**9)/(12*(-1+y)**3)+(672*r**3
            +2640*r**5+428*r**7-55*r**9)/(24*(-1+y)**2)+(12*r**3-351*r**5-44*r**7)/(3*(
            -1+y))+(4*r*(-30+225*r**2+15*r**4+45*r**6-40*r**8+9*r**10)*y)/15
            -(r*(32-96*r**2+52*r**4-20*r**6+5*r**8)*y**2)/8-(2*r*(12-6*r**2-3*r**4
            +4*r**6)*y**3)/9-(r*(-3+16*r**2)*y**4)/24-(r*y**6)/360-(r*(24-264*r**2
            -339*r**4+44*r**6)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==1 and j == 3):
        return     (-1/420*(r*(-4092+151788*r**2-1050*r**4-28140*r**6-6195*r**8+8232*r**10
            -3206*r**12+480*r**14))+(7*r**13)/(2*(-1+y)**8)+(48*r**11+17*r**13)/(2*(
            -1+y)**7)+(375*r**9+414*r**11+28*r**13)/(6*(-1+y)**6)-(2*(-570*r**7
            -1545*r**9-432*r**11+7*r**13))/(15*(-1+y)**5)+(1680*r**5+11180*r**7
            +8825*r**9+288*r**11)/(40*(-1+y)**4)+(320*r**3+6720*r**5+13740*r**7
            +2725*r**9-96*r**11)/(40*(-1+y)**3)+(816*r**3+4860*r**5+2452*r**7-105*r**9)/(24*(
            -1+y)**2)+(176*r**3-1140*r**5-780*r**7+35*r**9)/(8*(-1+y))-(r*(48
            -660*r**2-711*r**4+56*r**6+60*r**8-36*r**10+7*r**12)*y)/6+(r*(-40
            +340*r**2+30*r**4+20*r**6-25*r**8+6*r**10)*y**2)/10+(r*(-64+240*r**2
            -60*r**4-28*r**6+15*r**8)*y**3)/24+(r*y**4)/12-(r*(-3+4*r**2)*y**5)/20
            -(r*y**7)/840+2*r*(-4+66*r**2+165*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 0):
        return     ((4*r*(31-130*r**2-28*r**6-26*r**8+3*r**10))/3+(20*r**9)/(-1+y)**6
            +(16*(3*r**7+5*r**9))/(-1+y)**5+(4*(9*r**5+47*r**7+27*r**9))/(-1+y)**4
            +(8*(r**3+17*r**5+29*r**7+5*r**9))/(-1+y)**3-(4*(-7*r**3-33*r**5-7*r**7
            +5*r**9))/(-1+y)**2-(16*(-r**3+9*r**5+6*r**7))/(-1+y)+8*r*(-2-6*r**4
            -8*r**6+r**8)*y+4*r*(-2-3*r**2-9*r**4+r**6)*y**2-(16*r*(1+2*r**2)*y**3)/3
            -4*r*y**4-16*r*(1-r**2-6*r**4+6*r**6)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 1):
        return     ((-2*r*(-324+2172*r**2+1560*r**4-1020*r**6+375*r**8-102*r**10+13*r**12))/15
            +(12*r**11)/(-1+y)**7+(2*(25*r**9+23*r**11))/(-1+y)**6+(6*(60*r**7
            +170*r**9+49*r**11))/(5*(-1+y)**5)+(42*r**5+304*r**7+290*r**9+21*r**11)/(
            -1+y)**4+(24*r**3+540*r**5+1316*r**7+418*r**9-21*r**11)/(3*(-1+y)**3)
            -(2*(-51*r**3-354*r**5-231*r**7+20*r**9))/(3*(-1+y)**2)+(2*(57*r**3
            -210*r**5-315*r**7+20*r**9))/(3*(-1+y))-r*(16-42*r**2-120*r**4+58*r**6
            -18*r**8+3*r**10)*y-(2*r*(12-12*r**2+12*r**4+r**8)*y**2)/3+(2*r*(
            -8-3*r**2-6*r**4+r**6)*y**3)/3-4*r*y**4-(r*(6+r**2)*y**5)/15+8*r*(
            -2+6*r**2+45*r**4)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==2 and j == 2):
        return     ((r*(4083-37900*r**2-60495*r**4+37080*r**6-4440*r**8+1698*r**10-555*r**12
            +70*r**14))/90+(7*r**13)/(-1+y)**8+(2*(21*r**11+13*r**13))/(-1+y)**7
            +(95*r**9+171*r**11+32*r**13)/(-1+y)**6+(4*(125*r**7+520*r**9+309*r**11
            +14*r**13))/(5*(-1+y)**5)+(240*r**5+2315*r**7+3340*r**9+669*r**11-14*r**13)/(5*(
            -1+y)**4)+(24*r**3+696*r**5+2340*r**7+1285*r**9+18*r**11)/(3*(-1+y)**3)
            +(240*r**3+2280*r**5+2580*r**7+199*r**9-18*r**11)/(6*(-1+y)**2)-(2*(
            -96*r**3+141*r**5+630*r**7+35*r**9))/(3*(-1+y))+(2*r*(-40+220*r**2
            +1050*r**4+20*r**6+25*r**8-18*r**10+3*r**12)*y)/5+(r*(-16+64*r**2
            +126*r**4-20*r**6+r**8)*y**2)/2-(4*r*(12-24*r**2-6*r**4+r**8)*y**3)/9
            -(r*(8-8*r**2+3*r**4)*y**4)/2-(4*r*y**5)/5-(r*(3+2*r**2)*y**6)/90-(2*r*(24
            -120*r**2-1155*r**4-450*r**6+35*r**8)*math.log((1-y)/r**2))/3)
    if (dEl == 0 and dr == 1 and i==3 and j == 0):
        return     ((r*(813-1930*r**2-9285*r**4+3380*r**6-1150*r**8-318*r**10+25*r**12))/10
            +(24*r**11)/(-1+y)**7+(4*(20*r**9+31*r**11))/(-1+y)**6+(4*(120*r**7
            +515*r**9+308*r**11))/(5*(-1+y)**5)+(12*(20*r**5+205*r**7+335*r**9
            +91*r**11))/(5*(-1+y)**4)+(8*r**3+244*r**5+924*r**7+657*r**9+56*r**11)/(
            -1+y)**3+(80*r**3+848*r**5+1140*r**7+115*r**9-56*r**11)/(2*(-1+y)**2)
            -(6*(-14*r**3+3*r**5+100*r**7+25*r**9))/(-1+y)+(8*r*(-15-20*r**2
            +190*r**4-60*r**6-50*r**8+4*r**10)*y)/5+(3*r*(-8-12*r**2+6*r**4-44*r**6
            +3*r**8)*y**2)/2-8*r*(1+2*r**2+3*r**4)*y**3-(r*(12+22*r**2+r**4)*y**4)/2
            -(24*r*y**5)/5-(r*y**6)/10-6*r*(4+10*r**2-105*r**4-60*r**6+25*r**8)*math.log((1
            -y)/r**2))
    if (dEl == 0 and dr == 1 and i==3 and j == 1):
        return     ((-2*r*(-4437+14826*r**2+101052*r**4-17780*r**6-13650*r**8+2772*r**10
            -511*r**12+48*r**14))/105+(14*r**13)/(-1+y)**8+(2*(36*r**11+35*r**13))/(
            -1+y)**7+(4*(105*r**9+282*r**11+100*r**13))/(3*(-1+y)**6)+(2*(64*r**7
            +377*r**9+382*r**11+56*r**13))/(-1+y)**5+(540*r**5+7060*r**7+15835*r**9
            +7192*r**11+280*r**13)/(10*(-1+y)**4)+(240*r**3+9120*r**5+44780*r**7
            +45225*r**9+7656*r**11-280*r**13)/(30*(-1+y)**3)+(92*r**3+1258*r**5
            +2452*r**7+751*r**9-40*r**11)/(2*(-1+y)**2)+(236*r**3+338*r**5-1740*r**7
            -905*r**9+40*r**11)/(2*(-1+y))-(r*(120-10*r**2-4155*r**4-2060*r**6
            +425*r**8-84*r**10+10*r**12)*y)/5-(r*(60-20*r**2-890*r**4+60*r**6
            -5*r**8+4*r**10)*y**2)/5+(r*(-48-12*r**2+126*r**4-44*r**6+3*r**8)*y**3)/6
            -2*r*(3+2*r**2)*y**4-(r*(24-2*r**2+r**4)*y**5)/5-(13*r*y**6)/15-(r*y**7)/70
            +24*r*(-1-2*r**2+51*r**4+60*r**6)*math.log((1-y)/r**2))
    if (dEl == 0 and dr == 1 and i==4 and j == 0):
        return     ((2*r*(6644+6762*r**2-180810*r**4-81410*r**6+73920*r**8-11781*r**10
            -1582*r**12+90*r**14))/105+(28*r**13)/(-1+y)**8+(8*(15*r**11+22*r**13))/(
            -1+y)**7+(4*(150*r**9+567*r**11+340*r**13))/(3*(-1+y)**6)+(8*(60*r**7
            +474*r**9+729*r**11+224*r**13))/(3*(-1+y)**5)+(2*(150*r**5+2540*r**7
            +8080*r**9+6309*r**11+980*r**13))/(5*(-1+y)**4)+(8*(15*r**3+720*r**5
            +4810*r**7+7530*r**9+2889*r**11+140*r**13))/(15*(-1+y)**3)-(4*(-39*r**3
            -720*r**5-2114*r**7-1320*r**9-81*r**11+28*r**13))/(3*(-1+y)**2)-(8*(
            -23*r**3-96*r**5+126*r**7+230*r**9+27*r**11))/(-1+y)+(16*r*(-6-30*r**2
            +225*r**4+320*r**6-30*r**8-18*r**10+r**12)*y)/3+(8*r*(-10-35*r**2
            +195*r**4+80*r**6-65*r**8+3*r**10)*y**2)/5-(16*r*(2+6*r**2-9*r**4
            +8*r**6)*y**3)/3-(2*r*(12+33*r**2+18*r**4+2*r**6)*y**4)/3-(32*r*(1
            +2*r**2)*y**5)/5-(16*r*y**6)/3-(16*r*y**7)/105-8*r*(4+33*r**2-180*r**4
            -490*r**6-120*r**8+27*r**10)*math.log((1-y)/r**2))
    return 0. 

def moment_1_KIN_MS(elcut, par, hqe, wc, **kwargs):
    y=2*elcut/par.mbkin
    Elcuthat=elcut/par.mbkin
    r=par.mcMS/par.mbkin
    MBhat=par.MB/par.mbkin
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    logy =math.log((1-y)/r**2)

    lr =math.log(r)

    pi2=math.pi**2
    z3=1.202056903159594
    beta0=9
    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    flagNONBLM=kwargs.get('flag_NONBLM', 1)
    flagTEST=kwargs.get('flag_TEST', 1)
    FLAGcD=0
    FLAGcf=0.5*( 3.*( math.log(par.scale_muG/par.mbkin)))
    FLAGcs=0

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Mupipert1 = kin.MuPiPert(1,par)/par.mbkin**2
    Mupipert2 = kin.MuPiPert(2,par)/par.mbkin**2
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    deltambkin2BLM = kin.deltambkin(-2,par)
    deltamcMS2BLM = kin.deltamcMS(-2,par)
    Mupipert2BLM = kin.MuPiPert(-2,par)/par.mbkin**2
    Rhodpert2BLM = kin.RhoDPert(-2,par)/par.mbkin**3

    flagNLOMuPi  = 1
    flagNLOMuG   = 1
    flagNLORhoD  = 0  # set to 1 when grids for NLO rhoD are available 
    flagNLORhoLS = 0  # set to 1 when grids for NLO rhoLS are available 

    res = 0
    res +=(MBhat**2-(2*MBhat*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        +X(1,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)+(2*MBhat*rhoD*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(rhoD*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(2*MBhat*rhoD*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        )
    res += ((rhoD*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(2*MBhat*muG*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(muG*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(2*MBhat*muG*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))
    res += ((muG*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        +(2*MBhat*rhoLS*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(rhoLS*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(2*MBhat*rhoLS*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        )
    res += ((rhoLS*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(2*MBhat*mupi*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(mupi*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(2*MBhat*mupi*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        )
    res += ((mupi*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))

    if( flagDEBUG == 1):
        print("MXmoment n. 1 LO = ",res*par.mbkin**2)

    resNLO = 0
    resNLO +=((-2*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        -(2*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(2*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(2*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
        -(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
        +(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
        +(2*deltambkin1*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        +(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(deltambkin1*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
        +(deltamcMS1*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
        +(2*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(2*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlo.X1mix(1,0,'SM',Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
        -(2*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(2*MBhat*Rhodpert1*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        -(Rhodpert1*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(2*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(Mupipert1*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(2*MBhat*Mupipert1*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
        -(Mupipert1*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))

    if( flagDEBUG == 1):
        print("MXmoment n. 1 NLO = api*",resNLO*par.mbkin**2)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=((-2*beta0*deltambkin2BLM*MBhat*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*deltambkin2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltamcMS2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltambkin2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*deltamcMS2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(2*beta0*deltambkin2BLM*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltambkin2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltamcMS2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltambkin2BLM*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltamcMS2BLM*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(beta0*MBhat*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(2*X(0,0,Elcuthat,r,0,0)**2)
            -(beta0*math.log(mus**2)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(4*X(0,0,Elcuthat,r,0,0)**2)
            -(beta0*MBhat*math.log(mus**2)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(2*X(0,0,Elcuthat,r,0,0))
            +(beta0*math.log(mus**2)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(4*X(0,0,Elcuthat,r,0,0))
            +(2*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*MBhat*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(beta0*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(2*beta0*MBhat*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*MBhat*Rhodpert2BLM*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(beta0*Rhodpert2BLM*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*MBhat*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*MBhat*Mupipert2BLM*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(beta0*Mupipert2BLM*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))
        if( flagDEBUG == 1):
            print("MXmoment n. 1 NNLO BLM = api^2*beta0*",resNNLO/beta0*par.mbkin**2)
        res += api**2*resNNLO

        resNNLO = 0
        resNNLO +=((2*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin2*MBhat*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(2*beta0*deltambkin2BLM*MBhat*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltambkin2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*deltamcMS2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin2*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)**2*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,1)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,2,0)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin2*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*deltambkin2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS2*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*beta0*deltamcMS2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*deltamcMS1*MBhat*r**2*X(0,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            -(deltamcMS1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin2*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1**2*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1**2*Elcuthat**2*MBhat*X(0,1,Elcuthat,r,2,0))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*deltambkin2BLM*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1**2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltambkin2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*deltamcMS2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            +(deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            +(deltambkin1**2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,1,0)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,1,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,1,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0)*X(1,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            -(deltambkin1**2*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin2*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltambkin2BLM*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*deltamcMS1*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS2*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(beta0*deltamcMS2BLM*r*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*r**2*X(1,0,Elcuthat,r,0,2))/(2*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1*deltamcMS1*r**2*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1**2*r**2*X(1,0,Elcuthat,r,0,2))/(2*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1**2*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin2*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*Elcuthat*r*X(1,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*deltamcMS1*Elcuthat*r*X(1,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/(2*X(0,0,Elcuthat,r,0,0))
            +(2*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(MBhat*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(4*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(math.log(mus**2)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(6*X(0,0,Elcuthat,r,0,0)**2)
            -(2*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(MBhat*math.log(mus**2)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0))
            -(2*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(math.log(mus**2)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(6*X(0,0,Elcuthat,r,0,0))
            +(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*Rhodpert2*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*MBhat*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(MBhat*Rhodpert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(4*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Rhodpert2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(Rhodpert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0)**2)
            +(2*deltambkin1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Elcuthat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Rhodpert1*X(1,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*Rhodpert1*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(2*MBhat*Rhodpert2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*MBhat*Rhodpert2BLM*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(MBhat*Rhodpert1*math.log(mus**2/r**2)*XD(0,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0))
            +(2*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*r*Rhodpert1*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltamcMS1*MBhat*r*Rhodpert1*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*Elcuthat*MBhat*Rhodpert1*XD(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Rhodpert1*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(Rhodpert2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(beta0*Rhodpert2BLM*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(Rhodpert1*math.log(mus**2/r**2)*XD(1,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*Rhodpert1*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltamcMS1*r*Rhodpert1*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Elcuthat*Rhodpert1*XD(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*Mupipert2*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*MBhat*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(MBhat*Mupipert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(4*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(Mupipert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0)**2)
            +(2*deltambkin1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Elcuthat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1*Mupipert1*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*Mupipert1*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Mupipert1*X(1,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Mupipert1*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*Mupipert1*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(2*MBhat*Mupipert2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*beta0*MBhat*Mupipert2BLM*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(MBhat*Mupipert1*math.log(mus**2/r**2)*Xpi(0,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0))
            +(2*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*Mupipert1*r*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltamcMS1*MBhat*Mupipert1*r*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*Elcuthat*MBhat*Mupipert1*Xpi(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(Mupipert2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(beta0*Mupipert2BLM*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(Mupipert1*math.log(mus**2/r**2)*Xpi(1,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Mupipert1*X(0,0,Elcuthat,r,1,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Mupipert1*r*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltamcMS1*Mupipert1*r*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Elcuthat*Mupipert1*Xpi(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))

        resNNLO += nnlo.fitMXNNLOnonBLM(1, Elcuthat, r)*flagNONBLM*flagTEST
        if( flagDEBUG == 1):
            print("MXmoment n. 1 NNLO non-BLM = api^2*",resNNLO*par.mbkin**2)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=(flagNLORhoD*rhoD*((2*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixRhoD(1,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*FLAGcD*MBhat*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*r*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS1*MBhat*r*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*Elcuthat*MBhat*XD(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(FLAGcD*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*r*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1*r*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +flagNLOMuG*muG*((2*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixMuG(1,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcf*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*FLAGcf*MBhat*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*r*XG(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS1*MBhat*r*XG(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*Elcuthat*MBhat*XG(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(FLAGcf*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*r*XG(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1*r*XG(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +flagNLORhoLS*rhoLS*((2*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixRhoLS(1,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcs*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*FLAGcs*MBhat*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*r*XLS(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS1*MBhat*r*XLS(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*Elcuthat*MBhat*XLS(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(FLAGcs*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*r*XLS(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1*r*XLS(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +flagNLOMuPi*mupi*((2*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixMuPi(1,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*MBhat*r*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS1*MBhat*r*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*Elcuthat*MBhat*Xpi(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*r*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1*r*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
        if( flagDEBUG == 1):
            print("MXmoment n. 1 NLO pw = api*",resNLO*par.mbkin**2)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=(flagNLORhoD*((-2*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Rhodpert1*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(Rhodpert1*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(2*FLAGcD*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcD*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*MBhat*Rhodpert1*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Rhodpert1*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*FLAGcD*MBhat*Rhodpert1*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*MBhat*Rhodpert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*Rhodpert1*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(Rhodpert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2)
            +flagNLOMuPi*((-2*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert1*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Mupipert1*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(Mupipert1*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(4*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*MBhat*Mupipert1*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert1*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*Mupipert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2))
        if( flagDEBUG == 1):
            print("MXmoment n. 1 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**2)
        res += api**2*resNNLO

    return res*par.mbkin**2


def moment_2_KIN_MS(elcut, par, hqe, wc, **kwargs):
    y=2*elcut/par.mbkin
    Elcuthat=elcut/par.mbkin
    r=par.mcMS/par.mbkin
    MBhat=par.MB/par.mbkin
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    logy =math.log((1-y)/r**2)

    lr =math.log(r)

    pi2=math.pi**2
    z3=1.202056903159594
    beta0=9
    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    flagNONBLM=kwargs.get('flag_NONBLM', 1)
    flagTEST=kwargs.get('flag_TEST', 0)
    FLAGcD=0
    FLAGcf=0.5*( 3.*( math.log(par.scale_muG/par.mbkin)))
    FLAGcs=0

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Mupipert1 = kin.MuPiPert(1,par)/par.mbkin**2
    Mupipert2 = kin.MuPiPert(2,par)/par.mbkin**2
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    deltambkin2BLM = kin.deltambkin(-2,par)
    deltamcMS2BLM = kin.deltamcMS(-2,par)
    Mupipert2BLM = kin.MuPiPert(-2,par)/par.mbkin**2
    Rhodpert2BLM = kin.RhoDPert(-2,par)/par.mbkin**3

    flagNLOMuPi  = 1
    flagNLOMuG   = 1
    flagNLORhoD  = 0  # set to 1 when grids for NLO rhoD are available 
    flagNLORhoLS = 0  # set to 1 when grids for NLO rhoLS are available 

    res = 0
    res +=(MBhat**2*((-4*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))-X(1,0,Elcuthat,r,0,0)**2/X(0,0,Elcuthat,r,0,0)**2
        +MBhat*((4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+X(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)
        +rhoD*((2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))-(2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+XD(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +muG*((2*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))-(2*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+XG(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +rhoLS*((2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))-(2*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+XLS(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +mupi*((2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))-(2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+Xpi(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)))

    if( flagDEBUG == 1):
        print("MXmoment n. 2 LO = ",res*par.mbkin**4)

    resNLO = 0
    resNLO +=(deltamcMS1*((2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        -(2*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
        -(r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
        -4*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2-(2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        -(2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        +(2*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
        +(2*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(Elcuthat*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +(2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat**2*(deltamcMS1*((8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        -(8*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
        -(4*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
        -8*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2-(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        -(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        +(8*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
        +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(4*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(4*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +(8*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(4*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0))-(2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +MBhat*(deltamcMS1*((-8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
        +(4*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((12*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*Elcuthat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
        -(4*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
        -(12*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(4*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)+(4*Elcuthat*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        -(8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(4*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0))+nlo.X1mix(2,0,'SM',Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
        +MBhat**2*Rhodpert1*((-8*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+MBhat*Rhodpert1*((8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+Rhodpert1*((
        -2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -XD(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))+MBhat**2*Mupipert1*((
        -8*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(4*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+MBhat*Mupipert1*((8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(4*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+Mupipert1*((
        -2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -Xpi(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)))

    if( flagDEBUG == 1):
        print("MXmoment n. 2 NLO = api*",resNLO*par.mbkin**4)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=((-8*beta0*deltambkin2BLM*MBhat**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*deltambkin2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*deltamcMS2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*deltambkin2BLM*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*deltamcMS2BLM*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*deltambkin2BLM*MBhat**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltambkin2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltamcMS2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltamcMS2BLM*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(12*beta0*deltambkin2BLM*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*deltambkin2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*deltamcMS2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*beta0*deltambkin2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltamcMS2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*deltambkin2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*beta0*deltamcMS2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(4*beta0*deltambkin2BLM*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltamcMS2BLM*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltambkin2BLM*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*deltamcMS2BLM*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*deltambkin2BLM*MBhat*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltambkin2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltamcMS2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltamcMS2BLM*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltambkin2BLM*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltambkin2BLM*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltamcMS2BLM*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltambkin2BLM*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltamcMS2BLM*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(2*beta0*MBhat**2*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(beta0*MBhat**2*math.log(mus**2)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*MBhat*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(beta0*math.log(mus**2)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/(2*X(0,0,Elcuthat,r,0,0)**3)
            +(beta0*MBhat*math.log(mus**2)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*math.log(mus**2)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(4*X(0,0,Elcuthat,r,0,0)**2)
            -(2*beta0*MBhat**2*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*MBhat*math.log(mus**2)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*MBhat**2*math.log(mus**2)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(beta0*MBhat*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*math.log(mus**2)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(2*X(0,0,Elcuthat,r,0,0)**2)
            -(beta0*MBhat*math.log(mus**2)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(beta0*math.log(mus**2)*nlo.X1mix(2,0,'SM',Elcuthat,r))/(4*X(0,0,Elcuthat,r,0,0))
            +(8*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*beta0*MBhat**2*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(2*beta0*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*beta0*MBhat*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat**2*nnlo.X2mixBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat*nnlo.X2mixBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(beta0*nnlo.X2mixBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(8*beta0*MBhat**2*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*beta0*MBhat**2*Rhodpert2BLM*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*MBhat*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*beta0*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*beta0*MBhat*Rhodpert2BLM*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*Rhodpert2BLM*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*MBhat**2*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat**2*Rhodpert2BLM*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*MBhat*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Rhodpert2BLM*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(beta0*Rhodpert2BLM*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(8*beta0*MBhat**2*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*beta0*MBhat**2*Mupipert2BLM*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*MBhat*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*beta0*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*beta0*MBhat*Mupipert2BLM*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*Mupipert2BLM*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*MBhat**2*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat**2*Mupipert2BLM*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*MBhat*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Mupipert2BLM*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(beta0*Mupipert2BLM*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))
        if( flagDEBUG == 1):
            print("MXmoment n. 2 NNLO BLM = api^2*beta0*",resNNLO/beta0*par.mbkin**4)
        res += api**2*resNNLO

        resNNLO = 0
        resNNLO +=((-4*deltambkin1*Elcuthat*MBhat**2*nlo.X1mix_DerivativeEl(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*MBhat**2*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(4*deltamcMS1*MBhat**2*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(4*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*deltambkin1*Elcuthat*MBhat**2*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*MBhat**2*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltamcMS1*MBhat**2*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat**2*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*MBhat**2*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat**2*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1**2*MBhat**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin2*MBhat**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*deltambkin2BLM*MBhat**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin2*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*deltambkin2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS2*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*deltamcMS2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(12*deltambkin1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(12*deltamcMS1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(4*deltambkin1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin2*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(24*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(12*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,0,Elcuthat,r,1,0)**2*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(8*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,1)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,0,Elcuthat,r,2,0)*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1**2*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin2*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*deltambkin2BLM*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*deltamcMS1*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltamcMS2*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*deltamcMS2BLM*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(16*deltambkin1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(32*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltamcMS1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1**2*MBhat**2*r**2*X(0,1,Elcuthat,r,0,1)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,1,Elcuthat,r,0,1)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1**2*MBhat**2*r**2*X(0,1,Elcuthat,r,0,1)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*MBhat**2*r**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1**2*MBhat**2*r**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1**2*Elcuthat*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin2*Elcuthat*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,1,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,1,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,1,Elcuthat,r,1,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,2,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat**2*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat**2*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat**2*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*MBhat**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*deltambkin2*MBhat**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(8*beta0*deltambkin2BLM*MBhat**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1**2*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin2*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS2*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltamcMS2BLM*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,2)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,0,Elcuthat,r,0,2)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,2)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin2*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,0,Elcuthat,r,1,0)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,1)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,0,Elcuthat,r,2,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin2*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltambkin2BLM*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*deltamcMS1*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltamcMS2*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltamcMS2BLM*MBhat**2*r*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1**2*MBhat**2*r**2*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*MBhat**2*r**2*X(0,2,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*deltamcMS1*MBhat**2*r**2*X(0,2,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(2*deltamcMS1**2*MBhat**2*r**2*X(0,2,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1**2*Elcuthat*MBhat**2*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin2*Elcuthat*MBhat**2*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltambkin2BLM*Elcuthat*MBhat**2*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat*MBhat**2*r*X(0,2,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat**2*r*X(0,2,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1**2*Elcuthat**2*MBhat**2*X(0,2,Elcuthat,r,2,0))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*deltambkin1**2*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*deltambkin2*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*deltambkin2BLM*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*deltambkin1**2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*deltambkin2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltambkin1*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*deltamcMS2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(4*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1**2*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin2*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,2,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS2*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltamcMS2BLM*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*MBhat*r**2*X(0,1,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,1,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,1,1)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat**2*MBhat*X(0,1,Elcuthat,r,2,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(6*deltambkin1**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*deltambkin1**2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*beta0*deltambkin2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(6*deltambkin1*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS2*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*beta0*deltamcMS2BLM*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(3*deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(6*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(3*deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,2)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(6*deltambkin1**2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(6*deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(6*deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(3*deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,1,0)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,1,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,1,1)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS2*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltamcMS2BLM*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*deltamcMS1*MBhat*r**2*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1**2*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin2*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*deltambkin2BLM*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(6*deltambkin1*deltamcMS1*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS2*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*beta0*deltamcMS2BLM*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1**2*r**2*X(1,0,Elcuthat,r,0,1)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(1,0,Elcuthat,r,0,1)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1**2*r**2*X(1,0,Elcuthat,r,0,1)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*MBhat*r**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1**2*MBhat*r**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*r**2*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1**2*r**2*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat**2*MBhat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1**2*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin2*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1**2*Elcuthat*r*X(1,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Elcuthat*r*X(1,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,1,0)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1**2*Elcuthat*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Elcuthat*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat**2*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,2,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,2,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*deltambkin1**2*MBhat*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(12*deltambkin2*MBhat*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(12*beta0*deltambkin2BLM*MBhat*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(8*deltambkin1**2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS2*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*deltamcMS2BLM*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,2)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin2*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,1)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,2,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1**2*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin2*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltambkin2BLM*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(8*deltambkin1*deltamcMS1*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltamcMS2*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*deltamcMS2BLM*MBhat*r*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*deltamcMS1*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1**2*MBhat*r**2*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1**2*MBhat*r**2*X(1,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*deltamcMS1*MBhat*r**2*X(1,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            -(2*deltamcMS1**2*MBhat*r**2*X(1,1,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(8*deltambkin1**2*Elcuthat*MBhat*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin2*Elcuthat*MBhat*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltambkin2BLM*Elcuthat*MBhat*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1**2*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1**2*Elcuthat**2*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1**2*Elcuthat*MBhat*r*X(1,1,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*deltamcMS1*Elcuthat*MBhat*r*X(1,1,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1**2*Elcuthat**2*MBhat*X(1,1,Elcuthat,r,2,0))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*deltambkin2BLM*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(3*deltambkin1**2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltambkin2BLM*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*deltambkin1*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(beta0*deltamcMS2BLM*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,1)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,1)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,2)*X(2,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            +(deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,2)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,2)*X(2,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            +(3*deltambkin1**2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,1,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,1,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,1,1)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0)*X(2,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            -(3*deltambkin1**2*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin2*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltambkin2BLM*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(3*deltambkin1*deltamcMS1*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS2*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(beta0*deltamcMS2BLM*r*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1**2*r**2*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*r**2*X(2,0,Elcuthat,r,0,2))/(2*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1*deltamcMS1*r**2*X(2,0,Elcuthat,r,0,2))/X(0,0,Elcuthat,r,0,0)
            +(deltamcMS1**2*r**2*X(2,0,Elcuthat,r,0,2))/(2*X(0,0,Elcuthat,r,0,0))
            -(3*deltambkin1**2*Elcuthat*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin2*Elcuthat*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1**2*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*deltamcMS1*Elcuthat*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1**2*Elcuthat*r*X(2,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*deltamcMS1*Elcuthat*r*X(2,0,Elcuthat,r,1,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1**2*Elcuthat**2*X(2,0,Elcuthat,r,2,0))/(2*X(0,0,Elcuthat,r,0,0))
            +(16*deltambkin1*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat**2*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**3)
            +(24*deltambkin1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltamcMS1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltambkin1*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(16*deltambkin1*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltamcMS1*MBhat**2*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltambkin1*Elcuthat*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat**2*math.log(mus**2)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat**2*r*X(0,2,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat**2*r*X(0,2,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat**2*X(0,2,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**3)
            -(24*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(8*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(math.log(mus**2)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**3)
            +(6*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(6*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(8*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*deltambkin1*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*math.log(mus**2)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(8*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat*r*X(1,1,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*X(1,1,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat*X(1,1,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(math.log(mus**2)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/(6*X(0,0,Elcuthat,r,0,0)**2)
            -(2*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*r*X(2,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(2,0,Elcuthat,r,0,1)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(4*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(3*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(4*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltambkin1*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat**2*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(16*deltambkin1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltamcMS1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltambkin1*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*MBhat**2*r*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltamcMS1*MBhat**2*r*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*Elcuthat*MBhat**2*X(0,1,Elcuthat,r,1,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*deltambkin1*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*math.log(mus**2)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(8*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat*r*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat*X(1,0,Elcuthat,r,1,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(2*MBhat**2*math.log(mus**2)*nlo.X1mix(0,2,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0))
            +(4*deltambkin1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat**2*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat**2*X(0,0,Elcuthat,r,1,0)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*deltambkin1*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat*math.log(mus**2)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(8*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat*r*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*X(0,1,Elcuthat,r,0,1)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat*X(0,1,Elcuthat,r,1,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(math.log(mus**2)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(4*deltambkin1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*r*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltamcMS1*r*X(1,0,Elcuthat,r,0,1)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -nlo.X1mix(1,0,'SM',Elcuthat,r)**2/X(0,0,Elcuthat,r,0,0)**2-(12*deltambkin1*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(2*MBhat*math.log(mus**2)*nlo.X1mix(1,1,'SM',Elcuthat,r))/(3*X(0,0,Elcuthat,r,0,0))
            -(4*deltambkin1*MBhat*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat*X(0,0,Elcuthat,r,1,0)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(math.log(mus**2)*nlo.X1mix(2,0,'SM',Elcuthat,r))/(6*X(0,0,Elcuthat,r,0,0))
            +(deltambkin1*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(deltamcMS1*r*X(0,0,Elcuthat,r,0,1)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat**2*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**2*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat**2*flagTEST*nnlo.X2mixnonBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(4*MBhat*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*flagTEST*nnlo.X2mixnonBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+flagTEST*nnlo.X2mixnonBLM(2,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(8*deltambkin1*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat**2*Rhodpert2*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*MBhat**2*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Rhodpert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**3)
            -(24*deltambkin1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*deltambkin1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat**2*Rhodpert1*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat**2*Rhodpert2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat**2*Rhodpert2BLM*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat**2*Rhodpert1*math.log(mus**2/r**2)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(8*deltambkin1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat**2*r*Rhodpert1*X(0,2,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,2,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,2,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Rhodpert2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*MBhat*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Rhodpert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**3)
            +(24*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Rhodpert2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*beta0*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(Rhodpert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**3)
            -(6*deltambkin1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*deltamcMS1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(6*deltambkin1*Elcuthat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Rhodpert2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Rhodpert2BLM*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Rhodpert1*math.log(mus**2/r**2)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(1,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(1,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(1,1,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Rhodpert1*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Rhodpert2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*Rhodpert2BLM*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(Rhodpert1*math.log(mus**2/r**2)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0)**2)
            +(2*deltambkin1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Elcuthat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(2,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(2,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Rhodpert1*X(2,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat**2*r*Rhodpert1*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*r*Rhodpert1*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Elcuthat*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Rhodpert1*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**2*Rhodpert2*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*MBhat**2*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat**2*Rhodpert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(16*deltambkin1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,1,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Rhodpert2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Rhodpert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(1,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(1,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(1,0,Elcuthat,r,1,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat**2*Rhodpert1*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*MBhat**2*Rhodpert2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*MBhat**2*Rhodpert2BLM*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(2*MBhat**2*Rhodpert1*math.log(mus**2/r**2)*XD(0,2,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0))
            -(4*deltambkin1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat**2*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*X(0,0,Elcuthat,r,1,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat**2*r*Rhodpert1*XD(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltamcMS1*MBhat**2*r*Rhodpert1*XD(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*Elcuthat*MBhat**2*Rhodpert1*XD(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(4*MBhat*Rhodpert2*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Rhodpert2BLM*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Rhodpert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,1,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Rhodpert2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*Rhodpert2BLM*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(Rhodpert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(4*deltambkin1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*r*Rhodpert1*X(1,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*r*Rhodpert1*X(1,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*Rhodpert1*X(1,0,Elcuthat,r,1,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*r*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat*Rhodpert2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*MBhat*Rhodpert2BLM*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*MBhat*Rhodpert1*math.log(mus**2/r**2)*XD(1,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0))
            +(4*deltambkin1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*r*Rhodpert1*XD(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltamcMS1*MBhat*r*Rhodpert1*XD(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*Elcuthat*MBhat*Rhodpert1*XD(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*Rhodpert1*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(Rhodpert2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(beta0*Rhodpert2BLM*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(Rhodpert1*math.log(mus**2/r**2)*XD(2,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(0,0,Elcuthat,r,0,1)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Rhodpert1*X(0,0,Elcuthat,r,1,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*r*Rhodpert1*XD(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltamcMS1*r*Rhodpert1*XD(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Elcuthat*Rhodpert1*XD(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(8*MBhat**2*Mupipert2*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*MBhat**2*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Mupipert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**3)
            -(24*deltambkin1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltamcMS1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*deltambkin1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltamcMS1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Mupipert2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*MBhat**2*Mupipert2BLM*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*MBhat**2*Mupipert1*math.log(mus**2/r**2)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(8*deltambkin1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat**2*Mupipert1*r*X(0,2,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat**2*Mupipert1*r*X(0,2,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,2,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*Mupipert2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*beta0*MBhat*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Mupipert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**3)
            +(24*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Mupipert1*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Mupipert2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*beta0*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(Mupipert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**3)
            -(6*deltambkin1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*deltamcMS1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(6*deltambkin1*Elcuthat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*Mupipert1*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat*Mupipert1*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Mupipert2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Mupipert2BLM*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Mupipert1*math.log(mus**2/r**2)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*Mupipert1*r*X(1,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(1,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(1,1,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*Mupipert1*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*Mupipert2BLM*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(Mupipert1*math.log(mus**2/r**2)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0)**2)
            +(2*deltambkin1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Elcuthat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(deltambkin1*Mupipert1*r*X(2,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*Mupipert1*r*X(2,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Mupipert1*X(2,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltamcMS1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*MBhat**2*Mupipert1*r*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat**2*Mupipert1*r*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltamcMS1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*Mupipert1*r*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Mupipert1*r*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*Mupipert1*r*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*Elcuthat*Mupipert1*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Mupipert1*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**2*Mupipert2*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*MBhat**2*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat**2*Mupipert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(16*deltambkin1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*deltamcMS1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltamcMS1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,1,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*Mupipert1*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Mupipert2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Mupipert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*Mupipert1*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(1,0,Elcuthat,r,1,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltamcMS1*MBhat**2*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat**2*Mupipert2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*beta0*MBhat**2*Mupipert2BLM*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(2*MBhat**2*Mupipert1*math.log(mus**2/r**2)*Xpi(0,2,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0))
            -(4*deltambkin1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltamcMS1*MBhat**2*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*Elcuthat*MBhat**2*Mupipert1*X(0,0,Elcuthat,r,1,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat**2*Mupipert1*r*Xpi(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltamcMS1*MBhat**2*Mupipert1*r*Xpi(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltambkin1*Elcuthat*MBhat**2*Mupipert1*Xpi(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Mupipert2*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*beta0*MBhat*Mupipert2BLM*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*MBhat*Mupipert1*math.log(mus**2/r**2)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            -(8*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,1,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Mupipert1*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Mupipert2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*beta0*Mupipert2BLM*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(Mupipert1*math.log(mus**2/r**2)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0)**2)
            +(4*deltambkin1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*deltamcMS1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*deltambkin1*Elcuthat*Mupipert1*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*deltambkin1*Mupipert1*r*X(1,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*Mupipert1*r*X(1,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*Mupipert1*X(1,0,Elcuthat,r,1,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltamcMS1*Mupipert1*r*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*deltambkin1*Elcuthat*Mupipert1*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*MBhat*Mupipert1*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*MBhat*Mupipert2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*beta0*MBhat*Mupipert2BLM*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(2*MBhat*Mupipert1*math.log(mus**2/r**2)*Xpi(1,1,Elcuthat,r,0,0))/(3*X(0,0,Elcuthat,r,0,0))
            +(4*deltambkin1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltamcMS1*MBhat*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*Elcuthat*MBhat*Mupipert1*X(0,0,Elcuthat,r,1,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*deltambkin1*MBhat*Mupipert1*r*Xpi(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(4*deltamcMS1*MBhat*Mupipert1*r*Xpi(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*Elcuthat*MBhat*Mupipert1*Xpi(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*Mupipert1*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(Mupipert2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(beta0*Mupipert2BLM*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(Mupipert1*math.log(mus**2/r**2)*Xpi(2,0,Elcuthat,r,0,0))/(6*X(0,0,Elcuthat,r,0,0))
            -(deltambkin1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltamcMS1*Mupipert1*r*X(0,0,Elcuthat,r,0,1)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(deltambkin1*Elcuthat*Mupipert1*X(0,0,Elcuthat,r,1,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*Mupipert1*r*Xpi(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            -(deltamcMS1*Mupipert1*r*Xpi(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*Elcuthat*Mupipert1*Xpi(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))

        resNNLO += nnlo.fitMXNNLOnonBLM(2, Elcuthat, r)
        if( flagDEBUG == 1):
            print("MXmoment n. 2 NNLO non-BLM = api^2*",resNNLO*par.mbkin**4)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=(flagNLORhoD*rhoD*((2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +nlopw.X1mixRhoD(2,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)-(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcD*((8*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((
            -24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*XD(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
            -8*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,2,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*Elcuthat*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*Elcuthat*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(4*r*X(0,0,Elcuthat,r,0,1)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*XD(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(4*Elcuthat*XD(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcD*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,1,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*XD(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
            -24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,1,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,0,Elcuthat,r,0,1)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,1,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,1,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,1,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*XD(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)+(4*Elcuthat*XD(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcD*((2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +XD(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((
            -6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*r*X(1,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(r*X(0,0,Elcuthat,r,0,1)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(r*XD(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,1)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(2,0,Elcuthat,r,1,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*Elcuthat*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(Elcuthat*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(1,0,Elcuthat,r,0,1)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,1,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +XD(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)+(r*X(0,0,Elcuthat,r,0,1)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(0,0,Elcuthat,r,1,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(r*XD(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(Elcuthat*XD(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +flagNLOMuG*muG*((2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +nlopw.X1mixMuG(2,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)-(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlopw.X1mixMuG(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcf*((8*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((
            -24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,1)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*XG(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,2,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*Elcuthat*X(0,1,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*Elcuthat*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,1)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*XG(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(4*Elcuthat*XG(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlopw.X1mixMuG(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcf*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,0,Elcuthat,r,0,1)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,1,Elcuthat,r,0,1)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*XG(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
            -8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,1,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,0,Elcuthat,r,0,1)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,1,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,1,Elcuthat,r,0,1)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,1,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(4*r*X(0,0,Elcuthat,r,0,1)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*XG(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)+(4*Elcuthat*XG(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcf*((2*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +XG(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((
            -6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*r*X(1,0,Elcuthat,r,0,1)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(r*X(0,0,Elcuthat,r,0,1)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(r*XG(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((4*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,1)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(2,0,Elcuthat,r,1,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*Elcuthat*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(Elcuthat*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(1,0,Elcuthat,r,0,1)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,1,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(r*X(0,0,Elcuthat,r,0,1)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(0,0,Elcuthat,r,1,0)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(r*XG(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(Elcuthat*XG(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +flagNLORhoLS*rhoLS*((2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +nlopw.X1mixRhoLS(2,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)-(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlopw.X1mixRhoLS(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcs*((8*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((
            -24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,1)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*XLS(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
            -8*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,2,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*Elcuthat*X(0,1,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*Elcuthat*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,1)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(4*r*X(0,0,Elcuthat,r,0,1)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*XLS(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(4*Elcuthat*XLS(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlopw.X1mixRhoLS(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcs*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,0,Elcuthat,r,0,1)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,1,Elcuthat,r,0,1)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*XLS(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
            -24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,1,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,0,Elcuthat,r,0,1)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,1,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,1,Elcuthat,r,0,1)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,1,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*XLS(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)+(4*Elcuthat*XLS(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +FLAGcs*((2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +XLS(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))+deltamcMS1*((
            -6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*r*X(1,0,Elcuthat,r,0,1)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(r*X(0,0,Elcuthat,r,0,1)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(r*XLS(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,1)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(2,0,Elcuthat,r,1,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*Elcuthat*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(Elcuthat*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(1,0,Elcuthat,r,0,1)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,1,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +XLS(2,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)+(r*X(0,0,Elcuthat,r,0,1)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(0,0,Elcuthat,r,1,0)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(r*XLS(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(Elcuthat*XLS(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +flagNLOMuPi*mupi*((2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +nlopw.X1mixMuPi(2,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)-(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat**2*((8*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +deltamcMS1*((-24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*Xpi(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*r*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,2,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*Elcuthat*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*Elcuthat*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*Xpi(0,2,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(4*Elcuthat*Xpi(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            +(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +MBhat*((-8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +deltamcMS1*((24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*r*X(0,1,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*Xpi(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((
            -8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*r*X(0,1,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,1,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(8*Elcuthat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(1,0,Elcuthat,r,0,1)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,1,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*r*X(0,0,Elcuthat,r,0,1)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*r*X(0,1,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,1,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(4*r*X(0,0,Elcuthat,r,0,1)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*Xpi(1,1,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)+(4*Elcuthat*Xpi(1,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +deltamcMS1*((-6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            -(r*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*r*X(1,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(r*X(0,0,Elcuthat,r,0,1)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(r*Xpi(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))+deltambkin1*((4*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(4*r*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(1,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(0,0,Elcuthat,r,0,1)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,1)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(2,0,Elcuthat,r,1,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(2*r*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**3
            +(r*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            -(2*Elcuthat*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3
            +(Elcuthat*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*r*X(0,0,Elcuthat,r,0,1)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Elcuthat*X(0,0,Elcuthat,r,1,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*r*X(1,0,Elcuthat,r,0,1)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,1,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*r*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Elcuthat*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(r*X(0,0,Elcuthat,r,0,1)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Elcuthat*X(0,0,Elcuthat,r,1,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(r*Xpi(2,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(Elcuthat*Xpi(2,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))))
        if( flagDEBUG == 1):
            print("MXmoment n. 2 NLO pw = api*",resNLO*par.mbkin**4)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=(flagNLORhoD*((-8*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Rhodpert1*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Rhodpert1*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(Rhodpert1*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat**2*Rhodpert1*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(4*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat*Rhodpert1*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(Rhodpert1*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(8*FLAGcD*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*FLAGcD*MBhat**2*Rhodpert1*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcD*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*FLAGcD*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*FLAGcD*MBhat*Rhodpert1*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcD*Rhodpert1*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*MBhat**2*Rhodpert1*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*Rhodpert1*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*MBhat*Rhodpert1*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Rhodpert1*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Rhodpert1*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Rhodpert1*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Rhodpert1*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcD*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcD*MBhat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat**2*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**2*Rhodpert1*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Rhodpert1*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcD*MBhat**2*Rhodpert1*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(4*MBhat**2*Rhodpert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcD*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*FLAGcD*Rhodpert1*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Rhodpert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Rhodpert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Rhodpert1*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Rhodpert1*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*FLAGcD*MBhat*Rhodpert1*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(4*MBhat*Rhodpert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*Rhodpert1*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(Rhodpert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2)
            +flagNLOMuPi*((-8*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Mupipert1*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Mupipert1*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Mupipert1*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert1*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat**2*Mupipert1*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(4*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat*Mupipert1*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(Mupipert1*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(8*MBhat**2*Mupipert1*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*Mupipert1*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(8*MBhat*Mupipert1*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*Mupipert1*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(4*MBhat**2*Mupipert1*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Mupipert1*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert1*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat**2*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**2*Mupipert1*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Mupipert1*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*MBhat**2*Mupipert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Mupipert1*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*Mupipert1*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(4*MBhat*Mupipert1*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(2*Mupipert1*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*MBhat*Mupipert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(Mupipert1*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2))
        if( flagDEBUG == 1):
            print("MXmoment n. 2 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**4)
        res += api**2*resNNLO

    return res*par.mbkin**4


def moment_3_KIN_MS(elcut, par, hqe, wc, **kwargs):
    y=2*elcut/par.mbkin
    Elcuthat=elcut/par.mbkin
    r=par.mcMS/par.mbkin
    MBhat=par.MB/par.mbkin
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    logy =math.log((1-y)/r**2)

    lr =math.log(r)

    pi2=math.pi**2
    z3=1.202056903159594
    beta0=9
    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    flagNONBLM=kwargs.get('flag_NONBLM', 1)
    flagTEST=kwargs.get('flag_TEST', 0)
    FLAGcD=0
    FLAGcf=0.5*( 3.*( math.log(par.scale_muG/par.mbkin)))
    FLAGcs=0

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Mupipert1 = kin.MuPiPert(1,par)/par.mbkin**2
    Mupipert2 = kin.MuPiPert(2,par)/par.mbkin**2
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    deltambkin2BLM = kin.deltambkin(-2,par)
    deltamcMS2BLM = kin.deltamcMS(-2,par)
    Mupipert2BLM = kin.MuPiPert(-2,par)/par.mbkin**2
    Rhodpert2BLM = kin.RhoDPert(-2,par)/par.mbkin**3

    flagNLOMuPi  = 1
    flagNLOMuG   = 1
    flagNLORhoD  = 0  # set to 1 when grids for NLO rhoD are available 
    flagNLORhoLS = 0  # set to 1 when grids for NLO rhoLS are available 

    res = 0
    res +=((-16*MBhat**3*X(0,1,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat**3*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        +(2*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(12*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(12*MBhat**2*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(3*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(6*MBhat*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(3,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)
        )
    res += ((rhoD*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
        -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
        )
    res += ((muG*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0)
        -6*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XG(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XG(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XG(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XG(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
        )
    res += ((rhoLS*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0)
        -6*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XLS(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XLS(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XLS(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XLS(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
        )
    res += ((mupi*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
        -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)

    if( flagDEBUG == 1):
        print("MXmoment n. 3 LO = ",res*par.mbkin**6)

    resNLO = 0
    resNLO +=(-16*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**3
        +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +24*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,2,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -8*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        -12*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
        +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -12*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
        +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +2*((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
        +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        +12*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        +12*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +6*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -3*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -6*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
        +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +6*deltambkin1*((-16*MBhat**3*X(0,1,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat**3*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        +(2*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(12*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(12*MBhat**2*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(3*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(6*MBhat*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(3,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
        +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,3,'SM',Elcuthat,r)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,2,'SM',Elcuthat,r)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(2,1,'SM',Elcuthat,r)+X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(Rhodpert1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
        -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
        -(Mupipert1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
        -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
        -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
        +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
        -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
        -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
        -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
        +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
        +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
        -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
        -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)

    if( flagDEBUG == 1):
        print("MXmoment n. 3 NLO = api*",resNLO*par.mbkin**6)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=(-16*MBhat**3*(((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**3*((((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat**3*(((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)-((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,3,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +24*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -12*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -12*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3)-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +2*((-3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +12*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2)-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +12*MBhat**2*(((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)-((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +6*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2)-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -3*(((-2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -6*MBhat*((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0))-((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -(((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +6*beta0*deltambkin2BLM*((-16*MBhat**3*X(0,1,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(3,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(3,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            -(beta0*math.log(mus**2)*(-48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)-48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,3,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,2,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            +3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(2,1,'SM',Elcuthat,r)-X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(3,0,'SM',Elcuthat,r)))/(4*X(0,0,Elcuthat,r,0,0)**4)
            -(6*beta0*MBhat**6*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*MBhat**3*X(0,3,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*beta0*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*beta0*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(6*beta0*MBhat**8*nnlo.X2mixBLM(0,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))-(36*beta0*MBhat**7*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*beta0*MBhat**6*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(6*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(48*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*beta0*MBhat**2*X(1,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*beta0*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*beta0*MBhat*X(2,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*X(3,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*beta0*MBhat**5*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(48*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*beta0*MBhat**3*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*beta0*MBhat**3*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*MBhat**7*nnlo.X2mixBLM(0,1,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))+(72*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(144*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*beta0*MBhat**5*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(144*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(144*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*beta0*MBhat**3*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(12*beta0*MBhat*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(24*beta0*MBhat**2*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*beta0*MBhat*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*MBhat**3*nnlo.X2mixBLM(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(6*beta0*MBhat**4*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*MBhat**2*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*beta0*MBhat**6*nnlo.X2mixBLM(1,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))-(36*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(6*beta0*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(12*beta0*MBhat*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*beta0*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*beta0*MBhat**2*nnlo.X2mixBLM(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(6*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*beta0*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*beta0*MBhat*nnlo.X2mixBLM(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(beta0*nnlo.X2mixBLM(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(beta0*Rhodpert2BLM*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            -(beta0*Mupipert2BLM*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)
        if( flagDEBUG == 1):
            print("MXmoment n. 3 NNLO BLM = api^2*beta0*",resNNLO/beta0*par.mbkin**6)
        res += api**2*resNNLO

        resNNLO = 0
        resNNLO +=(-16*MBhat**3*(((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(9*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)**3
            +3*((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**3*((3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2)/X(0,1,Elcuthat,r,0,0)**2
            +(3*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,1,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**3*((((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(0,2,Elcuthat,r,0,0)
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,2,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,2,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,2,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,2,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat**3*(((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)
            -(3*deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(0,3,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,3,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,3,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,3,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,3,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,3,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +24*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)**2
            +2*((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2
            +2*X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**2*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            -12*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,2,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,2,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,2,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,2,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,2,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,2,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +2*((((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))*X(1,0,Elcuthat,r,0,0)**3)/(2*X(0,0,Elcuthat,r,0,0)**3)
            -(9*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +(X(1,0,Elcuthat,r,0,0)**3*((3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2)/X(1,0,Elcuthat,r,0,0)**2
            +(3*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(1,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -12*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3
            +(3*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
            +2*((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*(((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2+2*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1+deltamcMS2
            -beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2
            +(deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +12*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(1,0,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +12*MBhat**2*(((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(1,2,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,2,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,2,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,2,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,2,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,2,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +6*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(2,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -3*(((((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))*X(1,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            -(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
            +((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(2,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -6*MBhat*(((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(2,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(2,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(2,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(2,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(2,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +(((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(15*deltambkin1**2+6*(deltambkin2-beta0*deltambkin2BLM))*((-16*MBhat**3*X(0,1,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**3-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(3,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +6*deltambkin1*(-16*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3
            -(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -12*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -12*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +2*((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +12*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +12*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +6*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -3*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -6*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(3,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(3,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(3,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(3,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(3,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)
            -(6*MBhat**6*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**5*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat**4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            -(36*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(6*MBhat**10*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2-(36*MBhat**9*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**8*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(48*MBhat**7*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(18*MBhat**8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**7*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(18*MBhat**6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(36*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(6*MBhat**4*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(18*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(9*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**5*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**9*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2+(144*MBhat**8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**7*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(192*MBhat**6*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**7*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**6*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**5*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(144*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(24*MBhat**3*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**8*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2-(144*MBhat**7*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(192*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(12*MBhat**8*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2-(72*MBhat**7*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(144*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(96*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(36*MBhat**6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(144*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(144*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(36*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(12*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**7*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2+(144*MBhat**6*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**5*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**3*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat**6*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2-(36*MBhat**5*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(18*MBhat**4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(18*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(math.log(mus**2)*(-48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)-48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,3,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,2,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            +3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(2,1,'SM',Elcuthat,r)-X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(3,0,'SM',Elcuthat,r)))/(3*X(0,0,Elcuthat,r,0,0)**4)
            +(6*deltambkin1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,3,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,2,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(2,1,'SM',Elcuthat,r)+X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(3,0,'SM',Elcuthat,r)))/X(0,0,Elcuthat,r,0,0)**4
            +(math.log(mus**2)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,3,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,2,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(2,1,'SM',Elcuthat,r)+X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(3,0,'SM',Elcuthat,r)))/(6*X(0,0,Elcuthat,r,0,0)**4)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,3,'SM',Elcuthat,r)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,2,'SM',Elcuthat,r)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(2,1,'SM',Elcuthat,r)+X(0,0,Elcuthat,r,0,0)**3*nlo.X1mix(3,0,'SM',Elcuthat,r))
            +((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(3,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(3,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**3
            -X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            -X(3,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(0,0,'SM',Elcuthat,r))
            +48*MBhat**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**3
            +3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))
            -6*(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)**3*(-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)+3*deltambkin1*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            +8*MBhat**3*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,3,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            -12*MBhat**2*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            +6*MBhat*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(2,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            -72*MBhat**2*(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)*(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,1,Elcuthat,r,0,0)**2*(-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)+deltambkin1*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +36*MBhat*(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)**2*(((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,1,Elcuthat,r,0,0)*(-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r)+2*deltambkin1*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -48*MBhat**3*(X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,2,Elcuthat,r,0,0)*(X(0,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +48*MBhat**2*(X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,1,Elcuthat,r,0,0)*(X(0,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +24*MBhat**2*(X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)*(X(0,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +(deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -12*MBhat*(X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(2,0,Elcuthat,r,0,0)*(X(0,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -24*MBhat*(X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,1,Elcuthat,r,0,0)*(X(0,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +6*(X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(2,0,Elcuthat,r,0,0)*(X(0,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +(3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -48*MBhat**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2
            +(((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*nlo.X1mix(0,1,'SM',Elcuthat,r))
            +24*MBhat**3*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +X(0,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -12*MBhat*(2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -24*MBhat**2*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +X(1,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +6*MBhat*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +X(2,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +48*MBhat**2*(X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)
            +((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +24*MBhat**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,2,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)
            +(2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*nlo.X1mix(0,2,'SM',Elcuthat,r))
            -12*MBhat**2*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,2,'SM',Elcuthat,r)
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,2,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(0,2,'SM',Elcuthat,r)))
            -8*MBhat**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,3,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(0,3,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,3,'SM',Elcuthat,r))
            +24*MBhat**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2
            +((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*nlo.X1mix(1,0,'SM',Elcuthat,r))
            -12*MBhat**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)
            +((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*nlo.X1mix(1,0,'SM',Elcuthat,r))
            -24*MBhat*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)
            +(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*nlo.X1mix(1,0,'SM',Elcuthat,r))
            +6*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2
            +((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*nlo.X1mix(1,0,'SM',Elcuthat,r))
            +12*MBhat*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +X(1,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            -3*(X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r)
            +X(2,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2
            +(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            -24*MBhat**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)
            +((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*nlo.X1mix(1,1,'SM',Elcuthat,r))
            +12*MBhat*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)
            +((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*nlo.X1mix(1,1,'SM',Elcuthat,r))
            +12*MBhat**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,2,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,2,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**3
            +(deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(1,2,'SM',Elcuthat,r))
            +6*MBhat*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)
            +((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*nlo.X1mix(2,0,'SM',Elcuthat,r))
            -3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)
            +((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*nlo.X1mix(2,0,'SM',Elcuthat,r))
            -6*MBhat*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,1,Elcuthat,r))*X(0,0,Elcuthat,r,0,0)**3
            +(2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(2,1,'SM',Elcuthat,r))
            +(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*MBhat**6*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**5*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**4*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(6*MBhat**8*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))-(36*MBhat**7*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*MBhat**6*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(6*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*X(2,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(3,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**5*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(48*MBhat**4*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**7*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))+(72*MBhat**6*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(144*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*MBhat**5*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(144*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*MBhat**3*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(12*MBhat*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*flagTEST*nnlo.X2mixnonBLM(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(6*MBhat**4*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat**6*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))-(36*MBhat**5*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*MBhat**4*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(18*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(6*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(12*MBhat*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*flagTEST*nnlo.X2mixnonBLM(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(6*MBhat*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*flagTEST*nnlo.X2mixnonBLM(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+flagTEST*nnlo.X2mixnonBLM(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +((-Rhodpert2+beta0*Rhodpert2BLM+(Rhodpert1*math.log(mus**2/r**2))/6)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            +Rhodpert1*(-(((-6*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))
            -(-(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0))
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0))
            +48*MBhat**3*(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*XD(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))-48*MBhat**3*(((((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +8*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -72*MBhat**2*(((deltambkin1*X(0,1,Elcuthat,r,0,0)**2+2*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +24*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +36*MBhat*(((2*deltambkin1*X(0,1,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -6*((3*deltambkin1*X(1,0,Elcuthat,r,0,0)**3+3*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))+48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -12*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +6*((((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -48*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            -12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))
            -8*MBhat**3*(3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XD(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,3,Elcuthat,r,1,0)))+24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            +6*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))
            +12*MBhat**2*((deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XD(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,2,Elcuthat,r,1,0)))+6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))
            -6*MBhat*((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XD(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,1,Elcuthat,r,1,0)))+(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XD(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +((-Mupipert2+beta0*Mupipert2BLM+(Mupipert1*math.log(mus**2/r**2))/6)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            +6*deltambkin1*(-((Rhodpert1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -(Mupipert1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +Mupipert1*(-(((-5*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))
            -(-(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0))
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0))
            +48*MBhat**3*(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))-48*MBhat**3*(((((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +8*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -72*MBhat**2*(((deltambkin1*X(0,1,Elcuthat,r,0,0)**2+2*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +24*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +36*MBhat*(((2*deltambkin1*X(0,1,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -6*((3*deltambkin1*X(1,0,Elcuthat,r,0,0)**3+3*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))+48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -12*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +6*((((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -48*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            -12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))
            -8*MBhat**3*(3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*Xpi(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,3,Elcuthat,r,1,0)))+24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            +6*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))
            +12*MBhat**2*((deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*Xpi(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,2,Elcuthat,r,1,0)))+6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))
            -6*MBhat*((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*Xpi(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,1,Elcuthat,r,1,0)))+(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*Xpi(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4))

        resNNLO += nnlo.fitMXNNLOnonBLM(3, Elcuthat, r)
        if( flagDEBUG == 1):
            print("MXmoment n. 3 NNLO non-BLM = api^2*",resNNLO*par.mbkin**6)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=(flagNLORhoD*rhoD*((48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*nlopw.X1mixRhoD(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlopw.X1mixRhoD(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlopw.X1mixRhoD(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixRhoD(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(48*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcD*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(6*FLAGcD*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcD*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcD*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcD*MBhat*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(24*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat**3*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcD*MBhat*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*FLAGcD*MBhat**3*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcD*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcD*MBhat*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*FLAGcD*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcD*MBhat**2*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*FLAGcD*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*FLAGcD*MBhat*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcD*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((-6*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XD(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XD(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XD(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XD(3,0,Elcuthat,r,0,0))
            +(-(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0))
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0))
            +48*MBhat**3*(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*XD(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))-48*MBhat**3*(((((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +8*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -72*MBhat**2*(((deltambkin1*X(0,1,Elcuthat,r,0,0)**2+2*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +24*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +36*MBhat*(((2*deltambkin1*X(0,1,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -6*((3*deltambkin1*X(1,0,Elcuthat,r,0,0)**3+3*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))+48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -12*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +6*((((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))*XD(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))
            -48*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            -12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))
            -8*MBhat**3*(3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XD(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,3,Elcuthat,r,1,0)))+24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            +6*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XD(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))
            +12*MBhat**2*((deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XD(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,2,Elcuthat,r,1,0)))+6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XD(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XD(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))
            -6*MBhat*((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XD(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,1,Elcuthat,r,1,0)))+(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XD(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +flagNLOMuG*muG*((48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*nlopw.X1mixMuG(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlopw.X1mixMuG(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlopw.X1mixMuG(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixMuG(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(48*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcf*MBhat**3*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*FLAGcf*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(6*FLAGcf*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcf*MBhat**2*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcf*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcf*MBhat*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcf*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(24*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(3,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcf*MBhat**3*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcf*MBhat**2*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcf*MBhat*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcf*MBhat**2*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*FLAGcf*MBhat**3*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcf*MBhat**2*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcf*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcf*MBhat*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*FLAGcf*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcf*MBhat**2*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*FLAGcf*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*FLAGcf*MBhat*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcf*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XG(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XG(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XG(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XG(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((-5*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XG(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XG(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XG(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XG(3,0,Elcuthat,r,0,0))
            +(-(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0))
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0))
            +48*MBhat**3*(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*XG(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))-48*MBhat**3*(((((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            +8*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            -72*MBhat**2*(((deltambkin1*X(0,1,Elcuthat,r,0,0)**2+2*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            +24*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            +36*MBhat*(((2*deltambkin1*X(0,1,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            -6*((3*deltambkin1*X(1,0,Elcuthat,r,0,0)**3+3*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))+48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            -12*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            +6*((((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))*XG(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))
            -48*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XG(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XG(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))
            +48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))
            -12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XG(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XG(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XG(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,2,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,2,Elcuthat,r,1,0)))
            -8*MBhat**3*(3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XG(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,3,Elcuthat,r,1,0)))+24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XG(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XG(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))
            +6*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XG(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XG(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XG(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,1,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,1,Elcuthat,r,1,0)))
            +12*MBhat**2*((deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XG(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,2,Elcuthat,r,1,0)))+6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XG(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(2,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XG(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(2,0,Elcuthat,r,1,0)))
            -6*MBhat*((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XG(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(2,1,Elcuthat,r,1,0)))+(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XG(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +flagNLORhoLS*rhoLS*((48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*nlopw.X1mixRhoLS(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlopw.X1mixRhoLS(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(6*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlopw.X1mixRhoLS(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixRhoLS(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(48*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcs*MBhat**3*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*FLAGcs*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(6*FLAGcs*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcs*MBhat**2*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcs*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcs*MBhat*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcs*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(24*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(3,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcs*MBhat**3*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcs*MBhat**2*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcs*MBhat*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcs*MBhat**2*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*FLAGcs*MBhat**3*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcs*MBhat**2*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcs*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcs*MBhat*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*FLAGcs*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcs*MBhat**2*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*FLAGcs*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*FLAGcs*MBhat*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcs*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XLS(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XLS(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XLS(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XLS(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((-6*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*XLS(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*XLS(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*XLS(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*XLS(3,0,Elcuthat,r,0,0))
            +(-(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0))
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0))
            +48*MBhat**3*(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))-48*MBhat**3*(((((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            +8*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            -72*MBhat**2*(((deltambkin1*X(0,1,Elcuthat,r,0,0)**2+2*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            +24*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            +36*MBhat*(((2*deltambkin1*X(0,1,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            -6*((3*deltambkin1*X(1,0,Elcuthat,r,0,0)**3+3*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))+48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            -12*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            +6*((((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))*XLS(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))
            -48*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XLS(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XLS(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))
            +48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))
            -12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XLS(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XLS(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XLS(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,2,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,2,Elcuthat,r,1,0)))
            -8*MBhat**3*(3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XLS(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,3,Elcuthat,r,1,0)))+24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XLS(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*XLS(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))
            +6*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*XLS(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*XLS(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XLS(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,1,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,1,Elcuthat,r,1,0)))
            +12*MBhat**2*((deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XLS(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,2,Elcuthat,r,1,0)))+6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*XLS(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(2,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*XLS(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(2,0,Elcuthat,r,1,0)))
            -6*MBhat*((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XLS(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(2,1,Elcuthat,r,1,0)))+(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*XLS(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +flagNLOMuPi*mupi*((48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(6*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*nlopw.X1mixMuPi(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlopw.X1mixMuPi(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(6*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlopw.X1mixMuPi(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixMuPi(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(24*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((-5*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +8*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0)
            -6*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0)+48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0)
            -48*MBhat**3*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +48*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            -12*MBhat*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0)
            +24*MBhat**3*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0)
            -8*MBhat**3*X(0,0,Elcuthat,r,0,0)**3*Xpi(0,3,Elcuthat,r,0,0)+24*MBhat**2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            -12*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            +6*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0)
            -24*MBhat**2*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0)
            +12*MBhat**2*X(0,0,Elcuthat,r,0,0)**3*Xpi(1,2,Elcuthat,r,0,0)+6*MBhat*X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -3*X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0)
            -6*MBhat*X(0,0,Elcuthat,r,0,0)**3*Xpi(2,1,Elcuthat,r,0,0)+X(0,0,Elcuthat,r,0,0)**3*Xpi(3,0,Elcuthat,r,0,0))
            +(-(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0))
            -X(0,0,Elcuthat,r,0,0)**2*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0))
            +48*MBhat**3*(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))-48*MBhat**3*(((((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +8*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -72*MBhat**2*(((deltambkin1*X(0,1,Elcuthat,r,0,0)**2+2*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +24*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +36*MBhat*(((2*deltambkin1*X(0,1,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -6*((3*deltambkin1*X(1,0,Elcuthat,r,0,0)**3+3*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))+48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -12*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +6*((((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))*Xpi(0,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))
            -48*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +48*MBhat**2*((((deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            -12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))
            +24*MBhat**3*((2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))
            -8*MBhat**3*(3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*Xpi(0,3,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,3,Elcuthat,r,1,0)))+24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)
            +(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -12*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -24*MBhat*((((2*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            +6*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)+(-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))*Xpi(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))
            -24*MBhat**2*(((deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))
            +12*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(1,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))
            +12*MBhat**2*((deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*Xpi(1,2,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,2,Elcuthat,r,1,0)))+6*MBhat*(((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**2
            +2*X(0,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(0,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))*Xpi(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))
            -3*(((3*deltambkin1*X(0,0,Elcuthat,r,0,0)**2+2*X(0,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*X(1,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))*Xpi(2,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))
            -6*MBhat*((2*deltambkin1*X(0,0,Elcuthat,r,0,0)**3+3*X(0,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*Xpi(2,1,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,1,Elcuthat,r,1,0)))+(3*deltambkin1*X(0,0,Elcuthat,r,0,0)**3
            +3*X(0,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))*Xpi(3,0,Elcuthat,r,0,0)
            +X(0,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4))
        if( flagDEBUG == 1):
            print("MXmoment n. 3 NLO pw = api*",resNLO*par.mbkin**6)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=(flagNLORhoD*Rhodpert1*((-48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(6*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**3*nlopw.X1mixRhoD(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(3*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlopw.X1mixRhoD(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(6*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlopw.X1mixRhoD(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-nlopw.X1mixRhoD(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(48*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*FLAGcD*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(36*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(6*FLAGcD*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcD*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*FLAGcD*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*FLAGcD*MBhat*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcD*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(24*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat**3*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat**2*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*FLAGcD*MBhat*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**3*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcD*MBhat**3*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*FLAGcD*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcD*MBhat*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(3*FLAGcD*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(3*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*FLAGcD*MBhat**2*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(3*FLAGcD*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(6*FLAGcD*MBhat*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2)
            +flagNLOMuPi*Mupipert1*((-48*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(6*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat**3*nlopw.X1mixMuPi(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(3*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat**2*nlopw.X1mixMuPi(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(6*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(6*MBhat*nlopw.X1mixMuPi(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-nlopw.X1mixMuPi(3,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(24*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(36*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**3*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*MBhat**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(3*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(12*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(12*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(6*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2))
        if( flagDEBUG == 1):
            print("MXmoment n. 3 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**6)
        res += api**2*resNNLO

    return res*par.mbkin**6


def moment_4_KIN_MS(elcut, par, hqe, wc, **kwargs):
    y=2*elcut/par.mbkin
    Elcuthat=elcut/par.mbkin
    r=par.mcMS/par.mbkin
    MBhat=par.MB/par.mbkin
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    logy =math.log((1-y)/r**2)

    lr =math.log(r)

    pi2=math.pi**2
    z3=1.202056903159594
    beta0=9
    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    flagNONBLM=kwargs.get('flag_NONBLM', 1)
    flagTEST=kwargs.get('flag_TEST', 0)
    FLAGcD=0
    FLAGcf=0.5*( 3.*( math.log(par.scale_muG/par.mbkin)))
    FLAGcs=0

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Mupipert1 = kin.MuPiPert(1,par)/par.mbkin**2
    Mupipert2 = kin.MuPiPert(2,par)/par.mbkin**2
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    deltambkin2BLM = kin.deltambkin(-2,par)
    deltamcMS2BLM = kin.deltamcMS(-2,par)
    Mupipert2BLM = kin.MuPiPert(-2,par)/par.mbkin**2
    Rhodpert2BLM = kin.RhoDPert(-2,par)/par.mbkin**3

    flagNLOMuPi  = 1
    flagNLOMuG   = 1
    flagNLORhoD  = 0  # set to 1 when grids for NLO rhoD are available 
    flagNLORhoLS = 0  # set to 1 when grids for NLO rhoLS are available 

    res = 0
    res +=((-48*MBhat**4*X(0,1,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*X(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
        -(3*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*X(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*X(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*X(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)
        +rhoD*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +muG*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(12*X(1,0,Elcuthat,r,0,0)**4*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(X(4,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*XG(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(12*X(1,0,Elcuthat,r,0,0)**3*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(3,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*XG(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*XG(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*XG(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XG(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +rhoLS*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(12*X(1,0,Elcuthat,r,0,0)**4*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(X(4,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*XLS(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(12*X(1,0,Elcuthat,r,0,0)**3*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(3,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*XLS(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*XLS(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*XLS(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XLS(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +mupi*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)))

    if( flagDEBUG == 1):
        print("MXmoment n. 4 LO = ",res*par.mbkin**8)

    resNLO = 0
    resNLO +=(-48*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**4
        +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
        +96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        +16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,4,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
        +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
        -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
        +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
        +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -72*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
        +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
        +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
        +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
        +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**3
        +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
        +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
        -3*((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**5
        +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
        -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
        +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
        +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,3,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
        +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
        +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        +6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
        +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
        -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        +24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,2,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
        +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
        +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
        +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
        -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
        -8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
        -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,1,Elcuthat,r,0,0)
        +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
        +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
        +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +8*deltambkin1*((-48*MBhat**4*X(0,1,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*X(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
        -(3*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*X(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*X(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*X(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
        -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
        +(12*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        -(12*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
        +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        +(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
        +(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlo.X1mix(4,0,'SM',Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
        +Rhodpert1*((-192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
        +Mupipert1*((-192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
        -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        +(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
        -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        -(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
        +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        -(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
        +(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)))

    if( flagDEBUG == 1):
        print("MXmoment n. 4 NLO = api*",resNLO*par.mbkin**8)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=(-48*MBhat**4*(((-4*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**4*((((-4*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((-4*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,3,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +16*MBhat**4*(((-4*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)-((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,4,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,4,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +96*MBhat**3*((((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3
            -(3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,3,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -72*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**4)-(4*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -3*((-4*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**5
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*((((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3
            -(3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3)-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -32*MBhat**3*(((-3*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)-((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,3,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,3,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +24*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3)-(3*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((-3*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2)-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat**2*(((-2*beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)-((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,2,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(2,2,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +8*MBhat*(((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2)-(2*((
            -(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(3,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(3,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat*((-((beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0))-((-(beta0*deltambkin2BLM)
            +beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)-beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,1,Elcuthat,r,0,0)
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(3,1,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -(((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +8*beta0*deltambkin2BLM*((-48*MBhat**4*X(0,1,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            -(3*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((-(beta0*deltambkin2BLM)+beta0*deltamcMS2BLM)*r*X(4,0,Elcuthat,r,0,1)
            -beta0*deltambkin2BLM*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)
            +(beta0*math.log(mus**2)*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlo.X1mix(4,0,'SM',Elcuthat,r)/X(0,0,Elcuthat,r,0,0)))/4
            +(12*beta0*MBhat**8*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(72*beta0*MBhat**7*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(144*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*beta0*MBhat**4*X(0,4,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(36*beta0*MBhat**6*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(144*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(384*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*beta0*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(288*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*beta0*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(12*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*beta0*X(1,0,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(12*beta0*MBhat**10*nnlo.X2mixBLM(0,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))+(96*beta0*MBhat**9*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*beta0*MBhat**8*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*beta0*MBhat**7*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*beta0*MBhat**8*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*beta0*MBhat**7*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*beta0*MBhat**6*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(12*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*beta0*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*beta0*MBhat**3*X(1,3,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*beta0*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*beta0*MBhat**2*X(2,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*beta0*MBhat*X(3,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(beta0*X(4,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*beta0*MBhat**7*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(144*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(288*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(192*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*beta0*MBhat**4*X(0,3,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*beta0*MBhat**5*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*beta0*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*beta0*MBhat**3*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*beta0*MBhat**9*nnlo.X2mixBLM(0,1,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))-(192*beta0*MBhat**8*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(576*beta0*MBhat**7*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(768*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*beta0*MBhat**7*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(1152*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(768*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(144*beta0*MBhat**5*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(576*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*beta0*MBhat**3*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(24*beta0*MBhat*X(1,0,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*beta0*MBhat**3*X(1,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*beta0*MBhat**2*X(2,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*beta0*MBhat*X(3,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*beta0*MBhat**3*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*beta0*MBhat**4*nnlo.X2mixBLM(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(12*beta0*MBhat**6*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(72*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(144*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*beta0*MBhat**3*X(0,3,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(36*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*beta0*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*beta0*MBhat**8*nnlo.X2mixBLM(1,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))+(96*beta0*MBhat**7*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*beta0*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*beta0*MBhat**6*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*beta0*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*beta0*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*beta0*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(12*beta0*X(1,0,Elcuthat,r,0,0)**4*nnlo.X2mixBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*beta0*MBhat**2*X(1,2,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*beta0*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*beta0*MBhat*X(2,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*X(3,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*beta0*MBhat*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*beta0*MBhat**3*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*beta0*MBhat**2*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*beta0*MBhat**3*nnlo.X2mixBLM(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*beta0*X(1,0,Elcuthat,r,0,0)**2*nnlo.X2mixBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*beta0*MBhat**2*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*beta0*MBhat*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*beta0*MBhat**2*nnlo.X2mixBLM(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*beta0*MBhat*X(0,1,Elcuthat,r,0,0)*nnlo.X2mixBLM(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*beta0*X(1,0,Elcuthat,r,0,0)*nnlo.X2mixBLM(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*beta0*MBhat*nnlo.X2mixBLM(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(beta0*nnlo.X2mixBLM(4,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-beta0*Rhodpert2BLM*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            -beta0*Mupipert2BLM*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)))
        if( flagDEBUG == 1):
            print("MXmoment n. 4 NNLO BLM = api^2*beta0*",resNNLO/beta0*par.mbkin**8)
        res += api**2*resNNLO

        resNNLO = 0
        resNNLO +=((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(4,0,Elcuthat,r))+(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(4,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -48*MBhat**4*(((2*(5*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM)))/X(0,0,Elcuthat,r,0,0)**4
            +(16*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5
            +((20*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**4))*X(0,1,Elcuthat,r,0,0)**4
            +4*((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**4*((6*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2)/X(0,1,Elcuthat,r,0,0)**2
            +(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,1,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**4*((((2*(5*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM)))/X(0,0,Elcuthat,r,0,0)**3
            +(12*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)**2
            +2*((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2
            +2*X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**2*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,2,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,2,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,2,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,2,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((2*(5*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM)))/X(0,0,Elcuthat,r,0,0)**2
            +(8*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
            +(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,3,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,3,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,3,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,3,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,3,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +16*MBhat**4*(((2*(5*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM)))/X(0,0,Elcuthat,r,0,0)
            -(4*deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(0,4,Elcuthat,r,0,0)
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,4,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,4,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,4,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,4,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,4,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +96*MBhat**3*((((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**4
            +(12*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5
            +((20*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**4))*X(0,1,Elcuthat,r,0,0)**3
            +3*((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**3*((3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2)/X(0,1,Elcuthat,r,0,0)**2
            +(3*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,1,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**3*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(9*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,2,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,2,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,2,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,2,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1+deltamcMS2
            -beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2
            +(deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,3,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,3,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,3,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,3,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,3,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,3,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,3,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**4
            +(4*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5
            +((20*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**4))*X(0,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**3
            +3*((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((3*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2)/X(1,0,Elcuthat,r,0,0)**2
            +(3*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(1,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -3*((((20*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))*X(1,0,Elcuthat,r,0,0)**4)/(2*X(0,0,Elcuthat,r,0,0)**4)
            -(16*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5
            +(X(1,0,Elcuthat,r,0,0)**4*((6*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2)/X(1,0,Elcuthat,r,0,0)**2
            +(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(1,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -72*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**4
            +(8*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5
            +((20*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(8*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**4))*X(0,1,Elcuthat,r,0,0)**2
            +2*((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2
            +2*X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**2*(((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2+2*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1+deltamcMS2
            -beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2
            +(deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,2,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*((
            -deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,2,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,2,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,2,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,2,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,2,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
            +2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,2,Elcuthat,r,0,0)*(((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2+2*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1+deltamcMS2
            -beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2
            +(deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*((((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(9*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)**2
            +2*((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2
            +2*X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**2*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1+deltamcMS2
            -beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)+((-deltambkin1+deltamcMS1)**2*r**2*X(1,1,Elcuthat,r,0,2))/2
            +(deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3
            +(3*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(1,0,Elcuthat,r,0,0)**2
            +2*(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2
            +2*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)**2*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,2,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,2,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,2,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,2,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,2,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(1,0,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,2,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,2,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,2,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,2,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,2,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -32*MBhat**3*(((3*(2*deltambkin1**2-deltambkin2+beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)
            -(3*deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(1,3,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,3,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,3,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,3,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,3,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,3,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +24*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**3
            +(6*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)**2
            +2*((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +(((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))**2
            +2*X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)**2*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(2,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**3
            +(3*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**3))*X(0,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1+deltamcMS2
            -beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)+((-deltambkin1+deltamcMS1)**2*r**2*X(2,0,Elcuthat,r,0,2))/2
            +(deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((((12*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(6*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))*X(1,0,Elcuthat,r,0,0)**2)/(2*X(0,0,Elcuthat,r,0,0)**3)
            -(6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4
            +(((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))**2
            +2*X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)**2*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(2,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)**2
            +(4*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(2,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(1,0,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,1,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(2,1,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(2,1,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(2,1,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(2,1,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat**2*(((3*deltambkin1**2-2*(deltambkin2-beta0*deltambkin2BLM))/X(0,0,Elcuthat,r,0,0)
            -(2*deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(2,2,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(2,2,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(2,2,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(2,2,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(2,2,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(2,2,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +8*MBhat*((((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)**2
            +(2*deltambkin1*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2))*X(0,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*((
            -deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))
            +(X(0,1,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(3,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(3,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(3,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(3,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(3,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((((6*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))**2)/X(0,0,Elcuthat,r,0,0)**2
            -(4*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0))*X(1,0,Elcuthat,r,0,0))/(2*X(0,0,Elcuthat,r,0,0)**2)
            -(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(1,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(1,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(1,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(1,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(1,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
            +((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))
            +(X(1,0,Elcuthat,r,0,0)*((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM
            -deltambkin1*deltamcMS1+deltamcMS2-beta0*deltamcMS2BLM)*r*X(3,0,Elcuthat,r,0,1)
            +((-deltambkin1+deltamcMS1)**2*r**2*X(3,0,Elcuthat,r,0,2))/2+(deltambkin1**2
            -deltambkin2+beta0*deltambkin2BLM)*Elcuthat*X(3,0,Elcuthat,r,1,0)
            -deltambkin1*(-deltambkin1+deltamcMS1)*Elcuthat*r*X(3,0,Elcuthat,r,1,1)
            +(deltambkin1**2*Elcuthat**2*X(3,0,Elcuthat,r,2,0))/2))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat*(((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM)/X(0,0,Elcuthat,r,0,0)
            -(deltambkin1*(deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0))*X(3,1,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*((
            -deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(3,1,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(3,1,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(3,1,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(3,1,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(3,1,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))
            +(((((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*(
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0))
            +(deltamcMS1*r*X(0,0,Elcuthat,r,0,1))/X(0,0,Elcuthat,r,0,0)-(deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))/X(0,0,Elcuthat,r,0,0)
            -((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(0,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(0,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(0,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(0,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(0,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(28*deltambkin1**2+8*(deltambkin2-beta0*deltambkin2BLM))*((-48*MBhat**4*X(0,1,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            -(3*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**4-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+X(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +8*deltambkin1*(-48*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4
            -(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -72*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -3*((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**5
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1**2-deltambkin2+beta0*deltambkin2BLM-deltambkin1*deltamcMS1
            +deltamcMS2-beta0*deltamcMS2BLM)*r*X(4,0,Elcuthat,r,0,1)+((-deltambkin1
            +deltamcMS1)**2*r**2*X(4,0,Elcuthat,r,0,2))/2+(deltambkin1**2-deltambkin2
            +beta0*deltambkin2BLM)*Elcuthat*X(4,0,Elcuthat,r,1,0)-deltambkin1*(
            -deltambkin1+deltamcMS1)*Elcuthat*r*X(4,0,Elcuthat,r,1,1)+(deltambkin1**2*Elcuthat**2*X(4,0,Elcuthat,r,2,0))/2)/X(0,0,Elcuthat,r,0,0)
            -(((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(18*MBhat**8*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**7*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(480*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**6
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat**6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(960*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**6
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(96*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(18*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(720*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**6
            +(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(240*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**6
            -(30*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**6
            -(18*MBhat**12*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2+(144*MBhat**11*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(432*MBhat**10*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(576*MBhat**9*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**8*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**10*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(432*MBhat**9*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(864*MBhat**8*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(576*MBhat**7*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(108*MBhat**8*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(432*MBhat**7*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(432*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**6*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(144*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(18*MBhat**4*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(576*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(36*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**4
            -(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -X(4,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3)
            +12*((4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(1,0,Elcuthat,r,0,0)**4*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**6))
            +192*MBhat**4*((4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(0,1,Elcuthat,r,0,0)**4*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            -16*MBhat**4*((((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(0,4,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            +32*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(1,3,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            -24*MBhat**2*((((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(2,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            +8*MBhat*((((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(3,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,0,'SM',Elcuthat,r)))
            -18*((X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(2,0,Elcuthat,r,0,0)*((2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5)))
            +8*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(3,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4)))
            -384*MBhat**3*((X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(1,0,Elcuthat,r,0,0)*((3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(0,1,Elcuthat,r,0,0)**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +288*MBhat**2*((2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(1,0,Elcuthat,r,0,0)**2*((2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(0,1,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -96*MBhat*((3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(1,0,Elcuthat,r,0,0)**3*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -288*MBhat**4*((X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,2,Elcuthat,r,0,0)*((2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,1,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +288*MBhat**3*((X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,1,Elcuthat,r,0,0)*((2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,1,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -72*MBhat**2*((X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(2,0,Elcuthat,r,0,0)*((2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,1,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -72*MBhat**2*((2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)**2*((((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +72*MBhat*((X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,1,Elcuthat,r,0,0)*((2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +128*MBhat**4*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,3,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -192*MBhat**3*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,2,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -64*MBhat**3*((X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,3,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +96*MBhat**2*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(2,1,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +96*MBhat**2*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,2,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -16*MBhat*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(3,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            -48*MBhat*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(2,1,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,0,'SM',Elcuthat,r))))
            +288*MBhat**3*((X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,2,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r)))))
            -288*MBhat**2*((X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,1,Elcuthat,r,0,0)*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r)))))
            +72*MBhat*((X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(2,0,Elcuthat,r,0,0)*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(0,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,0,'SM',Elcuthat,r)))))
            -(72*MBhat**7*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**6*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**5*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**3*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**11*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2-(576*MBhat**10*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1728*MBhat**9*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(2304*MBhat**8*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1152*MBhat**7*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**9*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**8*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(3456*MBhat**7*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(2304*MBhat**6*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(432*MBhat**7*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**6*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1728*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**5*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat**3*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat**6*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**5*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**10*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2+(576*MBhat**9*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**8*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(2304*MBhat**7*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1152*MBhat**6*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1728*MBhat**7*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(3456*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(2304*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(432*MBhat**6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1728*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**4*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(96*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -192*MBhat**4*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            +(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,1,'SM',Elcuthat,r))
            +24*MBhat*((3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -64*MBhat**4*((((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(0,3,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +96*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(1,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -48*MBhat**2*((((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(2,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +8*MBhat*((((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(3,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -144*MBhat**2*((2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +192*MBhat**4*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,2,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -192*MBhat**3*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,1,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +48*MBhat**2*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(2,0,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            +288*MBhat**3*((X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +X(1,0,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,1,'SM',Elcuthat,r)))
            -96*MBhat**3*((X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(0,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,1,'SM',Elcuthat,r))))
            +96*MBhat**2*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,1,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,1,'SM',Elcuthat,r))))
            -24*MBhat*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(2,0,Elcuthat,r,0,0)*((((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,1,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,1,'SM',Elcuthat,r))))
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +96*MBhat**4*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,2,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,2,'SM',Elcuthat,r))
            +24*MBhat**2*((2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,2,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(0,2,'SM',Elcuthat,r)))
            -96*MBhat**3*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,0,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,2,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,2,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,2,'SM',Elcuthat,r)))
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -64*MBhat**4*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,3,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,3,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(0,3,'SM',Elcuthat,r))
            +32*MBhat**3*((((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(1,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,3,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(0,3,'SM',Elcuthat,r)))
            -(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(0,4,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +16*MBhat**4*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(0,4,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(0,4,'SM',Elcuthat,r))
            +(36*MBhat**6*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**5*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat**4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(36*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(36*MBhat**10*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2+(288*MBhat**9*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(864*MBhat**8*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1152*MBhat**7*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(576*MBhat**6*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(144*MBhat**8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(864*MBhat**7*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1152*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(216*MBhat**6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(864*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(864*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(144*MBhat**4*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(36*MBhat**2*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**5*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat**9*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2-(576*MBhat**8*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1728*MBhat**7*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(2304*MBhat**6*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1152*MBhat**5*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**7*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**6*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(3456*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(2304*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(432*MBhat**5*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(1728*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(1728*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(288*MBhat**3*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(72*MBhat*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(18*MBhat**4*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            -(18*MBhat**8*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2+(144*MBhat**7*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(432*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(576*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**6*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(432*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(864*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(576*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(108*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(432*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(432*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)+(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(18*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))**2)-(24*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)**2)/X(0,0,Elcuthat,r,0,0)**3
            +96*MBhat**3*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            -96*MBhat**3*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            +32*MBhat**3*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            -144*MBhat**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            +48*MBhat**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            +72*MBhat*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            -12*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**4
            +((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*nlo.X1mix(1,0,'SM',Elcuthat,r))
            -4*((((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(3,0,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3))
            -48*MBhat**2*((((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(1,2,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            +24*MBhat*((((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +X(2,1,Elcuthat,r,0,0)*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            +96*MBhat**2*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,1,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            -24*MBhat*((X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(2,0,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            +12*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(2,0,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            -48*MBhat*((X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0))*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +X(1,1,Elcuthat,r,0,0)*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,0,Elcuthat,r))
            +(-deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,0,Elcuthat,r))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,0,'SM',Elcuthat,r)))
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -96*MBhat**3*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,1,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,1,'SM',Elcuthat,r))
            +96*MBhat**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,1,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,1,'SM',Elcuthat,r))
            -24*MBhat*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,1,Elcuthat,r))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(1,1,'SM',Elcuthat,r))
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,2,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,2,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(1,2,'SM',Elcuthat,r))
            -48*MBhat**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,2,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(1,2,Elcuthat,r))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(1,2,'SM',Elcuthat,r))
            +(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(1,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -32*MBhat**3*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(1,3,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(1,3,'SM',Elcuthat,r))
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +24*MBhat**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,0,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(2,0,'SM',Elcuthat,r))
            -24*MBhat*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(2,0,'SM',Elcuthat,r))
            +6*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,0,Elcuthat,r))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**3
            +((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*nlo.X1mix(2,0,'SM',Elcuthat,r))
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -48*MBhat**2*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,1,Elcuthat,r))+(
            -deltambkin1+deltamcMS1)*r*nlo.X1mix_Derivativer(2,1,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(2,1,'SM',Elcuthat,r))
            +24*MBhat*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,1,Elcuthat,r))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(2,1,'SM',Elcuthat,r))
            -(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(2,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +24*MBhat**2*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(2,2,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(2,2,'SM',Elcuthat,r))
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +8*MBhat*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(3,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(3,0,Elcuthat,r))*X(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(3,0,'SM',Elcuthat,r))
            -4*(((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(3,0,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(3,0,Elcuthat,r))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(3,0,'SM',Elcuthat,r))
            +(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(3,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -8*MBhat*((-(deltambkin1*Elcuthat*nlo.X1mix_DerivativeEl(3,1,Elcuthat,r))+(-deltambkin1
            +deltamcMS1)*r*nlo.X1mix_Derivativer(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*nlo.X1mix(3,1,'SM',Elcuthat,r))
            +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*nlo.X1mix(4,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*nlo.X1mix(4,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +8*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlo.X1mix(4,0,'SM',Elcuthat,r)/X(0,0,Elcuthat,r,0,0))
            -(math.log(mus**2)*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlo.X1mix(4,0,'SM',Elcuthat,r)/X(0,0,Elcuthat,r,0,0)))/6
            +(12*MBhat**8*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(72*MBhat**7*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(36*MBhat**6*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(12*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(12*MBhat**10*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))+(96*MBhat**9*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*MBhat**8*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*MBhat**7*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*MBhat**6*X(0,1,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*MBhat**8*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*MBhat**7*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*MBhat**6*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*MBhat**4*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(12*MBhat**2*X(1,0,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**7*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(144*MBhat**6*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**5*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**3*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**9*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))-(192*MBhat**8*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(576*MBhat**7*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(768*MBhat**6*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*MBhat**5*X(0,1,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*MBhat**7*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*MBhat**6*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(1152*MBhat**5*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(144*MBhat**5*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(576*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*MBhat**3*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(24*MBhat*X(1,0,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*flagTEST*nnlo.X2mixnonBLM(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(12*MBhat**6*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            -(72*MBhat**5*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(36*MBhat**4*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(36*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*MBhat**8*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0))+(96*MBhat**7*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*MBhat**6*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*MBhat**5*X(0,1,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*MBhat**6*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*MBhat**5*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(72*MBhat**4*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**2*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(48*MBhat**2*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**3*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))-(12*X(1,0,Elcuthat,r,0,0)**4*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/(X(0,0,Elcuthat,r,0,0)**4*(MBhat**2*X(0,0,Elcuthat,r,0,0)
            -2*MBhat*X(0,1,Elcuthat,r,0,0)+X(1,0,Elcuthat,r,0,0)))+(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*flagTEST*nnlo.X2mixnonBLM(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*flagTEST*nnlo.X2mixnonBLM(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*flagTEST*nnlo.X2mixnonBLM(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*X(0,1,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*flagTEST*nnlo.X2mixnonBLM(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*flagTEST*nnlo.X2mixnonBLM(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+flagTEST*nnlo.X2mixnonBLM(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(-Rhodpert2+beta0*Rhodpert2BLM+(Rhodpert1*math.log(mus**2/r**2))/6)*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +Rhodpert1*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(4,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            -192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -128*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +16*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,4,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +384*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**3*((((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +64*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -288*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -12*(((-5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**6
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +288*MBhat**2*((((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -72*MBhat*(((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +18*((((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(2,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +16*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -8*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(3,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,3,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +12*(((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -12*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XD(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +3*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            -((-deltambkin1+deltamcMS1)*r*XD(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(-Mupipert2+beta0*Mupipert2BLM+(Mupipert1*math.log(mus**2/r**2))/6)*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +8*deltambkin1*(Rhodpert1*((-192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +Mupipert1*((-192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)-Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0)))
            +Mupipert1*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(4,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            -192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -128*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +16*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,4,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +384*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**3*((((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +64*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -288*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -12*(((-5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**6
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +288*MBhat**2*((((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -72*MBhat*(((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +18*((((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(2,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +16*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -8*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(3,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,3,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +12*(((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -12*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*Xpi(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +2*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            -((-deltambkin1+deltamcMS1)*r*Xpi(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))

        resNNLO += nnlo.fitMXNNLOnonBLM(4, Elcuthat, r)
        if( flagDEBUG == 1):
            print("MXmoment n. 4 NNLO non-BLM = api^2*",resNNLO*par.mbkin**8)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=(flagNLORhoD*rhoD*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlopw.X1mixRhoD(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlopw.X1mixRhoD(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlopw.X1mixRhoD(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlopw.X1mixRhoD(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixRhoD(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(192*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*FLAGcD*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*FLAGcD*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*FLAGcD*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*FLAGcD*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*FLAGcD*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcD*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcD*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            -(960*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(384*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(32*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1920*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(192*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(1440*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(288*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(480*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(60*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(1152*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(4,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +128*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -16*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,4,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -384*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*((((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -64*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +288*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +12*(((-5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**6
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -288*MBhat**2*((((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*(((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -18*((((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(2,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -16*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,0,Elcuthat,r,0,0)
            +(X(3,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(192*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*FLAGcD*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcD*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcD*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcD*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcD*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,1,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(96*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(0,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(64*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*FLAGcD*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,3,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(16*FLAGcD*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*FLAGcD*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*FLAGcD*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcD*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcD*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcD*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -12*(((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +12*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,0,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(32*FLAGcD*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(24*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcD*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XD(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XD(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(24*FLAGcD*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(8*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcD*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XD(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XD(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(8*FLAGcD*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XD(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XD(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(FLAGcD*XD(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+((deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XD(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +5*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XD(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((-deltambkin1+deltamcMS1)*r*XD(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XD(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +flagNLOMuG*muG*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlopw.X1mixMuG(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlopw.X1mixMuG(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuG(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlopw.X1mixMuG(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuG(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuG(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlopw.X1mixMuG(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixMuG(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(192*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*FLAGcf*MBhat**4*X(0,4,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*FLAGcf*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*FLAGcf*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*FLAGcf*X(1,0,Elcuthat,r,0,0)**4*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcf*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*FLAGcf*MBhat**3*X(1,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*FLAGcf*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcf*MBhat**2*X(2,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcf*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcf*MBhat*X(3,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcf*X(4,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,0,Elcuthat,r,0,0)
            -(960*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(384*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(32*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1920*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(192*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(1440*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(288*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(480*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(60*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(1152*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(4,0,'SM',Elcuthat,r)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +128*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -16*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,4,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -384*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*((((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -64*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +288*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +12*(((-5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**6
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -288*MBhat**2*((((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*(((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -18*((((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,0,Elcuthat,r,0,0)
            +(X(2,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -16*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,0,Elcuthat,r,0,0)
            +(X(3,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(192*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*FLAGcf*MBhat**4*X(0,3,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcf*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)**3*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcf*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcf*MBhat**3*X(1,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcf*MBhat**2*X(2,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcf*MBhat*X(3,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,3,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(1,2,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(2,1,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*nlo.X1mix(3,0,'SM',Elcuthat,r)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,1,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,1,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,1,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(96*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcf*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XG(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(0,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XG(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(64*FLAGcf*MBhat**4*X(0,1,Elcuthat,r,0,0)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*FLAGcf*MBhat**3*X(1,0,Elcuthat,r,0,0)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,3,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(16*FLAGcf*MBhat**4*XG(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XG(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(96*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*FLAGcf*MBhat**3*X(0,3,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcf*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*FLAGcf*X(1,0,Elcuthat,r,0,0)**3*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcf*MBhat**2*X(1,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcf*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcf*MBhat*X(2,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcf*X(3,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(3,0,'SM',Elcuthat,r)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -12*(((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XG(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,0,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +12*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,0,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,0,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(96*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)**2*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XG(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XG(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +(96*FLAGcf*MBhat**3*X(0,1,Elcuthat,r,0,0)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcf*MBhat**2*X(1,0,Elcuthat,r,0,0)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(32*FLAGcf*MBhat**3*XG(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XG(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(24*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcf*X(1,0,Elcuthat,r,0,0)**2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XG(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XG(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XG(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XG(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(48*FLAGcf*MBhat**2*X(0,1,Elcuthat,r,0,0)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcf*MBhat*X(1,0,Elcuthat,r,0,0)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(24*FLAGcf*MBhat**2*XG(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XG(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(8*FLAGcf*MBhat*X(0,1,Elcuthat,r,0,0)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcf*X(1,0,Elcuthat,r,0,0)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XG(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XG(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(8*FLAGcf*MBhat*XG(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XG(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XG(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(FLAGcf*XG(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+((deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XG(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XG(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +6*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*XG(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XG(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XG(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XG(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*XG(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*XG(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XG(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XG(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*XG(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*XG(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XG(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*XG(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XG(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*XG(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XG(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((-deltambkin1+deltamcMS1)*r*XG(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XG(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +flagNLORhoLS*rhoLS*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlopw.X1mixRhoLS(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlopw.X1mixRhoLS(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoLS(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlopw.X1mixRhoLS(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoLS(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlopw.X1mixRhoLS(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixRhoLS(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(192*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*FLAGcs*MBhat**4*X(0,4,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*FLAGcs*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*FLAGcs*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*FLAGcs*X(1,0,Elcuthat,r,0,0)**4*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcs*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*FLAGcs*MBhat**3*X(1,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*FLAGcs*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcs*MBhat**2*X(2,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcs*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*FLAGcs*MBhat*X(3,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcs*X(4,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,0,Elcuthat,r,0,0)
            -(960*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(384*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(32*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1920*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(192*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(1440*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(288*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(480*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(60*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(1152*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(4,0,'SM',Elcuthat,r)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +128*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -16*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,4,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -384*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*((((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -64*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +288*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +12*(((-5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**6
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -288*MBhat**2*((((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*(((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -18*((((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,0,Elcuthat,r,0,0)
            +(X(2,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -16*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,0,Elcuthat,r,0,0)
            +(X(3,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(192*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*FLAGcs*MBhat**4*X(0,3,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcs*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)**3*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcs*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcs*MBhat**3*X(1,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcs*MBhat**2*X(2,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcs*MBhat*X(3,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,3,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(1,2,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(2,1,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*nlo.X1mix(3,0,'SM',Elcuthat,r)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,1,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,1,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,1,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(96*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcs*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XLS(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(0,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XLS(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(64*FLAGcs*MBhat**4*X(0,1,Elcuthat,r,0,0)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*FLAGcs*MBhat**3*X(1,0,Elcuthat,r,0,0)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,3,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(16*FLAGcs*MBhat**4*XLS(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XLS(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(96*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*FLAGcs*MBhat**3*X(0,3,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*FLAGcs*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*FLAGcs*X(1,0,Elcuthat,r,0,0)**3*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*FLAGcs*MBhat**2*X(1,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*FLAGcs*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcs*MBhat*X(2,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcs*X(3,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(3,0,'SM',Elcuthat,r)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -12*(((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*XLS(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,0,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +12*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,0,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,0,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(96*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)**2*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XLS(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XLS(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +(96*FLAGcs*MBhat**3*X(0,1,Elcuthat,r,0,0)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcs*MBhat**2*X(1,0,Elcuthat,r,0,0)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(32*FLAGcs*MBhat**3*XLS(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XLS(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(24*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*FLAGcs*X(1,0,Elcuthat,r,0,0)**2*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XLS(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*XLS(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*XLS(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*XLS(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(48*FLAGcs*MBhat**2*X(0,1,Elcuthat,r,0,0)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcs*MBhat*X(1,0,Elcuthat,r,0,0)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(24*FLAGcs*MBhat**2*XLS(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XLS(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(8*FLAGcs*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*FLAGcs*X(1,0,Elcuthat,r,0,0)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*XLS(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*XLS(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(8*FLAGcs*MBhat*XLS(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*XLS(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*XLS(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +(FLAGcs*XLS(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+((deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*XLS(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*XLS(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +5*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*XLS(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*XLS(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XLS(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*XLS(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*XLS(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*XLS(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*XLS(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*XLS(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*XLS(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*XLS(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*XLS(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*XLS(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*XLS(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*XLS(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+XLS(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((-deltambkin1+deltamcMS1)*r*XLS(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*XLS(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +flagNLOMuPi*mupi*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlopw.X1mixMuPi(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlopw.X1mixMuPi(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlopw.X1mixMuPi(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlopw.X1mixMuPi(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)+nlopw.X1mixMuPi(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            -(960*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(384*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(32*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1920*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(192*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(1440*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(288*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(480*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(60*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(1152*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(16*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(2*X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(4,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2
            +192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**4
            +(4*X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -288*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +128*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -16*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,4,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -384*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*((((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -64*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +288*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**5-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**5)-(5*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**6)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +12*(((-5*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**4)/X(0,0,Elcuthat,r,0,0)**6
            +(4*X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**4*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -288*MBhat**2*((((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((
            -deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -72*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +72*MBhat*(((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((
            -deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -18*((((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**5
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(2,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -16*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,0,Elcuthat,r,0,0)
            +(X(3,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -192*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +192*MBhat**4*(((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +288*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(1,0,Elcuthat,r,0,0)**3
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -192*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(3,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,1,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +96*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(0,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -64*MBhat**4*((((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,3,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,3,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(0,3,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(0,3,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +16*MBhat**4*(((-4*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(0,4,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(0,4,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(0,4,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(36*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**3
            +(3*X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -96*MBhat**3*(((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(0,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +32*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,3,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -144*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**4-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +48*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +72*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**4)-(4*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**5)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            -12*(((-4*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**3)/X(0,0,Elcuthat,r,0,0)**5
            +(3*X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**3*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,2,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +12*((((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(2,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(2,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(2,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(2,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(3,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(3,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,0,Elcuthat,r,0,0)
            +(X(3,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +96*MBhat**2*(((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(1,0,Elcuthat,r,0,0)**2
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(1,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(1,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +96*MBhat**3*((((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,2,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,2,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(1,2,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(1,2,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -32*MBhat**3*(((-3*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(1,3,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(1,3,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(1,3,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +24*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**3-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)**2
            +(2*X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            -24*MBhat*((((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**3)-(3*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**4)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(2,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*((-deltambkin1
            +deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +6*(((-3*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0)**2)/X(0,0,Elcuthat,r,0,0)**4
            +(2*X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*Xpi(2,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)**2*((-deltambkin1+deltamcMS1)*r*Xpi(2,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -48*MBhat**2*((((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)**2-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(2,1,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +24*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(1,0,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(2,1,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(2,1,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(2,1,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +24*MBhat**2*(((-2*deltambkin1)/X(0,0,Elcuthat,r,0,0)+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(2,2,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(2,2,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(2,2,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +8*MBhat*(((-(deltambkin1/X(0,0,Elcuthat,r,0,0)**2)-(2*((-deltambkin1
            +deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**3)*X(0,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*X(0,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(3,0,Elcuthat,r,0,0)
            +(X(0,1,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            -4*(((-2*((-deltambkin1+deltamcMS1)*r*X(0,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*X(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +((-deltambkin1+deltamcMS1)*r*X(1,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*X(1,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(3,0,Elcuthat,r,0,0)
            +(X(1,0,Elcuthat,r,0,0)*((-deltambkin1+deltamcMS1)*r*Xpi(3,0,Elcuthat,r,0,1)
            -deltambkin1*Elcuthat*Xpi(3,0,Elcuthat,r,1,0)))/X(0,0,Elcuthat,r,0,0)**2)
            +(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -8*MBhat*((-(deltambkin1/X(0,0,Elcuthat,r,0,0))+(deltambkin1*r*X(0,0,Elcuthat,r,0,1)
            -deltamcMS1*r*X(0,0,Elcuthat,r,0,1)+deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)**2)*Xpi(3,1,Elcuthat,r,0,0)
            +((-deltambkin1+deltamcMS1)*r*Xpi(3,1,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(3,1,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0))
            +((deltambkin1*r*X(0,0,Elcuthat,r,0,1)-deltamcMS1*r*X(0,0,Elcuthat,r,0,1)
            +deltambkin1*Elcuthat*X(0,0,Elcuthat,r,1,0))*Xpi(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +6*deltambkin1*((192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(12*X(1,0,Elcuthat,r,0,0)**4*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(8*MBhat*X(3,1,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(X(4,0,Elcuthat,r,0,0)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*X(3,0,Elcuthat,r,0,0)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(12*X(1,0,Elcuthat,r,0,0)**3*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,1,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(3,0,Elcuthat,r,0,0)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(6*X(1,0,Elcuthat,r,0,0)**2*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(8*MBhat*X(0,1,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(4*X(1,0,Elcuthat,r,0,0)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+Xpi(4,0,Elcuthat,r,0,0)/X(0,0,Elcuthat,r,0,0))
            +((-deltambkin1+deltamcMS1)*r*Xpi(4,0,Elcuthat,r,0,1)-deltambkin1*Elcuthat*Xpi(4,0,Elcuthat,r,1,0))/X(0,0,Elcuthat,r,0,0)))
        if( flagDEBUG == 1):
            print("MXmoment n. 4 NLO pw = api*",resNLO*par.mbkin**8)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=(flagNLORhoD*Rhodpert1*((-192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(12*X(1,0,Elcuthat,r,0,0)**4*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(X(4,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat**4*nlopw.X1mixRhoD(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(12*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*nlopw.X1mixRhoD(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixRhoD(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*nlopw.X1mixRhoD(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixRhoD(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixRhoD(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*nlopw.X1mixRhoD(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-nlopw.X1mixRhoD(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            -(192*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*FLAGcD*MBhat**4*X(0,4,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*FLAGcD*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(72*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(12*FLAGcD*X(1,0,Elcuthat,r,0,0)**4*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*FLAGcD*MBhat**3*X(1,3,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*FLAGcD*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat**2*X(2,2,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*FLAGcD*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*FLAGcD*MBhat*X(3,1,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(FLAGcD*X(4,0,Elcuthat,r,0,0)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(960*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(384*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(32*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1920*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(192*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(1440*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(288*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(480*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(60*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(1152*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(nlo.X1mix(4,0,'SM',Elcuthat,r)*XD(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(192*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*FLAGcD*MBhat**4*X(0,3,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(288*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcD*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**3*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcD*MBhat**3*X(1,2,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*FLAGcD*MBhat**2*X(2,1,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*FLAGcD*MBhat*X(3,0,Elcuthat,r,0,0)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*FLAGcD*MBhat**4*X(0,1,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*FLAGcD*MBhat**3*X(1,0,Elcuthat,r,0,0)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(16*FLAGcD*MBhat**4*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*FLAGcD*MBhat**3*X(0,3,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*FLAGcD*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(12*FLAGcD*X(1,0,Elcuthat,r,0,0)**3*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*FLAGcD*MBhat**2*X(1,2,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*FLAGcD*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*FLAGcD*MBhat*X(2,1,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*FLAGcD*X(3,0,Elcuthat,r,0,0)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(3,0,'SM',Elcuthat,r)*XD(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)**2*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*FLAGcD*MBhat**3*X(0,1,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*FLAGcD*MBhat**2*X(1,0,Elcuthat,r,0,0)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(32*FLAGcD*MBhat**3*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(6*FLAGcD*X(1,0,Elcuthat,r,0,0)**2*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*FLAGcD*MBhat**2*X(0,1,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat*X(1,0,Elcuthat,r,0,0)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*FLAGcD*MBhat**2*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            +(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*FLAGcD*MBhat*X(0,1,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*FLAGcD*X(1,0,Elcuthat,r,0,0)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*XD(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(8*FLAGcD*MBhat*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)
            -(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(FLAGcD*XD(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)+(nlo.X1mix(0,0,'SM',Elcuthat,r)*XD(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2)
            +flagNLOMuPi*Mupipert1*((-192*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            +(72*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(12*X(1,0,Elcuthat,r,0,0)**4*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*X(3,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(X(4,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(192*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(96*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(0,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(0,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(16*MBhat**4*nlopw.X1mixMuPi(0,4,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            +(12*X(1,0,Elcuthat,r,0,0)**3*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*X(2,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(3,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(1,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(1,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(32*MBhat**3*nlopw.X1mixMuPi(1,3,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(24*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            -(6*X(1,0,Elcuthat,r,0,0)**2*nlopw.X1mixMuPi(2,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(2,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat**2*nlopw.X1mixMuPi(2,2,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-(8*MBhat*X(0,1,Elcuthat,r,0,0)*nlopw.X1mixMuPi(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(4*X(1,0,Elcuthat,r,0,0)*nlopw.X1mixMuPi(3,0,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)**2
            +(8*MBhat*nlopw.X1mixMuPi(3,1,Elcuthat,r))/X(0,0,Elcuthat,r,0,0)-nlopw.X1mixMuPi(4,0,Elcuthat,r)/X(0,0,Elcuthat,r,0,0)
            +(960*MBhat**4*X(0,1,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(1152*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(384*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(32*MBhat**4*X(0,4,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(1920*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(192*MBhat**3*X(0,3,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(1440*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            -(288*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(480*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(60*X(1,0,Elcuthat,r,0,0)**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**6
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(1152*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat*X(1,0,Elcuthat,r,0,0)**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(1,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(72*X(1,0,Elcuthat,r,0,0)**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(2,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat*X(0,1,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(24*X(1,0,Elcuthat,r,0,0)*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(16*MBhat*X(3,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(2*X(4,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat**4*nlo.X1mix(0,4,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*nlo.X1mix(1,3,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat**2*nlo.X1mix(2,2,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*nlo.X1mix(3,1,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(nlo.X1mix(4,0,'SM',Elcuthat,r)*Xpi(0,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(768*MBhat**4*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(128*MBhat**4*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(1152*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(96*MBhat*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(16*MBhat*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(576*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(144*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(0,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(288*MBhat**4*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(72*MBhat**2*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(192*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(128*MBhat**4*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**3*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(64*MBhat**4*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(0,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat**4*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(0,4,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(384*MBhat**3*X(0,1,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(64*MBhat**3*X(0,3,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(576*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(144*MBhat**2*X(0,2,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            -(48*X(1,0,Elcuthat,r,0,0)**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**5
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(144*MBhat*X(1,0,Elcuthat,r,0,0)*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,2,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*X(1,0,Elcuthat,r,0,0)*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat*X(2,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(3,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(96*MBhat**3*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(96*MBhat**2*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(96*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(48*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(32*MBhat**3*nlo.X1mix(0,3,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(144*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,2,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(144*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(36*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(48*MBhat*X(1,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(2,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*nlo.X1mix(1,2,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(2,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(24*MBhat*nlo.X1mix(2,1,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(3,0,'SM',Elcuthat,r)*Xpi(1,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(288*MBhat**3*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(288*MBhat**2*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(192*MBhat**3*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**3*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(48*MBhat**2*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(1,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(32*MBhat**3*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(1,3,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(72*MBhat**2*X(0,1,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(72*MBhat*X(0,1,Elcuthat,r,0,0)*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            +(18*X(1,0,Elcuthat,r,0,0)**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**4
            -(48*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(24*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(12*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(96*MBhat**2*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            +(48*MBhat**2*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(24*MBhat*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(2,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(24*MBhat**2*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(2,2,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(16*MBhat*X(0,1,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*X(1,0,Elcuthat,r,0,0)*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**3
            -(8*MBhat*nlo.X1mix(0,1,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(4*nlo.X1mix(1,0,'SM',Elcuthat,r)*Xpi(3,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            -(8*MBhat*nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(3,1,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2
            +(nlo.X1mix(0,0,'SM',Elcuthat,r)*Xpi(4,0,Elcuthat,r,0,0))/X(0,0,Elcuthat,r,0,0)**2))
        if( flagDEBUG == 1):
            print("MXmoment n. 4 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**8)
        res += api**2*resNNLO

    return res*par.mbkin**8


