from numba import jit, int64, float64
from kolya import parameters
from kolya import NLOpartonic as nlo
from kolya import NNLOpartonic as nnlo
from kolya import NLOpw as nlopw
from kolya import schemechange_KINMS as kin
import math

@jit(float64(int64, float64, float64, int64, int64), cache=True, nopython=True)
def X(i,u,r,dQ2,dr):
    """ Tree level functions (partonic) for Q2 moments and their derivatives """
    logu = math.log(u)
    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return     (-24*logu*r**4-r**4/u**4+(2*(r**3+r**5))/u**3-(4*r**4)/u**2-(6*(r**3+r**5))/u
            +6*r**3*(1+r**2)*u+4*r**4*u**2-2*r**3*(1+r**2)*u**3+r**4*u**4)
    if (dQ2 == 0 and dr == 0 and i==1):
        return     (logu*(-36*r**4-36*r**6)+(4*r**5)/(5*u**5)-(5*(r**4+r**6))/(2*u**4)+(2*(r**3
            +4*r**5+r**7))/u**3-(4*(r**4+r**6))/u**2-(2*(3*r**3+14*r**5+3*r**7))/u
            +2*r**3*(3+14*r**2+3*r**4)*u+4*r**4*(1+r**2)*u**2-2*r**3*(1+4*r**2
            +r**4)*u**3+(5*r**4*(1+r**2)*u**4)/2-(4*r**5*u**5)/5)
    if (dQ2 == 0 and dr == 0 and i==2):
        return     (logu*(-48*r**4-128*r**6-48*r**8)-(2*r**6)/(3*u**6)+(14*(r**5+r**7))/(5*u**5)
            -(4*(r**4+3*r**6+r**8))/u**4+(2*(r**3+8*r**5+8*r**7+r**9))/u**3-(2*(2*r**4
            +3*r**6+2*r**8))/u**2-(2*(3*r**3+31*r**5+31*r**7+3*r**9))/u+2*r**3*(3
            +31*r**2+31*r**4+3*r**6)*u+2*r**4*(2+3*r**2+2*r**4)*u**2-2*r**3*(1
            +8*r**2+8*r**4+r**6)*u**3+4*r**4*(1+3*r**2+r**4)*u**4-(14*r**5*(1+r**2)*u**5)/5
            +(2*r**6*u**6)/3)
    if (dQ2 == 0 and dr == 0 and i==3):
        return     (logu*(-60*r**4-300*r**6-300*r**8-60*r**10)+(4*r**7)/(7*u**7)-(3*(r**6
            +r**8))/u**6+(2*(3*r**5+8*r**7+3*r**9))/u**5+(-11*r**4-63*r**6-63*r**8
            -11*r**10)/(2*u**4)+(2*(r**3+13*r**5+26*r**7+13*r**9+r**11))/u**3+(
            -4*r**4-3*r**6-3*r**8-4*r**10)/u**2-(6*(r**3+18*r**5+40*r**7+18*r**9
            +r**11))/u+6*r**3*(1+18*r**2+40*r**4+18*r**6+r**8)*u+r**4*(4+3*r**2
            +3*r**4+4*r**6)*u**2-2*r**3*(1+13*r**2+26*r**4+13*r**6+r**8)*u**3+(r**4*(11
            +63*r**2+63*r**4+11*r**6)*u**4)/2-2*r**5*(3+8*r**2+3*r**4)*u**5+3*r**6*(1
            +r**2)*u**6-(4*r**7*u**7)/7)
    if (dQ2 == 0 and dr == 0 and i==4):
        return     (logu*(-72*r**4-576*r**6-1080*r**8-576*r**10-72*r**12)-r**8/(2*u**8)+(22*(r**7
            +r**9))/(7*u**7)-(4*(2*r**6+5*r**8+2*r**10))/u**6+(2*(26*r**5+127*r**7
            +127*r**9+26*r**11))/(5*u**5)+(-7*r**4-64*r**6-122*r**8-64*r**10-7*r**12)/u**4
            +(2*(r**3+19*r**5+61*r**7+61*r**9+19*r**11+r**13))/u**3-(4*(r**4-2*r**6
            -9*r**8-2*r**10+r**12))/u**2-(2*(3*r**3+83*r**5+321*r**7+321*r**9+83*r**11
            +3*r**13))/u+2*r**3*(3+83*r**2+321*r**4+321*r**6+83*r**8+3*r**10)*u
            +4*r**4*(1-2*r**2-9*r**4-2*r**6+r**8)*u**2-2*r**3*(1+19*r**2+61*r**4
            +61*r**6+19*r**8+r**10)*u**3+r**4*(7+64*r**2+122*r**4+64*r**6+7*r**8)*u**4
            -(2*r**5*(26+127*r**2+127*r**4+26*r**6)*u**5)/5+4*r**6*(2+5*r**2+2*r**4)*u**6
            -(22*r**7*(1+r**2)*u**7)/7+(r**8*u**8)/2)

    # derivative wrt Q2
    if (dQ2 == 1 and dr == 0 and i==0):
        return     ((4*r**3)/u**3-(6*(r**2+r**4))/u**2+(12*r**3)/u-12*r**3*u+6*r**2*(1+r**2)*u**2
            -4*r**3*u**3)
    if (dQ2 == 1 and dr == 0 and i==1):
        return     ((-4*r**4)/u**4+(10*(r**3+r**5))/u**3-(2*(3*r**2+14*r**4+3*r**6))/u**2+(18*(r**3
            +r**5))/u-18*r**3*(1+r**2)*u+2*r**2*(3+14*r**2+3*r**4)*u**2-10*r**3*(1
            +r**2)*u**3+4*r**4*u**4)
    if (dQ2 == 1 and dr == 0 and i==2):
        return     ((4*r**5)/u**5-(14*(r**4+r**6))/u**4+(4*(4*r**3+13*r**5+4*r**7))/u**3-(2*(3*r**2
            +31*r**4+31*r**6+3*r**8))/u**2+(8*(3*r**3+8*r**5+3*r**7))/u-8*r**3*(3
            +8*r**2+3*r**4)*u+2*r**2*(3+31*r**2+31*r**4+3*r**6)*u**2-4*r**3*(4
            +13*r**2+4*r**4)*u**3+14*r**4*(1+r**2)*u**4-4*r**5*u**5)
    if (dQ2 == 1 and dr == 0 and i==3):
        return     ((-4*r**6)/u**6+(18*(r**5+r**7))/u**5-(6*(5*r**4+14*r**6+5*r**8))/u**4+(2*(11*r**3
            +72*r**5+72*r**7+11*r**9))/u**3-(6*(r**2+18*r**4+40*r**6+18*r**8+r**10))/u**2
            +(30*(r**3+5*r**5+5*r**7+r**9))/u-30*r**3*(1+5*r**2+5*r**4+r**6)*u
            +6*r**2*(1+18*r**2+40*r**4+18*r**6+r**8)*u**2-2*r**3*(11+72*r**2+72*r**4
            +11*r**6)*u**3+6*r**4*(5+14*r**2+5*r**4)*u**4-18*r**5*(1+r**2)*u**5+4*r**6*u**6)
    if (dQ2 == 1 and dr == 0 and i==4):
        return     ((4*r**7)/u**7-(22*(r**6+r**8))/u**6+(4*(12*r**5+31*r**7+12*r**9))/u**5
            -(4*(13*r**4+69*r**6+69*r**8+13*r**10))/u**4+(4*(7*r**3+76*r**5+153*r**7
            +76*r**9+7*r**11))/u**3-(2*(3*r**2+83*r**4+321*r**6+321*r**8+83*r**10
            +3*r**12))/u**2+(36*(r**3+8*r**5+15*r**7+8*r**9+r**11))/u-36*r**3*(1
            +8*r**2+15*r**4+8*r**6+r**8)*u+2*r**2*(3+83*r**2+321*r**4+321*r**6
            +83*r**8+3*r**10)*u**2-4*r**3*(7+76*r**2+153*r**4+76*r**6+7*r**8)*u**3
            +4*r**4*(13+69*r**2+69*r**4+13*r**6)*u**4-4*r**5*(12+31*r**2+12*r**4)*u**5
            +22*r**6*(1+r**2)*u**6-4*r**7*u**7)

    # derivative wrt r 
    if (dQ2 == 0 and dr == 1 and i==0):
        return     (-96*logu*r**3-(4*r**4)/u**3+(12*(r**3+r**5))/u**2-(12*(2*r**2+5*r**4))/u
            +12*r**2*(2+5*r**2)*u-12*r**3*(1+r**2)*u**2+4*r**4*u**3)
    if (dQ2 == 0 and dr == 1 and i==1):
        return     (-72*logu*r**3*(2+3*r**2)+(3*r**5)/u**4-(12*(r**4+r**6))/u**3+(12*(2*r**3
            +5*r**5+r**7))/u**2-(12*(2*r**2+17*r**4+7*r**6))/u+12*r**2*(2+17*r**2
            +7*r**4)*u-12*r**3*(2+5*r**2+r**4)*u**2+12*r**4*(1+r**2)*u**3-3*r**5*u**4)
    if (dQ2 == 0 and dr == 1 and i==2):
        return     (-192*logu*r**3*(1+4*r**2+2*r**4)-(12*r**6)/(5*u**5)+(12*(r**5+r**7))/u**4
            -(4*(7*r**4+17*r**6+5*r**8))/u**3+(12*(3*r**3+17*r**5+11*r**7+r**9))/u**2
            -(12*(2*r**2+35*r**4+52*r**6+9*r**8))/u+12*r**2*(2+35*r**2+52*r**4
            +9*r**6)*u-12*r**3*(3+17*r**2+11*r**4+r**6)*u**2+4*r**4*(7+17*r**2
            +5*r**4)*u**3-12*r**5*(1+r**2)*u**4+(12*r**6*u**5)/5)
    if (dQ2 == 0 and dr == 1 and i==3):
        return     (-120*logu*r**3*(2+15*r**2+20*r**4+5*r**6)+(2*r**7)/u**6-(12*(r**6+r**8))/u**5
            +(3*(11*r**5+26*r**7+9*r**9))/u**4-(4*(13*r**4+62*r**6+48*r**8+7*r**10))/u**3
            +(6*(8*r**3+82*r**5+125*r**7+38*r**9+2*r**11))/u**2-(12*(2*r**2+59*r**4
            +185*r**6+115*r**8+11*r**10))/u+12*r**2*(2+59*r**2+185*r**4+115*r**6
            +11*r**8)*u-6*r**3*(8+82*r**2+125*r**4+38*r**6+2*r**8)*u**2+4*r**4*(13
            +62*r**2+48*r**4+7*r**6)*u**3-3*r**5*(11+26*r**2+9*r**4)*u**4+12*r**6*(1
            +r**2)*u**5-2*r**7*u**6)
    if (dQ2 == 0 and dr == 1 and i==4):
        return     (-288*logu*r**3*(1+12*r**2+30*r**4+20*r**6+3*r**8)-(12*r**8)/(7*u**7)
            +(12*(r**7+r**9))/u**6-(12*(16*r**6+37*r**8+14*r**10))/(5*u**5)+(24*(3*r**5
            +13*r**7+11*r**9+2*r**11))/u**4-(12*(7*r**4+56*r**6+87*r**8+34*r**10
            +3*r**12))/u**3+(12*(5*r**3+81*r**5+227*r**7+163*r**9+29*r**11+r**13))/u**2
            -(12*(2*r**2+89*r**4+476*r**6+625*r**8+214*r**10+13*r**12))/u+12*r**2*(2
            +89*r**2+476*r**4+625*r**6+214*r**8+13*r**10)*u-12*r**3*(5+81*r**2
            +227*r**4+163*r**6+29*r**8+r**10)*u**2+12*r**4*(7+56*r**2+87*r**4+34*r**6
            +3*r**8)*u**3-24*r**5*(3+13*r**2+11*r**4+2*r**6)*u**4+(12*r**6*(16+37*r**2
            +14*r**4)*u**5)/5-12*r**7*(1+r**2)*u**6+(12*r**8*u**7)/7)

    # second derivative wrt Q2
    if (dQ2 == 2 and dr == 0 and i==0):
        return     (24*r**2-(12*(r-2*r**2+r**3))/(-1+u)-(12*r**2)/u**2+(12*r*(1+r**2))/u
            -12*r*(1+r**2)*u+12*r**2*u**2-(12*(r+2*r**2+r**3))/(1+u))
    if (dQ2 == 2 and dr == 0 and i==1):
        return     (48*r**2*(1+r**2)-(12*(r-4*r**2+6*r**3-4*r**4+r**5))/(-1+u)+(16*r**3)/u**3
            -(30*(r**2+r**4))/u**2+(12*(r+6*r**3+r**5))/u-12*r*(1+6*r**2+r**4)*u
            +30*r**2*(1+r**2)*u**2-16*r**3*u**3-(12*(r+4*r**2+6*r**3+4*r**4+r**5))/(1
            +u))
    if (dQ2 == 2 and dr == 0 and i==2):
        return     (24*r**2*(3+10*r**2+3*r**4)-(12*(r-6*r**2+15*r**3-20*r**4+15*r**5
            -6*r**6+r**7))/(-1+u)-(20*r**4)/u**4+(56*(r**3+r**5))/u**3-(16*(3*r**2
            +11*r**4+3*r**6))/u**2+(12*(r+15*r**3+15*r**5+r**7))/u-12*r*(1+15*r**2
            +15*r**4+r**6)*u+16*r**2*(3+11*r**2+3*r**4)*u**2-56*r**3*(1+r**2)*u**3
            +20*r**4*u**4-(12*(r+6*r**2+15*r**3+20*r**4+15*r**5+6*r**6+r**7))/(1
            +u))
    if (dQ2 == 2 and dr == 0 and i==3):
        return     (96*r**2*(1+7*r**2+7*r**4+r**6)-(12*(r-8*r**2+28*r**3-56*r**4+70*r**5
            -56*r**6+28*r**7-8*r**8+r**9))/(-1+u)+(24*r**5)/u**5-(90*(r**4+r**6))/u**4
            +(120*(r**3+3*r**5+r**7))/u**3-(6*(11*r**2+87*r**4+87*r**6+11*r**8))/u**2
            +(12*(r+28*r**3+70*r**5+28*r**7+r**9))/u-12*r*(1+28*r**2+70*r**4
            +28*r**6+r**8)*u+6*r**2*(11+87*r**2+87*r**4+11*r**6)*u**2-120*r**3*(1
            +3*r**2+r**4)*u**3+90*r**4*(1+r**2)*u**4-24*r**5*u**5-(12*(r+8*r**2
            +28*r**3+56*r**4+70*r**5+56*r**6+28*r**7+8*r**8+r**9))/(1+u))
    if (dQ2 == 2 and dr == 0 and i==4):
        return     (24*r**2*(5+60*r**2+126*r**4+60*r**6+5*r**8)-(12*(r-10*r**2+45*r**3
            -120*r**4+210*r**5-252*r**6+210*r**7-120*r**8+45*r**9-10*r**10+r**11))/(
            -1+u)-(28*r**6)/u**6+(132*(r**5+r**7))/u**5-(24*(10*r**4+27*r**6+10*r**8))/u**4
            +(4*(52*r**3+309*r**5+309*r**7+52*r**9))/u**3-(12*(7*r**2+96*r**4+207*r**6
            +96*r**8+7*r**10))/u**2+(12*(r+45*r**3+210*r**5+210*r**7+45*r**9+r**11))/u
            -12*r*(1+45*r**2+210*r**4+210*r**6+45*r**8+r**10)*u+12*r**2*(7+96*r**2
            +207*r**4+96*r**6+7*r**8)*u**2-4*r**3*(52+309*r**2+309*r**4+52*r**6)*u**3
            +24*r**4*(10+27*r**2+10*r**4)*u**4-132*r**5*(1+r**2)*u**5+28*r**6*u**6
            -(12*(r+10*r**2+45*r**3+120*r**4+210*r**5+252*r**6+210*r**7+120*r**8
            +45*r**9+10*r**10+r**11))/(1+u))

    # second derivative wrt r 
    if (dQ2 == 0 and dr == 2 and i==0):
        return     (96*r**2*(2-3*logu+2*r**2)-(48*(r-4*r**2+6*r**3-4*r**4+r**5))/(-1
            +u)-(4*r**3)/u**3+(12*(r**2+r**4))/u**2+(12*(-2*r-9*r**3+4*r**5))/u
            -12*r*(-2-9*r**2+4*r**4)*u-12*r**2*(1+r**2)*u**2+4*r**3*u**3-(48*(r
            +4*r**2+6*r**3+4*r**4+r**5))/(1+u))
    if (dQ2 == 0 and dr == 2 and i==1):
        return     (24*r**2*(12-18*logu+40*r**2-45*logu*r**2+12*r**4)-(48*(r-6*r**2+15*r**3
            -20*r**4+15*r**5-6*r**6+r**7))/(-1+u)+(3*r**4)/u**4-(12*(r**3+r**5))/u**3
            -(12*(-2*r**2-7*r**4+r**6))/u**2+(12*(-2*r-37*r**3-7*r**5+4*r**7))/u
            -12*r*(-2-37*r**2-7*r**4+4*r**6)*u+12*r**2*(-2-7*r**2+r**4)*u**2+12*r**3*(1
            +r**2)*u**3-3*r**4*u**4-(48*(r+6*r**2+15*r**3+20*r**4+15*r**5+6*r**6
            +r**7))/(1+u))
    if (dQ2 == 0 and dr == 2 and i==2):
        return     (192*r**2*(2-3*logu+14*r**2-20*logu*r**2+14*r**4-14*logu*r**4+2*r**6)
            -(48*(r-8*r**2+28*r**3-56*r**4+70*r**5-56*r**6+28*r**7-8*r**8+r**9))/(
            -1+u)-(12*r**5)/(5*u**5)+(12*(r**4+r**6))/u**4-(4*(7*r**3+21*r**5+r**7))/u**3
            -(12*(-3*r**2-29*r**4-11*r**6+3*r**8))/u**2+(12*(-2*r-79*r**3-148*r**5
            -r**7+4*r**9))/u-12*r*(-2-79*r**2-148*r**4-r**6+4*r**8)*u+12*r**2*(
            -3-29*r**2-11*r**4+3*r**6)*u**2+4*r**3*(7+21*r**2+r**4)*u**3-12*r**4*(1
            +r**2)*u**4+(12*r**5*u**5)/5-(48*(r+8*r**2+28*r**3+56*r**4+70*r**5
            +56*r**6+28*r**7+8*r**8+r**9))/(1+u))
    if (dQ2 == 0 and dr == 2 and i==3):
        return     (24*r**2*(20-30*logu+240*r**2-375*logu*r**2+504*r**4-700*logu*r**4+240*r**6
            -225*logu*r**6+20*r**8)-(48*(r-10*r**2+45*r**3-120*r**4+210*r**5
            -252*r**6+210*r**7-120*r**8+45*r**9-10*r**10+r**11))/(-1+u)+(2*r**6)/u**6
            -(12*(r**5+r**7))/u**5+(3*(11*r**4+30*r**6+5*r**8))/u**4+(4*(-13*r**3
            -90*r**5-48*r**7+5*r**9))/u**3-(6*(-8*r**2-150*r**4-249*r**6-18*r**8
            +10*r**10))/u**2+(12*(-2*r-135*r**3-605*r**5-395*r**7+9*r**9+4*r**11))/u
            -12*r*(-2-135*r**2-605*r**4-395*r**6+9*r**8+4*r**10)*u+6*r**2*(-8
            -150*r**2-249*r**4-18*r**6+10*r**8)*u**2-4*r**3*(-13-90*r**2-48*r**4
            +5*r**6)*u**3-3*r**4*(11+30*r**2+5*r**4)*u**4+12*r**5*(1+r**2)*u**5-2*r**6*u**6
            -(48*(r+10*r**2+45*r**3+120*r**4+210*r**5+252*r**6+210*r**7+120*r**8
            +45*r**9+10*r**10+r**11))/(1+u))
    if (dQ2 == 0 and dr == 2 and i==4):
        return     (96*r**2*(6-9*logu+110*r**2-180*logu*r**2+396*r**4-630*logu*r**4+396*r**6
            -540*logu*r**6+110*r**8-99*logu*r**8+6*r**10)-(48*(r-12*r**2+66*r**3
            -220*r**4+495*r**5-792*r**6+924*r**7-792*r**8+495*r**9-220*r**10+66*r**11
            -12*r**12+r**13))/(-1+u)-(12*r**7)/(7*u**7)+(12*(r**6+r**8))/u**6-(12*(16*r**5
            +41*r**7+10*r**9))/(5*u**5)+(24*(3*r**4+17*r**6+11*r**8))/u**4+(12*(
            -7*r**3-88*r**5-139*r**7-22*r**9+5*r**11))/u**3-(12*(-5*r**2-153*r**4
            -547*r**6-355*r**8+3*r**10+7*r**12))/u**2+(12*(-2*r-205*r**3-1640*r**5
            -2649*r**7-854*r**9+23*r**11+4*r**13))/u-12*r*(-2-205*r**2-1640*r**4
            -2649*r**6-854*r**8+23*r**10+4*r**12)*u+12*r**2*(-5-153*r**2-547*r**4
            -355*r**6+3*r**8+7*r**10)*u**2-12*r**3*(-7-88*r**2-139*r**4-22*r**6
            +5*r**8)*u**3-24*r**4*(3+17*r**2+11*r**4)*u**4+(12*r**5*(16+41*r**2+10*r**4)*u**5)/5
            -12*r**6*(1+r**2)*u**6+(12*r**7*u**7)/7-(48*(r+12*r**2+66*r**3+220*r**4
            +495*r**5+792*r**6+924*r**7+792*r**8+495*r**9+220*r**10+66*r**11+12*r**12
            +r**13))/(1+u))

    # second derivative wrt r and Q2 
    if (dQ2 == 1 and dr == 1 and i==0):
        return     (-24*r*(1+3*r**2)+(24*(-r+3*r**2-3*r**3+r**4))/(-1+u)+(12*r**3)/u**2
            -(24*r**2*(1+r**2))/u+24*r**2*(1+r**2)*u-12*r**3*u**2+(24*(r+3*r**2
            +3*r**3+r**4))/(1+u))
    if (dQ2 == 1 and dr == 1 and i==1):
        return     (-24*r*(1+10*r**2+5*r**4)+(24*(-r+5*r**2-10*r**3+10*r**4-5*r**5+r**6))/(
            -1+u)-(12*r**4)/u**3+(36*(r**3+r**5))/u**2-(12*(4*r**2+11*r**4+2*r**6))/u
            +12*r**2*(4+11*r**2+2*r**4)*u-36*r**3*(1+r**2)*u**2+12*r**4*u**3+(24*(r
            +5*r**2+10*r**3+10*r**4+5*r**5+r**6))/(1+u))
    if (dQ2 == 1 and dr == 1 and i==2):
        return     (-24*r*(1+21*r**2+35*r**4+7*r**6)+(24*(-r+7*r**2-21*r**3+35*r**4
            -35*r**5+21*r**6-7*r**7+r**8))/(-1+u)+(12*r**5)/u**4-(48*(r**4+r**6))/u**3
            +(12*(7*r**3+18*r**5+5*r**7))/u**2-(24*(3*r**2+19*r**4+13*r**6+r**8))/u
            +24*r**2*(3+19*r**2+13*r**4+r**6)*u-12*r**3*(7+18*r**2+5*r**4)*u**2
            +48*r**4*(1+r**2)*u**3-12*r**5*u**4+(24*(r+7*r**2+21*r**3+35*r**4+35*r**5
            +21*r**6+7*r**7+r**8))/(1+u))
    if (dQ2 == 1 and dr == 1 and i==3):
        return     (-24*r*(1+36*r**2+126*r**4+84*r**6+9*r**8)+(24*(-r+9*r**2-36*r**3
            +84*r**4-126*r**5+126*r**6-84*r**7+36*r**8-9*r**9+r**10))/(-1+u)
            -(12*r**6)/u**5+(60*(r**5+r**7))/u**4-(12*(11*r**4+27*r**6+9*r**8))/u**3
            +(12*(13*r**3+67*r**5+53*r**7+7*r**9))/u**2-(12*(8*r**2+93*r**4+152*r**6
            +47*r**8+2*r**10))/u+12*r**2*(8+93*r**2+152*r**4+47*r**6+2*r**8)*u
            -12*r**3*(13+67*r**2+53*r**4+7*r**6)*u**2+12*r**4*(11+27*r**2+9*r**4)*u**3
            -60*r**5*(1+r**2)*u**4+12*r**6*u**5+(24*(r+9*r**2+36*r**3+84*r**4+126*r**5
            +126*r**6+84*r**7+36*r**8+9*r**9+r**10))/(1+u))
    if (dQ2 == 1 and dr == 1 and i==4):
        return     (-24*r*(1+55*r**2+330*r**4+462*r**6+165*r**8+11*r**10)+(24*(-r+11*r**2
            -55*r**3+165*r**4-330*r**5+462*r**6-462*r**7+330*r**8-165*r**9+55*r**10
            -11*r**11+r**12))/(-1+u)+(12*r**7)/u**6-(72*(r**6+r**8))/u**5+(24*(8*r**5
            +19*r**7+7*r**9))/u**4-(24*(12*r**4+55*r**6+47*r**8+8*r**10))/u**3+(12*(21*r**3
            +184*r**5+299*r**7+116*r**9+9*r**11))/u**2-(24*(5*r**2+93*r**4+282*r**6
            +210*r**8+37*r**10+r**12))/u+24*r**2*(5+93*r**2+282*r**4+210*r**6
            +37*r**8+r**10)*u-12*r**3*(21+184*r**2+299*r**4+116*r**6+9*r**8)*u**2
            +24*r**4*(12+55*r**2+47*r**4+8*r**6)*u**3-24*r**5*(8+19*r**2+7*r**4)*u**4
            +72*r**6*(1+r**2)*u**5-12*r**7*u**6+(24*(r+11*r**2+55*r**3+165*r**4
            +330*r**5+462*r**6+462*r**7+330*r**8+165*r**9+55*r**10+11*r**11+r**12))/(1
            +u))
    return 0. 

@jit(float64(int64, float64, float64, int64, int64), cache=True, nopython=True)
def XG(i,u,r,dQ2,dr):
    """ Tree level functions (muG) for Q2 moments and their derivatives """
    logu = math.log(u)
    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return     (-12*logu*r**4-(5*r**4)/(2*u**4)+(5*(r**3+r**5))/u**3-(14*r**4)/u**2+(-4*r
            +9*r**3-3*r**5)/u+r*(4-9*r**2+3*r**4)*u+14*r**4*u**2-5*r**3*(1+r**2)*u**3
            +(5*r**4*u**4)/2)
    if (dQ2 == 0 and dr == 0 and i==1):
        return     (-2*logu*r**2*(4-3*r**2+9*r**4)+(2*r**5)/u**5-(25*(r**4+r**6))/(4*u**4)
            +(15*r**3+68*r**5+15*r**7)/(3*u**3)-(2*(-r**2+13*r**4+10*r**6))/u**2
            +(-4*r+5*r**3+22*r**5-3*r**7)/u+r*(4-5*r**2-22*r**4+3*r**6)*u+2*r**2*(
            -1+13*r**2+10*r**4)*u**2-(r**3*(15+68*r**2+15*r**4)*u**3)/3+(25*r**4*(1
            +r**2)*u**4)/4-2*r**5*u**5)
    if (dQ2 == 0 and dr == 0 and i==2):
        return     (-8*logu*(-1+r)**2*r**2*(1+r)**2*(2+3*r**2)-(5*r**6)/(3*u**6)+(7*(r**5
            +r**7))/u**5-(2*(5*r**4+16*r**6+5*r**8))/u**4+(11*r**3+160*r**5+148*r**7
            +15*r**9)/(3*u**3)+(4*r**2-34*r**4-91*r**6-26*r**8)/u**2+(-4*r-11*r**3
            +85*r**5+41*r**7-3*r**9)/u+r*(4+11*r**2-85*r**4-41*r**6+3*r**8)*u
            +r**2*(-4+34*r**2+91*r**4+26*r**6)*u**2-(r**3*(11+160*r**2+148*r**4
            +15*r**6)*u**3)/3+2*r**4*(5+16*r**2+5*r**4)*u**4-7*r**5*(1+r**2)*u**5
            +(5*r**6*u**6)/3)
    if (dQ2 == 0 and dr == 0 and i==3):
        return     (-6*logu*r**2*(4+5*r**2-35*r**4-15*r**6+5*r**8)+(10*r**7)/(7*u**7)-(15*(r**6
            +r**8))/(2*u**6)+(75*r**5+208*r**7+75*r**9)/(5*u**5)+(-51*r**4-363*r**6
            -351*r**8-55*r**10)/(4*u**4)+(r**3+93*r**5+206*r**7+85*r**9+5*r**11)/u**3
            +(12*r**2-60*r**4-495*r**6-423*r**8-64*r**10)/(2*u**2)+(-4*r-39*r**3
            +150*r**5+340*r**7+66*r**9-3*r**11)/u+r*(4+39*r**2-150*r**4-340*r**6
            -66*r**8+3*r**10)*u+(r**2*(-12+60*r**2+495*r**4+423*r**6+64*r**8)*u**2)/2
            -r**3*(1+93*r**2+206*r**4+85*r**6+5*r**8)*u**3+(r**4*(51+363*r**2+351*r**4
            +55*r**6)*u**4)/4-(r**5*(75+208*r**2+75*r**4)*u**5)/5+(15*r**6*(1+r**2)*u**6)/2
            -(10*r**7*u**7)/7)
    if (dQ2 == 0 and dr == 0 and i==4):
        return     (-4*logu*r**2*(8+33*r**2-120*r**4-245*r**6-48*r**8+9*r**10)-(5*r**8)/(4*u**8)
            +(55*(r**7+r**9))/(7*u**7)-(2*(30*r**6+77*r**8+30*r**10))/(3*u**6)+(126*r**5
            +691*r**7+679*r**9+130*r**11)/(5*u**5)+(-27*r**4-384*r**6-770*r**8-368*r**10
            -35*r**12)/(2*u**4)+(-9*r**3+393*r**5+1755*r**7+1647*r**9+389*r**11
            +15*r**13)/(3*u**3)-(2*(-4*r**2+3*r**4+246*r**6+469*r**8+202*r**10+19*r**12))/u**2
            +(-4*r-79*r**3+141*r**5+1195*r**7+919*r**9+97*r**11-3*r**13)/u+r*(4
            +79*r**2-141*r**4-1195*r**6-919*r**8-97*r**10+3*r**12)*u+2*r**2*(-4
            +3*r**2+246*r**4+469*r**6+202*r**8+19*r**10)*u**2-(r**3*(-9+393*r**2
            +1755*r**4+1647*r**6+389*r**8+15*r**10)*u**3)/3+(r**4*(27+384*r**2+770*r**4
            +368*r**6+35*r**8)*u**4)/2-(r**5*(126+691*r**2+679*r**4+130*r**6)*u**5)/5
            +(2*r**6*(30+77*r**2+30*r**4)*u**6)/3-(55*r**7*(1+r**2)*u**7)/7+(5*r**8*u**8)/4)

    # derivative wrt Q2
    if (dQ2 == 1 and dr == 0 and i==0):
        return     (4*(-1+6*r**2+3*r**4)+(4*(-1+6*r**2-8*r**3+3*r**4))/(-1+u)+(10*r**3)/u**3
            -(15*(r**2+r**4))/u**2+(38*r**3)/u-38*r**3*u+15*r**2*(1+r**2)*u**2-10*r**3*u**3
            -(4*(-1+6*r**2+8*r**3+3*r**4))/(1+u))
    if (dQ2 == 1 and dr == 0 and i==1):
        return     (4*(-1+5*r**2+25*r**4+3*r**6)+(4*(-1+2*r+5*r**2-20*r**3+25*r**4
            -14*r**5+3*r**6))/(-1+u)-(10*r**4)/u**4+(25*(r**3+r**5))/u**3-(3*(5*r**2
            +26*r**4+5*r**6))/u**2+(-4*r+77*r**3+65*r**5)/u-r*(-4+77*r**2+65*r**4)*u
            +3*r**2*(5+26*r**2+5*r**4)*u**2-25*r**3*(1+r**2)*u**3+10*r**4*u**4-(4*(
            -1-2*r+5*r**2+20*r**3+25*r**4+14*r**5+3*r**6))/(1+u))
    if (dQ2 == 1 and dr == 0 and i==2):
        return     (4*(-1+70*r**4+56*r**6+3*r**8)+(4*(-1+4*r-28*r**3+70*r**4-84*r**5
            +56*r**6-20*r**7+3*r**8))/(-1+u)+(10*r**5)/u**5-(35*(r**4+r**6))/u**4
            +(2*(20*r**3+69*r**5+20*r**7))/u**3+(-11*r**2-195*r**4-183*r**6-15*r**8)/u**2
            +(4*(-2*r+27*r**3+80*r**5+23*r**7))/u-4*r*(-2+27*r**2+80*r**4+23*r**6)*u
            +r**2*(11+195*r**2+183*r**4+15*r**6)*u**2-2*r**3*(20+69*r**2+20*r**4)*u**3
            +35*r**4*(1+r**2)*u**4-10*r**5*u**5-(4*(-1-4*r+28*r**3+70*r**4+84*r**5
            +56*r**6+20*r**7+3*r**8))/(1+u))
    if (dQ2 == 1 and dr == 0 and i==3):
        return     (4*(-1-9*r**2+126*r**4+294*r**6+99*r**8+3*r**10)+(4*(-1+6*r-9*r**2
            -24*r**3+126*r**4-252*r**5+294*r**6-216*r**7+99*r**8-26*r**9+3*r**10))/(
            -1+u)-(10*r**6)/u**6+(45*(r**5+r**7))/u**5+(-75*r**4-218*r**6-75*r**8)/u**4
            +(51*r**3+408*r**5+396*r**7+55*r**9)/u**3+(-3*r**2-354*r**4-836*r**6
            -330*r**8-15*r**10)/u**2+(-12*r+111*r**3+903*r**5+819*r**7+119*r**9)/u
            -r*(-12+111*r**2+903*r**4+819*r**6+119*r**8)*u+r**2*(3+354*r**2+836*r**4
            +330*r**6+15*r**8)*u**2-r**3*(51+408*r**2+396*r**4+55*r**6)*u**3+r**4*(75
            +218*r**2+75*r**4)*u**4-45*r**5*(1+r**2)*u**5+10*r**6*u**6-(4*(-1-6*r
            -9*r**2+24*r**3+126*r**4+252*r**5+294*r**6+216*r**7+99*r**8+26*r**9
            +3*r**10))/(1+u))
    if (dQ2 == 1 and dr == 0 and i==4):
        return     (4*(-1-22*r**2+165*r**4+924*r**6+825*r**8+154*r**10+3*r**12)+(4*(
            -1+8*r-22*r**2+165*r**4-528*r**5+924*r**6-1056*r**7+825*r**8-440*r**9
            +154*r**10-32*r**11+3*r**12))/(-1+u)+(10*r**7)/u**7-(55*(r**6+r**8))/u**6
            +(6*(20*r**5+53*r**7+20*r**9))/u**5-(2*(63*r**4+373*r**6+367*r**8+65*r**10))/u**4
            +(2*(27*r**3+444*r**5+929*r**7+428*r**9+35*r**11))/u**3+(9*r**2-519*r**4
            -2501*r**6-2381*r**8-519*r**10-15*r**12)/u**2+(2*(-8*r+33*r**3+936*r**5
            +1867*r**7+832*r**9+73*r**11))/u-2*r*(-8+33*r**2+936*r**4+1867*r**6
            +832*r**8+73*r**10)*u+r**2*(-9+519*r**2+2501*r**4+2381*r**6+519*r**8
            +15*r**10)*u**2-2*r**3*(27+444*r**2+929*r**4+428*r**6+35*r**8)*u**3+2*r**4*(63
            +373*r**2+367*r**4+65*r**6)*u**4-6*r**5*(20+53*r**2+20*r**4)*u**5+55*r**6*(1
            +r**2)*u**6-10*r**7*u**7-(4*(-1-8*r-22*r**2+165*r**4+528*r**5+924*r**6
            +1056*r**7+825*r**8+440*r**9+154*r**10+32*r**11+3*r**12))/(1+u))

    # derivative wrt r
    if (dQ2 == 0 and dr == 1 and i==0):
        return     (-8*r*(-1+14*r**2+6*logu*r**2+3*r**4)-(8*(1-r-6*r**2+14*r**3-11*r**4
            +3*r**5))/(-1+u)-(10*r**4)/u**3+(2*(11*r**3+15*r**5))/u**2-(2*(6*r**2
            +59*r**4))/u+2*r**2*(6+59*r**2)*u-2*r**3*(11+15*r**2)*u**2+10*r**4*u**3
            +(8*(-1-r+6*r**2+14*r**3+11*r**4+3*r**5))/(1+u))
    if (dQ2 == 0 and dr == 1 and i==1):
        return     (-4*r*(-6+4*logu+50*r**2-6*logu*r**2+78*r**4+27*logu*r**4+6*r**6)
            -(8*(1-3*r-3*r**2+25*r**3-45*r**4+39*r**5-17*r**6+3*r**7))/(-1
            +u)+(15*r**5)/(2*u**4)-(2*(37*r**4+45*r**6))/(3*u**3)+(2*(14*r**3+63*r**5
            +15*r**7))/u**2-(2*(6*r**2+111*r**4+89*r**6))/u+2*r**2*(6+111*r**2+89*r**4)*u
            -2*r**3*(14+63*r**2+15*r**4)*u**2+(2*r**4*(37+45*r**2)*u**3)/3-(15*r**5*u**4)/2
            +(8*(-1-3*r+3*r**2+25*r**3+45*r**4+39*r**5+17*r**6+3*r**7))/(1
            +u))
    if (dQ2 == 0 and dr == 1 and i==2):
        return     (-8*r*(-5+4*logu+28*r**2-4*logu*r**2+154*r**4-24*logu*r**4+76*r**6
            +24*logu*r**6+3*r**8)-(8*(1-5*r+4*r**2+28*r**3-98*r**4+154*r**5
            -140*r**6+76*r**7-23*r**8+3*r**9))/(-1+u)-(6*r**6)/u**5+(2*(13*r**5
            +15*r**7))/u**4-(2*(65*r**4+223*r**6+75*r**8))/(3*u**3)+(2*(17*r**3+151*r**5
            +145*r**7+15*r**9))/u**2-(2*(14*r**2+133*r**4+380*r**6+119*r**8))/u
            +2*r**2*(14+133*r**2+380*r**4+119*r**6)*u-2*r**3*(17+151*r**2+145*r**4
            +15*r**6)*u**2+(2*r**4*(65+223*r**2+75*r**4)*u**3)/3-2*r**5*(13+15*r**2)*u**4
            +6*r**6*u**5+(8*(-1-5*r-4*r**2+28*r**3+98*r**4+154*r**5+140*r**6
            +76*r**7+23*r**8+3*r**9))/(1+u))
    if (dQ2 == 0 and dr == 1 and i==3):
        return     (-4*r*(-14+12*logu+30*r**2+30*logu*r**2+756*r**4-315*logu*r**4+1020*r**6
            -180*logu*r**6+250*r**8+75*logu*r**8+6*r**10)-(8*(1-7*r+15*r**2
            +15*r**3-150*r**4+378*r**5-546*r**6+510*r**7-315*r**8+125*r**9-29*r**10
            +3*r**11))/(-1+u)+(5*r**7)/u**6-(2*(67*r**6+75*r**8))/(5*u**5)+(117*r**5
            +350*r**7+135*r**9)/(2*u**4)-(2*(33*r**4+214*r**6+216*r**8+35*r**10))/u**3
            +(48*r**3+534*r**5+1195*r**7+514*r**9+30*r**11)/u**2-(2*(30*r**2+165*r**4
            +719*r**6+885*r**8+149*r**10))/u+2*r**2*(30+165*r**2+719*r**4+885*r**6
            +149*r**8)*u-r**3*(48+534*r**2+1195*r**4+514*r**6+30*r**8)*u**2+2*r**4*(33
            +214*r**2+216*r**4+35*r**6)*u**3-(r**5*(117+350*r**2+135*r**4)*u**4)/2
            +(2*r**6*(67+75*r**2)*u**5)/5-5*r**7*u**6+(8*(-1-7*r-15*r**2+15*r**3
            +150*r**4+378*r**5+546*r**6+510*r**7+315*r**8+125*r**9+29*r**10+3*r**11))/(1
            +u))
    if (dQ2 == 0 and dr == 1 and i==4):
        return     (-8*r*(-9+8*logu-22*r**2+66*logu*r**2+693*r**4-360*logu*r**4+1980*r**6
            -980*logu*r**6+1265*r**8-240*logu*r**8+186*r**10+54*logu*r**10+3*r**12)
            -(8*(1-9*r+30*r**2-22*r**3-165*r**4+693*r**5-1452*r**6+1980*r**7
            -1881*r**8+1265*r**9-594*r**10+186*r**11-35*r**12+3*r**13))/(-1+u)
            -(30*r**8)/(7*u**7)+(2*(41*r**7+45*r**9))/(3*u**6)-(2*(184*r**6+507*r**8
            +210*r**10))/(5*u**5)+(4*(27*r**5+147*r**7+151*r**9+30*r**11))/u**4-(2*(147*r**4
            +1392*r**6+2835*r**8+1402*r**10+135*r**12))/(3*u**3)+(2*(39*r**3+423*r**5
            +1545*r**7+1621*r**9+399*r**11+15*r**13))/u**2-(2*(54*r**2+303*r**4
            +788*r**6+2439*r**8+1698*r**10+179*r**12))/u+2*r**2*(54+303*r**2+788*r**4
            +2439*r**6+1698*r**8+179*r**10)*u-2*r**3*(39+423*r**2+1545*r**4+1621*r**6
            +399*r**8+15*r**10)*u**2+(2*r**4*(147+1392*r**2+2835*r**4+1402*r**6
            +135*r**8)*u**3)/3-4*r**5*(27+147*r**2+151*r**4+30*r**6)*u**4+(2*r**6*(184
            +507*r**2+210*r**4)*u**5)/5-(2*r**7*(41+45*r**2)*u**6)/3+(30*r**8*u**7)/7
            +(8*(-1-9*r-30*r**2-22*r**3+165*r**4+693*r**5+1452*r**6+1980*r**7
            +1881*r**8+1265*r**9+594*r**10+186*r**11+35*r**12+3*r**13))/(1+u))
    return 0. 

@jit(float64(int64, float64, float64, int64, int64, int64), cache=True, nopython=True)
def XD(i,u,r,dQ2,dr,isPERP):
    """ Tree level functions (rhoD) for Q2 moments and their derivatives """
    logu = math.log(u)
    # tree level function
    if (dQ2 == 0 and dr == 0 and i==0):
        return     (8*(1+2*logu+10*r**2+5*r**4)-(8*(-1+5*r-10*r**2+10*r**3-5*r**4
            +r**5))/(-1+u)-(10*r**4)/(3*u**4)+(4*(9*r**3+5*r**5))/(3*u**3)-(4*(15*r**2
            +23*r**4))/(3*u**2)+(4*(23*r+45*r**3+6*r**5))/(3*u)-(4*r*(23+45*r**2
            +6*r**4)*u)/3+(4*r**2*(15+23*r**2)*u**2)/3-(4*r**3*(9+5*r**2)*u**3)/3
            +(10*r**4*u**4)/3-(8*(1+5*r+10*r**2+10*r**3+5*r**4+r**5))/(1+u)
            +isPERP*(12*logu*r**4+(5*r**4)/(2*u**4)-(5*(r**3+r**5))/u**3+(14*r**4)/u**2
            +(4*r-9*r**3+3*r**5)/u-r*(4-9*r**2+3*r**4)*u-14*r**4*u**2+5*r**3*(1
            +r**2)*u**3-(5*r**4*u**4)/2))
    if (dQ2 == 0 and dr == 0 and i==1):
        return     ((-8*(-3-6*logu-63*r**2+logu*r**2-105*r**4+15*logu*r**4-21*r**6))/3
            -(8*(-1+7*r-21*r**2+35*r**3-35*r**4+21*r**5-7*r**6+r**7))/(-1+u)
            +(8*r**5)/(3*u**5)+(-37*r**4-25*r**6)/(3*u**4)+(4*(57*r**3+98*r**5+15*r**7))/(9*u**3)
            -(2*(53*r**2+148*r**4+67*r**6))/(3*u**2)+(4*(35*r+98*r**3+97*r**5+6*r**7))/(3*u)
            -(4*r*(35+98*r**2+97*r**4+6*r**6)*u)/3+(2*r**2*(53+148*r**2+67*r**4)*u**2)/3
            -(4*r**3*(57+98*r**2+15*r**4)*u**3)/9+(r**4*(37+25*r**2)*u**4)/3-(8*r**5*u**5)/3
            -(8*(1+7*r+21*r**2+35*r**3+35*r**4+21*r**5+7*r**6+r**7))/(1+u)
            +isPERP*(2*logu*r**2*(4-3*r**2+9*r**4)-(2*r**5)/u**5+(25*(r**4+r**6))/(4*u**4)
            +(-15*r**3-68*r**5-15*r**7)/(3*u**3)+(2*(-r**2+13*r**4+10*r**6))/u**2
            +(4*r-5*r**3-22*r**5+3*r**7)/u-r*(4-5*r**2-22*r**4+3*r**6)*u-2*r**2*(
            -1+13*r**2+10*r**4)*u**2+(r**3*(15+68*r**2+15*r**4)*u**3)/3-(25*r**4*(1
            +r**2)*u**4)/4+2*r**5*u**5))
    if (dQ2 == 0 and dr == 0 and i==2):
        return     ((8*(3+6*logu+108*r**2-2*logu*r**2+378*r**4-128*logu*r**4+252*r**6
            -44*logu*r**6+27*r**8))/3-(8*(-1+9*r-36*r**2+84*r**3-126*r**4+126*r**5
            -84*r**6+36*r**7-9*r**8+r**9))/(-1+u)-(20*r**6)/(9*u**6)+(4*(47*r**5
            +35*r**7))/(15*u**5)-(2*(47*r**4+85*r**6+20*r**8))/(3*u**4)+(20*(22*r**3
            +68*r**5+41*r**7+3*r**9))/(9*u**3)-(4*(44*r**2+178*r**4+205*r**6+44*r**8))/(3*u**2)
            +(4*(47*r+184*r**3+313*r**5+170*r**7+6*r**9))/(3*u)-(4*r*(47+184*r**2
            +313*r**4+170*r**6+6*r**8)*u)/3+(4*r**2*(44+178*r**2+205*r**4+44*r**6)*u**2)/3
            -(20*r**3*(22+68*r**2+41*r**4+3*r**6)*u**3)/9+(2*r**4*(47+85*r**2+20*r**4)*u**4)/3
            -(4*r**5*(47+35*r**2)*u**5)/15+(20*r**6*u**6)/9-(8*(1+9*r+36*r**2+84*r**3
            +126*r**4+126*r**5+84*r**6+36*r**7+9*r**8+r**9))/(1+u)+isPERP*(8*logu*(2
            +3*r**2)*(-r+r**3)**2+(5*r**6)/(3*u**6)-(7*(r**5+r**7))/u**5+(2*(5*r**4
            +16*r**6+5*r**8))/u**4+(-11*r**3-160*r**5-148*r**7-15*r**9)/(3*u**3)
            +(-4*r**2+34*r**4+91*r**6+26*r**8)/u**2+(4*r+11*r**3-85*r**5-41*r**7
            +3*r**9)/u-r*(4+11*r**2-85*r**4-41*r**6+3*r**8)*u-r**2*(-4+34*r**2
            +91*r**4+26*r**6)*u**2+(r**3*(11+160*r**2+148*r**4+15*r**6)*u**3)/3-2*r**4*(5
            +16*r**2+5*r**4)*u**4+7*r**5*(1+r**2)*u**5-(5*r**6*u**6)/3))
    if (dQ2 == 0 and dr == 0 and i==3):
        return     (8*(1+2*logu+55*r**2-logu*r**2+330*r**4-150*logu*r**4+462*r**6-205*logu*r**6
            +165*r**8-30*logu*r**8+11*r**10)-(8*(-1+11*r-55*r**2+165*r**3-330*r**4
            +462*r**5-462*r**6+330*r**7-165*r**8+55*r**9-11*r**10+r**11))/(-1
            +u)+(40*r**7)/(21*u**7)-(2*(19*r**6+15*r**8))/(3*u**6)+(4*(141*r**5+262*r**7
            +75*r**9))/(15*u**5)+(-204*r**4-651*r**6-450*r**8-55*r**10)/(3*u**4)+(4*(66*r**3
            +300*r**5+375*r**7+116*r**9+5*r**11))/(3*u**3)-(2*(135*r**2+738*r**4
            +1419*r**6+873*r**8+109*r**10))/(3*u**2)+(4*(59*r+315*r**3+597*r**5
            +805*r**7+264*r**9+6*r**11))/(3*u)-(4*r*(59+315*r**2+597*r**4+805*r**6
            +264*r**8+6*r**10)*u)/3+(2*r**2*(135+738*r**2+1419*r**4+873*r**6+109*r**8)*u**2)/3
            -(4*r**3*(66+300*r**2+375*r**4+116*r**6+5*r**8)*u**3)/3+(r**4*(204+651*r**2
            +450*r**4+55*r**6)*u**4)/3-(4*r**5*(141+262*r**2+75*r**4)*u**5)/15+(2*r**6*(19
            +15*r**2)*u**6)/3-(40*r**7*u**7)/21-(8*(1+11*r+55*r**2+165*r**3+330*r**4
            +462*r**5+462*r**6+330*r**7+165*r**8+55*r**9+11*r**10+r**11))/(1+u)
            +isPERP*(6*logu*r**2*(4+5*r**2-35*r**4-15*r**6+5*r**8)-(10*r**7)/(7*u**7)
            +(15*(r**6+r**8))/(2*u**6)+(-75*r**5-208*r**7-75*r**9)/(5*u**5)+(51*r**4
            +363*r**6+351*r**8+55*r**10)/(4*u**4)+(-r**3-93*r**5-206*r**7-85*r**9
            -5*r**11)/u**3+(-12*r**2+60*r**4+495*r**6+423*r**8+64*r**10)/(2*u**2)
            +(4*r+39*r**3-150*r**5-340*r**7-66*r**9+3*r**11)/u-r*(4+39*r**2
            -150*r**4-340*r**6-66*r**8+3*r**10)*u-(r**2*(-12+60*r**2+495*r**4
            +423*r**6+64*r**8)*u**2)/2+r**3*(1+93*r**2+206*r**4+85*r**6+5*r**8)*u**3
            -(r**4*(51+363*r**2+351*r**4+55*r**6)*u**4)/4+(r**5*(75+208*r**2+75*r**4)*u**5)/5
            -(15*r**6*(1+r**2)*u**6)/2+(10*r**7*u**7)/7))
    if (dQ2 == 0 and dr == 0 and i==4):
        return     ((8*(3+6*logu+234*r**2-4*logu*r**2+2145*r**4-1128*logu*r**4+5148*r**6
            -3240*logu*r**6+3861*r**8-1880*logu*r**8+858*r**10-156*logu*r**10+39*r**12))/3
            -(8*(-1+13*r-78*r**2+286*r**3-715*r**4+1287*r**5-1716*r**6+1716*r**7
            -1287*r**8+715*r**9-286*r**10+78*r**11-13*r**12+r**13))/(-1+u)-(5*r**8)/(3*u**8)
            +(4*(67*r**7+55*r**9))/(21*u**7)-(4*(99*r**6+187*r**8+60*r**10))/(9*u**6)
            +(4*(345*r**5+1111*r**7+832*r**9+130*r**11))/(15*u**5)-(2*(201*r**4+948*r**6
            +1244*r**8+464*r**10+35*r**12))/(3*u**4)+(4*(333*r**3+2040*r**5+4095*r**7
            +2796*r**9+527*r**11+15*r**13))/(9*u**3)-(4*(97*r**2+693*r**4+1827*r**6
            +2111*r**8+797*r**10+65*r**12))/(3*u**2)+(4*(71*r+503*r**3+750*r**5
            +1591*r**7+1762*r**9+379*r**11+6*r**13))/(3*u)-(4*r*(71+503*r**2+750*r**4
            +1591*r**6+1762*r**8+379*r**10+6*r**12)*u)/3+(4*r**2*(97+693*r**2+1827*r**4
            +2111*r**6+797*r**8+65*r**10)*u**2)/3-(4*r**3*(333+2040*r**2+4095*r**4
            +2796*r**6+527*r**8+15*r**10)*u**3)/9+(2*r**4*(201+948*r**2+1244*r**4
            +464*r**6+35*r**8)*u**4)/3-(4*r**5*(345+1111*r**2+832*r**4+130*r**6)*u**5)/15
            +(4*r**6*(99+187*r**2+60*r**4)*u**6)/9-(4*r**7*(67+55*r**2)*u**7)/21+(5*r**8*u**8)/3
            -(8*(1+13*r+78*r**2+286*r**3+715*r**4+1287*r**5+1716*r**6+1716*r**7
            +1287*r**8+715*r**9+286*r**10+78*r**11+13*r**12+r**13))/(1+u)+isPERP*(4*logu*r**2*(8
            +33*r**2-120*r**4-245*r**6-48*r**8+9*r**10)+(5*r**8)/(4*u**8)-(55*(r**7
            +r**9))/(7*u**7)+(2*(30*r**6+77*r**8+30*r**10))/(3*u**6)+(-126*r**5-691*r**7
            -679*r**9-130*r**11)/(5*u**5)+(27*r**4+384*r**6+770*r**8+368*r**10+35*r**12)/(2*u**4)
            +(9*r**3-393*r**5-1755*r**7-1647*r**9-389*r**11-15*r**13)/(3*u**3)+(2*(
            -4*r**2+3*r**4+246*r**6+469*r**8+202*r**10+19*r**12))/u**2+(4*r+79*r**3
            -141*r**5-1195*r**7-919*r**9-97*r**11+3*r**13)/u-r*(4+79*r**2-141*r**4
            -1195*r**6-919*r**8-97*r**10+3*r**12)*u-2*r**2*(-4+3*r**2+246*r**4
            +469*r**6+202*r**8+19*r**10)*u**2+(r**3*(-9+393*r**2+1755*r**4+1647*r**6
            +389*r**8+15*r**10)*u**3)/3-(r**4*(27+384*r**2+770*r**4+368*r**6+35*r**8)*u**4)/2
            +(r**5*(126+691*r**2+679*r**4+130*r**6)*u**5)/5-(2*r**6*(30+77*r**2
            +30*r**4)*u**6)/3+(55*r**7*(1+r**2)*u**7)/7-(5*r**8*u**8)/4))

    # derivative wrt Q2
    if (dQ2 == 1 and dr == 0 and i==0):
        return     ((4*(23+72*r**2+21*r**4))/3-(4*(-1+5*r-10*r**2+10*r**3-5*r**4+r**5))/(r*(
            -1+u)**3)-(6*(-1+5*r-10*r**2+10*r**3-5*r**4+r**5))/(r*(-1+u)**2)
            +(2*(-12+31*r-60*r**2+114*r**3-112*r**4+39*r**5))/(3*r*(-1+u))
            +(40*r**3)/(3*u**3)-(4*(9*r**2+5*r**4))/u**2+(8*(15*r+28*r**3))/(3*u)
            -(8*r*(15+28*r**2)*u)/3+4*r**2*(9+5*r**2)*u**2-(40*r**3*u**3)/3+(4*(1
            +5*r+10*r**2+10*r**3+5*r**4+r**5))/(r*(1+u)**3)-(6*(1+5*r+10*r**2
            +10*r**3+5*r**4+r**5))/(r*(1+u)**2)-(2*(12+31*r+60*r**2+114*r**3
            +112*r**4+39*r**5))/(3*r*(1+u))+isPERP*(-4*(-1+6*r**2+3*r**4)-(4*(
            -1+6*r**2-8*r**3+3*r**4))/(-1+u)-(10*r**3)/u**3+(15*(r**2+r**4))/u**2
            -(38*r**3)/u+38*r**3*u-15*r**2*(1+r**2)*u**2+10*r**3*u**3+(4*(-1+6*r**2
            +8*r**3+3*r**4))/(1+u)))
    if (dQ2 == 1 and dr == 0 and i==1):
        return     ((4*(35+155*r**2+205*r**4+21*r**6))/3-(4*(-1+7*r-21*r**2+35*r**3
            -35*r**4+21*r**5-7*r**6+r**7))/(r*(-1+u)**3)-(6*(-1+7*r-21*r**2
            +35*r**3-35*r**4+21*r**5-7*r**6+r**7))/(r*(-1+u)**2)+(2*(-12+49*r
            -104*r**2+205*r**3-340*r**4+347*r**5-184*r**6+39*r**7))/(3*r*(-1+u))
            -(40*r**4)/(3*u**4)+(4*(37*r**3+25*r**5))/(3*u**3)-(4*(19*r**2+36*r**4
            +5*r**6))/u**2+(4*(53*r+185*r**3+92*r**5))/(3*u)-(4*r*(53+185*r**2
            +92*r**4)*u)/3+4*r**2*(19+36*r**2+5*r**4)*u**2-(4*r**3*(37+25*r**2)*u**3)/3
            +(40*r**4*u**4)/3+(4*(1+7*r+21*r**2+35*r**3+35*r**4+21*r**5+7*r**6
            +r**7))/(r*(1+u)**3)-(6*(1+7*r+21*r**2+35*r**3+35*r**4+21*r**5
            +7*r**6+r**7))/(r*(1+u)**2)-(2*(12+49*r+104*r**2+205*r**3+340*r**4
            +347*r**5+184*r**6+39*r**7))/(3*r*(1+u))+isPERP*(-4*(-1+5*r**2+25*r**4
            +3*r**6)-(4*(-1+2*r+5*r**2-20*r**3+25*r**4-14*r**5+3*r**6))/(-1
            +u)+(10*r**4)/u**4-(25*(r**3+r**5))/u**3+(3*(5*r**2+26*r**4+5*r**6))/u**2
            +(4*r-77*r**3-65*r**5)/u+r*(-4+77*r**2+65*r**4)*u-3*r**2*(5+26*r**2
            +5*r**4)*u**2+25*r**3*(1+r**2)*u**3-10*r**4*u**4+(4*(-1-2*r+5*r**2
            +20*r**3+25*r**4+14*r**5+3*r**6))/(1+u)))
    if (dQ2 == 1 and dr == 0 and i==2):
        return     ((4*(47+294*r**2+700*r**4+410*r**6+21*r**8))/3-(4*(-1+9*r-36*r**2
            +84*r**3-126*r**4+126*r**5-84*r**6+36*r**7-9*r**8+r**9))/(r*(-1+u)**3)
            -(6*(-1+9*r-36*r**2+84*r**3-126*r**4+126*r**5-84*r**6+36*r**7-9*r**8
            +r**9))/(r*(-1+u)**2)+(2*(-12+67*r-172*r**2+336*r**3-644*r**4+1022*r**5
            -1092*r**6+712*r**7-256*r**8+39*r**9))/(3*r*(-1+u))+(40*r**5)/(3*u**5)
            -(4*(47*r**4+35*r**6))/(3*u**4)+(8*(47*r**3+90*r**5+20*r**7))/(3*u**3)
            -(4*(110*r**2+387*r**4+240*r**6+15*r**8))/(3*u**2)+(8*(44*r+225*r**3
            +295*r**5+64*r**7))/(3*u)-(8*r*(44+225*r**2+295*r**4+64*r**6)*u)/3
            +(4*r**2*(110+387*r**2+240*r**4+15*r**6)*u**2)/3-(8*r**3*(47+90*r**2
            +20*r**4)*u**3)/3+(4*r**4*(47+35*r**2)*u**4)/3-(40*r**5*u**5)/3+(4*(1
            +9*r+36*r**2+84*r**3+126*r**4+126*r**5+84*r**6+36*r**7+9*r**8+r**9))/(r*(1
            +u)**3)-(6*(1+9*r+36*r**2+84*r**3+126*r**4+126*r**5+84*r**6+36*r**7
            +9*r**8+r**9))/(r*(1+u)**2)-(2*(12+67*r+172*r**2+336*r**3+644*r**4
            +1022*r**5+1092*r**6+712*r**7+256*r**8+39*r**9))/(3*r*(1+u))+isPERP*(
            -4*(-1+70*r**4+56*r**6+3*r**8)-(4*(-1+4*r-28*r**3+70*r**4-84*r**5
            +56*r**6-20*r**7+3*r**8))/(-1+u)-(10*r**5)/u**5+(35*(r**4+r**6))/u**4
            -(2*(20*r**3+69*r**5+20*r**7))/u**3+(11*r**2+195*r**4+183*r**6+15*r**8)/u**2
            -(4*(-2*r+27*r**3+80*r**5+23*r**7))/u+4*r*(-2+27*r**2+80*r**4+23*r**6)*u
            -r**2*(11+195*r**2+183*r**4+15*r**6)*u**2+2*r**3*(20+69*r**2+20*r**4)*u**3
            -35*r**4*(1+r**2)*u**4+10*r**5*u**5+(4*(-1-4*r+28*r**3+70*r**4+84*r**5
            +56*r**6+20*r**7+3*r**8))/(1+u)))
    if (dQ2 == 1 and dr == 0 and i==3):
        return     ((4*(59+513*r**2+1638*r**4+2202*r**6+687*r**8+21*r**10))/3-(4*(-1
            +11*r-55*r**2+165*r**3-330*r**4+462*r**5-462*r**6+330*r**7-165*r**8
            +55*r**9-11*r**10+r**11))/(r*(-1+u)**3)-(6*(-1+11*r-55*r**2+165*r**3
            -330*r**4+462*r**5-462*r**6+330*r**7-165*r**8+55*r**9-11*r**10+r**11))/(r*(
            -1+u)**2)+(2*(-12+85*r-264*r**2+531*r**3-984*r**4+1890*r**5-3024*r**6
            +3414*r**7-2556*r**8+1209*r**9-328*r**10+39*r**11))/(3*r*(-1+u))-(40*r**6)/(3*u**6)
            +(4*(19*r**5+15*r**7))/u**5-(4*(141*r**4+272*r**6+75*r**8))/(3*u**4)+(4*(204*r**3
            +708*r**5+495*r**7+55*r**9))/(3*u**3)-(4*(198*r**2+1041*r**4+1397*r**6
            +423*r**8+15*r**10))/(3*u**2)+(4*(135*r+942*r**3+2127*r**5+1368*r**7
            +164*r**9))/(3*u)-(4*r*(135+942*r**2+2127*r**4+1368*r**6+164*r**8)*u)/3
            +(4*r**2*(198+1041*r**2+1397*r**4+423*r**6+15*r**8)*u**2)/3-(4*r**3*(204
            +708*r**2+495*r**4+55*r**6)*u**3)/3+(4*r**4*(141+272*r**2+75*r**4)*u**4)/3
            -4*r**5*(19+15*r**2)*u**5+(40*r**6*u**6)/3+(4*(1+11*r+55*r**2+165*r**3
            +330*r**4+462*r**5+462*r**6+330*r**7+165*r**8+55*r**9+11*r**10+r**11))/(r*(1
            +u)**3)-(6*(1+11*r+55*r**2+165*r**3+330*r**4+462*r**5+462*r**6
            +330*r**7+165*r**8+55*r**9+11*r**10+r**11))/(r*(1+u)**2)-(2*(12+85*r
            +264*r**2+531*r**3+984*r**4+1890*r**5+3024*r**6+3414*r**7+2556*r**8
            +1209*r**9+328*r**10+39*r**11))/(3*r*(1+u))+isPERP*(-4*(-1-9*r**2
            +126*r**4+294*r**6+99*r**8+3*r**10)-(4*(-1+6*r-9*r**2-24*r**3+126*r**4
            -252*r**5+294*r**6-216*r**7+99*r**8-26*r**9+3*r**10))/(-1+u)+(10*r**6)/u**6
            -(45*(r**5+r**7))/u**5+(75*r**4+218*r**6+75*r**8)/u**4+(-51*r**3-408*r**5
            -396*r**7-55*r**9)/u**3+(3*r**2+354*r**4+836*r**6+330*r**8+15*r**10)/u**2
            +(12*r-111*r**3-903*r**5-819*r**7-119*r**9)/u+r*(-12+111*r**2+903*r**4
            +819*r**6+119*r**8)*u-r**2*(3+354*r**2+836*r**4+330*r**6+15*r**8)*u**2
            +r**3*(51+408*r**2+396*r**4+55*r**6)*u**3-r**4*(75+218*r**2+75*r**4)*u**4
            +45*r**5*(1+r**2)*u**5-10*r**6*u**6+(4*(-1-6*r-9*r**2+24*r**3+126*r**4
            +252*r**5+294*r**6+216*r**7+99*r**8+26*r**9+3*r**10))/(1+u)))
    if (dQ2 == 1 and dr == 0 and i==4):
        return     ((4*(71+836*r**2+3135*r**4+6864*r**6+5445*r**8+1036*r**10+21*r**12))/3
            -(4*(-1+13*r-78*r**2+286*r**3-715*r**4+1287*r**5-1716*r**6+1716*r**7
            -1287*r**8+715*r**9-286*r**10+78*r**11-13*r**12+r**13))/(r*(-1+u)**3)
            -(6*(-1+13*r-78*r**2+286*r**3-715*r**4+1287*r**5-1716*r**6+1716*r**7
            -1287*r**8+715*r**9-286*r**10+78*r**11-13*r**12+r**13))/(r*(-1+u)**2)
            +(2*(-12+103*r-380*r**2+814*r**3-1320*r**4+2409*r**5-5016*r**6+8580*r**7
            -10428*r**8+8745*r**9-4972*r**10+1838*r**11-400*r**12+39*r**13))/(3*r*(
            -1+u))+(40*r**7)/(3*u**7)-(4*(67*r**6+55*r**8))/(3*u**6)+(8*(33*r**5
            +64*r**7+20*r**9))/u**5-(4*(345*r**4+1178*r**6+887*r**8+130*r**10))/(3*u**4)
            +(8*(201*r**3+1047*r**5+1436*r**7+524*r**9+35*r**11))/(3*u**3)-(4*(333*r**2
            +2385*r**4+5273*r**6+3683*r**8+657*r**10+15*r**12))/(3*u**2)+(8*(97*r
            +894*r**3+2874*r**5+3547*r**7+1321*r**9+100*r**11))/(3*u)-(8*r*(97
            +894*r**2+2874*r**4+3547*r**6+1321*r**8+100*r**10)*u)/3+(4*r**2*(333
            +2385*r**2+5273*r**4+3683*r**6+657*r**8+15*r**10)*u**2)/3-(8*r**3*(201
            +1047*r**2+1436*r**4+524*r**6+35*r**8)*u**3)/3+(4*r**4*(345+1178*r**2
            +887*r**4+130*r**6)*u**4)/3-8*r**5*(33+64*r**2+20*r**4)*u**5+(4*r**6*(67
            +55*r**2)*u**6)/3-(40*r**7*u**7)/3+(4*(1+13*r+78*r**2+286*r**3+715*r**4
            +1287*r**5+1716*r**6+1716*r**7+1287*r**8+715*r**9+286*r**10+78*r**11
            +13*r**12+r**13))/(r*(1+u)**3)-(6*(1+13*r+78*r**2+286*r**3+715*r**4
            +1287*r**5+1716*r**6+1716*r**7+1287*r**8+715*r**9+286*r**10+78*r**11
            +13*r**12+r**13))/(r*(1+u)**2)-(2*(12+103*r+380*r**2+814*r**3+1320*r**4
            +2409*r**5+5016*r**6+8580*r**7+10428*r**8+8745*r**9+4972*r**10+1838*r**11
            +400*r**12+39*r**13))/(3*r*(1+u))+isPERP*(-4*(-1-22*r**2+165*r**4
            +924*r**6+825*r**8+154*r**10+3*r**12)-(4*(-1+8*r-22*r**2+165*r**4
            -528*r**5+924*r**6-1056*r**7+825*r**8-440*r**9+154*r**10-32*r**11
            +3*r**12))/(-1+u)-(10*r**7)/u**7+(55*(r**6+r**8))/u**6-(6*(20*r**5+53*r**7
            +20*r**9))/u**5+(2*(63*r**4+373*r**6+367*r**8+65*r**10))/u**4-(2*(27*r**3
            +444*r**5+929*r**7+428*r**9+35*r**11))/u**3+(-9*r**2+519*r**4+2501*r**6
            +2381*r**8+519*r**10+15*r**12)/u**2-(2*(-8*r+33*r**3+936*r**5+1867*r**7
            +832*r**9+73*r**11))/u+2*r*(-8+33*r**2+936*r**4+1867*r**6+832*r**8
            +73*r**10)*u-r**2*(-9+519*r**2+2501*r**4+2381*r**6+519*r**8+15*r**10)*u**2
            +2*r**3*(27+444*r**2+929*r**4+428*r**6+35*r**8)*u**3-2*r**4*(63+373*r**2
            +367*r**4+65*r**6)*u**4+6*r**5*(20+53*r**2+20*r**4)*u**5-55*r**6*(1+r**2)*u**6
            +10*r**7*u**7+(4*(-1-8*r-22*r**2+165*r**4+528*r**5+924*r**6+1056*r**7
            +825*r**8+440*r**9+154*r**10+32*r**11+3*r**12))/(1+u)))

    # derivative wrt r
    if (dQ2 == 0 and dr == 1 and i==0):
        return     ((-8*(6-7*r**2+68*r**4+21*r**6))/(3*r)+(8*(1-6*r+15*r**2-20*r**3
            +15*r**4-6*r**5+r**6))/(r*(-1+u)**3)+(12*(1-6*r+15*r**2-20*r**3
            +15*r**4-6*r**5+r**6))/(r*(-1+u)**2)-(4*(9+2*r-59*r**2+36*r**3
            +91*r**4-118*r**5+39*r**6))/(3*r*(-1+u))-(40*r**4)/(3*u**3)+(8*(14*r**3
            +15*r**5))/(3*u**2)-(8*(12*r**2+59*r**4))/(3*u)+(8*r**2*(12+59*r**2)*u)/3
            -(8*r**3*(14+15*r**2)*u**2)/3+(40*r**4*u**3)/3-(8*(1+6*r+15*r**2+20*r**3
            +15*r**4+6*r**5+r**6))/(r*(1+u)**3)+(12*(1+6*r+15*r**2+20*r**3
            +15*r**4+6*r**5+r**6))/(r*(1+u)**2)+(4*(9-2*r-59*r**2-36*r**3+91*r**4
            +118*r**5+39*r**6))/(3*r*(1+u))+isPERP*(8*r*(-1+14*r**2+6*logu*r**2
            +3*r**4)+(8*(1-r-6*r**2+14*r**3-11*r**4+3*r**5))/(-1+u)+(10*r**4)/u**3
            -(2*(11*r**3+15*r**5))/u**2+(2*(6*r**2+59*r**4))/u-2*r**2*(6+59*r**2)*u
            +2*r**3*(11+15*r**2)*u**2-10*r**4*u**3-(8*(-1-r+6*r**2+14*r**3+11*r**4
            +3*r**5))/(1+u)))
    if (dQ2 == 0 and dr == 1 and i==1):
        return     ((-8*(6-39*r**2+2*logu*r**2-95*r**4+60*logu*r**4+171*r**6+21*r**8))/(3*r)
            +(8*(1-8*r+28*r**2-56*r**3+70*r**4-56*r**5+28*r**6-8*r**7+r**8))/(r*(
            -1+u)**3)+(12*(1-8*r+28*r**2-56*r**3+70*r**4-56*r**5+28*r**6-8*r**7
            +r**8))/(r*(-1+u)**2)-(4*(9+2*r-162*r**2+426*r**3-400*r**4+6*r**5
            +258*r**6-178*r**7+39*r**8))/(3*r*(-1+u))+(10*r**5)/u**4-(8*(43*r**4
            +45*r**6))/(9*u**3)+(8*(20*r**3+66*r**5+15*r**7))/(3*u**2)-(8*(12*r**2
            +99*r**4+89*r**6))/(3*u)+(8*r**2*(12+99*r**2+89*r**4)*u)/3-(8*r**3*(20
            +66*r**2+15*r**4)*u**2)/3+(8*r**4*(43+45*r**2)*u**3)/9-10*r**5*u**4-(8*(1
            +8*r+28*r**2+56*r**3+70*r**4+56*r**5+28*r**6+8*r**7+r**8))/(r*(1
            +u)**3)+(12*(1+8*r+28*r**2+56*r**3+70*r**4+56*r**5+28*r**6+8*r**7
            +r**8))/(r*(1+u)**2)+(4*(9-2*r-162*r**2-426*r**3-400*r**4-6*r**5
            +258*r**6+178*r**7+39*r**8))/(3*r*(1+u))+isPERP*(4*r*(-6+4*logu
            +50*r**2-6*logu*r**2+78*r**4+27*logu*r**4+6*r**6)+(8*(1-3*r-3*r**2
            +25*r**3-45*r**4+39*r**5-17*r**6+3*r**7))/(-1+u)-(15*r**5)/(2*u**4)
            +(2*(37*r**4+45*r**6))/(3*u**3)-(2*(14*r**3+63*r**5+15*r**7))/u**2+(2*(6*r**2
            +111*r**4+89*r**6))/u-2*r**2*(6+111*r**2+89*r**4)*u+2*r**3*(14+63*r**2
            +15*r**4)*u**2-(2*r**4*(37+45*r**2)*u**3)/3+(15*r**5*u**4)/2-(8*(-1-3*r
            +3*r**2+25*r**3+45*r**4+39*r**5+17*r**6+3*r**7))/(1+u)))
    if (dQ2 == 0 and dr == 1 and i==2):
        return     ((-8*(6-83*r**2+4*logu*r**2-896*r**4+512*logu*r**4-266*r**6+264*logu*r**6
            +322*r**8+21*r**10))/(3*r)+(8*(1-10*r+45*r**2-120*r**3+210*r**4
            -252*r**5+210*r**6-120*r**7+45*r**8-10*r**9+r**10))/(r*(-1+u)**3)
            +(12*(1-10*r+45*r**2-120*r**3+210*r**4-252*r**5+210*r**6-120*r**7
            +45*r**8-10*r**9+r**10))/(r*(-1+u)**2)-(4*(9+2*r-301*r**2+1256*r**3
            -2422*r**4+2492*r**5-1162*r**6-184*r**7+509*r**8-238*r**9+39*r**10))/(3*r*(
            -1+u))-(8*r**6)/u**5+(4*(29*r**5+30*r**7))/(3*u**4)-(8*(83*r**4+235*r**6
            +75*r**8))/(9*u**3)+(8*(26*r**3+157*r**5+148*r**7+15*r**9))/(3*u**2)-(8*(14*r**2
            +211*r**4+320*r**6+119*r**8))/(3*u)+(8*r**2*(14+211*r**2+320*r**4+119*r**6)*u)/3
            -(8*r**3*(26+157*r**2+148*r**4+15*r**6)*u**2)/3+(8*r**4*(83+235*r**2
            +75*r**4)*u**3)/9-(4*r**5*(29+30*r**2)*u**4)/3+8*r**6*u**5-(8*(1+10*r
            +45*r**2+120*r**3+210*r**4+252*r**5+210*r**6+120*r**7+45*r**8+10*r**9
            +r**10))/(r*(1+u)**3)+(12*(1+10*r+45*r**2+120*r**3+210*r**4+252*r**5
            +210*r**6+120*r**7+45*r**8+10*r**9+r**10))/(r*(1+u)**2)+(4*(9-2*r
            -301*r**2-1256*r**3-2422*r**4-2492*r**5-1162*r**6+184*r**7+509*r**8
            +238*r**9+39*r**10))/(3*r*(1+u))+isPERP*(8*r*(-5+4*logu+28*r**2
            -4*logu*r**2+154*r**4-24*logu*r**4+76*r**6+24*logu*r**6+3*r**8)+(8*(1
            -5*r+4*r**2+28*r**3-98*r**4+154*r**5-140*r**6+76*r**7-23*r**8+3*r**9))/(
            -1+u)+(6*r**6)/u**5-(2*(13*r**5+15*r**7))/u**4+(2*(65*r**4+223*r**6
            +75*r**8))/(3*u**3)-(2*(17*r**3+151*r**5+145*r**7+15*r**9))/u**2+(2*(14*r**2
            +133*r**4+380*r**6+119*r**8))/u-2*r**2*(14+133*r**2+380*r**4+119*r**6)*u
            +2*r**3*(17+151*r**2+145*r**4+15*r**6)*u**2-(2*r**4*(65+223*r**2+75*r**4)*u**3)/3
            +2*r**5*(13+15*r**2)*u**4-6*r**6*u**5-(8*(-1-5*r-4*r**2+28*r**3+98*r**4
            +154*r**5+140*r**6+76*r**7+23*r**8+3*r**9))/(1+u)))
    if (dQ2 == 0 and dr == 1 and i==3):
        return     ((-8*(6-139*r**2+6*logu*r**2-2955*r**4+1800*logu*r**4-5166*r**6+3690*logu*r**6
            -480*r**8+720*logu*r**8+521*r**10+21*r**12))/(3*r)+(8*(1-12*r+66*r**2
            -220*r**3+495*r**4-792*r**5+924*r**6-792*r**7+495*r**8-220*r**9+66*r**10
            -12*r**11+r**12))/(r*(-1+u)**3)+(12*(1-12*r+66*r**2-220*r**3+495*r**4
            -792*r**5+924*r**6-792*r**7+495*r**8-220*r**9+66*r**10-12*r**11+r**12))/(r*(
            -1+u)**2)-(4*(9+2*r-476*r**2+2670*r**3-7395*r**4+12372*r**5-13104*r**6
            +8412*r**7-2445*r**8-630*r**9+844*r**10-298*r**11+39*r**12))/(3*r*(
            -1+u))+(20*r**7)/(3*u**6)-(8*(73*r**6+75*r**8))/(15*u**5)+(2*(141*r**5
            +368*r**7+135*r**9))/(3*u**4)-(8*(45*r**4+230*r**6+222*r**8+35*r**10))/(3*u**3)
            +(4*(66*r**3+660*r**5+1165*r**7+520*r**9+30*r**11))/(3*u**2)-(8*(18*r**2
            +789*r**4+1109*r**6+735*r**8+149*r**10))/(3*u)+(8*r**2*(18+789*r**2
            +1109*r**4+735*r**6+149*r**8)*u)/3-(4*r**3*(66+660*r**2+1165*r**4+520*r**6
            +30*r**8)*u**2)/3+(8*r**4*(45+230*r**2+222*r**4+35*r**6)*u**3)/3-(2*r**5*(141
            +368*r**2+135*r**4)*u**4)/3+(8*r**6*(73+75*r**2)*u**5)/15-(20*r**7*u**6)/3
            -(8*(1+12*r+66*r**2+220*r**3+495*r**4+792*r**5+924*r**6+792*r**7
            +495*r**8+220*r**9+66*r**10+12*r**11+r**12))/(r*(1+u)**3)+(12*(1
            +12*r+66*r**2+220*r**3+495*r**4+792*r**5+924*r**6+792*r**7+495*r**8
            +220*r**9+66*r**10+12*r**11+r**12))/(r*(1+u)**2)+(4*(9-2*r-476*r**2
            -2670*r**3-7395*r**4-12372*r**5-13104*r**6-8412*r**7-2445*r**8+630*r**9
            +844*r**10+298*r**11+39*r**12))/(3*r*(1+u))+isPERP*(4*r*(-14+12*logu
            +30*r**2+30*logu*r**2+756*r**4-315*logu*r**4+1020*r**6-180*logu*r**6
            +250*r**8+75*logu*r**8+6*r**10)+(8*(1-7*r+15*r**2+15*r**3-150*r**4
            +378*r**5-546*r**6+510*r**7-315*r**8+125*r**9-29*r**10+3*r**11))/(
            -1+u)-(5*r**7)/u**6+(2*(67*r**6+75*r**8))/(5*u**5)+(-117*r**5-350*r**7
            -135*r**9)/(2*u**4)+(2*(33*r**4+214*r**6+216*r**8+35*r**10))/u**3+(-48*r**3
            -534*r**5-1195*r**7-514*r**9-30*r**11)/u**2+(2*(30*r**2+165*r**4+719*r**6
            +885*r**8+149*r**10))/u-2*r**2*(30+165*r**2+719*r**4+885*r**6+149*r**8)*u
            +r**3*(48+534*r**2+1195*r**4+514*r**6+30*r**8)*u**2-2*r**4*(33+214*r**2
            +216*r**4+35*r**6)*u**3+(r**5*(117+350*r**2+135*r**4)*u**4)/2-(2*r**6*(67
            +75*r**2)*u**5)/5+5*r**7*u**6-(8*(-1-7*r-15*r**2+15*r**3+150*r**4
            +378*r**5+546*r**6+510*r**7+315*r**8+125*r**9+29*r**10+3*r**11))/(1
            +u)))
    if (dQ2 == 0 and dr == 1 and i==4):
        return     ((-8*(6-207*r**2+8*logu*r**2-7084*r**4+4512*logu*r**4-25245*r**6+19440*logu*r**6
            -18810*r**8+15040*logu*r**8-649*r**10+1560*logu*r**10+768*r**12+21*r**14))/(3*r)
            +(8*(1-14*r+91*r**2-364*r**3+1001*r**4-2002*r**5+3003*r**6-3432*r**7
            +3003*r**8-2002*r**9+1001*r**10-364*r**11+91*r**12-14*r**13+r**14))/(r*(
            -1+u)**3)+(12*(1-14*r+91*r**2-364*r**3+1001*r**4-2002*r**5+3003*r**6
            -3432*r**7+3003*r**8-2002*r**9+1001*r**10-364*r**11+91*r**12-14*r**13
            +r**14))/(r*(-1+u)**2)-(4*(9+2*r-687*r**2+4812*r**3-17171*r**4+38742*r**5
            -59499*r**6+63624*r**7-46629*r**8+21582*r**9-4301*r**10-1428*r**11
            +1263*r**12-358*r**13+39*r**14))/(3*r*(-1+u))-(40*r**8)/(7*u**7)+(8*(44*r**7
            +45*r**9))/(9*u**6)-(8*(214*r**6+531*r**8+210*r**10))/(15*u**5)+(8*(69*r**5
            +318*r**7+311*r**9+60*r**11))/(3*u**4)-(8*(201*r**4+1626*r**6+2889*r**8
            +1426*r**10+135*r**12))/(9*u**3)+(8*(42*r**3+825*r**5+1812*r**7+1543*r**9
            +402*r**11+15*r**13))/(3*u**2)-(8*(24*r**2+2673*r**4+6248*r**6+3729*r**8
            +1404*r**10+179*r**12))/(3*u)+(8*r**2*(24+2673*r**2+6248*r**4+3729*r**6
            +1404*r**8+179*r**10)*u)/3-(8*r**3*(42+825*r**2+1812*r**4+1543*r**6
            +402*r**8+15*r**10)*u**2)/3+(8*r**4*(201+1626*r**2+2889*r**4+1426*r**6
            +135*r**8)*u**3)/9-(8*r**5*(69+318*r**2+311*r**4+60*r**6)*u**4)/3+(8*r**6*(214
            +531*r**2+210*r**4)*u**5)/15-(8*r**7*(44+45*r**2)*u**6)/9+(40*r**8*u**7)/7
            -(8*(1+14*r+91*r**2+364*r**3+1001*r**4+2002*r**5+3003*r**6+3432*r**7
            +3003*r**8+2002*r**9+1001*r**10+364*r**11+91*r**12+14*r**13+r**14))/(r*(1
            +u)**3)+(12*(1+14*r+91*r**2+364*r**3+1001*r**4+2002*r**5+3003*r**6
            +3432*r**7+3003*r**8+2002*r**9+1001*r**10+364*r**11+91*r**12+14*r**13
            +r**14))/(r*(1+u)**2)+(4*(9-2*r-687*r**2-4812*r**3-17171*r**4-38742*r**5
            -59499*r**6-63624*r**7-46629*r**8-21582*r**9-4301*r**10+1428*r**11
            +1263*r**12+358*r**13+39*r**14))/(3*r*(1+u))+isPERP*(8*r*(-9+8*logu
            -22*r**2+66*logu*r**2+693*r**4-360*logu*r**4+1980*r**6-980*logu*r**6
            +1265*r**8-240*logu*r**8+186*r**10+54*logu*r**10+3*r**12)+(8*(1-9*r
            +30*r**2-22*r**3-165*r**4+693*r**5-1452*r**6+1980*r**7-1881*r**8+1265*r**9
            -594*r**10+186*r**11-35*r**12+3*r**13))/(-1+u)+(30*r**8)/(7*u**7)-(2*(41*r**7
            +45*r**9))/(3*u**6)+(2*(184*r**6+507*r**8+210*r**10))/(5*u**5)-(4*(27*r**5
            +147*r**7+151*r**9+30*r**11))/u**4+(2*(147*r**4+1392*r**6+2835*r**8
            +1402*r**10+135*r**12))/(3*u**3)-(2*(39*r**3+423*r**5+1545*r**7+1621*r**9
            +399*r**11+15*r**13))/u**2+(2*(54*r**2+303*r**4+788*r**6+2439*r**8+1698*r**10
            +179*r**12))/u-2*r**2*(54+303*r**2+788*r**4+2439*r**6+1698*r**8+179*r**10)*u
            +2*r**3*(39+423*r**2+1545*r**4+1621*r**6+399*r**8+15*r**10)*u**2-(2*r**4*(147
            +1392*r**2+2835*r**4+1402*r**6+135*r**8)*u**3)/3+4*r**5*(27+147*r**2
            +151*r**4+30*r**6)*u**4-(2*r**6*(184+507*r**2+210*r**4)*u**5)/5+(2*r**7*(41
            +45*r**2)*u**6)/3-(30*r**8*u**7)/7-(8*(-1-9*r-30*r**2-22*r**3+165*r**4
            +693*r**5+1452*r**6+1980*r**7+1881*r**8+1265*r**9+594*r**10+186*r**11
            +35*r**12+3*r**13))/(1+u)))
    return 0. 

def moment_1_KIN_MS(q2cut, par, hqe, wc, **kwargs):
    Q2=q2cut/par.mbkin**2
    r=par.mcMS/par.mbkin
    u=(1.+r**2-Q2-math.sqrt(r**4+(1.-Q2)**2-2*r**2*(1.+Q2)))/2/r
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    FLAGcf=0.5*(4./3. + 3.*(1.+ math.log(par.scale_muG/par.mbkin)))
    FLAGcD=-8./3.*4./3.*math.log(par.scale_rhoD/par.mbkin)+3.*(1./2.-2./3.*math.log(par.scale_rhoD/par.mbkin))
    FLAGcs=0.5*(4./3. + 3.*(1.+math.log(par.scale_muG/par.mbkin)))

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    res = 0
    res +=(X(1,u,r,0,0)/X(0,u,r,0,0)-(rhoD*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        )
    res += ((rhoD*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)-(muG*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
        +(flagPERP*rhoLS*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
        )
    res += ((muG*XG(1,u,r,0,0))/X(0,u,r,0,0)-(flagPERP*rhoLS*XG(1,u,r,0,0))/X(0,u,r,0,0))

    if( flagDEBUG == 1):
        print("Q2moment n. 1 LO = ",res*par.mbkin**2)

    resNLO = 0
    resNLO +=((2*deltambkin1*X(1,u,r,0,0))/X(0,u,r,0,0)+(deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
        -(deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2+(2*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
        -(deltambkin1*r*X(1,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*r*X(1,u,r,0,1))/X(0,u,r,0,0)
        -(2*deltambkin1*Q2*X(1,u,r,1,0))/X(0,u,r,0,0)-(X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
        +nlo.X1Q2(1,'SM',Q2,r)/X(0,u,r,0,0)+(Rhodpert1*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        -(Rhodpert1*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0))

    if( flagDEBUG == 1):
        print("Q2moment n. 1 NLO = api*",resNLO*par.mbkin**2)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=((-2*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r))/X(0,u,r,0,0)-(deltambkin1*r*nlo.X1Q2_Derivativer(1,Q2,r))/X(0,u,r,0,0)
            +(deltamcMS1*r*nlo.X1Q2_Derivativer(1,Q2,r))/X(0,u,r,0,0)+(2*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2+(deltambkin1**2*X(1,u,r,0,0))/X(0,u,r,0,0)
            +(2*deltambkin2*X(1,u,r,0,0))/X(0,u,r,0,0)+(deltambkin1**2*r*X(0,u,r,0,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin2*r*X(0,u,r,0,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2-(deltambkin1*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS2*r*X(0,u,r,0,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2+(deltambkin1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(deltambkin1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(deltambkin1*deltamcMS1*r**2*X(0,u,r,0,2)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(deltambkin1**2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1**2*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1**2*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1**2*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1**2*r*X(1,u,r,0,1))/X(0,u,r,0,0)-(deltambkin2*r*X(1,u,r,0,1))/X(0,u,r,0,0)
            +(deltambkin1*deltamcMS1*r*X(1,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS2*r*X(1,u,r,0,1))/X(0,u,r,0,0)
            -(deltambkin1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltambkin1**2*r**2*X(1,u,r,0,2))/(2*X(0,u,r,0,0))-(deltambkin1*deltamcMS1*r**2*X(1,u,r,0,2))/X(0,u,r,0,0)
            +(deltamcMS1**2*r**2*X(1,u,r,0,2))/(2*X(0,u,r,0,0))-(deltambkin1**2*Q2*X(1,u,r,1,0))/X(0,u,r,0,0)
            -(2*deltambkin2*Q2*X(1,u,r,1,0))/X(0,u,r,0,0)-(2*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1**2*Q2*r*X(1,u,r,1,1))/X(0,u,r,0,0)-(2*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,1,1))/X(0,u,r,0,0)
            +(2*deltambkin1**2*Q2**2*X(1,u,r,2,0))/X(0,u,r,0,0)-(2*deltambkin1*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(25*math.log(mus**2)*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(12*X(0,u,r,0,0)**2)
            -(2*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(deltambkin1*r*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*X(1,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**3+(2*deltambkin1*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)
            +(25*math.log(mus**2)*nlo.X1Q2(1,'SM',Q2,r))/(12*X(0,u,r,0,0))+(deltambkin1*r*X(0,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*X(0,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*X(0,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2-(X(1,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**2
            +nnlo.X2Q2(1,Q2,r)/X(0,u,r,0,0)-(deltambkin1*Rhodpert1*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(Rhodpert2*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0)**2)
            +(Rhodpert1*math.log(r)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(2*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltambkin1*Rhodpert1*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)-(Rhodpert2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)
            +(Rhodpert1*math.log(mus**2)*XD(1,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0))
            -(Rhodpert1*math.log(r)*XD(1,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0))-(deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltambkin1*r*Rhodpert1*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(deltamcMS1*r*Rhodpert1*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            +(2*deltambkin1*Q2*Rhodpert1*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 1 NNLO = api^2*",resNNLO*par.mbkin**2)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=(-((muG*X(1,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2)+(flagPERP*rhoLS*X(1,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2
            )
        resNLO += ((muG*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)-(flagPERP*rhoLS*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)
            -(rhoD*X(1,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2)
        resNLO += ((rhoD*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)
            +(deltambkin1*rhoD*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(FLAGcD*rhoD*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*rhoD*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(rhoD*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltambkin1*r*rhoD*X(1,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(1,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltambkin1*rhoD*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(FLAGcD*rhoD*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)
            +(deltambkin1*r*rhoD*X(0,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(0,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(rhoD*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltambkin1*r*rhoD*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)+(deltamcMS1*r*rhoD*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(2*deltambkin1*Q2*rhoD*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)-(FLAGcf*muG*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*muG*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*flagPERP*rhoLS*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(muG*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin1*muG*r*X(1,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(1,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(1,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            +(FLAGcf*muG*XG(1,u,r,0,0))/X(0,u,r,0,0)+(deltambkin1*flagPERP*rhoLS*XG(1,u,r,0,0))/X(0,u,r,0,0)
            -(FLAGcs*flagPERP*rhoLS*XG(1,u,r,0,0))/X(0,u,r,0,0)+(deltambkin1*muG*r*X(0,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(0,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(0,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(muG*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*muG*r*XG(1,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*muG*r*XG(1,u,r,0,1))/X(0,u,r,0,0)
            +(deltambkin1*flagPERP*r*rhoLS*XG(1,u,r,0,1))/X(0,u,r,0,0)-(deltamcMS1*flagPERP*r*rhoLS*XG(1,u,r,0,1))/X(0,u,r,0,0)
            -(2*deltambkin1*muG*Q2*XG(1,u,r,1,0))/X(0,u,r,0,0)+(2*deltambkin1*flagPERP*Q2*rhoLS*XG(1,u,r,1,0))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 1 NLO pw = api*",resNLO*par.mbkin**2)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=((Rhodpert1*X(1,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2-(Rhodpert1*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)
            +(FLAGcD*Rhodpert1*X(1,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*Rhodpert1*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(Rhodpert1*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(FLAGcD*Rhodpert1*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(Rhodpert1*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2)
        if( flagDEBUG == 1):
            print("Q2moment n. 1 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**2)
        res += api**2*resNNLO

    return res*par.mbkin**2


def moment_2_KIN_MS(q2cut, par, hqe, wc, **kwargs):
    Q2=q2cut/par.mbkin**2
    r=par.mcMS/par.mbkin
    u=(1.+r**2-Q2-math.sqrt(r**4+(1.-Q2)**2-2*r**2*(1.+Q2)))/2/r
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    FLAGcf=0.5*(4./3. + 3.*(1.+ math.log(par.scale_muG/par.mbkin)))
    FLAGcD=-8./3.*4./3.*math.log(par.scale_rhoD/par.mbkin)+3.*(1./2.-2./3.*math.log(par.scale_rhoD/par.mbkin))
    FLAGcs=0.5*(4./3. + 3.*(1.+math.log(par.scale_muG/par.mbkin)))

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    res = 0
    res +=(-(X(1,u,r,0,0)**2/X(0,u,r,0,0)**2)+X(2,u,r,0,0)/X(0,u,r,0,0)
        +(2*rhoD*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        -(rhoD*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        -(2*rhoD*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        )
    res += ((rhoD*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(2*muG*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
        -(2*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
        -(muG*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
        -(2*muG*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(2*flagPERP*rhoLS*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
        )
    res += ((muG*XG(2,u,r,0,0))/X(0,u,r,0,0)-(flagPERP*rhoLS*XG(2,u,r,0,0))/X(0,u,r,0,0))

    if( flagDEBUG == 1):
        print("Q2moment n. 2 LO = ",res*par.mbkin**4)

    resNLO = 0
    resNLO +=((-4*deltambkin1*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**2-(2*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
        +(2*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
        -(4*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
        +(2*deltambkin1*r*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
        -(2*deltamcMS1*r*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
        +(4*deltambkin1*Q2*X(1,u,r,0,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
        +(4*deltambkin1*X(2,u,r,0,0))/X(0,u,r,0,0)+(deltambkin1*r*X(0,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        -(deltamcMS1*r*X(0,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2+(2*deltambkin1*Q2*X(0,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        -(deltambkin1*r*X(2,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*r*X(2,u,r,0,1))/X(0,u,r,0,0)
        -(2*deltambkin1*Q2*X(2,u,r,1,0))/X(0,u,r,0,0)+(2*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
        -(X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2-(2*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
        +nlo.X1Q2(2,'SM',Q2,r)/X(0,u,r,0,0)-(2*Rhodpert1*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        +(Rhodpert1*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        +(2*Rhodpert1*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        -(Rhodpert1*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0))

    if( flagDEBUG == 1):
        print("Q2moment n. 2 NLO = api*",resNLO*par.mbkin**4)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=((-2*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(2,Q2,r))/X(0,u,r,0,0)-(deltambkin1*r*nlo.X1Q2_Derivativer(2,Q2,r))/X(0,u,r,0,0)
            +(deltamcMS1*r*nlo.X1Q2_Derivativer(2,Q2,r))/X(0,u,r,0,0)+(4*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltamcMS1*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(2*deltambkin1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(6*deltambkin1**2*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**2-(4*deltambkin2*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**2
            -(6*deltambkin1**2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(2*deltambkin2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(6*deltambkin1*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(2*deltamcMS2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(3*deltambkin1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**4
            +(6*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**4
            -(3*deltamcMS1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**4
            +(deltambkin1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(10*deltambkin1**2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(4*deltambkin2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(12*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**4
            +(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**4
            -(12*deltambkin1**2*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**4
            +(4*deltambkin1**2*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(4*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(4*deltambkin1**2*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(6*deltambkin1**2*r*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin2*r*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(6*deltambkin1*deltamcMS1*r*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*deltamcMS2*r*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(4*deltambkin1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(4*deltamcMS1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(8*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(deltambkin1**2*r**2*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**2+(2*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**2-(deltambkin1**2*r**2*X(1,u,r,0,0)*X(1,u,r,0,2))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,0)*X(1,u,r,0,2))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(1,u,r,0,0)*X(1,u,r,0,2))/X(0,u,r,0,0)**2
            +(10*deltambkin1**2*Q2*X(1,u,r,0,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(4*deltambkin2*Q2*X(1,u,r,0,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(8*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            -(8*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            +(16*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1**2*Q2*r*X(1,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1**2*Q2**2*X(1,u,r,1,0)**2)/X(0,u,r,0,0)**2-(4*deltambkin1**2*Q2*r*X(1,u,r,0,0)*X(1,u,r,1,1))/X(0,u,r,0,0)**2
            +(4*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,0,0)*X(1,u,r,1,1))/X(0,u,r,0,0)**2
            -(4*deltambkin1**2*Q2**2*X(1,u,r,0,0)*X(1,u,r,2,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(2,u,r,0,0))/X(0,u,r,0,0)**2+(6*deltambkin1**2*X(2,u,r,0,0))/X(0,u,r,0,0)
            +(4*deltambkin2*X(2,u,r,0,0))/X(0,u,r,0,0)+(3*deltambkin1**2*r*X(0,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin2*r*X(0,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2-(3*deltambkin1*deltamcMS1*r*X(0,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS2*r*X(0,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2+(deltambkin1**2*r**2*X(0,u,r,0,1)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,u,r,0,1)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(deltambkin1**2*r**2*X(0,u,r,0,2)*X(2,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(deltambkin1*deltamcMS1*r**2*X(0,u,r,0,2)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,u,r,0,2)*X(2,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(5*deltambkin1**2*Q2*X(0,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin2*Q2*X(0,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1**2*Q2**2*X(0,u,r,1,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1**2*Q2*r*X(0,u,r,1,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1**2*Q2**2*X(0,u,r,2,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltambkin1**2*r*X(2,u,r,0,1))/X(0,u,r,0,0)-(deltambkin2*r*X(2,u,r,0,1))/X(0,u,r,0,0)
            +(3*deltambkin1*deltamcMS1*r*X(2,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS2*r*X(2,u,r,0,1))/X(0,u,r,0,0)
            -(deltambkin1**2*r**2*X(0,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltambkin1**2*r**2*X(2,u,r,0,2))/(2*X(0,u,r,0,0))-(deltambkin1*deltamcMS1*r**2*X(2,u,r,0,2))/X(0,u,r,0,0)
            +(deltamcMS1**2*r**2*X(2,u,r,0,2))/(2*X(0,u,r,0,0))-(5*deltambkin1**2*Q2*X(2,u,r,1,0))/X(0,u,r,0,0)
            -(2*deltambkin2*Q2*X(2,u,r,1,0))/X(0,u,r,0,0)-(2*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1**2*Q2*r*X(2,u,r,1,1))/X(0,u,r,0,0)-(2*deltambkin1*deltamcMS1*Q2*r*X(2,u,r,1,1))/X(0,u,r,0,0)
            +(2*deltambkin1**2*Q2**2*X(2,u,r,2,0))/X(0,u,r,0,0)+(8*deltambkin1*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(25*math.log(mus**2)*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r))/(6*X(0,u,r,0,0)**3)
            +(6*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(6*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(12*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(4*deltambkin1*r*X(1,u,r,0,0)*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(4*deltamcMS1*r*X(1,u,r,0,0)*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(8*deltambkin1*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*deltambkin1*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(25*math.log(mus**2)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(12*X(0,u,r,0,0)**2)
            -(2*deltambkin1*r*X(0,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*X(0,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*X(0,u,r,1,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(deltambkin1*r*X(2,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*X(2,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*X(2,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(3*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**4+(X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**3
            -(8*deltambkin1*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(25*math.log(mus**2)*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/(6*X(0,u,r,0,0)**2)
            -(4*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(4*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(8*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(2*deltambkin1*r*X(1,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(2*deltamcMS1*r*X(1,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(4*deltambkin1*Q2*X(1,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(4*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -nlo.X1Q2(1,'SM',Q2,r)**2/X(0,u,r,0,0)**2+(4*deltambkin1*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)
            +(25*math.log(mus**2)*nlo.X1Q2(2,'SM',Q2,r))/(12*X(0,u,r,0,0))+(deltambkin1*r*X(0,u,r,0,1)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*X(0,u,r,0,1)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*X(0,u,r,1,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2+(2*X(1,u,r,0,0)**2*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**3
            -(X(2,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**2-(2*X(1,u,r,0,0)*nnlo.X2Q2(1,Q2,r))/X(0,u,r,0,0)**2
            +nnlo.X2Q2(2,Q2,r)/X(0,u,r,0,0)-(2*deltambkin1*Rhodpert1*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*Rhodpert2*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**3)
            -(2*Rhodpert1*math.log(r)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**3)
            -(6*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(6*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(4*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*Rhodpert1*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(Rhodpert2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(Rhodpert1*math.log(mus**2)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0)**2)
            +(Rhodpert1*math.log(r)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(2*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(2,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)**2*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)**2*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(4*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)**2*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltambkin1*Q2*Rhodpert1*X(2,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Rhodpert1*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*Rhodpert2*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(2*Rhodpert1*math.log(r)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(4*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltambkin1*r*Rhodpert1*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltamcMS1*r*Rhodpert1*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltambkin1*Q2*Rhodpert1*X(1,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltambkin1*Rhodpert1*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)-(Rhodpert2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)
            +(Rhodpert1*math.log(mus**2)*XD(2,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0))
            -(Rhodpert1*math.log(r)*XD(2,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0))-(deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltambkin1*r*Rhodpert1*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(deltamcMS1*r*Rhodpert1*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            +(2*deltambkin1*Q2*Rhodpert1*XD(2,u,r,1,0,flagPERP))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 2 NNLO = api^2*",resNNLO*par.mbkin**4)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=((2*muG*X(1,u,r,0,0)**2*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**3-(2*flagPERP*rhoLS*X(1,u,r,0,0)**2*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**3
            -(muG*X(2,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*X(2,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2
            -(2*muG*X(1,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**2+(2*flagPERP*rhoLS*X(1,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**2
            )
        resNLO += ((muG*nlopw.X1Q2MuG(2,Q2,r))/X(0,u,r,0,0)-(flagPERP*rhoLS*nlopw.X1Q2MuG(2,Q2,r))/X(0,u,r,0,0)
            +(2*rhoD*X(1,u,r,0,0)**2*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**3
            -(rhoD*X(2,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2-(2*rhoD*X(1,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**2
            )
        resNLO += ((rhoD*nlopw.X1Q2RhoD(2,Q2,r))/X(0,u,r,0,0)+(2*deltambkin1*rhoD*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*FLAGcD*rhoD*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(6*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(4*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(deltambkin1*rhoD*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(FLAGcD*rhoD*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(2,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*rhoD*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(2*rhoD*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*rhoD*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(rhoD*nlo.X1Q2(2,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltambkin1*Q2*rhoD*X(2,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*rhoD*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*FLAGcD*rhoD*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltambkin1*r*rhoD*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltamcMS1*r*rhoD*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(4*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(4*rhoD*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*rhoD*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*r*rhoD*X(1,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltamcMS1*r*rhoD*X(1,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(4*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltambkin1*rhoD*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(FLAGcD*rhoD*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)
            +(deltambkin1*r*rhoD*X(0,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(0,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(rhoD*nlo.X1Q2(0,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltambkin1*r*rhoD*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)+(deltamcMS1*r*rhoD*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(2*deltambkin1*Q2*rhoD*XD(2,u,r,1,0,flagPERP))/X(0,u,r,0,0)+(4*deltambkin1*muG*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*FLAGcf*muG*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(6*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(6*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(12*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(4*deltambkin1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltamcMS1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1*muG*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(FLAGcf*muG*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(deltambkin1*flagPERP*rhoLS*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(FLAGcs*flagPERP*rhoLS*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*muG*r*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*muG*r*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(2,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(2,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*muG*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*flagPERP*rhoLS*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(2*muG*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*flagPERP*rhoLS*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*muG*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*flagPERP*rhoLS*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(muG*nlo.X1Q2(2,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*nlo.X1Q2(2,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*muG*r*X(1,u,r,0,0)**2*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*muG*r*X(1,u,r,0,0)**2*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(2*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(4*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*XG(0,u,r,1,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*XG(0,u,r,1,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*muG*Q2*X(2,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(2,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*muG*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*FLAGcf*muG*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(2*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*muG*r*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltamcMS1*muG*r*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1*muG*Q2*X(1,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*muG*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*flagPERP*rhoLS*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*muG*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(2*flagPERP*rhoLS*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*r*X(1,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*deltamcMS1*muG*r*X(1,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(4*deltambkin1*muG*Q2*X(1,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*XG(2,u,r,0,0))/X(0,u,r,0,0)+(FLAGcf*muG*XG(2,u,r,0,0))/X(0,u,r,0,0)
            -(deltambkin1*flagPERP*rhoLS*XG(2,u,r,0,0))/X(0,u,r,0,0)-(FLAGcs*flagPERP*rhoLS*XG(2,u,r,0,0))/X(0,u,r,0,0)
            +(deltambkin1*muG*r*X(0,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(0,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(0,u,r,1,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(muG*nlo.X1Q2(0,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*nlo.X1Q2(0,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*muG*r*XG(2,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*muG*r*XG(2,u,r,0,1))/X(0,u,r,0,0)
            +(deltambkin1*flagPERP*r*rhoLS*XG(2,u,r,0,1))/X(0,u,r,0,0)-(deltamcMS1*flagPERP*r*rhoLS*XG(2,u,r,0,1))/X(0,u,r,0,0)
            -(2*deltambkin1*muG*Q2*XG(2,u,r,1,0))/X(0,u,r,0,0)+(2*deltambkin1*flagPERP*Q2*rhoLS*XG(2,u,r,1,0))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 2 NLO pw = api*",resNLO*par.mbkin**4)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=((-2*Rhodpert1*X(1,u,r,0,0)**2*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**3
            +(Rhodpert1*X(2,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2
            +(2*Rhodpert1*X(1,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**2
            -(Rhodpert1*nlopw.X1Q2RhoD(2,Q2,r))/X(0,u,r,0,0)-(2*FLAGcD*Rhodpert1*X(1,u,r,0,0)**2*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(FLAGcD*Rhodpert1*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*Rhodpert1*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(2*Rhodpert1*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*Rhodpert1*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(Rhodpert1*nlo.X1Q2(2,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*FLAGcD*Rhodpert1*X(1,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(4*Rhodpert1*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*Rhodpert1*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(FLAGcD*Rhodpert1*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(Rhodpert1*nlo.X1Q2(0,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2)
        if( flagDEBUG == 1):
            print("Q2moment n. 2 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**4)
        res += api**2*resNNLO

    return res*par.mbkin**4


def moment_3_KIN_MS(q2cut, par, hqe, wc, **kwargs):
    Q2=q2cut/par.mbkin**2
    r=par.mcMS/par.mbkin
    u=(1.+r**2-Q2-math.sqrt(r**4+(1.-Q2)**2-2*r**2*(1.+Q2)))/2/r
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    FLAGcf=0.5*(4./3. + 3.*(1.+ math.log(par.scale_muG/par.mbkin)))
    FLAGcD=-8./3.*4./3.*math.log(par.scale_rhoD/par.mbkin)+3.*(1./2.-2./3.*math.log(par.scale_rhoD/par.mbkin))
    FLAGcs=0.5*(4./3. + 3.*(1.+math.log(par.scale_muG/par.mbkin)))

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    res = 0
    res +=((2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**3-(3*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        +X(3,u,r,0,0)/X(0,u,r,0,0)-(6*rhoD*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
        +(6*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        -(rhoD*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        +(6*rhoD*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        -(3*rhoD*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        -(3*rhoD*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        )
    res += ((rhoD*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)-(6*muG*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
        +(6*flagPERP*rhoLS*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
        +(6*muG*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
        -(6*flagPERP*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
        -(muG*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
        +(6*muG*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**3-(6*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
        -(3*muG*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(3*flagPERP*rhoLS*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
        -(3*muG*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2+(3*flagPERP*rhoLS*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
        )
    res += ((muG*XG(3,u,r,0,0))/X(0,u,r,0,0)-(flagPERP*rhoLS*XG(3,u,r,0,0))/X(0,u,r,0,0))

    if( flagDEBUG == 1):
        print("Q2moment n. 3 LO = ",res*par.mbkin**6)

    resNLO = 0
    resNLO +=((12*deltambkin1*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**3+(6*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
        -(6*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
        +(12*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
        -(6*deltambkin1*r*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**3
        +(6*deltamcMS1*r*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**3
        -(12*deltambkin1*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0))/X(0,u,r,0,0)**3
        -(18*deltambkin1*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        -(6*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        +(6*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        -(12*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        +(3*deltambkin1*r*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        -(3*deltamcMS1*r*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        +(6*deltambkin1*Q2*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
        +(3*deltambkin1*r*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
        -(3*deltamcMS1*r*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
        +(6*deltambkin1*Q2*X(1,u,r,0,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
        +(6*deltambkin1*X(3,u,r,0,0))/X(0,u,r,0,0)+(deltambkin1*r*X(0,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
        -(deltamcMS1*r*X(0,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(2*deltambkin1*Q2*X(0,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
        -(deltambkin1*r*X(3,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*r*X(3,u,r,0,1))/X(0,u,r,0,0)
        -(2*deltambkin1*Q2*X(3,u,r,1,0))/X(0,u,r,0,0)-(6*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
        +(6*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
        -(X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
        -(3*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2-(3*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
        +nlo.X1Q2(3,'SM',Q2,r)/X(0,u,r,0,0)+(6*Rhodpert1*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
        -(6*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        +(Rhodpert1*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        -(6*Rhodpert1*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        +(3*Rhodpert1*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        +(3*Rhodpert1*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
        -(Rhodpert1*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0))

    if( flagDEBUG == 1):
        print("Q2moment n. 3 NLO = api*",resNLO*par.mbkin**6)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=((-2*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(3,Q2,r))/X(0,u,r,0,0)-(deltambkin1*r*nlo.X1Q2_Derivativer(3,Q2,r))/X(0,u,r,0,0)
            +(deltamcMS1*r*nlo.X1Q2_Derivativer(3,Q2,r))/X(0,u,r,0,0)+(6*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(2,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*deltambkin1*r*nlo.X1Q2_Derivativer(2,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*nlo.X1Q2_Derivativer(2,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(12*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            -(6*deltambkin1*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(12*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(6*deltambkin1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(6*deltamcMS1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(30*deltambkin1**2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**3+(12*deltambkin2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**3
            +(30*deltambkin1**2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(6*deltambkin2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(30*deltambkin1*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(6*deltamcMS2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(12*deltambkin1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**5
            -(24*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**5
            +(12*deltamcMS1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**5
            -(3*deltambkin1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(6*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(3*deltamcMS1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(54*deltambkin1**2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(12*deltambkin2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(48*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**5
            -(48*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**5
            +(48*deltambkin1**2*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**5
            -(12*deltambkin1**2*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(12*deltambkin1**2*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(30*deltambkin1**2*r*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(6*deltambkin2*r*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(30*deltambkin1*deltamcMS1*r*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltamcMS2*r*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(18*deltambkin1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            +(36*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            -(18*deltamcMS1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            -(36*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            +(36*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            +(6*deltambkin1**2*r**2*X(1,u,r,0,0)*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,0)*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**3
            +(6*deltamcMS1**2*r**2*X(1,u,r,0,0)*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**3
            +(3*deltambkin1**2*r**2*X(1,u,r,0,0)**2*X(1,u,r,0,2))/X(0,u,r,0,0)**3
            -(6*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,0)**2*X(1,u,r,0,2))/X(0,u,r,0,0)**3
            +(3*deltamcMS1**2*r**2*X(1,u,r,0,0)**2*X(1,u,r,0,2))/X(0,u,r,0,0)**3
            -(54*deltambkin1**2*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            -(12*deltambkin2*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            -(36*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(1,u,r,1,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(1,u,r,1,0))/X(0,u,r,0,0)**4
            -(72*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(1,u,r,1,0))/X(0,u,r,0,0)**4
            +(24*deltambkin1**2*Q2*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1**2*Q2**2*X(1,u,r,0,0)*X(1,u,r,1,0)**2)/X(0,u,r,0,0)**3
            +(12*deltambkin1**2*Q2*r*X(1,u,r,0,0)**2*X(1,u,r,1,1))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,0,0)**2*X(1,u,r,1,1))/X(0,u,r,0,0)**3
            +(12*deltambkin1**2*Q2**2*X(1,u,r,0,0)**2*X(1,u,r,2,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*deltambkin1*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(12*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(45*deltambkin1**2*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(18*deltambkin2*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(30*deltambkin1**2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(30*deltambkin1*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(9*deltambkin1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(9*deltamcMS1**2*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(3*deltambkin1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(3*deltamcMS1**2*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(54*deltambkin1**2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin2*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(36*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1**2*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(12*deltambkin1**2*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1**2*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(15*deltambkin1**2*r*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*deltambkin2*r*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(15*deltambkin1*deltamcMS1*r*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltamcMS2*r*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(3*deltambkin1**2*r**2*X(1,u,r,0,2)*X(2,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(3*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,2)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltamcMS1**2*r**2*X(1,u,r,0,2)*X(2,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(27*deltambkin1**2*Q2*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*deltambkin2*Q2*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(12*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1**2*Q2*r*X(1,u,r,1,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,1,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1**2*Q2**2*X(1,u,r,2,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(15*deltambkin1**2*r*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(3*deltambkin2*r*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(15*deltambkin1*deltamcMS1*r*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltamcMS2*r*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(6*deltambkin1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltamcMS1**2*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(12*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(3*deltambkin1**2*r**2*X(1,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(6*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltamcMS1**2*r**2*X(1,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(6*deltambkin1**2*Q2*r*X(1,u,r,1,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(6*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,1,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltambkin1**2*r**2*X(1,u,r,0,0)*X(2,u,r,0,2))/(2*X(0,u,r,0,0)**2)
            +(3*deltambkin1*deltamcMS1*r**2*X(1,u,r,0,0)*X(2,u,r,0,2))/X(0,u,r,0,0)**2
            -(3*deltamcMS1**2*r**2*X(1,u,r,0,0)*X(2,u,r,0,2))/(2*X(0,u,r,0,0)**2)
            +(27*deltambkin1**2*Q2*X(1,u,r,0,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            +(6*deltambkin2*Q2*X(1,u,r,0,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            +(12*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1**2*Q2*r*X(1,u,r,0,1)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,0,1)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            -(12*deltambkin1**2*Q2**2*X(1,u,r,1,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1**2*Q2*r*X(1,u,r,0,0)*X(2,u,r,1,1))/X(0,u,r,0,0)**2
            +(6*deltambkin1*deltamcMS1*Q2*r*X(1,u,r,0,0)*X(2,u,r,1,1))/X(0,u,r,0,0)**2
            -(6*deltambkin1**2*Q2**2*X(1,u,r,0,0)*X(2,u,r,2,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(15*deltambkin1**2*X(3,u,r,0,0))/X(0,u,r,0,0)
            +(6*deltambkin2*X(3,u,r,0,0))/X(0,u,r,0,0)+(5*deltambkin1**2*r*X(0,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltambkin2*r*X(0,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(5*deltambkin1*deltamcMS1*r*X(0,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS2*r*X(0,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(deltambkin1**2*r**2*X(0,u,r,0,1)**2*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)**2*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltamcMS1**2*r**2*X(0,u,r,0,1)**2*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(deltambkin1**2*r**2*X(0,u,r,0,2)*X(3,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(deltambkin1*deltamcMS1*r**2*X(0,u,r,0,2)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,u,r,0,2)*X(3,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(9*deltambkin1**2*Q2*X(0,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin2*Q2*X(0,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1**2*Q2**2*X(0,u,r,1,0)**2*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltambkin1**2*Q2*r*X(0,u,r,1,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1**2*Q2**2*X(0,u,r,2,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(5*deltambkin1**2*r*X(3,u,r,0,1))/X(0,u,r,0,0)-(deltambkin2*r*X(3,u,r,0,1))/X(0,u,r,0,0)
            +(5*deltambkin1*deltamcMS1*r*X(3,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS2*r*X(3,u,r,0,1))/X(0,u,r,0,0)
            -(deltambkin1**2*r**2*X(0,u,r,0,1)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*r**2*X(0,u,r,0,1)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1**2*r**2*X(0,u,r,0,1)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*deltambkin1**2*Q2*r*X(0,u,r,1,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,1,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltambkin1**2*r**2*X(3,u,r,0,2))/(2*X(0,u,r,0,0))-(deltambkin1*deltamcMS1*r**2*X(3,u,r,0,2))/X(0,u,r,0,0)
            +(deltamcMS1**2*r**2*X(3,u,r,0,2))/(2*X(0,u,r,0,0))-(9*deltambkin1**2*Q2*X(3,u,r,1,0))/X(0,u,r,0,0)
            -(2*deltambkin2*Q2*X(3,u,r,1,0))/X(0,u,r,0,0)-(2*deltambkin1**2*Q2*r*X(0,u,r,0,1)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*deltamcMS1*Q2*r*X(0,u,r,0,1)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1**2*Q2**2*X(0,u,r,1,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1**2*Q2*r*X(3,u,r,1,1))/X(0,u,r,0,0)-(2*deltambkin1*deltamcMS1*Q2*r*X(3,u,r,1,1))/X(0,u,r,0,0)
            +(2*deltambkin1**2*Q2**2*X(3,u,r,2,0))/X(0,u,r,0,0)-(36*deltambkin1*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(25*math.log(mus**2)*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r))/(2*X(0,u,r,0,0)**4)
            -(24*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            +(24*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(48*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            +(18*deltambkin1*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(36*deltambkin1*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(25*math.log(mus**2)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(2*X(0,u,r,0,0)**3)
            +(18*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(6*deltambkin1*r*X(1,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*X(1,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*X(1,u,r,1,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(6*deltambkin1*r*X(1,u,r,0,0)*X(2,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*X(1,u,r,0,0)*X(2,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*X(1,u,r,0,0)*X(2,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(6*deltambkin1*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(25*math.log(mus**2)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(12*X(0,u,r,0,0)**2)
            -(2*deltambkin1*r*X(0,u,r,0,1)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*X(0,u,r,0,1)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*X(0,u,r,1,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(deltambkin1*r*X(3,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*X(3,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*X(3,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(12*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**5-(9*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**4
            +(X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**3+(36*deltambkin1*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(25*math.log(mus**2)*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r))/(2*X(0,u,r,0,0)**3)
            +(18*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(12*deltambkin1*r*X(1,u,r,0,0)*X(1,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*r*X(1,u,r,0,0)*X(1,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(24*deltambkin1*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(18*deltambkin1*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(25*math.log(mus**2)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/(4*X(0,u,r,0,0)**2)
            -(6*deltambkin1*r*X(0,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*X(0,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*X(0,u,r,1,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(3*deltambkin1*r*X(2,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*X(2,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(6*deltambkin1*Q2*X(2,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(18*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(6*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(6*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)**2)/X(0,u,r,0,0)**3-(18*deltambkin1*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(25*math.log(mus**2)*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r))/(4*X(0,u,r,0,0)**2)
            -(6*deltambkin1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(3*deltambkin1*r*X(1,u,r,0,1)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*X(1,u,r,0,1)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(6*deltambkin1*Q2*X(1,u,r,1,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(6*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(3*nlo.X1Q2(1,'SM',Q2,r)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**2+(6*deltambkin1*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)
            +(25*math.log(mus**2)*nlo.X1Q2(3,'SM',Q2,r))/(12*X(0,u,r,0,0))+(deltambkin1*r*X(0,u,r,0,1)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*X(0,u,r,0,1)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*X(0,u,r,1,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**3*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**4
            +(6*X(1,u,r,0,0)*X(2,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**3
            -(X(3,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*nnlo.X2Q2(1,Q2,r))/X(0,u,r,0,0)**3
            -(3*X(2,u,r,0,0)*nnlo.X2Q2(1,Q2,r))/X(0,u,r,0,0)**2-(3*X(1,u,r,0,0)*nnlo.X2Q2(2,Q2,r))/X(0,u,r,0,0)**2
            +nnlo.X2Q2(3,Q2,r)/X(0,u,r,0,0)+(18*deltambkin1*Rhodpert1*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(6*Rhodpert2*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(2*Rhodpert1*math.log(r)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(24*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(24*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(48*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(18*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltambkin1*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*Rhodpert2*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*Rhodpert1*math.log(r)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(18*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(6*deltambkin1*r*Rhodpert1*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*r*Rhodpert1*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltambkin1*Q2*Rhodpert1*X(1,u,r,1,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(3*deltambkin1*Rhodpert1*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(Rhodpert2*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(Rhodpert1*math.log(mus**2)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0)**2)
            +(Rhodpert1*math.log(r)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(2*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(3,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)**3*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            +(6*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)**3*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            +(6*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(deltambkin1*r*Rhodpert1*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(12*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)**3*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*deltambkin1*Q2*Rhodpert1*X(3,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(18*deltambkin1*Rhodpert1*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*Rhodpert2*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*Rhodpert1*math.log(r)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(18*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(24*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*X(1,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(9*deltambkin1*Rhodpert1*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*Rhodpert2*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(Rhodpert1*math.log(mus**2)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/(2*X(0,u,r,0,0)**2)
            +(Rhodpert1*math.log(r)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(3*deltambkin1*r*Rhodpert1*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*r*Rhodpert1*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*Q2*Rhodpert1*X(2,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)**2*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)**2*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(3*deltambkin1*r*Rhodpert1*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*r*Rhodpert1*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(12*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)**2*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltambkin1*Q2*Rhodpert1*X(2,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(9*deltambkin1*Rhodpert1*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*Rhodpert2*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(Rhodpert1*math.log(mus**2)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/(2*X(0,u,r,0,0)**2)
            +(Rhodpert1*math.log(r)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(3*deltambkin1*r*Rhodpert1*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*r*Rhodpert1*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*Q2*Rhodpert1*X(1,u,r,1,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(3*deltambkin1*r*Rhodpert1*X(1,u,r,0,0)*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*r*Rhodpert1*X(1,u,r,0,0)*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*Q2*Rhodpert1*X(1,u,r,0,0)*XD(2,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(3*deltambkin1*Rhodpert1*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)
            -(Rhodpert2*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(Rhodpert1*math.log(mus**2)*XD(3,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0))
            -(Rhodpert1*math.log(r)*XD(3,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0))-(deltambkin1*r*Rhodpert1*X(0,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltamcMS1*r*Rhodpert1*X(0,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*Q2*Rhodpert1*X(0,u,r,1,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(deltambkin1*r*Rhodpert1*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(deltamcMS1*r*Rhodpert1*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            +(2*deltambkin1*Q2*Rhodpert1*XD(3,u,r,1,0,flagPERP))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 3 NNLO = api^2*",resNNLO*par.mbkin**6)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=((-6*muG*X(1,u,r,0,0)**3*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**4+(6*flagPERP*rhoLS*X(1,u,r,0,0)**3*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**4
            +(6*muG*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**3
            -(6*flagPERP*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**3
            -(muG*X(3,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*X(3,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2
            +(6*muG*X(1,u,r,0,0)**2*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**3-(6*flagPERP*rhoLS*X(1,u,r,0,0)**2*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**3
            -(3*muG*X(2,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**2+(3*flagPERP*rhoLS*X(2,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**2
            -(3*muG*X(1,u,r,0,0)*nlopw.X1Q2MuG(2,Q2,r))/X(0,u,r,0,0)**2+(3*flagPERP*rhoLS*X(1,u,r,0,0)*nlopw.X1Q2MuG(2,Q2,r))/X(0,u,r,0,0)**2
            )
        resNLO += ((muG*nlopw.X1Q2MuG(3,Q2,r))/X(0,u,r,0,0)-(flagPERP*rhoLS*nlopw.X1Q2MuG(3,Q2,r))/X(0,u,r,0,0)
            -(6*rhoD*X(1,u,r,0,0)**3*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**4
            +(6*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**3
            -(rhoD*X(3,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2+(6*rhoD*X(1,u,r,0,0)**2*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**3
            -(3*rhoD*X(2,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**2-(3*rhoD*X(1,u,r,0,0)*nlopw.X1Q2RhoD(2,Q2,r))/X(0,u,r,0,0)**2
            )
        resNLO += ((rhoD*nlopw.X1Q2RhoD(3,Q2,r))/X(0,u,r,0,0)-(18*deltambkin1*rhoD*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(6*FLAGcD*rhoD*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(24*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(24*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(48*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(18*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(18*deltambkin1*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*FLAGcD*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(18*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(6*deltambkin1*r*rhoD*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(2,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(3*deltambkin1*rhoD*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(FLAGcD*rhoD*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(3,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(24*rhoD*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(18*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(2*rhoD*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(18*rhoD*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(6*rhoD*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*rhoD*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(rhoD*nlo.X1Q2(3,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*r*rhoD*X(1,u,r,0,0)**3*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(6*deltamcMS1*r*rhoD*X(1,u,r,0,0)**3*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(6*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(12*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**3*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltambkin1*Q2*rhoD*X(3,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(18*deltambkin1*rhoD*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*FLAGcD*rhoD*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(18*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(24*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(1,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(9*deltambkin1*rhoD*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(3*FLAGcD*rhoD*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(3*deltambkin1*r*rhoD*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*rhoD*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*Q2*rhoD*X(2,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(18*rhoD*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(6*rhoD*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*rhoD*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(3*rhoD*nlo.X1Q2(2,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(3*deltambkin1*r*rhoD*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*rhoD*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(12*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltambkin1*Q2*rhoD*X(2,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(9*deltambkin1*rhoD*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(3*FLAGcD*rhoD*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(3*deltambkin1*r*rhoD*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*rhoD*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(6*rhoD*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(3*rhoD*nlo.X1Q2(1,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*deltambkin1*r*rhoD*X(1,u,r,0,0)*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*r*rhoD*X(1,u,r,0,0)*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(6*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*XD(2,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*deltambkin1*rhoD*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(FLAGcD*rhoD*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)
            +(deltambkin1*r*rhoD*X(0,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(0,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(rhoD*nlo.X1Q2(0,'SM',Q2,r)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltambkin1*r*rhoD*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)+(deltamcMS1*r*rhoD*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(2*deltambkin1*Q2*rhoD*XD(3,u,r,1,0,flagPERP))/X(0,u,r,0,0)-(24*deltambkin1*muG*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(6*FLAGcf*muG*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(24*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(24*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(24*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(24*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(48*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(48*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(18*deltambkin1*muG*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*muG*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(24*deltambkin1*muG*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*FLAGcf*muG*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(18*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(18*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(6*deltambkin1*muG*r*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*Q2*X(1,u,r,1,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(2,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(2,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*muG*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(FLAGcf*muG*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(3*deltambkin1*flagPERP*rhoLS*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(FLAGcs*flagPERP*rhoLS*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*muG*r*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*muG*r*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(3,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(3,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(24*muG*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(24*flagPERP*rhoLS*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(18*muG*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*flagPERP*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(2*muG*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*flagPERP*rhoLS*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(18*muG*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*flagPERP*rhoLS*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*muG*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*flagPERP*rhoLS*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*muG*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*flagPERP*rhoLS*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(muG*nlo.X1Q2(3,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*nlo.X1Q2(3,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1*muG*r*X(1,u,r,0,0)**3*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            -(6*deltamcMS1*muG*r*X(1,u,r,0,0)**3*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            -(6*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**3*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            +(6*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**3*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            -(6*deltambkin1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            +(12*deltambkin1*muG*Q2*X(1,u,r,0,0)**3*XG(0,u,r,1,0))/X(0,u,r,0,0)**4
            -(12*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**3*XG(0,u,r,1,0))/X(0,u,r,0,0)**4
            -(12*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*muG*Q2*X(3,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(3,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            +(24*deltambkin1*muG*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*FLAGcf*muG*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(18*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(18*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*deltambkin1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(1,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*FLAGcf*muG*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(9*deltambkin1*flagPERP*rhoLS*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*FLAGcs*flagPERP*rhoLS*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*muG*r*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(3*deltambkin1*muG*r*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*muG*r*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltambkin1*flagPERP*r*rhoLS*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*flagPERP*r*rhoLS*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1*muG*Q2*X(2,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*flagPERP*Q2*rhoLS*X(2,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(18*muG*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*flagPERP*rhoLS*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*muG*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*flagPERP*rhoLS*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*muG*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*flagPERP*rhoLS*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(3*muG*nlo.X1Q2(2,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(3*flagPERP*rhoLS*nlo.X1Q2(2,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*muG*r*X(1,u,r,0,0)**2*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(1,u,r,0,0)**2*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(3*deltambkin1*muG*r*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*muG*r*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltambkin1*flagPERP*r*rhoLS*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*flagPERP*r*rhoLS*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(12*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*XG(1,u,r,1,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*XG(1,u,r,1,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*muG*Q2*X(2,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*flagPERP*Q2*rhoLS*X(2,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**2
            -(12*deltambkin1*muG*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*FLAGcf*muG*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2+(9*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*FLAGcs*flagPERP*rhoLS*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(3*deltambkin1*muG*r*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*muG*r*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(3*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1*muG*Q2*X(1,u,r,1,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*muG*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*flagPERP*rhoLS*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(3*muG*nlo.X1Q2(1,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2+(3*flagPERP*rhoLS*nlo.X1Q2(1,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**2
            +(3*deltambkin1*muG*r*X(1,u,r,0,0)*XG(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltamcMS1*muG*r*X(1,u,r,0,0)*XG(2,u,r,0,1))/X(0,u,r,0,0)**2
            -(3*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(3*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(2,u,r,0,1))/X(0,u,r,0,0)**2
            +(6*deltambkin1*muG*Q2*X(1,u,r,0,0)*XG(2,u,r,1,0))/X(0,u,r,0,0)**2
            -(6*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*XG(2,u,r,1,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1*muG*XG(3,u,r,0,0))/X(0,u,r,0,0)+(FLAGcf*muG*XG(3,u,r,0,0))/X(0,u,r,0,0)
            -(3*deltambkin1*flagPERP*rhoLS*XG(3,u,r,0,0))/X(0,u,r,0,0)-(FLAGcs*flagPERP*rhoLS*XG(3,u,r,0,0))/X(0,u,r,0,0)
            +(deltambkin1*muG*r*X(0,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(0,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(0,u,r,1,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(muG*nlo.X1Q2(0,'SM',Q2,r)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2+(flagPERP*rhoLS*nlo.X1Q2(0,'SM',Q2,r)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*muG*r*XG(3,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*muG*r*XG(3,u,r,0,1))/X(0,u,r,0,0)
            +(deltambkin1*flagPERP*r*rhoLS*XG(3,u,r,0,1))/X(0,u,r,0,0)-(deltamcMS1*flagPERP*r*rhoLS*XG(3,u,r,0,1))/X(0,u,r,0,0)
            -(2*deltambkin1*muG*Q2*XG(3,u,r,1,0))/X(0,u,r,0,0)+(2*deltambkin1*flagPERP*Q2*rhoLS*XG(3,u,r,1,0))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 3 NLO pw = api*",resNLO*par.mbkin**6)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=((6*Rhodpert1*X(1,u,r,0,0)**3*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**4
            -(6*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**3
            +(Rhodpert1*X(3,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2
            -(6*Rhodpert1*X(1,u,r,0,0)**2*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**3
            +(3*Rhodpert1*X(2,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**2
            +(3*Rhodpert1*X(1,u,r,0,0)*nlopw.X1Q2RhoD(2,Q2,r))/X(0,u,r,0,0)**2
            -(Rhodpert1*nlopw.X1Q2RhoD(3,Q2,r))/X(0,u,r,0,0)+(6*FLAGcD*Rhodpert1*X(1,u,r,0,0)**3*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(6*FLAGcD*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(FLAGcD*Rhodpert1*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(24*Rhodpert1*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(18*Rhodpert1*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(2*Rhodpert1*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(18*Rhodpert1*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(6*Rhodpert1*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*Rhodpert1*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(Rhodpert1*nlo.X1Q2(3,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*FLAGcD*Rhodpert1*X(1,u,r,0,0)**2*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(3*FLAGcD*Rhodpert1*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(18*Rhodpert1*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(6*Rhodpert1*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*Rhodpert1*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(3*Rhodpert1*nlo.X1Q2(2,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(3*FLAGcD*Rhodpert1*X(1,u,r,0,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(6*Rhodpert1*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(3*Rhodpert1*nlo.X1Q2(1,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(FLAGcD*Rhodpert1*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(Rhodpert1*nlo.X1Q2(0,'SM',Q2,r)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2)
        if( flagDEBUG == 1):
            print("Q2moment n. 3 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**6)
        res += api**2*resNNLO

    return res*par.mbkin**6


def moment_4_KIN_MS(q2cut, par, hqe, wc, **kwargs):
    Q2=q2cut/par.mbkin**2
    r=par.mcMS/par.mbkin
    u=(1.+r**2-Q2-math.sqrt(r**4+(1.-Q2)**2-2*r**2*(1.+Q2)))/2/r
    mus=par.scale_alphas/par.mbkin
    mu0=par.scale_mcMS/par.mbkin
    muWC=par.scale_mbkin/par.mbkin
    api=par.alphas/math.pi

    rhoD=hqe.rhoD/par.mbkin**3
    rhoLS=hqe.rhoLS/par.mbkin**3
    muG=hqe.muG/par.mbkin**2
    mupi=hqe.mupi/par.mbkin**2

    flagPERP=kwargs.get('flag_basisPERP', 1)
    flagDEBUG=kwargs.get('flag_DEBUG', 0)
    FLAGcf=0.5*(4./3. + 3.*(1.+ math.log(par.scale_muG/par.mbkin)))
    FLAGcD=-8./3.*4./3.*math.log(par.scale_rhoD/par.mbkin)+3.*(1./2.-2./3.*math.log(par.scale_rhoD/par.mbkin))
    FLAGcs=0.5*(4./3. + 3.*(1.+math.log(par.scale_muG/par.mbkin)))

    deltambkin1 = kin.deltambkin(1,par)
    deltambkin2 = kin.deltambkin(2,par)
    deltamcMS1 = kin.deltamcMS(1,par)
    deltamcMS2 = kin.deltamcMS(2,par)
    Rhodpert1 = kin.RhoDPert(1,par)/par.mbkin**3
    Rhodpert2 = kin.RhoDPert(2,par)/par.mbkin**3

    res = 0
    res +=((-3*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**4+(6*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        -(4*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+X(4,u,r,0,0)/X(0,u,r,0,0)
        +rhoD*((12*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
        -(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
        +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        -(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
        +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        -(4*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        -(4*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+XD(4,u,r,0,0,flagPERP)/X(0,u,r,0,0))
        +flagPERP*rhoLS*((-12*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
        +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
        -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
        +(X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
        -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
        +(4*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
        +(4*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2-XG(4,u,r,0,0)/X(0,u,r,0,0))
        +muG*((12*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**5-(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
        +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
        -(X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
        +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
        -(4*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
        -(4*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2+XG(4,u,r,0,0)/X(0,u,r,0,0)))

    if( flagDEBUG == 1):
        print("Q2moment n. 4 LO = ",res*par.mbkin**8)

    resNLO = 0
    resNLO +=(deltamcMS1*((12*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
        -(12*r*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**4-(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
        +(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        +(6*r*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**3+(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
        -(4*r*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(4*r*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
        -(r*X(0,u,r,0,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2+(r*X(4,u,r,0,1))/X(0,u,r,0,0))
        +deltambkin1*((-24*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**4-(12*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
        -(24*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5+(12*r*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**4
        +(24*Q2*X(1,u,r,0,0)**3*X(1,u,r,1,0))/X(0,u,r,0,0)**4+(48*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        +(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
        +(36*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
        -(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        -(24*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
        -(6*r*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**3-(12*Q2*X(1,u,r,0,0)**2*X(2,u,r,1,0))/X(0,u,r,0,0)**3
        -(32*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
        -(16*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
        +(4*r*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(8*Q2*X(1,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
        +(4*r*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2+(8*Q2*X(1,u,r,0,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
        +(8*X(4,u,r,0,0))/X(0,u,r,0,0)+(r*X(0,u,r,0,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2
        +(2*Q2*X(0,u,r,1,0)*X(4,u,r,0,0))/X(0,u,r,0,0)**2-(r*X(4,u,r,0,1))/X(0,u,r,0,0)
        -(2*Q2*X(4,u,r,1,0))/X(0,u,r,0,0))+(12*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
        -(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
        +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
        -(X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
        +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
        -(4*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
        -(4*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2+nlo.X1Q2(4,'SM',Q2,r)/X(0,u,r,0,0)
        +Rhodpert1*((-12*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
        +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
        -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        +(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
        -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        +(4*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
        +(4*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-XD(4,u,r,0,0,flagPERP)/X(0,u,r,0,0)))

    if( flagDEBUG == 1):
        print("Q2moment n. 4 NLO = api*",resNLO*par.mbkin**8)
    res += api*resNLO

    if(kwargs.get('flag_includeNNLO', 1) == 1):
        resNNLO = 0
        resNNLO +=(deltamcMS2*((12*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            -(12*r*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**4-(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(6*r*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**3+(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*r*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(4*r*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(r*X(0,u,r,0,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2+(r*X(4,u,r,0,1))/X(0,u,r,0,0))
            +deltamcMS1**2*((-30*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**6
            +(6*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5+(48*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**5
            -(18*r**2*X(1,u,r,0,0)**2*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**4-(6*r**2*X(1,u,r,0,0)**3*X(1,u,r,0,2))/X(0,u,r,0,0)**4
            +(36*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**5
            -(9*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*r**2*X(1,u,r,0,1)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3+(6*r**2*X(1,u,r,0,0)*X(1,u,r,0,2)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(18*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**4
            +(12*r**2*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(3*r**2*X(1,u,r,0,0)**2*X(2,u,r,0,2))/X(0,u,r,0,0)**3-(12*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**4
            +(4*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*r**2*X(0,u,r,0,1)*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*r**2*X(1,u,r,0,2)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(8*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**3
            -(4*r**2*X(1,u,r,0,1)*X(3,u,r,0,1))/X(0,u,r,0,0)**2-(2*r**2*X(1,u,r,0,0)*X(3,u,r,0,2))/X(0,u,r,0,0)**2
            +(r**2*X(0,u,r,0,1)**2*X(4,u,r,0,0))/X(0,u,r,0,0)**3-(r**2*X(0,u,r,0,2)*X(4,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            -(r**2*X(0,u,r,0,1)*X(4,u,r,0,1))/X(0,u,r,0,0)**2+(r**2*X(4,u,r,0,2))/(2*X(0,u,r,0,0)))
            +deltambkin2*((-24*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**4-(12*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            -(24*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5+(12*r*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            +(24*Q2*X(1,u,r,0,0)**3*X(1,u,r,1,0))/X(0,u,r,0,0)**4+(48*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*r*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**3-(12*Q2*X(1,u,r,0,0)**2*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            -(32*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(16*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*r*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(8*Q2*X(1,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*r*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2+(8*Q2*X(1,u,r,0,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
            +(8*X(4,u,r,0,0))/X(0,u,r,0,0)+(r*X(0,u,r,0,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*Q2*X(0,u,r,1,0)*X(4,u,r,0,0))/X(0,u,r,0,0)**2-(r*X(4,u,r,0,1))/X(0,u,r,0,0)
            -(2*Q2*X(4,u,r,1,0))/X(0,u,r,0,0))+deltambkin1**2*((-84*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**4
            -(84*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5-(30*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**6
            +(6*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5-(156*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            -(120*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**6
            -(120*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**6+(24*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            +(24*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5+(84*r*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            +(48*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**5
            +(96*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**5
            -(18*r**2*X(1,u,r,0,0)**2*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**4-(6*r**2*X(1,u,r,0,0)**3*X(1,u,r,0,2))/X(0,u,r,0,0)**4
            +(156*Q2*X(1,u,r,0,0)**3*X(1,u,r,1,0))/X(0,u,r,0,0)**4+(96*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*X(1,u,r,1,0))/X(0,u,r,0,0)**5
            +(192*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)**3*X(1,u,r,1,0))/X(0,u,r,0,0)**5
            -(72*Q2*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**4
            -(72*Q2**2*X(1,u,r,0,0)**2*X(1,u,r,1,0)**2)/X(0,u,r,0,0)**4-(24*Q2*r*X(1,u,r,0,0)**3*X(1,u,r,1,1))/X(0,u,r,0,0)**4
            -(24*Q2**2*X(1,u,r,0,0)**3*X(1,u,r,2,0))/X(0,u,r,0,0)**4+(168*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(126*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**5
            -(9*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(234*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(144*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**5
            +(144*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**5
            -(36*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(84*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(36*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(72*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(6*r**2*X(1,u,r,0,1)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3+(6*r**2*X(1,u,r,0,0)*X(1,u,r,0,2)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(156*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(72*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(144*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(24*Q2*r*X(1,u,r,0,1)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*Q2**2*X(1,u,r,1,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3+(24*Q2*r*X(1,u,r,0,0)*X(1,u,r,1,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*Q2**2*X(1,u,r,0,0)*X(1,u,r,2,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(42*r*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**3-(18*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**4
            -(36*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**4
            +(12*r**2*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(24*Q2*r*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(3*r**2*X(1,u,r,0,0)**2*X(2,u,r,0,2))/X(0,u,r,0,0)**3-(78*Q2*X(1,u,r,0,0)**2*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            -(36*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,1,0))/X(0,u,r,0,0)**4
            -(72*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,1,0))/X(0,u,r,0,0)**4
            +(24*Q2*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            +(48*Q2**2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            +(12*Q2*r*X(1,u,r,0,0)**2*X(2,u,r,1,1))/X(0,u,r,0,0)**3+(12*Q2**2*X(1,u,r,0,0)**2*X(2,u,r,2,0))/X(0,u,r,0,0)**3
            -(112*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(56*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**4
            +(4*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(104*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(48*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**4
            -(48*Q2**2*X(0,u,r,1,0)**2*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**4
            +(16*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(16*Q2**2*X(0,u,r,2,0)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(28*r*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(8*r**2*X(0,u,r,0,1)*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(16*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*r**2*X(1,u,r,0,2)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(52*Q2*X(1,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(16*Q2*r*X(0,u,r,0,1)*X(1,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(32*Q2**2*X(0,u,r,1,0)*X(1,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*Q2*r*X(1,u,r,1,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(8*Q2**2*X(1,u,r,2,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(28*r*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2+(8*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**3
            +(16*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**3
            -(4*r**2*X(1,u,r,0,1)*X(3,u,r,0,1))/X(0,u,r,0,0)**2-(8*Q2*r*X(1,u,r,1,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(2*r**2*X(1,u,r,0,0)*X(3,u,r,0,2))/X(0,u,r,0,0)**2+(52*Q2*X(1,u,r,0,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
            +(16*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**3
            +(32*Q2**2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**3
            -(8*Q2*r*X(1,u,r,0,1)*X(3,u,r,1,0))/X(0,u,r,0,0)**2-(16*Q2**2*X(1,u,r,1,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**2
            -(8*Q2*r*X(1,u,r,0,0)*X(3,u,r,1,1))/X(0,u,r,0,0)**2-(8*Q2**2*X(1,u,r,0,0)*X(3,u,r,2,0))/X(0,u,r,0,0)**2
            +(28*X(4,u,r,0,0))/X(0,u,r,0,0)+(7*r*X(0,u,r,0,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2
            +(r**2*X(0,u,r,0,1)**2*X(4,u,r,0,0))/X(0,u,r,0,0)**3-(r**2*X(0,u,r,0,2)*X(4,u,r,0,0))/(2*X(0,u,r,0,0)**2)
            +(13*Q2*X(0,u,r,1,0)*X(4,u,r,0,0))/X(0,u,r,0,0)**2+(4*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(4,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*Q2**2*X(0,u,r,1,0)**2*X(4,u,r,0,0))/X(0,u,r,0,0)**3-(2*Q2*r*X(0,u,r,1,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*Q2**2*X(0,u,r,2,0)*X(4,u,r,0,0))/X(0,u,r,0,0)**2-(7*r*X(4,u,r,0,1))/X(0,u,r,0,0)
            -(r**2*X(0,u,r,0,1)*X(4,u,r,0,1))/X(0,u,r,0,0)**2-(2*Q2*r*X(0,u,r,1,0)*X(4,u,r,0,1))/X(0,u,r,0,0)**2
            +(r**2*X(4,u,r,0,2))/(2*X(0,u,r,0,0))-(13*Q2*X(4,u,r,1,0))/X(0,u,r,0,0)
            -(2*Q2*r*X(0,u,r,0,1)*X(4,u,r,1,0))/X(0,u,r,0,0)**2-(4*Q2**2*X(0,u,r,1,0)*X(4,u,r,1,0))/X(0,u,r,0,0)**2
            +(2*Q2*r*X(4,u,r,1,1))/X(0,u,r,0,0)+(2*Q2**2*X(4,u,r,2,0))/X(0,u,r,0,0))
            -(30*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**6+(36*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**5
            -(12*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**4
            +(X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)**2)/X(0,u,r,0,0)**3+(48*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(8*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(18*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)**2)/X(0,u,r,0,0)**4+(6*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)**2)/X(0,u,r,0,0)**3
            -(18*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(8*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*nlo.X1Q2(1,'SM',Q2,r)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2-(nlo.X1Q2(0,'SM',Q2,r)*nlo.X1Q2(4,'SM',Q2,r))/X(0,u,r,0,0)**2
            +math.log(mus**2)*((25*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(75*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(2*X(0,u,r,0,0)**4)
            +(50*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(3*X(0,u,r,0,0)**3)
            -(25*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/(12*X(0,u,r,0,0)**2)-(25*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(25*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(25*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/(3*X(0,u,r,0,0)**2)+(25*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r))/(2*X(0,u,r,0,0)**3)
            -(25*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r))/(3*X(0,u,r,0,0)**2)+(25*nlo.X1Q2(4,'SM',Q2,r))/(12*X(0,u,r,0,0)))
            +deltamcMS1*((r*nlo.X1Q2_Derivativer(4,Q2,r))/X(0,u,r,0,0)-(4*r*nlo.X1Q2_Derivativer(3,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(6*r*nlo.X1Q2_Derivativer(2,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3-(12*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            +(12*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5+(12*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(18*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(4*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(8*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(r*nlo.X1Q2_Derivativer(0,Q2,r)*X(4,u,r,0,0))/X(0,u,r,0,0)**2-(60*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**6
            +(48*r*X(1,u,r,0,0)**3*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            +(72*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(36*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(18*r*X(1,u,r,0,0)**2*X(2,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(24*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(8*r*X(1,u,r,0,1)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(8*r*X(1,u,r,0,0)*X(3,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(2*r*X(0,u,r,0,1)*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(r*X(4,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2+(48*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(36*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(36*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(12*r*X(1,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(12*r*X(1,u,r,0,0)*X(2,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(8*r*X(0,u,r,0,1)*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*r*X(3,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2-(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*r*X(1,u,r,0,1)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2-(r*X(0,u,r,0,1)*nlo.X1Q2(4,'SM',Q2,r))/X(0,u,r,0,0)**2)
            +deltambkin1*((-2*Q2*nlo.X1Q2_DerivativeQ2(4,Q2,r))/X(0,u,r,0,0)-(r*nlo.X1Q2_Derivativer(4,Q2,r))/X(0,u,r,0,0)
            +(8*Q2*nlo.X1Q2_DerivativeQ2(3,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2+(4*r*nlo.X1Q2_Derivativer(3,Q2,r)*X(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(12*Q2*nlo.X1Q2_DerivativeQ2(2,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3-(6*r*nlo.X1Q2_Derivativer(2,Q2,r)*X(1,u,r,0,0)**2)/X(0,u,r,0,0)**3
            +(24*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4+(12*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0)**3)/X(0,u,r,0,0)**4
            -(24*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5-(12*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            -(24*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(1,u,r,0,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(36*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(8*Q2*nlo.X1Q2_DerivativeQ2(1,Q2,r)*X(3,u,r,0,0))/X(0,u,r,0,0)**2+(4*r*nlo.X1Q2_Derivativer(1,Q2,r)*X(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(16*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*r*nlo.X1Q2_Derivativer(0,Q2,r)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*Q2*nlo.X1Q2_DerivativeQ2(0,Q2,r)*X(4,u,r,0,0))/X(0,u,r,0,0)**2+(r*nlo.X1Q2_Derivativer(0,Q2,r)*X(4,u,r,0,0))/X(0,u,r,0,0)**2
            +deltamcMS1*((84*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            +(60*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**6-(12*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5
            +(120*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**6
            -(24*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**4)/X(0,u,r,0,0)**5-(84*r*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**4
            -(96*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**5
            -(96*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)**3*X(1,u,r,0,1))/X(0,u,r,0,0)**5
            +(36*r**2*X(1,u,r,0,0)**2*X(1,u,r,0,1)**2)/X(0,u,r,0,0)**4+(12*r**2*X(1,u,r,0,0)**3*X(1,u,r,0,2))/X(0,u,r,0,0)**4
            -(96*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*X(1,u,r,1,0))/X(0,u,r,0,0)**5
            +(72*Q2*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*X(1,u,r,1,0))/X(0,u,r,0,0)**4
            +(24*Q2*r*X(1,u,r,0,0)**3*X(1,u,r,1,1))/X(0,u,r,0,0)**4-(126*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(72*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**5
            +(18*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(144*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**5
            +(36*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(84*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(72*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(72*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*r**2*X(1,u,r,0,1)**2*X(2,u,r,0,0))/X(0,u,r,0,0)**3-(12*r**2*X(1,u,r,0,0)*X(1,u,r,0,2)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(72*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(24*Q2*r*X(1,u,r,0,1)*X(1,u,r,1,0)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*Q2*r*X(1,u,r,0,0)*X(1,u,r,1,1)*X(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(42*r*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**3+(36*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**4
            +(36*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,1))/X(0,u,r,0,0)**4
            -(24*r**2*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(24*Q2*r*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(6*r**2*X(1,u,r,0,0)**2*X(2,u,r,0,2))/X(0,u,r,0,0)**3+(36*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,1,0))/X(0,u,r,0,0)**4
            -(24*Q2*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,1,0))/X(0,u,r,0,0)**3
            -(12*Q2*r*X(1,u,r,0,0)**2*X(2,u,r,1,1))/X(0,u,r,0,0)**3+(56*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*r**2*X(0,u,r,0,1)**2*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**4
            -(8*r**2*X(0,u,r,0,2)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(48*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**4
            -(16*Q2*r*X(0,u,r,1,1)*X(1,u,r,0,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(28*r*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(16*r**2*X(0,u,r,0,1)*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(16*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*r**2*X(1,u,r,0,2)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(16*Q2*r*X(0,u,r,0,1)*X(1,u,r,1,0)*X(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*Q2*r*X(1,u,r,1,1)*X(3,u,r,0,0))/X(0,u,r,0,0)**2-(28*r*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(16*r**2*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**3
            -(16*Q2*r*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**3
            +(8*r**2*X(1,u,r,0,1)*X(3,u,r,0,1))/X(0,u,r,0,0)**2+(8*Q2*r*X(1,u,r,1,0)*X(3,u,r,0,1))/X(0,u,r,0,0)**2
            +(4*r**2*X(1,u,r,0,0)*X(3,u,r,0,2))/X(0,u,r,0,0)**2-(16*Q2*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,1,0))/X(0,u,r,0,0)**3
            +(8*Q2*r*X(1,u,r,0,1)*X(3,u,r,1,0))/X(0,u,r,0,0)**2+(8*Q2*r*X(1,u,r,0,0)*X(3,u,r,1,1))/X(0,u,r,0,0)**2
            -(7*r*X(0,u,r,0,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2-(2*r**2*X(0,u,r,0,1)**2*X(4,u,r,0,0))/X(0,u,r,0,0)**3
            +(r**2*X(0,u,r,0,2)*X(4,u,r,0,0))/X(0,u,r,0,0)**2-(4*Q2*r*X(0,u,r,0,1)*X(0,u,r,1,0)*X(4,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*Q2*r*X(0,u,r,1,1)*X(4,u,r,0,0))/X(0,u,r,0,0)**2+(7*r*X(4,u,r,0,1))/X(0,u,r,0,0)
            +(2*r**2*X(0,u,r,0,1)*X(4,u,r,0,1))/X(0,u,r,0,0)**2+(2*Q2*r*X(0,u,r,1,0)*X(4,u,r,0,1))/X(0,u,r,0,0)**2
            -(r**2*X(4,u,r,0,2))/X(0,u,r,0,0)+(2*Q2*r*X(0,u,r,0,1)*X(4,u,r,1,0))/X(0,u,r,0,0)**2
            -(2*Q2*r*X(4,u,r,1,1))/X(0,u,r,0,0))+(96*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            +(60*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**6
            +(120*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**6
            -(48*r*X(1,u,r,0,0)**3*X(1,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(96*Q2*X(1,u,r,0,0)**3*X(1,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(144*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(72*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(144*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**5
            +(36*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(72*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(18*r*X(1,u,r,0,0)**2*X(2,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(36*Q2*X(1,u,r,0,0)**2*X(2,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(64*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(24*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(48*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(8*r*X(1,u,r,0,1)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(16*Q2*X(1,u,r,1,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(8*r*X(1,u,r,0,0)*X(3,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(16*Q2*X(1,u,r,0,0)*X(3,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(8*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2-(2*r*X(0,u,r,0,1)*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(4*Q2*X(0,u,r,1,0)*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(r*X(4,u,r,0,1)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2+(2*Q2*X(4,u,r,1,0)*nlo.X1Q2(0,'SM',Q2,r))/X(0,u,r,0,0)**2
            -(96*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4-(48*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**5
            -(96*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**5
            +(36*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(72*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(96*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(36*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(72*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(12*r*X(1,u,r,0,1)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(24*Q2*X(1,u,r,1,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(12*r*X(1,u,r,0,0)*X(2,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(24*Q2*X(1,u,r,0,0)*X(2,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(32*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2-(8*r*X(0,u,r,0,1)*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(16*Q2*X(0,u,r,1,0)*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(4*r*X(3,u,r,0,1)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2+(8*Q2*X(3,u,r,1,0)*nlo.X1Q2(1,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(48*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3+(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**4
            +(36*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**4
            -(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(24*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*nlo.X1Q2(2,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(32*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2-(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**3
            -(16*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**3
            +(4*r*X(1,u,r,0,1)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2+(8*Q2*X(1,u,r,1,0)*nlo.X1Q2(3,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(8*nlo.X1Q2(4,'SM',Q2,r))/X(0,u,r,0,0)+(r*X(0,u,r,0,1)*nlo.X1Q2(4,'SM',Q2,r))/X(0,u,r,0,0)**2
            +(2*Q2*X(0,u,r,1,0)*nlo.X1Q2(4,'SM',Q2,r))/X(0,u,r,0,0)**2)+(12*X(1,u,r,0,0)**4*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**5
            -(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**3
            -(X(4,u,r,0,0)*nnlo.X2Q2(0,Q2,r))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*nnlo.X2Q2(1,Q2,r))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*nnlo.X2Q2(1,Q2,r))/X(0,u,r,0,0)**3
            -(4*X(3,u,r,0,0)*nnlo.X2Q2(1,Q2,r))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*nnlo.X2Q2(2,Q2,r))/X(0,u,r,0,0)**3
            -(4*X(1,u,r,0,0)*nnlo.X2Q2(3,Q2,r))/X(0,u,r,0,0)**2+nnlo.X2Q2(4,Q2,r)/X(0,u,r,0,0)
            +Rhodpert2*((-12*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-XD(4,u,r,0,0,flagPERP)/X(0,u,r,0,0))
            +Rhodpert1*(math.log(r)*((-4*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(6*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**3)
            +(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(4*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(4*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            -(2*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            -XD(4,u,r,0,0,flagPERP)/(3*X(0,u,r,0,0)))+math.log(mus**2)*((2*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(3*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(4*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**3)
            -(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/(6*X(0,u,r,0,0)**2)
            -(2*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(2*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +(X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3-(2*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/(3*X(0,u,r,0,0)**2)
            +XD(4,u,r,0,0,flagPERP)/(6*X(0,u,r,0,0))))+deltamcMS1*Rhodpert1*((60*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            -(48*r*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(72*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(18*r*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(24*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*r*X(1,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*r*X(1,u,r,0,0)*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*r*X(0,u,r,0,1)*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(r*X(4,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(12*r*X(1,u,r,0,0)**4*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**5
            +(18*r*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(8*r*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(r*X(4,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2-(48*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(36*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*r*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*r*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*r*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*r*X(3,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(12*r*X(1,u,r,0,0)**3*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(12*r*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(4*r*X(3,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*r*X(1,u,r,0,0)**2*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*r*X(1,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(4*r*X(1,u,r,0,0)*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(r*X(0,u,r,0,1)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(r*XD(4,u,r,0,1,flagPERP))/X(0,u,r,0,0))
            +deltambkin1*Rhodpert1*((-60*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(60*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            -(120*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            +(48*r*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(96*Q2*X(1,u,r,0,0)**3*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(90*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(72*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(144*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(36*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(72*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*r*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*Q2*X(1,u,r,0,0)**2*X(2,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(40*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(24*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(48*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(8*r*X(1,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(16*Q2*X(1,u,r,1,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*r*X(1,u,r,0,0)*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(16*Q2*X(1,u,r,0,0)*X(3,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(5*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(2*r*X(0,u,r,0,1)*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*Q2*X(0,u,r,1,0)*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(r*X(4,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(2*Q2*X(4,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(12*r*X(1,u,r,0,0)**4*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**5
            -(18*r*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            +(8*r*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(r*X(4,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2+(24*Q2*X(1,u,r,0,0)**4*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**5
            -(36*Q2*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**4
            +(16*Q2*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            -(2*Q2*X(4,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(60*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(48*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(96*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(36*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(72*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(60*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(36*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(72*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*r*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(24*Q2*X(1,u,r,1,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*r*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(24*Q2*X(1,u,r,0,0)*X(2,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(20*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(8*r*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(16*Q2*X(0,u,r,1,0)*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*r*X(3,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(8*Q2*X(3,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(12*r*X(1,u,r,0,0)**3*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            +(12*r*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(4*r*X(3,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(24*Q2*X(1,u,r,0,0)**3*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**4
            +(24*Q2*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*Q2*X(3,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(30*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(18*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(24*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(6*r*X(1,u,r,0,0)**2*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(12*Q2*X(1,u,r,0,0)**2*XD(2,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            +(20*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(8*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(16*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*r*X(1,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(8*Q2*X(1,u,r,1,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(4*r*X(1,u,r,0,0)*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(8*Q2*X(1,u,r,0,0)*XD(3,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(5*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)-(r*X(0,u,r,0,1)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*Q2*X(0,u,r,1,0)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(r*XD(4,u,r,0,1,flagPERP))/X(0,u,r,0,0)+(2*Q2*XD(4,u,r,1,0,flagPERP))/X(0,u,r,0,0)))
        if( flagDEBUG == 1):
            print("Q2moment n. 4 NNLO = api^2*",resNNLO*par.mbkin**8)
        res += api**2*resNNLO

    if(kwargs.get('flag_includeNLOpw', 1) == 1):
        resNLO = 0
        resNLO +=((60*deltambkin1*rhoD*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(60*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            -(60*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            +(120*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            -(48*deltambkin1*r*rhoD*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(48*deltamcMS1*r*rhoD*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(96*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**3*X(1,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(90*deltambkin1*rhoD*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(72*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(72*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(144*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(72*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(18*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*X(2,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(40*deltambkin1*rhoD*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(24*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(24*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(48*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*deltambkin1*r*rhoD*X(1,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*r*rhoD*X(1,u,r,0,1)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(16*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(3,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(16*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(3,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(5*deltambkin1*rhoD*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(2*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(4,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(4,u,r,0,1)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(4,u,r,1,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(12*deltambkin1*r*rhoD*X(1,u,r,0,0)**4*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**5
            +(12*deltamcMS1*r*rhoD*X(1,u,r,0,0)**4*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**5
            +(18*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(8*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(deltambkin1*r*rhoD*X(4,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(4,u,r,0,0)*XD(0,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(24*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**4*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**4
            -(16*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            +(2*deltambkin1*Q2*rhoD*X(4,u,r,0,0)*XD(0,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            -(60*deltambkin1*rhoD*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(48*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(48*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(96*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(72*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(60*deltambkin1*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(36*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(36*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(72*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltambkin1*r*rhoD*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*r*rhoD*X(1,u,r,0,1)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(24*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(24*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(2,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(20*deltambkin1*rhoD*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(8*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(16*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltambkin1*r*rhoD*X(3,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*r*rhoD*X(3,u,r,0,1)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(8*deltambkin1*Q2*rhoD*X(3,u,r,1,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(12*deltambkin1*r*rhoD*X(1,u,r,0,0)**3*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltamcMS1*r*rhoD*X(1,u,r,0,0)**3*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltambkin1*r*rhoD*X(3,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*r*rhoD*X(3,u,r,0,0)*XD(1,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(24*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**3*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**4
            -(24*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltambkin1*Q2*rhoD*X(3,u,r,0,0)*XD(1,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(30*deltambkin1*rhoD*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(18*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(36*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*deltambkin1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*r*rhoD*X(1,u,r,0,0)*X(1,u,r,0,1)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(24*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*X(1,u,r,1,0)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(6*deltambkin1*r*rhoD*X(1,u,r,0,0)**2*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*r*rhoD*X(1,u,r,0,0)**2*XD(2,u,r,0,1,flagPERP))/X(0,u,r,0,0)**3
            -(12*deltambkin1*Q2*rhoD*X(1,u,r,0,0)**2*XD(2,u,r,1,0,flagPERP))/X(0,u,r,0,0)**3
            -(20*deltambkin1*rhoD*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(8*deltambkin1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*r*rhoD*X(0,u,r,0,1)*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(16*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*deltambkin1*r*rhoD*X(1,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*r*rhoD*X(1,u,r,0,1)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(8*deltambkin1*Q2*rhoD*X(1,u,r,1,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(4*deltambkin1*r*rhoD*X(1,u,r,0,0)*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*r*rhoD*X(1,u,r,0,0)*XD(3,u,r,0,1,flagPERP))/X(0,u,r,0,0)**2
            +(8*deltambkin1*Q2*rhoD*X(1,u,r,0,0)*XD(3,u,r,1,0,flagPERP))/X(0,u,r,0,0)**2
            +(5*deltambkin1*rhoD*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)+(deltambkin1*r*rhoD*X(0,u,r,0,1)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(deltamcMS1*r*rhoD*X(0,u,r,0,1)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(2*deltambkin1*Q2*rhoD*X(0,u,r,1,0)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +rhoD*((12*X(1,u,r,0,0)**4*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**5
            -(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**3
            -(X(4,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**3
            -(4*X(3,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*nlopw.X1Q2RhoD(2,Q2,r))/X(0,u,r,0,0)**3
            -(4*X(1,u,r,0,0)*nlopw.X1Q2RhoD(3,Q2,r))/X(0,u,r,0,0)**2+nlopw.X1Q2RhoD(4,Q2,r)/X(0,u,r,0,0)
            -(60*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            +(72*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(24*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(2*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(48*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(8*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(18*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(nlo.X1Q2(4,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(48*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(8*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(36*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(12*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*nlo.X1Q2(3,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(18*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(8*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*nlo.X1Q2(1,'SM',Q2,r)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(nlo.X1Q2(0,'SM',Q2,r)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +FLAGcD*((12*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            -(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(4*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+XD(4,u,r,0,0,flagPERP)/X(0,u,r,0,0)))
            -(deltambkin1*r*rhoD*XD(4,u,r,0,1,flagPERP))/X(0,u,r,0,0)+(deltamcMS1*r*rhoD*XD(4,u,r,0,1,flagPERP))/X(0,u,r,0,0)
            -(2*deltambkin1*Q2*rhoD*XD(4,u,r,1,0,flagPERP))/X(0,u,r,0,0)+(72*deltambkin1*muG*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(60*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(60*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            -(60*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            -(60*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            +(60*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            +(120*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            -(120*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            -(48*deltambkin1*muG*r*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(48*deltamcMS1*muG*r*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(48*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(48*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**3*X(1,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(96*deltambkin1*muG*Q2*X(1,u,r,0,0)**3*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(96*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**3*X(1,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(108*deltambkin1*muG*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(90*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(72*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(72*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(72*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(72*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(144*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(144*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(36*deltambkin1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltamcMS1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(72*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(72*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(1,u,r,1,0)*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltambkin1*muG*r*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*muG*r*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*X(2,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(48*deltambkin1*muG*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(40*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(24*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(24*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(24*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(48*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(48*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(8*deltambkin1*muG*r*X(1,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*muG*r*X(1,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(16*deltambkin1*muG*Q2*X(1,u,r,1,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(16*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltambkin1*muG*r*X(1,u,r,0,0)*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*muG*r*X(1,u,r,0,0)*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(3,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(16*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(3,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(16*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(3,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1*muG*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(5*deltambkin1*flagPERP*rhoLS*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*muG*r*X(0,u,r,0,1)*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltamcMS1*muG*r*X(0,u,r,0,1)*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(2*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(4,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(4,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(4,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(4,u,r,0,1)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(4,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(4,u,r,1,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2
            -(12*deltambkin1*muG*r*X(1,u,r,0,0)**4*XG(0,u,r,0,1))/X(0,u,r,0,0)**5
            +(12*deltamcMS1*muG*r*X(1,u,r,0,0)**4*XG(0,u,r,0,1))/X(0,u,r,0,0)**5
            +(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**4*XG(0,u,r,0,1))/X(0,u,r,0,0)**5
            -(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**4*XG(0,u,r,0,1))/X(0,u,r,0,0)**5
            +(18*deltambkin1*muG*r*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*muG*r*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            -(18*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**4
            -(8*deltambkin1*muG*r*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*muG*r*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            -(8*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**3
            +(deltambkin1*muG*r*X(4,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(4,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(4,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(4,u,r,0,0)*XG(0,u,r,0,1))/X(0,u,r,0,0)**2
            -(24*deltambkin1*muG*Q2*X(1,u,r,0,0)**4*XG(0,u,r,1,0))/X(0,u,r,0,0)**5
            +(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**4*XG(0,u,r,1,0))/X(0,u,r,0,0)**5
            +(36*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**4
            -(16*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**3
            +(16*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**3
            +(2*deltambkin1*muG*Q2*X(4,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(4,u,r,0,0)*XG(0,u,r,1,0))/X(0,u,r,0,0)**2
            -(72*deltambkin1*muG*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(60*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(48*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            +(48*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            +(48*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            -(48*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            -(96*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            +(96*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            +(36*deltambkin1*muG*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltamcMS1*muG*r*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*X(1,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(72*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(72*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*X(1,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(72*deltambkin1*muG*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(60*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(36*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(72*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(72*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*deltambkin1*muG*r*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*muG*r*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*muG*Q2*X(1,u,r,1,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(2,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(2,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*muG*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(20*deltambkin1*flagPERP*rhoLS*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(8*deltambkin1*muG*r*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*muG*r*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(16*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(16*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*muG*r*X(3,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*muG*r*X(3,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*flagPERP*r*rhoLS*X(3,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltamcMS1*flagPERP*r*rhoLS*X(3,u,r,0,1)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(8*deltambkin1*muG*Q2*X(3,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            -(8*deltambkin1*flagPERP*Q2*rhoLS*X(3,u,r,1,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2
            +(12*deltambkin1*muG*r*X(1,u,r,0,0)**3*XG(1,u,r,0,1))/X(0,u,r,0,0)**4
            -(12*deltamcMS1*muG*r*X(1,u,r,0,0)**3*XG(1,u,r,0,1))/X(0,u,r,0,0)**4
            -(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**3*XG(1,u,r,0,1))/X(0,u,r,0,0)**4
            +(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**3*XG(1,u,r,0,1))/X(0,u,r,0,0)**4
            -(12*deltambkin1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*muG*r*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            -(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**3
            +(4*deltambkin1*muG*r*X(3,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*muG*r*X(3,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            -(4*deltambkin1*flagPERP*r*rhoLS*X(3,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(4*deltamcMS1*flagPERP*r*rhoLS*X(3,u,r,0,0)*XG(1,u,r,0,1))/X(0,u,r,0,0)**2
            +(24*deltambkin1*muG*Q2*X(1,u,r,0,0)**3*XG(1,u,r,1,0))/X(0,u,r,0,0)**4
            -(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**3*XG(1,u,r,1,0))/X(0,u,r,0,0)**4
            -(24*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*muG*Q2*X(3,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**2
            -(8*deltambkin1*flagPERP*Q2*rhoLS*X(3,u,r,0,0)*XG(1,u,r,1,0))/X(0,u,r,0,0)**2
            +(36*deltambkin1*muG*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(30*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(18*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(18*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(18*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(36*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(36*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*deltambkin1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltamcMS1*muG*r*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*X(1,u,r,0,1)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*muG*Q2*X(1,u,r,0,0)*X(1,u,r,1,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(24*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*X(1,u,r,1,0)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(6*deltambkin1*muG*r*X(1,u,r,0,0)**2*XG(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltamcMS1*muG*r*X(1,u,r,0,0)**2*XG(2,u,r,0,1))/X(0,u,r,0,0)**3
            +(6*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*XG(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(6*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)**2*XG(2,u,r,0,1))/X(0,u,r,0,0)**3
            -(12*deltambkin1*muG*Q2*X(1,u,r,0,0)**2*XG(2,u,r,1,0))/X(0,u,r,0,0)**3
            +(12*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)**2*XG(2,u,r,1,0))/X(0,u,r,0,0)**3
            -(24*deltambkin1*muG*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(20*deltambkin1*flagPERP*rhoLS*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(8*deltambkin1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltamcMS1*muG*r*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(16*deltambkin1*muG*Q2*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(16*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*deltambkin1*muG*r*X(1,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*muG*r*X(1,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(4*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,1)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(8*deltambkin1*muG*Q2*X(1,u,r,1,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            -(8*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,1,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2
            +(4*deltambkin1*muG*r*X(1,u,r,0,0)*XG(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(4*deltamcMS1*muG*r*X(1,u,r,0,0)*XG(3,u,r,0,1))/X(0,u,r,0,0)**2
            -(4*deltambkin1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(3,u,r,0,1))/X(0,u,r,0,0)**2
            +(4*deltamcMS1*flagPERP*r*rhoLS*X(1,u,r,0,0)*XG(3,u,r,0,1))/X(0,u,r,0,0)**2
            +(8*deltambkin1*muG*Q2*X(1,u,r,0,0)*XG(3,u,r,1,0))/X(0,u,r,0,0)**2
            -(8*deltambkin1*flagPERP*Q2*rhoLS*X(1,u,r,0,0)*XG(3,u,r,1,0))/X(0,u,r,0,0)**2
            +(6*deltambkin1*muG*XG(4,u,r,0,0))/X(0,u,r,0,0)-(5*deltambkin1*flagPERP*rhoLS*XG(4,u,r,0,0))/X(0,u,r,0,0)
            +(deltambkin1*muG*r*X(0,u,r,0,1)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltamcMS1*muG*r*X(0,u,r,0,1)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            -(deltambkin1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            +(deltamcMS1*flagPERP*r*rhoLS*X(0,u,r,0,1)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            +(2*deltambkin1*muG*Q2*X(0,u,r,1,0)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            -(2*deltambkin1*flagPERP*Q2*rhoLS*X(0,u,r,1,0)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            +flagPERP*rhoLS*((-12*X(1,u,r,0,0)**4*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**5
            +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**3
            +(X(4,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**3
            +(4*X(3,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*nlopw.X1Q2MuG(2,Q2,r))/X(0,u,r,0,0)**3
            +(4*X(1,u,r,0,0)*nlopw.X1Q2MuG(3,Q2,r))/X(0,u,r,0,0)**2-nlopw.X1Q2MuG(4,Q2,r)/X(0,u,r,0,0)
            +(60*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            -(72*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(24*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(2*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(48*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(8*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(18*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(nlo.X1Q2(4,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2-(48*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            +(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(8*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(36*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(12*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*nlo.X1Q2(3,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(18*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(8*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*nlo.X1Q2(1,'SM',Q2,r)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2+(nlo.X1Q2(0,'SM',Q2,r)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            +FLAGcs*((-12*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(4*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2-XG(4,u,r,0,0)/X(0,u,r,0,0)))
            +muG*((12*X(1,u,r,0,0)**4*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**5-(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**3
            -(X(4,u,r,0,0)*nlopw.X1Q2MuG(0,Q2,r))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**3
            -(4*X(3,u,r,0,0)*nlopw.X1Q2MuG(1,Q2,r))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*nlopw.X1Q2MuG(2,Q2,r))/X(0,u,r,0,0)**3
            -(4*X(1,u,r,0,0)*nlopw.X1Q2MuG(3,Q2,r))/X(0,u,r,0,0)**2+nlopw.X1Q2MuG(4,Q2,r)/X(0,u,r,0,0)
            -(60*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**6
            +(72*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(24*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(2*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            +(48*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**5
            -(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(8*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(18*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(nlo.X1Q2(4,'SM',Q2,r)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2+(48*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**5
            -(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(8*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(36*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(12*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            +(12*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*nlo.X1Q2(3,'SM',Q2,r)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2-(18*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            +(8*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XG(3,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*nlo.X1Q2(1,'SM',Q2,r)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2-(nlo.X1Q2(0,'SM',Q2,r)*XG(4,u,r,0,0))/X(0,u,r,0,0)**2
            +FLAGcf*((12*X(1,u,r,0,0)**4*XG(0,u,r,0,0))/X(0,u,r,0,0)**5-(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**4
            +(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**3
            -(X(4,u,r,0,0)*XG(0,u,r,0,0))/X(0,u,r,0,0)**2-(12*X(1,u,r,0,0)**3*XG(1,u,r,0,0))/X(0,u,r,0,0)**4
            +(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*X(3,u,r,0,0)*XG(1,u,r,0,0))/X(0,u,r,0,0)**2+(6*X(1,u,r,0,0)**2*XG(2,u,r,0,0))/X(0,u,r,0,0)**3
            -(4*X(1,u,r,0,0)*XG(3,u,r,0,0))/X(0,u,r,0,0)**2+XG(4,u,r,0,0)/X(0,u,r,0,0)))
            -(deltambkin1*muG*r*XG(4,u,r,0,1))/X(0,u,r,0,0)+(deltamcMS1*muG*r*XG(4,u,r,0,1))/X(0,u,r,0,0)
            +(deltambkin1*flagPERP*r*rhoLS*XG(4,u,r,0,1))/X(0,u,r,0,0)-(deltamcMS1*flagPERP*r*rhoLS*XG(4,u,r,0,1))/X(0,u,r,0,0)
            -(2*deltambkin1*muG*Q2*XG(4,u,r,1,0))/X(0,u,r,0,0)+(2*deltambkin1*flagPERP*Q2*rhoLS*XG(4,u,r,1,0))/X(0,u,r,0,0))
        if( flagDEBUG == 1):
            print("Q2moment n. 4 NLO pw = api*",resNLO*par.mbkin**8)
        res += api*resNLO

        resNNLO = 0
        resNNLO +=(Rhodpert1*((-12*X(1,u,r,0,0)**4*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**5
            +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**3
            +(X(4,u,r,0,0)*nlopw.X1Q2RhoD(0,Q2,r))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**3
            +(4*X(3,u,r,0,0)*nlopw.X1Q2RhoD(1,Q2,r))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*nlopw.X1Q2RhoD(2,Q2,r))/X(0,u,r,0,0)**3
            +(4*X(1,u,r,0,0)*nlopw.X1Q2RhoD(3,Q2,r))/X(0,u,r,0,0)**2-nlopw.X1Q2RhoD(4,Q2,r)/X(0,u,r,0,0)
            +(60*X(1,u,r,0,0)**4*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**6
            -(72*X(1,u,r,0,0)**2*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(24*X(1,u,r,0,0)*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(2*X(4,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(48*X(1,u,r,0,0)**3*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*X(3,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(18*X(1,u,r,0,0)**2*nlo.X1Q2(2,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*nlo.X1Q2(3,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(nlo.X1Q2(4,'SM',Q2,r)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            -(48*X(1,u,r,0,0)**3*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(36*X(1,u,r,0,0)*X(2,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*X(3,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(36*X(1,u,r,0,0)**2*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*X(2,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(12*X(1,u,r,0,0)*nlo.X1Q2(2,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*nlo.X1Q2(3,'SM',Q2,r)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(18*X(1,u,r,0,0)**2*nlo.X1Q2(0,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*nlo.X1Q2(1,'SM',Q2,r)*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            -(8*X(1,u,r,0,0)*nlo.X1Q2(0,'SM',Q2,r)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*nlo.X1Q2(1,'SM',Q2,r)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +(nlo.X1Q2(0,'SM',Q2,r)*XD(4,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2
            +FLAGcD*((-12*X(1,u,r,0,0)**4*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**5
            +(18*X(1,u,r,0,0)**2*X(2,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(8*X(1,u,r,0,0)*X(3,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(X(4,u,r,0,0)*XD(0,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2+(12*X(1,u,r,0,0)**3*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**4
            -(12*X(1,u,r,0,0)*X(2,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*X(3,u,r,0,0)*XD(1,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-(6*X(1,u,r,0,0)**2*XD(2,u,r,0,0,flagPERP))/X(0,u,r,0,0)**3
            +(4*X(1,u,r,0,0)*XD(3,u,r,0,0,flagPERP))/X(0,u,r,0,0)**2-XD(4,u,r,0,0,flagPERP)/X(0,u,r,0,0))))
        if( flagDEBUG == 1):
            print("Q2moment n. 4 NNLO from NLO pw = api^2*",resNNLO*par.mbkin**8)
        res += api**2*resNNLO

    return res*par.mbkin**8


